/**
 * Copyright (c) 2022, Salesforce, Inc.,
 * All rights reserved.
 * For full license text, see the LICENSE.txt file
 */

import { createWireAdapterConstructor, bindWireRefresh } from '@luvio/lwc-luvio';
import { register, withDefaultLuvio } from '@salesforce/lds-default-luvio';

var SnapshotState;
(function (SnapshotState) {
    SnapshotState["Fulfilled"] = "Fulfilled";
    SnapshotState["Unfulfilled"] = "Unfulfilled";
    SnapshotState["Error"] = "Error";
    SnapshotState["Pending"] = "Pending";
    SnapshotState["Stale"] = "Stale";
})(SnapshotState || (SnapshotState = {}));

const { create: create$1, entries, freeze: freeze$1, keys: keys$1, values, assign: assign$1 } = Object;
const { isArray: isArray$1 } = Array;
const { parse: parse$2, stringify: stringify$1 } = JSON;
const WeakSetCtor = WeakSet;

const deeplyFrozen = new WeakSetCtor();
function deepFreeze(value) {
    // No need to freeze primitives or already frozen stuff
    if (typeof value !== 'object' ||
        value === null ||
        deeplyFrozen.has(value)) {
        return;
    }
    deeplyFrozen.add(value);
    if (isArray$1(value)) {
        for (let i = 0, len = value.length; i < len; i += 1) {
            deepFreeze(value[i]);
        }
    }
    else {
        const keys$1$1 = keys$1(value);
        for (let i = 0, len = keys$1$1.length; i < len; i += 1) {
            deepFreeze(value[keys$1$1[i]]);
        }
    }
    freeze$1(value);
}

Promise.resolve();

var StoreErrorStatus;
(function (StoreErrorStatus) {
    StoreErrorStatus[StoreErrorStatus["RESOURCE_NOT_FOUND"] = 404] = "RESOURCE_NOT_FOUND";
})(StoreErrorStatus || (StoreErrorStatus = {}));
var StoreRecordType;
(function (StoreRecordType) {
    StoreRecordType["Error"] = "error";
})(StoreRecordType || (StoreRecordType = {}));
var StoreLinkStateValues$1;
(function (StoreLinkStateValues) {
    StoreLinkStateValues[StoreLinkStateValues["NotPresent"] = 0] = "NotPresent";
    StoreLinkStateValues[StoreLinkStateValues["RefNotPresent"] = 1] = "RefNotPresent";
    StoreLinkStateValues[StoreLinkStateValues["RefPresent"] = 2] = "RefPresent";
    StoreLinkStateValues[StoreLinkStateValues["Null"] = 3] = "Null";
    StoreLinkStateValues[StoreLinkStateValues["Missing"] = 4] = "Missing";
    StoreLinkStateValues[StoreLinkStateValues["Pending"] = 5] = "Pending";
})(StoreLinkStateValues$1 || (StoreLinkStateValues$1 = {}));
var StoreResolveResultState;
(function (StoreResolveResultState) {
    StoreResolveResultState[StoreResolveResultState["Found"] = 0] = "Found";
    StoreResolveResultState[StoreResolveResultState["Error"] = 1] = "Error";
    StoreResolveResultState[StoreResolveResultState["Null"] = 2] = "Null";
    StoreResolveResultState[StoreResolveResultState["NotPresent"] = 3] = "NotPresent";
    StoreResolveResultState[StoreResolveResultState["Stale"] = 4] = "Stale";
})(StoreResolveResultState || (StoreResolveResultState = {}));

/**
 * A deterministic JSON stringify implementation. Heavily adapted from https://github.com/epoberezkin/fast-json-stable-stringify.
 * This is needed because insertion order for JSON.stringify(object) affects output:
 * JSON.stringify({a: 1, b: 2})
 *      "{"a":1,"b":2}"
 * JSON.stringify({b: 2, a: 1})
 *      "{"b":2,"a":1}"
 * @param data Data to be JSON-stringified.
 * @returns JSON.stringified value with consistent ordering of keys.
 */
function stableJSONStringify$1(node) {
    // This is for Date values.
    if (node && node.toJSON && typeof node.toJSON === 'function') {
        // eslint-disable-next-line no-param-reassign
        node = node.toJSON();
    }
    if (node === undefined) {
        return;
    }
    if (typeof node === 'number') {
        return isFinite(node) ? '' + node : 'null';
    }
    if (typeof node !== 'object') {
        return stringify$1(node);
    }
    let i;
    let out;
    if (isArray$1(node)) {
        out = '[';
        for (i = 0; i < node.length; i++) {
            if (i) {
                out += ',';
            }
            out += stableJSONStringify$1(node[i]) || 'null';
        }
        return out + ']';
    }
    if (node === null) {
        return 'null';
    }
    const keys$1$1 = keys$1(node).sort();
    out = '';
    for (i = 0; i < keys$1$1.length; i++) {
        const key = keys$1$1[i];
        const value = stableJSONStringify$1(node[key]);
        if (!value) {
            continue;
        }
        if (out) {
            out += ',';
        }
        out += stringify$1(key) + ':' + value;
    }
    return '{' + out + '}';
}
const structuredKeySerializationCache = new WeakMap();
function serializeStructuredKey(key) {
    if (typeof key === 'string') {
        return key;
    }
    const cacheValue = structuredKeySerializationCache.get(key);
    if (cacheValue === undefined) {
        const value = stableJSONStringify$1(key);
        structuredKeySerializationCache.set(key, value);
        return value;
    }
    return cacheValue;
}

const undefinedKeyError$1 = 'Undefined value used in StoreKeyMap operation';
class StoreKeyMap {
    constructor() {
        this.keyMap = new Map();
        this.valueMap = new Map();
    }
    clear() {
        this.valueMap.clear();
        this.keyMap.clear();
    }
    delete(key) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            this.keyMap.delete(stringifiedKey);
            return this.valueMap.delete(stringifiedKey);
        }
        else {
            throw new Error(`${undefinedKeyError$1}: delete`);
        }
    }
    entries() {
        const recordEntries = this.valueMap.entries();
        const keyRecordArray = [];
        for (const [stringifiedKey, value] of Array.from(recordEntries)) {
            const structuredKey = this.keyMap.get(stringifiedKey);
            if (structuredKey !== undefined) {
                keyRecordArray.push([structuredKey, value]);
            }
        }
        return keyRecordArray.values();
    }
    forEachKey(callbackFn, thisArg) {
        return this.keyMap.forEach(callbackFn, thisArg);
    }
    forEachValue(callbackFn, thisArg) {
        return this.valueMap.forEach(callbackFn, thisArg);
    }
    get(key) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            return this.valueMap.get(stringifiedKey);
        }
        else {
            throw new Error(`${undefinedKeyError$1}: get`);
        }
    }
    has(key) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            return this.valueMap.has(stringifiedKey);
        }
        else {
            throw new Error(`${undefinedKeyError$1}: has`);
        }
    }
    keys() {
        return this.keyMap.values();
    }
    keysAsArray() {
        return Array.from(this.keys());
    }
    keysAsStrings() {
        return this.keyMap.keys();
    }
    /**
     * Merges in the values of the passed-in StoreKeyMap. Overwrites existing values.
     * @param sourceSet - The StoreKeyMap to merge in.
     */
    merge(sourceMap) {
        sourceMap.keyMap.forEach((value, key) => {
            this.keyMap.set(key, value);
        });
        sourceMap.valueMap.forEach((value, key) => {
            this.valueMap.set(key, value);
        });
    }
    set(key, value) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            this.keyMap.set(stringifiedKey, key);
            return this.valueMap.set(stringifiedKey, value);
        }
        else {
            throw new Error(`${undefinedKeyError$1}: set`);
        }
    }
    size() {
        return this.valueMap.size;
    }
    values() {
        return this.valueMap.values();
    }
}

const undefinedKeyError = 'Undefined value used in StoreKeySet operation';
class StoreKeySet {
    constructor() {
        this.set = new Set();
        this.valueMap = new Map();
    }
    add(key) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            this.set.add(stringifiedKey);
            this.valueMap.set(stringifiedKey, key);
        }
        else {
            throw new Error(`${undefinedKeyError}: add`);
        }
        return this;
    }
    clear() {
        this.set.clear();
        this.valueMap.clear();
    }
    delete(key) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            this.set.delete(stringifiedKey);
            return this.valueMap.delete(stringifiedKey);
        }
        else {
            throw new Error(`${undefinedKeyError}: delete`);
        }
    }
    entries() {
        return this.valueMap.entries();
    }
    forEach(callbackFn, thisArg) {
        return this.valueMap.forEach(callbackFn, thisArg);
    }
    has(key) {
        const stringifiedKey = serializeStructuredKey(key);
        if (stringifiedKey !== undefined) {
            return this.set.has(stringifiedKey);
        }
        else {
            throw new Error(`${undefinedKeyError}: has`);
        }
    }
    keys() {
        return this.valueMap.values();
    }
    keysAsArray() {
        return Array.from(this.keys());
    }
    keysAsStrings() {
        return this.set.keys();
    }
    /**
     * Merges in the values of the passed-in StoreKeySet. Overwrites existing values.
     * @param sourceSet - The StoreKeySet to merge in.
     */
    merge(sourceSet) {
        sourceSet.set.forEach((value) => {
            this.set.add(value);
        });
        sourceSet.valueMap.forEach((value, key) => {
            this.valueMap.set(key, value);
        });
    }
    size() {
        return this.set.size;
    }
    values() {
        return this.valueMap.values();
    }
}

function adapterToNetworkPriority(priority) {
    switch (priority) {
        case 'background':
            return 'background';
        case 'high':
            return 'high';
        case 'normal':
        default:
            return 'normal';
    }
}
var HttpStatusCode;
(function (HttpStatusCode) {
    HttpStatusCode[HttpStatusCode["Ok"] = 200] = "Ok";
    HttpStatusCode[HttpStatusCode["Created"] = 201] = "Created";
    HttpStatusCode[HttpStatusCode["NoContent"] = 204] = "NoContent";
    HttpStatusCode[HttpStatusCode["NotModified"] = 304] = "NotModified";
    HttpStatusCode[HttpStatusCode["BadRequest"] = 400] = "BadRequest";
    HttpStatusCode[HttpStatusCode["Unauthorized"] = 401] = "Unauthorized";
    HttpStatusCode[HttpStatusCode["Forbidden"] = 403] = "Forbidden";
    HttpStatusCode[HttpStatusCode["NotFound"] = 404] = "NotFound";
    HttpStatusCode[HttpStatusCode["ServerError"] = 500] = "ServerError";
    HttpStatusCode[HttpStatusCode["GatewayTimeout"] = 504] = "GatewayTimeout";
})(HttpStatusCode || (HttpStatusCode = {}));

function coerceAdapterRequestContext(adapterRequestContext) {
    const { priority, requestCorrelator, eventObservers, sourceContext } = adapterRequestContext;
    return {
        networkPriority: adapterToNetworkPriority(priority),
        requestCorrelator,
        eventObservers,
        sourceContext,
    };
}
var GraphNodeType;
(function (GraphNodeType) {
    GraphNodeType["Link"] = "Link";
    GraphNodeType["Node"] = "Node";
    GraphNodeType["Error"] = "Error";
    GraphNodeType["Locked"] = "Locked";
})(GraphNodeType || (GraphNodeType = {}));

var StoreLinkStateValues;
(function (StoreLinkStateValues) {
    StoreLinkStateValues[StoreLinkStateValues["NotPresent"] = 0] = "NotPresent";
    StoreLinkStateValues[StoreLinkStateValues["RefNotPresent"] = 1] = "RefNotPresent";
    StoreLinkStateValues[StoreLinkStateValues["RefPresent"] = 2] = "RefPresent";
    StoreLinkStateValues[StoreLinkStateValues["Null"] = 3] = "Null";
    StoreLinkStateValues[StoreLinkStateValues["Missing"] = 4] = "Missing";
    StoreLinkStateValues[StoreLinkStateValues["Pending"] = 5] = "Pending";
})(StoreLinkStateValues || (StoreLinkStateValues = {}));
var FragmentReadResultState;
(function (FragmentReadResultState) {
    FragmentReadResultState[FragmentReadResultState["Missing"] = 0] = "Missing";
    FragmentReadResultState[FragmentReadResultState["Success"] = 1] = "Success";
    FragmentReadResultState[FragmentReadResultState["Error"] = 2] = "Error";
})(FragmentReadResultState || (FragmentReadResultState = {}));
({
    state: FragmentReadResultState.Missing,
});
function resolveLink(reader, storeLink, version) {
    const { StoreLinkStateValues } = reader;
    const linkState = reader.getLinkState(storeLink);
    switch (linkState.state) {
        case StoreLinkStateValues.RefNotPresent:
        case StoreLinkStateValues.NotPresent:
        case StoreLinkStateValues.Missing:
            reader.markMissingLink(storeLink.__ref);
            reader.markMissing();
            return;
        case StoreLinkStateValues.Pending:
            reader.markPending();
            return;
        case StoreLinkStateValues.Null:
            if (process.env.NODE_ENV !== 'production') {
                throw new Error(`TODO: Invalid Link State. Link on "${reader.currentPath.fullPath}"`);
            }
            return;
    }
    const { key: __ref } = linkState;
    return reader.read({
        recordId: __ref,
        node: {
            kind: 'Fragment',
            private: [],
            opaque: true,
            version,
        },
        variables: {},
    });
}

var ResourceParamType;
(function (ResourceParamType) {
    ResourceParamType[ResourceParamType["UrlParameter"] = 0] = "UrlParameter";
    ResourceParamType[ResourceParamType["QueryParameter"] = 1] = "QueryParameter";
    ResourceParamType[ResourceParamType["Body"] = 2] = "Body";
    ResourceParamType[ResourceParamType["Header"] = 3] = "Header";
})(ResourceParamType || (ResourceParamType = {}));
var TypeCheckShapes;
(function (TypeCheckShapes) {
    TypeCheckShapes[TypeCheckShapes["String"] = 0] = "String";
    TypeCheckShapes[TypeCheckShapes["Boolean"] = 1] = "Boolean";
    TypeCheckShapes[TypeCheckShapes["Number"] = 2] = "Number";
    TypeCheckShapes[TypeCheckShapes["Integer"] = 3] = "Integer";
    TypeCheckShapes[TypeCheckShapes["Unsupported"] = 4] = "Unsupported";
})(TypeCheckShapes || (TypeCheckShapes = {}));

// Note: these should be in sync with the compiler ones:
// https://github.com/salesforce-experience-platform-emu/luvio/blob/main/packages/%40luvio/compiler/src/intermediate/resource.ts#L76-L79
const CONFIG_PROPERTY_URL_PARAMS = 'urlParams';
const CONFIG_PROPERTY_QUERY_PARAMS = 'queryParams';
const CONFIG_PROPERTY_BODY = 'body';
const CONFIG_PROPERTY_HEADERS = 'headers';
function isCorrectScalarType(value, type) {
    switch (type) {
        case TypeCheckShapes.String:
            return typeof value === 'string';
        case TypeCheckShapes.Boolean:
            return typeof value === 'boolean';
        case TypeCheckShapes.Number:
            return typeof value === 'number';
        case TypeCheckShapes.Integer:
            return typeof value === 'number' && Math.floor(value) === value;
        default:
            return false;
    }
}
function typeCheckArrayOfScalars(untrustedConfig, config, name, typeCheckShape) {
    const untrustedConfig_field = untrustedConfig[name];
    if (isArray$1(untrustedConfig_field)) {
        const untrustedConfig_field_array = [];
        for (let i = 0, arrayLength = untrustedConfig_field.length; i < arrayLength; i++) {
            const untrustedConfig_field_item = untrustedConfig_field[i];
            if (isCorrectScalarType(untrustedConfig_field_item, typeCheckShape)) {
                untrustedConfig_field_array.push(untrustedConfig_field_item);
            }
        }
        config[name] = untrustedConfig_field_array;
    }
}
function typeCheckConfig$l(untrustedConfig, config, configMetadata) {
    configMetadata.forEach(({ name, typeCheckShape, isArrayShape }) => {
        switch (typeCheckShape) {
            case TypeCheckShapes.Unsupported:
                return;
            case TypeCheckShapes.String:
            case TypeCheckShapes.Boolean:
            case TypeCheckShapes.Number:
            case TypeCheckShapes.Integer: {
                if (isArrayShape) {
                    typeCheckArrayOfScalars(untrustedConfig, config, name, typeCheckShape);
                }
                else {
                    const untrustedConfig_field = untrustedConfig[name];
                    if (isCorrectScalarType(untrustedConfig_field, typeCheckShape)) {
                        config[name] = untrustedConfig_field;
                    }
                }
                return;
            }
            default: {
                return;
            }
        }
    });
}
function coerceConfig$1(uncoercedConfig, configMetadata) {
    const config = {};
    configMetadata.forEach(({ name, coerceFn }) => {
        const value = coerceFn === undefined ? uncoercedConfig[name] : coerceFn(uncoercedConfig[name]);
        if (value !== undefined) {
            config[name] = value;
        }
    });
    return config;
}
function buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshotIdentifier, alternativeMethod, includeCacheSnapshot) {
    const { luvio, config } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
            luvioRequestMethod: alternativeMethod,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return includeCacheSnapshot
        ? buildNetworkSnapshotIdentifier(luvio, config, dispatchOptions, context.cacheSnapshot)
        : buildNetworkSnapshotIdentifier(luvio, config, dispatchOptions);
}
function ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize, namespace, version, representationName, equals) {
    const existingRecord = store.readEntry(key);
    let incomingRecord = normalize(input, existingRecord, {
        fullPath: key,
        parent: path.parent,
        propertyName: path.propertyName,
        ttl: ttlToUse,
    }, luvio, store, timestamp);
    if (existingRecord === undefined || equals(existingRecord, incomingRecord) === false) {
        luvio.storePublish(key, incomingRecord);
    }
    if (ttlToUse !== undefined) {
        const storeMetadataParams = {
            ttl: ttlToUse,
            namespace,
            version,
            representationName,
            ingestionTimestamp: timestamp,
        };
        luvio.publishStoreMetadata(key, storeMetadataParams);
    }
}
function createResourceParams$k(configMetadata) {
    return (config) => createResourceParamsImpl(config, configMetadata);
}
function createResourceParamsImpl(config, configMetadata) {
    const parametersReducer = (acc, { name }) => {
        acc[name] = config[name];
        return acc;
    };
    const urlParams = configMetadata
        .filter((p) => p.resourceType === ResourceParamType.UrlParameter)
        .reduce(parametersReducer, {});
    const queryParams = configMetadata
        .filter((p) => p.resourceType === ResourceParamType.QueryParameter)
        .reduce(parametersReducer, {});
    const headerParams = configMetadata
        .filter((p) => p.resourceType === ResourceParamType.Header)
        .reduce(parametersReducer, {});
    const bodyParams = configMetadata.filter((p) => p.resourceType === ResourceParamType.Body);
    const actualBodyParams = bodyParams.reduce((acc, { name, required }) => {
        const configValue = config[name];
        if (required) {
            acc[name] = configValue;
        }
        else if (configValue !== undefined) {
            acc[name] = configValue;
        }
        return acc;
    }, {});
    const resourceParams = {};
    if (keys$1(urlParams).length > 0) {
        resourceParams[CONFIG_PROPERTY_URL_PARAMS] = urlParams;
    }
    if (keys$1(queryParams).length > 0) {
        resourceParams[CONFIG_PROPERTY_QUERY_PARAMS] = queryParams;
    }
    if (bodyParams.length > 0) {
        resourceParams[CONFIG_PROPERTY_BODY] = actualBodyParams;
    }
    if (keys$1(headerParams).length > 0) {
        resourceParams[CONFIG_PROPERTY_HEADERS] = headerParams;
    }
    return resourceParams;
}
// engine version: 0.156.7-d2b9c7ef

/**
 * Returns true if the value acts like a Promise, i.e. has a "then" function,
 * otherwise returns false.
 */
// eslint-disable-next-line no-redeclare

function _typeof$4(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$4 = function _typeof(obj) { return typeof obj; }; } else { _typeof$4 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$4(obj); }

/**
 * Return true if `value` is object-like. A value is object-like if it's not
 * `null` and has a `typeof` result of "object".
 */
function isObjectLike(value) {
  return _typeof$4(value) == 'object' && value !== null;
}

// In ES2015 (or a polyfilled) environment, this will be Symbol.iterator
// istanbul ignore next (See: 'https://github.com/graphql/graphql-js/issues/2317')
var SYMBOL_ITERATOR = typeof Symbol === 'function' && Symbol.iterator != null ? Symbol.iterator : '@@iterator'; // In ES2017 (or a polyfilled) environment, this will be Symbol.asyncIterator

var SYMBOL_TO_STRING_TAG = typeof Symbol === 'function' && Symbol.toStringTag != null ? Symbol.toStringTag : '@@toStringTag';

/**
 * Represents a location in a Source.
 */

/**
 * Takes a Source and a UTF-8 character offset, and returns the corresponding
 * line and column as a SourceLocation.
 */
function getLocation(source, position) {
  var lineRegexp = /\r\n|[\n\r]/g;
  var line = 1;
  var column = position + 1;
  var match;

  while ((match = lineRegexp.exec(source.body)) && match.index < position) {
    line += 1;
    column = position + 1 - (match.index + match[0].length);
  }

  return {
    line: line,
    column: column
  };
}

/**
 * Render a helpful description of the location in the GraphQL Source document.
 */

function printLocation(location) {
  return printSourceLocation(location.source, getLocation(location.source, location.start));
}
/**
 * Render a helpful description of the location in the GraphQL Source document.
 */

function printSourceLocation(source, sourceLocation) {
  var firstLineColumnOffset = source.locationOffset.column - 1;
  var body = whitespace(firstLineColumnOffset) + source.body;
  var lineIndex = sourceLocation.line - 1;
  var lineOffset = source.locationOffset.line - 1;
  var lineNum = sourceLocation.line + lineOffset;
  var columnOffset = sourceLocation.line === 1 ? firstLineColumnOffset : 0;
  var columnNum = sourceLocation.column + columnOffset;
  var locationStr = "".concat(source.name, ":").concat(lineNum, ":").concat(columnNum, "\n");
  var lines = body.split(/\r\n|[\n\r]/g);
  var locationLine = lines[lineIndex]; // Special case for minified documents

  if (locationLine.length > 120) {
    var subLineIndex = Math.floor(columnNum / 80);
    var subLineColumnNum = columnNum % 80;
    var subLines = [];

    for (var i = 0; i < locationLine.length; i += 80) {
      subLines.push(locationLine.slice(i, i + 80));
    }

    return locationStr + printPrefixedLines([["".concat(lineNum), subLines[0]]].concat(subLines.slice(1, subLineIndex + 1).map(function (subLine) {
      return ['', subLine];
    }), [[' ', whitespace(subLineColumnNum - 1) + '^'], ['', subLines[subLineIndex + 1]]]));
  }

  return locationStr + printPrefixedLines([// Lines specified like this: ["prefix", "string"],
  ["".concat(lineNum - 1), lines[lineIndex - 1]], ["".concat(lineNum), locationLine], ['', whitespace(columnNum - 1) + '^'], ["".concat(lineNum + 1), lines[lineIndex + 1]]]);
}

function printPrefixedLines(lines) {
  var existingLines = lines.filter(function (_ref) {
    _ref[0];
        var line = _ref[1];
    return line !== undefined;
  });
  var padLen = Math.max.apply(Math, existingLines.map(function (_ref2) {
    var prefix = _ref2[0];
    return prefix.length;
  }));
  return existingLines.map(function (_ref3) {
    var prefix = _ref3[0],
        line = _ref3[1];
    return leftPad(padLen, prefix) + (line ? ' | ' + line : ' |');
  }).join('\n');
}

function whitespace(len) {
  return Array(len + 1).join(' ');
}

function leftPad(len, str) {
  return whitespace(len - str.length) + str;
}

function _typeof$3(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$3 = function _typeof(obj) { return typeof obj; }; } else { _typeof$3 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$3(obj); }

function ownKeys$1(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread$1(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys$1(Object(source), true).forEach(function (key) { _defineProperty$2(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys$1(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty$2(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties$4(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass$4(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties$4(Constructor.prototype, protoProps); if (staticProps) _defineProperties$4(Constructor, staticProps); return Constructor; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _createSuper(Derived) { var hasNativeReflectConstruct = _isNativeReflectConstruct(); return function _createSuperInternal() { var Super = _getPrototypeOf(Derived), result; if (hasNativeReflectConstruct) { var NewTarget = _getPrototypeOf(this).constructor; result = Reflect.construct(Super, arguments, NewTarget); } else { result = Super.apply(this, arguments); } return _possibleConstructorReturn(this, result); }; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof$3(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _wrapNativeSuper(Class) { var _cache = typeof Map === "function" ? new Map() : undefined; _wrapNativeSuper = function _wrapNativeSuper(Class) { if (Class === null || !_isNativeFunction(Class)) return Class; if (typeof Class !== "function") { throw new TypeError("Super expression must either be null or a function"); } if (typeof _cache !== "undefined") { if (_cache.has(Class)) return _cache.get(Class); _cache.set(Class, Wrapper); } function Wrapper() { return _construct(Class, arguments, _getPrototypeOf(this).constructor); } Wrapper.prototype = Object.create(Class.prototype, { constructor: { value: Wrapper, enumerable: false, writable: true, configurable: true } }); return _setPrototypeOf(Wrapper, Class); }; return _wrapNativeSuper(Class); }

function _construct(Parent, args, Class) { if (_isNativeReflectConstruct()) { _construct = Reflect.construct; } else { _construct = function _construct(Parent, args, Class) { var a = [null]; a.push.apply(a, args); var Constructor = Function.bind.apply(Parent, a); var instance = new Constructor(); if (Class) _setPrototypeOf(instance, Class.prototype); return instance; }; } return _construct.apply(null, arguments); }

function _isNativeReflectConstruct() { if (typeof Reflect === "undefined" || !Reflect.construct) return false; if (Reflect.construct.sham) return false; if (typeof Proxy === "function") return true; try { Date.prototype.toString.call(Reflect.construct(Date, [], function () {})); return true; } catch (e) { return false; } }

function _isNativeFunction(fn) { return Function.toString.call(fn).indexOf("[native code]") !== -1; }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }
/**
 * A GraphQLError describes an Error found during the parse, validate, or
 * execute phases of performing a GraphQL operation. In addition to a message
 * and stack trace, it also includes information about the locations in a
 * GraphQL document and/or execution result that correspond to the Error.
 */

var GraphQLError = /*#__PURE__*/function (_Error) {
  _inherits(GraphQLError, _Error);

  var _super = _createSuper(GraphQLError);

  /**
   * An array of { line, column } locations within the source GraphQL document
   * which correspond to this error.
   *
   * Errors during validation often contain multiple locations, for example to
   * point out two things with the same name. Errors during execution include a
   * single location, the field which produced the error.
   *
   * Enumerable, and appears in the result of JSON.stringify().
   */

  /**
   * An array describing the JSON-path into the execution response which
   * corresponds to this error. Only included for errors during execution.
   *
   * Enumerable, and appears in the result of JSON.stringify().
   */

  /**
   * An array of GraphQL AST Nodes corresponding to this error.
   */

  /**
   * The source GraphQL document for the first location of this error.
   *
   * Note that if this Error represents more than one node, the source may not
   * represent nodes after the first node.
   */

  /**
   * An array of character offsets within the source GraphQL document
   * which correspond to this error.
   */

  /**
   * The original error thrown from a field resolver during execution.
   */

  /**
   * Extension fields to add to the formatted error.
   */
  function GraphQLError(message, nodes, source, positions, path, originalError, extensions) {
    var _nodeLocations, _nodeLocations2, _nodeLocations3;

    var _this;

    _classCallCheck(this, GraphQLError);

    _this = _super.call(this, message);
    _this.name = 'GraphQLError';
    _this.originalError = originalError !== null && originalError !== void 0 ? originalError : undefined; // Compute list of blame nodes.

    _this.nodes = undefinedIfEmpty(Array.isArray(nodes) ? nodes : nodes ? [nodes] : undefined);
    var nodeLocations = [];

    for (var _i2 = 0, _ref3 = (_this$nodes = _this.nodes) !== null && _this$nodes !== void 0 ? _this$nodes : []; _i2 < _ref3.length; _i2++) {
      var _this$nodes;

      var _ref4 = _ref3[_i2];
      var loc = _ref4.loc;

      if (loc != null) {
        nodeLocations.push(loc);
      }
    }

    nodeLocations = undefinedIfEmpty(nodeLocations); // Compute locations in the source for the given nodes/positions.

    _this.source = source !== null && source !== void 0 ? source : (_nodeLocations = nodeLocations) === null || _nodeLocations === void 0 ? void 0 : _nodeLocations[0].source;
    _this.positions = positions !== null && positions !== void 0 ? positions : (_nodeLocations2 = nodeLocations) === null || _nodeLocations2 === void 0 ? void 0 : _nodeLocations2.map(function (loc) {
      return loc.start;
    });
    _this.locations = positions && source ? positions.map(function (pos) {
      return getLocation(source, pos);
    }) : (_nodeLocations3 = nodeLocations) === null || _nodeLocations3 === void 0 ? void 0 : _nodeLocations3.map(function (loc) {
      return getLocation(loc.source, loc.start);
    });
    _this.path = path !== null && path !== void 0 ? path : undefined;
    var originalExtensions = originalError === null || originalError === void 0 ? void 0 : originalError.extensions;

    if (extensions == null && isObjectLike(originalExtensions)) {
      _this.extensions = _objectSpread$1({}, originalExtensions);
    } else {
      _this.extensions = extensions !== null && extensions !== void 0 ? extensions : {};
    } // By being enumerable, JSON.stringify will include bellow properties in the resulting output.
    // This ensures that the simplest possible GraphQL service adheres to the spec.


    Object.defineProperties(_assertThisInitialized(_this), {
      message: {
        enumerable: true
      },
      locations: {
        enumerable: _this.locations != null
      },
      path: {
        enumerable: _this.path != null
      },
      extensions: {
        enumerable: _this.extensions != null && Object.keys(_this.extensions).length > 0
      },
      name: {
        enumerable: false
      },
      nodes: {
        enumerable: false
      },
      source: {
        enumerable: false
      },
      positions: {
        enumerable: false
      },
      originalError: {
        enumerable: false
      }
    }); // Include (non-enumerable) stack trace.

    if (originalError !== null && originalError !== void 0 && originalError.stack) {
      Object.defineProperty(_assertThisInitialized(_this), 'stack', {
        value: originalError.stack,
        writable: true,
        configurable: true
      });
      return _possibleConstructorReturn(_this);
    } // istanbul ignore next (See: 'https://github.com/graphql/graphql-js/issues/2317')


    if (Error.captureStackTrace) {
      Error.captureStackTrace(_assertThisInitialized(_this), GraphQLError);
    } else {
      Object.defineProperty(_assertThisInitialized(_this), 'stack', {
        value: Error().stack,
        writable: true,
        configurable: true
      });
    }

    return _this;
  }

  _createClass$4(GraphQLError, [{
    key: "toString",
    value: function toString() {
      return printError(this);
    } // FIXME: workaround to not break chai comparisons, should be remove in v16
    // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet

  }, {
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'Object';
    }
  }]);

  return GraphQLError;
}( /*#__PURE__*/_wrapNativeSuper(Error));

function undefinedIfEmpty(array) {
  return array === undefined || array.length === 0 ? undefined : array;
}
/**
 * Prints a GraphQLError to a string, representing useful location information
 * about the error's position in the source.
 */


function printError(error) {
  var output = error.message;

  if (error.nodes) {
    for (var _i4 = 0, _error$nodes2 = error.nodes; _i4 < _error$nodes2.length; _i4++) {
      var node = _error$nodes2[_i4];

      if (node.loc) {
        output += '\n\n' + printLocation(node.loc);
      }
    }
  } else if (error.source && error.locations) {
    for (var _i6 = 0, _error$locations2 = error.locations; _i6 < _error$locations2.length; _i6++) {
      var location = _error$locations2[_i6];
      output += '\n\n' + printSourceLocation(error.source, location);
    }
  }

  return output;
}

/**
 * Produces a GraphQLError representing a syntax error, containing useful
 * descriptive information about the syntax error's position in the source.
 */

function syntaxError(source, position, description) {
  return new GraphQLError("Syntax Error: ".concat(description), undefined, source, [position]);
}

/**
 * The set of allowed kind values for AST nodes.
 */
var Kind = Object.freeze({
  // Name
  NAME: 'Name',
  // Document
  DOCUMENT: 'Document',
  OPERATION_DEFINITION: 'OperationDefinition',
  VARIABLE_DEFINITION: 'VariableDefinition',
  SELECTION_SET: 'SelectionSet',
  FIELD: 'Field',
  ARGUMENT: 'Argument',
  // Fragments
  FRAGMENT_SPREAD: 'FragmentSpread',
  INLINE_FRAGMENT: 'InlineFragment',
  FRAGMENT_DEFINITION: 'FragmentDefinition',
  // Values
  VARIABLE: 'Variable',
  INT: 'IntValue',
  FLOAT: 'FloatValue',
  STRING: 'StringValue',
  BOOLEAN: 'BooleanValue',
  NULL: 'NullValue',
  ENUM: 'EnumValue',
  LIST: 'ListValue',
  OBJECT: 'ObjectValue',
  OBJECT_FIELD: 'ObjectField',
  // Directives
  DIRECTIVE: 'Directive',
  // Types
  NAMED_TYPE: 'NamedType',
  LIST_TYPE: 'ListType',
  NON_NULL_TYPE: 'NonNullType',
  // Type System Definitions
  SCHEMA_DEFINITION: 'SchemaDefinition',
  OPERATION_TYPE_DEFINITION: 'OperationTypeDefinition',
  // Type Definitions
  SCALAR_TYPE_DEFINITION: 'ScalarTypeDefinition',
  OBJECT_TYPE_DEFINITION: 'ObjectTypeDefinition',
  FIELD_DEFINITION: 'FieldDefinition',
  INPUT_VALUE_DEFINITION: 'InputValueDefinition',
  INTERFACE_TYPE_DEFINITION: 'InterfaceTypeDefinition',
  UNION_TYPE_DEFINITION: 'UnionTypeDefinition',
  ENUM_TYPE_DEFINITION: 'EnumTypeDefinition',
  ENUM_VALUE_DEFINITION: 'EnumValueDefinition',
  INPUT_OBJECT_TYPE_DEFINITION: 'InputObjectTypeDefinition',
  // Directive Definitions
  DIRECTIVE_DEFINITION: 'DirectiveDefinition',
  // Type System Extensions
  SCHEMA_EXTENSION: 'SchemaExtension',
  // Type Extensions
  SCALAR_TYPE_EXTENSION: 'ScalarTypeExtension',
  OBJECT_TYPE_EXTENSION: 'ObjectTypeExtension',
  INTERFACE_TYPE_EXTENSION: 'InterfaceTypeExtension',
  UNION_TYPE_EXTENSION: 'UnionTypeExtension',
  ENUM_TYPE_EXTENSION: 'EnumTypeExtension',
  INPUT_OBJECT_TYPE_EXTENSION: 'InputObjectTypeExtension'
});
/**
 * The enum type representing the possible kind values of AST nodes.
 */

function invariant(condition, message) {
  var booleanCondition = Boolean(condition); // istanbul ignore else (See transformation done in './resources/inlineInvariant.js')

  if (!booleanCondition) {
    throw new Error(message != null ? message : 'Unexpected invariant triggered.');
  }
}

// istanbul ignore next (See: 'https://github.com/graphql/graphql-js/issues/2317')
var nodejsCustomInspectSymbol = typeof Symbol === 'function' && typeof Symbol.for === 'function' ? Symbol.for('nodejs.util.inspect.custom') : undefined;
var nodejsCustomInspectSymbol$1 = nodejsCustomInspectSymbol;

/**
 * The `defineInspect()` function defines `inspect()` prototype method as alias of `toJSON`
 */

function defineInspect(classObject) {
  var fn = classObject.prototype.toJSON;
  typeof fn === 'function' || invariant(0);
  classObject.prototype.inspect = fn; // istanbul ignore else (See: 'https://github.com/graphql/graphql-js/issues/2317')

  if (nodejsCustomInspectSymbol$1) {
    classObject.prototype[nodejsCustomInspectSymbol$1] = fn;
  }
}

/**
 * Contains a range of UTF-8 character offsets and token references that
 * identify the region of the source from which the AST derived.
 */
var Location = /*#__PURE__*/function () {
  /**
   * The character offset at which this Node begins.
   */

  /**
   * The character offset at which this Node ends.
   */

  /**
   * The Token at which this Node begins.
   */

  /**
   * The Token at which this Node ends.
   */

  /**
   * The Source document the AST represents.
   */
  function Location(startToken, endToken, source) {
    this.start = startToken.start;
    this.end = endToken.end;
    this.startToken = startToken;
    this.endToken = endToken;
    this.source = source;
  }

  var _proto = Location.prototype;

  _proto.toJSON = function toJSON() {
    return {
      start: this.start,
      end: this.end
    };
  };

  return Location;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(Location);
/**
 * Represents a range of characters represented by a lexical token
 * within a Source.
 */

var Token = /*#__PURE__*/function () {
  /**
   * The kind of Token.
   */

  /**
   * The character offset at which this Node begins.
   */

  /**
   * The character offset at which this Node ends.
   */

  /**
   * The 1-indexed line number on which this Token appears.
   */

  /**
   * The 1-indexed column number at which this Token begins.
   */

  /**
   * For non-punctuation tokens, represents the interpreted value of the token.
   */

  /**
   * Tokens exist as nodes in a double-linked-list amongst all tokens
   * including ignored tokens. <SOF> is always the first node and <EOF>
   * the last.
   */
  function Token(kind, start, end, line, column, prev, value) {
    this.kind = kind;
    this.start = start;
    this.end = end;
    this.line = line;
    this.column = column;
    this.value = value;
    this.prev = prev;
    this.next = null;
  }

  var _proto2 = Token.prototype;

  _proto2.toJSON = function toJSON() {
    return {
      kind: this.kind,
      value: this.value,
      line: this.line,
      column: this.column
    };
  };

  return Token;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(Token);
/**
 * @internal
 */

function isNode(maybeNode) {
  return maybeNode != null && typeof maybeNode.kind === 'string';
}
/**
 * The list of all possible AST node types.
 */

/**
 * An exported enum describing the different kinds of tokens that the
 * lexer emits.
 */
var TokenKind = Object.freeze({
  SOF: '<SOF>',
  EOF: '<EOF>',
  BANG: '!',
  DOLLAR: '$',
  AMP: '&',
  PAREN_L: '(',
  PAREN_R: ')',
  SPREAD: '...',
  COLON: ':',
  EQUALS: '=',
  AT: '@',
  BRACKET_L: '[',
  BRACKET_R: ']',
  BRACE_L: '{',
  PIPE: '|',
  BRACE_R: '}',
  NAME: 'Name',
  INT: 'Int',
  FLOAT: 'Float',
  STRING: 'String',
  BLOCK_STRING: 'BlockString',
  COMMENT: 'Comment'
});
/**
 * The enum type representing the token kinds values.
 */

function _typeof$2(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$2 = function _typeof(obj) { return typeof obj; }; } else { _typeof$2 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$2(obj); }
var MAX_ARRAY_LENGTH = 10;
var MAX_RECURSIVE_DEPTH = 2;
/**
 * Used to print values in error messages.
 */

function inspect(value) {
  return formatValue(value, []);
}

function formatValue(value, seenValues) {
  switch (_typeof$2(value)) {
    case 'string':
      return JSON.stringify(value);

    case 'function':
      return value.name ? "[function ".concat(value.name, "]") : '[function]';

    case 'object':
      if (value === null) {
        return 'null';
      }

      return formatObjectValue(value, seenValues);

    default:
      return String(value);
  }
}

function formatObjectValue(value, previouslySeenValues) {
  if (previouslySeenValues.indexOf(value) !== -1) {
    return '[Circular]';
  }

  var seenValues = [].concat(previouslySeenValues, [value]);
  var customInspectFn = getCustomFn(value);

  if (customInspectFn !== undefined) {
    var customValue = customInspectFn.call(value); // check for infinite recursion

    if (customValue !== value) {
      return typeof customValue === 'string' ? customValue : formatValue(customValue, seenValues);
    }
  } else if (Array.isArray(value)) {
    return formatArray(value, seenValues);
  }

  return formatObject(value, seenValues);
}

function formatObject(object, seenValues) {
  var keys = Object.keys(object);

  if (keys.length === 0) {
    return '{}';
  }

  if (seenValues.length > MAX_RECURSIVE_DEPTH) {
    return '[' + getObjectTag(object) + ']';
  }

  var properties = keys.map(function (key) {
    var value = formatValue(object[key], seenValues);
    return key + ': ' + value;
  });
  return '{ ' + properties.join(', ') + ' }';
}

function formatArray(array, seenValues) {
  if (array.length === 0) {
    return '[]';
  }

  if (seenValues.length > MAX_RECURSIVE_DEPTH) {
    return '[Array]';
  }

  var len = Math.min(MAX_ARRAY_LENGTH, array.length);
  var remaining = array.length - len;
  var items = [];

  for (var i = 0; i < len; ++i) {
    items.push(formatValue(array[i], seenValues));
  }

  if (remaining === 1) {
    items.push('... 1 more item');
  } else if (remaining > 1) {
    items.push("... ".concat(remaining, " more items"));
  }

  return '[' + items.join(', ') + ']';
}

function getCustomFn(object) {
  var customInspectFn = object[String(nodejsCustomInspectSymbol$1)];

  if (typeof customInspectFn === 'function') {
    return customInspectFn;
  }

  if (typeof object.inspect === 'function') {
    return object.inspect;
  }
}

function getObjectTag(object) {
  var tag = Object.prototype.toString.call(object).replace(/^\[object /, '').replace(/]$/, '');

  if (tag === 'Object' && typeof object.constructor === 'function') {
    var name = object.constructor.name;

    if (typeof name === 'string' && name !== '') {
      return name;
    }
  }

  return tag;
}

function devAssert(condition, message) {
  var booleanCondition = Boolean(condition); // istanbul ignore else (See transformation done in './resources/inlineInvariant.js')

  if (!booleanCondition) {
    throw new Error(message);
  }
}

function _typeof$1(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof$1 = function _typeof(obj) { return typeof obj; }; } else { _typeof$1 = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof$1(obj); }
/**
 * A replacement for instanceof which includes an error warning when multi-realm
 * constructors are detected.
 */

// See: https://expressjs.com/en/advanced/best-practice-performance.html#set-node_env-to-production
// See: https://webpack.js.org/guides/production/
var instanceOf = process.env.NODE_ENV === 'production' ? // istanbul ignore next (See: 'https://github.com/graphql/graphql-js/issues/2317')
// eslint-disable-next-line no-shadow
function instanceOf(value, constructor) {
  return value instanceof constructor;
} : // eslint-disable-next-line no-shadow
function instanceOf(value, constructor) {
  if (value instanceof constructor) {
    return true;
  }

  if (_typeof$1(value) === 'object' && value !== null) {
    var _value$constructor;

    var className = constructor.prototype[Symbol.toStringTag];
    var valueClassName = // We still need to support constructor's name to detect conflicts with older versions of this library.
    Symbol.toStringTag in value ? value[Symbol.toStringTag] : (_value$constructor = value.constructor) === null || _value$constructor === void 0 ? void 0 : _value$constructor.name;

    if (className === valueClassName) {
      var stringifiedValue = inspect(value);
      throw new Error("Cannot use ".concat(className, " \"").concat(stringifiedValue, "\" from another module or realm.\n\nEnsure that there is only one instance of \"graphql\" in the node_modules\ndirectory. If different versions of \"graphql\" are the dependencies of other\nrelied on modules, use \"resolutions\" to ensure only one version is installed.\n\nhttps://yarnpkg.com/en/docs/selective-version-resolutions\n\nDuplicate \"graphql\" modules cannot be used at the same time since different\nversions may have different capabilities and behavior. The data from one\nversion used in the function from another could produce confusing and\nspurious results."));
    }
  }

  return false;
};

function _defineProperties$3(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass$3(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties$3(Constructor.prototype, protoProps); if (staticProps) _defineProperties$3(Constructor, staticProps); return Constructor; }

/**
 * A representation of source input to GraphQL. The `name` and `locationOffset` parameters are
 * optional, but they are useful for clients who store GraphQL documents in source files.
 * For example, if the GraphQL input starts at line 40 in a file named `Foo.graphql`, it might
 * be useful for `name` to be `"Foo.graphql"` and location to be `{ line: 40, column: 1 }`.
 * The `line` and `column` properties in `locationOffset` are 1-indexed.
 */
var Source = /*#__PURE__*/function () {
  function Source(body) {
    var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'GraphQL request';
    var locationOffset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
      line: 1,
      column: 1
    };
    typeof body === 'string' || devAssert(0, "Body must be a string. Received: ".concat(inspect(body), "."));
    this.body = body;
    this.name = name;
    this.locationOffset = locationOffset;
    this.locationOffset.line > 0 || devAssert(0, 'line in locationOffset is 1-indexed and must be positive.');
    this.locationOffset.column > 0 || devAssert(0, 'column in locationOffset is 1-indexed and must be positive.');
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet


  _createClass$3(Source, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'Source';
    }
  }]);

  return Source;
}();
/**
 * Test if the given value is a Source object.
 *
 * @internal
 */

// eslint-disable-next-line no-redeclare
function isSource(source) {
  return instanceOf(source, Source);
}

/**
 * The set of allowed directive location values.
 */
var DirectiveLocation = Object.freeze({
  // Request Definitions
  QUERY: 'QUERY',
  MUTATION: 'MUTATION',
  SUBSCRIPTION: 'SUBSCRIPTION',
  FIELD: 'FIELD',
  FRAGMENT_DEFINITION: 'FRAGMENT_DEFINITION',
  FRAGMENT_SPREAD: 'FRAGMENT_SPREAD',
  INLINE_FRAGMENT: 'INLINE_FRAGMENT',
  VARIABLE_DEFINITION: 'VARIABLE_DEFINITION',
  // Type System Definitions
  SCHEMA: 'SCHEMA',
  SCALAR: 'SCALAR',
  OBJECT: 'OBJECT',
  FIELD_DEFINITION: 'FIELD_DEFINITION',
  ARGUMENT_DEFINITION: 'ARGUMENT_DEFINITION',
  INTERFACE: 'INTERFACE',
  UNION: 'UNION',
  ENUM: 'ENUM',
  ENUM_VALUE: 'ENUM_VALUE',
  INPUT_OBJECT: 'INPUT_OBJECT',
  INPUT_FIELD_DEFINITION: 'INPUT_FIELD_DEFINITION'
});
/**
 * The enum type representing the directive location values.
 */

/**
 * Produces the value of a block string from its parsed raw value, similar to
 * CoffeeScript's block string, Python's docstring trim or Ruby's strip_heredoc.
 *
 * This implements the GraphQL spec's BlockStringValue() static algorithm.
 *
 * @internal
 */
function dedentBlockStringValue(rawString) {
  // Expand a block string's raw value into independent lines.
  var lines = rawString.split(/\r\n|[\n\r]/g); // Remove common indentation from all lines but first.

  var commonIndent = getBlockStringIndentation(rawString);

  if (commonIndent !== 0) {
    for (var i = 1; i < lines.length; i++) {
      lines[i] = lines[i].slice(commonIndent);
    }
  } // Remove leading and trailing blank lines.


  var startLine = 0;

  while (startLine < lines.length && isBlank(lines[startLine])) {
    ++startLine;
  }

  var endLine = lines.length;

  while (endLine > startLine && isBlank(lines[endLine - 1])) {
    --endLine;
  } // Return a string of the lines joined with U+000A.


  return lines.slice(startLine, endLine).join('\n');
}

function isBlank(str) {
  for (var i = 0; i < str.length; ++i) {
    if (str[i] !== ' ' && str[i] !== '\t') {
      return false;
    }
  }

  return true;
}
/**
 * @internal
 */


function getBlockStringIndentation(value) {
  var _commonIndent;

  var isFirstLine = true;
  var isEmptyLine = true;
  var indent = 0;
  var commonIndent = null;

  for (var i = 0; i < value.length; ++i) {
    switch (value.charCodeAt(i)) {
      case 13:
        //  \r
        if (value.charCodeAt(i + 1) === 10) {
          ++i; // skip \r\n as one symbol
        }

      // falls through

      case 10:
        //  \n
        isFirstLine = false;
        isEmptyLine = true;
        indent = 0;
        break;

      case 9: //   \t

      case 32:
        //  <space>
        ++indent;
        break;

      default:
        if (isEmptyLine && !isFirstLine && (commonIndent === null || indent < commonIndent)) {
          commonIndent = indent;
        }

        isEmptyLine = false;
    }
  }

  return (_commonIndent = commonIndent) !== null && _commonIndent !== void 0 ? _commonIndent : 0;
}
/**
 * Print a block string in the indented block form by adding a leading and
 * trailing blank line. However, if a block string starts with whitespace and is
 * a single-line, adding a leading blank line would strip that whitespace.
 *
 * @internal
 */

function printBlockString(value) {
  var indentation = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var preferMultipleLines = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var isSingleLine = value.indexOf('\n') === -1;
  var hasLeadingSpace = value[0] === ' ' || value[0] === '\t';
  var hasTrailingQuote = value[value.length - 1] === '"';
  var hasTrailingSlash = value[value.length - 1] === '\\';
  var printAsMultipleLines = !isSingleLine || hasTrailingQuote || hasTrailingSlash || preferMultipleLines;
  var result = ''; // Format a multi-line block quote to account for leading space.

  if (printAsMultipleLines && !(isSingleLine && hasLeadingSpace)) {
    result += '\n' + indentation;
  }

  result += indentation ? value.replace(/\n/g, '\n' + indentation) : value;

  if (printAsMultipleLines) {
    result += '\n';
  }

  return '"""' + result.replace(/"""/g, '\\"""') + '"""';
}

/**
 * Given a Source object, creates a Lexer for that source.
 * A Lexer is a stateful stream generator in that every time
 * it is advanced, it returns the next token in the Source. Assuming the
 * source lexes, the final Token emitted by the lexer will be of kind
 * EOF, after which the lexer will repeatedly return the same EOF token
 * whenever called.
 */

var Lexer = /*#__PURE__*/function () {
  /**
   * The previously focused non-ignored token.
   */

  /**
   * The currently focused non-ignored token.
   */

  /**
   * The (1-indexed) line containing the current token.
   */

  /**
   * The character offset at which the current line begins.
   */
  function Lexer(source) {
    var startOfFileToken = new Token(TokenKind.SOF, 0, 0, 0, 0, null);
    this.source = source;
    this.lastToken = startOfFileToken;
    this.token = startOfFileToken;
    this.line = 1;
    this.lineStart = 0;
  }
  /**
   * Advances the token stream to the next non-ignored token.
   */


  var _proto = Lexer.prototype;

  _proto.advance = function advance() {
    this.lastToken = this.token;
    var token = this.token = this.lookahead();
    return token;
  }
  /**
   * Looks ahead and returns the next non-ignored token, but does not change
   * the state of Lexer.
   */
  ;

  _proto.lookahead = function lookahead() {
    var token = this.token;

    if (token.kind !== TokenKind.EOF) {
      do {
        var _token$next;

        // Note: next is only mutable during parsing, so we cast to allow this.
        token = (_token$next = token.next) !== null && _token$next !== void 0 ? _token$next : token.next = readToken(this, token);
      } while (token.kind === TokenKind.COMMENT);
    }

    return token;
  };

  return Lexer;
}();
/**
 * @internal
 */

function isPunctuatorTokenKind(kind) {
  return kind === TokenKind.BANG || kind === TokenKind.DOLLAR || kind === TokenKind.AMP || kind === TokenKind.PAREN_L || kind === TokenKind.PAREN_R || kind === TokenKind.SPREAD || kind === TokenKind.COLON || kind === TokenKind.EQUALS || kind === TokenKind.AT || kind === TokenKind.BRACKET_L || kind === TokenKind.BRACKET_R || kind === TokenKind.BRACE_L || kind === TokenKind.PIPE || kind === TokenKind.BRACE_R;
}

function printCharCode(code) {
  return (// NaN/undefined represents access beyond the end of the file.
    isNaN(code) ? TokenKind.EOF : // Trust JSON for ASCII.
    code < 0x007f ? JSON.stringify(String.fromCharCode(code)) : // Otherwise print the escaped form.
    "\"\\u".concat(('00' + code.toString(16).toUpperCase()).slice(-4), "\"")
  );
}
/**
 * Gets the next token from the source starting at the given position.
 *
 * This skips over whitespace until it finds the next lexable token, then lexes
 * punctuators immediately or calls the appropriate helper function for more
 * complicated tokens.
 */


function readToken(lexer, prev) {
  var source = lexer.source;
  var body = source.body;
  var bodyLength = body.length;
  var pos = prev.end;

  while (pos < bodyLength) {
    var code = body.charCodeAt(pos);
    var _line = lexer.line;

    var _col = 1 + pos - lexer.lineStart; // SourceCharacter


    switch (code) {
      case 0xfeff: // <BOM>

      case 9: //   \t

      case 32: //  <space>

      case 44:
        //  ,
        ++pos;
        continue;

      case 10:
        //  \n
        ++pos;
        ++lexer.line;
        lexer.lineStart = pos;
        continue;

      case 13:
        //  \r
        if (body.charCodeAt(pos + 1) === 10) {
          pos += 2;
        } else {
          ++pos;
        }

        ++lexer.line;
        lexer.lineStart = pos;
        continue;

      case 33:
        //  !
        return new Token(TokenKind.BANG, pos, pos + 1, _line, _col, prev);

      case 35:
        //  #
        return readComment(source, pos, _line, _col, prev);

      case 36:
        //  $
        return new Token(TokenKind.DOLLAR, pos, pos + 1, _line, _col, prev);

      case 38:
        //  &
        return new Token(TokenKind.AMP, pos, pos + 1, _line, _col, prev);

      case 40:
        //  (
        return new Token(TokenKind.PAREN_L, pos, pos + 1, _line, _col, prev);

      case 41:
        //  )
        return new Token(TokenKind.PAREN_R, pos, pos + 1, _line, _col, prev);

      case 46:
        //  .
        if (body.charCodeAt(pos + 1) === 46 && body.charCodeAt(pos + 2) === 46) {
          return new Token(TokenKind.SPREAD, pos, pos + 3, _line, _col, prev);
        }

        break;

      case 58:
        //  :
        return new Token(TokenKind.COLON, pos, pos + 1, _line, _col, prev);

      case 61:
        //  =
        return new Token(TokenKind.EQUALS, pos, pos + 1, _line, _col, prev);

      case 64:
        //  @
        return new Token(TokenKind.AT, pos, pos + 1, _line, _col, prev);

      case 91:
        //  [
        return new Token(TokenKind.BRACKET_L, pos, pos + 1, _line, _col, prev);

      case 93:
        //  ]
        return new Token(TokenKind.BRACKET_R, pos, pos + 1, _line, _col, prev);

      case 123:
        // {
        return new Token(TokenKind.BRACE_L, pos, pos + 1, _line, _col, prev);

      case 124:
        // |
        return new Token(TokenKind.PIPE, pos, pos + 1, _line, _col, prev);

      case 125:
        // }
        return new Token(TokenKind.BRACE_R, pos, pos + 1, _line, _col, prev);

      case 34:
        //  "
        if (body.charCodeAt(pos + 1) === 34 && body.charCodeAt(pos + 2) === 34) {
          return readBlockString(source, pos, _line, _col, prev, lexer);
        }

        return readString(source, pos, _line, _col, prev);

      case 45: //  -

      case 48: //  0

      case 49: //  1

      case 50: //  2

      case 51: //  3

      case 52: //  4

      case 53: //  5

      case 54: //  6

      case 55: //  7

      case 56: //  8

      case 57:
        //  9
        return readNumber(source, pos, code, _line, _col, prev);

      case 65: //  A

      case 66: //  B

      case 67: //  C

      case 68: //  D

      case 69: //  E

      case 70: //  F

      case 71: //  G

      case 72: //  H

      case 73: //  I

      case 74: //  J

      case 75: //  K

      case 76: //  L

      case 77: //  M

      case 78: //  N

      case 79: //  O

      case 80: //  P

      case 81: //  Q

      case 82: //  R

      case 83: //  S

      case 84: //  T

      case 85: //  U

      case 86: //  V

      case 87: //  W

      case 88: //  X

      case 89: //  Y

      case 90: //  Z

      case 95: //  _

      case 97: //  a

      case 98: //  b

      case 99: //  c

      case 100: // d

      case 101: // e

      case 102: // f

      case 103: // g

      case 104: // h

      case 105: // i

      case 106: // j

      case 107: // k

      case 108: // l

      case 109: // m

      case 110: // n

      case 111: // o

      case 112: // p

      case 113: // q

      case 114: // r

      case 115: // s

      case 116: // t

      case 117: // u

      case 118: // v

      case 119: // w

      case 120: // x

      case 121: // y

      case 122:
        // z
        return readName(source, pos, _line, _col, prev);
    }

    throw syntaxError(source, pos, unexpectedCharacterMessage(code));
  }

  var line = lexer.line;
  var col = 1 + pos - lexer.lineStart;
  return new Token(TokenKind.EOF, bodyLength, bodyLength, line, col, prev);
}
/**
 * Report a message that an unexpected character was encountered.
 */


function unexpectedCharacterMessage(code) {
  if (code < 0x0020 && code !== 0x0009 && code !== 0x000a && code !== 0x000d) {
    return "Cannot contain the invalid character ".concat(printCharCode(code), ".");
  }

  if (code === 39) {
    // '
    return 'Unexpected single quote character (\'), did you mean to use a double quote (")?';
  }

  return "Cannot parse the unexpected character ".concat(printCharCode(code), ".");
}
/**
 * Reads a comment token from the source file.
 *
 * #[\u0009\u0020-\uFFFF]*
 */


function readComment(source, start, line, col, prev) {
  var body = source.body;
  var code;
  var position = start;

  do {
    code = body.charCodeAt(++position);
  } while (!isNaN(code) && ( // SourceCharacter but not LineTerminator
  code > 0x001f || code === 0x0009));

  return new Token(TokenKind.COMMENT, start, position, line, col, prev, body.slice(start + 1, position));
}
/**
 * Reads a number token from the source file, either a float
 * or an int depending on whether a decimal point appears.
 *
 * Int:   -?(0|[1-9][0-9]*)
 * Float: -?(0|[1-9][0-9]*)(\.[0-9]+)?((E|e)(+|-)?[0-9]+)?
 */


function readNumber(source, start, firstCode, line, col, prev) {
  var body = source.body;
  var code = firstCode;
  var position = start;
  var isFloat = false;

  if (code === 45) {
    // -
    code = body.charCodeAt(++position);
  }

  if (code === 48) {
    // 0
    code = body.charCodeAt(++position);

    if (code >= 48 && code <= 57) {
      throw syntaxError(source, position, "Invalid number, unexpected digit after 0: ".concat(printCharCode(code), "."));
    }
  } else {
    position = readDigits(source, position, code);
    code = body.charCodeAt(position);
  }

  if (code === 46) {
    // .
    isFloat = true;
    code = body.charCodeAt(++position);
    position = readDigits(source, position, code);
    code = body.charCodeAt(position);
  }

  if (code === 69 || code === 101) {
    // E e
    isFloat = true;
    code = body.charCodeAt(++position);

    if (code === 43 || code === 45) {
      // + -
      code = body.charCodeAt(++position);
    }

    position = readDigits(source, position, code);
    code = body.charCodeAt(position);
  } // Numbers cannot be followed by . or NameStart


  if (code === 46 || isNameStart(code)) {
    throw syntaxError(source, position, "Invalid number, expected digit but got: ".concat(printCharCode(code), "."));
  }

  return new Token(isFloat ? TokenKind.FLOAT : TokenKind.INT, start, position, line, col, prev, body.slice(start, position));
}
/**
 * Returns the new position in the source after reading digits.
 */


function readDigits(source, start, firstCode) {
  var body = source.body;
  var position = start;
  var code = firstCode;

  if (code >= 48 && code <= 57) {
    // 0 - 9
    do {
      code = body.charCodeAt(++position);
    } while (code >= 48 && code <= 57); // 0 - 9


    return position;
  }

  throw syntaxError(source, position, "Invalid number, expected digit but got: ".concat(printCharCode(code), "."));
}
/**
 * Reads a string token from the source file.
 *
 * "([^"\\\u000A\u000D]|(\\(u[0-9a-fA-F]{4}|["\\/bfnrt])))*"
 */


function readString(source, start, line, col, prev) {
  var body = source.body;
  var position = start + 1;
  var chunkStart = position;
  var code = 0;
  var value = '';

  while (position < body.length && !isNaN(code = body.charCodeAt(position)) && // not LineTerminator
  code !== 0x000a && code !== 0x000d) {
    // Closing Quote (")
    if (code === 34) {
      value += body.slice(chunkStart, position);
      return new Token(TokenKind.STRING, start, position + 1, line, col, prev, value);
    } // SourceCharacter


    if (code < 0x0020 && code !== 0x0009) {
      throw syntaxError(source, position, "Invalid character within String: ".concat(printCharCode(code), "."));
    }

    ++position;

    if (code === 92) {
      // \
      value += body.slice(chunkStart, position - 1);
      code = body.charCodeAt(position);

      switch (code) {
        case 34:
          value += '"';
          break;

        case 47:
          value += '/';
          break;

        case 92:
          value += '\\';
          break;

        case 98:
          value += '\b';
          break;

        case 102:
          value += '\f';
          break;

        case 110:
          value += '\n';
          break;

        case 114:
          value += '\r';
          break;

        case 116:
          value += '\t';
          break;

        case 117:
          {
            // uXXXX
            var charCode = uniCharCode(body.charCodeAt(position + 1), body.charCodeAt(position + 2), body.charCodeAt(position + 3), body.charCodeAt(position + 4));

            if (charCode < 0) {
              var invalidSequence = body.slice(position + 1, position + 5);
              throw syntaxError(source, position, "Invalid character escape sequence: \\u".concat(invalidSequence, "."));
            }

            value += String.fromCharCode(charCode);
            position += 4;
            break;
          }

        default:
          throw syntaxError(source, position, "Invalid character escape sequence: \\".concat(String.fromCharCode(code), "."));
      }

      ++position;
      chunkStart = position;
    }
  }

  throw syntaxError(source, position, 'Unterminated string.');
}
/**
 * Reads a block string token from the source file.
 *
 * """("?"?(\\"""|\\(?!=""")|[^"\\]))*"""
 */


function readBlockString(source, start, line, col, prev, lexer) {
  var body = source.body;
  var position = start + 3;
  var chunkStart = position;
  var code = 0;
  var rawValue = '';

  while (position < body.length && !isNaN(code = body.charCodeAt(position))) {
    // Closing Triple-Quote (""")
    if (code === 34 && body.charCodeAt(position + 1) === 34 && body.charCodeAt(position + 2) === 34) {
      rawValue += body.slice(chunkStart, position);
      return new Token(TokenKind.BLOCK_STRING, start, position + 3, line, col, prev, dedentBlockStringValue(rawValue));
    } // SourceCharacter


    if (code < 0x0020 && code !== 0x0009 && code !== 0x000a && code !== 0x000d) {
      throw syntaxError(source, position, "Invalid character within String: ".concat(printCharCode(code), "."));
    }

    if (code === 10) {
      // new line
      ++position;
      ++lexer.line;
      lexer.lineStart = position;
    } else if (code === 13) {
      // carriage return
      if (body.charCodeAt(position + 1) === 10) {
        position += 2;
      } else {
        ++position;
      }

      ++lexer.line;
      lexer.lineStart = position;
    } else if ( // Escape Triple-Quote (\""")
    code === 92 && body.charCodeAt(position + 1) === 34 && body.charCodeAt(position + 2) === 34 && body.charCodeAt(position + 3) === 34) {
      rawValue += body.slice(chunkStart, position) + '"""';
      position += 4;
      chunkStart = position;
    } else {
      ++position;
    }
  }

  throw syntaxError(source, position, 'Unterminated string.');
}
/**
 * Converts four hexadecimal chars to the integer that the
 * string represents. For example, uniCharCode('0','0','0','f')
 * will return 15, and uniCharCode('0','0','f','f') returns 255.
 *
 * Returns a negative number on error, if a char was invalid.
 *
 * This is implemented by noting that char2hex() returns -1 on error,
 * which means the result of ORing the char2hex() will also be negative.
 */


function uniCharCode(a, b, c, d) {
  return char2hex(a) << 12 | char2hex(b) << 8 | char2hex(c) << 4 | char2hex(d);
}
/**
 * Converts a hex character to its integer value.
 * '0' becomes 0, '9' becomes 9
 * 'A' becomes 10, 'F' becomes 15
 * 'a' becomes 10, 'f' becomes 15
 *
 * Returns -1 on error.
 */


function char2hex(a) {
  return a >= 48 && a <= 57 ? a - 48 // 0-9
  : a >= 65 && a <= 70 ? a - 55 // A-F
  : a >= 97 && a <= 102 ? a - 87 // a-f
  : -1;
}
/**
 * Reads an alphanumeric + underscore name from the source.
 *
 * [_A-Za-z][_0-9A-Za-z]*
 */


function readName(source, start, line, col, prev) {
  var body = source.body;
  var bodyLength = body.length;
  var position = start + 1;
  var code = 0;

  while (position !== bodyLength && !isNaN(code = body.charCodeAt(position)) && (code === 95 || // _
  code >= 48 && code <= 57 || // 0-9
  code >= 65 && code <= 90 || // A-Z
  code >= 97 && code <= 122) // a-z
  ) {
    ++position;
  }

  return new Token(TokenKind.NAME, start, position, line, col, prev, body.slice(start, position));
} // _ A-Z a-z


function isNameStart(code) {
  return code === 95 || code >= 65 && code <= 90 || code >= 97 && code <= 122;
}

/**
 * Configuration options to control parser behavior
 */

/**
 * Given a GraphQL source, parses it into a Document.
 * Throws GraphQLError if a syntax error is encountered.
 */
function parse$1(source, options) {
  var parser = new Parser(source, options);
  return parser.parseDocument();
}
/**
 * This class is exported only to assist people in implementing their own parsers
 * without duplicating too much code and should be used only as last resort for cases
 * such as experimental syntax or if certain features could not be contributed upstream.
 *
 * It is still part of the internal API and is versioned, so any changes to it are never
 * considered breaking changes. If you still need to support multiple versions of the
 * library, please use the `versionInfo` variable for version detection.
 *
 * @internal
 */

var Parser = /*#__PURE__*/function () {
  function Parser(source, options) {
    var sourceObj = isSource(source) ? source : new Source(source);
    this._lexer = new Lexer(sourceObj);
    this._options = options;
  }
  /**
   * Converts a name lex token into a name parse node.
   */


  var _proto = Parser.prototype;

  _proto.parseName = function parseName() {
    var token = this.expectToken(TokenKind.NAME);
    return {
      kind: Kind.NAME,
      value: token.value,
      loc: this.loc(token)
    };
  } // Implements the parsing rules in the Document section.

  /**
   * Document : Definition+
   */
  ;

  _proto.parseDocument = function parseDocument() {
    var start = this._lexer.token;
    return {
      kind: Kind.DOCUMENT,
      definitions: this.many(TokenKind.SOF, this.parseDefinition, TokenKind.EOF),
      loc: this.loc(start)
    };
  }
  /**
   * Definition :
   *   - ExecutableDefinition
   *   - TypeSystemDefinition
   *   - TypeSystemExtension
   *
   * ExecutableDefinition :
   *   - OperationDefinition
   *   - FragmentDefinition
   */
  ;

  _proto.parseDefinition = function parseDefinition() {
    if (this.peek(TokenKind.NAME)) {
      switch (this._lexer.token.value) {
        case 'query':
        case 'mutation':
        case 'subscription':
          return this.parseOperationDefinition();

        case 'fragment':
          return this.parseFragmentDefinition();

        case 'schema':
        case 'scalar':
        case 'type':
        case 'interface':
        case 'union':
        case 'enum':
        case 'input':
        case 'directive':
          return this.parseTypeSystemDefinition();

        case 'extend':
          return this.parseTypeSystemExtension();
      }
    } else if (this.peek(TokenKind.BRACE_L)) {
      return this.parseOperationDefinition();
    } else if (this.peekDescription()) {
      return this.parseTypeSystemDefinition();
    }

    throw this.unexpected();
  } // Implements the parsing rules in the Operations section.

  /**
   * OperationDefinition :
   *  - SelectionSet
   *  - OperationType Name? VariableDefinitions? Directives? SelectionSet
   */
  ;

  _proto.parseOperationDefinition = function parseOperationDefinition() {
    var start = this._lexer.token;

    if (this.peek(TokenKind.BRACE_L)) {
      return {
        kind: Kind.OPERATION_DEFINITION,
        operation: 'query',
        name: undefined,
        variableDefinitions: [],
        directives: [],
        selectionSet: this.parseSelectionSet(),
        loc: this.loc(start)
      };
    }

    var operation = this.parseOperationType();
    var name;

    if (this.peek(TokenKind.NAME)) {
      name = this.parseName();
    }

    return {
      kind: Kind.OPERATION_DEFINITION,
      operation: operation,
      name: name,
      variableDefinitions: this.parseVariableDefinitions(),
      directives: this.parseDirectives(false),
      selectionSet: this.parseSelectionSet(),
      loc: this.loc(start)
    };
  }
  /**
   * OperationType : one of query mutation subscription
   */
  ;

  _proto.parseOperationType = function parseOperationType() {
    var operationToken = this.expectToken(TokenKind.NAME);

    switch (operationToken.value) {
      case 'query':
        return 'query';

      case 'mutation':
        return 'mutation';

      case 'subscription':
        return 'subscription';
    }

    throw this.unexpected(operationToken);
  }
  /**
   * VariableDefinitions : ( VariableDefinition+ )
   */
  ;

  _proto.parseVariableDefinitions = function parseVariableDefinitions() {
    return this.optionalMany(TokenKind.PAREN_L, this.parseVariableDefinition, TokenKind.PAREN_R);
  }
  /**
   * VariableDefinition : Variable : Type DefaultValue? Directives[Const]?
   */
  ;

  _proto.parseVariableDefinition = function parseVariableDefinition() {
    var start = this._lexer.token;
    return {
      kind: Kind.VARIABLE_DEFINITION,
      variable: this.parseVariable(),
      type: (this.expectToken(TokenKind.COLON), this.parseTypeReference()),
      defaultValue: this.expectOptionalToken(TokenKind.EQUALS) ? this.parseValueLiteral(true) : undefined,
      directives: this.parseDirectives(true),
      loc: this.loc(start)
    };
  }
  /**
   * Variable : $ Name
   */
  ;

  _proto.parseVariable = function parseVariable() {
    var start = this._lexer.token;
    this.expectToken(TokenKind.DOLLAR);
    return {
      kind: Kind.VARIABLE,
      name: this.parseName(),
      loc: this.loc(start)
    };
  }
  /**
   * SelectionSet : { Selection+ }
   */
  ;

  _proto.parseSelectionSet = function parseSelectionSet() {
    var start = this._lexer.token;
    return {
      kind: Kind.SELECTION_SET,
      selections: this.many(TokenKind.BRACE_L, this.parseSelection, TokenKind.BRACE_R),
      loc: this.loc(start)
    };
  }
  /**
   * Selection :
   *   - Field
   *   - FragmentSpread
   *   - InlineFragment
   */
  ;

  _proto.parseSelection = function parseSelection() {
    return this.peek(TokenKind.SPREAD) ? this.parseFragment() : this.parseField();
  }
  /**
   * Field : Alias? Name Arguments? Directives? SelectionSet?
   *
   * Alias : Name :
   */
  ;

  _proto.parseField = function parseField() {
    var start = this._lexer.token;
    var nameOrAlias = this.parseName();
    var alias;
    var name;

    if (this.expectOptionalToken(TokenKind.COLON)) {
      alias = nameOrAlias;
      name = this.parseName();
    } else {
      name = nameOrAlias;
    }

    return {
      kind: Kind.FIELD,
      alias: alias,
      name: name,
      arguments: this.parseArguments(false),
      directives: this.parseDirectives(false),
      selectionSet: this.peek(TokenKind.BRACE_L) ? this.parseSelectionSet() : undefined,
      loc: this.loc(start)
    };
  }
  /**
   * Arguments[Const] : ( Argument[?Const]+ )
   */
  ;

  _proto.parseArguments = function parseArguments(isConst) {
    var item = isConst ? this.parseConstArgument : this.parseArgument;
    return this.optionalMany(TokenKind.PAREN_L, item, TokenKind.PAREN_R);
  }
  /**
   * Argument[Const] : Name : Value[?Const]
   */
  ;

  _proto.parseArgument = function parseArgument() {
    var start = this._lexer.token;
    var name = this.parseName();
    this.expectToken(TokenKind.COLON);
    return {
      kind: Kind.ARGUMENT,
      name: name,
      value: this.parseValueLiteral(false),
      loc: this.loc(start)
    };
  };

  _proto.parseConstArgument = function parseConstArgument() {
    var start = this._lexer.token;
    return {
      kind: Kind.ARGUMENT,
      name: this.parseName(),
      value: (this.expectToken(TokenKind.COLON), this.parseValueLiteral(true)),
      loc: this.loc(start)
    };
  } // Implements the parsing rules in the Fragments section.

  /**
   * Corresponds to both FragmentSpread and InlineFragment in the spec.
   *
   * FragmentSpread : ... FragmentName Directives?
   *
   * InlineFragment : ... TypeCondition? Directives? SelectionSet
   */
  ;

  _proto.parseFragment = function parseFragment() {
    var start = this._lexer.token;
    this.expectToken(TokenKind.SPREAD);
    var hasTypeCondition = this.expectOptionalKeyword('on');

    if (!hasTypeCondition && this.peek(TokenKind.NAME)) {
      return {
        kind: Kind.FRAGMENT_SPREAD,
        name: this.parseFragmentName(),
        directives: this.parseDirectives(false),
        loc: this.loc(start)
      };
    }

    return {
      kind: Kind.INLINE_FRAGMENT,
      typeCondition: hasTypeCondition ? this.parseNamedType() : undefined,
      directives: this.parseDirectives(false),
      selectionSet: this.parseSelectionSet(),
      loc: this.loc(start)
    };
  }
  /**
   * FragmentDefinition :
   *   - fragment FragmentName on TypeCondition Directives? SelectionSet
   *
   * TypeCondition : NamedType
   */
  ;

  _proto.parseFragmentDefinition = function parseFragmentDefinition() {
    var _this$_options;

    var start = this._lexer.token;
    this.expectKeyword('fragment'); // Experimental support for defining variables within fragments changes
    // the grammar of FragmentDefinition:
    //   - fragment FragmentName VariableDefinitions? on TypeCondition Directives? SelectionSet

    if (((_this$_options = this._options) === null || _this$_options === void 0 ? void 0 : _this$_options.experimentalFragmentVariables) === true) {
      return {
        kind: Kind.FRAGMENT_DEFINITION,
        name: this.parseFragmentName(),
        variableDefinitions: this.parseVariableDefinitions(),
        typeCondition: (this.expectKeyword('on'), this.parseNamedType()),
        directives: this.parseDirectives(false),
        selectionSet: this.parseSelectionSet(),
        loc: this.loc(start)
      };
    }

    return {
      kind: Kind.FRAGMENT_DEFINITION,
      name: this.parseFragmentName(),
      typeCondition: (this.expectKeyword('on'), this.parseNamedType()),
      directives: this.parseDirectives(false),
      selectionSet: this.parseSelectionSet(),
      loc: this.loc(start)
    };
  }
  /**
   * FragmentName : Name but not `on`
   */
  ;

  _proto.parseFragmentName = function parseFragmentName() {
    if (this._lexer.token.value === 'on') {
      throw this.unexpected();
    }

    return this.parseName();
  } // Implements the parsing rules in the Values section.

  /**
   * Value[Const] :
   *   - [~Const] Variable
   *   - IntValue
   *   - FloatValue
   *   - StringValue
   *   - BooleanValue
   *   - NullValue
   *   - EnumValue
   *   - ListValue[?Const]
   *   - ObjectValue[?Const]
   *
   * BooleanValue : one of `true` `false`
   *
   * NullValue : `null`
   *
   * EnumValue : Name but not `true`, `false` or `null`
   */
  ;

  _proto.parseValueLiteral = function parseValueLiteral(isConst) {
    var token = this._lexer.token;

    switch (token.kind) {
      case TokenKind.BRACKET_L:
        return this.parseList(isConst);

      case TokenKind.BRACE_L:
        return this.parseObject(isConst);

      case TokenKind.INT:
        this._lexer.advance();

        return {
          kind: Kind.INT,
          value: token.value,
          loc: this.loc(token)
        };

      case TokenKind.FLOAT:
        this._lexer.advance();

        return {
          kind: Kind.FLOAT,
          value: token.value,
          loc: this.loc(token)
        };

      case TokenKind.STRING:
      case TokenKind.BLOCK_STRING:
        return this.parseStringLiteral();

      case TokenKind.NAME:
        this._lexer.advance();

        switch (token.value) {
          case 'true':
            return {
              kind: Kind.BOOLEAN,
              value: true,
              loc: this.loc(token)
            };

          case 'false':
            return {
              kind: Kind.BOOLEAN,
              value: false,
              loc: this.loc(token)
            };

          case 'null':
            return {
              kind: Kind.NULL,
              loc: this.loc(token)
            };

          default:
            return {
              kind: Kind.ENUM,
              value: token.value,
              loc: this.loc(token)
            };
        }

      case TokenKind.DOLLAR:
        if (!isConst) {
          return this.parseVariable();
        }

        break;
    }

    throw this.unexpected();
  };

  _proto.parseStringLiteral = function parseStringLiteral() {
    var token = this._lexer.token;

    this._lexer.advance();

    return {
      kind: Kind.STRING,
      value: token.value,
      block: token.kind === TokenKind.BLOCK_STRING,
      loc: this.loc(token)
    };
  }
  /**
   * ListValue[Const] :
   *   - [ ]
   *   - [ Value[?Const]+ ]
   */
  ;

  _proto.parseList = function parseList(isConst) {
    var _this = this;

    var start = this._lexer.token;

    var item = function item() {
      return _this.parseValueLiteral(isConst);
    };

    return {
      kind: Kind.LIST,
      values: this.any(TokenKind.BRACKET_L, item, TokenKind.BRACKET_R),
      loc: this.loc(start)
    };
  }
  /**
   * ObjectValue[Const] :
   *   - { }
   *   - { ObjectField[?Const]+ }
   */
  ;

  _proto.parseObject = function parseObject(isConst) {
    var _this2 = this;

    var start = this._lexer.token;

    var item = function item() {
      return _this2.parseObjectField(isConst);
    };

    return {
      kind: Kind.OBJECT,
      fields: this.any(TokenKind.BRACE_L, item, TokenKind.BRACE_R),
      loc: this.loc(start)
    };
  }
  /**
   * ObjectField[Const] : Name : Value[?Const]
   */
  ;

  _proto.parseObjectField = function parseObjectField(isConst) {
    var start = this._lexer.token;
    var name = this.parseName();
    this.expectToken(TokenKind.COLON);
    return {
      kind: Kind.OBJECT_FIELD,
      name: name,
      value: this.parseValueLiteral(isConst),
      loc: this.loc(start)
    };
  } // Implements the parsing rules in the Directives section.

  /**
   * Directives[Const] : Directive[?Const]+
   */
  ;

  _proto.parseDirectives = function parseDirectives(isConst) {
    var directives = [];

    while (this.peek(TokenKind.AT)) {
      directives.push(this.parseDirective(isConst));
    }

    return directives;
  }
  /**
   * Directive[Const] : @ Name Arguments[?Const]?
   */
  ;

  _proto.parseDirective = function parseDirective(isConst) {
    var start = this._lexer.token;
    this.expectToken(TokenKind.AT);
    return {
      kind: Kind.DIRECTIVE,
      name: this.parseName(),
      arguments: this.parseArguments(isConst),
      loc: this.loc(start)
    };
  } // Implements the parsing rules in the Types section.

  /**
   * Type :
   *   - NamedType
   *   - ListType
   *   - NonNullType
   */
  ;

  _proto.parseTypeReference = function parseTypeReference() {
    var start = this._lexer.token;
    var type;

    if (this.expectOptionalToken(TokenKind.BRACKET_L)) {
      type = this.parseTypeReference();
      this.expectToken(TokenKind.BRACKET_R);
      type = {
        kind: Kind.LIST_TYPE,
        type: type,
        loc: this.loc(start)
      };
    } else {
      type = this.parseNamedType();
    }

    if (this.expectOptionalToken(TokenKind.BANG)) {
      return {
        kind: Kind.NON_NULL_TYPE,
        type: type,
        loc: this.loc(start)
      };
    }

    return type;
  }
  /**
   * NamedType : Name
   */
  ;

  _proto.parseNamedType = function parseNamedType() {
    var start = this._lexer.token;
    return {
      kind: Kind.NAMED_TYPE,
      name: this.parseName(),
      loc: this.loc(start)
    };
  } // Implements the parsing rules in the Type Definition section.

  /**
   * TypeSystemDefinition :
   *   - SchemaDefinition
   *   - TypeDefinition
   *   - DirectiveDefinition
   *
   * TypeDefinition :
   *   - ScalarTypeDefinition
   *   - ObjectTypeDefinition
   *   - InterfaceTypeDefinition
   *   - UnionTypeDefinition
   *   - EnumTypeDefinition
   *   - InputObjectTypeDefinition
   */
  ;

  _proto.parseTypeSystemDefinition = function parseTypeSystemDefinition() {
    // Many definitions begin with a description and require a lookahead.
    var keywordToken = this.peekDescription() ? this._lexer.lookahead() : this._lexer.token;

    if (keywordToken.kind === TokenKind.NAME) {
      switch (keywordToken.value) {
        case 'schema':
          return this.parseSchemaDefinition();

        case 'scalar':
          return this.parseScalarTypeDefinition();

        case 'type':
          return this.parseObjectTypeDefinition();

        case 'interface':
          return this.parseInterfaceTypeDefinition();

        case 'union':
          return this.parseUnionTypeDefinition();

        case 'enum':
          return this.parseEnumTypeDefinition();

        case 'input':
          return this.parseInputObjectTypeDefinition();

        case 'directive':
          return this.parseDirectiveDefinition();
      }
    }

    throw this.unexpected(keywordToken);
  };

  _proto.peekDescription = function peekDescription() {
    return this.peek(TokenKind.STRING) || this.peek(TokenKind.BLOCK_STRING);
  }
  /**
   * Description : StringValue
   */
  ;

  _proto.parseDescription = function parseDescription() {
    if (this.peekDescription()) {
      return this.parseStringLiteral();
    }
  }
  /**
   * SchemaDefinition : Description? schema Directives[Const]? { OperationTypeDefinition+ }
   */
  ;

  _proto.parseSchemaDefinition = function parseSchemaDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('schema');
    var directives = this.parseDirectives(true);
    var operationTypes = this.many(TokenKind.BRACE_L, this.parseOperationTypeDefinition, TokenKind.BRACE_R);
    return {
      kind: Kind.SCHEMA_DEFINITION,
      description: description,
      directives: directives,
      operationTypes: operationTypes,
      loc: this.loc(start)
    };
  }
  /**
   * OperationTypeDefinition : OperationType : NamedType
   */
  ;

  _proto.parseOperationTypeDefinition = function parseOperationTypeDefinition() {
    var start = this._lexer.token;
    var operation = this.parseOperationType();
    this.expectToken(TokenKind.COLON);
    var type = this.parseNamedType();
    return {
      kind: Kind.OPERATION_TYPE_DEFINITION,
      operation: operation,
      type: type,
      loc: this.loc(start)
    };
  }
  /**
   * ScalarTypeDefinition : Description? scalar Name Directives[Const]?
   */
  ;

  _proto.parseScalarTypeDefinition = function parseScalarTypeDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('scalar');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    return {
      kind: Kind.SCALAR_TYPE_DEFINITION,
      description: description,
      name: name,
      directives: directives,
      loc: this.loc(start)
    };
  }
  /**
   * ObjectTypeDefinition :
   *   Description?
   *   type Name ImplementsInterfaces? Directives[Const]? FieldsDefinition?
   */
  ;

  _proto.parseObjectTypeDefinition = function parseObjectTypeDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('type');
    var name = this.parseName();
    var interfaces = this.parseImplementsInterfaces();
    var directives = this.parseDirectives(true);
    var fields = this.parseFieldsDefinition();
    return {
      kind: Kind.OBJECT_TYPE_DEFINITION,
      description: description,
      name: name,
      interfaces: interfaces,
      directives: directives,
      fields: fields,
      loc: this.loc(start)
    };
  }
  /**
   * ImplementsInterfaces :
   *   - implements `&`? NamedType
   *   - ImplementsInterfaces & NamedType
   */
  ;

  _proto.parseImplementsInterfaces = function parseImplementsInterfaces() {
    var _this$_options2;

    if (!this.expectOptionalKeyword('implements')) {
      return [];
    }

    if (((_this$_options2 = this._options) === null || _this$_options2 === void 0 ? void 0 : _this$_options2.allowLegacySDLImplementsInterfaces) === true) {
      var types = []; // Optional leading ampersand

      this.expectOptionalToken(TokenKind.AMP);

      do {
        types.push(this.parseNamedType());
      } while (this.expectOptionalToken(TokenKind.AMP) || this.peek(TokenKind.NAME));

      return types;
    }

    return this.delimitedMany(TokenKind.AMP, this.parseNamedType);
  }
  /**
   * FieldsDefinition : { FieldDefinition+ }
   */
  ;

  _proto.parseFieldsDefinition = function parseFieldsDefinition() {
    var _this$_options3;

    // Legacy support for the SDL?
    if (((_this$_options3 = this._options) === null || _this$_options3 === void 0 ? void 0 : _this$_options3.allowLegacySDLEmptyFields) === true && this.peek(TokenKind.BRACE_L) && this._lexer.lookahead().kind === TokenKind.BRACE_R) {
      this._lexer.advance();

      this._lexer.advance();

      return [];
    }

    return this.optionalMany(TokenKind.BRACE_L, this.parseFieldDefinition, TokenKind.BRACE_R);
  }
  /**
   * FieldDefinition :
   *   - Description? Name ArgumentsDefinition? : Type Directives[Const]?
   */
  ;

  _proto.parseFieldDefinition = function parseFieldDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    var name = this.parseName();
    var args = this.parseArgumentDefs();
    this.expectToken(TokenKind.COLON);
    var type = this.parseTypeReference();
    var directives = this.parseDirectives(true);
    return {
      kind: Kind.FIELD_DEFINITION,
      description: description,
      name: name,
      arguments: args,
      type: type,
      directives: directives,
      loc: this.loc(start)
    };
  }
  /**
   * ArgumentsDefinition : ( InputValueDefinition+ )
   */
  ;

  _proto.parseArgumentDefs = function parseArgumentDefs() {
    return this.optionalMany(TokenKind.PAREN_L, this.parseInputValueDef, TokenKind.PAREN_R);
  }
  /**
   * InputValueDefinition :
   *   - Description? Name : Type DefaultValue? Directives[Const]?
   */
  ;

  _proto.parseInputValueDef = function parseInputValueDef() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    var name = this.parseName();
    this.expectToken(TokenKind.COLON);
    var type = this.parseTypeReference();
    var defaultValue;

    if (this.expectOptionalToken(TokenKind.EQUALS)) {
      defaultValue = this.parseValueLiteral(true);
    }

    var directives = this.parseDirectives(true);
    return {
      kind: Kind.INPUT_VALUE_DEFINITION,
      description: description,
      name: name,
      type: type,
      defaultValue: defaultValue,
      directives: directives,
      loc: this.loc(start)
    };
  }
  /**
   * InterfaceTypeDefinition :
   *   - Description? interface Name Directives[Const]? FieldsDefinition?
   */
  ;

  _proto.parseInterfaceTypeDefinition = function parseInterfaceTypeDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('interface');
    var name = this.parseName();
    var interfaces = this.parseImplementsInterfaces();
    var directives = this.parseDirectives(true);
    var fields = this.parseFieldsDefinition();
    return {
      kind: Kind.INTERFACE_TYPE_DEFINITION,
      description: description,
      name: name,
      interfaces: interfaces,
      directives: directives,
      fields: fields,
      loc: this.loc(start)
    };
  }
  /**
   * UnionTypeDefinition :
   *   - Description? union Name Directives[Const]? UnionMemberTypes?
   */
  ;

  _proto.parseUnionTypeDefinition = function parseUnionTypeDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('union');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    var types = this.parseUnionMemberTypes();
    return {
      kind: Kind.UNION_TYPE_DEFINITION,
      description: description,
      name: name,
      directives: directives,
      types: types,
      loc: this.loc(start)
    };
  }
  /**
   * UnionMemberTypes :
   *   - = `|`? NamedType
   *   - UnionMemberTypes | NamedType
   */
  ;

  _proto.parseUnionMemberTypes = function parseUnionMemberTypes() {
    return this.expectOptionalToken(TokenKind.EQUALS) ? this.delimitedMany(TokenKind.PIPE, this.parseNamedType) : [];
  }
  /**
   * EnumTypeDefinition :
   *   - Description? enum Name Directives[Const]? EnumValuesDefinition?
   */
  ;

  _proto.parseEnumTypeDefinition = function parseEnumTypeDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('enum');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    var values = this.parseEnumValuesDefinition();
    return {
      kind: Kind.ENUM_TYPE_DEFINITION,
      description: description,
      name: name,
      directives: directives,
      values: values,
      loc: this.loc(start)
    };
  }
  /**
   * EnumValuesDefinition : { EnumValueDefinition+ }
   */
  ;

  _proto.parseEnumValuesDefinition = function parseEnumValuesDefinition() {
    return this.optionalMany(TokenKind.BRACE_L, this.parseEnumValueDefinition, TokenKind.BRACE_R);
  }
  /**
   * EnumValueDefinition : Description? EnumValue Directives[Const]?
   *
   * EnumValue : Name
   */
  ;

  _proto.parseEnumValueDefinition = function parseEnumValueDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    return {
      kind: Kind.ENUM_VALUE_DEFINITION,
      description: description,
      name: name,
      directives: directives,
      loc: this.loc(start)
    };
  }
  /**
   * InputObjectTypeDefinition :
   *   - Description? input Name Directives[Const]? InputFieldsDefinition?
   */
  ;

  _proto.parseInputObjectTypeDefinition = function parseInputObjectTypeDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('input');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    var fields = this.parseInputFieldsDefinition();
    return {
      kind: Kind.INPUT_OBJECT_TYPE_DEFINITION,
      description: description,
      name: name,
      directives: directives,
      fields: fields,
      loc: this.loc(start)
    };
  }
  /**
   * InputFieldsDefinition : { InputValueDefinition+ }
   */
  ;

  _proto.parseInputFieldsDefinition = function parseInputFieldsDefinition() {
    return this.optionalMany(TokenKind.BRACE_L, this.parseInputValueDef, TokenKind.BRACE_R);
  }
  /**
   * TypeSystemExtension :
   *   - SchemaExtension
   *   - TypeExtension
   *
   * TypeExtension :
   *   - ScalarTypeExtension
   *   - ObjectTypeExtension
   *   - InterfaceTypeExtension
   *   - UnionTypeExtension
   *   - EnumTypeExtension
   *   - InputObjectTypeDefinition
   */
  ;

  _proto.parseTypeSystemExtension = function parseTypeSystemExtension() {
    var keywordToken = this._lexer.lookahead();

    if (keywordToken.kind === TokenKind.NAME) {
      switch (keywordToken.value) {
        case 'schema':
          return this.parseSchemaExtension();

        case 'scalar':
          return this.parseScalarTypeExtension();

        case 'type':
          return this.parseObjectTypeExtension();

        case 'interface':
          return this.parseInterfaceTypeExtension();

        case 'union':
          return this.parseUnionTypeExtension();

        case 'enum':
          return this.parseEnumTypeExtension();

        case 'input':
          return this.parseInputObjectTypeExtension();
      }
    }

    throw this.unexpected(keywordToken);
  }
  /**
   * SchemaExtension :
   *  - extend schema Directives[Const]? { OperationTypeDefinition+ }
   *  - extend schema Directives[Const]
   */
  ;

  _proto.parseSchemaExtension = function parseSchemaExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('schema');
    var directives = this.parseDirectives(true);
    var operationTypes = this.optionalMany(TokenKind.BRACE_L, this.parseOperationTypeDefinition, TokenKind.BRACE_R);

    if (directives.length === 0 && operationTypes.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.SCHEMA_EXTENSION,
      directives: directives,
      operationTypes: operationTypes,
      loc: this.loc(start)
    };
  }
  /**
   * ScalarTypeExtension :
   *   - extend scalar Name Directives[Const]
   */
  ;

  _proto.parseScalarTypeExtension = function parseScalarTypeExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('scalar');
    var name = this.parseName();
    var directives = this.parseDirectives(true);

    if (directives.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.SCALAR_TYPE_EXTENSION,
      name: name,
      directives: directives,
      loc: this.loc(start)
    };
  }
  /**
   * ObjectTypeExtension :
   *  - extend type Name ImplementsInterfaces? Directives[Const]? FieldsDefinition
   *  - extend type Name ImplementsInterfaces? Directives[Const]
   *  - extend type Name ImplementsInterfaces
   */
  ;

  _proto.parseObjectTypeExtension = function parseObjectTypeExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('type');
    var name = this.parseName();
    var interfaces = this.parseImplementsInterfaces();
    var directives = this.parseDirectives(true);
    var fields = this.parseFieldsDefinition();

    if (interfaces.length === 0 && directives.length === 0 && fields.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.OBJECT_TYPE_EXTENSION,
      name: name,
      interfaces: interfaces,
      directives: directives,
      fields: fields,
      loc: this.loc(start)
    };
  }
  /**
   * InterfaceTypeExtension :
   *  - extend interface Name ImplementsInterfaces? Directives[Const]? FieldsDefinition
   *  - extend interface Name ImplementsInterfaces? Directives[Const]
   *  - extend interface Name ImplementsInterfaces
   */
  ;

  _proto.parseInterfaceTypeExtension = function parseInterfaceTypeExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('interface');
    var name = this.parseName();
    var interfaces = this.parseImplementsInterfaces();
    var directives = this.parseDirectives(true);
    var fields = this.parseFieldsDefinition();

    if (interfaces.length === 0 && directives.length === 0 && fields.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.INTERFACE_TYPE_EXTENSION,
      name: name,
      interfaces: interfaces,
      directives: directives,
      fields: fields,
      loc: this.loc(start)
    };
  }
  /**
   * UnionTypeExtension :
   *   - extend union Name Directives[Const]? UnionMemberTypes
   *   - extend union Name Directives[Const]
   */
  ;

  _proto.parseUnionTypeExtension = function parseUnionTypeExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('union');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    var types = this.parseUnionMemberTypes();

    if (directives.length === 0 && types.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.UNION_TYPE_EXTENSION,
      name: name,
      directives: directives,
      types: types,
      loc: this.loc(start)
    };
  }
  /**
   * EnumTypeExtension :
   *   - extend enum Name Directives[Const]? EnumValuesDefinition
   *   - extend enum Name Directives[Const]
   */
  ;

  _proto.parseEnumTypeExtension = function parseEnumTypeExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('enum');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    var values = this.parseEnumValuesDefinition();

    if (directives.length === 0 && values.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.ENUM_TYPE_EXTENSION,
      name: name,
      directives: directives,
      values: values,
      loc: this.loc(start)
    };
  }
  /**
   * InputObjectTypeExtension :
   *   - extend input Name Directives[Const]? InputFieldsDefinition
   *   - extend input Name Directives[Const]
   */
  ;

  _proto.parseInputObjectTypeExtension = function parseInputObjectTypeExtension() {
    var start = this._lexer.token;
    this.expectKeyword('extend');
    this.expectKeyword('input');
    var name = this.parseName();
    var directives = this.parseDirectives(true);
    var fields = this.parseInputFieldsDefinition();

    if (directives.length === 0 && fields.length === 0) {
      throw this.unexpected();
    }

    return {
      kind: Kind.INPUT_OBJECT_TYPE_EXTENSION,
      name: name,
      directives: directives,
      fields: fields,
      loc: this.loc(start)
    };
  }
  /**
   * DirectiveDefinition :
   *   - Description? directive @ Name ArgumentsDefinition? `repeatable`? on DirectiveLocations
   */
  ;

  _proto.parseDirectiveDefinition = function parseDirectiveDefinition() {
    var start = this._lexer.token;
    var description = this.parseDescription();
    this.expectKeyword('directive');
    this.expectToken(TokenKind.AT);
    var name = this.parseName();
    var args = this.parseArgumentDefs();
    var repeatable = this.expectOptionalKeyword('repeatable');
    this.expectKeyword('on');
    var locations = this.parseDirectiveLocations();
    return {
      kind: Kind.DIRECTIVE_DEFINITION,
      description: description,
      name: name,
      arguments: args,
      repeatable: repeatable,
      locations: locations,
      loc: this.loc(start)
    };
  }
  /**
   * DirectiveLocations :
   *   - `|`? DirectiveLocation
   *   - DirectiveLocations | DirectiveLocation
   */
  ;

  _proto.parseDirectiveLocations = function parseDirectiveLocations() {
    return this.delimitedMany(TokenKind.PIPE, this.parseDirectiveLocation);
  }
  /*
   * DirectiveLocation :
   *   - ExecutableDirectiveLocation
   *   - TypeSystemDirectiveLocation
   *
   * ExecutableDirectiveLocation : one of
   *   `QUERY`
   *   `MUTATION`
   *   `SUBSCRIPTION`
   *   `FIELD`
   *   `FRAGMENT_DEFINITION`
   *   `FRAGMENT_SPREAD`
   *   `INLINE_FRAGMENT`
   *
   * TypeSystemDirectiveLocation : one of
   *   `SCHEMA`
   *   `SCALAR`
   *   `OBJECT`
   *   `FIELD_DEFINITION`
   *   `ARGUMENT_DEFINITION`
   *   `INTERFACE`
   *   `UNION`
   *   `ENUM`
   *   `ENUM_VALUE`
   *   `INPUT_OBJECT`
   *   `INPUT_FIELD_DEFINITION`
   */
  ;

  _proto.parseDirectiveLocation = function parseDirectiveLocation() {
    var start = this._lexer.token;
    var name = this.parseName();

    if (DirectiveLocation[name.value] !== undefined) {
      return name;
    }

    throw this.unexpected(start);
  } // Core parsing utility functions

  /**
   * Returns a location object, used to identify the place in the source that created a given parsed object.
   */
  ;

  _proto.loc = function loc(startToken) {
    var _this$_options4;

    if (((_this$_options4 = this._options) === null || _this$_options4 === void 0 ? void 0 : _this$_options4.noLocation) !== true) {
      return new Location(startToken, this._lexer.lastToken, this._lexer.source);
    }
  }
  /**
   * Determines if the next token is of a given kind
   */
  ;

  _proto.peek = function peek(kind) {
    return this._lexer.token.kind === kind;
  }
  /**
   * If the next token is of the given kind, return that token after advancing the lexer.
   * Otherwise, do not change the parser state and throw an error.
   */
  ;

  _proto.expectToken = function expectToken(kind) {
    var token = this._lexer.token;

    if (token.kind === kind) {
      this._lexer.advance();

      return token;
    }

    throw syntaxError(this._lexer.source, token.start, "Expected ".concat(getTokenKindDesc(kind), ", found ").concat(getTokenDesc(token), "."));
  }
  /**
   * If the next token is of the given kind, return that token after advancing the lexer.
   * Otherwise, do not change the parser state and return undefined.
   */
  ;

  _proto.expectOptionalToken = function expectOptionalToken(kind) {
    var token = this._lexer.token;

    if (token.kind === kind) {
      this._lexer.advance();

      return token;
    }

    return undefined;
  }
  /**
   * If the next token is a given keyword, advance the lexer.
   * Otherwise, do not change the parser state and throw an error.
   */
  ;

  _proto.expectKeyword = function expectKeyword(value) {
    var token = this._lexer.token;

    if (token.kind === TokenKind.NAME && token.value === value) {
      this._lexer.advance();
    } else {
      throw syntaxError(this._lexer.source, token.start, "Expected \"".concat(value, "\", found ").concat(getTokenDesc(token), "."));
    }
  }
  /**
   * If the next token is a given keyword, return "true" after advancing the lexer.
   * Otherwise, do not change the parser state and return "false".
   */
  ;

  _proto.expectOptionalKeyword = function expectOptionalKeyword(value) {
    var token = this._lexer.token;

    if (token.kind === TokenKind.NAME && token.value === value) {
      this._lexer.advance();

      return true;
    }

    return false;
  }
  /**
   * Helper function for creating an error when an unexpected lexed token is encountered.
   */
  ;

  _proto.unexpected = function unexpected(atToken) {
    var token = atToken !== null && atToken !== void 0 ? atToken : this._lexer.token;
    return syntaxError(this._lexer.source, token.start, "Unexpected ".concat(getTokenDesc(token), "."));
  }
  /**
   * Returns a possibly empty list of parse nodes, determined by the parseFn.
   * This list begins with a lex token of openKind and ends with a lex token of closeKind.
   * Advances the parser to the next lex token after the closing token.
   */
  ;

  _proto.any = function any(openKind, parseFn, closeKind) {
    this.expectToken(openKind);
    var nodes = [];

    while (!this.expectOptionalToken(closeKind)) {
      nodes.push(parseFn.call(this));
    }

    return nodes;
  }
  /**
   * Returns a list of parse nodes, determined by the parseFn.
   * It can be empty only if open token is missing otherwise it will always return non-empty list
   * that begins with a lex token of openKind and ends with a lex token of closeKind.
   * Advances the parser to the next lex token after the closing token.
   */
  ;

  _proto.optionalMany = function optionalMany(openKind, parseFn, closeKind) {
    if (this.expectOptionalToken(openKind)) {
      var nodes = [];

      do {
        nodes.push(parseFn.call(this));
      } while (!this.expectOptionalToken(closeKind));

      return nodes;
    }

    return [];
  }
  /**
   * Returns a non-empty list of parse nodes, determined by the parseFn.
   * This list begins with a lex token of openKind and ends with a lex token of closeKind.
   * Advances the parser to the next lex token after the closing token.
   */
  ;

  _proto.many = function many(openKind, parseFn, closeKind) {
    this.expectToken(openKind);
    var nodes = [];

    do {
      nodes.push(parseFn.call(this));
    } while (!this.expectOptionalToken(closeKind));

    return nodes;
  }
  /**
   * Returns a non-empty list of parse nodes, determined by the parseFn.
   * This list may begin with a lex token of delimiterKind followed by items separated by lex tokens of tokenKind.
   * Advances the parser to the next lex token after last item in the list.
   */
  ;

  _proto.delimitedMany = function delimitedMany(delimiterKind, parseFn) {
    this.expectOptionalToken(delimiterKind);
    var nodes = [];

    do {
      nodes.push(parseFn.call(this));
    } while (this.expectOptionalToken(delimiterKind));

    return nodes;
  };

  return Parser;
}();
/**
 * A helper function to describe a token as a string for debugging.
 */

function getTokenDesc(token) {
  var value = token.value;
  return getTokenKindDesc(token.kind) + (value != null ? " \"".concat(value, "\"") : '');
}
/**
 * A helper function to describe a token kind as a string for debugging.
 */


function getTokenKindDesc(kind) {
  return isPunctuatorTokenKind(kind) ? "\"".concat(kind, "\"") : kind;
}

/**
 * A visitor is provided to visit, it contains the collection of
 * relevant functions to be called during the visitor's traversal.
 */

var QueryDocumentKeys = {
  Name: [],
  Document: ['definitions'],
  OperationDefinition: ['name', 'variableDefinitions', 'directives', 'selectionSet'],
  VariableDefinition: ['variable', 'type', 'defaultValue', 'directives'],
  Variable: ['name'],
  SelectionSet: ['selections'],
  Field: ['alias', 'name', 'arguments', 'directives', 'selectionSet'],
  Argument: ['name', 'value'],
  FragmentSpread: ['name', 'directives'],
  InlineFragment: ['typeCondition', 'directives', 'selectionSet'],
  FragmentDefinition: ['name', // Note: fragment variable definitions are experimental and may be changed
  // or removed in the future.
  'variableDefinitions', 'typeCondition', 'directives', 'selectionSet'],
  IntValue: [],
  FloatValue: [],
  StringValue: [],
  BooleanValue: [],
  NullValue: [],
  EnumValue: [],
  ListValue: ['values'],
  ObjectValue: ['fields'],
  ObjectField: ['name', 'value'],
  Directive: ['name', 'arguments'],
  NamedType: ['name'],
  ListType: ['type'],
  NonNullType: ['type'],
  SchemaDefinition: ['description', 'directives', 'operationTypes'],
  OperationTypeDefinition: ['type'],
  ScalarTypeDefinition: ['description', 'name', 'directives'],
  ObjectTypeDefinition: ['description', 'name', 'interfaces', 'directives', 'fields'],
  FieldDefinition: ['description', 'name', 'arguments', 'type', 'directives'],
  InputValueDefinition: ['description', 'name', 'type', 'defaultValue', 'directives'],
  InterfaceTypeDefinition: ['description', 'name', 'interfaces', 'directives', 'fields'],
  UnionTypeDefinition: ['description', 'name', 'directives', 'types'],
  EnumTypeDefinition: ['description', 'name', 'directives', 'values'],
  EnumValueDefinition: ['description', 'name', 'directives'],
  InputObjectTypeDefinition: ['description', 'name', 'directives', 'fields'],
  DirectiveDefinition: ['description', 'name', 'arguments', 'locations'],
  SchemaExtension: ['directives', 'operationTypes'],
  ScalarTypeExtension: ['name', 'directives'],
  ObjectTypeExtension: ['name', 'interfaces', 'directives', 'fields'],
  InterfaceTypeExtension: ['name', 'interfaces', 'directives', 'fields'],
  UnionTypeExtension: ['name', 'directives', 'types'],
  EnumTypeExtension: ['name', 'directives', 'values'],
  InputObjectTypeExtension: ['name', 'directives', 'fields']
};
var BREAK = Object.freeze({});
/**
 * visit() will walk through an AST using a depth-first traversal, calling
 * the visitor's enter function at each node in the traversal, and calling the
 * leave function after visiting that node and all of its child nodes.
 *
 * By returning different values from the enter and leave functions, the
 * behavior of the visitor can be altered, including skipping over a sub-tree of
 * the AST (by returning false), editing the AST by returning a value or null
 * to remove the value, or to stop the whole traversal by returning BREAK.
 *
 * When using visit() to edit an AST, the original AST will not be modified, and
 * a new version of the AST with the changes applied will be returned from the
 * visit function.
 *
 *     const editedAST = visit(ast, {
 *       enter(node, key, parent, path, ancestors) {
 *         // @return
 *         //   undefined: no action
 *         //   false: skip visiting this node
 *         //   visitor.BREAK: stop visiting altogether
 *         //   null: delete this node
 *         //   any value: replace this node with the returned value
 *       },
 *       leave(node, key, parent, path, ancestors) {
 *         // @return
 *         //   undefined: no action
 *         //   false: no action
 *         //   visitor.BREAK: stop visiting altogether
 *         //   null: delete this node
 *         //   any value: replace this node with the returned value
 *       }
 *     });
 *
 * Alternatively to providing enter() and leave() functions, a visitor can
 * instead provide functions named the same as the kinds of AST nodes, or
 * enter/leave visitors at a named key, leading to four permutations of the
 * visitor API:
 *
 * 1) Named visitors triggered when entering a node of a specific kind.
 *
 *     visit(ast, {
 *       Kind(node) {
 *         // enter the "Kind" node
 *       }
 *     })
 *
 * 2) Named visitors that trigger upon entering and leaving a node of
 *    a specific kind.
 *
 *     visit(ast, {
 *       Kind: {
 *         enter(node) {
 *           // enter the "Kind" node
 *         }
 *         leave(node) {
 *           // leave the "Kind" node
 *         }
 *       }
 *     })
 *
 * 3) Generic visitors that trigger upon entering and leaving any node.
 *
 *     visit(ast, {
 *       enter(node) {
 *         // enter any node
 *       },
 *       leave(node) {
 *         // leave any node
 *       }
 *     })
 *
 * 4) Parallel visitors for entering and leaving nodes of a specific kind.
 *
 *     visit(ast, {
 *       enter: {
 *         Kind(node) {
 *           // enter the "Kind" node
 *         }
 *       },
 *       leave: {
 *         Kind(node) {
 *           // leave the "Kind" node
 *         }
 *       }
 *     })
 */

function visit(root, visitor) {
  var visitorKeys = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : QueryDocumentKeys;

  /* eslint-disable no-undef-init */
  var stack = undefined;
  var inArray = Array.isArray(root);
  var keys = [root];
  var index = -1;
  var edits = [];
  var node = undefined;
  var key = undefined;
  var parent = undefined;
  var path = [];
  var ancestors = [];
  var newRoot = root;
  /* eslint-enable no-undef-init */

  do {
    index++;
    var isLeaving = index === keys.length;
    var isEdited = isLeaving && edits.length !== 0;

    if (isLeaving) {
      key = ancestors.length === 0 ? undefined : path[path.length - 1];
      node = parent;
      parent = ancestors.pop();

      if (isEdited) {
        if (inArray) {
          node = node.slice();
        } else {
          var clone = {};

          for (var _i2 = 0, _Object$keys2 = Object.keys(node); _i2 < _Object$keys2.length; _i2++) {
            var k = _Object$keys2[_i2];
            clone[k] = node[k];
          }

          node = clone;
        }

        var editOffset = 0;

        for (var ii = 0; ii < edits.length; ii++) {
          var editKey = edits[ii][0];
          var editValue = edits[ii][1];

          if (inArray) {
            editKey -= editOffset;
          }

          if (inArray && editValue === null) {
            node.splice(editKey, 1);
            editOffset++;
          } else {
            node[editKey] = editValue;
          }
        }
      }

      index = stack.index;
      keys = stack.keys;
      edits = stack.edits;
      inArray = stack.inArray;
      stack = stack.prev;
    } else {
      key = parent ? inArray ? index : keys[index] : undefined;
      node = parent ? parent[key] : newRoot;

      if (node === null || node === undefined) {
        continue;
      }

      if (parent) {
        path.push(key);
      }
    }

    var result = void 0;

    if (!Array.isArray(node)) {
      if (!isNode(node)) {
        throw new Error("Invalid AST Node: ".concat(inspect(node), "."));
      }

      var visitFn = getVisitFn(visitor, node.kind, isLeaving);

      if (visitFn) {
        result = visitFn.call(visitor, node, key, parent, path, ancestors);

        if (result === BREAK) {
          break;
        }

        if (result === false) {
          if (!isLeaving) {
            path.pop();
            continue;
          }
        } else if (result !== undefined) {
          edits.push([key, result]);

          if (!isLeaving) {
            if (isNode(result)) {
              node = result;
            } else {
              path.pop();
              continue;
            }
          }
        }
      }
    }

    if (result === undefined && isEdited) {
      edits.push([key, node]);
    }

    if (isLeaving) {
      path.pop();
    } else {
      var _visitorKeys$node$kin;

      stack = {
        inArray: inArray,
        index: index,
        keys: keys,
        edits: edits,
        prev: stack
      };
      inArray = Array.isArray(node);
      keys = inArray ? node : (_visitorKeys$node$kin = visitorKeys[node.kind]) !== null && _visitorKeys$node$kin !== void 0 ? _visitorKeys$node$kin : [];
      index = -1;
      edits = [];

      if (parent) {
        ancestors.push(parent);
      }

      parent = node;
    }
  } while (stack !== undefined);

  if (edits.length !== 0) {
    newRoot = edits[edits.length - 1][1];
  }

  return newRoot;
}
/**
 * Given a visitor instance, if it is leaving or not, and a node kind, return
 * the function the visitor runtime should call.
 */

function getVisitFn(visitor, kind, isLeaving) {
  var kindVisitor = visitor[kind];

  if (kindVisitor) {
    if (!isLeaving && typeof kindVisitor === 'function') {
      // { Kind() {} }
      return kindVisitor;
    }

    var kindSpecificVisitor = isLeaving ? kindVisitor.leave : kindVisitor.enter;

    if (typeof kindSpecificVisitor === 'function') {
      // { Kind: { enter() {}, leave() {} } }
      return kindSpecificVisitor;
    }
  } else {
    var specificVisitor = isLeaving ? visitor.leave : visitor.enter;

    if (specificVisitor) {
      if (typeof specificVisitor === 'function') {
        // { enter() {}, leave() {} }
        return specificVisitor;
      }

      var specificKindVisitor = specificVisitor[kind];

      if (typeof specificKindVisitor === 'function') {
        // { enter: { Kind() {} }, leave: { Kind() {} } }
        return specificKindVisitor;
      }
    }
  }
}

/* eslint-disable no-redeclare */
// $FlowFixMe[name-already-bound] workaround for: https://github.com/facebook/flow/issues/4441
var objectValues = Object.values || function (obj) {
  return Object.keys(obj).map(function (key) {
    return obj[key];
  });
};

var objectValues$1 = objectValues;

/* eslint-disable no-redeclare */
// $FlowFixMe[name-already-bound] workaround for: https://github.com/facebook/flow/issues/4441
var objectEntries = Object.entries || function (obj) {
  return Object.keys(obj).map(function (key) {
    return [key, obj[key]];
  });
};

var objectEntries$1 = objectEntries;

/**
 * Creates a keyed JS object from an array, given a function to produce the keys
 * for each value in the array.
 *
 * This provides a convenient lookup for the array items if the key function
 * produces unique results.
 *
 *     const phoneBook = [
 *       { name: 'Jon', num: '555-1234' },
 *       { name: 'Jenny', num: '867-5309' }
 *     ]
 *
 *     // { Jon: { name: 'Jon', num: '555-1234' },
 *     //   Jenny: { name: 'Jenny', num: '867-5309' } }
 *     const entriesByName = keyMap(
 *       phoneBook,
 *       entry => entry.name
 *     )
 *
 *     // { name: 'Jenny', num: '857-6309' }
 *     const jennyEntry = entriesByName['Jenny']
 *
 */
function keyMap(list, keyFn) {
  return list.reduce(function (map, item) {
    map[keyFn(item)] = item;
    return map;
  }, Object.create(null));
}

/**
 * Creates an object map with the same keys as `map` and values generated by
 * running each value of `map` thru `fn`.
 */
function mapValue(map, fn) {
  var result = Object.create(null);

  for (var _i2 = 0, _objectEntries2 = objectEntries$1(map); _i2 < _objectEntries2.length; _i2++) {
    var _ref2 = _objectEntries2[_i2];
    var _key = _ref2[0];
    var _value = _ref2[1];
    result[_key] = fn(_value, _key);
  }

  return result;
}

function toObjMap(obj) {
  /* eslint-enable no-redeclare */
  if (Object.getPrototypeOf(obj) === null) {
    return obj;
  }

  var map = Object.create(null);

  for (var _i2 = 0, _objectEntries2 = objectEntries$1(obj); _i2 < _objectEntries2.length; _i2++) {
    var _ref2 = _objectEntries2[_i2];
    var key = _ref2[0];
    var value = _ref2[1];
    map[key] = value;
  }

  return map;
}

/**
 * Creates a keyed JS object from an array, given a function to produce the keys
 * and a function to produce the values from each item in the array.
 *
 *     const phoneBook = [
 *       { name: 'Jon', num: '555-1234' },
 *       { name: 'Jenny', num: '867-5309' }
 *     ]
 *
 *     // { Jon: '555-1234', Jenny: '867-5309' }
 *     const phonesByName = keyValMap(
 *       phoneBook,
 *       entry => entry.name,
 *       entry => entry.num
 *     )
 *
 */
function keyValMap(list, keyFn, valFn) {
  return list.reduce(function (map, item) {
    map[keyFn(item)] = valFn(item);
    return map;
  }, Object.create(null));
}

var MAX_SUGGESTIONS = 5;
/**
 * Given [ A, B, C ] return ' Did you mean A, B, or C?'.
 */

// eslint-disable-next-line no-redeclare
function didYouMean(firstArg, secondArg) {
  var _ref = typeof firstArg === 'string' ? [firstArg, secondArg] : [undefined, firstArg],
      subMessage = _ref[0],
      suggestionsArg = _ref[1];

  var message = ' Did you mean ';

  if (subMessage) {
    message += subMessage + ' ';
  }

  var suggestions = suggestionsArg.map(function (x) {
    return "\"".concat(x, "\"");
  });

  switch (suggestions.length) {
    case 0:
      return '';

    case 1:
      return message + suggestions[0] + '?';

    case 2:
      return message + suggestions[0] + ' or ' + suggestions[1] + '?';
  }

  var selected = suggestions.slice(0, MAX_SUGGESTIONS);
  var lastItem = selected.pop();
  return message + selected.join(', ') + ', or ' + lastItem + '?';
}

/**
 * Returns the first argument it receives.
 */
function identityFunc(x) {
  return x;
}

/**
 * Returns a number indicating whether a reference string comes before, or after,
 * or is the same as the given string in natural sort order.
 *
 * See: https://en.wikipedia.org/wiki/Natural_sort_order
 *
 */
function naturalCompare(aStr, bStr) {
  var aIdx = 0;
  var bIdx = 0;

  while (aIdx < aStr.length && bIdx < bStr.length) {
    var aChar = aStr.charCodeAt(aIdx);
    var bChar = bStr.charCodeAt(bIdx);

    if (isDigit(aChar) && isDigit(bChar)) {
      var aNum = 0;

      do {
        ++aIdx;
        aNum = aNum * 10 + aChar - DIGIT_0;
        aChar = aStr.charCodeAt(aIdx);
      } while (isDigit(aChar) && aNum > 0);

      var bNum = 0;

      do {
        ++bIdx;
        bNum = bNum * 10 + bChar - DIGIT_0;
        bChar = bStr.charCodeAt(bIdx);
      } while (isDigit(bChar) && bNum > 0);

      if (aNum < bNum) {
        return -1;
      }

      if (aNum > bNum) {
        return 1;
      }
    } else {
      if (aChar < bChar) {
        return -1;
      }

      if (aChar > bChar) {
        return 1;
      }

      ++aIdx;
      ++bIdx;
    }
  }

  return aStr.length - bStr.length;
}
var DIGIT_0 = 48;
var DIGIT_9 = 57;

function isDigit(code) {
  return !isNaN(code) && DIGIT_0 <= code && code <= DIGIT_9;
}

/**
 * Given an invalid input string and a list of valid options, returns a filtered
 * list of valid options sorted based on their similarity with the input.
 */

function suggestionList(input, options) {
  var optionsByDistance = Object.create(null);
  var lexicalDistance = new LexicalDistance(input);
  var threshold = Math.floor(input.length * 0.4) + 1;

  for (var _i2 = 0; _i2 < options.length; _i2++) {
    var option = options[_i2];
    var distance = lexicalDistance.measure(option, threshold);

    if (distance !== undefined) {
      optionsByDistance[option] = distance;
    }
  }

  return Object.keys(optionsByDistance).sort(function (a, b) {
    var distanceDiff = optionsByDistance[a] - optionsByDistance[b];
    return distanceDiff !== 0 ? distanceDiff : naturalCompare(a, b);
  });
}
/**
 * Computes the lexical distance between strings A and B.
 *
 * The "distance" between two strings is given by counting the minimum number
 * of edits needed to transform string A into string B. An edit can be an
 * insertion, deletion, or substitution of a single character, or a swap of two
 * adjacent characters.
 *
 * Includes a custom alteration from Damerau-Levenshtein to treat case changes
 * as a single edit which helps identify mis-cased values with an edit distance
 * of 1.
 *
 * This distance can be useful for detecting typos in input or sorting
 */

var LexicalDistance = /*#__PURE__*/function () {
  function LexicalDistance(input) {
    this._input = input;
    this._inputLowerCase = input.toLowerCase();
    this._inputArray = stringToArray(this._inputLowerCase);
    this._rows = [new Array(input.length + 1).fill(0), new Array(input.length + 1).fill(0), new Array(input.length + 1).fill(0)];
  }

  var _proto = LexicalDistance.prototype;

  _proto.measure = function measure(option, threshold) {
    if (this._input === option) {
      return 0;
    }

    var optionLowerCase = option.toLowerCase(); // Any case change counts as a single edit

    if (this._inputLowerCase === optionLowerCase) {
      return 1;
    }

    var a = stringToArray(optionLowerCase);
    var b = this._inputArray;

    if (a.length < b.length) {
      var tmp = a;
      a = b;
      b = tmp;
    }

    var aLength = a.length;
    var bLength = b.length;

    if (aLength - bLength > threshold) {
      return undefined;
    }

    var rows = this._rows;

    for (var j = 0; j <= bLength; j++) {
      rows[0][j] = j;
    }

    for (var i = 1; i <= aLength; i++) {
      var upRow = rows[(i - 1) % 3];
      var currentRow = rows[i % 3];
      var smallestCell = currentRow[0] = i;

      for (var _j = 1; _j <= bLength; _j++) {
        var cost = a[i - 1] === b[_j - 1] ? 0 : 1;
        var currentCell = Math.min(upRow[_j] + 1, // delete
        currentRow[_j - 1] + 1, // insert
        upRow[_j - 1] + cost // substitute
        );

        if (i > 1 && _j > 1 && a[i - 1] === b[_j - 2] && a[i - 2] === b[_j - 1]) {
          // transposition
          var doubleDiagonalCell = rows[(i - 2) % 3][_j - 2];
          currentCell = Math.min(currentCell, doubleDiagonalCell + 1);
        }

        if (currentCell < smallestCell) {
          smallestCell = currentCell;
        }

        currentRow[_j] = currentCell;
      } // Early exit, since distance can't go smaller than smallest element of the previous row.


      if (smallestCell > threshold) {
        return undefined;
      }
    }

    var distance = rows[aLength % 3][bLength];
    return distance <= threshold ? distance : undefined;
  };

  return LexicalDistance;
}();

function stringToArray(str) {
  var strLength = str.length;
  var array = new Array(strLength);

  for (var i = 0; i < strLength; ++i) {
    array[i] = str.charCodeAt(i);
  }

  return array;
}

/**
 * Converts an AST into a string, using one set of reasonable
 * formatting rules.
 */

function print(ast) {
  return visit(ast, {
    leave: printDocASTReducer
  });
}
var MAX_LINE_LENGTH = 80; // TODO: provide better type coverage in future

var printDocASTReducer = {
  Name: function Name(node) {
    return node.value;
  },
  Variable: function Variable(node) {
    return '$' + node.name;
  },
  // Document
  Document: function Document(node) {
    return join(node.definitions, '\n\n') + '\n';
  },
  OperationDefinition: function OperationDefinition(node) {
    var op = node.operation;
    var name = node.name;
    var varDefs = wrap('(', join(node.variableDefinitions, ', '), ')');
    var directives = join(node.directives, ' ');
    var selectionSet = node.selectionSet; // Anonymous queries with no directives or variable definitions can use
    // the query short form.

    return !name && !directives && !varDefs && op === 'query' ? selectionSet : join([op, join([name, varDefs]), directives, selectionSet], ' ');
  },
  VariableDefinition: function VariableDefinition(_ref) {
    var variable = _ref.variable,
        type = _ref.type,
        defaultValue = _ref.defaultValue,
        directives = _ref.directives;
    return variable + ': ' + type + wrap(' = ', defaultValue) + wrap(' ', join(directives, ' '));
  },
  SelectionSet: function SelectionSet(_ref2) {
    var selections = _ref2.selections;
    return block(selections);
  },
  Field: function Field(_ref3) {
    var alias = _ref3.alias,
        name = _ref3.name,
        args = _ref3.arguments,
        directives = _ref3.directives,
        selectionSet = _ref3.selectionSet;
    var prefix = wrap('', alias, ': ') + name;
    var argsLine = prefix + wrap('(', join(args, ', '), ')');

    if (argsLine.length > MAX_LINE_LENGTH) {
      argsLine = prefix + wrap('(\n', indent(join(args, '\n')), '\n)');
    }

    return join([argsLine, join(directives, ' '), selectionSet], ' ');
  },
  Argument: function Argument(_ref4) {
    var name = _ref4.name,
        value = _ref4.value;
    return name + ': ' + value;
  },
  // Fragments
  FragmentSpread: function FragmentSpread(_ref5) {
    var name = _ref5.name,
        directives = _ref5.directives;
    return '...' + name + wrap(' ', join(directives, ' '));
  },
  InlineFragment: function InlineFragment(_ref6) {
    var typeCondition = _ref6.typeCondition,
        directives = _ref6.directives,
        selectionSet = _ref6.selectionSet;
    return join(['...', wrap('on ', typeCondition), join(directives, ' '), selectionSet], ' ');
  },
  FragmentDefinition: function FragmentDefinition(_ref7) {
    var name = _ref7.name,
        typeCondition = _ref7.typeCondition,
        variableDefinitions = _ref7.variableDefinitions,
        directives = _ref7.directives,
        selectionSet = _ref7.selectionSet;
    return (// Note: fragment variable definitions are experimental and may be changed
      // or removed in the future.
      "fragment ".concat(name).concat(wrap('(', join(variableDefinitions, ', '), ')'), " ") + "on ".concat(typeCondition, " ").concat(wrap('', join(directives, ' '), ' ')) + selectionSet
    );
  },
  // Value
  IntValue: function IntValue(_ref8) {
    var value = _ref8.value;
    return value;
  },
  FloatValue: function FloatValue(_ref9) {
    var value = _ref9.value;
    return value;
  },
  StringValue: function StringValue(_ref10, key) {
    var value = _ref10.value,
        isBlockString = _ref10.block;
    return isBlockString ? printBlockString(value, key === 'description' ? '' : '  ') : JSON.stringify(value);
  },
  BooleanValue: function BooleanValue(_ref11) {
    var value = _ref11.value;
    return value ? 'true' : 'false';
  },
  NullValue: function NullValue() {
    return 'null';
  },
  EnumValue: function EnumValue(_ref12) {
    var value = _ref12.value;
    return value;
  },
  ListValue: function ListValue(_ref13) {
    var values = _ref13.values;
    return '[' + join(values, ', ') + ']';
  },
  ObjectValue: function ObjectValue(_ref14) {
    var fields = _ref14.fields;
    return '{' + join(fields, ', ') + '}';
  },
  ObjectField: function ObjectField(_ref15) {
    var name = _ref15.name,
        value = _ref15.value;
    return name + ': ' + value;
  },
  // Directive
  Directive: function Directive(_ref16) {
    var name = _ref16.name,
        args = _ref16.arguments;
    return '@' + name + wrap('(', join(args, ', '), ')');
  },
  // Type
  NamedType: function NamedType(_ref17) {
    var name = _ref17.name;
    return name;
  },
  ListType: function ListType(_ref18) {
    var type = _ref18.type;
    return '[' + type + ']';
  },
  NonNullType: function NonNullType(_ref19) {
    var type = _ref19.type;
    return type + '!';
  },
  // Type System Definitions
  SchemaDefinition: addDescription(function (_ref20) {
    var directives = _ref20.directives,
        operationTypes = _ref20.operationTypes;
    return join(['schema', join(directives, ' '), block(operationTypes)], ' ');
  }),
  OperationTypeDefinition: function OperationTypeDefinition(_ref21) {
    var operation = _ref21.operation,
        type = _ref21.type;
    return operation + ': ' + type;
  },
  ScalarTypeDefinition: addDescription(function (_ref22) {
    var name = _ref22.name,
        directives = _ref22.directives;
    return join(['scalar', name, join(directives, ' ')], ' ');
  }),
  ObjectTypeDefinition: addDescription(function (_ref23) {
    var name = _ref23.name,
        interfaces = _ref23.interfaces,
        directives = _ref23.directives,
        fields = _ref23.fields;
    return join(['type', name, wrap('implements ', join(interfaces, ' & ')), join(directives, ' '), block(fields)], ' ');
  }),
  FieldDefinition: addDescription(function (_ref24) {
    var name = _ref24.name,
        args = _ref24.arguments,
        type = _ref24.type,
        directives = _ref24.directives;
    return name + (hasMultilineItems(args) ? wrap('(\n', indent(join(args, '\n')), '\n)') : wrap('(', join(args, ', '), ')')) + ': ' + type + wrap(' ', join(directives, ' '));
  }),
  InputValueDefinition: addDescription(function (_ref25) {
    var name = _ref25.name,
        type = _ref25.type,
        defaultValue = _ref25.defaultValue,
        directives = _ref25.directives;
    return join([name + ': ' + type, wrap('= ', defaultValue), join(directives, ' ')], ' ');
  }),
  InterfaceTypeDefinition: addDescription(function (_ref26) {
    var name = _ref26.name,
        interfaces = _ref26.interfaces,
        directives = _ref26.directives,
        fields = _ref26.fields;
    return join(['interface', name, wrap('implements ', join(interfaces, ' & ')), join(directives, ' '), block(fields)], ' ');
  }),
  UnionTypeDefinition: addDescription(function (_ref27) {
    var name = _ref27.name,
        directives = _ref27.directives,
        types = _ref27.types;
    return join(['union', name, join(directives, ' '), types && types.length !== 0 ? '= ' + join(types, ' | ') : ''], ' ');
  }),
  EnumTypeDefinition: addDescription(function (_ref28) {
    var name = _ref28.name,
        directives = _ref28.directives,
        values = _ref28.values;
    return join(['enum', name, join(directives, ' '), block(values)], ' ');
  }),
  EnumValueDefinition: addDescription(function (_ref29) {
    var name = _ref29.name,
        directives = _ref29.directives;
    return join([name, join(directives, ' ')], ' ');
  }),
  InputObjectTypeDefinition: addDescription(function (_ref30) {
    var name = _ref30.name,
        directives = _ref30.directives,
        fields = _ref30.fields;
    return join(['input', name, join(directives, ' '), block(fields)], ' ');
  }),
  DirectiveDefinition: addDescription(function (_ref31) {
    var name = _ref31.name,
        args = _ref31.arguments,
        repeatable = _ref31.repeatable,
        locations = _ref31.locations;
    return 'directive @' + name + (hasMultilineItems(args) ? wrap('(\n', indent(join(args, '\n')), '\n)') : wrap('(', join(args, ', '), ')')) + (repeatable ? ' repeatable' : '') + ' on ' + join(locations, ' | ');
  }),
  SchemaExtension: function SchemaExtension(_ref32) {
    var directives = _ref32.directives,
        operationTypes = _ref32.operationTypes;
    return join(['extend schema', join(directives, ' '), block(operationTypes)], ' ');
  },
  ScalarTypeExtension: function ScalarTypeExtension(_ref33) {
    var name = _ref33.name,
        directives = _ref33.directives;
    return join(['extend scalar', name, join(directives, ' ')], ' ');
  },
  ObjectTypeExtension: function ObjectTypeExtension(_ref34) {
    var name = _ref34.name,
        interfaces = _ref34.interfaces,
        directives = _ref34.directives,
        fields = _ref34.fields;
    return join(['extend type', name, wrap('implements ', join(interfaces, ' & ')), join(directives, ' '), block(fields)], ' ');
  },
  InterfaceTypeExtension: function InterfaceTypeExtension(_ref35) {
    var name = _ref35.name,
        interfaces = _ref35.interfaces,
        directives = _ref35.directives,
        fields = _ref35.fields;
    return join(['extend interface', name, wrap('implements ', join(interfaces, ' & ')), join(directives, ' '), block(fields)], ' ');
  },
  UnionTypeExtension: function UnionTypeExtension(_ref36) {
    var name = _ref36.name,
        directives = _ref36.directives,
        types = _ref36.types;
    return join(['extend union', name, join(directives, ' '), types && types.length !== 0 ? '= ' + join(types, ' | ') : ''], ' ');
  },
  EnumTypeExtension: function EnumTypeExtension(_ref37) {
    var name = _ref37.name,
        directives = _ref37.directives,
        values = _ref37.values;
    return join(['extend enum', name, join(directives, ' '), block(values)], ' ');
  },
  InputObjectTypeExtension: function InputObjectTypeExtension(_ref38) {
    var name = _ref38.name,
        directives = _ref38.directives,
        fields = _ref38.fields;
    return join(['extend input', name, join(directives, ' '), block(fields)], ' ');
  }
};

function addDescription(cb) {
  return function (node) {
    return join([node.description, cb(node)], '\n');
  };
}
/**
 * Given maybeArray, print an empty string if it is null or empty, otherwise
 * print all items together separated by separator if provided
 */


function join(maybeArray) {
  var _maybeArray$filter$jo;

  var separator = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  return (_maybeArray$filter$jo = maybeArray === null || maybeArray === void 0 ? void 0 : maybeArray.filter(function (x) {
    return x;
  }).join(separator)) !== null && _maybeArray$filter$jo !== void 0 ? _maybeArray$filter$jo : '';
}
/**
 * Given array, print each item on its own line, wrapped in an
 * indented "{ }" block.
 */


function block(array) {
  return wrap('{\n', indent(join(array, '\n')), '\n}');
}
/**
 * If maybeString is not null or empty, then wrap with start and end, otherwise print an empty string.
 */


function wrap(start, maybeString) {
  var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  return maybeString != null && maybeString !== '' ? start + maybeString + end : '';
}

function indent(str) {
  return wrap('  ', str.replace(/\n/g, '\n  '));
}

function isMultiline(str) {
  return str.indexOf('\n') !== -1;
}

function hasMultilineItems(maybeArray) {
  return maybeArray != null && maybeArray.some(isMultiline);
}

/**
 * Produces a JavaScript value given a GraphQL Value AST.
 *
 * Unlike `valueFromAST()`, no type is provided. The resulting JavaScript value
 * will reflect the provided GraphQL value AST.
 *
 * | GraphQL Value        | JavaScript Value |
 * | -------------------- | ---------------- |
 * | Input Object         | Object           |
 * | List                 | Array            |
 * | Boolean              | Boolean          |
 * | String / Enum        | String           |
 * | Int / Float          | Number           |
 * | Null                 | null             |
 *
 */
function valueFromASTUntyped(valueNode, variables) {
  switch (valueNode.kind) {
    case Kind.NULL:
      return null;

    case Kind.INT:
      return parseInt(valueNode.value, 10);

    case Kind.FLOAT:
      return parseFloat(valueNode.value);

    case Kind.STRING:
    case Kind.ENUM:
    case Kind.BOOLEAN:
      return valueNode.value;

    case Kind.LIST:
      return valueNode.values.map(function (node) {
        return valueFromASTUntyped(node, variables);
      });

    case Kind.OBJECT:
      return keyValMap(valueNode.fields, function (field) {
        return field.name.value;
      }, function (field) {
        return valueFromASTUntyped(field.value, variables);
      });

    case Kind.VARIABLE:
      return variables === null || variables === void 0 ? void 0 : variables[valueNode.name.value];
  } // istanbul ignore next (Not reachable. All possible value nodes have been considered)


  invariant(0, 'Unexpected value node: ' + inspect(valueNode));
}

function _defineProperties$2(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass$2(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties$2(Constructor.prototype, protoProps); if (staticProps) _defineProperties$2(Constructor, staticProps); return Constructor; }
function isType(type) {
  return isScalarType(type) || isObjectType(type) || isInterfaceType(type) || isUnionType(type) || isEnumType(type) || isInputObjectType(type) || isListType(type) || isNonNullType(type);
}
function assertType(type) {
  if (!isType(type)) {
    throw new Error("Expected ".concat(inspect(type), " to be a GraphQL type."));
  }

  return type;
}
/**
 * There are predicates for each kind of GraphQL type.
 */

// eslint-disable-next-line no-redeclare
function isScalarType(type) {
  return instanceOf(type, GraphQLScalarType);
}
// eslint-disable-next-line no-redeclare
function isObjectType(type) {
  return instanceOf(type, GraphQLObjectType);
}
// eslint-disable-next-line no-redeclare
function isInterfaceType(type) {
  return instanceOf(type, GraphQLInterfaceType);
}
// eslint-disable-next-line no-redeclare
function isUnionType(type) {
  return instanceOf(type, GraphQLUnionType);
}
// eslint-disable-next-line no-redeclare
function isEnumType(type) {
  return instanceOf(type, GraphQLEnumType);
}
// eslint-disable-next-line no-redeclare
function isInputObjectType(type) {
  return instanceOf(type, GraphQLInputObjectType);
}
// eslint-disable-next-line no-redeclare
function isListType(type) {
  return instanceOf(type, GraphQLList);
}
// eslint-disable-next-line no-redeclare
function isNonNullType(type) {
  return instanceOf(type, GraphQLNonNull);
}
/**
 * These types may describe types which may be leaf values.
 */

function isLeafType(type) {
  return isScalarType(type) || isEnumType(type);
}
/**
 * These types may describe the parent context of a selection set.
 */

function isAbstractType(type) {
  return isInterfaceType(type) || isUnionType(type);
}
/**
 * List Type Wrapper
 *
 * A list is a wrapping type which points to another type.
 * Lists are often created within the context of defining the fields of
 * an object type.
 *
 * Example:
 *
 *     const PersonType = new GraphQLObjectType({
 *       name: 'Person',
 *       fields: () => ({
 *         parents: { type: new GraphQLList(PersonType) },
 *         children: { type: new GraphQLList(PersonType) },
 *       })
 *     })
 *
 */
// FIXME: workaround to fix issue with Babel parser

/* ::
declare class GraphQLList<+T: GraphQLType> {
  +ofType: T;
  static <T>(ofType: T): GraphQLList<T>;
  // Note: constructors cannot be used for covariant types. Drop the "new".
  constructor(ofType: GraphQLType): void;
}
*/

function GraphQLList(ofType) {
  // istanbul ignore else (to be removed in v16.0.0)
  if (this instanceof GraphQLList) {
    this.ofType = assertType(ofType);
  } else {
    return new GraphQLList(ofType);
  }
} // Need to cast through any to alter the prototype.

GraphQLList.prototype.toString = function toString() {
  return '[' + String(this.ofType) + ']';
};

GraphQLList.prototype.toJSON = function toJSON() {
  return this.toString();
};

Object.defineProperty(GraphQLList.prototype, SYMBOL_TO_STRING_TAG, {
  get: function get() {
    return 'GraphQLList';
  }
}); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLList);
/**
 * Non-Null Type Wrapper
 *
 * A non-null is a wrapping type which points to another type.
 * Non-null types enforce that their values are never null and can ensure
 * an error is raised if this ever occurs during a request. It is useful for
 * fields which you can make a strong guarantee on non-nullability, for example
 * usually the id field of a database row will never be null.
 *
 * Example:
 *
 *     const RowType = new GraphQLObjectType({
 *       name: 'Row',
 *       fields: () => ({
 *         id: { type: new GraphQLNonNull(GraphQLString) },
 *       })
 *     })
 *
 * Note: the enforcement of non-nullability occurs within the executor.
 */
// FIXME: workaround to fix issue with Babel parser

/* ::
declare class GraphQLNonNull<+T: GraphQLNullableType> {
  +ofType: T;
  static <T>(ofType: T): GraphQLNonNull<T>;
  // Note: constructors cannot be used for covariant types. Drop the "new".
  constructor(ofType: GraphQLType): void;
}
*/

function GraphQLNonNull(ofType) {
  // istanbul ignore else (to be removed in v16.0.0)
  if (this instanceof GraphQLNonNull) {
    this.ofType = assertNullableType(ofType);
  } else {
    return new GraphQLNonNull(ofType);
  }
} // Need to cast through any to alter the prototype.

GraphQLNonNull.prototype.toString = function toString() {
  return String(this.ofType) + '!';
};

GraphQLNonNull.prototype.toJSON = function toJSON() {
  return this.toString();
};

Object.defineProperty(GraphQLNonNull.prototype, SYMBOL_TO_STRING_TAG, {
  get: function get() {
    return 'GraphQLNonNull';
  }
}); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLNonNull);
/**
 * These types can all accept null as a value.
 */

function isNullableType(type) {
  return isType(type) && !isNonNullType(type);
}
function assertNullableType(type) {
  if (!isNullableType(type)) {
    throw new Error("Expected ".concat(inspect(type), " to be a GraphQL nullable type."));
  }

  return type;
}
/**
 * Used while defining GraphQL types to allow for circular references in
 * otherwise immutable type definitions.
 */

function resolveThunk(thunk) {
  // $FlowFixMe[incompatible-use]
  return typeof thunk === 'function' ? thunk() : thunk;
}

function undefineIfEmpty(arr) {
  return arr && arr.length > 0 ? arr : undefined;
}
/**
 * Scalar Type Definition
 *
 * The leaf values of any request and input values to arguments are
 * Scalars (or Enums) and are defined with a name and a series of functions
 * used to parse input from ast or variables and to ensure validity.
 *
 * If a type's serialize function does not return a value (i.e. it returns
 * `undefined`) then an error will be raised and a `null` value will be returned
 * in the response. If the serialize function returns `null`, then no error will
 * be included in the response.
 *
 * Example:
 *
 *     const OddType = new GraphQLScalarType({
 *       name: 'Odd',
 *       serialize(value) {
 *         if (value % 2 === 1) {
 *           return value;
 *         }
 *       }
 *     });
 *
 */


var GraphQLScalarType = /*#__PURE__*/function () {
  function GraphQLScalarType(config) {
    var _config$parseValue, _config$serialize, _config$parseLiteral;

    var parseValue = (_config$parseValue = config.parseValue) !== null && _config$parseValue !== void 0 ? _config$parseValue : identityFunc;
    this.name = config.name;
    this.description = config.description;
    this.specifiedByUrl = config.specifiedByUrl;
    this.serialize = (_config$serialize = config.serialize) !== null && _config$serialize !== void 0 ? _config$serialize : identityFunc;
    this.parseValue = parseValue;
    this.parseLiteral = (_config$parseLiteral = config.parseLiteral) !== null && _config$parseLiteral !== void 0 ? _config$parseLiteral : function (node, variables) {
      return parseValue(valueFromASTUntyped(node, variables));
    };
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = undefineIfEmpty(config.extensionASTNodes);
    typeof config.name === 'string' || devAssert(0, 'Must provide name.');
    config.specifiedByUrl == null || typeof config.specifiedByUrl === 'string' || devAssert(0, "".concat(this.name, " must provide \"specifiedByUrl\" as a string, ") + "but got: ".concat(inspect(config.specifiedByUrl), "."));
    config.serialize == null || typeof config.serialize === 'function' || devAssert(0, "".concat(this.name, " must provide \"serialize\" function. If this custom Scalar is also used as an input type, ensure \"parseValue\" and \"parseLiteral\" functions are also provided."));

    if (config.parseLiteral) {
      typeof config.parseValue === 'function' && typeof config.parseLiteral === 'function' || devAssert(0, "".concat(this.name, " must provide both \"parseValue\" and \"parseLiteral\" functions."));
    }
  }

  var _proto = GraphQLScalarType.prototype;

  _proto.toConfig = function toConfig() {
    var _this$extensionASTNod;

    return {
      name: this.name,
      description: this.description,
      specifiedByUrl: this.specifiedByUrl,
      serialize: this.serialize,
      parseValue: this.parseValue,
      parseLiteral: this.parseLiteral,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: (_this$extensionASTNod = this.extensionASTNodes) !== null && _this$extensionASTNod !== void 0 ? _this$extensionASTNod : []
    };
  };

  _proto.toString = function toString() {
    return this.name;
  };

  _proto.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$2(GraphQLScalarType, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLScalarType';
    }
  }]);

  return GraphQLScalarType;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLScalarType);

/**
 * Object Type Definition
 *
 * Almost all of the GraphQL types you define will be object types. Object types
 * have a name, but most importantly describe their fields.
 *
 * Example:
 *
 *     const AddressType = new GraphQLObjectType({
 *       name: 'Address',
 *       fields: {
 *         street: { type: GraphQLString },
 *         number: { type: GraphQLInt },
 *         formatted: {
 *           type: GraphQLString,
 *           resolve(obj) {
 *             return obj.number + ' ' + obj.street
 *           }
 *         }
 *       }
 *     });
 *
 * When two types need to refer to each other, or a type needs to refer to
 * itself in a field, you can use a function expression (aka a closure or a
 * thunk) to supply the fields lazily.
 *
 * Example:
 *
 *     const PersonType = new GraphQLObjectType({
 *       name: 'Person',
 *       fields: () => ({
 *         name: { type: GraphQLString },
 *         bestFriend: { type: PersonType },
 *       })
 *     });
 *
 */
var GraphQLObjectType = /*#__PURE__*/function () {
  function GraphQLObjectType(config) {
    this.name = config.name;
    this.description = config.description;
    this.isTypeOf = config.isTypeOf;
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = undefineIfEmpty(config.extensionASTNodes);
    this._fields = defineFieldMap.bind(undefined, config);
    this._interfaces = defineInterfaces.bind(undefined, config);
    typeof config.name === 'string' || devAssert(0, 'Must provide name.');
    config.isTypeOf == null || typeof config.isTypeOf === 'function' || devAssert(0, "".concat(this.name, " must provide \"isTypeOf\" as a function, ") + "but got: ".concat(inspect(config.isTypeOf), "."));
  }

  var _proto2 = GraphQLObjectType.prototype;

  _proto2.getFields = function getFields() {
    if (typeof this._fields === 'function') {
      this._fields = this._fields();
    }

    return this._fields;
  };

  _proto2.getInterfaces = function getInterfaces() {
    if (typeof this._interfaces === 'function') {
      this._interfaces = this._interfaces();
    }

    return this._interfaces;
  };

  _proto2.toConfig = function toConfig() {
    return {
      name: this.name,
      description: this.description,
      interfaces: this.getInterfaces(),
      fields: fieldsToFieldsConfig(this.getFields()),
      isTypeOf: this.isTypeOf,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: this.extensionASTNodes || []
    };
  };

  _proto2.toString = function toString() {
    return this.name;
  };

  _proto2.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$2(GraphQLObjectType, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLObjectType';
    }
  }]);

  return GraphQLObjectType;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLObjectType);

function defineInterfaces(config) {
  var _resolveThunk;

  var interfaces = (_resolveThunk = resolveThunk(config.interfaces)) !== null && _resolveThunk !== void 0 ? _resolveThunk : [];
  Array.isArray(interfaces) || devAssert(0, "".concat(config.name, " interfaces must be an Array or a function which returns an Array."));
  return interfaces;
}

function defineFieldMap(config) {
  var fieldMap = resolveThunk(config.fields);
  isPlainObj(fieldMap) || devAssert(0, "".concat(config.name, " fields must be an object with field names as keys or a function which returns such an object."));
  return mapValue(fieldMap, function (fieldConfig, fieldName) {
    var _fieldConfig$args;

    isPlainObj(fieldConfig) || devAssert(0, "".concat(config.name, ".").concat(fieldName, " field config must be an object."));
    !('isDeprecated' in fieldConfig) || devAssert(0, "".concat(config.name, ".").concat(fieldName, " should provide \"deprecationReason\" instead of \"isDeprecated\"."));
    fieldConfig.resolve == null || typeof fieldConfig.resolve === 'function' || devAssert(0, "".concat(config.name, ".").concat(fieldName, " field resolver must be a function if ") + "provided, but got: ".concat(inspect(fieldConfig.resolve), "."));
    var argsConfig = (_fieldConfig$args = fieldConfig.args) !== null && _fieldConfig$args !== void 0 ? _fieldConfig$args : {};
    isPlainObj(argsConfig) || devAssert(0, "".concat(config.name, ".").concat(fieldName, " args must be an object with argument names as keys."));
    var args = objectEntries$1(argsConfig).map(function (_ref) {
      var argName = _ref[0],
          argConfig = _ref[1];
      return {
        name: argName,
        description: argConfig.description,
        type: argConfig.type,
        defaultValue: argConfig.defaultValue,
        deprecationReason: argConfig.deprecationReason,
        extensions: argConfig.extensions && toObjMap(argConfig.extensions),
        astNode: argConfig.astNode
      };
    });
    return {
      name: fieldName,
      description: fieldConfig.description,
      type: fieldConfig.type,
      args: args,
      resolve: fieldConfig.resolve,
      subscribe: fieldConfig.subscribe,
      isDeprecated: fieldConfig.deprecationReason != null,
      deprecationReason: fieldConfig.deprecationReason,
      extensions: fieldConfig.extensions && toObjMap(fieldConfig.extensions),
      astNode: fieldConfig.astNode
    };
  });
}

function isPlainObj(obj) {
  return isObjectLike(obj) && !Array.isArray(obj);
}

function fieldsToFieldsConfig(fields) {
  return mapValue(fields, function (field) {
    return {
      description: field.description,
      type: field.type,
      args: argsToArgsConfig(field.args),
      resolve: field.resolve,
      subscribe: field.subscribe,
      deprecationReason: field.deprecationReason,
      extensions: field.extensions,
      astNode: field.astNode
    };
  });
}
/**
 * @internal
 */


function argsToArgsConfig(args) {
  return keyValMap(args, function (arg) {
    return arg.name;
  }, function (arg) {
    return {
      description: arg.description,
      type: arg.type,
      defaultValue: arg.defaultValue,
      deprecationReason: arg.deprecationReason,
      extensions: arg.extensions,
      astNode: arg.astNode
    };
  });
}

/**
 * Interface Type Definition
 *
 * When a field can return one of a heterogeneous set of types, a Interface type
 * is used to describe what types are possible, what fields are in common across
 * all types, as well as a function to determine which type is actually used
 * when the field is resolved.
 *
 * Example:
 *
 *     const EntityType = new GraphQLInterfaceType({
 *       name: 'Entity',
 *       fields: {
 *         name: { type: GraphQLString }
 *       }
 *     });
 *
 */
var GraphQLInterfaceType = /*#__PURE__*/function () {
  function GraphQLInterfaceType(config) {
    this.name = config.name;
    this.description = config.description;
    this.resolveType = config.resolveType;
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = undefineIfEmpty(config.extensionASTNodes);
    this._fields = defineFieldMap.bind(undefined, config);
    this._interfaces = defineInterfaces.bind(undefined, config);
    typeof config.name === 'string' || devAssert(0, 'Must provide name.');
    config.resolveType == null || typeof config.resolveType === 'function' || devAssert(0, "".concat(this.name, " must provide \"resolveType\" as a function, ") + "but got: ".concat(inspect(config.resolveType), "."));
  }

  var _proto3 = GraphQLInterfaceType.prototype;

  _proto3.getFields = function getFields() {
    if (typeof this._fields === 'function') {
      this._fields = this._fields();
    }

    return this._fields;
  };

  _proto3.getInterfaces = function getInterfaces() {
    if (typeof this._interfaces === 'function') {
      this._interfaces = this._interfaces();
    }

    return this._interfaces;
  };

  _proto3.toConfig = function toConfig() {
    var _this$extensionASTNod2;

    return {
      name: this.name,
      description: this.description,
      interfaces: this.getInterfaces(),
      fields: fieldsToFieldsConfig(this.getFields()),
      resolveType: this.resolveType,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: (_this$extensionASTNod2 = this.extensionASTNodes) !== null && _this$extensionASTNod2 !== void 0 ? _this$extensionASTNod2 : []
    };
  };

  _proto3.toString = function toString() {
    return this.name;
  };

  _proto3.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$2(GraphQLInterfaceType, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLInterfaceType';
    }
  }]);

  return GraphQLInterfaceType;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLInterfaceType);

/**
 * Union Type Definition
 *
 * When a field can return one of a heterogeneous set of types, a Union type
 * is used to describe what types are possible as well as providing a function
 * to determine which type is actually used when the field is resolved.
 *
 * Example:
 *
 *     const PetType = new GraphQLUnionType({
 *       name: 'Pet',
 *       types: [ DogType, CatType ],
 *       resolveType(value) {
 *         if (value instanceof Dog) {
 *           return DogType;
 *         }
 *         if (value instanceof Cat) {
 *           return CatType;
 *         }
 *       }
 *     });
 *
 */
var GraphQLUnionType = /*#__PURE__*/function () {
  function GraphQLUnionType(config) {
    this.name = config.name;
    this.description = config.description;
    this.resolveType = config.resolveType;
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = undefineIfEmpty(config.extensionASTNodes);
    this._types = defineTypes.bind(undefined, config);
    typeof config.name === 'string' || devAssert(0, 'Must provide name.');
    config.resolveType == null || typeof config.resolveType === 'function' || devAssert(0, "".concat(this.name, " must provide \"resolveType\" as a function, ") + "but got: ".concat(inspect(config.resolveType), "."));
  }

  var _proto4 = GraphQLUnionType.prototype;

  _proto4.getTypes = function getTypes() {
    if (typeof this._types === 'function') {
      this._types = this._types();
    }

    return this._types;
  };

  _proto4.toConfig = function toConfig() {
    var _this$extensionASTNod3;

    return {
      name: this.name,
      description: this.description,
      types: this.getTypes(),
      resolveType: this.resolveType,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: (_this$extensionASTNod3 = this.extensionASTNodes) !== null && _this$extensionASTNod3 !== void 0 ? _this$extensionASTNod3 : []
    };
  };

  _proto4.toString = function toString() {
    return this.name;
  };

  _proto4.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$2(GraphQLUnionType, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLUnionType';
    }
  }]);

  return GraphQLUnionType;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLUnionType);

function defineTypes(config) {
  var types = resolveThunk(config.types);
  Array.isArray(types) || devAssert(0, "Must provide Array of types or a function which returns such an array for Union ".concat(config.name, "."));
  return types;
}

/**
 * Enum Type Definition
 *
 * Some leaf values of requests and input values are Enums. GraphQL serializes
 * Enum values as strings, however internally Enums can be represented by any
 * kind of type, often integers.
 *
 * Example:
 *
 *     const RGBType = new GraphQLEnumType({
 *       name: 'RGB',
 *       values: {
 *         RED: { value: 0 },
 *         GREEN: { value: 1 },
 *         BLUE: { value: 2 }
 *       }
 *     });
 *
 * Note: If a value is not provided in a definition, the name of the enum value
 * will be used as its internal value.
 */
var GraphQLEnumType
/* <T> */
= /*#__PURE__*/function () {
  function GraphQLEnumType(config) {
    this.name = config.name;
    this.description = config.description;
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = undefineIfEmpty(config.extensionASTNodes);
    this._values = defineEnumValues(this.name, config.values);
    this._valueLookup = new Map(this._values.map(function (enumValue) {
      return [enumValue.value, enumValue];
    }));
    this._nameLookup = keyMap(this._values, function (value) {
      return value.name;
    });
    typeof config.name === 'string' || devAssert(0, 'Must provide name.');
  }

  var _proto5 = GraphQLEnumType.prototype;

  _proto5.getValues = function getValues() {
    return this._values;
  };

  _proto5.getValue = function getValue(name) {
    return this._nameLookup[name];
  };

  _proto5.serialize = function serialize(outputValue) {
    var enumValue = this._valueLookup.get(outputValue);

    if (enumValue === undefined) {
      throw new GraphQLError("Enum \"".concat(this.name, "\" cannot represent value: ").concat(inspect(outputValue)));
    }

    return enumValue.name;
  };

  _proto5.parseValue = function parseValue(inputValue)
  /* T */
  {
    if (typeof inputValue !== 'string') {
      var valueStr = inspect(inputValue);
      throw new GraphQLError("Enum \"".concat(this.name, "\" cannot represent non-string value: ").concat(valueStr, ".") + didYouMeanEnumValue(this, valueStr));
    }

    var enumValue = this.getValue(inputValue);

    if (enumValue == null) {
      throw new GraphQLError("Value \"".concat(inputValue, "\" does not exist in \"").concat(this.name, "\" enum.") + didYouMeanEnumValue(this, inputValue));
    }

    return enumValue.value;
  };

  _proto5.parseLiteral = function parseLiteral(valueNode, _variables)
  /* T */
  {
    // Note: variables will be resolved to a value before calling this function.
    if (valueNode.kind !== Kind.ENUM) {
      var valueStr = print(valueNode);
      throw new GraphQLError("Enum \"".concat(this.name, "\" cannot represent non-enum value: ").concat(valueStr, ".") + didYouMeanEnumValue(this, valueStr), valueNode);
    }

    var enumValue = this.getValue(valueNode.value);

    if (enumValue == null) {
      var _valueStr = print(valueNode);

      throw new GraphQLError("Value \"".concat(_valueStr, "\" does not exist in \"").concat(this.name, "\" enum.") + didYouMeanEnumValue(this, _valueStr), valueNode);
    }

    return enumValue.value;
  };

  _proto5.toConfig = function toConfig() {
    var _this$extensionASTNod4;

    var values = keyValMap(this.getValues(), function (value) {
      return value.name;
    }, function (value) {
      return {
        description: value.description,
        value: value.value,
        deprecationReason: value.deprecationReason,
        extensions: value.extensions,
        astNode: value.astNode
      };
    });
    return {
      name: this.name,
      description: this.description,
      values: values,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: (_this$extensionASTNod4 = this.extensionASTNodes) !== null && _this$extensionASTNod4 !== void 0 ? _this$extensionASTNod4 : []
    };
  };

  _proto5.toString = function toString() {
    return this.name;
  };

  _proto5.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$2(GraphQLEnumType, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLEnumType';
    }
  }]);

  return GraphQLEnumType;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLEnumType);

function didYouMeanEnumValue(enumType, unknownValueStr) {
  var allNames = enumType.getValues().map(function (value) {
    return value.name;
  });
  var suggestedValues = suggestionList(unknownValueStr, allNames);
  return didYouMean('the enum value', suggestedValues);
}

function defineEnumValues(typeName, valueMap) {
  isPlainObj(valueMap) || devAssert(0, "".concat(typeName, " values must be an object with value names as keys."));
  return objectEntries$1(valueMap).map(function (_ref2) {
    var valueName = _ref2[0],
        valueConfig = _ref2[1];
    isPlainObj(valueConfig) || devAssert(0, "".concat(typeName, ".").concat(valueName, " must refer to an object with a \"value\" key ") + "representing an internal value but got: ".concat(inspect(valueConfig), "."));
    !('isDeprecated' in valueConfig) || devAssert(0, "".concat(typeName, ".").concat(valueName, " should provide \"deprecationReason\" instead of \"isDeprecated\"."));
    return {
      name: valueName,
      description: valueConfig.description,
      value: valueConfig.value !== undefined ? valueConfig.value : valueName,
      isDeprecated: valueConfig.deprecationReason != null,
      deprecationReason: valueConfig.deprecationReason,
      extensions: valueConfig.extensions && toObjMap(valueConfig.extensions),
      astNode: valueConfig.astNode
    };
  });
}

/**
 * Input Object Type Definition
 *
 * An input object defines a structured collection of fields which may be
 * supplied to a field argument.
 *
 * Using `NonNull` will ensure that a value must be provided by the query
 *
 * Example:
 *
 *     const GeoPoint = new GraphQLInputObjectType({
 *       name: 'GeoPoint',
 *       fields: {
 *         lat: { type: new GraphQLNonNull(GraphQLFloat) },
 *         lon: { type: new GraphQLNonNull(GraphQLFloat) },
 *         alt: { type: GraphQLFloat, defaultValue: 0 },
 *       }
 *     });
 *
 */
var GraphQLInputObjectType = /*#__PURE__*/function () {
  function GraphQLInputObjectType(config) {
    this.name = config.name;
    this.description = config.description;
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    this.extensionASTNodes = undefineIfEmpty(config.extensionASTNodes);
    this._fields = defineInputFieldMap.bind(undefined, config);
    typeof config.name === 'string' || devAssert(0, 'Must provide name.');
  }

  var _proto6 = GraphQLInputObjectType.prototype;

  _proto6.getFields = function getFields() {
    if (typeof this._fields === 'function') {
      this._fields = this._fields();
    }

    return this._fields;
  };

  _proto6.toConfig = function toConfig() {
    var _this$extensionASTNod5;

    var fields = mapValue(this.getFields(), function (field) {
      return {
        description: field.description,
        type: field.type,
        defaultValue: field.defaultValue,
        deprecationReason: field.deprecationReason,
        extensions: field.extensions,
        astNode: field.astNode
      };
    });
    return {
      name: this.name,
      description: this.description,
      fields: fields,
      extensions: this.extensions,
      astNode: this.astNode,
      extensionASTNodes: (_this$extensionASTNod5 = this.extensionASTNodes) !== null && _this$extensionASTNod5 !== void 0 ? _this$extensionASTNod5 : []
    };
  };

  _proto6.toString = function toString() {
    return this.name;
  };

  _proto6.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$2(GraphQLInputObjectType, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLInputObjectType';
    }
  }]);

  return GraphQLInputObjectType;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLInputObjectType);

function defineInputFieldMap(config) {
  var fieldMap = resolveThunk(config.fields);
  isPlainObj(fieldMap) || devAssert(0, "".concat(config.name, " fields must be an object with field names as keys or a function which returns such an object."));
  return mapValue(fieldMap, function (fieldConfig, fieldName) {
    !('resolve' in fieldConfig) || devAssert(0, "".concat(config.name, ".").concat(fieldName, " field has a resolve property, but Input Types cannot define resolvers."));
    return {
      name: fieldName,
      description: fieldConfig.description,
      type: fieldConfig.type,
      defaultValue: fieldConfig.defaultValue,
      deprecationReason: fieldConfig.deprecationReason,
      extensions: fieldConfig.extensions && toObjMap(fieldConfig.extensions),
      astNode: fieldConfig.astNode
    };
  });
}

/* eslint-disable no-redeclare */
// $FlowFixMe[name-already-bound] workaround for: https://github.com/facebook/flow/issues/4441
var isFinitePolyfill = Number.isFinite || function (value) {
  return typeof value === 'number' && isFinite(value);
};

var isFinite$1 = isFinitePolyfill;

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }
/**
 * Safer version of `Array.from` that return `null` if value isn't convertible to array.
 * Also protects against Array-like objects without items.
 *
 * @example
 *
 * safeArrayFrom([ 1, 2, 3 ]) // [1, 2, 3]
 * safeArrayFrom('ABC') // null
 * safeArrayFrom({ length: 1 }) // null
 * safeArrayFrom({ length: 1, 0: 'Alpha' }) // ['Alpha']
 * safeArrayFrom({ key: 'value' }) // null
 * safeArrayFrom(new Map()) // []
 *
 */

function safeArrayFrom(collection) {
  var mapFn = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function (item) {
    return item;
  };

  if (collection == null || _typeof(collection) !== 'object') {
    return null;
  }

  if (Array.isArray(collection)) {
    return collection.map(mapFn);
  } // Is Iterable?


  var iteratorMethod = collection[SYMBOL_ITERATOR];

  if (typeof iteratorMethod === 'function') {
    // $FlowFixMe[incompatible-use]
    var iterator = iteratorMethod.call(collection);
    var result = [];
    var step;

    for (var i = 0; !(step = iterator.next()).done; ++i) {
      result.push(mapFn(step.value, i));
    }

    return result;
  } // Is Array like?


  var length = collection.length;

  if (typeof length === 'number' && length >= 0 && length % 1 === 0) {
    var _result = [];

    for (var _i = 0; _i < length; ++_i) {
      if (!Object.prototype.hasOwnProperty.call(collection, _i)) {
        return null;
      }

      _result.push(mapFn(collection[String(_i)], _i));
    }

    return _result;
  }

  return null;
}

/* eslint-disable no-redeclare */
// $FlowFixMe[name-already-bound] workaround for: https://github.com/facebook/flow/issues/4441
var isInteger = Number.isInteger || function (value) {
  return typeof value === 'number' && isFinite(value) && Math.floor(value) === value;
};

var isInteger$1 = isInteger;

// 32-bit signed integer, providing the broadest support across platforms.
//
// n.b. JavaScript's integers are safe between -(2^53 - 1) and 2^53 - 1 because
// they are internally represented as IEEE 754 doubles.

var MAX_INT = 2147483647;
var MIN_INT = -2147483648;

function serializeInt(outputValue) {
  var coercedValue = serializeObject(outputValue);

  if (typeof coercedValue === 'boolean') {
    return coercedValue ? 1 : 0;
  }

  var num = coercedValue;

  if (typeof coercedValue === 'string' && coercedValue !== '') {
    num = Number(coercedValue);
  }

  if (!isInteger$1(num)) {
    throw new GraphQLError("Int cannot represent non-integer value: ".concat(inspect(coercedValue)));
  }

  if (num > MAX_INT || num < MIN_INT) {
    throw new GraphQLError('Int cannot represent non 32-bit signed integer value: ' + inspect(coercedValue));
  }

  return num;
}

function coerceInt(inputValue) {
  if (!isInteger$1(inputValue)) {
    throw new GraphQLError("Int cannot represent non-integer value: ".concat(inspect(inputValue)));
  }

  if (inputValue > MAX_INT || inputValue < MIN_INT) {
    throw new GraphQLError("Int cannot represent non 32-bit signed integer value: ".concat(inputValue));
  }

  return inputValue;
}

var GraphQLInt = new GraphQLScalarType({
  name: 'Int',
  description: 'The `Int` scalar type represents non-fractional signed whole numeric values. Int can represent values between -(2^31) and 2^31 - 1.',
  serialize: serializeInt,
  parseValue: coerceInt,
  parseLiteral: function parseLiteral(valueNode) {
    if (valueNode.kind !== Kind.INT) {
      throw new GraphQLError("Int cannot represent non-integer value: ".concat(print(valueNode)), valueNode);
    }

    var num = parseInt(valueNode.value, 10);

    if (num > MAX_INT || num < MIN_INT) {
      throw new GraphQLError("Int cannot represent non 32-bit signed integer value: ".concat(valueNode.value), valueNode);
    }

    return num;
  }
});

function serializeFloat(outputValue) {
  var coercedValue = serializeObject(outputValue);

  if (typeof coercedValue === 'boolean') {
    return coercedValue ? 1 : 0;
  }

  var num = coercedValue;

  if (typeof coercedValue === 'string' && coercedValue !== '') {
    num = Number(coercedValue);
  }

  if (!isFinite$1(num)) {
    throw new GraphQLError("Float cannot represent non numeric value: ".concat(inspect(coercedValue)));
  }

  return num;
}

function coerceFloat(inputValue) {
  if (!isFinite$1(inputValue)) {
    throw new GraphQLError("Float cannot represent non numeric value: ".concat(inspect(inputValue)));
  }

  return inputValue;
}

var GraphQLFloat = new GraphQLScalarType({
  name: 'Float',
  description: 'The `Float` scalar type represents signed double-precision fractional values as specified by [IEEE 754](https://en.wikipedia.org/wiki/IEEE_floating_point).',
  serialize: serializeFloat,
  parseValue: coerceFloat,
  parseLiteral: function parseLiteral(valueNode) {
    if (valueNode.kind !== Kind.FLOAT && valueNode.kind !== Kind.INT) {
      throw new GraphQLError("Float cannot represent non numeric value: ".concat(print(valueNode)), valueNode);
    }

    return parseFloat(valueNode.value);
  }
}); // Support serializing objects with custom valueOf() or toJSON() functions -
// a common way to represent a complex value which can be represented as
// a string (ex: MongoDB id objects).

function serializeObject(outputValue) {
  if (isObjectLike(outputValue)) {
    if (typeof outputValue.valueOf === 'function') {
      var valueOfResult = outputValue.valueOf();

      if (!isObjectLike(valueOfResult)) {
        return valueOfResult;
      }
    }

    if (typeof outputValue.toJSON === 'function') {
      // $FlowFixMe[incompatible-use]
      return outputValue.toJSON();
    }
  }

  return outputValue;
}

function serializeString(outputValue) {
  var coercedValue = serializeObject(outputValue); // Serialize string, boolean and number values to a string, but do not
  // attempt to coerce object, function, symbol, or other types as strings.

  if (typeof coercedValue === 'string') {
    return coercedValue;
  }

  if (typeof coercedValue === 'boolean') {
    return coercedValue ? 'true' : 'false';
  }

  if (isFinite$1(coercedValue)) {
    return coercedValue.toString();
  }

  throw new GraphQLError("String cannot represent value: ".concat(inspect(outputValue)));
}

function coerceString(inputValue) {
  if (typeof inputValue !== 'string') {
    throw new GraphQLError("String cannot represent a non string value: ".concat(inspect(inputValue)));
  }

  return inputValue;
}

var GraphQLString = new GraphQLScalarType({
  name: 'String',
  description: 'The `String` scalar type represents textual data, represented as UTF-8 character sequences. The String type is most often used by GraphQL to represent free-form human-readable text.',
  serialize: serializeString,
  parseValue: coerceString,
  parseLiteral: function parseLiteral(valueNode) {
    if (valueNode.kind !== Kind.STRING) {
      throw new GraphQLError("String cannot represent a non string value: ".concat(print(valueNode)), valueNode);
    }

    return valueNode.value;
  }
});

function serializeBoolean(outputValue) {
  var coercedValue = serializeObject(outputValue);

  if (typeof coercedValue === 'boolean') {
    return coercedValue;
  }

  if (isFinite$1(coercedValue)) {
    return coercedValue !== 0;
  }

  throw new GraphQLError("Boolean cannot represent a non boolean value: ".concat(inspect(coercedValue)));
}

function coerceBoolean(inputValue) {
  if (typeof inputValue !== 'boolean') {
    throw new GraphQLError("Boolean cannot represent a non boolean value: ".concat(inspect(inputValue)));
  }

  return inputValue;
}

var GraphQLBoolean = new GraphQLScalarType({
  name: 'Boolean',
  description: 'The `Boolean` scalar type represents `true` or `false`.',
  serialize: serializeBoolean,
  parseValue: coerceBoolean,
  parseLiteral: function parseLiteral(valueNode) {
    if (valueNode.kind !== Kind.BOOLEAN) {
      throw new GraphQLError("Boolean cannot represent a non boolean value: ".concat(print(valueNode)), valueNode);
    }

    return valueNode.value;
  }
});

function serializeID(outputValue) {
  var coercedValue = serializeObject(outputValue);

  if (typeof coercedValue === 'string') {
    return coercedValue;
  }

  if (isInteger$1(coercedValue)) {
    return String(coercedValue);
  }

  throw new GraphQLError("ID cannot represent value: ".concat(inspect(outputValue)));
}

function coerceID(inputValue) {
  if (typeof inputValue === 'string') {
    return inputValue;
  }

  if (isInteger$1(inputValue)) {
    return inputValue.toString();
  }

  throw new GraphQLError("ID cannot represent value: ".concat(inspect(inputValue)));
}

var GraphQLID = new GraphQLScalarType({
  name: 'ID',
  description: 'The `ID` scalar type represents a unique identifier, often used to refetch an object or as key for a cache. The ID type appears in a JSON response as a String; however, it is not intended to be human-readable. When expected as an input type, any string (such as `"4"`) or integer (such as `4`) input value will be accepted as an ID.',
  serialize: serializeID,
  parseValue: coerceID,
  parseLiteral: function parseLiteral(valueNode) {
    if (valueNode.kind !== Kind.STRING && valueNode.kind !== Kind.INT) {
      throw new GraphQLError('ID cannot represent a non-string and non-integer value: ' + print(valueNode), valueNode);
    }

    return valueNode.value;
  }
});
var specifiedScalarTypes = Object.freeze([GraphQLString, GraphQLInt, GraphQLFloat, GraphQLBoolean, GraphQLID]);

/**
 * Produces a GraphQL Value AST given a JavaScript object.
 * Function will match JavaScript/JSON values to GraphQL AST schema format
 * by using suggested GraphQLInputType. For example:
 *
 *     astFromValue("value", GraphQLString)
 *
 * A GraphQL type must be provided, which will be used to interpret different
 * JavaScript values.
 *
 * | JSON Value    | GraphQL Value        |
 * | ------------- | -------------------- |
 * | Object        | Input Object         |
 * | Array         | List                 |
 * | Boolean       | Boolean              |
 * | String        | String / Enum Value  |
 * | Number        | Int / Float          |
 * | Mixed         | Enum Value           |
 * | null          | NullValue            |
 *
 */

function astFromValue(value, type) {
  if (isNonNullType(type)) {
    var astValue = astFromValue(value, type.ofType);

    if ((astValue === null || astValue === void 0 ? void 0 : astValue.kind) === Kind.NULL) {
      return null;
    }

    return astValue;
  } // only explicit null, not undefined, NaN


  if (value === null) {
    return {
      kind: Kind.NULL
    };
  } // undefined


  if (value === undefined) {
    return null;
  } // Convert JavaScript array to GraphQL list. If the GraphQLType is a list, but
  // the value is not an array, convert the value using the list's item type.


  if (isListType(type)) {
    var itemType = type.ofType;
    var items = safeArrayFrom(value);

    if (items != null) {
      var valuesNodes = [];

      for (var _i2 = 0; _i2 < items.length; _i2++) {
        var item = items[_i2];
        var itemNode = astFromValue(item, itemType);

        if (itemNode != null) {
          valuesNodes.push(itemNode);
        }
      }

      return {
        kind: Kind.LIST,
        values: valuesNodes
      };
    }

    return astFromValue(value, itemType);
  } // Populate the fields of the input object by creating ASTs from each value
  // in the JavaScript object according to the fields in the input type.


  if (isInputObjectType(type)) {
    if (!isObjectLike(value)) {
      return null;
    }

    var fieldNodes = [];

    for (var _i4 = 0, _objectValues2 = objectValues$1(type.getFields()); _i4 < _objectValues2.length; _i4++) {
      var field = _objectValues2[_i4];
      var fieldValue = astFromValue(value[field.name], field.type);

      if (fieldValue) {
        fieldNodes.push({
          kind: Kind.OBJECT_FIELD,
          name: {
            kind: Kind.NAME,
            value: field.name
          },
          value: fieldValue
        });
      }
    }

    return {
      kind: Kind.OBJECT,
      fields: fieldNodes
    };
  } // istanbul ignore else (See: 'https://github.com/graphql/graphql-js/issues/2618')


  if (isLeafType(type)) {
    // Since value is an internally represented value, it must be serialized
    // to an externally represented value before converting into an AST.
    var serialized = type.serialize(value);

    if (serialized == null) {
      return null;
    } // Others serialize based on their corresponding JavaScript scalar types.


    if (typeof serialized === 'boolean') {
      return {
        kind: Kind.BOOLEAN,
        value: serialized
      };
    } // JavaScript numbers can be Int or Float values.


    if (typeof serialized === 'number' && isFinite$1(serialized)) {
      var stringNum = String(serialized);
      return integerStringRegExp.test(stringNum) ? {
        kind: Kind.INT,
        value: stringNum
      } : {
        kind: Kind.FLOAT,
        value: stringNum
      };
    }

    if (typeof serialized === 'string') {
      // Enum types use Enum literals.
      if (isEnumType(type)) {
        return {
          kind: Kind.ENUM,
          value: serialized
        };
      } // ID types can use Int literals.


      if (type === GraphQLID && integerStringRegExp.test(serialized)) {
        return {
          kind: Kind.INT,
          value: serialized
        };
      }

      return {
        kind: Kind.STRING,
        value: serialized
      };
    }

    throw new TypeError("Cannot convert value to AST: ".concat(inspect(serialized), "."));
  } // istanbul ignore next (Not reachable. All possible input types have been considered)


  invariant(0, 'Unexpected input type: ' + inspect(type));
}
/**
 * IntValue:
 *   - NegativeSign? 0
 *   - NegativeSign? NonZeroDigit ( Digit+ )?
 */

var integerStringRegExp = /^-?(?:0|[1-9][0-9]*)$/;

var __Schema = new GraphQLObjectType({
  name: '__Schema',
  description: 'A GraphQL Schema defines the capabilities of a GraphQL server. It exposes all available types and directives on the server, as well as the entry points for query, mutation, and subscription operations.',
  fields: function fields() {
    return {
      description: {
        type: GraphQLString,
        resolve: function resolve(schema) {
          return schema.description;
        }
      },
      types: {
        description: 'A list of all types supported by this server.',
        type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(__Type))),
        resolve: function resolve(schema) {
          return objectValues$1(schema.getTypeMap());
        }
      },
      queryType: {
        description: 'The type that query operations will be rooted at.',
        type: new GraphQLNonNull(__Type),
        resolve: function resolve(schema) {
          return schema.getQueryType();
        }
      },
      mutationType: {
        description: 'If this server supports mutation, the type that mutation operations will be rooted at.',
        type: __Type,
        resolve: function resolve(schema) {
          return schema.getMutationType();
        }
      },
      subscriptionType: {
        description: 'If this server support subscription, the type that subscription operations will be rooted at.',
        type: __Type,
        resolve: function resolve(schema) {
          return schema.getSubscriptionType();
        }
      },
      directives: {
        description: 'A list of all directives supported by this server.',
        type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(__Directive))),
        resolve: function resolve(schema) {
          return schema.getDirectives();
        }
      }
    };
  }
});
var __Directive = new GraphQLObjectType({
  name: '__Directive',
  description: "A Directive provides a way to describe alternate runtime execution and type validation behavior in a GraphQL document.\n\nIn some cases, you need to provide options to alter GraphQL's execution behavior in ways field arguments will not suffice, such as conditionally including or skipping a field. Directives provide this by describing additional information to the executor.",
  fields: function fields() {
    return {
      name: {
        type: new GraphQLNonNull(GraphQLString),
        resolve: function resolve(directive) {
          return directive.name;
        }
      },
      description: {
        type: GraphQLString,
        resolve: function resolve(directive) {
          return directive.description;
        }
      },
      isRepeatable: {
        type: new GraphQLNonNull(GraphQLBoolean),
        resolve: function resolve(directive) {
          return directive.isRepeatable;
        }
      },
      locations: {
        type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(__DirectiveLocation))),
        resolve: function resolve(directive) {
          return directive.locations;
        }
      },
      args: {
        type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(__InputValue))),
        args: {
          includeDeprecated: {
            type: GraphQLBoolean,
            defaultValue: false
          }
        },
        resolve: function resolve(field, _ref) {
          var includeDeprecated = _ref.includeDeprecated;
          return includeDeprecated ? field.args : field.args.filter(function (arg) {
            return arg.deprecationReason == null;
          });
        }
      }
    };
  }
});
var __DirectiveLocation = new GraphQLEnumType({
  name: '__DirectiveLocation',
  description: 'A Directive can be adjacent to many parts of the GraphQL language, a __DirectiveLocation describes one such possible adjacencies.',
  values: {
    QUERY: {
      value: DirectiveLocation.QUERY,
      description: 'Location adjacent to a query operation.'
    },
    MUTATION: {
      value: DirectiveLocation.MUTATION,
      description: 'Location adjacent to a mutation operation.'
    },
    SUBSCRIPTION: {
      value: DirectiveLocation.SUBSCRIPTION,
      description: 'Location adjacent to a subscription operation.'
    },
    FIELD: {
      value: DirectiveLocation.FIELD,
      description: 'Location adjacent to a field.'
    },
    FRAGMENT_DEFINITION: {
      value: DirectiveLocation.FRAGMENT_DEFINITION,
      description: 'Location adjacent to a fragment definition.'
    },
    FRAGMENT_SPREAD: {
      value: DirectiveLocation.FRAGMENT_SPREAD,
      description: 'Location adjacent to a fragment spread.'
    },
    INLINE_FRAGMENT: {
      value: DirectiveLocation.INLINE_FRAGMENT,
      description: 'Location adjacent to an inline fragment.'
    },
    VARIABLE_DEFINITION: {
      value: DirectiveLocation.VARIABLE_DEFINITION,
      description: 'Location adjacent to a variable definition.'
    },
    SCHEMA: {
      value: DirectiveLocation.SCHEMA,
      description: 'Location adjacent to a schema definition.'
    },
    SCALAR: {
      value: DirectiveLocation.SCALAR,
      description: 'Location adjacent to a scalar definition.'
    },
    OBJECT: {
      value: DirectiveLocation.OBJECT,
      description: 'Location adjacent to an object type definition.'
    },
    FIELD_DEFINITION: {
      value: DirectiveLocation.FIELD_DEFINITION,
      description: 'Location adjacent to a field definition.'
    },
    ARGUMENT_DEFINITION: {
      value: DirectiveLocation.ARGUMENT_DEFINITION,
      description: 'Location adjacent to an argument definition.'
    },
    INTERFACE: {
      value: DirectiveLocation.INTERFACE,
      description: 'Location adjacent to an interface definition.'
    },
    UNION: {
      value: DirectiveLocation.UNION,
      description: 'Location adjacent to a union definition.'
    },
    ENUM: {
      value: DirectiveLocation.ENUM,
      description: 'Location adjacent to an enum definition.'
    },
    ENUM_VALUE: {
      value: DirectiveLocation.ENUM_VALUE,
      description: 'Location adjacent to an enum value definition.'
    },
    INPUT_OBJECT: {
      value: DirectiveLocation.INPUT_OBJECT,
      description: 'Location adjacent to an input object type definition.'
    },
    INPUT_FIELD_DEFINITION: {
      value: DirectiveLocation.INPUT_FIELD_DEFINITION,
      description: 'Location adjacent to an input object field definition.'
    }
  }
});
var __Type = new GraphQLObjectType({
  name: '__Type',
  description: 'The fundamental unit of any GraphQL Schema is the type. There are many kinds of types in GraphQL as represented by the `__TypeKind` enum.\n\nDepending on the kind of a type, certain fields describe information about that type. Scalar types provide no information beyond a name, description and optional `specifiedByUrl`, while Enum types provide their values. Object and Interface types provide the fields they describe. Abstract types, Union and Interface, provide the Object types possible at runtime. List and NonNull types compose other types.',
  fields: function fields() {
    return {
      kind: {
        type: new GraphQLNonNull(__TypeKind),
        resolve: function resolve(type) {
          if (isScalarType(type)) {
            return TypeKind.SCALAR;
          }

          if (isObjectType(type)) {
            return TypeKind.OBJECT;
          }

          if (isInterfaceType(type)) {
            return TypeKind.INTERFACE;
          }

          if (isUnionType(type)) {
            return TypeKind.UNION;
          }

          if (isEnumType(type)) {
            return TypeKind.ENUM;
          }

          if (isInputObjectType(type)) {
            return TypeKind.INPUT_OBJECT;
          }

          if (isListType(type)) {
            return TypeKind.LIST;
          } // istanbul ignore else (See: 'https://github.com/graphql/graphql-js/issues/2618')


          if (isNonNullType(type)) {
            return TypeKind.NON_NULL;
          } // istanbul ignore next (Not reachable. All possible types have been considered)


          invariant(0, "Unexpected type: \"".concat(inspect(type), "\"."));
        }
      },
      name: {
        type: GraphQLString,
        resolve: function resolve(type) {
          return type.name !== undefined ? type.name : undefined;
        }
      },
      description: {
        type: GraphQLString,
        resolve: function resolve(type) {
          return type.description !== undefined ? type.description : undefined;
        }
      },
      specifiedByUrl: {
        type: GraphQLString,
        resolve: function resolve(obj) {
          return obj.specifiedByUrl !== undefined ? obj.specifiedByUrl : undefined;
        }
      },
      fields: {
        type: new GraphQLList(new GraphQLNonNull(__Field)),
        args: {
          includeDeprecated: {
            type: GraphQLBoolean,
            defaultValue: false
          }
        },
        resolve: function resolve(type, _ref2) {
          var includeDeprecated = _ref2.includeDeprecated;

          if (isObjectType(type) || isInterfaceType(type)) {
            var fields = objectValues$1(type.getFields());
            return includeDeprecated ? fields : fields.filter(function (field) {
              return field.deprecationReason == null;
            });
          }
        }
      },
      interfaces: {
        type: new GraphQLList(new GraphQLNonNull(__Type)),
        resolve: function resolve(type) {
          if (isObjectType(type) || isInterfaceType(type)) {
            return type.getInterfaces();
          }
        }
      },
      possibleTypes: {
        type: new GraphQLList(new GraphQLNonNull(__Type)),
        resolve: function resolve(type, _args, _context, _ref3) {
          var schema = _ref3.schema;

          if (isAbstractType(type)) {
            return schema.getPossibleTypes(type);
          }
        }
      },
      enumValues: {
        type: new GraphQLList(new GraphQLNonNull(__EnumValue)),
        args: {
          includeDeprecated: {
            type: GraphQLBoolean,
            defaultValue: false
          }
        },
        resolve: function resolve(type, _ref4) {
          var includeDeprecated = _ref4.includeDeprecated;

          if (isEnumType(type)) {
            var values = type.getValues();
            return includeDeprecated ? values : values.filter(function (field) {
              return field.deprecationReason == null;
            });
          }
        }
      },
      inputFields: {
        type: new GraphQLList(new GraphQLNonNull(__InputValue)),
        args: {
          includeDeprecated: {
            type: GraphQLBoolean,
            defaultValue: false
          }
        },
        resolve: function resolve(type, _ref5) {
          var includeDeprecated = _ref5.includeDeprecated;

          if (isInputObjectType(type)) {
            var values = objectValues$1(type.getFields());
            return includeDeprecated ? values : values.filter(function (field) {
              return field.deprecationReason == null;
            });
          }
        }
      },
      ofType: {
        type: __Type,
        resolve: function resolve(type) {
          return type.ofType !== undefined ? type.ofType : undefined;
        }
      }
    };
  }
});
var __Field = new GraphQLObjectType({
  name: '__Field',
  description: 'Object and Interface types are described by a list of Fields, each of which has a name, potentially a list of arguments, and a return type.',
  fields: function fields() {
    return {
      name: {
        type: new GraphQLNonNull(GraphQLString),
        resolve: function resolve(field) {
          return field.name;
        }
      },
      description: {
        type: GraphQLString,
        resolve: function resolve(field) {
          return field.description;
        }
      },
      args: {
        type: new GraphQLNonNull(new GraphQLList(new GraphQLNonNull(__InputValue))),
        args: {
          includeDeprecated: {
            type: GraphQLBoolean,
            defaultValue: false
          }
        },
        resolve: function resolve(field, _ref6) {
          var includeDeprecated = _ref6.includeDeprecated;
          return includeDeprecated ? field.args : field.args.filter(function (arg) {
            return arg.deprecationReason == null;
          });
        }
      },
      type: {
        type: new GraphQLNonNull(__Type),
        resolve: function resolve(field) {
          return field.type;
        }
      },
      isDeprecated: {
        type: new GraphQLNonNull(GraphQLBoolean),
        resolve: function resolve(field) {
          return field.deprecationReason != null;
        }
      },
      deprecationReason: {
        type: GraphQLString,
        resolve: function resolve(field) {
          return field.deprecationReason;
        }
      }
    };
  }
});
var __InputValue = new GraphQLObjectType({
  name: '__InputValue',
  description: 'Arguments provided to Fields or Directives and the input fields of an InputObject are represented as Input Values which describe their type and optionally a default value.',
  fields: function fields() {
    return {
      name: {
        type: new GraphQLNonNull(GraphQLString),
        resolve: function resolve(inputValue) {
          return inputValue.name;
        }
      },
      description: {
        type: GraphQLString,
        resolve: function resolve(inputValue) {
          return inputValue.description;
        }
      },
      type: {
        type: new GraphQLNonNull(__Type),
        resolve: function resolve(inputValue) {
          return inputValue.type;
        }
      },
      defaultValue: {
        type: GraphQLString,
        description: 'A GraphQL-formatted string representing the default value for this input value.',
        resolve: function resolve(inputValue) {
          var type = inputValue.type,
              defaultValue = inputValue.defaultValue;
          var valueAST = astFromValue(defaultValue, type);
          return valueAST ? print(valueAST) : null;
        }
      },
      isDeprecated: {
        type: new GraphQLNonNull(GraphQLBoolean),
        resolve: function resolve(field) {
          return field.deprecationReason != null;
        }
      },
      deprecationReason: {
        type: GraphQLString,
        resolve: function resolve(obj) {
          return obj.deprecationReason;
        }
      }
    };
  }
});
var __EnumValue = new GraphQLObjectType({
  name: '__EnumValue',
  description: 'One possible value for a given Enum. Enum values are unique values, not a placeholder for a string or numeric value. However an Enum value is returned in a JSON response as a string.',
  fields: function fields() {
    return {
      name: {
        type: new GraphQLNonNull(GraphQLString),
        resolve: function resolve(enumValue) {
          return enumValue.name;
        }
      },
      description: {
        type: GraphQLString,
        resolve: function resolve(enumValue) {
          return enumValue.description;
        }
      },
      isDeprecated: {
        type: new GraphQLNonNull(GraphQLBoolean),
        resolve: function resolve(enumValue) {
          return enumValue.deprecationReason != null;
        }
      },
      deprecationReason: {
        type: GraphQLString,
        resolve: function resolve(enumValue) {
          return enumValue.deprecationReason;
        }
      }
    };
  }
});
var TypeKind = Object.freeze({
  SCALAR: 'SCALAR',
  OBJECT: 'OBJECT',
  INTERFACE: 'INTERFACE',
  UNION: 'UNION',
  ENUM: 'ENUM',
  INPUT_OBJECT: 'INPUT_OBJECT',
  LIST: 'LIST',
  NON_NULL: 'NON_NULL'
});
var __TypeKind = new GraphQLEnumType({
  name: '__TypeKind',
  description: 'An enum describing what kind of type a given `__Type` is.',
  values: {
    SCALAR: {
      value: TypeKind.SCALAR,
      description: 'Indicates this type is a scalar.'
    },
    OBJECT: {
      value: TypeKind.OBJECT,
      description: 'Indicates this type is an object. `fields` and `interfaces` are valid fields.'
    },
    INTERFACE: {
      value: TypeKind.INTERFACE,
      description: 'Indicates this type is an interface. `fields`, `interfaces`, and `possibleTypes` are valid fields.'
    },
    UNION: {
      value: TypeKind.UNION,
      description: 'Indicates this type is a union. `possibleTypes` is a valid field.'
    },
    ENUM: {
      value: TypeKind.ENUM,
      description: 'Indicates this type is an enum. `enumValues` is a valid field.'
    },
    INPUT_OBJECT: {
      value: TypeKind.INPUT_OBJECT,
      description: 'Indicates this type is an input object. `inputFields` is a valid field.'
    },
    LIST: {
      value: TypeKind.LIST,
      description: 'Indicates this type is a list. `ofType` is a valid field.'
    },
    NON_NULL: {
      value: TypeKind.NON_NULL,
      description: 'Indicates this type is a non-null. `ofType` is a valid field.'
    }
  }
});
/**
 * Note that these are GraphQLField and not GraphQLFieldConfig,
 * so the format for args is different.
 */

({
  name: '__schema',
  type: new GraphQLNonNull(__Schema),
  description: 'Access the current type schema of this server.',
  args: [],
  resolve: function resolve(_source, _args, _context, _ref7) {
    var schema = _ref7.schema;
    return schema;
  },
  isDeprecated: false,
  deprecationReason: undefined,
  extensions: undefined,
  astNode: undefined
});
({
  name: '__type',
  type: __Type,
  description: 'Request the type information of a single type.',
  args: [{
    name: 'name',
    description: undefined,
    type: new GraphQLNonNull(GraphQLString),
    defaultValue: undefined,
    deprecationReason: undefined,
    extensions: undefined,
    astNode: undefined
  }],
  resolve: function resolve(_source, _ref8, _context, _ref9) {
    var name = _ref8.name;
    var schema = _ref9.schema;
    return schema.getType(name);
  },
  isDeprecated: false,
  deprecationReason: undefined,
  extensions: undefined,
  astNode: undefined
});
({
  name: '__typename',
  type: new GraphQLNonNull(GraphQLString),
  description: 'The name of the current Object type at runtime.',
  args: [],
  resolve: function resolve(_source, _args, _context, _ref10) {
    var parentType = _ref10.parentType;
    return parentType.name;
  },
  isDeprecated: false,
  deprecationReason: undefined,
  extensions: undefined,
  astNode: undefined
});
var introspectionTypes = Object.freeze([__Schema, __Directive, __DirectiveLocation, __Type, __Field, __InputValue, __EnumValue, __TypeKind]);

function _defineProperties$1(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass$1(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties$1(Constructor.prototype, protoProps); if (staticProps) _defineProperties$1(Constructor, staticProps); return Constructor; }
/**
 * Directives are used by the GraphQL runtime as a way of modifying execution
 * behavior. Type system creators will usually not create these directly.
 */

var GraphQLDirective = /*#__PURE__*/function () {
  function GraphQLDirective(config) {
    var _config$isRepeatable, _config$args;

    this.name = config.name;
    this.description = config.description;
    this.locations = config.locations;
    this.isRepeatable = (_config$isRepeatable = config.isRepeatable) !== null && _config$isRepeatable !== void 0 ? _config$isRepeatable : false;
    this.extensions = config.extensions && toObjMap(config.extensions);
    this.astNode = config.astNode;
    config.name || devAssert(0, 'Directive must be named.');
    Array.isArray(config.locations) || devAssert(0, "@".concat(config.name, " locations must be an Array."));
    var args = (_config$args = config.args) !== null && _config$args !== void 0 ? _config$args : {};
    isObjectLike(args) && !Array.isArray(args) || devAssert(0, "@".concat(config.name, " args must be an object with argument names as keys."));
    this.args = objectEntries$1(args).map(function (_ref) {
      var argName = _ref[0],
          argConfig = _ref[1];
      return {
        name: argName,
        description: argConfig.description,
        type: argConfig.type,
        defaultValue: argConfig.defaultValue,
        deprecationReason: argConfig.deprecationReason,
        extensions: argConfig.extensions && toObjMap(argConfig.extensions),
        astNode: argConfig.astNode
      };
    });
  }

  var _proto = GraphQLDirective.prototype;

  _proto.toConfig = function toConfig() {
    return {
      name: this.name,
      description: this.description,
      locations: this.locations,
      args: argsToArgsConfig(this.args),
      isRepeatable: this.isRepeatable,
      extensions: this.extensions,
      astNode: this.astNode
    };
  };

  _proto.toString = function toString() {
    return '@' + this.name;
  };

  _proto.toJSON = function toJSON() {
    return this.toString();
  } // $FlowFixMe[unsupported-syntax] Flow doesn't support computed properties yet
  ;

  _createClass$1(GraphQLDirective, [{
    key: SYMBOL_TO_STRING_TAG,
    get: function get() {
      return 'GraphQLDirective';
    }
  }]);

  return GraphQLDirective;
}(); // Print a simplified form when appearing in `inspect` and `util.inspect`.

defineInspect(GraphQLDirective);

/**
 * Used to conditionally include fields or fragments.
 */
new GraphQLDirective({
  name: 'include',
  description: 'Directs the executor to include this field or fragment only when the `if` argument is true.',
  locations: [DirectiveLocation.FIELD, DirectiveLocation.FRAGMENT_SPREAD, DirectiveLocation.INLINE_FRAGMENT],
  args: {
    if: {
      type: new GraphQLNonNull(GraphQLBoolean),
      description: 'Included when true.'
    }
  }
});
/**
 * Used to conditionally skip (exclude) fields or fragments.
 */

new GraphQLDirective({
  name: 'skip',
  description: 'Directs the executor to skip this field or fragment when the `if` argument is true.',
  locations: [DirectiveLocation.FIELD, DirectiveLocation.FRAGMENT_SPREAD, DirectiveLocation.INLINE_FRAGMENT],
  args: {
    if: {
      type: new GraphQLNonNull(GraphQLBoolean),
      description: 'Skipped when true.'
    }
  }
});
/**
 * Constant string used for default reason for a deprecation.
 */

var DEFAULT_DEPRECATION_REASON = 'No longer supported';
/**
 * Used to declare element of a GraphQL schema as deprecated.
 */

new GraphQLDirective({
  name: 'deprecated',
  description: 'Marks an element of a GraphQL schema as no longer supported.',
  locations: [DirectiveLocation.FIELD_DEFINITION, DirectiveLocation.ARGUMENT_DEFINITION, DirectiveLocation.INPUT_FIELD_DEFINITION, DirectiveLocation.ENUM_VALUE],
  args: {
    reason: {
      type: GraphQLString,
      description: 'Explains why this element was deprecated, usually also including a suggestion for how to access supported similar data. Formatted using the Markdown syntax, as specified by [CommonMark](https://commonmark.org/).',
      defaultValue: DEFAULT_DEPRECATION_REASON
    }
  }
});
/**
 * Used to provide a URL for specifying the behaviour of custom scalar definitions.
 */

new GraphQLDirective({
  name: 'specifiedBy',
  description: 'Exposes a URL that specifies the behaviour of this scalar.',
  locations: [DirectiveLocation.SCALAR],
  args: {
    url: {
      type: new GraphQLNonNull(GraphQLString),
      description: 'The URL that specifies the behaviour of this scalar.'
    }
  }
});
[].concat(specifiedScalarTypes, introspectionTypes).map(function (type) {
  return type.name;
});

var _defKindToExtKind;

function _defineProperty$1(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
(_defKindToExtKind = {}, _defineProperty$1(_defKindToExtKind, Kind.SCALAR_TYPE_DEFINITION, Kind.SCALAR_TYPE_EXTENSION), _defineProperty$1(_defKindToExtKind, Kind.OBJECT_TYPE_DEFINITION, Kind.OBJECT_TYPE_EXTENSION), _defineProperty$1(_defKindToExtKind, Kind.INTERFACE_TYPE_DEFINITION, Kind.INTERFACE_TYPE_EXTENSION), _defineProperty$1(_defKindToExtKind, Kind.UNION_TYPE_DEFINITION, Kind.UNION_TYPE_EXTENSION), _defineProperty$1(_defKindToExtKind, Kind.ENUM_TYPE_DEFINITION, Kind.ENUM_TYPE_EXTENSION), _defineProperty$1(_defKindToExtKind, Kind.INPUT_OBJECT_TYPE_DEFINITION, Kind.INPUT_OBJECT_TYPE_EXTENSION), _defKindToExtKind);
keyMap(specifiedScalarTypes.concat(introspectionTypes), function (type) {
  return type.name;
});

function createFragmentMap(documentNode) {
    const fragments = {};
    documentNode.definitions.forEach((node) => {
        if (node.kind === 'FragmentDefinition') {
            fragments[node.name.value] = node;
        }
    });
    return fragments;
}
function mergeFragmentWithExistingSelections(fragmentFieldSelection, existingSelections) {
    // Check if there is an existing selection for this field we can merge with
    const existingField = getRequestedField(fragmentFieldSelection.name.value, existingSelections);
    if (existingField === undefined) {
        existingSelections.push(fragmentFieldSelection);
    }
    else {
        if (existingField.selectionSet) {
            const mergedSelections = mergeSelectionSets(existingField.selectionSet, fragmentFieldSelection.selectionSet);
            existingField.selectionSet.selections = mergedSelections
                ? mergedSelections.selections
                : [];
        }
    }
}

function buildFieldState(state, fullPath, data) {
    return {
        ...state,
        path: {
            parent: null,
            propertyName: null,
            fullPath,
        },
        data,
    };
}
function serializeFieldArguments(argumentNodes, variables) {
    const mutableArgumentNodes = Object.assign([], argumentNodes);
    return `args__(${mutableArgumentNodes
        .sort((a, b) => {
        const aName = a.name.value.toUpperCase();
        const bName = b.name.value.toUpperCase();
        return aName < bName ? -1 : aName > bName ? 1 : 0;
    })
        .map((node) => serializeArgNode(node, variables))
        .join('::')})`;
}
function serializeArgNode(argumentNode, variables) {
    const argName = argumentNode.name.value;
    return `${argName}:${serializeValueNode(argumentNode.value, variables)}`;
}
function serializeValueNode(valueNode, variables) {
    switch (valueNode.kind) {
        case 'BooleanValue':
            return valueNode.value + '';
        case 'IntValue':
        case 'FloatValue':
        case 'EnumValue':
        case 'StringValue':
            return valueNode.value;
        case 'ListValue': {
            const mutableValueNodeList = Object.assign([], valueNode.values);
            return mutableValueNodeList
                .sort((a, b) => {
                const aVal = serializeValueNode(a, variables).toUpperCase();
                const bVal = serializeValueNode(b, variables).toUpperCase();
                return aVal < bVal ? -1 : aVal > bVal ? 1 : 0;
            })
                .map((val, i) => `${serializeValueNode(val, variables)}[${i}]`)
                .join(',');
        }
        case 'Variable': {
            const variableValue = variables[valueNode.name.value];
            return typeof variableValue === 'string'
                ? variableValue
                : JSON.stringify(variableValue);
        }
        case 'NullValue':
            return 'null';
        case 'ObjectValue': {
            const mutableFieldNodeList = Object.assign([], valueNode.fields);
            return mutableFieldNodeList
                .sort((a, b) => {
                const aName = a.name.value.toUpperCase();
                const bName = b.name.value.toUpperCase();
                return aName < bName ? -1 : aName > bName ? 1 : 0;
            })
                .map((field) => field.name.value + ':' + serializeValueNode(field.value, variables))
                .join(',');
        }
    }
}
let selectionSetRequestedFieldsWeakMap = new WeakMap();
function getRequestedFieldsForType(typename, selectionSet, namedFragmentsMap, isFragmentApplicable) {
    let cachedRequestedFieldsConfigurations = selectionSetRequestedFieldsWeakMap.get(selectionSet);
    if (cachedRequestedFieldsConfigurations === undefined) {
        cachedRequestedFieldsConfigurations = new Map();
        selectionSetRequestedFieldsWeakMap.set(selectionSet, cachedRequestedFieldsConfigurations);
    }
    const cachedRequestedFieldsForType = cachedRequestedFieldsConfigurations.get(typename);
    if (cachedRequestedFieldsForType !== undefined) {
        return cachedRequestedFieldsForType;
    }
    const selections = calculateRequestedFieldsForType(typename, selectionSet, namedFragmentsMap, isFragmentApplicable);
    cachedRequestedFieldsConfigurations.set(typename, selections);
    return selections;
}
function getRequestedField(responseDataFieldName, requestedFields) {
    return requestedFields.find((fieldNode) => (fieldNode.alias && fieldNode.alias.value === responseDataFieldName) ||
        (!fieldNode.alias && fieldNode.name.value === responseDataFieldName));
}
function calculateRequestedFieldsForType(typename, selectionSet, namedFragmentsMap, isFragmentApplicable) {
    const selections = [];
    selectionSet.selections.forEach((selection) => {
        if (selection.kind === 'Field') {
            selections.push(selection);
        }
        if (selection.kind === 'InlineFragment' && isFragmentApplicable(selection, typename)) {
            // loops over the map to get the values.
            getRequestedFieldsForType(typename, selection.selectionSet, namedFragmentsMap, isFragmentApplicable).forEach((fragmentFieldSelection) => {
                mergeFragmentWithExistingSelections(fragmentFieldSelection, selections);
            });
        }
        if (selection.kind === 'FragmentSpread') {
            const namedFragment = namedFragmentsMap[selection.name.value];
            if (namedFragment && isFragmentApplicable(namedFragment, typename)) {
                // loops over the map to get the values.
                getRequestedFieldsForType(typename, namedFragment.selectionSet, namedFragmentsMap, isFragmentApplicable).forEach((fragmentFieldSelection) => {
                    mergeFragmentWithExistingSelections(fragmentFieldSelection, selections);
                });
            }
        }
    });
    // Needs to happen after the selections are merged.
    return selections.reduce((acc, fieldNode) => {
        const fieldName = fieldNode.alias ? fieldNode.alias.value : fieldNode.name.value;
        // TODO: W-13485835. Some fields are not being merged, and this logic reproduces the current behavior in which the first field with name is being used.
        if (!acc.has(fieldName)) {
            acc.set(fieldName, fieldNode);
        }
        else {
            const existingFieldNode = acc.get(fieldName);
            acc.set(fieldName, {
                ...existingFieldNode,
                selectionSet: mergeSelectionSets(existingFieldNode === null || existingFieldNode === void 0 ? void 0 : existingFieldNode.selectionSet, fieldNode.selectionSet),
            });
        }
        return acc;
    }, new Map());
}
function mergeSelectionSets(set1, set2) {
    if (!set2) {
        return set1;
    }
    const set1Selections = !set1 ? [] : set1.selections;
    const mergedSelections = [...set1Selections];
    for (const selection of set2.selections) {
        if (selection.kind === 'Field') {
            const existingFieldIndex = mergedSelections.findIndex((sel) => {
                return sel.kind === 'Field' && isExactSameField(sel, selection);
            });
            if (existingFieldIndex > -1) {
                const existingField = mergedSelections[existingFieldIndex];
                const mergedSelectionSet = mergeSelectionSets(existingField.selectionSet, selection.selectionSet);
                if (mergedSelectionSet) {
                    mergedSelections[existingFieldIndex] = {
                        ...existingField,
                        selectionSet: mergedSelectionSet,
                    };
                }
            }
            else {
                mergedSelections.push(selection);
            }
        }
        else if (selection.kind === 'FragmentSpread') {
            if (!mergedSelections.some((sel) => sel.kind === 'FragmentSpread' &&
                sel.name.value === selection.name.value &&
                areDirectivesEqual(sel.directives, selection.directives))) {
                mergedSelections.push(selection);
            }
        }
        else if (selection.kind === 'InlineFragment') {
            const existingFragmentIndex = mergedSelections.findIndex((sel) => {
                return (sel.kind === 'InlineFragment' &&
                    areTypeConditionsEqual(sel.typeCondition, selection.typeCondition) &&
                    areDirectivesEqual(sel.directives, selection.directives));
            });
            if (existingFragmentIndex > -1) {
                const existingFragment = mergedSelections[existingFragmentIndex];
                const mergedSelectionSet = mergeSelectionSets(existingFragment.selectionSet, selection.selectionSet);
                if (mergedSelectionSet) {
                    mergedSelections[existingFragmentIndex] = {
                        ...existingFragment,
                        selectionSet: mergedSelectionSet,
                    };
                }
            }
            else {
                mergedSelections.push(selection);
            }
        }
    }
    return { kind: 'SelectionSet', selections: mergedSelections };
}
function areArgumentsEqual(args1, args2) {
    if (!args1 && !args2) {
        return true;
    }
    // length 0 arguments is semantically equivalent to falsy arguments.
    if ((!args1 || args1.length === 0) && (!args2 || args2.length === 0)) {
        return true;
    }
    if ((!args1 && args2) || (args1 && !args2)) {
        return false;
    }
    if (args1.length !== args2.length) {
        return false;
    }
    for (const arg of args1) {
        const matchingArg = args2.find((a) => a.name.value === arg.name.value && areValuesEqual(a.value, arg.value));
        if (!matchingArg) {
            return false;
        }
    }
    return true;
}
function areDirectivesEqual(dirs1, dirs2) {
    if (!dirs1 && !dirs2) {
        return true;
    }
    if ((!dirs1 || dirs1.length === 0) && (!dirs2 || dirs2.length === 0)) {
        return true;
    }
    if ((!dirs1 && dirs2) || (dirs1 && !dirs2)) {
        return false;
    }
    if (dirs1.length !== dirs2.length) {
        return false;
    }
    for (const dir of dirs1) {
        const matchingDir = dirs2.find((d) => d.name.value === dir.name.value && areArgumentsEqual(d.arguments, dir.arguments));
        if (!matchingDir) {
            return false;
        }
    }
    return true;
}
function areValuesEqual(valueNode1, valueNode2) {
    if (valueNode1.kind !== valueNode2.kind) {
        return false;
    }
    // Typescript doesn't understand that the above check means that the switch applies to both values. So there will be casts here.
    switch (valueNode1.kind) {
        case 'StringValue':
        case 'IntValue':
        case 'FloatValue':
        case 'BooleanValue':
        case 'EnumValue':
            return valueNode1.value === valueNode2.value;
        case 'NullValue':
            return true; // Both Null value nodes? Always true.
        case 'ListValue':
            if (valueNode1.values.length !== valueNode2.values.length) {
                return false;
            }
            for (let i = 0; i < valueNode1.values.length; i++) {
                if (!areValuesEqual(valueNode1.values[i], valueNode2.values[i])) {
                    return false;
                }
            }
            return true;
        case 'ObjectValue':
            if (valueNode1.fields.length !== valueNode2.fields.length) {
                return false;
            }
            for (let i = 0; i < valueNode1.fields.length; i++) {
                const field1 = valueNode1.fields[i];
                const field2 = valueNode2.fields.find((f) => f.name.value === field1.name.value);
                if (!field2 || !areValuesEqual(field1.value, field2.value)) {
                    return false;
                }
            }
            return true;
        case 'Variable':
            return valueNode1.name.value === valueNode2.name.value;
        default:
            return false;
    }
}
function areTypeConditionsEqual(typeCondition1, typeCondition2) {
    if (typeCondition1 === undefined && typeCondition2 === undefined) {
        return true;
    }
    if ((!typeCondition1 && typeCondition2) || (typeCondition1 && !typeCondition2)) {
        return false;
    }
    return typeCondition1.name.value === typeCondition2.name.value;
}
function isExactSameField(field1, field2) {
    var _a, _b;
    return (field1.name.value === field2.name.value &&
        ((_a = field1.alias) === null || _a === void 0 ? void 0 : _a.value) === ((_b = field2.alias) === null || _b === void 0 ? void 0 : _b.value) &&
        areArgumentsEqual(field1.arguments, field2.arguments) &&
        areDirectivesEqual(field1.directives, field2.directives));
}

function serializeOperationNode(operationNode, variables, fragmentMap) {
    return `${serializeSelectionSet(operationNode.selectionSet, variables, fragmentMap)}`;
}
function serializeSelectionSet(selectionSetNode, variables, fragmentMap) {
    return `${selectionSetNode.selections
        .map((selection) => serializeSelectionNode(selection, variables, fragmentMap))
        .join()}`;
}
/**
 *
 * @description This function takes a GraphQL SelectionNode from an AST and serializes it in a stable way, so we can
 * use it for property names and Store keys.
 * @param selectionNode
 * @param variables
 * @param fragmentMap
 * @returns string
 */
function serializeSelectionNode(selectionNode, variables, fragmentMap) {
    switch (selectionNode.kind) {
        case 'Field': {
            const hasArguments = selectionNode.arguments !== undefined && selectionNode.arguments.length > 0;
            const argumentSuffix = hasArguments
                ? `__${serializeFieldArguments(selectionNode.arguments, variables)}`
                : '';
            return `${selectionNode.name.value}${argumentSuffix}`;
        }
        case 'FragmentSpread': {
            const fragment = fragmentMap[selectionNode.name.value];
            return fragment === undefined
                ? selectionNode.name.value
                : serializeSelectionSet(fragment.selectionSet, variables, fragmentMap);
        }
        case 'InlineFragment':
            return serializeSelectionSet(selectionNode.selectionSet, variables, fragmentMap);
    }
}
function buildQueryTypeStringKey(args) {
    const { keyPrefix, schemaName, queryTypeName, operationNode, variables, fragmentMap } = args;
    return `${keyPrefix}::${schemaName}::${queryTypeName}[${serializeOperationNode(operationNode, variables, fragmentMap)}]`;
}

/**
 * @description Spec compliant way to retrieve the correct Operation from the Document that Luvio should operate on. https://spec.graphql.org/June2018/#sec-Named-Operation-Definitions
 * @param document
 * @param operationName
 * @returns The Operation in the GraphQL document we should use for the current call.
 */
function getOperationFromDocument(document, operationName) {
    const operations = document.definitions.filter((def) => def.kind === 'OperationDefinition');
    if (operationName) {
        return operations.find((def) => def.name !== undefined && def.name.value === operationName);
    }
    return operations[0]; // If a named operation is not provided, we return the first one
}

const { hasOwnProperty: ObjectPrototypeHasOwnProperty$1 } = Object.prototype;
// Deep merge that works on GraphQL data. It:
// - recursively merges any Object properties that are present in both
// - recursively merges arrays by deepMerging each Object item in the array (by index).
function deepMerge(target, ...sources) {
    for (const source of sources) {
        for (const key in source) {
            if (ObjectPrototypeHasOwnProperty$1.call(source, key)) {
                const targetValue = target[key];
                const sourceValue = source[key];
                if (Array.isArray(targetValue) && Array.isArray(sourceValue)) {
                    const maxLength = Math.max(targetValue.length, sourceValue.length);
                    target[key] = Array.from({ length: maxLength }, (_, index) => {
                        const targetItem = targetValue[index];
                        const sourceItem = sourceValue[index];
                        if (targetItem === undefined) {
                            return sourceItem;
                        }
                        else if (sourceItem === undefined) {
                            return targetItem;
                        }
                        else if (targetItem instanceof Object &&
                            !Array.isArray(targetItem) &&
                            sourceItem instanceof Object &&
                            !Array.isArray(sourceItem)) {
                            return deepMerge({}, targetItem, sourceItem);
                        }
                        else {
                            return sourceItem;
                        }
                    });
                }
                else if (targetValue instanceof Object &&
                    !Array.isArray(targetValue) &&
                    sourceValue instanceof Object &&
                    !Array.isArray(sourceValue)) {
                    deepMerge(targetValue, sourceValue);
                }
                else {
                    target[key] = sourceValue;
                }
            }
        }
    }
    return target;
}

/**
 * Copyright (c) 2022, Salesforce, Inc.,
 * All rights reserved.
 * For full license text, see the LICENSE.txt file
 */


/**
 * Defines configuration for the module with a default value which can be overridden by the runtime environment.
 */
// A holder for a configurable adapter override
class Configurable {
    constructor() {
        this.getAdapter = () => {
            return this.adapter;
        };
        this.setAdapter = (value) => {
            this.adapter = value;
        };
        this.adapter = undefined;
    }
}
// Configurable adapters that can have environmental overrides
let configurableCreateRecordAdapter = new Configurable();
let configurableUpdateRecordAdapter = new Configurable();
let configurableDeleteRecordAdapter = new Configurable();
let configurablePerformQuickActionAdapter = new Configurable();
let configurablePerformUpdateRecordQuickActionAdapter = new Configurable();
let configurableCreateContentDocumentAndVersion = new Configurable();
let configurableCreateContentVersion = new Configurable();
/**
 * Depth to which tracked fields will be added to a request that results from a cache miss.
 * A value of 0 inhibits the addition of tracked fields, 1 will add tracked fields that can
 * be reached by following 1 relationship from the root record, etc.
 * @defaultValue '5', replicates the current behavior
 */
let trackedFieldDepthOnCacheMiss = 5;
/**
 * Depth to which tracked fields will be added to a request that results from a merge conflict
 * A value of 0 inhibits the addition of tracked fields, 1 will add tracked fields that can
 * be reached by following 1 relationship from the root record, etc.
 * @defaultValue '5', replicates the current behavior
 */
let trackedFieldDepthOnCacheMergeConflict = 5;
/**
 * Depth to which tracked fields will be added to a request that results from a notify change invocation by the consumer
 * A value of 0 inhibits the addition of tracked fields, 1 will add tracked fields that can
 * be reached by following 1 relationship from the root record, etc.
 * @defaultValue '5', replicates the current behavior
 */
let trackedFieldDepthOnNotifyChange = 5;
/**
 * Determines if we will only fetch the 'Id' field for the leaf relationship record
 * @defaultValue 'false', replicates the current behavior and fetches all fields in the store for the leaf relationship record
 */
let trackedFieldLeafNodeIdAndNameOnly = false;
/**
 * One store enabled Get Object Info adapter
 */
let oneStoreGetObjectInfoAdapter = undefined;
/**
 * One store enabled Get Object Infos adapter
 */
let oneStoreGetObjectInfosAdapter = undefined;
/**
 * Determines when to include PDL strategies for Related Lists
 */
let relatedListsPredictionsEnabled = false;
/**
 * Determines whether to include additional PDL strategies for Related Lists
 */
let relatedListsPlusPredictionsEnabled = false;
let recordRepresentationIngestionOverride = undefined;
/**
 * Defines the configuration API and is exposed internally as well as externally.
 * Configuration for one store enabled REST adapters only.
 */
const configurationForOneStoreEnabledAdapters = {
    setGetObjectInfoAdapter: function (adapter) {
        oneStoreGetObjectInfoAdapter = adapter;
    },
    getGetObjectInfoAdapter: function () {
        return oneStoreGetObjectInfoAdapter;
    },
    setGetObjectInfosAdapter: function (adapter) {
        oneStoreGetObjectInfosAdapter = adapter;
    },
    getGetObjectInfosAdapter: function () {
        return oneStoreGetObjectInfosAdapter;
    },
};
const getKeywordSearchResultsFactory = new Configurable();
const getListRecordsByNameFactory = new Configurable();
const getListUiFactory = new Configurable();
const getLookupRecordsFactory = new Configurable();
const getQuickActionDefaultsFactory = new Configurable();
const getRecordCreateDefaultsFactory = new Configurable();
const getRecordTemplateCloneFactory = new Configurable();
const getRecordTemplateCreateFactory = new Configurable();
const getRecordUiFactory = new Configurable();
const getRecordFactory = new Configurable();
const getRecordsFactory = new Configurable();
const getRelatedListRecordsFactory = new Configurable();
const getRelatedListRecordsBatchFactory = new Configurable();
const getSearchResultsFactory = new Configurable();
// The following set of adapters all touch RecordRepresentation directly or indirectly,
// so they need to support having a custom factory provided by the mobile environment.
const configurationForEnvironmentFactoryOverrides = {
    getKeywordSearchResultsAdapterFactory: getKeywordSearchResultsFactory.getAdapter,
    setGetKeywordSearchResultsAdapterFactory: getKeywordSearchResultsFactory.setAdapter,
    getListRecordsByNameAdapterFactory: getListRecordsByNameFactory.getAdapter,
    setGetListRecordsByNameAdapterFactory: getListRecordsByNameFactory.setAdapter,
    getListUiAdapterFactory: getListUiFactory.getAdapter,
    setGetListUiAdapterFactory: getListUiFactory.setAdapter,
    getLookupRecordsAdapterFactory: getLookupRecordsFactory.getAdapter,
    setGetLookupRecordsAdapterFactory: getLookupRecordsFactory.setAdapter,
    getQuickActionDefaultsAdapterFactory: getQuickActionDefaultsFactory.getAdapter,
    setGetQuickActionDefaultsAdapterFactory: getQuickActionDefaultsFactory.setAdapter,
    getRecordCreateDefaultsAdapterFactory: getRecordCreateDefaultsFactory.getAdapter,
    setGetRecordCreateDefaultsAdapterFactory: getRecordCreateDefaultsFactory.setAdapter,
    getRecordTemplateCloneAdapterFactory: getRecordTemplateCloneFactory.getAdapter,
    setGetRecordTemplateCloneAdapterFactory: getRecordTemplateCloneFactory.setAdapter,
    getRecordTemplateCreateAdapterFactory: getRecordTemplateCreateFactory.getAdapter,
    setGetRecordTemplateCreateAdapterFactory: getRecordTemplateCreateFactory.setAdapter,
    getRecordUiAdapterFactory: getRecordUiFactory.getAdapter,
    setGetRecordUiAdapterFactory: getRecordUiFactory.setAdapter,
    getRecordAdapterFactory: getRecordFactory.getAdapter,
    setGetRecordAdapterFactory: getRecordFactory.setAdapter,
    getRecordsAdapterFactory: getRecordsFactory.getAdapter,
    setGetRecordsAdapterFactory: getRecordsFactory.setAdapter,
    getRelatedListRecordsAdapterFactory: getRelatedListRecordsFactory.getAdapter,
    setGetRelatedListRecordsAdapterFactory: getRelatedListRecordsFactory.setAdapter,
    getRelatedListRecordsBatchAdapterFactory: getRelatedListRecordsBatchFactory.getAdapter,
    setGetRelatedListRecordsBatchAdapterFactory: getRelatedListRecordsBatchFactory.setAdapter,
    getSearchResultsAdapterFactory: getSearchResultsFactory.getAdapter,
    setGetSearchResultsAdapterFactory: getSearchResultsFactory.setAdapter,
};
/**
 * Defines the configuration API and is exposed internally as well as externally.
 * Configuration for REST adapters only.
 */
const configurationForRestAdapters = {
    setTrackedFieldDepthOnCacheMiss: function (trackedFieldDepthOnCacheMissParam) {
        trackedFieldDepthOnCacheMiss = trackedFieldDepthOnCacheMissParam;
    },
    getTrackedFieldDepthOnCacheMiss: function () {
        return trackedFieldDepthOnCacheMiss;
    },
    setTrackedFieldDepthOnCacheMergeConflict: function (trackedFieldDepthOnCacheMergeConflictParam) {
        trackedFieldDepthOnCacheMergeConflict = trackedFieldDepthOnCacheMergeConflictParam;
    },
    getTrackedFieldDepthOnCacheMergeConflict: function () {
        return trackedFieldDepthOnCacheMergeConflict;
    },
    setTrackedFieldDepthOnNotifyChange: function (trackedFieldDepthOnNotifyChangeParam) {
        trackedFieldDepthOnNotifyChange = trackedFieldDepthOnNotifyChangeParam;
    },
    getTrackedFieldDepthOnNotifyChange: function () {
        return trackedFieldDepthOnNotifyChange;
    },
    setTrackedFieldLeafNodeIdAndNameOnly: function (trackedFieldLeafNodeIdAndNameOnlyParam) {
        trackedFieldLeafNodeIdAndNameOnly = trackedFieldLeafNodeIdAndNameOnlyParam;
    },
    getTrackedFieldLeafNodeIdAndNameOnly: function () {
        return trackedFieldLeafNodeIdAndNameOnly;
    },
    setRelatedListsPredictionsEnabled: function (f) {
        relatedListsPredictionsEnabled = f;
    },
    areRelatedListsPredictionsEnabled: function () {
        return relatedListsPredictionsEnabled === true;
    },
    setRelatedListsPlusPredictionsEnabled: function (f) {
        relatedListsPlusPredictionsEnabled = f;
    },
    areRelatedListsPlusPredictionsEnabled: function () {
        return relatedListsPlusPredictionsEnabled === true;
    },
    setRecordRepresentationIngestionOverride: function (ingest) {
        recordRepresentationIngestionOverride = ingest;
    },
    getRecordRepresentationIngestionOverride: function () {
        return recordRepresentationIngestionOverride;
    },
    // createRecord
    getDraftAwareCreateRecordAdapter: configurableCreateRecordAdapter.getAdapter,
    setDraftAwareCreateRecordAdapter: configurableCreateRecordAdapter.setAdapter,
    // updateRecord
    getDraftAwareUpdateRecordAdapter: configurableUpdateRecordAdapter.getAdapter,
    setDraftAwareUpdateRecordAdapter: configurableUpdateRecordAdapter.setAdapter,
    // deleteRecord
    getDraftAwareDeleteRecordAdapter: configurableDeleteRecordAdapter.getAdapter,
    setDraftAwareDeleteRecordAdapter: configurableDeleteRecordAdapter.setAdapter,
    // performQuickAction
    getDraftAwarePerformQuickActionAdapter: configurablePerformQuickActionAdapter.getAdapter,
    setDraftAwarePerformQuickActionAdapter: configurablePerformQuickActionAdapter.setAdapter,
    // performRecordUpdateQuickAction
    getDraftAwarePerformUpdateRecordQuickActionAdapter: configurablePerformUpdateRecordQuickActionAdapter.getAdapter,
    setDraftAwarePerformUpdateRecordQuickActionAdapter: configurablePerformUpdateRecordQuickActionAdapter.setAdapter,
    // createContentDocumentAndVersion
    getDraftAwareCreateContentDocumentAndVersionAdapter: configurableCreateContentDocumentAndVersion.getAdapter,
    setDraftAwareCreateContentDocumentAndVersionAdapter: configurableCreateContentDocumentAndVersion.setAdapter,
    // createContentVersion
    getDraftAwareCreateContentVersionAdapter: configurableCreateContentVersion.getAdapter,
    setDraftAwareCreateContentVersionAdapter: configurableCreateContentVersion.setAdapter,
    ...configurationForOneStoreEnabledAdapters,
    ...configurationForEnvironmentFactoryOverrides,
};
let configurableGraphQLAdapter = new Configurable();
let configurableGraphQLBatchAdapter = new Configurable();
/**
 * Defines the configuration API and is exposed internally as well as externally.
 * Configuration for GraphQL adapters only.
 */
const configurationForGraphQLAdapters = {
    getDraftAwareGraphQLAdapter: configurableGraphQLAdapter.getAdapter,
    setDraftAwareGraphQLAdapter: configurableGraphQLAdapter.setAdapter,
    getEnvironmentAwareGraphQLBatchAdapter: configurableGraphQLBatchAdapter.getAdapter,
    setEnvironmentAwareGraphQLBatchAdapter: configurableGraphQLBatchAdapter.setAdapter,
};
const registrations = new Set();
/**
 * lds-adapter-uiapi is special.  The non-SFDC bundle combines REST and GQL adapters,
 * yet for SFDC those are separate bundles.  So non-SFDC bundle (./main.ts) registers
 * both REST and GQL configs.  We want each SFDC bundle to register (we don't want one
 * SFDC bundle to assume the other is being loaded and therefore the other one will
 * take care of registration) but we don't want to double register either.
 *
 * So we make this function that memoizes if it's been called before and only
 * actually registers once.
 */
function ensureRegisteredOnce(registration) {
    const { id } = registration;
    if (!registrations.has(id)) {
        register(registration);
        registrations.add(id);
    }
}

// For use by callers within this module to instrument interesting things.
let instrumentation = {
    /**
     * Called when a set of record conflicts has been fully resolved. The
     * parameter indicates the number of server requests that were needed
     * to fully resolve all the record conflicts.
     */
    recordConflictsResolved: (_serverRequestCount) => { },
    /**
     * Called during merging of incoming and existing FieldValues.
     * Called when the incoming FieldValue display value is null.
     *
     * Note: Temporary instrumentation to capture distribution and frequency, W-8990630
     * Flipped to counter metric due to W-9611107
     */
    nullDisplayValueConflict: (_fieldInfo) => { },
    /**
     * SFDC Throttling
     * getRecordNotifyChange and notifyRecordUpdateAvailable are both throttled
     */
    getRecordNotifyChangeAllowed: () => { },
    getRecordNotifyChangeDropped: () => { },
    notifyRecordUpdateAvailableAllowed: () => { },
    notifyRecordUpdateAvailableDropped: () => { },
    /**
     * RecordRepresentation merge
     * Called when records change apiName
     */
    recordApiNameChanged: (_existingApiName, _incomingApiName) => { },
    recordTypeIdIsNull: (_apiName) => { },
    /**
     * RecordRepresentation merge
     * Called when either incoming or existing RecordRepresentation has a weakEtag=0
     */
    weakEtagZero: (_incomingWeakEtagZero, _existingWeakEtagZero, _apiName) => { },
    /**
     * getRecord notifyChangeFactory
     * Called when dispatchResourceRequest is resolved/rejected
     */
    getRecordNotifyChangeNetworkResult: (_uniqueWeakEtags, _error) => { },
};
/**
 * Allows external modules (typically a runtime environment) to set
 * instrumentation hooks for this module. Note that the hooks are
 * incremental - hooks not suppiled in newInstrumentation will retain
 * their previous values. The default instrumentation hooks are no-ops.
 *
 * @param newInstrumentation instrumentation hooks to be overridden
 */
function instrument(newInstrumentation) {
    instrumentation = Object.assign(instrumentation, newInstrumentation);
}

const { hasOwnProperty: ObjectPrototypeHasOwnProperty } = Object.prototype;
const { keys: ObjectKeys$1, create: ObjectCreate$1 } = Object;
const { stringify: JSONStringify$1 } = JSON;
const { isArray: ArrayIsArray$1 } = Array;
const { push: ArrayPrototypePush } = Array.prototype;
/**
 * Validates an adapter config is well-formed.
 * @param config The config to validate.
 * @param adapter The adapter validation configuration.
 * @param oneOf The keys the config must contain at least one of.
 * @throws A TypeError if config doesn't satisfy the adapter's config validation.
 */
function validateConfig(config, adapter, oneOf) {
    const { displayName } = adapter;
    const { required, optional, unsupported } = adapter.parameters;
    if (config === undefined ||
        required.every(req => ObjectPrototypeHasOwnProperty.call(config, req)) === false) {
        throw new TypeError(`adapter ${displayName} configuration must specify ${required.sort().join(', ')}`);
    }
    if (oneOf && oneOf.some(req => ObjectPrototypeHasOwnProperty.call(config, req)) === false) {
        throw new TypeError(`adapter ${displayName} configuration must specify one of ${oneOf.sort().join(', ')}`);
    }
    if (unsupported !== undefined &&
        unsupported.some(req => ObjectPrototypeHasOwnProperty.call(config, req))) {
        throw new TypeError(`adapter ${displayName} does not yet support ${unsupported.sort().join(', ')}`);
    }
    const supported = required.concat(optional);
    if (ObjectKeys$1(config).some(key => !supported.includes(key))) {
        throw new TypeError(`adapter ${displayName} configuration supports only ${supported.sort().join(', ')}`);
    }
}
function untrustedIsObject(untrusted) {
    return typeof untrusted === 'object' && untrusted !== null && ArrayIsArray$1(untrusted) === false;
}
function areRequiredParametersPresent(config, configPropertyNames) {
    return configPropertyNames.parameters.required.every(req => req in config);
}
const SNAPSHOT_STATE_UNFULFILLED = 'Unfulfilled';
const snapshotRefreshOptions = {
    overrides: {
        headers: {
            'Cache-Control': 'no-cache',
        },
    }
};
/**
 * A deterministic JSON stringify implementation. Heavily adapted from https://github.com/epoberezkin/fast-json-stable-stringify.
 * This is needed because insertion order for JSON.stringify(object) affects output:
 * JSON.stringify({a: 1, b: 2})
 *      "{"a":1,"b":2}"
 * JSON.stringify({b: 2, a: 1})
 *      "{"b":2,"a":1}"
 * @param data Data to be JSON-stringified.
 * @returns JSON.stringified value with consistent ordering of keys.
 */
function stableJSONStringify(node) {
    // This is for Date values.
    if (node && node.toJSON && typeof node.toJSON === 'function') {
        // eslint-disable-next-line no-param-reassign
        node = node.toJSON();
    }
    if (node === undefined) {
        return;
    }
    if (typeof node === 'number') {
        return isFinite(node) ? '' + node : 'null';
    }
    if (typeof node !== 'object') {
        return JSONStringify$1(node);
    }
    let i;
    let out;
    if (ArrayIsArray$1(node)) {
        out = '[';
        for (i = 0; i < node.length; i++) {
            if (i) {
                out += ',';
            }
            out += stableJSONStringify(node[i]) || 'null';
        }
        return out + ']';
    }
    if (node === null) {
        return 'null';
    }
    const keys = ObjectKeys$1(node).sort();
    out = '';
    for (i = 0; i < keys.length; i++) {
        const key = keys[i];
        const value = stableJSONStringify(node[key]);
        if (!value) {
            continue;
        }
        if (out) {
            out += ',';
        }
        out += JSONStringify$1(key) + ':' + value;
    }
    return '{' + out + '}';
}
function getFetchResponseStatusText(status) {
    switch (status) {
        case 200:
            return 'OK';
        case 201:
            return 'Created';
        case 304:
            return 'Not Modified';
        case 404:
            return 'Not Found';
        case 400:
            return 'Bad Request';
        case 500:
            return 'Server Error';
        default:
            return `Unexpected HTTP Status Code: ${status}`;
    }
}
function isUnfulfilledSnapshot$1(snapshot) {
    return snapshot.state === SNAPSHOT_STATE_UNFULFILLED;
}
function generateParamConfigMetadata(name, required, resourceType, typeCheckShape, isArrayShape = false, coerceFn) {
    return {
        name,
        required,
        resourceType,
        typeCheckShape,
        isArrayShape,
        coerceFn,
    };
}
function buildAdapterValidationConfig(displayName, paramsMeta) {
    const required = paramsMeta.filter(p => p.required).map(p => p.name);
    const optional = paramsMeta.filter(p => !p.required).map(p => p.name);
    return {
        displayName,
        parameters: {
            required,
            optional,
        }
    };
}
const keyPrefix = 'UiApi';

const { assign, create, freeze, isFrozen, keys } = Object;
const { hasOwnProperty } = Object.prototype;
const { split, endsWith } = String.prototype;
const { isArray } = Array;
const { concat, filter, includes, push, reduce } = Array.prototype;
const { parse, stringify } = JSON;

function isString(value) {
    return typeof value === 'string';
}
/**
 * @param value The array to inspect.
 * @returns True if the array is non-empty and contains only non-empty strings.
 */
function isArrayOfNonEmptyStrings(value) {
    if (value.length === 0) {
        return false;
    }
    return value.every((v) => isString(v) && v.trim().length > 0);
}
/**
 * @param value The array to dedupe
 * @returns An array without duplicates.
 */
function dedupe(value) {
    const result = {};
    for (let i = 0, len = value.length; i < len; i += 1) {
        result[value[i]] = true;
    }
    return keys(result);
}
/**
 * @param source The array of string to filter
 * @param compare The array to filter against
 * @returns An array with values from source that do not exist in compare
 * If the "compare" array is empty, "source" array itself is returned, not a shallow copy
 */
function difference(source, compare) {
    const { length: sourceLength } = source;
    const { length: compareLength } = compare;
    if (sourceLength === 0 || source === compare) {
        return [];
    }
    if (compareLength === 0) {
        return source;
    }
    // Put all the values from "compare" into a map
    // This should be faster than doing an indexOf for every string in source
    const map = {};
    for (let i = 0; i < compareLength; i += 1) {
        map[compare[i]] = true;
    }
    const strings = [];
    for (let i = 0; i < sourceLength; i += 1) {
        const string = source[i];
        if (map[string] === undefined) {
            strings.push(string);
        }
    }
    return strings;
}
/**
 * @param word The string to title case
 * @returns The sames string but in title case
 */
function toTitleCase(word) {
    return word.charAt(0).toUpperCase() + word.slice(1).toLowerCase();
}

function isObjectId(unknown) {
    if (typeof unknown !== 'object' || unknown === null) {
        return false;
    }
    return isString(unknown.objectApiName);
}

/**
 * Returns the object API name.
 * @param value The value from which to get the object API name.
 * @returns The object API name.
 */
function getObjectApiName$1(value) {
    // Note: tightening validation logic changes behavior from userland getting
    // a server-provided error to the adapter noop'ing. In 224 we decided to not
    // change the behavior.
    if (typeof value === 'string') {
        const trimmed = value.trim();
        if (trimmed.length > 0) {
            return trimmed;
        }
    }
    else if (isObjectId(value)) {
        return value.objectApiName.trim();
    }
    return undefined;
}

function isFieldId(unknown) {
    if (typeof unknown !== 'object' || unknown === null) {
        return false;
    }
    const value = unknown;
    return isString(value.objectApiName) && isString(value.fieldApiName);
}
function stringToFieldId(fieldApiName) {
    const split = fieldApiName.split('.');
    if (process.env.NODE_ENV !== 'production') {
        if (split.length === 1) {
            // object api name must non-empty
            throw new TypeError('Value does not include an object API name.');
        }
    }
    return {
        objectApiName: split[0],
        fieldApiName: split[1],
    };
}
function getFieldId(value) {
    if (isFieldId(value)) {
        return value;
    }
    return stringToFieldId(value);
}
/**
 * Split the object API name and field API name from a qualified field name.
 * Eg: Opportunity.Title returns ['Opportunity', 'Title']
 * Eg: Opportunity.Account.Name returns ['Opportunity', 'Account.Name']
 * @param fieldApiName The qualified field name.
 * @return The object and field API names.
 */
function splitQualifiedFieldApiName(fieldApiName) {
    const idx = fieldApiName.indexOf('.');
    if (idx < 1) {
        // object api name must non-empty
        throw new TypeError('Value does not include an object API name.');
    }
    return [fieldApiName.substring(0, idx), fieldApiName.substring(idx + 1)];
}

/**
 * Returns the field API name, qualified with an object name if possible.
 * @param value The value from which to get the qualified field API name.
 * @param onlyQualifiedFieldNames - Whether or not this function should skip fieldApiName that do not include the delimiter '.'
 * @return The qualified field API name.
 */
function getFieldApiName(value, onlyQualifiedFieldNames = false) {
    // Note: tightening validation logic changes behavior from userland getting
    // a server-provided error to the adapter noop'ing. In 224 we decided to not
    // change the behavior. In 250 we decided to add the 'onlyQualifiedFieldName' flag to tighten the logic
    // optionally to avoid issues with persisted invalid field names.
    if (isString(value)) {
        const trimmed = value.trim();
        if (trimmed.length > 0 && (onlyQualifiedFieldNames ? trimmed.indexOf('.') > -1 : true)) {
            return trimmed;
        }
    }
    else if (isFieldId(value)) {
        return value.objectApiName + '.' + value.fieldApiName;
    }
    return undefined;
}

/**
 * Returns the field API name.
 * @param value The value from which to get the field API name.
 * @param options Option bag. onlyQualifiedFieldNames is a boolean that allows this function to skip returning invalid FieldApiNames.
 * @returns The field API name.
 */
function getFieldApiNamesArray(value, options = { onlyQualifiedFieldNames: false }) {
    const valueArray = isArray(value) ? value : [value];
    const array = [];
    for (let i = 0, len = valueArray.length; i < len; i += 1) {
        const item = valueArray[i];
        const apiName = getFieldApiName(item, options.onlyQualifiedFieldNames);
        if (apiName === undefined) {
            if (options.onlyQualifiedFieldNames) {
                continue; // Just skips invalid field names rather than failing to return an array at all
            }
            return undefined;
        }
        push.call(array, apiName);
    }
    if (array.length === 0) {
        return undefined;
    }
    return dedupe(array).sort();
}

const { keys: ObjectKeys, create: ObjectCreate, assign: ObjectAssign } = Object;
const { isArray: ArrayIsArray } = Array;
const { stringify: JSONStringify } = JSON;
function equalsArray(a, b, equalsItem) {
    const aLength = a.length;
    const bLength = b.length;
    if (aLength !== bLength) {
        return false;
    }
    for (let i = 0; i < aLength; i++) {
        if (equalsItem(a[i], b[i]) === false) {
            return false;
        }
    }
    return true;
}
function equalsObject(a, b, equalsProp) {
    const aKeys = ObjectKeys(a).sort();
    const bKeys = ObjectKeys(b).sort();
    const aKeysLength = aKeys.length;
    const bKeysLength = bKeys.length;
    if (aKeysLength !== bKeysLength) {
        return false;
    }
    for (let i = 0; i < aKeys.length; i++) {
        const key = aKeys[i];
        if (key !== bKeys[i]) {
            return false;
        }
        if (equalsProp(a[key], b[key]) === false) {
            return false;
        }
    }
    return true;
}
function createLink$1(ref) {
    return {
        __ref: serializeStructuredKey(ref),
    };
}
function assignMetadataLink(entry, metadataKey) {
    entry['__metadata'] = createLink$1(metadataKey);
}

const VERSION$2Z = "623aa9ce3a11031e35faf5671a41746e";
function validate$2f(obj, path = 'ListFilterByInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_operandLabels = obj.operandLabels;
        const path_operandLabels = path + '.operandLabels';
        if (!ArrayIsArray(obj_operandLabels)) {
            return new TypeError('Expected "array" but received "' + typeof obj_operandLabels + '" (at "' + path_operandLabels + '")');
        }
        for (let i = 0; i < obj_operandLabels.length; i++) {
            const obj_operandLabels_item = obj_operandLabels[i];
            const path_operandLabels_item = path_operandLabels + '[' + i + ']';
            if (typeof obj_operandLabels_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_operandLabels_item + '" (at "' + path_operandLabels_item + '")');
            }
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3E = function ListFilterByInfoRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2Z,
        private: [],
        selections: [
            {
                name: 'fieldApiName',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'operandLabels',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'operator',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2Y = "f98dc7dec10408a9648ef8e5b37002a3";
function validate$2e(obj, path = 'ListInlineEditDetailsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_message = obj.message;
        const path_message = path + '.message';
        let obj_message_union0 = null;
        const obj_message_union0_error = (() => {
            if (typeof obj_message !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_message + '" (at "' + path_message + '")');
            }
        })();
        if (obj_message_union0_error != null) {
            obj_message_union0 = obj_message_union0_error.message;
        }
        let obj_message_union1 = null;
        const obj_message_union1_error = (() => {
            if (obj_message !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_message + '" (at "' + path_message + '")');
            }
        })();
        if (obj_message_union1_error != null) {
            obj_message_union1 = obj_message_union1_error.message;
        }
        if (obj_message_union0 && obj_message_union1) {
            let message = 'Object doesn\'t match union (at "' + path_message + '")';
            message += '\n' + obj_message_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_message_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_state = obj.state;
        const path_state = path + '.state';
        let obj_state_union0 = null;
        const obj_state_union0_error = (() => {
            if (typeof obj_state !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_state + '" (at "' + path_state + '")');
            }
        })();
        if (obj_state_union0_error != null) {
            obj_state_union0 = obj_state_union0_error.message;
        }
        let obj_state_union1 = null;
        const obj_state_union1_error = (() => {
            if (obj_state !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_state + '" (at "' + path_state + '")');
            }
        })();
        if (obj_state_union1_error != null) {
            obj_state_union1 = obj_state_union1_error.message;
        }
        if (obj_state_union0 && obj_state_union1) {
            let message = 'Object doesn\'t match union (at "' + path_state + '")';
            message += '\n' + obj_state_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_state_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3D = function ListInlineEditDetailsRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2Y,
        private: [],
        selections: [
            {
                name: 'message',
                kind: 'Scalar'
            },
            {
                name: 'state',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2X = "76042ff4af603b2ac0ec69fa0bd12046";
function validate$2d(obj, path = 'ListReferenceRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_listViewApiName = obj.listViewApiName;
        const path_listViewApiName = path + '.listViewApiName';
        let obj_listViewApiName_union0 = null;
        const obj_listViewApiName_union0_error = (() => {
            if (typeof obj_listViewApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_listViewApiName + '" (at "' + path_listViewApiName + '")');
            }
        })();
        if (obj_listViewApiName_union0_error != null) {
            obj_listViewApiName_union0 = obj_listViewApiName_union0_error.message;
        }
        let obj_listViewApiName_union1 = null;
        const obj_listViewApiName_union1_error = (() => {
            if (obj_listViewApiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_listViewApiName + '" (at "' + path_listViewApiName + '")');
            }
        })();
        if (obj_listViewApiName_union1_error != null) {
            obj_listViewApiName_union1 = obj_listViewApiName_union1_error.message;
        }
        if (obj_listViewApiName_union0 && obj_listViewApiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_listViewApiName + '")';
            message += '\n' + obj_listViewApiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_listViewApiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3C = function ListReferenceRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2X,
        private: [],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'listViewApiName',
                kind: 'Scalar'
            },
            {
                name: 'objectApiName',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$1g(existing, incoming) {
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_type = existing.type;
    const incoming_type = incoming.type;
    if (!(existing_type === incoming_type)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_listViewApiName = existing.listViewApiName;
    const incoming_listViewApiName = incoming.listViewApiName;
    if (!(existing_listViewApiName === incoming_listViewApiName)) {
        return false;
    }
    return true;
}

const VERSION$2W = "f1519bef57e70ab7c5ef160dc5ff0c47";
function validate$2c(obj, path = 'ListInfoShareRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_shareApiName = obj.shareApiName;
        const path_shareApiName = path + '.shareApiName';
        if (typeof obj_shareApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_shareApiName + '" (at "' + path_shareApiName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3B = function ListInfoShareRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2W,
        private: [],
        selections: [
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'shareApiName',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2V = "561657f32721ec1dac1601575b029640";
function validate$2b(obj, path = 'ListInfoShareCategoryRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_shareType = obj.shareType;
        const path_shareType = path + '.shareType';
        if (typeof obj_shareType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_shareType + '" (at "' + path_shareType + '")');
        }
        const obj_shares = obj.shares;
        const path_shares = path + '.shares';
        if (!ArrayIsArray(obj_shares)) {
            return new TypeError('Expected "array" but received "' + typeof obj_shares + '" (at "' + path_shares + '")');
        }
        for (let i = 0; i < obj_shares.length; i++) {
            const obj_shares_item = obj_shares[i];
            const path_shares_item = path_shares + '[' + i + ']';
            const referencepath_shares_itemValidationError = validate$2c(obj_shares_item, path_shares_item);
            if (referencepath_shares_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListInfoShareRepresentation (at "' + path_shares_item + '")\n';
                message += referencepath_shares_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3A = function ListInfoShareCategoryRepresentationSelect() {
    const { selections: ListInfoShareRepresentation__selections, opaque: ListInfoShareRepresentation__opaque, } = select$3B();
    return {
        kind: 'Fragment',
        version: VERSION$2V,
        private: [],
        selections: [
            {
                name: 'shareType',
                kind: 'Scalar'
            },
            {
                name: 'shares',
                kind: 'Object',
                plural: true,
                selections: ListInfoShareRepresentation__selections
            }
        ]
    };
};

const VERSION$2U = "32def9b631252c12b91a8209c1f49f5a";
function validate$2a(obj, path = 'ListOrderByInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_isAscending = obj.isAscending;
        const path_isAscending = path + '.isAscending';
        if (typeof obj_isAscending !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isAscending + '" (at "' + path_isAscending + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        let obj_label_union0 = null;
        const obj_label_union0_error = (() => {
            if (typeof obj_label !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
            }
        })();
        if (obj_label_union0_error != null) {
            obj_label_union0 = obj_label_union0_error.message;
        }
        let obj_label_union1 = null;
        const obj_label_union1_error = (() => {
            if (obj_label !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_label + '" (at "' + path_label + '")');
            }
        })();
        if (obj_label_union1_error != null) {
            obj_label_union1 = obj_label_union1_error.message;
        }
        if (obj_label_union0 && obj_label_union1) {
            let message = 'Object doesn\'t match union (at "' + path_label + '")';
            message += '\n' + obj_label_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_label_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3z = function ListOrderByInfoRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2U,
        private: [],
        selections: [
            {
                name: 'fieldApiName',
                kind: 'Scalar'
            },
            {
                name: 'isAscending',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$1f(existing, incoming) {
    const existing_isAscending = existing.isAscending;
    const incoming_isAscending = incoming.isAscending;
    if (!(existing_isAscending === incoming_isAscending)) {
        return false;
    }
    const existing_fieldApiName = existing.fieldApiName;
    const incoming_fieldApiName = incoming.fieldApiName;
    if (!(existing_fieldApiName === incoming_fieldApiName)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    return true;
}

const VERSION$2T = "2634258f216db34315c06d759a35676d";
function validate$29(obj, path = 'ListScopeEntityRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3y = function ListScopeEntityRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2T,
        private: [],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2S = "3b85c5a08d50ce328481b9f8ab56127b";
function validate$28(obj, path = 'ListScopeRelatedEntityRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3x = function ListScopeRelatedEntityRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2S,
        private: [],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2R = "2fe9814e7124b47f59da585483c1b3f9";
function validate$27(obj, path = 'ListScopeRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_entity = obj.entity;
        const path_entity = path + '.entity';
        let obj_entity_union0 = null;
        const obj_entity_union0_error = (() => {
            const referencepath_entityValidationError = validate$29(obj_entity, path_entity);
            if (referencepath_entityValidationError !== null) {
                let message = 'Object doesn\'t match ListScopeEntityRepresentation (at "' + path_entity + '")\n';
                message += referencepath_entityValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_entity_union0_error != null) {
            obj_entity_union0 = obj_entity_union0_error.message;
        }
        let obj_entity_union1 = null;
        const obj_entity_union1_error = (() => {
            if (obj_entity !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_entity + '" (at "' + path_entity + '")');
            }
        })();
        if (obj_entity_union1_error != null) {
            obj_entity_union1 = obj_entity_union1_error.message;
        }
        if (obj_entity_union0 && obj_entity_union1) {
            let message = 'Object doesn\'t match union (at "' + path_entity + '")';
            message += '\n' + obj_entity_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_entity_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        let obj_label_union0 = null;
        const obj_label_union0_error = (() => {
            if (typeof obj_label !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
            }
        })();
        if (obj_label_union0_error != null) {
            obj_label_union0 = obj_label_union0_error.message;
        }
        let obj_label_union1 = null;
        const obj_label_union1_error = (() => {
            if (obj_label !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_label + '" (at "' + path_label + '")');
            }
        })();
        if (obj_label_union1_error != null) {
            obj_label_union1 = obj_label_union1_error.message;
        }
        if (obj_label_union0 && obj_label_union1) {
            let message = 'Object doesn\'t match union (at "' + path_label + '")';
            message += '\n' + obj_label_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_label_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_relatedEntity = obj.relatedEntity;
        const path_relatedEntity = path + '.relatedEntity';
        let obj_relatedEntity_union0 = null;
        const obj_relatedEntity_union0_error = (() => {
            const referencepath_relatedEntityValidationError = validate$28(obj_relatedEntity, path_relatedEntity);
            if (referencepath_relatedEntityValidationError !== null) {
                let message = 'Object doesn\'t match ListScopeRelatedEntityRepresentation (at "' + path_relatedEntity + '")\n';
                message += referencepath_relatedEntityValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_relatedEntity_union0_error != null) {
            obj_relatedEntity_union0 = obj_relatedEntity_union0_error.message;
        }
        let obj_relatedEntity_union1 = null;
        const obj_relatedEntity_union1_error = (() => {
            if (obj_relatedEntity !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_relatedEntity + '" (at "' + path_relatedEntity + '")');
            }
        })();
        if (obj_relatedEntity_union1_error != null) {
            obj_relatedEntity_union1 = obj_relatedEntity_union1_error.message;
        }
        if (obj_relatedEntity_union0 && obj_relatedEntity_union1) {
            let message = 'Object doesn\'t match union (at "' + path_relatedEntity + '")';
            message += '\n' + obj_relatedEntity_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_relatedEntity_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3w = function ListScopeRepresentationSelect() {
    const { selections: ListScopeEntityRepresentation__selections, opaque: ListScopeEntityRepresentation__opaque, } = select$3y();
    const { selections: ListScopeRelatedEntityRepresentation__selections, opaque: ListScopeRelatedEntityRepresentation__opaque, } = select$3x();
    return {
        kind: 'Fragment',
        version: VERSION$2R,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'entity',
                kind: 'Object',
                nullable: true,
                selections: ListScopeEntityRepresentation__selections
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'relatedEntity',
                kind: 'Object',
                nullable: true,
                selections: ListScopeRelatedEntityRepresentation__selections
            }
        ]
    };
};

const VERSION$2Q = "6506134f4d72fdfa349fe60ef1af2413";
function validate$26(obj, path = 'ListUserPreferenceRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_columnWidths = obj.columnWidths;
        const path_columnWidths = path + '.columnWidths';
        if (typeof obj_columnWidths !== 'object' || ArrayIsArray(obj_columnWidths) || obj_columnWidths === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_columnWidths + '" (at "' + path_columnWidths + '")');
        }
        const obj_columnWidths_keys = ObjectKeys(obj_columnWidths);
        for (let i = 0; i < obj_columnWidths_keys.length; i++) {
            const key = obj_columnWidths_keys[i];
            const obj_columnWidths_prop = obj_columnWidths[key];
            const path_columnWidths_prop = path_columnWidths + '["' + key + '"]';
            if (typeof obj_columnWidths_prop !== 'number' || (typeof obj_columnWidths_prop === 'number' && Math.floor(obj_columnWidths_prop) !== obj_columnWidths_prop)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_columnWidths_prop + '" (at "' + path_columnWidths_prop + '")');
            }
        }
        const obj_columnWrap = obj.columnWrap;
        const path_columnWrap = path + '.columnWrap';
        if (typeof obj_columnWrap !== 'object' || ArrayIsArray(obj_columnWrap) || obj_columnWrap === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_columnWrap + '" (at "' + path_columnWrap + '")');
        }
        const obj_columnWrap_keys = ObjectKeys(obj_columnWrap);
        for (let i = 0; i < obj_columnWrap_keys.length; i++) {
            const key = obj_columnWrap_keys[i];
            const obj_columnWrap_prop = obj_columnWrap[key];
            const path_columnWrap_prop = path_columnWrap + '["' + key + '"]';
            if (typeof obj_columnWrap_prop !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_columnWrap_prop + '" (at "' + path_columnWrap_prop + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3v = function ListUserPreferenceRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2Q,
        private: [],
        selections: [
            {
                name: 'columnWidths',
                kind: 'Scalar',
                map: true
            },
            {
                name: 'columnWrap',
                kind: 'Scalar',
                map: true
            }
        ]
    };
};

function validate$25(obj, path = 'ListColumnInlineEditAttributesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_editable = obj.editable;
        const path_editable = path + '.editable';
        if (typeof obj_editable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_editable + '" (at "' + path_editable + '")');
        }
        const obj_required = obj.required;
        const path_required = path + '.required';
        if (typeof obj_required !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_required + '" (at "' + path_required + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$24(obj, path = 'ListColumnInlineEditAttributesRepresentationWrapper') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_keys = ObjectKeys(obj);
        for (let i = 0; i < obj_keys.length; i++) {
            const key = obj_keys[i];
            const obj_prop = obj[key];
            const path_prop = path + '["' + key + '"]';
            const referencepath_propValidationError = validate$25(obj_prop, path_prop);
            if (referencepath_propValidationError !== null) {
                let message = 'Object doesn\'t match ListColumnInlineEditAttributesRepresentation (at "' + path_prop + '")\n';
                message += referencepath_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

const VERSION$2P = "64e2cac6d374ad92f491a9f8a952c6c3";
function validate$23(obj, path = 'ListColumnRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_inlineEditAttributes = obj.inlineEditAttributes;
        const path_inlineEditAttributes = path + '.inlineEditAttributes';
        let obj_inlineEditAttributes_union0 = null;
        const obj_inlineEditAttributes_union0_error = (() => {
            const referencepath_inlineEditAttributesValidationError = validate$24(obj_inlineEditAttributes, path_inlineEditAttributes);
            if (referencepath_inlineEditAttributesValidationError !== null) {
                let message = 'Object doesn\'t match ListColumnInlineEditAttributesRepresentationWrapper (at "' + path_inlineEditAttributes + '")\n';
                message += referencepath_inlineEditAttributesValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_inlineEditAttributes_union0_error != null) {
            obj_inlineEditAttributes_union0 = obj_inlineEditAttributes_union0_error.message;
        }
        let obj_inlineEditAttributes_union1 = null;
        const obj_inlineEditAttributes_union1_error = (() => {
            if (obj_inlineEditAttributes !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_inlineEditAttributes + '" (at "' + path_inlineEditAttributes + '")');
            }
        })();
        if (obj_inlineEditAttributes_union1_error != null) {
            obj_inlineEditAttributes_union1 = obj_inlineEditAttributes_union1_error.message;
        }
        if (obj_inlineEditAttributes_union0 && obj_inlineEditAttributes_union1) {
            let message = 'Object doesn\'t match union (at "' + path_inlineEditAttributes + '")';
            message += '\n' + obj_inlineEditAttributes_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_inlineEditAttributes_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_lookupId = obj.lookupId;
        const path_lookupId = path + '.lookupId';
        let obj_lookupId_union0 = null;
        const obj_lookupId_union0_error = (() => {
            if (typeof obj_lookupId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lookupId + '" (at "' + path_lookupId + '")');
            }
        })();
        if (obj_lookupId_union0_error != null) {
            obj_lookupId_union0 = obj_lookupId_union0_error.message;
        }
        let obj_lookupId_union1 = null;
        const obj_lookupId_union1_error = (() => {
            if (obj_lookupId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lookupId + '" (at "' + path_lookupId + '")');
            }
        })();
        if (obj_lookupId_union1_error != null) {
            obj_lookupId_union1 = obj_lookupId_union1_error.message;
        }
        if (obj_lookupId_union0 && obj_lookupId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lookupId + '")';
            message += '\n' + obj_lookupId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lookupId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_searchable = obj.searchable;
        const path_searchable = path + '.searchable';
        if (typeof obj_searchable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_searchable + '" (at "' + path_searchable + '")');
        }
        const obj_sortable = obj.sortable;
        const path_sortable = path + '.sortable';
        if (typeof obj_sortable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_sortable + '" (at "' + path_sortable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$12 = 'ListColumnRepresentation';
function normalize$W(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3u = function ListColumnRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2P,
        private: [],
        opaque: true
    };
};
function equals$1e(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$2k = function ListColumnRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$23(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$W, "UiApi", VERSION$2P, RepresentationType$12, equals$1e);
    return createLink$1(key);
};
function getTypeCacheKeys$2o(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$12,
        mergeable: false
    });
}

const keyBuilderFromType$F = function ListInfoRepresentationKeyBuilderFromType(luvio, object) {
    const keyParams = {
        listViewApiName: object.listReference.listViewApiName
            ? object.listReference.listViewApiName.toLowerCase()
            : object.listReference.listViewApiName,
        objectApiName: object.listReference.objectApiName.toLowerCase(),
        type: object.listReference.type,
    };
    return keyBuilder$3F(luvio, keyParams);
};

const TTL$H = 900000;
const VERSION$2O = "5adf17d38be03c0f71b1c52485b377a6";
function validate$22(obj, path = 'ListInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_cloneable = obj.cloneable;
        const path_cloneable = path + '.cloneable';
        if (typeof obj_cloneable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_cloneable + '" (at "' + path_cloneable + '")');
        }
        const obj_createable = obj.createable;
        const path_createable = path + '.createable';
        if (typeof obj_createable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_createable + '" (at "' + path_createable + '")');
        }
        const obj_deletable = obj.deletable;
        const path_deletable = path + '.deletable';
        if (typeof obj_deletable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_deletable + '" (at "' + path_deletable + '")');
        }
        const obj_displayColumns = obj.displayColumns;
        const path_displayColumns = path + '.displayColumns';
        if (!ArrayIsArray(obj_displayColumns)) {
            return new TypeError('Expected "array" but received "' + typeof obj_displayColumns + '" (at "' + path_displayColumns + '")');
        }
        for (let i = 0; i < obj_displayColumns.length; i++) {
            const obj_displayColumns_item = obj_displayColumns[i];
            const path_displayColumns_item = path_displayColumns + '[' + i + ']';
            if (typeof obj_displayColumns_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_displayColumns_item + '" (at "' + path_displayColumns_item + '")');
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_filterLogicString = obj.filterLogicString;
        const path_filterLogicString = path + '.filterLogicString';
        let obj_filterLogicString_union0 = null;
        const obj_filterLogicString_union0_error = (() => {
            if (typeof obj_filterLogicString !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_filterLogicString + '" (at "' + path_filterLogicString + '")');
            }
        })();
        if (obj_filterLogicString_union0_error != null) {
            obj_filterLogicString_union0 = obj_filterLogicString_union0_error.message;
        }
        let obj_filterLogicString_union1 = null;
        const obj_filterLogicString_union1_error = (() => {
            if (obj_filterLogicString !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_filterLogicString + '" (at "' + path_filterLogicString + '")');
            }
        })();
        if (obj_filterLogicString_union1_error != null) {
            obj_filterLogicString_union1 = obj_filterLogicString_union1_error.message;
        }
        if (obj_filterLogicString_union0 && obj_filterLogicString_union1) {
            let message = 'Object doesn\'t match union (at "' + path_filterLogicString + '")';
            message += '\n' + obj_filterLogicString_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_filterLogicString_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_filteredByInfo = obj.filteredByInfo;
        const path_filteredByInfo = path + '.filteredByInfo';
        if (!ArrayIsArray(obj_filteredByInfo)) {
            return new TypeError('Expected "array" but received "' + typeof obj_filteredByInfo + '" (at "' + path_filteredByInfo + '")');
        }
        for (let i = 0; i < obj_filteredByInfo.length; i++) {
            const obj_filteredByInfo_item = obj_filteredByInfo[i];
            const path_filteredByInfo_item = path_filteredByInfo + '[' + i + ']';
            const referencepath_filteredByInfo_itemValidationError = validate$2f(obj_filteredByInfo_item, path_filteredByInfo_item);
            if (referencepath_filteredByInfo_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListFilterByInfoRepresentation (at "' + path_filteredByInfo_item + '")\n';
                message += referencepath_filteredByInfo_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        if (obj.hasMassActions !== undefined) {
            const obj_hasMassActions = obj.hasMassActions;
            const path_hasMassActions = path + '.hasMassActions';
            if (typeof obj_hasMassActions !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_hasMassActions + '" (at "' + path_hasMassActions + '")');
            }
        }
        const obj_inlineEditDetails = obj.inlineEditDetails;
        const path_inlineEditDetails = path + '.inlineEditDetails';
        const referencepath_inlineEditDetailsValidationError = validate$2e(obj_inlineEditDetails, path_inlineEditDetails);
        if (referencepath_inlineEditDetailsValidationError !== null) {
            let message = 'Object doesn\'t match ListInlineEditDetailsRepresentation (at "' + path_inlineEditDetails + '")\n';
            message += referencepath_inlineEditDetailsValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        let obj_label_union0 = null;
        const obj_label_union0_error = (() => {
            if (typeof obj_label !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
            }
        })();
        if (obj_label_union0_error != null) {
            obj_label_union0 = obj_label_union0_error.message;
        }
        let obj_label_union1 = null;
        const obj_label_union1_error = (() => {
            if (obj_label !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_label + '" (at "' + path_label + '")');
            }
        })();
        if (obj_label_union1_error != null) {
            obj_label_union1 = obj_label_union1_error.message;
        }
        if (obj_label_union0 && obj_label_union1) {
            let message = 'Object doesn\'t match union (at "' + path_label + '")';
            message += '\n' + obj_label_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_label_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_listReference = obj.listReference;
        const path_listReference = path + '.listReference';
        const referencepath_listReferenceValidationError = validate$2d(obj_listReference, path_listReference);
        if (referencepath_listReferenceValidationError !== null) {
            let message = 'Object doesn\'t match ListReferenceRepresentation (at "' + path_listReference + '")\n';
            message += referencepath_listReferenceValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_listShares = obj.listShares;
        const path_listShares = path + '.listShares';
        if (!ArrayIsArray(obj_listShares)) {
            return new TypeError('Expected "array" but received "' + typeof obj_listShares + '" (at "' + path_listShares + '")');
        }
        for (let i = 0; i < obj_listShares.length; i++) {
            const obj_listShares_item = obj_listShares[i];
            const path_listShares_item = path_listShares + '[' + i + ']';
            const referencepath_listShares_itemValidationError = validate$2b(obj_listShares_item, path_listShares_item);
            if (referencepath_listShares_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListInfoShareCategoryRepresentation (at "' + path_listShares_item + '")\n';
                message += referencepath_listShares_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_objectApiNames = obj.objectApiNames;
        const path_objectApiNames = path + '.objectApiNames';
        if (!ArrayIsArray(obj_objectApiNames)) {
            return new TypeError('Expected "array" but received "' + typeof obj_objectApiNames + '" (at "' + path_objectApiNames + '")');
        }
        for (let i = 0; i < obj_objectApiNames.length; i++) {
            const obj_objectApiNames_item = obj_objectApiNames[i];
            const path_objectApiNames_item = path_objectApiNames + '[' + i + ']';
            if (typeof obj_objectApiNames_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectApiNames_item + '" (at "' + path_objectApiNames_item + '")');
            }
        }
        const obj_orderedByInfo = obj.orderedByInfo;
        const path_orderedByInfo = path + '.orderedByInfo';
        if (!ArrayIsArray(obj_orderedByInfo)) {
            return new TypeError('Expected "array" but received "' + typeof obj_orderedByInfo + '" (at "' + path_orderedByInfo + '")');
        }
        for (let i = 0; i < obj_orderedByInfo.length; i++) {
            const obj_orderedByInfo_item = obj_orderedByInfo[i];
            const path_orderedByInfo_item = path_orderedByInfo + '[' + i + ']';
            const referencepath_orderedByInfo_itemValidationError = validate$2a(obj_orderedByInfo_item, path_orderedByInfo_item);
            if (referencepath_orderedByInfo_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListOrderByInfoRepresentation (at "' + path_orderedByInfo_item + '")\n';
                message += referencepath_orderedByInfo_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_scope = obj.scope;
        const path_scope = path + '.scope';
        let obj_scope_union0 = null;
        const obj_scope_union0_error = (() => {
            const referencepath_scopeValidationError = validate$27(obj_scope, path_scope);
            if (referencepath_scopeValidationError !== null) {
                let message = 'Object doesn\'t match ListScopeRepresentation (at "' + path_scope + '")\n';
                message += referencepath_scopeValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_scope_union0_error != null) {
            obj_scope_union0 = obj_scope_union0_error.message;
        }
        let obj_scope_union1 = null;
        const obj_scope_union1_error = (() => {
            if (obj_scope !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_scope + '" (at "' + path_scope + '")');
            }
        })();
        if (obj_scope_union1_error != null) {
            obj_scope_union1 = obj_scope_union1_error.message;
        }
        if (obj_scope_union0 && obj_scope_union1) {
            let message = 'Object doesn\'t match union (at "' + path_scope + '")';
            message += '\n' + obj_scope_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_scope_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_searchable = obj.searchable;
        const path_searchable = path + '.searchable';
        if (typeof obj_searchable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_searchable + '" (at "' + path_searchable + '")');
        }
        const obj_updateable = obj.updateable;
        const path_updateable = path + '.updateable';
        if (typeof obj_updateable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_updateable + '" (at "' + path_updateable + '")');
        }
        const obj_userPreferences = obj.userPreferences;
        const path_userPreferences = path + '.userPreferences';
        const referencepath_userPreferencesValidationError = validate$26(obj_userPreferences, path_userPreferences);
        if (referencepath_userPreferencesValidationError !== null) {
            let message = 'Object doesn\'t match ListUserPreferenceRepresentation (at "' + path_userPreferences + '")\n';
            message += referencepath_userPreferencesValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_visibility = obj.visibility;
        const path_visibility = path + '.visibility';
        if (typeof obj_visibility !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_visibility + '" (at "' + path_visibility + '")');
        }
        const obj_visibilityEditable = obj.visibilityEditable;
        const path_visibilityEditable = path + '.visibilityEditable';
        if (typeof obj_visibilityEditable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_visibilityEditable + '" (at "' + path_visibilityEditable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$11 = 'ListInfoRepresentation';
function keyBuilder$3F(luvio, config) {
    return keyPrefix + '::' + RepresentationType$11 + ':' + (config.listViewApiName === null ? '' : config.listViewApiName) + ':' + config.objectApiName + ':' + config.type;
}
function normalize$V(input, existing, path, luvio, store, timestamp) {
    const input_displayColumns = input.displayColumns;
    const input_displayColumns_id = path.fullPath + '__displayColumns';
    for (let i = 0; i < input_displayColumns.length; i++) {
        const input_displayColumns_item = input_displayColumns[i];
        let input_displayColumns_item_id = input_displayColumns_id + '__' + i;
        input_displayColumns[i] = ingest$2k(input_displayColumns_item, {
            fullPath: input_displayColumns_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$3t = function ListInfoRepresentationSelect() {
    const { selections: ListFilterByInfoRepresentation__selections, opaque: ListFilterByInfoRepresentation__opaque, } = select$3E();
    const { selections: ListInlineEditDetailsRepresentation__selections, opaque: ListInlineEditDetailsRepresentation__opaque, } = select$3D();
    const { selections: ListReferenceRepresentation__selections, opaque: ListReferenceRepresentation__opaque, } = select$3C();
    const { selections: ListInfoShareCategoryRepresentation__selections, opaque: ListInfoShareCategoryRepresentation__opaque, } = select$3A();
    const { selections: ListOrderByInfoRepresentation__selections, opaque: ListOrderByInfoRepresentation__opaque, } = select$3z();
    const { selections: ListScopeRepresentation__selections, opaque: ListScopeRepresentation__opaque, } = select$3w();
    const { selections: ListUserPreferenceRepresentation__selections, opaque: ListUserPreferenceRepresentation__opaque, } = select$3v();
    return {
        kind: 'Fragment',
        version: VERSION$2O,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'cloneable',
                kind: 'Scalar'
            },
            {
                name: 'createable',
                kind: 'Scalar'
            },
            {
                name: 'deletable',
                kind: 'Scalar'
            },
            {
                name: 'displayColumns',
                kind: 'Link',
                plural: true,
                fragment: select$3u()
            },
            {
                name: 'filterLogicString',
                kind: 'Scalar'
            },
            {
                name: 'filteredByInfo',
                kind: 'Object',
                plural: true,
                selections: ListFilterByInfoRepresentation__selections
            },
            {
                name: 'hasMassActions',
                kind: 'Scalar',
                required: false
            },
            {
                name: 'inlineEditDetails',
                kind: 'Object',
                selections: ListInlineEditDetailsRepresentation__selections
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'listReference',
                kind: 'Object',
                selections: ListReferenceRepresentation__selections
            },
            {
                name: 'listShares',
                kind: 'Object',
                plural: true,
                selections: ListInfoShareCategoryRepresentation__selections
            },
            {
                name: 'objectApiNames',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'orderedByInfo',
                kind: 'Object',
                plural: true,
                selections: ListOrderByInfoRepresentation__selections
            },
            {
                name: 'scope',
                kind: 'Object',
                nullable: true,
                selections: ListScopeRepresentation__selections
            },
            {
                name: 'searchable',
                kind: 'Scalar'
            },
            {
                name: 'updateable',
                kind: 'Scalar'
            },
            {
                name: 'userPreferences',
                kind: 'Object',
                selections: ListUserPreferenceRepresentation__selections
            },
            {
                name: 'visibility',
                kind: 'Scalar'
            },
            {
                name: 'visibilityEditable',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$1d(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$2j = function ListInfoRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$22(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$F(luvio, input);
    const ttlToUse = TTL$H;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$V, "UiApi", VERSION$2O, RepresentationType$11, equals$1d);
    return createLink$1(key);
};
function getTypeCacheKeys$2n(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$F(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$11,
        mergeable: false
    });
    const input_displayColumns_length = input.displayColumns.length;
    for (let i = 0; i < input_displayColumns_length; i++) {
        getTypeCacheKeys$2o(rootKeySet, luvio, input.displayColumns[i], () => '');
    }
}

const RECORD_ID_DECODER = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456';
/**
 * Converts to 18-char record ids. Details at http://sfdc.co/bnBMvm.
 * @param value A 15- or 18-char record id.
 * @returns An 18-char record id, and throws error if an invalid record id was provided.
 */
function getRecordId18(value) {
    if (isString(value)) {
        if (value.length === 18) {
            return value;
        }
        else if (value.length === 15) {
            // Add the 3 character suffix
            let recordId = value;
            for (let offset = 0; offset < 15; offset += 5) {
                let decodeValue = 0;
                for (let bit = 0; bit < 5; bit++) {
                    const c = value[offset + bit];
                    if (c >= 'A' && c <= 'Z') {
                        decodeValue += 1 << bit;
                    }
                }
                recordId += RECORD_ID_DECODER[decodeValue];
            }
            return recordId;
        }
    }
    else if (value === undefined || value === null) {
        // Not throwing here - input params might not have settled
        return undefined;
    }
    if (process.env.NODE_ENV !== 'production') {
        throw new Error(`Configuration Error: Invalid RecordId passed: ${JSON.stringify(value)}. RecordIds must be 15 or 18 character strings.`);
    }
    return undefined;
}

// Copy of SortedStringArray but calls toTitleCase
function toSortedStringArrayTitleCase(value) {
    const valueArray = isArray(value) ? value : [value];
    if (valueArray.length !== 0 && isArrayOfNonEmptyStrings(valueArray)) {
        return dedupe(valueArray)
            .sort()
            .map((value) => toTitleCase(value));
    }
    return undefined;
}

function toSortedStringArray(value) {
    const valueArray = isArray(value) ? value : [value];
    if (valueArray.length !== 0 && isArrayOfNonEmptyStrings(valueArray)) {
        return dedupe(valueArray).sort();
    }
    return undefined;
}

const VERSION$2N = "d1e589a127fb1060c89070cdb6f500b2";
function validate$21(obj, path = 'RecordTypeInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_available = obj.available;
        const path_available = path + '.available';
        if (typeof obj_available !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_available + '" (at "' + path_available + '")');
        }
        const obj_defaultRecordTypeMapping = obj.defaultRecordTypeMapping;
        const path_defaultRecordTypeMapping = path + '.defaultRecordTypeMapping';
        if (typeof obj_defaultRecordTypeMapping !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_defaultRecordTypeMapping + '" (at "' + path_defaultRecordTypeMapping + '")');
        }
        const obj_master = obj.master;
        const path_master = path + '.master';
        if (typeof obj_master !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_master + '" (at "' + path_master + '")');
        }
        const obj_name = obj.name;
        const path_name = path + '.name';
        if (typeof obj_name !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_name + '" (at "' + path_name + '")');
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        if (typeof obj_recordTypeId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$3s = function RecordTypeInfoRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2N,
        private: [],
        selections: [
            {
                name: 'available',
                kind: 'Scalar'
            },
            {
                name: 'defaultRecordTypeMapping',
                kind: 'Scalar'
            },
            {
                name: 'master',
                kind: 'Scalar'
            },
            {
                name: 'name',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$1c(existing, incoming) {
    const existing_available = existing.available;
    const incoming_available = incoming.available;
    if (!(existing_available === incoming_available)) {
        return false;
    }
    const existing_defaultRecordTypeMapping = existing.defaultRecordTypeMapping;
    const incoming_defaultRecordTypeMapping = incoming.defaultRecordTypeMapping;
    if (!(existing_defaultRecordTypeMapping === incoming_defaultRecordTypeMapping)) {
        return false;
    }
    const existing_master = existing.master;
    const incoming_master = incoming.master;
    if (!(existing_master === incoming_master)) {
        return false;
    }
    const existing_name = existing.name;
    const incoming_name = incoming.name;
    if (!(existing_name === incoming_name)) {
        return false;
    }
    const existing_recordTypeId = existing.recordTypeId;
    const incoming_recordTypeId = incoming.recordTypeId;
    if (!(existing_recordTypeId === incoming_recordTypeId)) {
        return false;
    }
    return true;
}

const VERSION$2M = "195d918987a35f45e1aa4dce9a11d8c5";
function validate$20(obj, path = 'FieldValueRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_displayValue = obj.displayValue;
        const path_displayValue = path + '.displayValue';
        let obj_displayValue_union0 = null;
        const obj_displayValue_union0_error = (() => {
            if (typeof obj_displayValue !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_displayValue + '" (at "' + path_displayValue + '")');
            }
        })();
        if (obj_displayValue_union0_error != null) {
            obj_displayValue_union0 = obj_displayValue_union0_error.message;
        }
        let obj_displayValue_union1 = null;
        const obj_displayValue_union1_error = (() => {
            if (obj_displayValue !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_displayValue + '" (at "' + path_displayValue + '")');
            }
        })();
        if (obj_displayValue_union1_error != null) {
            obj_displayValue_union1 = obj_displayValue_union1_error.message;
        }
        if (obj_displayValue_union0 && obj_displayValue_union1) {
            let message = 'Object doesn\'t match union (at "' + path_displayValue + '")';
            message += '\n' + obj_displayValue_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_displayValue_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_value = obj.value;
        const path_value = path + '.value';
        let obj_value_union0 = null;
        const obj_value_union0_error = (() => {
            if (typeof obj_value !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_value + '" (at "' + path_value + '")');
            }
        })();
        if (obj_value_union0_error != null) {
            obj_value_union0 = obj_value_union0_error.message;
        }
        let obj_value_union1 = null;
        const obj_value_union1_error = (() => {
            if (typeof obj_value !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_value + '" (at "' + path_value + '")');
            }
        })();
        if (obj_value_union1_error != null) {
            obj_value_union1 = obj_value_union1_error.message;
        }
        let obj_value_union2 = null;
        const obj_value_union2_error = (() => {
            if (typeof obj_value !== 'number') {
                return new TypeError('Expected "number" but received "' + typeof obj_value + '" (at "' + path_value + '")');
            }
        })();
        if (obj_value_union2_error != null) {
            obj_value_union2 = obj_value_union2_error.message;
        }
        let obj_value_union3 = null;
        const obj_value_union3_error = (() => {
            if (typeof obj_value !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_value + '" (at "' + path_value + '")');
            }
        })();
        if (obj_value_union3_error != null) {
            obj_value_union3 = obj_value_union3_error.message;
        }
        let obj_value_union4 = null;
        const obj_value_union4_error = (() => {
            if (obj_value !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_value + '" (at "' + path_value + '")');
            }
        })();
        if (obj_value_union4_error != null) {
            obj_value_union4 = obj_value_union4_error.message;
        }
        if (obj_value_union0 && obj_value_union1 && obj_value_union2 && obj_value_union3 && obj_value_union4) {
            let message = 'Object doesn\'t match union (at "' + path_value + '")';
            message += '\n' + obj_value_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_value_union1.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_value_union2.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_value_union3.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_value_union4.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$10 = 'FieldValueRepresentation';
function normalize$U(input, existing, path, luvio, store, timestamp) {
    const input_value = input.value;
    const input_value_id = path.fullPath + '__value';
    if (input_value !== null && typeof input_value === 'object') {
        input.value = ingest$2g(input_value, {
            fullPath: input_value_id,
            propertyName: 'value',
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$3r = function FieldValueRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2M,
        private: [],
        selections: [
            {
                name: 'displayValue',
                kind: 'Scalar'
            },
            {
                name: 'value',
                kind: 'Link',
                nullable: true,
                fragment: select$3p()
            }
        ]
    };
};
function equals$1b(existing, incoming) {
    const existing_displayValue = existing.displayValue;
    const incoming_displayValue = incoming.displayValue;
    if (!(existing_displayValue === incoming_displayValue)) {
        return false;
    }
    const existing_value = existing.value;
    const incoming_value = incoming.value;
    if (!(existing_value === incoming_value
        || (existing_value != null &&
            incoming_value != null &&
            existing_value.__ref != null &&
            incoming_value.__ref != null &&
            existing_value.__ref === incoming_value.__ref))) {
        return false;
    }
    return true;
}
const ingest$2i = function FieldValueRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$20(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$U, "UiApi", VERSION$2M, RepresentationType$10, equals$1b);
    return createLink$1(key);
};
function getTypeCacheKeys$2m(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$10,
        mergeable: false
    });
    if (input.value !== null && typeof input.value === 'object') {
        getTypeCacheKeys$2l(rootKeySet, luvio, input.value);
    }
}

// we override the generated so we can set "mergeable: true" on the root key
const getTypeCacheKeys$2l = (rootKeySet, luvio, input, _fullPathFactory) => {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$D(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$_,
        mergeable: true,
    });
    const input_childRelationships = input.childRelationships;
    const input_childRelationships_keys = keys(input_childRelationships);
    const input_childRelationships_length = input_childRelationships_keys.length;
    for (let i = 0; i < input_childRelationships_length; i++) {
        const key = input_childRelationships_keys[i];
        getTypeCacheKeys$2k(rootKeySet, luvio, input_childRelationships[key], () => rootKey + '__childRelationships' + '__' + key);
    }
    const input_fields = input.fields;
    const input_fields_keys = keys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        getTypeCacheKeys$2m(rootKeySet, luvio, input_fields[key], () => rootKey + '__fields__' + key);
    }
    return rootKeySet;
};

const TTL$G = 120000;
const VERSION$2L = "79cb5ac9f44542f683d00245fdfe500d";
function validate$1$(obj, path = 'RecordCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_count = obj.count;
        const path_count = path + '.count';
        if (typeof obj_count !== 'number' || (typeof obj_count === 'number' && Math.floor(obj_count) !== obj_count)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_count + '" (at "' + path_count + '")');
        }
        const obj_currentPageToken = obj.currentPageToken;
        const path_currentPageToken = path + '.currentPageToken';
        let obj_currentPageToken_union0 = null;
        const obj_currentPageToken_union0_error = (() => {
            if (typeof obj_currentPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
            }
        })();
        if (obj_currentPageToken_union0_error != null) {
            obj_currentPageToken_union0 = obj_currentPageToken_union0_error.message;
        }
        let obj_currentPageToken_union1 = null;
        const obj_currentPageToken_union1_error = (() => {
            if (obj_currentPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
            }
        })();
        if (obj_currentPageToken_union1_error != null) {
            obj_currentPageToken_union1 = obj_currentPageToken_union1_error.message;
        }
        if (obj_currentPageToken_union0 && obj_currentPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_currentPageToken + '")';
            message += '\n' + obj_currentPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_currentPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_currentPageUrl = obj.currentPageUrl;
        const path_currentPageUrl = path + '.currentPageUrl';
        if (typeof obj_currentPageUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
        }
        const obj_nextPageToken = obj.nextPageToken;
        const path_nextPageToken = path + '.nextPageToken';
        let obj_nextPageToken_union0 = null;
        const obj_nextPageToken_union0_error = (() => {
            if (typeof obj_nextPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union0_error != null) {
            obj_nextPageToken_union0 = obj_nextPageToken_union0_error.message;
        }
        let obj_nextPageToken_union1 = null;
        const obj_nextPageToken_union1_error = (() => {
            if (obj_nextPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union1_error != null) {
            obj_nextPageToken_union1 = obj_nextPageToken_union1_error.message;
        }
        if (obj_nextPageToken_union0 && obj_nextPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageToken + '")';
            message += '\n' + obj_nextPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageUrl = obj.nextPageUrl;
        const path_nextPageUrl = path + '.nextPageUrl';
        let obj_nextPageUrl_union0 = null;
        const obj_nextPageUrl_union0_error = (() => {
            if (typeof obj_nextPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union0_error != null) {
            obj_nextPageUrl_union0 = obj_nextPageUrl_union0_error.message;
        }
        let obj_nextPageUrl_union1 = null;
        const obj_nextPageUrl_union1_error = (() => {
            if (obj_nextPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union1_error != null) {
            obj_nextPageUrl_union1 = obj_nextPageUrl_union1_error.message;
        }
        if (obj_nextPageUrl_union0 && obj_nextPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageUrl + '")';
            message += '\n' + obj_nextPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_previousPageToken = obj.previousPageToken;
        const path_previousPageToken = path + '.previousPageToken';
        let obj_previousPageToken_union0 = null;
        const obj_previousPageToken_union0_error = (() => {
            if (typeof obj_previousPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union0_error != null) {
            obj_previousPageToken_union0 = obj_previousPageToken_union0_error.message;
        }
        let obj_previousPageToken_union1 = null;
        const obj_previousPageToken_union1_error = (() => {
            if (obj_previousPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union1_error != null) {
            obj_previousPageToken_union1 = obj_previousPageToken_union1_error.message;
        }
        if (obj_previousPageToken_union0 && obj_previousPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageToken + '")';
            message += '\n' + obj_previousPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_previousPageUrl = obj.previousPageUrl;
        const path_previousPageUrl = path + '.previousPageUrl';
        let obj_previousPageUrl_union0 = null;
        const obj_previousPageUrl_union0_error = (() => {
            if (typeof obj_previousPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union0_error != null) {
            obj_previousPageUrl_union0 = obj_previousPageUrl_union0_error.message;
        }
        let obj_previousPageUrl_union1 = null;
        const obj_previousPageUrl_union1_error = (() => {
            if (obj_previousPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union1_error != null) {
            obj_previousPageUrl_union1 = obj_previousPageUrl_union1_error.message;
        }
        if (obj_previousPageUrl_union0 && obj_previousPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageUrl + '")';
            message += '\n' + obj_previousPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (!ArrayIsArray(obj_records)) {
            return new TypeError('Expected "array" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
        for (let i = 0; i < obj_records.length; i++) {
            const obj_records_item = obj_records[i];
            const path_records_item = path_records + '[' + i + ']';
            if (typeof obj_records_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_records_item + '" (at "' + path_records_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$$ = 'RecordCollectionRepresentation';
function normalize$T(input, existing, path, luvio, store, timestamp) {
    const input_records = input.records;
    const input_records_id = path.fullPath + '__records';
    for (let i = 0; i < input_records.length; i++) {
        const input_records_item = input_records[i];
        let input_records_item_id = input_records_id + '__' + i;
        input_records[i] = ingest$2g(input_records_item, {
            fullPath: input_records_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$3q = function RecordCollectionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2L,
        private: [],
        selections: [
            {
                name: 'count',
                kind: 'Scalar'
            },
            {
                name: 'currentPageToken',
                kind: 'Scalar'
            },
            {
                name: 'currentPageUrl',
                kind: 'Scalar'
            },
            {
                name: 'nextPageToken',
                kind: 'Scalar'
            },
            {
                name: 'nextPageUrl',
                kind: 'Scalar'
            },
            {
                name: 'previousPageToken',
                kind: 'Scalar'
            },
            {
                name: 'previousPageUrl',
                kind: 'Scalar'
            },
            {
                name: 'records',
                kind: 'Link',
                plural: true,
                fragment: select$3p()
            }
        ]
    };
};
function equals$1a(existing, incoming) {
    const existing_count = existing.count;
    const incoming_count = incoming.count;
    if (!(existing_count === incoming_count)) {
        return false;
    }
    const existing_currentPageUrl = existing.currentPageUrl;
    const incoming_currentPageUrl = incoming.currentPageUrl;
    if (!(existing_currentPageUrl === incoming_currentPageUrl)) {
        return false;
    }
    const existing_currentPageToken = existing.currentPageToken;
    const incoming_currentPageToken = incoming.currentPageToken;
    if (!(existing_currentPageToken === incoming_currentPageToken)) {
        return false;
    }
    const existing_nextPageToken = existing.nextPageToken;
    const incoming_nextPageToken = incoming.nextPageToken;
    if (!(existing_nextPageToken === incoming_nextPageToken)) {
        return false;
    }
    const existing_nextPageUrl = existing.nextPageUrl;
    const incoming_nextPageUrl = incoming.nextPageUrl;
    if (!(existing_nextPageUrl === incoming_nextPageUrl)) {
        return false;
    }
    const existing_previousPageToken = existing.previousPageToken;
    const incoming_previousPageToken = incoming.previousPageToken;
    if (!(existing_previousPageToken === incoming_previousPageToken)) {
        return false;
    }
    const existing_previousPageUrl = existing.previousPageUrl;
    const incoming_previousPageUrl = incoming.previousPageUrl;
    if (!(existing_previousPageUrl === incoming_previousPageUrl)) {
        return false;
    }
    const existing_records = existing.records;
    const incoming_records = incoming.records;
    const equals_records_items = equalsArray(existing_records, incoming_records, (existing_records_item, incoming_records_item) => {
        if (!(existing_records_item.__ref === incoming_records_item.__ref)) {
            return false;
        }
    });
    if (equals_records_items === false) {
        return false;
    }
    return true;
}
const ingest$2h = function RecordCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1$(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$G;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$T, "UiApi", VERSION$2L, RepresentationType$$, equals$1a);
    return createLink$1(key);
};
function getTypeCacheKeys$2k(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$$,
        mergeable: false
    });
    const input_records_length = input.records.length;
    for (let i = 0; i < input_records_length; i++) {
        getTypeCacheKeys$2l(rootKeySet, luvio, input.records[i]);
    }
}

const VIEW_ENTITY_API_NAME = 'Name';
const VIEW_ENTITY_KEY_PREFIX = `${keyPrefix}::RecordViewEntityRepresentation:${VIEW_ENTITY_API_NAME}:`;
const keyBuilderFromType$E = function RecordRepresentationKeyBuilderFromType(luvio, object) {
    const { apiName, id } = object;
    if (apiName === VIEW_ENTITY_API_NAME) {
        return VIEW_ENTITY_KEY_PREFIX + id;
    }
    return keyBuilderFromType$D(luvio, object);
};

const TTL$F = 30000;
const VERSION$2K = "98c5b18512e48ca8d28727549507e4ba";
function validate$1_(obj, path = 'RecordRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_childRelationships = obj.childRelationships;
        const path_childRelationships = path + '.childRelationships';
        if (typeof obj_childRelationships !== 'object' || ArrayIsArray(obj_childRelationships) || obj_childRelationships === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_childRelationships + '" (at "' + path_childRelationships + '")');
        }
        const obj_childRelationships_keys = ObjectKeys(obj_childRelationships);
        for (let i = 0; i < obj_childRelationships_keys.length; i++) {
            const key = obj_childRelationships_keys[i];
            const obj_childRelationships_prop = obj_childRelationships[key];
            const path_childRelationships_prop = path_childRelationships + '["' + key + '"]';
            if (typeof obj_childRelationships_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_childRelationships_prop + '" (at "' + path_childRelationships_prop + '")');
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            if (typeof obj_fields_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
            }
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_lastModifiedById = obj.lastModifiedById;
        const path_lastModifiedById = path + '.lastModifiedById';
        let obj_lastModifiedById_union0 = null;
        const obj_lastModifiedById_union0_error = (() => {
            if (typeof obj_lastModifiedById !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lastModifiedById + '" (at "' + path_lastModifiedById + '")');
            }
        })();
        if (obj_lastModifiedById_union0_error != null) {
            obj_lastModifiedById_union0 = obj_lastModifiedById_union0_error.message;
        }
        let obj_lastModifiedById_union1 = null;
        const obj_lastModifiedById_union1_error = (() => {
            if (obj_lastModifiedById !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lastModifiedById + '" (at "' + path_lastModifiedById + '")');
            }
        })();
        if (obj_lastModifiedById_union1_error != null) {
            obj_lastModifiedById_union1 = obj_lastModifiedById_union1_error.message;
        }
        if (obj_lastModifiedById_union0 && obj_lastModifiedById_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lastModifiedById + '")';
            message += '\n' + obj_lastModifiedById_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lastModifiedById_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_lastModifiedDate = obj.lastModifiedDate;
        const path_lastModifiedDate = path + '.lastModifiedDate';
        let obj_lastModifiedDate_union0 = null;
        const obj_lastModifiedDate_union0_error = (() => {
            if (typeof obj_lastModifiedDate !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lastModifiedDate + '" (at "' + path_lastModifiedDate + '")');
            }
        })();
        if (obj_lastModifiedDate_union0_error != null) {
            obj_lastModifiedDate_union0 = obj_lastModifiedDate_union0_error.message;
        }
        let obj_lastModifiedDate_union1 = null;
        const obj_lastModifiedDate_union1_error = (() => {
            if (obj_lastModifiedDate !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lastModifiedDate + '" (at "' + path_lastModifiedDate + '")');
            }
        })();
        if (obj_lastModifiedDate_union1_error != null) {
            obj_lastModifiedDate_union1 = obj_lastModifiedDate_union1_error.message;
        }
        if (obj_lastModifiedDate_union0 && obj_lastModifiedDate_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lastModifiedDate + '")';
            message += '\n' + obj_lastModifiedDate_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lastModifiedDate_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        let obj_recordTypeId_union0 = null;
        const obj_recordTypeId_union0_error = (() => {
            if (typeof obj_recordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union0_error != null) {
            obj_recordTypeId_union0 = obj_recordTypeId_union0_error.message;
        }
        let obj_recordTypeId_union1 = null;
        const obj_recordTypeId_union1_error = (() => {
            if (obj_recordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union1_error != null) {
            obj_recordTypeId_union1 = obj_recordTypeId_union1_error.message;
        }
        if (obj_recordTypeId_union0 && obj_recordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeId + '")';
            message += '\n' + obj_recordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recordTypeInfo = obj.recordTypeInfo;
        const path_recordTypeInfo = path + '.recordTypeInfo';
        let obj_recordTypeInfo_union0 = null;
        const obj_recordTypeInfo_union0_error = (() => {
            const referencepath_recordTypeInfoValidationError = validate$21(obj_recordTypeInfo, path_recordTypeInfo);
            if (referencepath_recordTypeInfoValidationError !== null) {
                let message = 'Object doesn\'t match RecordTypeInfoRepresentation (at "' + path_recordTypeInfo + '")\n';
                message += referencepath_recordTypeInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_recordTypeInfo_union0_error != null) {
            obj_recordTypeInfo_union0 = obj_recordTypeInfo_union0_error.message;
        }
        let obj_recordTypeInfo_union1 = null;
        const obj_recordTypeInfo_union1_error = (() => {
            if (obj_recordTypeInfo !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeInfo + '" (at "' + path_recordTypeInfo + '")');
            }
        })();
        if (obj_recordTypeInfo_union1_error != null) {
            obj_recordTypeInfo_union1 = obj_recordTypeInfo_union1_error.message;
        }
        if (obj_recordTypeInfo_union0 && obj_recordTypeInfo_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeInfo + '")';
            message += '\n' + obj_recordTypeInfo_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeInfo_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_systemModstamp = obj.systemModstamp;
        const path_systemModstamp = path + '.systemModstamp';
        let obj_systemModstamp_union0 = null;
        const obj_systemModstamp_union0_error = (() => {
            if (typeof obj_systemModstamp !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_systemModstamp + '" (at "' + path_systemModstamp + '")');
            }
        })();
        if (obj_systemModstamp_union0_error != null) {
            obj_systemModstamp_union0 = obj_systemModstamp_union0_error.message;
        }
        let obj_systemModstamp_union1 = null;
        const obj_systemModstamp_union1_error = (() => {
            if (obj_systemModstamp !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_systemModstamp + '" (at "' + path_systemModstamp + '")');
            }
        })();
        if (obj_systemModstamp_union1_error != null) {
            obj_systemModstamp_union1 = obj_systemModstamp_union1_error.message;
        }
        if (obj_systemModstamp_union0 && obj_systemModstamp_union1) {
            let message = 'Object doesn\'t match union (at "' + path_systemModstamp + '")';
            message += '\n' + obj_systemModstamp_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_systemModstamp_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_weakEtag = obj.weakEtag;
        const path_weakEtag = path + '.weakEtag';
        if (typeof obj_weakEtag !== 'number' || (typeof obj_weakEtag === 'number' && Math.floor(obj_weakEtag) !== obj_weakEtag)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_weakEtag + '" (at "' + path_weakEtag + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$_ = 'RecordRepresentation';
function keyBuilder$3E(luvio, config) {
    return keyPrefix + '::' + RepresentationType$_ + ':' + config.recordId;
}
function keyBuilderFromType$D(luvio, object) {
    const keyParams = {
        recordId: object.id
    };
    return keyBuilder$3E(luvio, keyParams);
}
function dynamicNormalize$5(ingestParams) {
    return function normalize_dynamic(input, existing, path, luvio, store, timestamp) {
        const input_childRelationships = input.childRelationships;
        const input_childRelationships_id = path.fullPath + '__childRelationships';
        const input_childRelationships_keys = Object.keys(input_childRelationships);
        const input_childRelationships_length = input_childRelationships_keys.length;
        for (let i = 0; i < input_childRelationships_length; i++) {
            const key = input_childRelationships_keys[i];
            const input_childRelationships_prop = input_childRelationships[key];
            const input_childRelationships_prop_id = input_childRelationships_id + '__' + key;
            input_childRelationships[key] = ingestParams.childRelationships(input_childRelationships_prop, {
                fullPath: input_childRelationships_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        const input_fields = input.fields;
        const input_fields_id = path.fullPath + '__fields';
        const input_fields_keys = Object.keys(input_fields);
        const input_fields_length = input_fields_keys.length;
        for (let i = 0; i < input_fields_length; i++) {
            const key = input_fields_keys[i];
            const input_fields_prop = input_fields[key];
            const input_fields_prop_id = input_fields_id + '__' + key;
            input_fields[key] = ingestParams.fields(input_fields_prop, {
                fullPath: input_fields_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        return input;
    };
}
const select$3p = function RecordRepresentationSelect() {
    const { selections: RecordTypeInfoRepresentation__selections, opaque: RecordTypeInfoRepresentation__opaque, } = select$3s();
    return {
        kind: 'Fragment',
        version: VERSION$2K,
        private: [
            'eTag',
            'weakEtag'
        ],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'childRelationships',
                kind: 'Link',
                map: true,
                fragment: select$3q()
            },
            {
                name: 'fields',
                kind: 'Link',
                map: true,
                fragment: select$3r()
            },
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'lastModifiedById',
                kind: 'Scalar'
            },
            {
                name: 'lastModifiedDate',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeInfo',
                kind: 'Object',
                nullable: true,
                selections: RecordTypeInfoRepresentation__selections
            },
            {
                name: 'systemModstamp',
                kind: 'Scalar'
            }
        ]
    };
};
const dynamicSelect$8 = function dynamicRecordRepresentationSelect(params) {
    const childRelationshipsPathSelection = params.childRelationships === undefined ? {
        name: 'childRelationships',
        kind: 'Link',
        map: true,
        fragment: select$3q()
    } : params.childRelationships;
    const fieldsPathSelection = params.fields === undefined ? {
        name: 'fields',
        kind: 'Link',
        map: true,
        fragment: select$3r()
    } : params.fields;
    const { selections: RecordTypeInfoRepresentation__selections, opaque: RecordTypeInfoRepresentation__opaque, } = select$3s();
    return {
        kind: 'Fragment',
        version: VERSION$2K,
        private: [
            'eTag',
            'weakEtag'
        ],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            childRelationshipsPathSelection,
            fieldsPathSelection,
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'lastModifiedById',
                kind: 'Scalar'
            },
            {
                name: 'lastModifiedDate',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeInfo',
                kind: 'Object',
                nullable: true,
                selections: RecordTypeInfoRepresentation__selections
            },
            {
                name: 'systemModstamp',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$19(existing, incoming) {
    const existing_weakEtag = existing.weakEtag;
    const incoming_weakEtag = incoming.weakEtag;
    if (!(existing_weakEtag === incoming_weakEtag)) {
        return false;
    }
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_childRelationships = existing.childRelationships;
    const incoming_childRelationships = incoming.childRelationships;
    const equals_childRelationships_props = equalsObject(existing_childRelationships, incoming_childRelationships, (existing_childRelationships_prop, incoming_childRelationships_prop) => {
        if (!(existing_childRelationships_prop.__ref === incoming_childRelationships_prop.__ref)) {
            return false;
        }
    });
    if (equals_childRelationships_props === false) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_props = equalsObject(existing_fields, incoming_fields, (existing_fields_prop, incoming_fields_prop) => {
        if (!(existing_fields_prop.__ref === incoming_fields_prop.__ref)) {
            return false;
        }
    });
    if (equals_fields_props === false) {
        return false;
    }
    const existing_lastModifiedById = existing.lastModifiedById;
    const incoming_lastModifiedById = incoming.lastModifiedById;
    if (!(existing_lastModifiedById === incoming_lastModifiedById)) {
        return false;
    }
    const existing_lastModifiedDate = existing.lastModifiedDate;
    const incoming_lastModifiedDate = incoming.lastModifiedDate;
    if (!(existing_lastModifiedDate === incoming_lastModifiedDate)) {
        return false;
    }
    const existing_recordTypeId = existing.recordTypeId;
    const incoming_recordTypeId = incoming.recordTypeId;
    if (!(existing_recordTypeId === incoming_recordTypeId)) {
        return false;
    }
    const existing_recordTypeInfo = existing.recordTypeInfo;
    const incoming_recordTypeInfo = incoming.recordTypeInfo;
    if (!(existing_recordTypeInfo === incoming_recordTypeInfo
        || (existing_recordTypeInfo != null &&
            incoming_recordTypeInfo != null &&
            equals$1c(existing_recordTypeInfo, incoming_recordTypeInfo)))) {
        return false;
    }
    const existing_systemModstamp = existing.systemModstamp;
    const incoming_systemModstamp = incoming.systemModstamp;
    if (!(existing_systemModstamp === incoming_systemModstamp)) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$2j(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$E(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$_,
        mergeable: false
    });
    const input_childRelationships = input.childRelationships;
    const input_childRelationships_keys = ObjectKeys(input_childRelationships);
    const input_childRelationships_length = input_childRelationships_keys.length;
    for (let i = 0; i < input_childRelationships_length; i++) {
        const key = input_childRelationships_keys[i];
        getTypeCacheKeys$2k(rootKeySet, luvio, input_childRelationships[key], () => rootKey + "__childRelationships" + "__" + key);
    }
    const input_fields = input.fields;
    const input_fields_keys = ObjectKeys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        getTypeCacheKeys$2m(rootKeySet, luvio, input_fields[key], () => rootKey + "__fields" + "__" + key);
    }
}
const notifyUpdateAvailableFactory$5 = (luvio) => {
    return function notifyRecordUpdateAvailable(configs) {
        if (process.env.NODE_ENV !== 'production') {
            const requiredKeyParams = ['recordId'];
            configs.forEach(config => {
                if (false === requiredKeyParams.every(req => req in config)) {
                    throw new Error(`one of the configs did not contain all required parameters: ${JSONStringify(ObjectKeys(config))}`);
                }
            });
        }
        const keys = configs.map(c => keyBuilder$3E(luvio, c));
        return luvio.notifyStoreUpdateAvailable(keys);
    };
};

/**
 * The master record type id.
 */
const MAIN_RECORD_TYPE_ID = '012000000000000AAA';

/**
 * A set of the string names of known entities.
 *
 * Generated
 *     from: ui-services-private-object-allow-list.yaml
 *     API version: 63
 *     at: Wed, 21 Aug 2024 19:39:55 GMT
 */
const UIAPI_SUPPORTED_ENTITY_API_NAMES = new Set([
    'AIPredictionScore',
    'AISentimentResult',
    'AcademicSession',
    'AcademicTerm',
    'AcademicTermEnrollment',
    'AcademicYear',
    'Account',
    'AccountAccountRelation',
    'AccountBrand',
    'AccountContactRelation',
    'AccountForecast',
    'AccountForecastAdjustment',
    'AccountForecastPeriodMetric',
    'AccountParticipant',
    'AccountPartner',
    'AccountProductForecast',
    'AccountProductPeriodForecast',
    'AccountRelationship',
    'AccountTeamMember',
    'Accreditation',
    'AcctMgrPeriodicTargetDstr',
    'AcctMgrTarget',
    'AcctMgrTargetDstr',
    'ActionCadence',
    'ActionCadenceStep',
    'ActionCadenceStepTracker',
    'ActionCadenceTracker',
    'ActionPlan',
    'ActionPlanBaseTemplateAsgn',
    'ActionPlanItem',
    'ActionPlanItemDependency',
    'ActionPlanTemplate',
    'ActionPlanTemplateAssignment',
    'ActionPlanTemplateItem',
    'ActionPlanTemplateItemValue',
    'ActionPlanTemplateVersion',
    'ActionPlanTmplItmAssessmentInd',
    'ActionPlnTmplItmDependency',
    'ActionVisibility',
    'ActionableList',
    'ActionableListColumn',
    'ActionableListMember',
    'ActionableListMemberStatus',
    'ActivationTarget',
    'ActiveScratchOrg',
    'ActivityMetric',
    'ActivityMetricRollup',
    'ActivityTiming',
    'AdBuyServerAccount',
    'AdCreativeSizeType',
    'AdOpportunity',
    'AdOrderItem',
    'AdOrderItemCreativeSizeType',
    'AdOrderItemPrintIssue',
    'AdOrderItemUnitsSplit',
    'AdOrderLineAdTarget',
    'AdPageLayoutType',
    'AdProductDefaultTargetCategory',
    'AdProductTargetCategory',
    'AdQuote',
    'AdQuoteLine',
    'AdQuoteLineAdTarget',
    'AdQuoteLineCreativeSizeType',
    'AdQuoteLinePrintIssue',
    'AdQuoteLineUnitsSplit',
    'AdServer',
    'AdServerAccount',
    'AdServerUser',
    'AdSpaceCreativeSizeType',
    'AdSpaceGroupMember',
    'AdSpaceSpecification',
    'AdSpecMediaPrintIssue',
    'AdTargetCategory',
    'AdTargetCategorySegment',
    'AdUsage',
    'AdUsageSummary',
    'AddOnDefinition',
    'Address',
    'AdvAccountForecastFact',
    'AdvAcctForecastSetPartner',
    'AdvAcctForecastSetUse',
    'AiDataset',
    'AiImageDetectedObject',
    'AiImageTrainingObject',
    'AiVisionModel',
    'AiVisionModelMetric',
    'AiVisionModelObjectMetric',
    'AiVisitRecommendRequest',
    'AiVisitRecommendation',
    'AiVisitSummary',
    'AiVisitTaskRcmd',
    'AiVisitTaskRcmdRequest',
    'AirTravelEmssnFctr',
    'AirTravelEnrgyUse',
    'AllergyIntolerance',
    'AlternativePaymentMethod',
    'AnnualEmssnInventory',
    'AnnualEmssnRdctnTarget',
    'AntiCorruptionInitSum',
    'ApiAnomalyEventStore',
    'AppAnalyticsDeliveryConfig',
    'AppAnalyticsQueryRequest',
    'AppExtension',
    'AppMenuItem',
    'AppUsageAssignment',
    'ApplicationCase',
    'ApplicationDecision',
    'ApplicationFormTemplate',
    'ApplicationRecommendation',
    'ApplicationRecommender',
    'ApplicationReview',
    'ApplicationTimeline',
    'AppointmentInvitation',
    'AppointmentInvitee',
    'AppointmentReason',
    'AppointmentTopicTimeSlot',
    'ApptBundleAggrDurDnscale',
    'ApptBundleAggrPolicy',
    'ApptBundleConfig',
    'ApptBundlePolicy',
    'ApptBundlePolicySvcTerr',
    'ApptBundlePropagatePolicy',
    'ApptBundleRestrictPolicy',
    'ApptBundleSortPolicy',
    'ApptReasonEngmtChannelType',
    'AsmtQstnRespRecommendation',
    'Assessment',
    'AssessmentEnvelope',
    'AssessmentEnvelopeItem',
    'AssessmentIndDefinedValue',
    'AssessmentIndValue',
    'AssessmentIndicatorDefinition',
    'AssessmentQuestion',
    'AssessmentQuestionAssignment',
    'AssessmentQuestionResponse',
    'AssessmentQuestionSet',
    'AssessmentQuestionVersion',
    'AssessmentSignature',
    'AssessmentTask',
    'AssessmentTaskContentDocument',
    'AssessmentTaskDefinition',
    'AssessmentTaskIndDefinition',
    'AssessmentTaskOrder',
    'Asset',
    'AssetAccountParticipant',
    'AssetAction',
    'AssetActionSource',
    'AssetAttribute',
    'AssetContactParticipant',
    'AssetDowntimePeriod',
    'AssetMilestone',
    'AssetRelationship',
    'AssetStatePeriod',
    'AssetWarranty',
    'AssignedResource',
    'AssistantText',
    'AssociatedLocation',
    'Assortment',
    'AssortmentProduct',
    'AsyncOperationLog',
    'AttachedContentNote',
    'AttributeCategory',
    'AttributeDefinition',
    'AttributePicklist',
    'AttributePicklistValue',
    'AuthApplicationAsset',
    'AuthApplicationPlace',
    'AuthFormRequestRecord',
    'AuthLocationAccessSchedule',
    'AuthorNote',
    'AuthorizationForm',
    'AuthorizationFormConsent',
    'AuthorizationFormDataUse',
    'AuthorizationFormText',
    'AuthorizedInsuranceLine',
    'AutomatedAction',
    'AutomatedActionCondition',
    'AutomatedActionParameter',
    'Award',
    'BCEntityPermission',
    'BCEntityPermissionSet',
    'BCFieldPermission',
    'BCParticipant',
    'BCParticipantAccess',
    'BCPermissionSet',
    'BCRecordAccess',
    'BCRecordAccessApproval',
    'BCRelatedParticipant',
    'BackgroundOperation',
    'Banker',
    'BatchJob',
    'BatchJobPartFailedRecord',
    'Benefit',
    'BenefitAssignment',
    'BenefitAssignmentAdjustment',
    'BenefitDisbursement',
    'BenefitDisbursementAdj',
    'BenefitParameterValue',
    'BenefitSchedule',
    'BenefitSession',
    'BenefitType',
    'BillingBatchFilterCriteria',
    'BillingBatchScheduler',
    'BillingPeriodItem',
    'BillingPolicy',
    'BillingSchedule',
    'BillingScheduleCreationStatus',
    'BillingScheduleGroup',
    'BillingTreatment',
    'BillingTreatmentItem',
    'BldgEnrgyIntensity',
    'BldgEnrgyIntensityVal',
    'BldgSizeCategory',
    'BlockchainAppMember',
    'BlockchainApplication',
    'BlockchainEntity',
    'BlockchainField',
    'BlockchainMember',
    'BoardCertification',
    'BotDefinition',
    'BranchUnit',
    'BranchUnitBusinessMember',
    'BranchUnitCustomer',
    'BranchUnitRelatedRecord',
    'BroadcastCommAudience',
    'BroadcastCommunication',
    'BroadcastTopic',
    'Budget',
    'BudgetAllocation',
    'BudgetCategory',
    'BudgetCategoryValue',
    'BudgetPeriod',
    'BundledCareFeeAgreement',
    'BusRegAuthTypeDependency',
    'BusRegAuthorizationType',
    'BusinessAlert',
    'BusinessAlertStatus',
    'BusinessBrand',
    'BusinessHours',
    'BusinessLicense',
    'BusinessLicenseApplication',
    'BusinessMilestone',
    'BusinessProfile',
    'BusinessType',
    'BuyerAccount',
    'BuyerGroup',
    'BuyerGroupPricebook',
    'CalcProcStepRelationship',
    'CalculationMatrix',
    'CalculationMatrixColumn',
    'CalculationMatrixRow',
    'CalculationMatrixVersion',
    'CalculationProcedure',
    'CalculationProcedureStep',
    'CalculationProcedureVariable',
    'CalculationProcedureVersion',
    'Campaign',
    'CampaignInfluence',
    'CampaignInsight',
    'CampaignMember',
    'CampaignMemberStatus',
    'CapitationCareFeeAgreement',
    'CardPaymentMethod',
    'CareBarrier',
    'CareBarrierDeterminant',
    'CareBarrierType',
    'CareBenefitVerifyRequest',
    'CareDeterminant',
    'CareDeterminantType',
    'CareDiagnosis',
    'CareEpisode',
    'CareEpisodeDetail',
    'CareFeeScheduleItem',
    'CareInterventionType',
    'CareLimitType',
    'CareMetricTarget',
    'CareObservation',
    'CareObservationComponent',
    'CarePerformer',
    'CarePgmEnrolleeWkOrdStep',
    'CarePgmEnrolleeWorkOrder',
    'CarePgmProvHealthcareProvider',
    'CarePlan',
    'CarePlanActivity',
    'CarePlanActivityDetail',
    'CarePlanDetail',
    'CarePlanTemplate',
    'CarePlanTemplateBenefit',
    'CarePlanTemplateGoal',
    'CarePlanTemplateProblem',
    'CarePractnFacilityAppt',
    'CarePreauth',
    'CarePreauthItem',
    'CareProgram',
    'CareProgramCampaign',
    'CareProgramEligibilityRule',
    'CareProgramEnrollee',
    'CareProgramEnrolleeProduct',
    'CareProgramEnrollmentCard',
    'CareProgramGoal',
    'CareProgramProduct',
    'CareProgramProvider',
    'CareProgramTeamMember',
    'CareProviderAdverseAction',
    'CareProviderFacilitySpecialty',
    'CareProviderSearchableField',
    'CareRegisteredDevice',
    'CareRequest',
    'CareRequestDrug',
    'CareRequestExtension',
    'CareRequestItem',
    'CareRequestReviewer',
    'CareSpecialty',
    'CareSpecialtyTaxonomy',
    'CareTaxonomy',
    'CartDeliveryGroup',
    'CartItem',
    'CartTax',
    'Case',
    'CaseArticle',
    'CaseComment',
    'CaseContactRole',
    'CaseMilestone',
    'CaseParticipant',
    'CaseProgram',
    'CaseRelatedIssue',
    'CategorizedCareFeeAgreement',
    'ChangeRequest',
    'ChangeRequestRelatedIssue',
    'ChangeRequestRelatedItem',
    'ChannelObjectLinkingRule',
    'ChannelProgram',
    'ChannelProgramLevel',
    'ChannelProgramMember',
    'Claim',
    'ClaimCase',
    'ClaimCovPaymentAdjustment',
    'ClaimCovReserveAdjustment',
    'ClaimCoverage',
    'ClaimCoveragePaymentDetail',
    'ClaimCoverageReserveDetail',
    'ClaimItem',
    'ClaimParticipant',
    'ClaimPaymentSummary',
    'ClaimPayoutPlan',
    'ClaimRecovery',
    'ClaimTeamMember',
    'ClauseCatgConfiguration',
    'ClinicalAlert',
    'ClinicalDetectedIssue',
    'ClinicalDetectedIssueDetail',
    'ClinicalEncounter',
    'ClinicalEncounterDiagnosis',
    'ClinicalEncounterFacility',
    'ClinicalEncounterIdentifier',
    'ClinicalEncounterProvider',
    'ClinicalEncounterReason',
    'ClinicalEncounterSvcRequest',
    'ClinicalServiceRequest',
    'ClinicalServiceRequestDetail',
    'CloudServiceProvider',
    'CodeSet',
    'CodeSetBundle',
    'CodeSetRelationship',
    'CollaborationRoom',
    'CommSubscription',
    'CommSubscriptionChannelType',
    'CommSubscriptionConsent',
    'CommSubscriptionTiming',
    'CommerceEntitlementPolicy',
    'CommissionSchedule',
    'CommissionScheduleAssignment',
    'CommsRegionDiscount',
    'CommsVolumeDiscount',
    'ComplaintCase',
    'ComplaintParticipant',
    'ConsentMetric',
    'ConsentUsageMetric',
    'ConstituentRole',
    'ConsumptionRate',
    'ConsumptionSchedule',
    'Contact',
    'ContactContactRelation',
    'ContactEncounter',
    'ContactEncounterParticipant',
    'ContactPointAddress',
    'ContactPointConsent',
    'ContactPointEmail',
    'ContactPointPhone',
    'ContactPointTypeConsent',
    'ContactProfile',
    'ContactRequest',
    'ContentDocument',
    'ContentDocumentLink',
    'ContentDocumentListViewMapping',
    'ContentFolderItem',
    'ContentFolderLink',
    'ContentNote',
    'ContentVersion',
    'ContentWorkspace',
    'Contract',
    'ContractContactRole',
    'ContractDocVerContentDoc',
    'ContractDocVersionSection',
    'ContractDocumentVersion',
    'ContractGroupPlan',
    'ContractGroupPlanGroupClass',
    'ContractLineItem',
    'ContractLineOutcome',
    'ContractLineOutcomeData',
    'ContractPaymentAgreement',
    'ContractType',
    'ContractTypeConfig',
    'ConversationBroadcast',
    'ConversationBroadcastEntry',
    'Coupon',
    'CourseOffering',
    'CourseOfferingParticipant',
    'CourseOfferingPtcpResult',
    'CourseOfferingSchedule',
    'CourseOfferingScheduleTmpl',
    'CoverageBenefit',
    'CoverageBenefitItem',
    'CoverageBenefitItemLimit',
    'CoverageLimit',
    'CoverageType',
    'CrbnCreditAlloc',
    'CrbnCreditAllocItem',
    'CrbnCreditDistribution',
    'CrbnCreditProject',
    'CrbnEmssnScopeAlloc',
    'CrbnEmssnScopeAllocVal',
    'CredentialStuffingEventStore',
    'CreditMemo',
    'CreditMemoAddressGroup',
    'CreditMemoInvApplication',
    'CreditMemoLine',
    'Crisis',
    'CryptoProdCatgWalletGroup',
    'CryptoProdtCatgWalletRole',
    'CryptoProduct',
    'CryptoTransEnvelopeChgLog',
    'CryptoTransaction',
    'CryptoTransactionEnvelope',
    'CryptoWallet',
    'CryptoWalletGroup',
    'CryptoWalletGroupItem',
    'CspTrustedSite',
    'CustomObjectParticipant',
    'Customer',
    'CustomerProperty',
    'DandBCompany',
    'DataAction',
    'DataActionTarget',
    'DataConnectorS3',
    'DataLakeObjectInstance',
    'DataStream',
    'DataUseLegalBasis',
    'DataUsePurpose',
    'DelegatedAccount',
    'DeleteEvent',
    'DeliveryTask',
    'DiagnosticSummary',
    'DiagnosticSummaryDetail',
    'DigitalSignature',
    'DigitalWallet',
    'Disclosure',
    'DisclosureDefinition',
    'DisclosureDefinitionVersion',
    'DisclosureType',
    'DisclsReportingPeriod',
    'DistributorAuthorization',
    'DivrsEquityInclSum',
    'DocTemplateSectionCondition',
    'DocumentChecklistItem',
    'DocumentClause',
    'DocumentClauseSet',
    'DocumentEnvelope',
    'DocumentGenerationProcess',
    'DocumentGenerationSetting',
    'DocumentRecipient',
    'DocumentTemplate',
    'DocumentTemplateContentDoc',
    'DocumentTemplateSection',
    'DocumentTemplateToken',
    'DrugDisplayTerm',
    'DsarPolicyLog',
    'DuplicateRecordItem',
    'DuplicateRecordSet',
    'ECart',
    'EconomicPerformanceSum',
    'EditionDefinition',
    'EducationalInfoRequest',
    'ElectricityEmssnFctrSet',
    'ElectronicMediaGroup',
    'EmailContent',
    'EmailMessage',
    'EmailMessageRelation',
    'EmissionsActivity',
    'EmissionsForecastFact',
    'EmpBenefitSummary',
    'EmpUserProvisionProcessErr',
    'EmpUserProvisioningProcess',
    'Employee',
    'EmployeeAsset',
    'EmployeeContact',
    'EmployeeCrisisAssessment',
    'EmployeeDemographicSum',
    'EmployeeDevelopmentSum',
    'EmployeeJob',
    'EmployeeJobPosition',
    'EmployeeOrganization',
    'EmssnRdctnCommitment',
    'EmssnReductionTarget',
    'EnablementProgram',
    'EnblPgmTaskMeasureProgress',
    'EnblProgramSection',
    'EnblProgramTaskDefinition',
    'EnblProgramTaskMeasure',
    'EnblProgramTaskProgress',
    'EngagementAttendee',
    'EngagementChannelType',
    'EngagementChannelWorkType',
    'EngagementInteraction',
    'EngagementProgram',
    'EngagementProgramVersion',
    'EngagementTopic',
    'EnhancedEmailTemplate',
    'EnhancedLetterhead',
    'EnrollmentEligibilityCriteria',
    'Entitlement',
    'EntitlementContact',
    'EntityArchivingException',
    'EntityArchivingJob',
    'EntityArchivingSetup',
    'EntityDefinition',
    'EntityMilestone',
    'EnvironmentHubMember',
    'EnvironmentalRisk',
    'Event',
    'Examination',
    'Expense',
    'ExpenseReport',
    'ExpenseReportEntry',
    'ExplainabilityMsgTemplate',
    'ExpressionSet',
    'ExpressionSetVersion',
    'ExtDataShare',
    'ExtDataShareTarget',
    'ExternalAccountHierarchy',
    'FTestVirtualBpoLookup',
    'FeeScheduleDefinition',
    'FeedItem',
    'FieldServiceMobileExtension',
    'FieldServiceMobileSettings',
    'FieldServiceOrgSettings',
    'FinanceBalanceSnapshot',
    'FinanceBook',
    'FinancePeriod',
    'FinanceTransaction',
    'FinanceTransactionErrorLog',
    'FinancialDeal',
    'FinancialDealAsset',
    'FinancialDealBid',
    'FinancialDealInteraction',
    'FinancialDealParticipant',
    'FinancialDealParty',
    'FinancialDealProduct',
    'FinclDealInteractionSummary',
    'FlowInterview',
    'FlowOrchestrationInstance',
    'FlowOrchestrationStageInstance',
    'FlowOrchestrationStepInstance',
    'FlowOrchestrationWorkItem',
    'ForecastingOwnerAdjustment',
    'FrgtHaulingEmssnFctr',
    'FrgtHaulingEnrgyUse',
    'FtestBigObjectLookup',
    'FtestConcrete',
    'FtestConcrete6',
    'FtestConcrete7',
    'FtestConcrete8',
    'FtestVirtualMain',
    'FtestZosUiPrototypeChild1',
    'FtestZosUiPrototypeChild2',
    'FtestZosUiPrototypeParent',
    'FulfillmentOrder',
    'FulfillmentOrderLineItem',
    'FundingAward',
    'FundingAwardAmendment',
    'FundingAwardRequirement',
    'FundingDisbursement',
    'FundingOpportunity',
    'GeneratedDocument',
    'GeneratedDocumentSection',
    'GeneratedWaste',
    'GenericVisitTask',
    'GenericVisitTaskContext',
    'GeoCountry',
    'GeoState',
    'GnrcVstKeyPerformanceInd',
    'GnrcVstTaskContextRelation',
    'GoalAssignment',
    'GoalAssignmentDetail',
    'GoalDefinition',
    'GoalLink',
    'GovtFinancialAsstSum',
    'GroundTravelEmssnFctr',
    'GroundTravelEnrgyUse',
    'Group',
    'GroupCensus',
    'GroupCensusMember',
    'GroupCensusMemberPlan',
    'GroupClass',
    'GroupClassContribution',
    'GroupMember',
    'GuestBuyerProfile',
    'HealthCareDiagnosis',
    'HealthCareProcedure',
    'HealthCondition',
    'HealthConditionDetail',
    'HealthScore',
    'HealthScoreActionLog',
    'HealthcareFacility',
    'HealthcareFacilityNetwork',
    'HealthcarePayerNetwork',
    'HealthcarePractitionerFacility',
    'HealthcareProvider',
    'HealthcareProviderNpi',
    'HealthcareProviderSpecialty',
    'HealthcareProviderTaxonomy',
    'Holiday',
    'HotelStayEmssnFctr',
    'HotelStayEnrgyUse',
    'IPAddressRange',
    'Identifier',
    'IdentityDocument',
    'IdentityResolution',
    'Image',
    'InStoreLocation',
    'Incident',
    'IncidentRelatedItem',
    'IncludedLicenseDefinition',
    'Individual',
    'IndividualApplication',
    'IndividualApplicationItem',
    'IndividualCrisisAssessment',
    'InflationRate',
    'InfoAuthRequestForm',
    'InfoAuthorizationRequest',
    'InsPolicyTransactionDetail',
    'InsightsExternalData',
    'InsightsExternalDataPart',
    'InspectionAssessmentInd',
    'InspectionType',
    'InsuranceClaimAsset',
    'InsuranceContract',
    'InsurancePolicy',
    'InsurancePolicyAsset',
    'InsurancePolicyCoverage',
    'InsurancePolicyMemberAsset',
    'InsurancePolicyOwner',
    'InsurancePolicyParticipant',
    'InsurancePolicySurcharge',
    'InsurancePolicyTransaction',
    'InsuranceProfile',
    'IntegrationProviderDef',
    'Interaction',
    'InteractionAttendee',
    'InteractionParticipant',
    'InteractionRelatedAccount',
    'InteractionSumDiscussedAccount',
    'InteractionSummary',
    'InteractionSummaryParticipant',
    'InterestTag',
    'InternalOrganizationUnit',
    'InventoryItemReservation',
    'InventoryReservation',
    'Invoice',
    'InvoiceAddressGroup',
    'InvoiceBatchRun',
    'InvoiceBatchRunCriteria',
    'InvoiceBatchRunRecovery',
    'InvoiceLine',
    'JobFamily',
    'JobProfile',
    'JournalSubType',
    'JournalType',
    'KnowledgeArticleVersion',
    'Lead',
    'LeadLineItem',
    'LeadPreferredSeller',
    'LearnerProgram',
    'LearnerProgramRequirement',
    'LearnerProgramRqmtProgress',
    'Learning',
    'LearningAchievement',
    'LearningCourse',
    'LearningFoundationItem',
    'LearningItem',
    'LearningItemAssignment',
    'LearningItemProgress',
    'LearningOutcomeItem',
    'LearningProgram',
    'LearningProgramPlan',
    'LearningProgramPlanRqmt',
    'LegalEntity',
    'LicensingError',
    'LicensingRequest',
    'LicensingRequestOrderItem',
    'LicensingSnapshotRequest',
    'LinkedArticle',
    'ListEmail',
    'LiveAgentSession',
    'LiveChatButton',
    'LiveChatDeployment',
    'LiveChatSensitiveDataRule',
    'LiveChatTranscript',
    'LiveChatVisitor',
    'LoanApplicant',
    'LoanApplicantAddress',
    'LoanApplicantAsset',
    'LoanApplicantDeclaration',
    'LoanApplicantEmployment',
    'LoanApplicantIncome',
    'LoanApplicantLiability',
    'LoanApplicationAsset',
    'LoanApplicationFinancial',
    'LoanApplicationLiability',
    'LoanApplicationProperty',
    'LoanApplicationTitleHolder',
    'LocWaitlistMsgTemplate',
    'Location',
    'LocationGroup',
    'LocationGroupAssignment',
    'LocationTrustMeasure',
    'LocationWaitlist',
    'LocationWaitlistedParty',
    'LoyaltyLedger',
    'LoyaltyMemberCurrency',
    'LoyaltyMemberTier',
    'LoyaltyMembershipLifecycle',
    'LoyaltyPartnerProduct',
    'LoyaltyPgmEngmtAttrProm',
    'LoyaltyPgmEngmtAttribute',
    'LoyaltyPgmGroupMbrRlnsp',
    'LoyaltyPgmMbrAttributeVal',
    'LoyaltyPgmPartnerCurrency',
    'LoyaltyPgmPartnerPromotion',
    'LoyaltyPgmPtnrLdgrSummary',
    'LoyaltyPgmPtnrPrepaidPack',
    'LoyaltyProgram',
    'LoyaltyProgramBadge',
    'LoyaltyProgramCurrency',
    'LoyaltyProgramMbrPromotion',
    'LoyaltyProgramMember',
    'LoyaltyProgramMemberBadge',
    'LoyaltyProgramMemberCase',
    'LoyaltyProgramMemberMerge',
    'LoyaltyProgramPartner',
    'LoyaltyProgramProcess',
    'LoyaltyProgramWidget',
    'LoyaltyTier',
    'LoyaltyTierBenefit',
    'LoyaltyTierGroup',
    'Macro',
    'MaintenanceAsset',
    'MaintenancePlan',
    'MaintenanceWorkRule',
    'ManagedContent',
    'ManagedContentVariant',
    'ManufacturingProgram',
    'MarketSegment',
    'MarketSegmentActivation',
    'MarketingAction',
    'MarketingForm',
    'MarketingLink',
    'MarketingLinkContent',
    'MarketingResource',
    'MedReconRecommendation',
    'MedReconStmtRecommendation',
    'MedTherapyStmtReviewIssue',
    'MediaChannel',
    'MediaContentTitle',
    'MediaPrintIssue',
    'MediaTitleAvailability',
    'MediaTitleReleaseDate',
    'MediaTitleRevenue',
    'Medication',
    'MedicationDispense',
    'MedicationReconciliation',
    'MedicationRequest',
    'MedicationStatement',
    'MedicationStatementDetail',
    'MedicationTherapyReview',
    'MedicationTherapyStmtReview',
    'MedicinalIngredient',
    'MemberBenefit',
    'MemberPlan',
    'MerchantAccount',
    'MessagingChannel',
    'MessagingDeliveryError',
    'MessagingEndUser',
    'MessagingSession',
    'Metric',
    'MfgProgramCpntFrcstFact',
    'MfgProgramForecastFact',
    'MfgProgramVariantFrcstFact',
    'MilestoneType',
    'MktAiPredictiveInsight',
    'MktCalculatedInsight',
    'MktDataTransform',
    'MktMLModel',
    'MobileSettingsAssignment',
    'MsgChannelLanguageKeyword',
    'MultipartyInfoAuthRequest',
    'NamespaceRegistry',
    'NetworkReferencedObject',
    'Note',
    'ObjectRelatedUrl',
    'ObjectStateActionDefinition',
    'ObjectStateDefinition',
    'ObjectStateTransition',
    'ObjectStateTransitionAction',
    'ObjectStateValue',
    'ObjectTerritory2Association',
    'OcrDocumentScanResult',
    'OmniAssessmentTask',
    'OmniDataPack',
    'OmniDataTransform',
    'OmniDataTransformItem',
    'OmniESignatureTemplate',
    'OmniProcess',
    'OmniProcessAsmtQuestionVer',
    'OmniProcessCompilation',
    'OmniProcessElement',
    'OmniProcessTransientData',
    'OmniScriptSavedSession',
    'OmniUiCard',
    'OperatingHours',
    'OperatingHoursHoliday',
    'Opportunity',
    'OpportunityContactRole',
    'OpportunityHistory',
    'OpportunityLineItem',
    'OpportunityLineItemSchedule',
    'OpportunityLineItemSplit',
    'OpportunityParticipant',
    'OpportunityPartner',
    'OpportunityPreferredSeller',
    'OpportunitySplit',
    'OpportunityTeamMember',
    'Order',
    'OrderAction',
    'OrderAdjustmentGroup',
    'OrderAdjustmentGroupSummary',
    'OrderChangeLog',
    'OrderDeliveryGroup',
    'OrderDeliveryGroupSummary',
    'OrderDeliveryMethod',
    'OrderItem',
    'OrderItemAdjustmentLineItem',
    'OrderItemAdjustmentLineSummary',
    'OrderItemConsumptionRate',
    'OrderItemConsumptionSchedule',
    'OrderItemRelationship',
    'OrderItemSummary',
    'OrderItemSummaryChange',
    'OrderItemTaxLineItem',
    'OrderItemTaxLineItemSummary',
    'OrderPaymentSummary',
    'OrderSummary',
    'OrderSummaryRoutingSchedule',
    'OrgIncidentSummary',
    'OrgMetric',
    'OrgMetricScanResult',
    'OrgMetricScanSummary',
    'Organization',
    'OtherComponentTask',
    'OtherEmssnFctrSet',
    'OtherEmssnFctrSetItem',
    'ParticipantRole',
    'Partner',
    'PartnerFundAllocation',
    'PartnerFundClaim',
    'PartnerFundRequest',
    'PartnerMarketingBudget',
    'PartyCertifiedCapacity',
    'PartyConsent',
    'PartyIdentityVerification',
    'PartyIdentityVerificationStep',
    'PartyIncome',
    'PartyProfile',
    'PartyProfileAddress',
    'PartyProfileRisk',
    'PartyRelationshipGroup',
    'PartyRoleRelation',
    'PartyScreeningStep',
    'PartyScreeningSummary',
    'PatientHealthReaction',
    'PatientImmunization',
    'PatientImmunizationProtocol',
    'PatientMedicalProcedure',
    'PatientMedicalProcedureDetail',
    'PatientMedicationDosage',
    'Payment',
    'PaymentAuthAdjustment',
    'PaymentBatchRun',
    'PaymentBatchRunCriteria',
    'PaymentGateway',
    'PaymentGatewayLog',
    'PaymentGatewayProvider',
    'PaymentIntent',
    'PaymentLineInvoice',
    'PaymentRequest',
    'PaymentRequestLine',
    'PaymentSchedule',
    'PaymentScheduleItem',
    'PaymentSchedulePolicy',
    'PaymentScheduleTreatment',
    'PaymentScheduleTreatmentDtl',
    'PaymentTerm',
    'PaymentTermItem',
    'PcmtEmssnFctrSet',
    'PcmtEmssnFctrSetItem',
    'PercentileBsdCareFeeAgreement',
    'Period',
    'PersonAccount',
    'PersonDisability',
    'PersonEducation',
    'PersonEmployment',
    'PersonExamination',
    'PersonLanguage',
    'PersonLifeEvent',
    'PersonList',
    'PersonName',
    'PgmRebateTypBnftMapping',
    'PickTicket',
    'PickTicketAssignment',
    'PickTicketProduct',
    'PlanBenefit',
    'PlanBenefitItem',
    'PlatformLicenseDefinition',
    'Polygon',
    'PreliminaryApplicationRef',
    'PreventiveCareAgreement',
    'PriceAdjustmentGroupShape',
    'PriceAdjustmentItemShape',
    'PriceAdjustmentSchedule',
    'PriceAdjustmentTier',
    'PriceList',
    'PriceRule',
    'PriceRuleAction',
    'PriceRuleActionItem',
    'PriceRuleCondition',
    'PriceRuleConditionFilter',
    'PriceRuleExecutionStage',
    'Pricebook2',
    'PricebookEntry',
    'PricingHeader',
    'PricingLineItem',
    'PricingProduct',
    'PrivacyHold',
    'PrivacyHoldReason',
    'PrivacyJobSession',
    'PrivacyPolicy',
    'PrivacyRequest',
    'Problem',
    'ProblemDefinition',
    'ProblemGoalDefinition',
    'ProblemIncident',
    'ProblemRelatedItem',
    'ProcessDefinition',
    'ProcessException',
    'ProcessInstance',
    'Producer',
    'ProducerCommission',
    'ProducerPolicyAssignment',
    'Product2',
    'Product2DataTranslation',
    'ProductAttribute',
    'ProductAttributeDefinition',
    'ProductAttributeSet',
    'ProductAttributeSetItem',
    'ProductAttributeSetProduct',
    'ProductAvailabilityProjection',
    'ProductCatalog',
    'ProductCategory',
    'ProductCategoryAttribute',
    'ProductCategoryDataTranslation',
    'ProductCategoryMedia',
    'ProductCategoryProduct',
    'ProductCategoryServiceProcess',
    'ProductComponentGroup',
    'ProductConsumed',
    'ProductConsumedState',
    'ProductConsumptionSchedule',
    'ProductCoverage',
    'ProductCoverageLimit',
    'ProductEmissionsFactor',
    'ProductFaultCode',
    'ProductFulfillmentLocation',
    'ProductItem',
    'ProductItemTransaction',
    'ProductLaborCode',
    'ProductLicenseMap',
    'ProductMedia',
    'ProductOverridePathItem',
    'ProductQuantityRule',
    'ProductRelComponentOverride',
    'ProductRelatedComponent',
    'ProductRelatedMaterial',
    'ProductRelationshipType',
    'ProductRequest',
    'ProductRequestLineItem',
    'ProductRequired',
    'ProductSellingModel',
    'ProductSellingModelOption',
    'ProductServiceCampaign',
    'ProductServiceCampaignItem',
    'ProductSpecificationType',
    'ProductTransfer',
    'ProductTransferState',
    'ProductWarrantyTerm',
    'Profile',
    'ProfileSkill',
    'ProfileSkillEndorsement',
    'ProfileSkillUser',
    'Program',
    'ProgramEnrollment',
    'ProgramProduct',
    'ProgramRebateTypPayoutSrc',
    'ProgramRebateType',
    'ProgramRebateTypeBenefit',
    'ProgramRebateTypeFilter',
    'ProgramRebateTypePayout',
    'ProgramRebateTypeProduct',
    'ProgramRebateTypeReference',
    'ProgramTermApplnTimeline',
    'Promotion',
    'PromotionChannel',
    'PromotionDataTranslation',
    'PromotionLoyaltyPtnrProdt',
    'PromotionMarketSegment',
    'PromotionProduct',
    'PromotionProductCategory',
    'PromotionQualifier',
    'PromotionSegment',
    'PromotionSegmentBuyerGroup',
    'PromotionSegmentSalesStore',
    'PromotionTarget',
    'PromotionTier',
    'ProrationPolicy',
    'ProviderNetworkContract',
    'ProviderNetworkTier',
    'ProviderSearchSyncLog',
    'ProvisioningRun',
    'ProvisioningRunEvent',
    'ProvisioningTargetGroup',
    'ProvisioningTargetItem',
    'ProvisioningTargetRole',
    'PublicApplicationParticipant',
    'PublicComplaint',
    'PublicProgram',
    'PurchaseQuantityRule',
    'PurchaserPlan',
    'PurchaserPlanAssn',
    'PymtSchdDistributionMethod',
    'QuickText',
    'Quote',
    'QuoteDocument',
    'QuoteItemTaxItem',
    'QuoteLineItem',
    'QuoteLineItemAttribute',
    'QuoteLinePriceAdjustment',
    'QuoteLineRelationship',
    'RebateClaim',
    'RebateMemberAggregateItem',
    'RebateMemberProductAggregate',
    'RebatePayment',
    'RebatePayoutAdjustment',
    'RebateProgram',
    'RebateProgramMember',
    'RebateProgramMemberPayout',
    'RebateProgramPayoutPeriod',
    'ReceivedDocument',
    'ReceivedDocumentType',
    'Recommendation',
    'RecordAction',
    'RecordAlert',
    'RecordType',
    'RecordsetFilterCriteria',
    'RecordsetFilterCriteriaRule',
    'RecordsetFltrCritMonitor',
    'RecurrenceSchedule',
    'ReferencedDashboard',
    'Referral',
    'RefrigerantEmssnFctr',
    'RefundLinePayment',
    'RegAuthorizationTypeProduct',
    'RegionStatusDiscount',
    'RegulatoryAuthority',
    'RegulatoryAuthorizationType',
    'RegulatoryCode',
    'RegulatoryCodeAssessmentInd',
    'RegulatoryCodeUse',
    'RegulatoryCodeViolation',
    'RegulatoryTrxnFee',
    'RegulatoryTrxnFeeItem',
    'ReinsAgreementLineItem',
    'ReinsPolicyBordereauFact',
    'ReinsProtocolProdtComponent',
    'ReinsuranceAgreement',
    'ReinsuranceProtocol',
    'ReinsuranceProtocolAgreement',
    'RentalCarEmssnFctr',
    'RentalCarEnrgyUse',
    'ReplyText',
    'Report',
    'ReportAnomalyEventStore',
    'ReportedConsumption',
    'ResidentialLoanApplication',
    'ResourceAbsence',
    'ResourcePreference',
    'RetailLocationGroup',
    'RetailStore',
    'RetailStoreGroupAssignment',
    'RetailStoreKpi',
    'RetailVisitKpi',
    'ReturnOrder',
    'ReturnOrderItemAdjustment',
    'ReturnOrderItemTax',
    'ReturnOrderLineItem',
    'RevRecogTreatmentItemGroup',
    'RevenueAsyncOperation',
    'RevenueDistributionMethod',
    'RevenueElement',
    'RevenueElementDistribution',
    'RevenueRecogTreatmentItem',
    'RevenueRecognitionPolicy',
    'RevenueRecognitionTreatment',
    'RevenueTransactionErrorLog',
    'RightsQuote',
    'RightsQuoteLineItem',
    'RightsQuoteLineMediaTitle',
    'SalesAgreement',
    'SalesAgreementProduct',
    'SalesAgreementProductSchedule',
    'SalesChannel',
    'SalesContractLine',
    'SalesTransactionItemShape',
    'SalesTransactionShape',
    'SalesTrxnItemRelationShape',
    'SalesforceContract',
    'SalesforceInvoice',
    'SalesforceQuote',
    'SchedulingConstraint',
    'Scope3CrbnFtprnt',
    'Scope3EmssnSrc',
    'Scope3PcmtItem',
    'Scope3PcmtSummary',
    'Scorecard',
    'ScorecardAssociation',
    'ScorecardMetric',
    'ScratchOrgInfo',
    'SearchPromotionRule',
    'SecuritiesHolding',
    'Seller',
    'SerializedProduct',
    'SerializedProductTransaction',
    'ServiceAppointment',
    'ServiceAppointmentCapacityUsage',
    'ServiceAppointmentGroup',
    'ServiceContract',
    'ServiceCrew',
    'ServiceCrewMember',
    'ServiceReport',
    'ServiceReportLayout',
    'ServiceResource',
    'ServiceResourceCapacity',
    'ServiceResourceDataTranslation',
    'ServiceResourcePreference',
    'ServiceResourceSkill',
    'ServiceTerritory',
    'ServiceTerritoryDataTranslation',
    'ServiceTerritoryLocation',
    'ServiceTerritoryMember',
    'ServiceTerritoryRelationship',
    'ServiceTerritoryWorkType',
    'SessionHijackingEventStore',
    'SettingUsageDefinition',
    'SettingUsageMap',
    'SetupAssistantStep',
    'SharedSavingPaymentAgreement',
    'SharingRecordCollection',
    'SharingRecordCollectionItem',
    'Shift',
    'ShiftEngagementChannel',
    'ShiftPattern',
    'ShiftPatternEntry',
    'ShiftSegment',
    'ShiftSegmentType',
    'ShiftTemplate',
    'ShiftWorkTopic',
    'Shipment',
    'ShipmentItem',
    'SignatureTask',
    'SignatureTaskLineItem',
    'Skill',
    'SkillLevelDefinition',
    'SkillRequirement',
    'Snippet',
    'SnippetAssignment',
    'SocialContributionSum',
    'SocialPersona',
    'SocialPost',
    'StandardCareFeeAgreement',
    'StandardPermissionSet',
    'StaticResource',
    'StnryAssetAnnualFact',
    'StnryAssetCrbnFtprnt',
    'StnryAssetCrbnFtprntItm',
    'StnryAssetEnrgyUse',
    'StnryAssetEnvrSrc',
    'StnryAssetWaterActvty',
    'StnryAssetWaterFtprnt',
    'StnryAssetWtrFtprntItm',
    'StoreActionPlanTemplate',
    'StoreAssortment',
    'StoreIntegratedService',
    'StoreProduct',
    'StreamingChannel',
    'SuccessTeam',
    'Supplier',
    'SupplierProduct',
    'Survey',
    'SurveyEmailBranding',
    'SurveyInvitation',
    'SurveyQuestion',
    'SurveyQuestionChoice',
    'SurveyQuestionResponse',
    'SurveyResponse',
    'SurveySubject',
    'SurveyVersion',
    'SustainabilityCntrLine',
    'SustainabilityCredit',
    'SustainabilityPurchase',
    'SustainabilityScorecard',
    'SvcCatalogCategory',
    'SvcCatalogCategoryItem',
    'SvcCatalogItemDef',
    'SvcCatalogReqExtdAttrVal',
    'SvcCatalogReqRelatedItem',
    'SvcCatalogRequest',
    'Swarm',
    'SwarmMember',
    'TagCategory',
    'TagCategoryAssignment',
    'Task',
    'TaxDisclosureSummary',
    'TaxEngine',
    'TaxEngineInteractionLog',
    'TaxEngineProvider',
    'TaxGeoConfig',
    'TaxPolicy',
    'TaxRate',
    'TaxTreatment',
    'Tenant',
    'TenantParameterMap',
    'TenantSecurityNotificationRule',
    'Territory2',
    'ThreatDetectionFeedback',
    'TimeSheet',
    'TimeSheetEntry',
    'TimeSlot',
    'Topic',
    'TopicAssignment',
    'TrackedCommunication',
    'TrackedCommunicationDetail',
    'TrainingCourseParticipant',
    'TransactionJournal',
    'Translation',
    'TravelMode',
    'TrialEnvironmentSignup',
    'TrnCourse',
    'UnitOfMeasure',
    'UnitOfMeasureConversion',
    'UsageEntitlement',
    'UsageEntitlementPeriod',
    'UsageEntitlementUsage',
    'UsageFactor',
    'UsageImpactFactor',
    'UsageImpactGroup',
    'UsageImpactGroupFactor',
    'UsageImpactGroupPgmMeasure',
    'UsageImpactGroupVersion',
    'User',
    'UserDefinedLabel',
    'UserDefinedLabelAssignment',
    'UserEsignVendorIdentifier',
    'UserFinancialAuthority',
    'UserLicenseDefinition',
    'UserLocationAssignment',
    'UserRole',
    'UserTerritory2AssocLog',
    'Vehicle',
    'VehicleAssetCrbnFtprnt',
    'VehicleAssetEmssnSrc',
    'VehicleAssetEnrgyUse',
    'VehicleDefinition',
    'VehicleUserAssignment',
    'VideoCall',
    'ViolationEnforcementAction',
    'ViolationType',
    'ViolationTypeAssessmentInd',
    'ViolationTypeUse',
    'Visit',
    'VisitedParty',
    'Visitor',
    'VoiceCall',
    'VolumeDiscount',
    'VolunteerProject',
    'Voucher',
    'VoucherDefinition',
    'WarrantyTerm',
    'WarrantyTermCoverage',
    'WasteFootprint',
    'WasteFootprintItem',
    'WaveAutoInstallRequest',
    'WebCart',
    'WebStore',
    'WebStoreBuyerGroup',
    'WebStoreCatalog',
    'WebStoreInventorySource',
    'WebStorePricebook',
    'WebStoreSearchProdSettings',
    'WorkCapacityLimit',
    'WorkCapacityUsage',
    'WorkCoaching',
    'WorkContract',
    'WorkOrder',
    'WorkOrderLineItem',
    'WorkPlan',
    'WorkPlanSelectionRule',
    'WorkPlanTemplate',
    'WorkPlanTemplateEntry',
    'WorkProcedure',
    'WorkProcedureStep',
    'WorkRewardFund',
    'WorkRewardFundType',
    'WorkStep',
    'WorkStepTemplate',
    'WorkType',
    'WorkTypeCareSpecialty',
    'WorkTypeCodeSetBundle',
    'WorkTypeExtension',
    'WorkTypeGroup',
    'WorkTypeGroupDataTranslation',
    'WorkTypeGroupMember',
    'WorkTypeGroupRole',
    'WorkerCompCoverageClass',
    'WstDispoEmssnFctrSet',
    'WstDispoEmssnFctrSetItm',
]);
/**
 * A set of the string names of known entities.
 *
 * Generated
 *     from: ui-services-private-object-deny-list.yaml
 *     API version: 63
 *     at: Wed, 21 Aug 2024 19:39:55 GMT
 */
const UIAPI_DENIED_ENTITY_API_NAMES = new Set([
    'AIProductOptimizedField',
    'AITenantProvisionedFeature',
    'ASEOrgSyncHealthStats',
    'ASEOrgSyncHealthTxns',
    'AbnExperimentCohortAttrVal',
    'ActionableOrchResponseEvent',
    'ActionableOrchSourceEvent',
    'ActnblListKeyPrfmIndAsgnt',
    'AiMetadataSyncStatus',
    'AnalyticsChangeEventLog',
    'AnalyticsDownloadEventLog',
    'AnalyticsGenerativeMetadata',
    'AnalyticsInteractEventLog',
    'AnalyticsPerfEventLog',
    'AnalyticsTaskStatusEvent',
    'ApexCalloutEventLog',
    'ApexExecutionEventLog',
    'ApexExtlCalloutEventLog',
    'ApexRestApiEventLog',
    'ApexSoapApiEventLog',
    'ApexTriggerEventLog',
    'ApexUnexpectedExcpEventLog',
    'ApiTotalUsageEventLog',
    'ApplnFormUsageTracking',
    'AssetThresholdUpdateEvent',
    'AsyncReportRunEventLog',
    'AuraRequestEventLog',
    'BulkApi2EventLog',
    'BulkApi2JobResultsEvent',
    'BulkApi2JobStatusEvent',
    'BulkApiEventLog',
    'BulkMessage',
    'CapabilityUserPreference',
    'CaseFileProcessDependency',
    'CaseFileProcessExecution',
    'CatalogListing',
    'CheckoutResetConfig',
    'CommerceJobStatus',
    'CommerceSearchIndexError',
    'CommerceSearchResultsRule',
    'CompiledProduct',
    'ConcurApexLimitEventLog',
    'ConsentUnsubscribeAllEvent',
    'ConsentUnsubscribeEvent',
    'ContentTaxonomyModel',
    'ContentTransferEventLog',
    'ContextPersistenceEvent',
    'ContractCreationEvent',
    'ConvIntelligenceVoiceCall',
    'ConvMessageSendRequest',
    'ConversationInsightEvent',
    'ConversationTranscriptSendRequest',
    'CuratedExpJobEvent',
    'CuratedExpPlanEvent',
    'DTRecordsetReplica',
    'DataActionJobStatusEvent',
    'DataCloudCurrencyConfig',
    'DataCloudFiscalCalendar',
    'DataKitDeployEvent',
    'DataMultiOrgServiceEvent',
    'DataShellTntUpgradeEvent',
    'DcsnTblSourceObjectRecord',
    'DecisionTableDataset',
    'DecisionTableRecordset',
    'DecisionTblSrcChangedEvent',
    'DiscoveryFrwrkSampleTemplate',
    'DiscoveryFrwrkTemplateMember',
    'DocGenBtchStsChgEvent',
    'DocGenProcStsChgEvent',
    'DocumentExtReviewerConsent',
    'EmailBounceEvent',
    'EnblMlstnProgQryTmotEvent',
    'EngagementSignalCmpndMetric',
    'EngagementSignalDistinct',
    'EngagementSignalFilter',
    'EngagementSignalFltrGrp',
    'EngagementSignalFltrVal',
    'EngagementSignalJoinDim',
    'EngagementSignalMetric',
    'ExtKAImportError',
    'ExtKAImportResult',
    'ExtKnowledgeIngestionRun',
    'ExtKnowledgeIngestionStatus',
    'ExtKnowledgeIngestionTask',
    'ExtKnowledgeOrgStatus',
    'ExtKnowledgePartnerOrg',
    'ExtKnowledgeXDSConnector',
    'ExternalDataChgEvent',
    'ExternalDocRefMapping',
    'ExternalEncryptionRootKey',
    'ExternalTrigger',
    'ExternalTriggerData',
    'ExternalTriggerSubscriber',
    'ExtlRecShrCnctAccnt',
    'ExtlRecShrEvent',
    'ExtlRecShrProcessEvent',
    'ExtlRecShrRecordMap',
    'ExtlRecShrResultEvent',
    'FirstBillPaymentSetupEvent',
    'FlowAutoSaveStsChngEvent',
    'FlowAutosaveInterviewStatus',
    'FlowDefDataActionMapping',
    'FlowNavMetricEventLog',
    'GenericHammerResult',
    'GuestUserAnomalyEvent',
    'InsufficientAccessEventLog',
    'InsuranceAsyncBulkRecordDetail',
    'InsuranceRatingInput',
    'InsuranceRatingOutput',
    'InsuranceRatingRequestEvent',
    'InsuranceRatingRequestItem',
    'InterviewItemWaitCondition',
    'KnowledgeArticleEventLog',
    'LightningLoggerEventLog',
    'LightningPageViewEventLog',
    'LoginAsEventLog',
    'LoginEventLog',
    'MLModelDataAlert',
    'ManagedContentJobBody',
    'ManagedContentOffCoreLocation',
    'ManagedContentSpaceFolderShare',
    'MerchantAccountEvent',
    'MessagingChannelUpdateEvent',
    'MetadataApiOpEventLog',
    'MetricsDeletionJobStatus',
    'MktMLModelPartitionRun',
    'MktMLModelSetupRun',
    'MktMLSetupRunChgEvent',
    'MngContentFormActionEvent',
    'ObjectDataImportResult',
    'ObjectUserTerritory2View',
    'OmniTrackingEvent',
    'OrderItemAssetizationState',
    'OrgDayZeroStatus',
    'OrgSharingEvent',
    'OvenSdbExperimentRequest',
    'PackageInstallEventLog',
    'PackagePropUpdateJob',
    'PackagePropUpdateRequest',
    'PackageSubscriberOrgInfo',
    'PaymentIntentEvent',
    'PaymentLinkEvent',
    'PersnlDecisionAttrValue',
    'PersonalizationAttribute',
    'PlatformEventUsage',
    'PriceBookPriceGuidance',
    'PriceSheetData',
    'PricingDecisionHashMap',
    'PricingLookupSyncStatus',
    'PricingWaterfallLog',
    'PushUpgradeCustomization',
    'QuoteSaveEvent',
    'RebatePayoutSnapshot',
    'RecommenderStatusChgEvent',
    'RelationshipGraphView',
    'ReminderRecord',
    'ReportEventLog',
    'RestApiEventLog',
    'Rule',
    'RuleAction',
    'RuleActionParameter',
    'RuleCondition',
    'RuleDeploymentRequestEvent',
    'RuleExpression',
    'RuleExpressionTerm',
    'RuleFilterCriteria',
    'RuleLibraryContextTag',
    'RuleLibraryVersionSnapshot',
    'RuleReferenceVariable',
    'RuleValue',
    'RuleVersion',
    'RuleVersionSnapshot',
    'Ruleset',
    'RulesetVersion',
    'RulesetVersionSnapshot',
    'RuntimeCatalogSnapshot',
    'RuntimeCatalogSnapshotIndex',
    'SalesTrxnItemPrcSht',
    'SalesTrxnItemPrcShtShape',
    'SalesforcePayment',
    'SandboxStatusEventLog',
    'SavedPaymentMethodEvent',
    'ScheduledReminder',
    'SearchBBRule',
    'SearchBBRuleCondition',
    'SearchClickEventLog',
    'SearchEventLog',
    'SearchIndexFieldConfig',
    'SearchIndexObjectConfig',
    'SearchRecencyIndexingJob',
    'ServiceAppointmentEvent',
    'ServiceAppointmentShift',
    'SiqOrgMigrationStatus',
    'SiteEventLog',
    'SoapApiEventLog',
    'StatsInvalidationEvent',
    'TenantBillingUsageEvent',
    'TransactionSecurityEventLog',
    'UnifiedAnalyticsMonEvent',
    'UnitOfMeasureUnit',
    'UriEventLog',
    'VisualforceRequestEventLog',
    'WebPushMessageSubscription',
    'WebStoreCMS',
    'WebStoreUserCreatedEvent',
    'YourAccountDataEvent',
]);

const MAX_RECORD_DEPTH = 5;
const FIELD_SEPARATOR = '.';
const SCALAR_VALUE_SELECTION = {
    kind: 'Scalar',
    name: 'value',
};
const DISPLAY_VALUE_SELECTION = {
    kind: 'Scalar',
    name: 'displayValue',
};
const FIELDS_SELECTION = {
    kind: 'Object',
    name: 'fields',
};
function convertRecordFieldsArrayToTrie(fields, optionalFields = []) {
    const root = {
        name: '<root>',
        optional: false,
        scalar: false,
        children: {},
    };
    insertFieldsIntoTrie(root, fields, false);
    insertFieldsIntoTrie(root, optionalFields, true);
    return root;
}
function createPathSelection(propertyName, fieldDefinition) {
    const fieldsSelection = [];
    const { children } = fieldDefinition;
    const childrenKeys = keys(children);
    for (let i = 0, len = childrenKeys.length; i < len; i += 1) {
        const childKey = childrenKeys[i];
        const childFieldDefinition = children[childKey];
        let fieldValueSelection;
        if (childFieldDefinition.scalar === true) {
            fieldValueSelection = SCALAR_VALUE_SELECTION;
        }
        else {
            fieldValueSelection = {
                kind: 'Link',
                name: 'value',
                nullable: true,
                fragment: {
                    kind: 'Fragment',
                    private: ['eTag', 'weakEtag'],
                    selections: createRecordSelection(childFieldDefinition),
                    version: VERSION$2K,
                },
            };
        }
        const fieldSelection = {
            kind: 'Link',
            name: childFieldDefinition.name,
            required: childFieldDefinition.optional === true ? false : undefined,
            fragment: {
                kind: 'Fragment',
                version: VERSION$2M,
                private: [],
                selections: [DISPLAY_VALUE_SELECTION, fieldValueSelection],
            },
        };
        push.call(fieldsSelection, fieldSelection);
    }
    return {
        kind: 'Object',
        name: propertyName,
        selections: fieldsSelection,
    };
}
/**
 * Convert a RecordRepresentationLike into its equivalent selection.
 */
function createPathSelectionFromValue(fields) {
    const fieldsSelection = [];
    const fieldNames = keys(fields);
    for (let i = 0, len = fieldNames.length; i < len; i++) {
        const fieldName = fieldNames[i];
        const { value: fieldValue } = fields[fieldName];
        let fieldValueSelection = SCALAR_VALUE_SELECTION;
        if (isSpanningRecord(fieldValue)) {
            fieldValueSelection = {
                kind: 'Link',
                name: 'value',
                nullable: true,
                fragment: {
                    kind: 'Fragment',
                    private: [],
                    selections: buildSelectionFromRecord(fieldValue),
                    version: VERSION$2K,
                },
            };
        }
        const fieldSelection = {
            kind: 'Link',
            name: fieldName,
            required: undefined,
            fragment: {
                kind: 'Fragment',
                version: VERSION$2M,
                private: [],
                selections: [DISPLAY_VALUE_SELECTION, fieldValueSelection],
            },
        };
        push.call(fieldsSelection, fieldSelection);
    }
    return {
        kind: 'Object',
        name: FIELDS_SELECTION.name,
        selections: fieldsSelection,
    };
}
function extractRecordFieldsRecursively(record) {
    const fields = [];
    const fieldNames = keys(record.fields);
    for (let i = 0, len = fieldNames.length; i < len; i++) {
        const fieldName = fieldNames[i];
        const { value: fieldValue } = record.fields[fieldName];
        if (isSpanningRecord(fieldValue)) {
            const spanningRecordFields = extractRecordFieldsRecursively(fieldValue);
            for (let j = 0, len = spanningRecordFields.length; j < len; j++) {
                spanningRecordFields[j] = `${fieldName}.${spanningRecordFields[j]}`;
            }
            push.apply(fields, spanningRecordFields);
        }
        else {
            push.call(fields, fieldName);
        }
    }
    return fields;
}
/**
 * Returns a list of fields for a RecordRepresentationLike.
 *
 * TODO [W-6900271]: Remove this function once getRelatedList don't depend on it anymore. Always prefer
 * generating a selection out of a record, than convert a record to a field list and back to a
 * selection.
 */
function extractFields(value) {
    const { apiName } = value;
    const fields = extractRecordFieldsRecursively(value);
    for (let i = 0, len = fields.length; i < len; i++) {
        fields[i] = `${apiName}.${fields[i]}`;
    }
    return fields;
}
function insertFieldsIntoTrie(root, fields, optional) {
    for (let i = 0, len = fields.length; i < len; i++) {
        const field = fields[i].split(FIELD_SEPARATOR);
        let current = root;
        for (let j = 1, len = field.length; j < len && j <= MAX_RECORD_DEPTH + 1; j++) {
            const fieldName = field[j];
            let next = current.children[fieldName];
            if (next === undefined) {
                // A field is scalar only if it is the last field name in the field.
                const scalar = j === len - 1;
                // LDS restricts the numbers of fields that can be traversed to MAX_RECORD_DEPTH,
                // however we still denormalize fields at MAX_RECORD_DEPTH + 1, only if they are
                // scalar fields.
                if (j <= MAX_RECORD_DEPTH || scalar === true) {
                    // We now know that there are children fields, so we can mark the parent
                    // as not a scalar
                    current.scalar = false;
                    next = {
                        name: fieldName,
                        scalar,
                        optional,
                        children: {},
                    };
                    current.children[fieldName] = next;
                }
            }
            current = next;
        }
    }
}

const CHILD_RELATIONSHIP_SELECTION = {
    // We don't support RecordRep.childRelationships because it has a nasty
    // degenerate case of multiple pages of child records
    kind: 'Object',
    name: 'childRelationships',
};
// This selection is a client-side only selection and will only
// be applied to a RecordRepresentation in environments configured with
// drafts when the record has draft changes applied to it
// TODO [W-8237087]: explore if this selection can only be added in environments where drafts are enabled
const DRAFTS_SELECTION = {
    kind: 'Object',
    opaque: true,
    name: 'drafts',
    required: false,
};
function isSpanningRecord(fieldValue) {
    return fieldValue !== null && typeof fieldValue === 'object';
}
function createRecordSelection(fieldDefinition) {
    const sel = dynamicSelect$8({
        childRelationships: CHILD_RELATIONSHIP_SELECTION,
        fields: createPathSelection('fields', fieldDefinition),
    });
    return [...sel.selections, DRAFTS_SELECTION];
}
/**
 * Convert a list of fields and optional fields into RecordRepresentation its equivalent
 * selection.
 */
function buildSelectionFromFields(fields, optionalFields = []) {
    return createRecordSelection(convertRecordFieldsArrayToTrie(fields, optionalFields));
}
/**
 * Convert a RecordRepresentationLike into its equivalent selection.
 */
function buildSelectionFromRecord(record) {
    const sel = dynamicSelect$8({
        childRelationships: CHILD_RELATIONSHIP_SELECTION,
        fields: createPathSelectionFromValue(record.fields),
    });
    return [...sel.selections, DRAFTS_SELECTION];
}

const CUSTOM_API_NAME_SUFFIX = '__c';
const DMO_API_NAME_SUFFIX = '__dlm';
const CUSTOM_EXTERNAL_OBJECT_FIELD_SUFFIX = '__x';
const RECORD_REPRESENTATION_ERROR_VERSION = 'RECORD_REPRESENTATION_ERROR_VERSION_1';
const RECORD_REPRESENTATION_ERROR_STORE_METADATA_PARAMS = {
    representationName: '',
    namespace: keyPrefix,
    ttl: TTL$F,
    version: RECORD_REPRESENTATION_ERROR_VERSION,
};
function isGraphNode(node) {
    return !!node && node.type === 'Node';
}
function addScalarFieldId(current) {
    addScalarField(current, 'Id');
}
function addScalarFieldName(current) {
    addScalarField(current, 'Name');
}
function addScalarField(current, leafNodeFieldKey) {
    if (current.children[leafNodeFieldKey] === undefined) {
        current.children[leafNodeFieldKey] = {
            name: leafNodeFieldKey,
            children: {},
        };
    }
}
function extractTrackedFieldsToTrie(recordId, node, root, config, visitedRecordIds = {}, depth = 0) {
    // Filter Error and null nodes
    if (!isGraphNode(node)) {
        return;
    }
    // Stop the traversal if the key has already been visited, since the fields for this record
    // have already been gathered at this point.
    if (hasOwnProperty.call(visitedRecordIds, recordId)) {
        return;
    }
    // The visitedRecordIds object passed to the spanning record is a copy of the original
    // visitedRecordIds + the current record id, since we want to detect circular references within
    // a given path.
    let spanningVisitedRecordIds = {
        ...visitedRecordIds,
        [recordId]: true,
    };
    const fields = node.object('fields');
    const keys$1 = fields.keys();
    let current = root;
    for (let i = 0, len = keys$1.length; i < len; i += 1) {
        const key = keys$1[i];
        const fieldValueRep = fields.link(key);
        let next = current.children[key];
        if (next === undefined) {
            next = {
                name: key,
                children: {},
            };
            if (fieldValueRep.isMissing()) {
                current.children[key] = next;
                continue;
            }
            const field = fieldValueRep.follow();
            // Filter Error and null nodes
            if (!isGraphNode(field)) {
                continue;
            }
            const { maxDepth, onlyFetchLeafNodeIdAndName } = config;
            if (field.isScalar('value') === false) {
                if (depth + 1 > maxDepth) {
                    if (onlyFetchLeafNodeIdAndName === true) {
                        addScalarFieldId(current);
                        addScalarFieldName(current);
                    }
                    continue;
                }
                const spanningLink = field.link('value');
                const spanning = spanningLink.follow();
                // W-8058425, do not include external lookups added by getTrackedFields
                if (isExternalLookupFieldKey(spanning)) {
                    // NOTE: the logic to get here is mimicked in RecordRepresentation/merge::mergePendingFields as of
                    // W-11899329 due to issues with external lookups being marked pending but never fetched
                    continue;
                }
                extractTrackedFieldsToTrie(spanningLink.data.__ref, spanning, next, config, spanningVisitedRecordIds, depth + 1);
                // For a spanning record that is detected to be a circular reference, we add the field along with Id and Name.
                // It's possible for spanning record lookup fields to sometimes be circular, and sometimes not - depending on the value of the lookup field.
                // For more information on scenarios that caused this fix: search "LDS Recursive Spanning Fields Problem" in Quip
                if (keys(next.children).length === 0) {
                    addScalarFieldId(next);
                    addScalarFieldName(next);
                }
                current.children[key] = next;
            }
            else {
                // Skip the field, if its value is null at the max level depth.
                // Ideally, it should only skip relationship field. However,
                // on the client, there is not a reliable way to determine the
                // the field type.
                if (depth === maxDepth) {
                    if (onlyFetchLeafNodeIdAndName === true) {
                        addScalarFieldId(current);
                        addScalarFieldName(current);
                        continue;
                    }
                    if (field.scalar('value') === null) {
                        continue;
                    }
                }
                const state = fieldValueRep.linkData();
                if (state !== undefined) {
                    const { fields } = state;
                    // W-8058425, do not include external lookups added by getTrackedFields
                    if (includes.call(fields, 'ExternalId')) {
                        // NOTE: the logic to get here is mimicked in RecordRepresentation/merge::mergePendingFields as of
                        // W-11899329 due to issues with external lookups being marked pending but never fetched
                        continue;
                    }
                    for (let s = 0, len = fields.length; s < len; s += 1) {
                        const childFieldName = fields[s];
                        next.children[childFieldName] = {
                            name: childFieldName,
                            children: {},
                        };
                    }
                }
            }
            current.children[key] = next;
        }
    }
}
function isExternalLookupFieldKey(spanningNode) {
    // The only way to know if a record is an external lookup is to look into its fields
    // list and find an "ExternalId" field.
    // Filter Error and null nodes
    if (!isGraphNode(spanningNode) || !spanningNode.isScalar('apiName')) {
        return false;
    }
    return endsWith.call(spanningNode.scalar('apiName'), CUSTOM_EXTERNAL_OBJECT_FIELD_SUFFIX);
}
function convertTrieToFields(root) {
    if (keys(root.children).length === 0) {
        return [];
    }
    return convertTrieToFieldsRecursively(root);
}
function convertTrieToFieldsRecursively(root) {
    const childKeys = keys(root.children);
    if (childKeys.length === 0) {
        if (root.name === '') {
            return [];
        }
        return [root.name];
    }
    return reduce.call(childKeys, (acc, cur) => concat.call(acc, convertTrieToFieldsRecursively(root.children[cur]).map((i) => `${root.name}.${i}`)), []);
}
const BLANK_RECORD_FIELDS_TRIE = freeze({
    name: '',
    children: {},
});
const convertFieldsToTrie = (fields = [], isOptional = false, objectName // This should be a required field.
) => {
    if (fields.length === 0) {
        return BLANK_RECORD_FIELDS_TRIE;
    }
    const name = objectName || getObjectNameFromField(fields[0]); // Pulling object name from first field is not a safe or correct operation
    const fieldsTrie = {
        name,
        children: {},
        optional: isOptional,
    };
    const filteredFields = fields.filter((field) => objectName !== undefined && isOptional ? field.startsWith(`${objectName}.`) : true);
    insertFieldsIntoTrie(fieldsTrie, filteredFields, isOptional);
    return fieldsTrie;
};
const getObjectNameFromField = (field) => {
    const fieldApiName = getFieldApiName(field);
    if (fieldApiName === undefined) {
        return '';
    }
    return splitQualifiedFieldApiName(fieldApiName)[0];
};
// merge all nodes in Trie B into Trie A
function mergeFieldsTries(rootA, rootB) {
    const rootAchildren = rootA.children;
    const rootBchildren = rootB.children;
    const childBKeys = keys(rootBchildren);
    for (let i = 0, len = childBKeys.length; i < len; i++) {
        const childBKey = childBKeys[i];
        if (rootAchildren[childBKey] === undefined) {
            rootAchildren[childBKey] = rootBchildren[childBKey];
        }
        else {
            mergeFieldsTries(rootAchildren[childBKey], rootBchildren[childBKey]);
        }
    }
}
function getTrackedFields(key, graphNode, config, fieldsFromConfig) {
    const fieldsList = fieldsFromConfig === undefined ? [] : [...fieldsFromConfig];
    if (!isGraphNode(graphNode)) {
        return fieldsList;
    }
    const name = graphNode.scalar('apiName');
    const root = {
        name,
        children: {},
    };
    extractTrackedFieldsToTrie(key, graphNode, root, config);
    const rootFromConfig = {
        name,
        children: {},
    };
    insertFieldsIntoTrie(rootFromConfig, fieldsList);
    mergeFieldsTries(root, rootFromConfig);
    return convertTrieToFields(root).sort();
}
/**
 * Returns a new object that has a list of fields that has been filtered by
 * edited fields. Only contains fields that have been edited from their original
 * values (excluding Id which is always copied over).
 * @param input The RecordInputRepresentation object to filter.
 * @param original The Record object that contains the original field values.
 * @returns RecordInputRepresentation, see the description
 */
function createRecordInputFilteredByEditedFields(input, original) {
    const filteredRecordInput = getRecordInput();
    // Always copy over any existing id.
    if (original.id) {
        filteredRecordInput.fields.Id = original.id;
    }
    const recordInputFields = input.fields;
    const originalRecordFields = original.fields;
    const recordInputFieldPropertyNames = keys(recordInputFields);
    for (let i = 0, len = recordInputFieldPropertyNames.length; i < len; i++) {
        const fieldName = recordInputFieldPropertyNames[i];
        let originalRecordFieldsEntry;
        if (originalRecordFields) {
            originalRecordFieldsEntry = originalRecordFields[fieldName];
        }
        if (!originalRecordFieldsEntry ||
            (originalRecordFields &&
                recordInputFields[fieldName] !== originalRecordFieldsEntry.value)) {
            filteredRecordInput.fields[fieldName] = recordInputFields[fieldName];
        }
    }
    return filteredRecordInput;
}
/**
 * Returns an object with its data populated from the given record. All fields
 * with values that aren't nested records will be assigned. This object can be
 * used to create a record.
 * @param record The record that contains the source data.
 * @param objectInfo The ObjectInfo corresponding to the apiName on the record.
 *        If provided, only fields that are createable=true (excluding Id) will
 *        be assigned to the object return value.
 * @returns RecordInputRepresentation See description.
 */
function generateRecordInputForCreate(record, objectInfo) {
    const recordInput = _generateRecordInput(record, (field) => field.createable === true, objectInfo);
    recordInput.apiName = record.apiName;
    // fields.Id is not required for CREATE which might have been copied over,
    // so delete fields.Id
    delete recordInput.fields.Id;
    return recordInput;
}
/**
 * Returns an object with its data populated from the given record. All fields
 * with values that aren't nested records will be assigned. This object can be
 * used to update a record.
 * @param record The record that contains the source data.
 * @param objectInfo The ObjectInfo corresponding to the apiName on the record.
 *        If provided, only fields that are updateable=true (excluding Id) will
 *        be assigned to the object return value.
 * @returns RecordInputRepresentation See description.
 */
function generateRecordInputForUpdate(record, objectInfo) {
    const recordInput = _generateRecordInput(record, (field) => field.updateable === true, objectInfo);
    if (!record.id) {
        if (process.env.NODE_ENV !== 'production') {
            throw new Error('record must have id for update');
        }
    }
    // Always copy over any existing id.
    recordInput.fields.Id = record.id;
    return recordInput;
}
function isRecordInputFieldValue(unknown) {
    const type = typeof unknown;
    return unknown === null || type === 'string' || type === 'number' || type === 'boolean';
}
/**
 * Returns an object with its data populated from the given record. All fields
 * with values that aren't nested records will be assigned.
 * @param record The record that contains the source data.
 * @param copyFieldPredicate predicate to determine if a field should be copied.
 *        Required if "objectInfo" parameter is passed.
 * @param objectInfo The ObjectInfo corresponding to the apiName on the record.
 *        If provided, only fields that match the copyFieldPredicate (excluding
 *        Id) will be assigned to the object return value.
 * @returns RecordInputRepresentation
 */
function _generateRecordInput(record, copyFieldPredicate, objectInfo) {
    const recordInput = getRecordInput();
    const recordFields = record.fields;
    let objectInfoFields;
    if (objectInfo) {
        objectInfoFields = objectInfo.fields;
    }
    const recordFieldPropertyNames = keys(recordFields);
    for (let i = 0, len = recordFieldPropertyNames.length; i < len; i++) {
        const fieldName = recordFieldPropertyNames[i];
        const recordFieldsFieldNameEntry = recordFields[fieldName].value;
        if (isRecordInputFieldValue(recordFieldsFieldNameEntry)) {
            if (objectInfoFields && copyFieldPredicate) {
                const objectInfoFieldsFieldNameValue = objectInfoFields[fieldName];
                if (objectInfoFieldsFieldNameValue &&
                    copyFieldPredicate(objectInfoFieldsFieldNameValue)) {
                    recordInput.fields[fieldName] = recordFieldsFieldNameEntry;
                }
            }
            else {
                recordInput.fields[fieldName] = recordFieldsFieldNameEntry;
            }
        }
    }
    return recordInput;
}
/**
 * Gets a new Record Input.
 */
function getRecordInput() {
    return {
        apiName: undefined,
        fields: {},
    };
}
/**
 * Gets a field's value from a record.
 * @param record The record.
 * @param field The qualified API name of the field to return.
 * @returns The field's value (which may be a record in the case of spanning
 *          fields), or undefined if the field isn't found.
 */
function getFieldValue(record, field) {
    const fieldValueRepresentation = getField(record, field);
    if (fieldValueRepresentation === undefined) {
        return undefined;
    }
    if (isFieldValueRepresentation(fieldValueRepresentation)) {
        return fieldValueRepresentation.value;
    }
    return fieldValueRepresentation;
}
/**
 * Gets a field's display value from a record.
 * @param record The record.
 * @param field The qualified API name of the field to return.
 * @returns The field's display value, or undefined if the field isn't found.
 */
function getFieldDisplayValue$1(record, field) {
    const fieldValueRepresentation = getField(record, field);
    if (fieldValueRepresentation === undefined) {
        return undefined;
    }
    if (isFieldValueRepresentation(fieldValueRepresentation)) {
        return fieldValueRepresentation.displayValue;
    }
    return fieldValueRepresentation;
}
function isFieldValueRepresentation(unknown) {
    if (typeof unknown !== 'object' || unknown === null) {
        return false;
    }
    return 'value' in unknown && 'displayValue' in unknown;
}
function getField(record, field) {
    const fieldApiName = getFieldApiName(field);
    if (fieldApiName === undefined) {
        return undefined;
    }
    const unqualifiedField = splitQualifiedFieldApiName(fieldApiName)[1];
    const fields = unqualifiedField.split('.');
    let r = record;
    while (r && r.fields) {
        const f = fields.shift();
        const fvr = r.fields[f];
        if (fvr === undefined) {
            return undefined;
        }
        else if (fields.length > 0) {
            r = fvr.value;
        }
        else {
            return fvr;
        }
    }
    return r;
}
function getRecordTypeId$2(record) {
    return record.recordTypeId === null ? MAIN_RECORD_TYPE_ID : record.recordTypeId;
}
// This function traverses through a record and marks missing
// optional fields as "missing"
function markMissingOptionalFields(record, optionalFields) {
    if (!isGraphNode(record)) {
        return;
    }
    const apiName = record.scalar('apiName');
    for (let a = 0, aLen = optionalFields.length; a < aLen; a++) {
        const parts = optionalFields[a].split('.');
        if (parts[0] === apiName) {
            _markMissingPath(record, parts.slice(1));
        }
    }
}
function markNulledOutPath(record, path) {
    if (!isGraphNode(record)) {
        return;
    }
    const fieldValueRepresentation = record.object('fields');
    const fieldName = path.shift();
    if (fieldValueRepresentation.isUndefined(fieldName)) {
        return;
    }
    const link = fieldValueRepresentation.link(fieldName);
    const resolved = link.follow();
    if (isGraphNode(resolved) &&
        resolved.isScalar('value') &&
        path.length > 0 && // TODO [W-14082782]: temporary fix
        !isFrozen(link.data)) {
        const linkState = link.linkData();
        const fields = linkState === undefined ? [] : linkState.fields;
        link.writeLinkData({
            fields: dedupe([...fields, path.join('.')]),
        });
    }
}
function markNulledOutRequiredFields(record, fields) {
    if (!isGraphNode(record)) {
        return;
    }
    const apiName = record.scalar('apiName');
    for (let a = 0, aLen = fields.length; a < aLen; a++) {
        const parts = fields[a].split('.');
        if (parts[0] === apiName) {
            markNulledOutPath(record, parts.slice(1));
        }
    }
}
function _markMissingPath(record, path) {
    // Filter out Error and null nodes
    if (!isGraphNode(record)) {
        return;
    }
    const fieldValueRepresentation = record.object('fields');
    const fieldName = path.shift();
    if (fieldValueRepresentation.isUndefined(fieldName) === true) {
        writeMissingFieldToStore(fieldValueRepresentation, fieldName);
        return;
    }
    const link = fieldValueRepresentation.link(fieldName);
    if (link.isPending()) {
        writeMissingFieldToStore(fieldValueRepresentation, fieldName);
    }
    else if (path.length > 0 && link.isMissing() === false) {
        const fieldValue = link.follow();
        // Filter out Error and null nodes
        if (!isGraphNode(fieldValue)) {
            return;
        }
        // if value is not a scalar, follow the link and mark it as missing
        if (fieldValue.isScalar('value') === false) {
            _markMissingPath(fieldValue
                .link('value')
                .follow(), path);
        }
    }
}
/**
 * Graph Node Directly modifies store entries, which is generally a non-starter.
 * Until we can refactor this mess, you need to use this function to safely mark the RecordRepresentation
 * as a seenId in the store when you perform this mutation.
 */
function writeMissingFieldToStore(field, fieldName) {
    // TODO [W-6900046]: remove cast, make RecordRepresentationNormalized['fields'] accept
    // an undefined/non-present __ref if isMissing is present
    field.write(fieldName, {
        __ref: undefined,
        isMissing: true,
    });
}
/**
 * Tells you if an objectApiName is supported by UI API or not.
 * Note: Luvio does not currently support all the entities, the list is limited to UI API supported entities
 * @param objectApiName the object API name from a record.
 * @return True if the provided objectApiName is supported in UI API.
 */
function isSupportedEntity(objectApiName) {
    return ((UIAPI_SUPPORTED_ENTITY_API_NAMES.has(objectApiName) ||
        endsWith.call(objectApiName, CUSTOM_API_NAME_SUFFIX) ||
        endsWith.call(objectApiName, DMO_API_NAME_SUFFIX)) &&
        !UIAPI_DENIED_ENTITY_API_NAMES.has(objectApiName));
}
/** Return true if a >= b */
function isSuperset(a, b) {
    if (b.length > a.length) {
        return false;
    }
    const aMap = {};
    // Put all keys from subset into a map
    // so we don't have to use subset.includes which will be slow
    for (let i = 0, len = a.length; i < len; i += 1) {
        aMap[a[i]] = true;
    }
    for (let i = 0, len = b.length; i < len; i += 1) {
        if (aMap[b[i]] === undefined) {
            return false;
        }
    }
    return true;
}
/** return true if a and b start with the same root name and a contains all nodes in b */
function isSuperRecordFieldTrie(a, b) {
    if (a.name !== b.name) {
        return false;
    }
    const childrenA = a.children;
    const childrenB = b.children;
    const childKeysA = keys(childrenA);
    const childKeysB = keys(childrenB);
    const childKeysBLength = childKeysB.length;
    if (childKeysBLength > childKeysA.length) {
        return false;
    }
    let ret = true;
    for (let i = 0; i < childKeysBLength; i++) {
        const childBKey = childKeysB[i];
        const childA = childrenA[childBKey];
        if (childA === undefined) {
            return false;
        }
        ret = ret && isSuperRecordFieldTrie(childA, childrenB[childBKey]);
    }
    return ret;
}
function ingestRecordResponse(luvio, response, recordId, recordIngest, conflictMap) {
    const { body } = response;
    const sel = buildSelectionFromRecord(body);
    const key = keyBuilder$3E(luvio, {
        recordId,
    });
    luvio.storeIngest(key, recordIngest, body);
    resolveConflict(luvio, conflictMap);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: {
            kind: 'Fragment',
            private: [],
            selections: sel,
            version: VERSION$2K,
        },
        variables: {},
    });
    return snapshot;
}

// This function sets fields that we are refreshing to pending
// These values will go into the store
function mergePendingFields(newRecord, oldRecord, existingNode) {
    const mergedFields = { ...newRecord.fields };
    const merged = { ...newRecord, fields: mergedFields };
    const existingFields = keys(oldRecord.fields);
    for (let i = 0, len = existingFields.length; i < len; i += 1) {
        const spanningFieldName = existingFields[i];
        if (newRecord.fields[spanningFieldName] === undefined) {
            /*
             * Per W-8058425 external lookups fields are excluded from the tracked fields. However, as covered in
             * W-11899329, situations can arise in which a merge conflict occurs when the existing record has a
             * reference to an external lookup field.  The exclusion ultimately results in a snapshot stuck in the
             * pending state. This is an approach to prevent that situation.
             *
             * The same logic checks for W-8058425 to "continue" as it relates to not tracking external lookups is
             * mimicked here as it relates to not marking them as pending.
             */
            // make sure external lookups are NOT marked as pending when `existingNode` is provided
            if (isGraphNode(existingNode)) {
                // get the node for the spanning field
                const fieldValueRep = existingNode
                    .object('fields')
                    .link(spanningFieldName);
                const field = fieldValueRep.follow();
                if (isGraphNode(field)) {
                    if (field.isScalar('value') === false) {
                        const record = field
                            .link('value')
                            .follow();
                        if (isExternalLookupFieldKey(record)) {
                            continue;
                        }
                    }
                }
                // If the fields requested off of this spanning record are external fields, skip them to avoid creating request that exceed the SOQL limits on external data. https://developer.salesforce.com/docs/atlas.en-us.soql_sosl.meta/soql_sosl/sforce_api_calls_soql_limits.htm
                const state = fieldValueRep.linkData();
                if (state !== undefined) {
                    const { fields } = state;
                    if (includes.call(fields, 'ExternalId')) {
                        continue;
                    }
                }
            }
            // TODO [W-6900046]: fix above casting issue so we're not stuffing arbitrary things
            // into RecordRepresentationNormalized['fields']
            mergedFields[spanningFieldName] = {
                __ref: undefined,
                pending: true,
            };
        }
    }
    return merged;
}
// This method gets called
// when incoming record has a higher version
// than the record that is currently in the store
function mergeAndRefreshHigherVersionRecord(incoming, existing, incomingTrackedFieldsTrieRoot, existingTrackedFieldsTrieRoot, recordConflictMap, existingNode) {
    // If the higher version (incoming) does not contain a superset of fields as existing
    // then we need to refresh to get updated versions of fields in existing
    if (isSuperRecordFieldTrie(incomingTrackedFieldsTrieRoot, existingTrackedFieldsTrieRoot) ===
        false) {
        // If this is an unsupported entity, do NOT attempt to go to the network
        // Simply merge what we have and move on
        if (isSupportedEntity(incoming.apiName) === false) {
            return mergeRecordFields(incoming, existing);
        }
        // update the conflict map to resolve the record conflict in resolveConflict
        recordConflictMap.conflicts[incoming.id] = {
            recordId: incoming.id,
            trackedFields: existingTrackedFieldsTrieRoot,
        };
        // We want to mark fields in the store as pending
        // Because we don't want to emit any data to components
        return mergePendingFields(incoming, existing, existingNode);
    }
    return incoming;
}
// This method gets called
// when incoming record has a lower version
// than the record that is currently in the store
function mergeAndRefreshLowerVersionRecord(luvio, incoming, existing, incomingTrackedFieldsTrieRoot, existingTrackedFieldsTrieRoot, recordConflictMap, existingNode) {
    // If the higher version (existing) does not have a superset of fields as incoming
    // then we need to refresh to get updated versions of fields on incoming
    if (isSuperRecordFieldTrie(existingTrackedFieldsTrieRoot, incomingTrackedFieldsTrieRoot) ===
        false) {
        // If this is an unsupported entity, do NOT attempt to go to the network
        // Simply merge what we have and move on
        if (isSupportedEntity(incoming.apiName) === false) {
            return mergeRecordFields(existing, incoming);
        }
        const merged = mergePendingFields(existing, incoming, existingNode);
        // update the conflict map to resolve the record conflict in resolveConflict
        if (recordConflictMap) {
            recordConflictMap.conflicts[incoming.id] = {
                recordId: incoming.id,
                trackedFields: incomingTrackedFieldsTrieRoot,
            };
        }
        else {
            buildNetworkSnapshot$1b(luvio, {
                recordId: incoming.id,
                optionalFields: convertTrieToFields(incomingTrackedFieldsTrieRoot),
            });
        }
        return merged;
    }
    return existing;
}
function mergeRecordConflict(luvio, incoming, existing, recordConflictMap) {
    const recordKey = keyBuilder$3E(luvio, {
        recordId: incoming.id,
    });
    const incomingNode = luvio.wrapNormalizedGraphNode(incoming, recordKey);
    const existingNode = luvio.wrapNormalizedGraphNode(existing, recordKey);
    const incomingTrackedFieldsTrieRoot = {
        name: incoming.apiName,
        children: {},
    };
    const existingTrackedFieldsTrieRoot = {
        name: existing.apiName,
        children: {},
    };
    const trackedFieldsConfig = {
        maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMergeConflict(),
        onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
    };
    extractTrackedFieldsToTrie(recordKey, incomingNode, incomingTrackedFieldsTrieRoot, trackedFieldsConfig);
    extractTrackedFieldsToTrie(recordKey, existingNode, existingTrackedFieldsTrieRoot, trackedFieldsConfig);
    if (incoming.weakEtag > existing.weakEtag) {
        return mergeAndRefreshHigherVersionRecord(incoming, existing, incomingTrackedFieldsTrieRoot, existingTrackedFieldsTrieRoot, recordConflictMap, existingNode);
    }
    return mergeAndRefreshLowerVersionRecord(luvio, incoming, existing, incomingTrackedFieldsTrieRoot, existingTrackedFieldsTrieRoot, recordConflictMap);
}
function getNotNull(recordAValue, recordBValue) {
    return recordAValue === null ? recordBValue : recordAValue;
}
function mergeRecordFields(recordA, recordB) {
    const lastModifiedDate = getNotNull(recordA.lastModifiedDate, recordB.lastModifiedDate);
    const lastModifiedById = getNotNull(recordA.lastModifiedById, recordB.lastModifiedById);
    const systemModstamp = getNotNull(recordA.systemModstamp, recordB.systemModstamp);
    return {
        ...recordA,
        fields: {
            ...recordB.fields,
            ...recordA.fields,
        },
        lastModifiedDate,
        lastModifiedById,
        systemModstamp,
    };
}
function isErrorEntry(entry) {
    return entry.__type === 'error';
}
function merge$3(existing, incoming, luvio, _path, recordConflictMap) {
    if (existing === undefined || isErrorEntry(existing)) {
        return incoming;
    }
    // recordTypeId may get changed based on record state.
    // Evicts all dependencies from store.
    if (incoming.recordTypeId !== existing.recordTypeId) {
        if (incoming.recordTypeId === null ||
            incoming.recordTypeId === undefined ||
            existing.recordTypeId === null ||
            existing.recordTypeId === undefined) {
            instrumentation.recordTypeIdIsNull(incoming.apiName);
        }
        const recordDepKey = dependencyKeyBuilder({ recordId: existing.id });
        const node = luvio.getNode(recordDepKey);
        if (isGraphNode(node)) {
            const dependencies = node.retrieve();
            if (dependencies !== null) {
                const depKeys = keys(dependencies);
                for (let i = 0, len = depKeys.length; i < len; i++) {
                    luvio.storeEvict(depKeys[i]);
                }
            }
        }
    }
    // eslint-disable-next-line @salesforce/lds/no-invalid-todo
    // TODO - handle merging of records that change apiName
    // if (existing.apiName !== incoming.apiName) {
    //     if (process.env.NODE_ENV === 'production') {
    //         lds.log(`API Name changed from ${existing.apiName} to ${incoming.apiName}`);
    //     } else {
    //         throw new Error('API Name cannot be different for merging records.');
    //     }
    // }
    // Adding instrumentation to see how frequently this occurs
    if (existing.apiName !== incoming.apiName) {
        instrumentation.recordApiNameChanged(incoming.apiName, existing.apiName);
    }
    const incomingWeakEtagZero = incoming.weakEtag === 0;
    const existingWeakEtagZero = existing.weakEtag === 0;
    if (incomingWeakEtagZero || existingWeakEtagZero) {
        instrumentation.weakEtagZero(incomingWeakEtagZero, existingWeakEtagZero, incoming.apiName);
    }
    // TODO [W-6900085]: UIAPI returns weakEtag=0 when the record is >2 levels nested. For now
    // we treat the record as mergeable.
    if (!incomingWeakEtagZero && !existingWeakEtagZero && incoming.weakEtag !== existing.weakEtag) {
        return mergeRecordConflict(luvio, incoming, existing, recordConflictMap);
    }
    return mergeRecordFields(incoming, existing);
}
function dependencyKeyBuilder(config) {
    return `UiApi::RecordRepresentationDependency:${config.recordId}`;
}

function merge$2(existing, incoming, _luvio, path) {
    if (existing === undefined) {
        return incoming;
    }
    fixDisplayValue(existing, incoming, path);
    const { value } = incoming;
    if (value === null || value.__ref === undefined) {
        // Parent will never be null this field only exists in the context of a RecordRep.
        const parent = path.parent;
        // It may happen that a parent.exists is null, this is the case when the same field is
        // ingested multiple times in the same ingestion cycle. For example: when the same record
        // is present multiple time in the ingested payload.
        if (parent.existing === undefined) {
            return incoming;
        }
        const existingVersion = parent.existing.weakEtag;
        const incomingVersion = parent.data.weakEtag;
        if (existingVersion > incomingVersion) {
            return existing;
        }
    }
    return incoming;
}
function fixDisplayValue(existing, incoming, path) {
    // Temporary fix for the issue that non-null displayValue gets replaced by null in mistake. (W-8904195)
    // If displayValue and value are both null, it means the field is empty.
    //
    // Couple sources may emit incorrect null displayValue to LDS:
    //   1. ADS may receive records with incorrect null displayValue from RecordGvp and emit to LDS.
    //      This case may affect certain types of fields:
    //      a. Spanning record field.
    //      b. Formattable fields, such as date, currency.
    //   2. AggregateUi endpoint. For some fields, displayValue only gets populated when certain combination
    //      of fields are requested. Currently, LDS handles large record requests with aggregateUi endpoint.
    //      On the network layer, record fields get split into separate chunks. Potentially, when certain
    //      combination of fields gets split into separate chunks, it would result in displayValue getting set
    //      to null.
    //   3. Any resource that returns nested Records, like list-records and list-ui, potentially exhibits this behavior
    //
    // There might be more cases masked by this workaround. We need to be careful when removing this code.
    // It would be required to have some telemetries in prod to verify if any code hits this workaround.
    const incomingValue = incoming.value;
    if (existing.displayValue !== null &&
        incoming.displayValue === null &&
        incomingValue !== null) {
        incoming.displayValue = existing.displayValue;
        const isSpanningRecord = typeof incomingValue === 'object';
        instrumentation.nullDisplayValueConflict({
            entityName: path.parent.data.apiName,
            fieldName: path.propertyName,
            fieldType: isSpanningRecord ? 'spanning-record' : 'scalar',
            areValuesEqual: isSpanningRecord ? undefined : incomingValue === existing.value,
        });
    }
}

function normalize$S(input, existing, path, luvio, store, timestamp, fieldsTrie, optionalFieldsTrie, recordConflictMap) {
    const input_value = input.value;
    const input_value_id = path.fullPath + '__value';
    if (input_value !== null && typeof input_value === 'object') {
        const ingest = createRecordIngest(fieldsTrie, optionalFieldsTrie, recordConflictMap);
        input.value = ingest(input_value, {
            fullPath: input_value_id,
            propertyName: 'value',
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
        }, luvio, store, timestamp);
    }
    return input;
}

function makeIngest(fieldsTrie, optionalFieldsTrie, recordConflictMap) {
    return (input, path, luvio, store, timestamp) => {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$20(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        const key = path.fullPath;
        let existingRecord = store.readEntry(key);
        let incomingRecord = normalize$S(input, store.readEntry(key), {
            fullPath: key,
            parent: path.parent,
            propertyName: path.propertyName,
        }, luvio, store, timestamp, fieldsTrie, optionalFieldsTrie, recordConflictMap);
        // read again after children ingested in case of a circular ref
        existingRecord = store.readEntry(key);
        incomingRecord = merge$2(existingRecord, incomingRecord, luvio, path);
        if (existingRecord === undefined || equals$1b(existingRecord, incomingRecord) === false) {
            luvio.storePublish(key, incomingRecord);
        }
        return createLink$1(key);
    };
}

/**
 * Adds fields listed in the two trie parameters into the store link.
 * Functionally analogous to markNulledOutRequiredFields.
 * @param fieldsTrie
 * @param optionalFieldsTrie
 * @param storeLink
 */
function addFieldsToStoreLink(fieldsTrie, optionalFieldsTrie, storeLink) {
    let fields = [];
    const fieldSubtries = [];
    if (fieldsTrie) {
        push.call(fieldSubtries, fieldsTrie);
    }
    if (optionalFieldsTrie) {
        push.call(fieldSubtries, optionalFieldsTrie);
    }
    for (let i = 0; i < fieldSubtries.length; i++) {
        const subtrie = fieldSubtries[i];
        const fieldNames = keys(subtrie.children);
        for (let i = 0; i < fieldNames.length; i++) {
            const fieldName = fieldNames[i];
            const childTrie = subtrie.children[fieldName];
            if (childTrie) {
                fields = [...fields, ...convertTrieToFieldsRecursively(childTrie)];
            }
        }
    }
    fields = dedupe(fields);
    if (fields.length > 0) {
        storeLink.data = {
            fields,
        };
    }
}

function getChildRecordFieldTrie(parent, key) {
    const value = parent.children[key];
    if (value === undefined) {
        return BLANK_RECORD_FIELDS_TRIE;
    }
    return value;
}
function createFieldsIngestion(fieldsTrie, optionalFieldsTrie, recordConflictMap) {
    return (data, path, luvio, store, timestamp) => {
        if (process.env.NODE_ENV !== 'production') {
            if (typeof path.propertyName !== 'string') {
                throw new Error('FieldValueRepresentation should always have a string propertyName value');
            }
        }
        const key = path.propertyName;
        const fieldsSubtrie = getChildRecordFieldTrie(fieldsTrie, key);
        const optionalFieldsSubtrie = getChildRecordFieldTrie(optionalFieldsTrie, key);
        const fieldValueLink = makeIngest(fieldsSubtrie, optionalFieldsSubtrie, recordConflictMap)(data, path, luvio, store, timestamp);
        // If a relationship field is null, memorize names of subordinate fields.
        // Spannning fields are specified in the dot notation.
        if (data.value === null) {
            addFieldsToStoreLink(fieldsSubtrie, optionalFieldsSubtrie, fieldValueLink);
        }
        return fieldValueLink;
    };
}
function createChildRecordNormalize(fieldsTrie, optionalFieldsTrie, recordConflictMap) {
    return dynamicNormalize$5({
        childRelationships: ingest$2h,
        fields: createFieldsIngestion(fieldsTrie, optionalFieldsTrie, recordConflictMap),
    });
}
const createRecordIngest = (fieldsTrie, optionalFieldsTrie, recordConflictMap) => {
    const childNormalize = createChildRecordNormalize(fieldsTrie, optionalFieldsTrie, recordConflictMap);
    return (input, path, luvio, store, timestamp) => {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$1_(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        input.eTag = ''; //[W-13724550] Record Reps ALWAYS have '' etag, to be consistent with GraphQL, as it is not queryable via GraphQL.
        const key = keyBuilderFromType$E(luvio, input);
        let existingRecord = store.readEntry(key);
        const recordPath = {
            fullPath: key,
            parent: path.parent,
            propertyName: path.propertyName,
        };
        let incomingRecord = childNormalize(input, existingRecord, recordPath, luvio, store, timestamp);
        // read from the store again since it might have been ingested as a nested child ref
        existingRecord = store.readEntry(key);
        incomingRecord = merge$3(existingRecord, incomingRecord, luvio, path, recordConflictMap);
        if (existingRecord === undefined ||
            equals$19(existingRecord, incomingRecord) === false ||
            // always publish if existing record has drafts
            existingRecord.drafts !==
                undefined) {
            luvio.storePublish(key, incomingRecord);
        }
        luvio.publishStoreMetadata(key, {
            ttl: TTL$F,
            representationName: RepresentationType$_,
            namespace: keyPrefix,
            version: VERSION$2K,
            ingestionTimestamp: timestamp,
        });
        return createLink$1(key);
    };
};

function createFieldsIngestSuccess$3(params) {
    const { fields, optionalFields, trackedFields, serverRequestCount = 1 } = params;
    const recordConflictMap = {
        conflicts: {},
        serverRequestCount,
    };
    const ingest = createRecordIngest(fields, optionalFields, recordConflictMap);
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        resolveConflict(luvio, recordConflictMap);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}

function fulfill(existing, incoming) {
    // early out if incoming isn't a request only for fields and optionalFields
    const { queryParams, headers, basePath, baseUri, urlParams } = incoming;
    const { basePath: existingBasePath, baseUri: existingBaseUri, headers: existingHeaders, urlParams: existingUrlParams, } = existing;
    const path = `${baseUri}${basePath}`;
    const existingPath = `${existingBaseUri}${existingBasePath}`;
    if (queryParams.layoutTypes !== undefined) {
        return false;
    }
    if (existingPath !== path) {
        // W-11964675 - Correlate an incoming single record request to an existing batch record
        // request requesting the same single record
        const incomingUrlRecords = getRecordIdsFromUrlParams(urlParams);
        const existingUrlRecords = getRecordIdsFromUrlParams(existingUrlParams);
        const batchRequestWithSingleRequest = isSingleBatchRecordRequest(existingUrlParams) &&
            isSingleRecordRequest(urlParams) &&
            incomingUrlRecords[0] === existingUrlRecords[0];
        if (!batchRequestWithSingleRequest || isRestrictedPathCondition(existingPath, path)) {
            return false;
        }
    }
    const headersKeys = keys(headers);
    const headersKeyLength = headersKeys.length;
    if (headersKeyLength !== keys(existingHeaders).length) {
        return false;
    }
    for (let i = 0, len = headersKeyLength; i < len; i++) {
        let key = headersKeys[i];
        if (headers[key] !== existingHeaders[key]) {
            return false;
        }
    }
    // TODO [W-6900100]: handle when incoming.fields are only in existing.optionalFields, and
    // existing's response doesn't include those fields. We need to detect this then
    // re-issue the request to get the relevant error response.
    const existingFieldsUnion = unionFields(existing.queryParams.fields, existing.queryParams.optionalFields);
    const incomingFieldsUnion = unionFields(queryParams.fields, queryParams.optionalFields);
    return isSuperset(existingFieldsUnion, incomingFieldsUnion);
}
function unionFields(fields, optionalFields) {
    const fieldsArray = isArray(fields) ? fields : [];
    const optionalFieldsArray = isArray(optionalFields) ? optionalFields : [];
    return [...fieldsArray, ...optionalFieldsArray];
}
function getRecordIdsFromUrlParams(urlParams) {
    if (hasOwnProperty.call(urlParams, 'recordId')) {
        return [urlParams.recordId];
    }
    else if (hasOwnProperty.call(urlParams, 'recordIds')) {
        return urlParams.recordIds;
    }
    return [];
}
function isSingleBatchRecordRequest(urlParams) {
    return (hasOwnProperty.call(urlParams, 'recordIds') &&
        urlParams.recordIds.length === 1);
}
function isSingleRecordRequest(urlParams) {
    return hasOwnProperty.call(urlParams, 'recordId');
}
function isRestrictedPathCondition(existingPath, path) {
    // should not dedupe getRecordUi and getRecord as both of their representation is different
    // records call cannot digest response of getRecordUi
    return ((existingPath.includes('/record-ui') && path.includes('/records')) ||
        (existingPath.includes('/records') && path.includes('/record-ui')));
}

const createResourceRequest$1h = function getUiApiRecordsByRecordIdCreateResourceRequest(config) {
    return {
        ...createResourceRequest$1g(config),
        fulfill: fulfill,
    };
};

function keyBuilder$3D(luvio, params) {
    return keyBuilder$3E(luvio, {
        recordId: params.urlParams.recordId
    });
}
function getResponseCacheKeys$17(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2l(storeKeyMap, luvio, response);
}
function createResourceRequest$1g(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/' + config.urlParams.recordId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$$ = 'getRecord';
const oneOfConfigPropertiesIdentifier$1 = [
    'layoutTypes',
    'fields',
    'optionalFields'
];
const getRecord_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('childRelationships', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('fields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('forms', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('layoutTypes', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArrayTitleCase),
    generateParamConfigMetadata('modes', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('updateMru', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
const createResourceParams$15 = /*#__PURE__*/ createResourceParams$k(getRecord_ConfigPropertyMetadata);
function typeCheckConfig$19(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecord_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$1c(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames, oneOfConfigPropertiesIdentifier$1);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecord_ConfigPropertyMetadata);
    const config = typeCheckConfig$19(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    if (config.layoutTypes === undefined &&
        config.fields === undefined &&
        config.optionalFields === undefined) {
        return null;
    }
    return config;
}

// used by getUiApiRecordsBatchByRecordIds#selectChildResourceParams
function buildRecordSelector(luvio, recordId, fields, optionalFields) {
    return {
        recordId: keyBuilder$3E(luvio, { recordId }),
        node: {
            kind: 'Fragment',
            private: ['eTag', 'weakEtag'],
            selections: buildSelectionFromFields(fields, optionalFields),
            version: VERSION$2K,
        },
        variables: {},
    };
}
function buildSnapshotRefresh$5(luvio, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot$1b(luvio, config),
    };
}
function prepareRequest$7(luvio, config) {
    const { recordId, fields } = config;
    // Should this go into the coersion logic?
    const key = keyBuilder$3D(luvio, createResourceParams$15(config));
    const allTrackedFields = getTrackedFields(key, luvio.getNode(key), {
        maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMiss(),
        onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
    }, config.optionalFields);
    const optionalFields = fields === undefined ? allTrackedFields : difference(allTrackedFields, fields);
    const resourceParams = createResourceParams$15({
        recordId,
        fields,
        optionalFields: optionalFields.length > 0 ? optionalFields : undefined,
    });
    const request = createResourceRequest$1h(resourceParams);
    return { request, key, allTrackedFields, resourceParams };
}
function ingestSuccess$X(luvio, config, key, allTrackedFields, response, serverRequestCount) {
    const { body } = response;
    const fields = config.fields === undefined ? [] : config.fields;
    const optionalFields = config.optionalFields === undefined ? [] : config.optionalFields;
    const fieldTrie = convertFieldsToTrie(fields, false);
    luvio.storeIngest(key, createFieldsIngestSuccess$3({
        fields: fieldTrie,
        optionalFields: convertFieldsToTrie(optionalFields, true),
        trackedFields: convertFieldsToTrie(allTrackedFields, true),
        serverRequestCount,
    }), body);
    return luvio.storeLookup(buildRecordSelector(luvio, config.recordId, fields, optionalFields), buildSnapshotRefresh$5(luvio, config));
}
function onResourceSuccess(luvio, config, key, allTrackedFields, response, serverRequestCount) {
    const snapshot = ingestSuccess$X(luvio, config, key, allTrackedFields, response, serverRequestCount);
    return luvio.storeBroadcast().then(() => snapshot);
}
function ingestError$R(luvio, config, key, err) {
    const errorSnapshot = luvio.errorSnapshot(err, buildSnapshotRefresh$5(luvio, config));
    luvio.storeIngestError(key, errorSnapshot, RECORD_REPRESENTATION_ERROR_STORE_METADATA_PARAMS);
    return errorSnapshot;
}
function onResourceError(luvio, config, key, err) {
    const errorSnapshot = ingestError$R(luvio, config, key, err);
    return luvio.storeBroadcast().then(() => errorSnapshot);
}
function buildNetworkSnapshot$1b(luvio, config, serverRequestCount = 0, options) {
    const { request, key, allTrackedFields, resourceParams } = prepareRequest$7(luvio, config);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        // W-11964675 - Condition to dedupe a very specific set of requests for
        // Komaci - a batch record request with a single record followed by a single
        // record request. The fulfill logic sends the same network response, so
        // there is some TypeScript massaging to extract the RecordRepresentation
        //
        // W-14381091 - Ensure hoisting the response body happens prior to
        // calling `luvio.handleSuccessResponse`, since both arguments capture
        // the response.
        if (isSingleBatchRecordResponse(response.body)) {
            response.body = response.body.results[0]
                .result;
        }
        return luvio.handleSuccessResponse(() => {
            return onResourceSuccess(luvio, config, key, allTrackedFields, response, serverRequestCount + 1);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$17(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return onResourceError(luvio, config, key, err);
        });
    });
}
// used by getRecordLayoutType#refresh
function buildCachedSnapshot$9(luvio, config, refresh) {
    const fields = config.fields === undefined ? [] : config.fields;
    const optionalFields = config.optionalFields === undefined ? [] : config.optionalFields;
    const sel = buildRecordSelector(luvio, config.recordId, fields, optionalFields);
    return luvio.storeLookup(sel, refresh ? refresh : buildSnapshotRefresh$5(luvio, config));
}
function buildCachedSnapshotCachePolicy$V(context, storeLookup) {
    const { config, luvio } = context;
    const fields = config.fields === undefined ? [] : config.fields;
    const optionalFields = config.optionalFields === undefined ? [] : config.optionalFields;
    const sel = buildRecordSelector(luvio, config.recordId, fields, optionalFields);
    return storeLookup(sel, buildSnapshotRefresh$5(luvio, config));
}
function buildNetworkSnapshotCachePolicy$W(context, coercedAdapterRequestContext) {
    const { config, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$1b(luvio, config, 0, dispatchOptions);
}
function getRecordByFields(luvio, config, requestContext) {
    return luvio.applyCachePolicy(requestContext || {}, { config, luvio }, buildCachedSnapshotCachePolicy$V, buildNetworkSnapshotCachePolicy$W);
}
function isSingleBatchRecordResponse(response) {
    return (isArray(response.results) &&
        response.results.length === 1);
}

const VERSION$2J = "98cce53b8d13b1883d001bbdaab24383";

const nonCachedErrors$8 = ObjectCreate$1(null);
function ingestSuccessChildResourceParams$9(luvio, childResourceParamsArray, childEnvelopes, serverRequestCount = 0) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$3D(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childTrackedFields = getTrackedFields(childKey, luvio.getNode(childKey), {
                maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMiss(),
                onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
            }, childResourceParams.queryParams.optionalFields);
            const childSnapshot = ingestSuccess$X(luvio, {
                recordId: childResourceParams.urlParams.recordId,
                fields: childResourceParams.queryParams.fields,
                optionalFields: childResourceParams.queryParams.optionalFields,
            }, childKey, childTrackedFields, childResponse, serverRequestCount + 1);
            if (childSnapshot.state === 'Stale') {
                snapshotState = 'Stale';
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: childStatusCode,
                result: childSnapshot.data,
            };
            childSnapshotDataResponses.push(childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(childStatusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse',
            };
            ingestError$R(luvio, {
                recordId: childResourceParams.urlParams.recordId,
                fields: childResourceParams.queryParams.fields,
                optionalFields: childResourceParams.queryParams.optionalFields,
            }, childKey, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            childSnapshotDataResponses.push(childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$8[childKey] = {
                expiration: now + TTL$F,
                response: childBody,
                status: childStatusCode,
            };
        }
        else {
            delete nonCachedErrors$8[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses,
    };
    deepFreeze(childSnapshotData);
    return {
        childSnapshotData,
        seenRecords,
        snapshotState,
    };
}

function selectChildResourceParams$8(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const compositeSnapshot = luvio.storeLookup({
                recordId: keyBuilder$3C(luvio, resourceParams),
                node: {
                    kind: 'Fragment',
                    private: [],
                    version: VERSION$2J,
                },
                variables: {},
            });
            if (compositeSnapshot.state === 'Error' &&
                compositeSnapshot.error.errorType === 'fetchResponse' &&
                compositeSnapshot.error.status === 404) {
                return {
                    state: compositeSnapshot.state,
                    value: compositeSnapshot.error,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const timestamp = reader.getTimeStamp();
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$3D(luvio, childResource);
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read(buildRecordSelector(luvio, childResource.urlParams.recordId, childResource.queryParams.fields || [], childResource.queryParams.optionalFields || []));
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    // eslint-disable-next-line no-fallthrough
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 
                        // child snapshots will always have FetchResponses
                        childSnapshot.error.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, 
                        // child snapshots will always have FetchResponses
                        childSnapshot.error.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$8[childKey];
                            if (nonCachedError === undefined ||
                                nonCachedError.expiration < timestamp) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            deepFreeze(sink);
            reader.exitPath();
            return sink;
        },
    };
}

ObjectCreate$1(null);
function createChildResourceParams$c(resourceParams) {
    let childConfigs = [];
    for (let index = 0, len = resourceParams.urlParams.recordIds.length; index < len; index += 1) {
        const item = resourceParams.urlParams.recordIds[index];
        childConfigs.push({
            urlParams: {
                recordId: item
            },
            queryParams: {
                fields: resourceParams.queryParams.fields,
                optionalFields: resourceParams.queryParams.optionalFields
            }
        });
    }
    return childConfigs;
}
function keyBuilder$3C(luvio, params) {
    return keyPrefix + '::BatchRepresentation:(' + 'childRelationships:' + params.queryParams.childRelationships + ',' + 'fields:' + params.queryParams.fields + ',' + 'layoutTypes:' + params.queryParams.layoutTypes + ',' + 'modes:' + params.queryParams.modes + ',' + 'optionalFields:' + params.queryParams.optionalFields + ',' + 'pageSize:' + params.queryParams.pageSize + ',' + 'updateMru:' + params.queryParams.updateMru + ',' + 'recordIds:' + params.urlParams.recordIds + ')';
}
function getResponseCacheKeys$16(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$c(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$17(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$3D(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$_,
                mergeable: false
            });
        }
    }
}
function ingestError$Q(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$3C(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$1f(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/batch/' + config.urlParams.recordIds + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

function createResourceParams$14(config) {
    const { records: configRecords } = config;
    const recordIds = [];
    const resourceConfigFields = [];
    const resourceConfigOptionalFields = [];
    for (let index = 0, len = configRecords.length; index < len; index += 1) {
        const { recordIds: recordIdsFromConfig, fields = [], optionalFields = [], } = configRecords[index];
        ArrayPrototypePush.call(recordIds, ...recordIdsFromConfig);
        if (fields.length > 0) {
            ArrayPrototypePush.call(resourceConfigFields, ...fields);
        }
        if (optionalFields.length > 0) {
            ArrayPrototypePush.call(resourceConfigOptionalFields, ...optionalFields);
        }
    }
    return {
        urlParams: {
            recordIds,
        },
        queryParams: {
            fields: resourceConfigFields.length > 0 ? resourceConfigFields : undefined,
            optionalFields: resourceConfigOptionalFields.length > 0 ? resourceConfigOptionalFields : undefined,
        },
    };
}

function createChildResourceParams$b(config) {
    const childResources = [];
    const { records } = config;
    for (let outerIdx = 0, numOfRecordBatches = records.length; outerIdx < numOfRecordBatches; outerIdx += 1) {
        const currentRecordBatch = records[outerIdx];
        const { recordIds, fields, optionalFields } = currentRecordBatch;
        for (let innerIdx = 0, numOfRecordIds = recordIds.length; innerIdx < numOfRecordIds; innerIdx += 1) {
            const currentRecordId = recordIds[innerIdx];
            const queryParams = {};
            if (ArrayIsArray$1(fields)) {
                queryParams.fields = fields;
            }
            if (ArrayIsArray$1(optionalFields)) {
                queryParams.optionalFields = optionalFields;
            }
            childResources.push({
                urlParams: {
                    recordId: currentRecordId,
                },
                queryParams,
            });
        }
    }
    return childResources;
}

function adapterFragment$O(luvio, config) {
    const childResources = createChildResourceParams$b(config);
    const resourceParams = createResourceParams$14(config);
    return selectChildResourceParams$8(luvio, childResources, resourceParams);
}

function onFetchResponseSuccess$S(luvio, config, resourceParams, response, serverRequestCount = 0) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$b(config);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' +
                childResourceParamsArray.length +
                ' items, received ' +
                childEnvelopes.length);
        }
    }
    const snapshotStateFulfilled = 'Fulfilled';
    const key = keyBuilder$3C(luvio, resourceParams);
    const { childSnapshotData, seenRecords } = ingestSuccessChildResourceParams$9(luvio, childResourceParamsArray, childEnvelopes, serverRequestCount);
    const snapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotStateFulfilled,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: adapterFragment$O(luvio, config),
            variables: {},
        },
        refresh: {
            config,
            resolve: () => buildNetworkSnapshot$1a(luvio, config, serverRequestCount, snapshotRefreshOptions),
        },
        variables: {},
    };
    return luvio.storeBroadcast().then(() => snapshot);
}

function onFetchResponseError$T(luvio, config, resourceParams, response) {
    const snapshot = ingestError$Q(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$1a(luvio, config, 0, snapshotRefreshOptions),
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$1a(luvio, config, serverRequestCount = 0, options) {
    const resourceParams = createResourceParams$14(config);
    const request = createResourceRequest$1f(resourceParams);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$S(luvio, config, resourceParams, response, serverRequestCount + 1), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$16(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$T(luvio, config, resourceParams, response));
    });
}

// iterate through the map to build configs for network calls
function resolveConflict(luvio, map) {
    const ids = keys(map.conflicts);
    if (ids.length === 0) {
        instrumentation.recordConflictsResolved(map.serverRequestCount);
        return;
    }
    if (ids.length === 1) {
        const recordId = ids[0];
        const config = {
            recordId,
            optionalFields: convertTrieToFields(map.conflicts[recordId].trackedFields),
        };
        buildNetworkSnapshot$1b(luvio, config, map.serverRequestCount);
    }
    else {
        const records = reduce.call(ids, (acc, id) => {
            const { trackedFields } = map.conflicts[id];
            push.call(acc, {
                recordIds: [id],
                optionalFields: convertTrieToFields(trackedFields),
            });
            return acc;
        }, []);
        const config = { records };
        buildNetworkSnapshot$1a(luvio, config, map.serverRequestCount);
    }
}

const createIngestRecordWithFields = (fields, optionalFields) => {
    return (input, path, luvio, store, timestamp) => {
        input.eTag = ''; //[W-13724550] Record Reps ALWAYS have '' etag, to be consistent with GraphQL, as it is not queryable via GraphQL.
        const conflictMap = {
            conflicts: {},
            serverRequestCount: 0,
        };
        const result = createRecordIngest(fields, optionalFields, conflictMap)(input, path, luvio, store, timestamp);
        resolveConflict(luvio, conflictMap);
        return result;
    };
};
const ingest$2g = (input, path, luvio, store, timestamp) => {
    const conflictMap = {
        conflicts: {},
        serverRequestCount: 0,
    };
    input.eTag = ''; //[W-13724550] Record Reps ALWAYS have '' etag, to be consistent with GraphQL, as it is not queryable via GraphQL.
    const result = createRecordIngest(BLANK_RECORD_FIELDS_TRIE, BLANK_RECORD_FIELDS_TRIE, conflictMap)(input, path, luvio, store, timestamp);
    resolveConflict(luvio, conflictMap);
    return result;
};

const PAGINATION_VERSION$1 = "0.111.0";
const paginationStoreMetadata$1 = {
    ttl: Number.MAX_SAFE_INTEGER,
    namespace: '',
    representationName: '',
    version: PAGINATION_VERSION$1
};
function createPaginationMetadata(startOffset, edges) {
    const paginationMetadata = {};
    for (let i = 0; i < edges.length; i++) {
        const cursor = edges[i].cursor;
        Object.assign(paginationMetadata, {
            [cursor]: startOffset + i,
        });
    }
    return paginationMetadata;
}
function createPageInfo(edges, startOffset, endOffset, maxOffset, metadata) {
    if (edges === undefined) {
        return undefined;
    }
    const listIsIncomplete = ((metadata === null || metadata === void 0 ? void 0 : metadata.__END__) === undefined);
    const hasNextPage = listIsIncomplete || (endOffset !== undefined && maxOffset !== undefined && (endOffset - 1 < maxOffset));
    const hasPreviousPage = startOffset !== undefined && startOffset > 0;
    let startCursor = null;
    let endCursor = null;
    if (edges.length && metadata !== undefined) {
        const startOffsetIsDefined = startOffset !== undefined;
        const endOffsetIsDefined = endOffset !== undefined;
        Object.keys(metadata)
            .forEach((token) => {
            const index = metadata[token];
            if (startOffsetIsDefined && index === startOffset) {
                startCursor = token;
            }
            if (endOffsetIsDefined && index === endOffset - 1) {
                endCursor = token;
            }
        });
    }
    return {
        startCursor,
        endCursor,
        hasNextPage,
        hasPreviousPage,
        __typename: 'PageInfo',
    };
}
function findMaxOffset(metadata) {
    if (metadata === undefined)
        return -1;
    const endOfList = metadata["__END__"];
    if (endOfList !== undefined) {
        return endOfList > 0 ? endOfList - 1 : 0; // Cursor based pagination in gql is off by one :)
    }
    // end is included in the above case.
    return Object.values(metadata)
        .reduce((acc, value) => Math.max(value !== null && value !== void 0 ? value : -1, acc), -1);
}
function getPageTokenAndOffset$1(paginationMetadata, token) {
    if (token === undefined) {
        const startToken = tokenFromOffset(paginationMetadata, 0);
        if (startToken === undefined) {
            return;
        }
        return [startToken, 0];
    }
    const startOffset = offsetFromToken(paginationMetadata, token);
    if (startOffset < 0) {
        return;
    }
    return [token, startOffset];
}
function hasGraphQLPaginationFields(data) {
    return (data !== null &&
        data !== undefined &&
        data.pageInfo !== undefined &&
        data.pageInfo !== null &&
        data.pageInfo.endCursor !== undefined &&
        data.pageInfo.startCursor !== undefined &&
        data.edges !== null &&
        data.edges !== undefined);
}
function mergeMetadata$1(existingMetadata, incomingMetadata) {
    if (existingMetadata === undefined) {
        return incomingMetadata;
    }
    return {
        ...existingMetadata,
        ...incomingMetadata,
    };
}
function getNextPageToken(paginationMetadata, pageEndOffset) {
    const listEndOffset = paginationMetadata.__END__;
    if (listEndOffset === undefined || pageEndOffset < listEndOffset) {
        return tokenFromOffset(paginationMetadata, pageEndOffset);
    }
    return null;
}
function offsetFromToken(paginationMetadata, token) {
    if (token === undefined || token === null) {
        return 0;
    }
    const offset = paginationMetadata[token];
    return offset === undefined ? -1 : offset;
}
function pageEndOffset$1(paginationMetadata, startOffset, pageSize) {
    const listEndOffset = paginationMetadata.__END__;
    const nextPageOffset = startOffset + pageSize;
    if (listEndOffset === undefined) {
        return nextPageOffset;
    }
    return Math.min(nextPageOffset, listEndOffset);
}
function getPreviousPageToken(paginationMetadata, startOffset, pageSize) {
    const previousPageOffset = startOffset - pageSize;
    if (previousPageOffset < 0) {
        return null;
    }
    const listEndOffset = paginationMetadata.__END__;
    if (listEndOffset !== undefined && previousPageOffset >= listEndOffset) {
        return;
    }
    return tokenFromOffset(paginationMetadata, previousPageOffset);
}
function tokenFromOffset(paginationMetadata, offset) {
    const tokens = Object.keys(paginationMetadata);
    for (let i = 0, len = tokens.length; i < len; i++) {
        const token = tokens[i];
        if (paginationMetadata[token] === offset &&
            token !== '__END__') {
            return token;
        }
    }
}
function tokenForAtMost(paginationMetadata, maxOffset) {
    const tokens = Object.keys(paginationMetadata);
    let tokenResult = undefined;
    let offsetResult = -1;
    for (let i = 0, len = tokens.length; i < len; i++) {
        const token = tokens[i];
        const offset = paginationMetadata[token];
        if (token === '__END__' || offset === undefined) {
            continue;
        }
        if (offset === maxOffset) {
            return [token, offset];
        }
        if (offset < maxOffset && offset > offsetResult) {
            tokenResult = token;
            offsetResult = offset;
        }
    }
    return [tokenResult, offsetResult];
}
function trimEdges(edges, startOffset, endOffset) {
    const trimmedEdges = [];
    if (edges !== undefined) {
        for (let i = startOffset; i < endOffset; i++) {
            if (edges[i] === undefined) {
                return undefined;
            }
            trimmedEdges[i - startOffset] = edges[i];
        }
    }
    return trimmedEdges;
}
function optimizePagination(metadata, firstArgName, afterArgName, variables) {
    if (typeof metadata === 'object' && metadata !== null) {
        const requestedAfterCursor = variables[afterArgName];
        const afterCursorOffset = offsetFromToken(metadata, requestedAfterCursor);
        // optimize only if after cursor is unspecified in the request or it's already loaded in metadata
        if (afterCursorOffset >= 0) {
            const requestedItems = variables[firstArgName];
            const atMostOffset = afterCursorOffset + requestedItems; // The largest offset we care about for this page.
            const lastTokenAndOffset = tokenForAtMost(metadata, atMostOffset);
            if (lastTokenAndOffset !== undefined) {
                const [token, offset] = lastTokenAndOffset;
                const isSubpage = atMostOffset === offset; // We already have all the cursors we seem to need
                const isExactPagination = atMostOffset === offset + requestedItems; // Using the last cursor we have to paginate
                if (!(isSubpage || isExactPagination)) { // If we're asking for more than we have, only request what we're missing.
                    const existingItems = afterCursorOffset === 0 ? offset + 1 : offset - afterCursorOffset;
                    const itemsToRequest = requestedItems - existingItems;
                    variables[firstArgName + '_original'] = requestedItems;
                    variables[firstArgName] = itemsToRequest;
                    variables[afterArgName + '_original'] = requestedAfterCursor;
                    variables[afterArgName] = token;
                }
            }
        }
    }
}
function revertPaginationOptimization(variables) {
    if (variables) {
        const toBeRemoved = [];
        for (const key in variables) {
            if (key.endsWith('_original')) {
                const origKey = key.substring(0, key.length - '_original'.length);
                if (Object.prototype.hasOwnProperty.call(variables, origKey)) {
                    variables[origKey] = variables[key];
                    toBeRemoved.push(key);
                }
            }
        }
        toBeRemoved.forEach((k) => delete variables[k]);
    }
}

const keyBuilderFromType$C = function ListRecordCollectionRepresentationKeyBuilderFromType(luvio, object) {
    const keyParams = {
        objectApiName: object.listReference.objectApiName.toLowerCase(),
        searchTerm: object.searchTerm,
        sortBy: object.sortBy,
        where: object.where,
        listViewApiName: object.listReference.listViewApiName
            ? object.listReference.listViewApiName.toLowerCase()
            : object.listReference.listViewApiName,
    };
    return keyBuilder$3B(luvio, keyParams);
};

const TTL$E = 30000;
const VERSION$2I = "e5c90c4081cd557f8ffec53028ede1e8";
function validate$1Z(obj, path = 'ListRecordCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_count = obj.count;
        const path_count = path + '.count';
        if (typeof obj_count !== 'number' || (typeof obj_count === 'number' && Math.floor(obj_count) !== obj_count)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_count + '" (at "' + path_count + '")');
        }
        const obj_currentPageToken = obj.currentPageToken;
        const path_currentPageToken = path + '.currentPageToken';
        if (typeof obj_currentPageToken !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
        }
        const obj_currentPageUrl = obj.currentPageUrl;
        const path_currentPageUrl = path + '.currentPageUrl';
        if (typeof obj_currentPageUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (!ArrayIsArray(obj_fields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        for (let i = 0; i < obj_fields.length; i++) {
            const obj_fields_item = obj_fields[i];
            const path_fields_item = path_fields + '[' + i + ']';
            if (typeof obj_fields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_fields_item + '" (at "' + path_fields_item + '")');
            }
        }
        const obj_listInfoETag = obj.listInfoETag;
        const path_listInfoETag = path + '.listInfoETag';
        let obj_listInfoETag_union0 = null;
        const obj_listInfoETag_union0_error = (() => {
            if (typeof obj_listInfoETag !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_listInfoETag + '" (at "' + path_listInfoETag + '")');
            }
        })();
        if (obj_listInfoETag_union0_error != null) {
            obj_listInfoETag_union0 = obj_listInfoETag_union0_error.message;
        }
        let obj_listInfoETag_union1 = null;
        const obj_listInfoETag_union1_error = (() => {
            if (obj_listInfoETag !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_listInfoETag + '" (at "' + path_listInfoETag + '")');
            }
        })();
        if (obj_listInfoETag_union1_error != null) {
            obj_listInfoETag_union1 = obj_listInfoETag_union1_error.message;
        }
        if (obj_listInfoETag_union0 && obj_listInfoETag_union1) {
            let message = 'Object doesn\'t match union (at "' + path_listInfoETag + '")';
            message += '\n' + obj_listInfoETag_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_listInfoETag_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_listReference = obj.listReference;
        const path_listReference = path + '.listReference';
        const referencepath_listReferenceValidationError = validate$2d(obj_listReference, path_listReference);
        if (referencepath_listReferenceValidationError !== null) {
            let message = 'Object doesn\'t match ListReferenceRepresentation (at "' + path_listReference + '")\n';
            message += referencepath_listReferenceValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageToken = obj.nextPageToken;
        const path_nextPageToken = path + '.nextPageToken';
        let obj_nextPageToken_union0 = null;
        const obj_nextPageToken_union0_error = (() => {
            if (typeof obj_nextPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union0_error != null) {
            obj_nextPageToken_union0 = obj_nextPageToken_union0_error.message;
        }
        let obj_nextPageToken_union1 = null;
        const obj_nextPageToken_union1_error = (() => {
            if (obj_nextPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union1_error != null) {
            obj_nextPageToken_union1 = obj_nextPageToken_union1_error.message;
        }
        if (obj_nextPageToken_union0 && obj_nextPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageToken + '")';
            message += '\n' + obj_nextPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageUrl = obj.nextPageUrl;
        const path_nextPageUrl = path + '.nextPageUrl';
        let obj_nextPageUrl_union0 = null;
        const obj_nextPageUrl_union0_error = (() => {
            if (typeof obj_nextPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union0_error != null) {
            obj_nextPageUrl_union0 = obj_nextPageUrl_union0_error.message;
        }
        let obj_nextPageUrl_union1 = null;
        const obj_nextPageUrl_union1_error = (() => {
            if (obj_nextPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union1_error != null) {
            obj_nextPageUrl_union1 = obj_nextPageUrl_union1_error.message;
        }
        if (obj_nextPageUrl_union0 && obj_nextPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageUrl + '")';
            message += '\n' + obj_nextPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_optionalFields = obj.optionalFields;
        const path_optionalFields = path + '.optionalFields';
        if (!ArrayIsArray(obj_optionalFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_optionalFields + '" (at "' + path_optionalFields + '")');
        }
        for (let i = 0; i < obj_optionalFields.length; i++) {
            const obj_optionalFields_item = obj_optionalFields[i];
            const path_optionalFields_item = path_optionalFields + '[' + i + ']';
            if (typeof obj_optionalFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_optionalFields_item + '" (at "' + path_optionalFields_item + '")');
            }
        }
        const obj_pageSize = obj.pageSize;
        const path_pageSize = path + '.pageSize';
        if (typeof obj_pageSize !== 'number' || (typeof obj_pageSize === 'number' && Math.floor(obj_pageSize) !== obj_pageSize)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_pageSize + '" (at "' + path_pageSize + '")');
        }
        const obj_previousPageToken = obj.previousPageToken;
        const path_previousPageToken = path + '.previousPageToken';
        let obj_previousPageToken_union0 = null;
        const obj_previousPageToken_union0_error = (() => {
            if (typeof obj_previousPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union0_error != null) {
            obj_previousPageToken_union0 = obj_previousPageToken_union0_error.message;
        }
        let obj_previousPageToken_union1 = null;
        const obj_previousPageToken_union1_error = (() => {
            if (obj_previousPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union1_error != null) {
            obj_previousPageToken_union1 = obj_previousPageToken_union1_error.message;
        }
        if (obj_previousPageToken_union0 && obj_previousPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageToken + '")';
            message += '\n' + obj_previousPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_previousPageUrl = obj.previousPageUrl;
        const path_previousPageUrl = path + '.previousPageUrl';
        let obj_previousPageUrl_union0 = null;
        const obj_previousPageUrl_union0_error = (() => {
            if (typeof obj_previousPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union0_error != null) {
            obj_previousPageUrl_union0 = obj_previousPageUrl_union0_error.message;
        }
        let obj_previousPageUrl_union1 = null;
        const obj_previousPageUrl_union1_error = (() => {
            if (obj_previousPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union1_error != null) {
            obj_previousPageUrl_union1 = obj_previousPageUrl_union1_error.message;
        }
        if (obj_previousPageUrl_union0 && obj_previousPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageUrl + '")';
            message += '\n' + obj_previousPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (!ArrayIsArray(obj_records)) {
            return new TypeError('Expected "array" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
        for (let i = 0; i < obj_records.length; i++) {
            const obj_records_item = obj_records[i];
            const path_records_item = path_records + '[' + i + ']';
            if (typeof obj_records_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_records_item + '" (at "' + path_records_item + '")');
            }
        }
        const obj_searchTerm = obj.searchTerm;
        const path_searchTerm = path + '.searchTerm';
        let obj_searchTerm_union0 = null;
        const obj_searchTerm_union0_error = (() => {
            if (typeof obj_searchTerm !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_searchTerm + '" (at "' + path_searchTerm + '")');
            }
        })();
        if (obj_searchTerm_union0_error != null) {
            obj_searchTerm_union0 = obj_searchTerm_union0_error.message;
        }
        let obj_searchTerm_union1 = null;
        const obj_searchTerm_union1_error = (() => {
            if (obj_searchTerm !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_searchTerm + '" (at "' + path_searchTerm + '")');
            }
        })();
        if (obj_searchTerm_union1_error != null) {
            obj_searchTerm_union1 = obj_searchTerm_union1_error.message;
        }
        if (obj_searchTerm_union0 && obj_searchTerm_union1) {
            let message = 'Object doesn\'t match union (at "' + path_searchTerm + '")';
            message += '\n' + obj_searchTerm_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_searchTerm_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_sortBy = obj.sortBy;
        const path_sortBy = path + '.sortBy';
        let obj_sortBy_union0 = null;
        const obj_sortBy_union0_error = (() => {
            if (!ArrayIsArray(obj_sortBy)) {
                return new TypeError('Expected "array" but received "' + typeof obj_sortBy + '" (at "' + path_sortBy + '")');
            }
            for (let i = 0; i < obj_sortBy.length; i++) {
                const obj_sortBy_item = obj_sortBy[i];
                const path_sortBy_item = path_sortBy + '[' + i + ']';
                if (typeof obj_sortBy_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_sortBy_item + '" (at "' + path_sortBy_item + '")');
                }
            }
        })();
        if (obj_sortBy_union0_error != null) {
            obj_sortBy_union0 = obj_sortBy_union0_error.message;
        }
        let obj_sortBy_union1 = null;
        const obj_sortBy_union1_error = (() => {
            if (obj_sortBy !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_sortBy + '" (at "' + path_sortBy + '")');
            }
        })();
        if (obj_sortBy_union1_error != null) {
            obj_sortBy_union1 = obj_sortBy_union1_error.message;
        }
        if (obj_sortBy_union0 && obj_sortBy_union1) {
            let message = 'Object doesn\'t match union (at "' + path_sortBy + '")';
            message += '\n' + obj_sortBy_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_sortBy_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_where = obj.where;
        const path_where = path + '.where';
        let obj_where_union0 = null;
        const obj_where_union0_error = (() => {
            if (typeof obj_where !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_where + '" (at "' + path_where + '")');
            }
        })();
        if (obj_where_union0_error != null) {
            obj_where_union0 = obj_where_union0_error.message;
        }
        let obj_where_union1 = null;
        const obj_where_union1_error = (() => {
            if (obj_where !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_where + '" (at "' + path_where + '")');
            }
        })();
        if (obj_where_union1_error != null) {
            obj_where_union1 = obj_where_union1_error.message;
        }
        if (obj_where_union0 && obj_where_union1) {
            let message = 'Object doesn\'t match union (at "' + path_where + '")';
            message += '\n' + obj_where_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_where_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$Z = 'ListRecordCollectionRepresentation';
function keyBuilder$3B(luvio, config) {
    return keyPrefix + '::' + RepresentationType$Z + ':' + config.objectApiName + ':' + (config.searchTerm === null ? '' : config.searchTerm) + ':' + (config.sortBy === null ? '' : '[' + config.sortBy.join(',') + ']') + ':' + (config.where === null ? '' : config.where) + ':' + (config.listViewApiName === null ? '' : config.listViewApiName);
}
function paginationKeyBuilder(luvio, config) {
    return keyBuilder$3B(luvio, config) + '__pagination';
}
function normalize$R(input, existing, path, luvio, store, timestamp) {
    const input_records = input.records;
    const input_records_id = path.fullPath + '__records';
    for (let i = 0; i < input_records.length; i++) {
        const input_records_item = input_records[i];
        let input_records_item_id = input_records_id + '__' + i;
        input_records[i] = ingest$2g(input_records_item, {
            fullPath: input_records_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$3o = function ListRecordCollectionRepresentationSelect(paginationParams) {
    const { selections: ListReferenceRepresentation__selections, opaque: ListReferenceRepresentation__opaque, } = select$3C();
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: false,
        version: VERSION$2I,
        read: (source, reader) => {
            const sink = {};
            if (paginationParams === undefined) {
                return sink;
            }
            const metadata = reader.resolveMetadata(source, PAGINATION_VERSION$1);
            const currentPageMetadata = getPageMetadata$4(metadata, paginationParams);
            const { startOffset, endOffset, metadataProperties } = currentPageMetadata;
            if (metadata !== undefined && startOffset !== undefined && endOffset !== undefined) {
                const itemsProperty = 'records';
                reader.enterPath(itemsProperty);
                const itemsSource = source[itemsProperty];
                const itemsSink = [];
                const allSink = [];
                for (let i = startOffset; i < endOffset; i++) {
                    reader.enterPath(i);
                    reader.readSingleLink(i, {
                        name: 'records',
                        kind: 'Link',
                        fragment: select$3p()
                    }, itemsSource, allSink, i - startOffset);
                    if (reader.getIsDataMissing() === false) {
                        itemsSink[i - startOffset] = allSink[i - startOffset];
                    }
                    reader.exitPath();
                }
                reader.exitPath();
                reader.assignNonScalar(sink, itemsProperty, itemsSink);
            }
            reader.enterPath('currentPageToken');
            reader.readScalar('currentPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('nextPageToken');
            reader.readScalar('nextPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('previousPageToken');
            reader.readScalar('previousPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('pageSize');
            reader.readScalar('pageSize', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('count');
            reader.readScalar('count', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('fields');
            reader.readScalarPlural('fields', source, sink, true);
            reader.exitPath();
            reader.enterPath('listInfoETag');
            reader.readScalar('listInfoETag', source, sink);
            reader.exitPath();
            reader.enterPath('listReference');
            reader.readObject('listReference', {
                name: 'listReference',
                kind: 'Object',
                selections: ListReferenceRepresentation__selections
            }, source, sink);
            reader.exitPath();
            reader.enterPath('optionalFields');
            reader.readScalarPlural('optionalFields', source, sink, true);
            reader.exitPath();
            reader.enterPath('searchTerm');
            reader.readScalar('searchTerm', source, sink);
            reader.exitPath();
            reader.enterPath('sortBy');
            if (source['sortBy'] === null) {
                reader.readScalar('sortBy', source, sink);
            }
            else {
                reader.readScalarPlural('sortBy', source, sink, true);
            }
            reader.exitPath();
            reader.enterPath('where');
            reader.readScalar('where', source, sink);
            reader.exitPath();
            return sink;
        },
    };
};
function getPageMetadata$4(paginationMetadata, paginationParams) {
    const metadataProperties = {};
    const sink = {
        metadataProperties
    };
    if (paginationMetadata === undefined) {
        return sink;
    }
    const { token, pageSize } = paginationParams;
    const pageTokenAndOffset = getPageTokenAndOffset$1(paginationMetadata, token);
    if (pageTokenAndOffset === undefined) {
        return sink;
    }
    const [pageToken, startOffset] = pageTokenAndOffset;
    const endOffset = pageEndOffset$1(paginationMetadata, startOffset, pageSize);
    sink.startOffset = startOffset;
    sink.endOffset = endOffset;
    metadataProperties.currentPageToken = pageToken;
    const nextToken = getNextPageToken(paginationMetadata, endOffset);
    if (nextToken !== undefined) {
        metadataProperties.nextPageToken = nextToken;
    }
    const previousToken = getPreviousPageToken(paginationMetadata, startOffset, pageSize);
    if (previousToken !== undefined) {
        metadataProperties.previousPageToken = previousToken;
    }
    metadataProperties.pageSize = pageSize;
    metadataProperties.count = endOffset - startOffset;
    return sink;
}
const dynamicSelect$7 = function dynamicListRecordCollectionRepresentationSelect(params, paginationParams) {
    const recordsPathSelection = params.records === undefined ? {
        name: 'records',
        kind: 'Link',
        fragment: select$3p()
    } : params.records;
    const { selections: ListReferenceRepresentation__selections, opaque: ListReferenceRepresentation__opaque, } = select$3C();
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: false,
        version: VERSION$2I,
        read: (source, reader) => {
            const sink = {};
            if (paginationParams === undefined) {
                return sink;
            }
            const metadata = reader.resolveMetadata(source, PAGINATION_VERSION$1);
            const currentPageMetadata = getPageMetadata$4(metadata, paginationParams);
            const { startOffset, endOffset, metadataProperties } = currentPageMetadata;
            if (metadata !== undefined && startOffset !== undefined && endOffset !== undefined) {
                const itemsProperty = 'records';
                reader.enterPath(itemsProperty);
                const itemsSource = source[itemsProperty];
                const itemsSink = [];
                const allSink = [];
                for (let i = startOffset; i < endOffset; i++) {
                    reader.enterPath(i);
                    reader.readSingleLink(i, recordsPathSelection, itemsSource, allSink, i - startOffset);
                    if (reader.getIsDataMissing() === false) {
                        itemsSink[i - startOffset] = allSink[i - startOffset];
                    }
                    reader.exitPath();
                }
                reader.exitPath();
                reader.assignNonScalar(sink, itemsProperty, itemsSink);
            }
            reader.enterPath('currentPageToken');
            reader.readScalar('currentPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('nextPageToken');
            reader.readScalar('nextPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('previousPageToken');
            reader.readScalar('previousPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('pageSize');
            reader.readScalar('pageSize', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('count');
            reader.readScalar('count', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('fields');
            reader.readScalarPlural('fields', source, sink, true);
            reader.exitPath();
            reader.enterPath('listInfoETag');
            reader.readScalar('listInfoETag', source, sink);
            reader.exitPath();
            reader.enterPath('listReference');
            reader.readObject('listReference', {
                name: 'listReference',
                kind: 'Object',
                selections: ListReferenceRepresentation__selections
            }, source, sink);
            reader.exitPath();
            reader.enterPath('optionalFields');
            reader.readScalarPlural('optionalFields', source, sink, true);
            reader.exitPath();
            reader.enterPath('searchTerm');
            reader.readScalar('searchTerm', source, sink);
            reader.exitPath();
            reader.enterPath('sortBy');
            if (source['sortBy'] === null) {
                reader.readScalar('sortBy', source, sink);
            }
            else {
                reader.readScalarPlural('sortBy', source, sink, true);
            }
            reader.exitPath();
            reader.enterPath('where');
            reader.readScalar('where', source, sink);
            reader.exitPath();
            return sink;
        },
    };
};
function equals$18(existing, incoming) {
    const existing_count = existing.count;
    const incoming_count = incoming.count;
    if (!(existing_count === incoming_count)) {
        return false;
    }
    const existing_pageSize = existing.pageSize;
    const incoming_pageSize = incoming.pageSize;
    if (!(existing_pageSize === incoming_pageSize)) {
        return false;
    }
    const existing_currentPageToken = existing.currentPageToken;
    const incoming_currentPageToken = incoming.currentPageToken;
    if (!(existing_currentPageToken === incoming_currentPageToken)) {
        return false;
    }
    const existing_currentPageUrl = existing.currentPageUrl;
    const incoming_currentPageUrl = incoming.currentPageUrl;
    if (!(existing_currentPageUrl === incoming_currentPageUrl)) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_items = equalsArray(existing_fields, incoming_fields, (existing_fields_item, incoming_fields_item) => {
        if (!(existing_fields_item === incoming_fields_item)) {
            return false;
        }
    });
    if (equals_fields_items === false) {
        return false;
    }
    const existing_listInfoETag = existing.listInfoETag;
    const incoming_listInfoETag = incoming.listInfoETag;
    if (!(existing_listInfoETag === incoming_listInfoETag)) {
        return false;
    }
    const existing_listReference = existing.listReference;
    const incoming_listReference = incoming.listReference;
    if (!(equals$1g(existing_listReference, incoming_listReference))) {
        return false;
    }
    const existing_nextPageToken = existing.nextPageToken;
    const incoming_nextPageToken = incoming.nextPageToken;
    if (!(existing_nextPageToken === incoming_nextPageToken)) {
        return false;
    }
    const existing_nextPageUrl = existing.nextPageUrl;
    const incoming_nextPageUrl = incoming.nextPageUrl;
    if (!(existing_nextPageUrl === incoming_nextPageUrl)) {
        return false;
    }
    const existing_optionalFields = existing.optionalFields;
    const incoming_optionalFields = incoming.optionalFields;
    const equals_optionalFields_items = equalsArray(existing_optionalFields, incoming_optionalFields, (existing_optionalFields_item, incoming_optionalFields_item) => {
        if (!(existing_optionalFields_item === incoming_optionalFields_item)) {
            return false;
        }
    });
    if (equals_optionalFields_items === false) {
        return false;
    }
    const existing_previousPageToken = existing.previousPageToken;
    const incoming_previousPageToken = incoming.previousPageToken;
    if (!(existing_previousPageToken === incoming_previousPageToken)) {
        return false;
    }
    const existing_previousPageUrl = existing.previousPageUrl;
    const incoming_previousPageUrl = incoming.previousPageUrl;
    if (!(existing_previousPageUrl === incoming_previousPageUrl)) {
        return false;
    }
    const existing_records = existing.records;
    const incoming_records = incoming.records;
    const equals_records_items = equalsArray(existing_records, incoming_records, (existing_records_item, incoming_records_item) => {
        if (!(existing_records_item.__ref === incoming_records_item.__ref)) {
            return false;
        }
    });
    if (equals_records_items === false) {
        return false;
    }
    const existing_searchTerm = existing.searchTerm;
    const incoming_searchTerm = incoming.searchTerm;
    if (!(existing_searchTerm === incoming_searchTerm)) {
        return false;
    }
    const existing_sortBy = existing.sortBy;
    const incoming_sortBy = incoming.sortBy;
    if (!((() => {
        if (existing_sortBy === null || incoming_sortBy === null) {
            return existing_sortBy === incoming_sortBy;
        }
        const equals_sortBy_items = equalsArray(existing_sortBy, incoming_sortBy, (existing_sortBy_item, incoming_sortBy_item) => {
            if (!(existing_sortBy_item === incoming_sortBy_item)) {
                return false;
            }
        });
        if (equals_sortBy_items === false) {
            return false;
        }
    })())) {
        return false;
    }
    const existing_where = existing.where;
    const incoming_where = incoming.where;
    if (!(existing_where === incoming_where)) {
        return false;
    }
    return true;
}
const ingest$2f = function ListRecordCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1Z(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$C(luvio, input);
    const existingRecord = store.readEntry(key);
    const ttlToUse = TTL$E;
    let incomingRecord = normalize$R(input, store.readEntry(key), {
        fullPath: key,
        parent: path.parent,
        propertyName: path.propertyName,
        ttl: ttlToUse
    }, luvio, store, timestamp);
    const paginationKey = key + '__pagination';
    const existingPaginationMetadata = store.readEntry(paginationKey);
    const { count: count, currentPageToken: token, nextPageToken: nextToken, pageSize: pageSize, previousPageToken: previousToken, records: items } = input;
    const startOffset = existingPaginationMetadata === undefined ? 0 : offsetFromToken(existingPaginationMetadata, token);
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0 ||
            (existingRecord && existingRecord.records && existingRecord.records.length < startOffset)) {
            throw new RangeError('currentPageToken value ' + token + ' is not recognized');
        }
    }
    const incomingPaginationMetadata = {
        [token]: startOffset,
    };
    const nextOffset = startOffset + items.length;
    if (nextToken === null) {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    else {
        incomingPaginationMetadata[nextToken] = nextOffset;
        if (existingPaginationMetadata !== undefined) {
            const listEndOffset = existingPaginationMetadata.__END__;
            if (listEndOffset !== undefined && nextOffset >= listEndOffset) {
                incomingPaginationMetadata.__END__ = undefined;
            }
        }
    }
    if (previousToken !== null) {
        incomingPaginationMetadata[previousToken] = Math.max(startOffset - pageSize, 0);
    }
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    if (existingPaginationMetadata === undefined ||
        equalsObject(existingPaginationMetadata, mergedPaginationMetadata, (existingProp, incomingProp) => {
            return existingProp === incomingProp;
        }) === false) {
        luvio.storePublish(paginationKey, mergedPaginationMetadata);
        luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    }
    assignMetadataLink(incomingRecord, paginationKey);
    if (existingRecord !== undefined && existingRecord.records !== undefined) {
        incomingRecord.records = [
            ...existingRecord.records.slice(0, startOffset),
            ...incomingRecord.records,
            ...existingRecord.records.slice(nextOffset),
        ];
    }
    if (existingRecord === undefined || equals$18(existingRecord, incomingRecord) === false) {
        luvio.storePublish(key, incomingRecord);
    }
    {
        const storeMetadataParams = {
            ttl: ttlToUse,
            namespace: "UiApi",
            version: VERSION$2I,
            representationName: RepresentationType$Z,
            ingestionTimestamp: timestamp,
        };
        luvio.publishStoreMetadata(key, storeMetadataParams);
    }
    return createLink$1(key);
};
function getTypeCacheKeys$2i(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$C(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$Z,
        mergeable: true
    });
    rootKeySet.set(rootKey + '__pagination', {
        namespace: keyPrefix,
        representationName: RepresentationType$Z,
        mergeable: true
    });
    const input_records_length = input.records.length;
    for (let i = 0; i < input_records_length; i++) {
        getTypeCacheKeys$2l(rootKeySet, luvio, input.records[i]);
    }
}

const VERSION$2H = "cf8d2eb042188974da26ffe6618cfd07";
function validate$1Y(obj, path = 'ListUiRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_info = obj.info;
        const path_info = path + '.info';
        if (typeof obj_info !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_info + '" (at "' + path_info + '")');
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (typeof obj_records !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$Y = 'ListUiRepresentation';
function keyBuilder$3A(luvio, config) {
    return keyPrefix + '::' + RepresentationType$Y + ':' + (config.listViewApiName === null ? '' : config.listViewApiName) + ':' + config.objectApiName + ':' + config.type + ':' + (config.sortBy === null ? '' : '[' + config.sortBy.join(',') + ']');
}
function keyBuilderFromType$B(luvio, object) {
    const keyParams = {
        listViewApiName: object.info.listReference.listViewApiName,
        objectApiName: object.info.listReference.objectApiName,
        type: object.info.listReference.type,
        sortBy: object.records.sortBy
    };
    return keyBuilder$3A(luvio, keyParams);
}
function normalize$Q(input, existing, path, luvio, store, timestamp) {
    const input_info = input.info;
    const input_info_id = path.fullPath + '__info';
    input.info = ingest$2j(input_info, {
        fullPath: input_info_id,
        propertyName: 'info',
        parent: {
            data: input,
            key: path.fullPath,
            existing: existing,
        },
        ttl: path.ttl
    }, luvio, store, timestamp);
    const input_records = input.records;
    const input_records_id = path.fullPath + '__records';
    input.records = ingest$2f(input_records, {
        fullPath: input_records_id,
        propertyName: 'records',
        parent: {
            data: input,
            key: path.fullPath,
            existing: existing,
        },
        ttl: path.ttl
    }, luvio, store, timestamp);
    return input;
}
const dynamicSelect$6 = function dynamicListUiRepresentationSelect(params, paginationParams) {
    const infoPathSelection = params.info === undefined ? {
        name: 'info',
        kind: 'Link',
        fragment: select$3t()
    } : params.info;
    const recordsPathSelection = params.records === undefined ? {
        name: 'records',
        kind: 'Link',
        fragment: select$3o(paginationParams)
    } : params.records;
    return {
        kind: 'Fragment',
        version: VERSION$2H,
        private: [
            'eTag'
        ],
        selections: [
            infoPathSelection,
            recordsPathSelection
        ]
    };
};
function equals$17(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_info = existing.info;
    const incoming_info = incoming.info;
    if (!(existing_info.__ref === incoming_info.__ref)) {
        return false;
    }
    const existing_records = existing.records;
    const incoming_records = incoming.records;
    if (!(existing_records.__ref === incoming_records.__ref)) {
        return false;
    }
    return true;
}
const ingest$2e = function ListUiRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1Y(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$B(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$Q, "UiApi", VERSION$2H, RepresentationType$Y, equals$17);
    return createLink$1(key);
};
function getTypeCacheKeys$2h(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$B(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$Y,
        mergeable: false
    });
    getTypeCacheKeys$2n(rootKeySet, luvio, input.info);
    getTypeCacheKeys$2i(rootKeySet, luvio, input.records);
}

function createPaginationParams$6(params) {
    const { queryParams } = params;
    return {
        token: queryParams.pageToken,
        pageSize: queryParams.pageSize === undefined ? 50 : queryParams.pageSize
    };
}
function createResourceRequest$1e(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-ui/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$_ = 'getListUiByApiName';
const getListUiByApiName_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('fields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sortBy', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
];
const getListUiByApiName_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$_, getListUiByApiName_ConfigPropertyMetadata);
const createResourceParams$13 = /*#__PURE__*/ createResourceParams$k(getListUiByApiName_ConfigPropertyMetadata);
function typeCheckConfig$18(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListUiByApiName_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$1b(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getListUiByApiName_ConfigPropertyMetadata);
    const config = typeCheckConfig$18(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

function createPaginationParams$5(params) {
    const { queryParams } = params;
    return {
        token: queryParams.pageToken,
        pageSize: queryParams.pageSize === undefined ? 50 : queryParams.pageSize
    };
}
function createResourceRequest$1d(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-ui/' + config.urlParams.listViewId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$Z = 'getListUiByListViewId';
const getListUiByListViewId_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('fields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sortBy', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
];
const getListUiByListViewId_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$Z, getListUiByListViewId_ConfigPropertyMetadata);
const createResourceParams$12 = /*#__PURE__*/ createResourceParams$k(getListUiByListViewId_ConfigPropertyMetadata);
function typeCheckConfig$17(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListUiByListViewId_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$1a(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getListUiByListViewId_ConfigPropertyMetadata);
    const config = typeCheckConfig$17(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

function createResourceRequest$1c(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-records/' + config.urlParams.listViewId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const VERSION$2G = "0b29241fd9ed04bb61fde26b3f558ac4";
function validate$1X(obj, path = 'ListInfoSummaryRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_url = obj.url;
        const path_url = path + '.url';
        if (typeof obj_url !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_url + '" (at "' + path_url + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$X = 'ListInfoSummaryRepresentation';
function keyBuilder$3v(luvio, config) {
    return keyPrefix + '::' + RepresentationType$X + ':' + (config.id === null ? '' : config.id);
}
function keyBuilderFromType$A(luvio, object) {
    const keyParams = {
        id: object.id
    };
    return keyBuilder$3v(luvio, keyParams);
}
function normalize$P(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3k = function ListInfoSummaryRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2G,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'url',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$16(existing, incoming) {
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_url = existing.url;
    const incoming_url = incoming.url;
    if (!(existing_url === incoming_url)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    return true;
}
const ingest$2d = function ListInfoSummaryRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1X(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$A(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$P, "UiApi", VERSION$2G, RepresentationType$X, equals$16);
    return createLink$1(key);
};
function getTypeCacheKeys$2g(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$A(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$X,
        mergeable: false
    });
}

const VERSION$2F = "d0cadb0927f908b5b64694e1cff0b2d7";
function validate$1W(obj, path = 'ListInfoSummaryCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_count = obj.count;
        const path_count = path + '.count';
        if (typeof obj_count !== 'number' || (typeof obj_count === 'number' && Math.floor(obj_count) !== obj_count)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_count + '" (at "' + path_count + '")');
        }
        const obj_currentPageToken = obj.currentPageToken;
        const path_currentPageToken = path + '.currentPageToken';
        if (typeof obj_currentPageToken !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
        }
        const obj_currentPageUrl = obj.currentPageUrl;
        const path_currentPageUrl = path + '.currentPageUrl';
        if (typeof obj_currentPageUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_lists = obj.lists;
        const path_lists = path + '.lists';
        if (!ArrayIsArray(obj_lists)) {
            return new TypeError('Expected "array" but received "' + typeof obj_lists + '" (at "' + path_lists + '")');
        }
        for (let i = 0; i < obj_lists.length; i++) {
            const obj_lists_item = obj_lists[i];
            const path_lists_item = path_lists + '[' + i + ']';
            if (typeof obj_lists_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_lists_item + '" (at "' + path_lists_item + '")');
            }
        }
        const obj_nextPageToken = obj.nextPageToken;
        const path_nextPageToken = path + '.nextPageToken';
        let obj_nextPageToken_union0 = null;
        const obj_nextPageToken_union0_error = (() => {
            if (typeof obj_nextPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union0_error != null) {
            obj_nextPageToken_union0 = obj_nextPageToken_union0_error.message;
        }
        let obj_nextPageToken_union1 = null;
        const obj_nextPageToken_union1_error = (() => {
            if (obj_nextPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union1_error != null) {
            obj_nextPageToken_union1 = obj_nextPageToken_union1_error.message;
        }
        if (obj_nextPageToken_union0 && obj_nextPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageToken + '")';
            message += '\n' + obj_nextPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageUrl = obj.nextPageUrl;
        const path_nextPageUrl = path + '.nextPageUrl';
        let obj_nextPageUrl_union0 = null;
        const obj_nextPageUrl_union0_error = (() => {
            if (typeof obj_nextPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union0_error != null) {
            obj_nextPageUrl_union0 = obj_nextPageUrl_union0_error.message;
        }
        let obj_nextPageUrl_union1 = null;
        const obj_nextPageUrl_union1_error = (() => {
            if (obj_nextPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union1_error != null) {
            obj_nextPageUrl_union1 = obj_nextPageUrl_union1_error.message;
        }
        if (obj_nextPageUrl_union0 && obj_nextPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageUrl + '")';
            message += '\n' + obj_nextPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_pageSize = obj.pageSize;
        const path_pageSize = path + '.pageSize';
        if (typeof obj_pageSize !== 'number' || (typeof obj_pageSize === 'number' && Math.floor(obj_pageSize) !== obj_pageSize)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_pageSize + '" (at "' + path_pageSize + '")');
        }
        const obj_previousPageToken = obj.previousPageToken;
        const path_previousPageToken = path + '.previousPageToken';
        let obj_previousPageToken_union0 = null;
        const obj_previousPageToken_union0_error = (() => {
            if (typeof obj_previousPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union0_error != null) {
            obj_previousPageToken_union0 = obj_previousPageToken_union0_error.message;
        }
        let obj_previousPageToken_union1 = null;
        const obj_previousPageToken_union1_error = (() => {
            if (obj_previousPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union1_error != null) {
            obj_previousPageToken_union1 = obj_previousPageToken_union1_error.message;
        }
        if (obj_previousPageToken_union0 && obj_previousPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageToken + '")';
            message += '\n' + obj_previousPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_previousPageUrl = obj.previousPageUrl;
        const path_previousPageUrl = path + '.previousPageUrl';
        let obj_previousPageUrl_union0 = null;
        const obj_previousPageUrl_union0_error = (() => {
            if (typeof obj_previousPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union0_error != null) {
            obj_previousPageUrl_union0 = obj_previousPageUrl_union0_error.message;
        }
        let obj_previousPageUrl_union1 = null;
        const obj_previousPageUrl_union1_error = (() => {
            if (obj_previousPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union1_error != null) {
            obj_previousPageUrl_union1 = obj_previousPageUrl_union1_error.message;
        }
        if (obj_previousPageUrl_union0 && obj_previousPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageUrl + '")';
            message += '\n' + obj_previousPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_queryString = obj.queryString;
        const path_queryString = path + '.queryString';
        let obj_queryString_union0 = null;
        const obj_queryString_union0_error = (() => {
            if (typeof obj_queryString !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_queryString + '" (at "' + path_queryString + '")');
            }
        })();
        if (obj_queryString_union0_error != null) {
            obj_queryString_union0 = obj_queryString_union0_error.message;
        }
        let obj_queryString_union1 = null;
        const obj_queryString_union1_error = (() => {
            if (obj_queryString !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_queryString + '" (at "' + path_queryString + '")');
            }
        })();
        if (obj_queryString_union1_error != null) {
            obj_queryString_union1 = obj_queryString_union1_error.message;
        }
        if (obj_queryString_union0 && obj_queryString_union1) {
            let message = 'Object doesn\'t match union (at "' + path_queryString + '")';
            message += '\n' + obj_queryString_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_queryString_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recentListsOnly = obj.recentListsOnly;
        const path_recentListsOnly = path + '.recentListsOnly';
        if (typeof obj_recentListsOnly !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_recentListsOnly + '" (at "' + path_recentListsOnly + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$W = 'ListInfoSummaryCollectionRepresentation';
function keyBuilder$3u(luvio, config) {
    return keyPrefix + '::' + RepresentationType$W + ':' + config.objectApiName + ':' + (config.queryString === null ? '' : config.queryString) + ':' + config.recentListsOnly;
}
function keyBuilderFromType$z(luvio, object) {
    const keyParams = {
        objectApiName: object.objectApiName,
        queryString: object.queryString,
        recentListsOnly: object.recentListsOnly
    };
    return keyBuilder$3u(luvio, keyParams);
}
function normalize$O(input, existing, path, luvio, store, timestamp) {
    const input_lists = input.lists;
    const input_lists_id = path.fullPath + '__lists';
    for (let i = 0; i < input_lists.length; i++) {
        const input_lists_item = input_lists[i];
        let input_lists_item_id = input_lists_id + '__' + i;
        input_lists[i] = ingest$2d(input_lists_item, {
            fullPath: input_lists_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$3j = function ListInfoSummaryCollectionRepresentationSelect(paginationParams) {
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: false,
        version: VERSION$2F,
        read: (source, reader) => {
            const sink = {};
            if (paginationParams === undefined) {
                return sink;
            }
            const metadata = reader.resolveMetadata(source, PAGINATION_VERSION$1);
            const currentPageMetadata = getPageMetadata$3(metadata, paginationParams);
            const { startOffset, endOffset, metadataProperties } = currentPageMetadata;
            if (metadata !== undefined && startOffset !== undefined && endOffset !== undefined) {
                const itemsProperty = 'lists';
                reader.enterPath(itemsProperty);
                const itemsSource = source[itemsProperty];
                const itemsSink = [];
                const allSink = [];
                for (let i = startOffset; i < endOffset; i++) {
                    reader.enterPath(i);
                    reader.readSingleLink(i, {
                        name: 'lists',
                        kind: 'Link',
                        fragment: select$3k()
                    }, itemsSource, allSink, i - startOffset);
                    if (reader.getIsDataMissing() === false) {
                        itemsSink[i - startOffset] = allSink[i - startOffset];
                    }
                    reader.exitPath();
                }
                reader.exitPath();
                reader.assignNonScalar(sink, itemsProperty, itemsSink);
            }
            reader.enterPath('currentPageToken');
            reader.readScalar('currentPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('nextPageToken');
            reader.readScalar('nextPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('previousPageToken');
            reader.readScalar('previousPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('pageSize');
            reader.readScalar('pageSize', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('count');
            reader.readScalar('count', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('objectApiName');
            reader.readScalar('objectApiName', source, sink);
            reader.exitPath();
            reader.enterPath('queryString');
            reader.readScalar('queryString', source, sink);
            reader.exitPath();
            reader.enterPath('recentListsOnly');
            reader.readScalar('recentListsOnly', source, sink);
            reader.exitPath();
            return sink;
        },
    };
};
function getPageMetadata$3(paginationMetadata, paginationParams) {
    const metadataProperties = {};
    const sink = {
        metadataProperties
    };
    if (paginationMetadata === undefined) {
        return sink;
    }
    const { token, pageSize } = paginationParams;
    const pageTokenAndOffset = getPageTokenAndOffset$1(paginationMetadata, token);
    if (pageTokenAndOffset === undefined) {
        return sink;
    }
    const [pageToken, startOffset] = pageTokenAndOffset;
    const endOffset = pageEndOffset$1(paginationMetadata, startOffset, pageSize);
    sink.startOffset = startOffset;
    sink.endOffset = endOffset;
    metadataProperties.currentPageToken = pageToken;
    const nextToken = getNextPageToken(paginationMetadata, endOffset);
    if (nextToken !== undefined) {
        metadataProperties.nextPageToken = nextToken;
    }
    const previousToken = getPreviousPageToken(paginationMetadata, startOffset, pageSize);
    if (previousToken !== undefined) {
        metadataProperties.previousPageToken = previousToken;
    }
    metadataProperties.pageSize = pageSize;
    metadataProperties.count = endOffset - startOffset;
    return sink;
}
function equals$15(existing, incoming) {
    const existing_recentListsOnly = existing.recentListsOnly;
    const incoming_recentListsOnly = incoming.recentListsOnly;
    if (!(existing_recentListsOnly === incoming_recentListsOnly)) {
        return false;
    }
    const existing_count = existing.count;
    const incoming_count = incoming.count;
    if (!(existing_count === incoming_count)) {
        return false;
    }
    const existing_pageSize = existing.pageSize;
    const incoming_pageSize = incoming.pageSize;
    if (!(existing_pageSize === incoming_pageSize)) {
        return false;
    }
    const existing_currentPageToken = existing.currentPageToken;
    const incoming_currentPageToken = incoming.currentPageToken;
    if (!(existing_currentPageToken === incoming_currentPageToken)) {
        return false;
    }
    const existing_currentPageUrl = existing.currentPageUrl;
    const incoming_currentPageUrl = incoming.currentPageUrl;
    if (!(existing_currentPageUrl === incoming_currentPageUrl)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_lists = existing.lists;
    const incoming_lists = incoming.lists;
    const equals_lists_items = equalsArray(existing_lists, incoming_lists, (existing_lists_item, incoming_lists_item) => {
        if (!(existing_lists_item.__ref === incoming_lists_item.__ref)) {
            return false;
        }
    });
    if (equals_lists_items === false) {
        return false;
    }
    const existing_nextPageToken = existing.nextPageToken;
    const incoming_nextPageToken = incoming.nextPageToken;
    if (!(existing_nextPageToken === incoming_nextPageToken)) {
        return false;
    }
    const existing_nextPageUrl = existing.nextPageUrl;
    const incoming_nextPageUrl = incoming.nextPageUrl;
    if (!(existing_nextPageUrl === incoming_nextPageUrl)) {
        return false;
    }
    const existing_previousPageToken = existing.previousPageToken;
    const incoming_previousPageToken = incoming.previousPageToken;
    if (!(existing_previousPageToken === incoming_previousPageToken)) {
        return false;
    }
    const existing_previousPageUrl = existing.previousPageUrl;
    const incoming_previousPageUrl = incoming.previousPageUrl;
    if (!(existing_previousPageUrl === incoming_previousPageUrl)) {
        return false;
    }
    const existing_queryString = existing.queryString;
    const incoming_queryString = incoming.queryString;
    if (!(existing_queryString === incoming_queryString)) {
        return false;
    }
    return true;
}
const ingest$2c = function ListInfoSummaryCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1W(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$z(luvio, input);
    const existingRecord = store.readEntry(key);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    let incomingRecord = normalize$O(input, store.readEntry(key), {
        fullPath: key,
        parent: path.parent,
        propertyName: path.propertyName,
        ttl: ttlToUse
    }, luvio, store, timestamp);
    const paginationKey = key + '__pagination';
    const existingPaginationMetadata = store.readEntry(paginationKey);
    const { count: count, currentPageToken: token, nextPageToken: nextToken, pageSize: pageSize, previousPageToken: previousToken, lists: items } = input;
    const startOffset = existingPaginationMetadata === undefined ? 0 : offsetFromToken(existingPaginationMetadata, token);
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0 ||
            (existingRecord && existingRecord.lists && existingRecord.lists.length < startOffset)) {
            throw new RangeError('currentPageToken value ' + token + ' is not recognized');
        }
    }
    const incomingPaginationMetadata = {
        [token]: startOffset,
    };
    const nextOffset = startOffset + items.length;
    if (nextToken === null) {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    else {
        incomingPaginationMetadata[nextToken] = nextOffset;
        if (existingPaginationMetadata !== undefined) {
            const listEndOffset = existingPaginationMetadata.__END__;
            if (listEndOffset !== undefined && nextOffset >= listEndOffset) {
                incomingPaginationMetadata.__END__ = undefined;
            }
        }
    }
    if (previousToken !== null) {
        incomingPaginationMetadata[previousToken] = Math.max(startOffset - pageSize, 0);
    }
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    if (existingPaginationMetadata === undefined ||
        equalsObject(existingPaginationMetadata, mergedPaginationMetadata, (existingProp, incomingProp) => {
            return existingProp === incomingProp;
        }) === false) {
        luvio.storePublish(paginationKey, mergedPaginationMetadata);
        luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    }
    assignMetadataLink(incomingRecord, paginationKey);
    if (existingRecord !== undefined && existingRecord.lists !== undefined) {
        incomingRecord.lists = [
            ...existingRecord.lists.slice(0, startOffset),
            ...incomingRecord.lists,
            ...existingRecord.lists.slice(nextOffset),
        ];
    }
    if (existingRecord === undefined || equals$15(existingRecord, incomingRecord) === false) {
        luvio.storePublish(key, incomingRecord);
    }
    if (ttlToUse !== undefined) {
        const storeMetadataParams = {
            ttl: ttlToUse,
            namespace: "UiApi",
            version: VERSION$2F,
            representationName: RepresentationType$W,
            ingestionTimestamp: timestamp,
        };
        luvio.publishStoreMetadata(key, storeMetadataParams);
    }
    return createLink$1(key);
};
function getTypeCacheKeys$2f(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$z(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$W,
        mergeable: true
    });
    rootKeySet.set(rootKey + '__pagination', {
        namespace: keyPrefix,
        representationName: RepresentationType$W,
        mergeable: true
    });
    const input_lists_length = input.lists.length;
    for (let i = 0; i < input_lists_length; i++) {
        getTypeCacheKeys$2g(rootKeySet, luvio, input.lists[i]);
    }
}

function buildListRefByIdContextKey(listViewId) {
    return `listReferenceById::${listViewId}`;
}
function buildListRefByApiNameContextKey(objectApiName, listViewApiName) {
    return `listReferenceByApiName::${objectApiName.toLowerCase()}:${listViewApiName.toLowerCase()}`;
}
/**
 * Adds a list reference so it can be retrieved with #getListReference later.
 *
 * @param listRef list reference
 */
function addListReference(listRef, context) {
    addListReferenceWithId(listRef, context, listRef.id);
}
/**
 * Adds a list reference with some specified key so it can be retrieved with #getListReference later.
 *
 * @param listRef
 * @param context
 * @param id - Specified Id to use in the context key
 */
function addListReferenceWithId(listRef, context, id) {
    // Add listReference by id if present
    if (id) {
        context.set(buildListRefByIdContextKey(id), listRef);
    }
    // Add listReference by objectApiName & listViewApiName
    context.set(buildListRefByApiNameContextKey(listRef.objectApiName, listRef.listViewApiName || ''), listRef);
}
/**
 * Returns a list reference from the store if it's present.
 *
 * @param query list view to look for
 * @param luvio Luvio
 */
function getListReference(query, context) {
    return query.listViewId
        ? context.get(buildListRefByIdContextKey(query.listViewId))
        : context.get(buildListRefByApiNameContextKey(query.objectApiName || '', query.listViewApiName || ''));
}
/**
 * Reader selections to copy a list info
 */
const LIST_INFO_SELECTIONS = select$3t().selections;
/**
 * List info private memebers
 */
select$3t().private;
const LIST_INFO_SELECTIONS_ETAG = [
    ...LIST_INFO_SELECTIONS,
    { kind: 'Scalar', name: 'eTag' },
];
/**
 * Retrieves the snapshot of the list info corresponding to the specified list
 * reference from the store.
 *
 * @param listRef list reference
 * @param luvio Luvio
 */
function getListInfo(luvio, listRef, storeLookup) {
    let mutatedListRef = { ...listRef };
    mutatedListRef.objectApiName = mutatedListRef.objectApiName.toLowerCase();
    mutatedListRef.listViewApiName = mutatedListRef.listViewApiName
        ? mutatedListRef.listViewApiName.toLowerCase()
        : mutatedListRef.listViewApiName;
    const key = keyBuilder$3F(luvio, { ...mutatedListRef });
    return storeLookup({
        recordId: key,
        node: {
            kind: 'Fragment',
            selections: LIST_INFO_SELECTIONS_ETAG,
            private: [],
            version: VERSION$2O,
        },
        variables: {},
    });
}
function buildDefaultsKey(listRef) {
    const { objectApiName, listViewApiName } = listRef;
    return `defaults::${objectApiName.toLowerCase()}:${listViewApiName ? listViewApiName.toLowerCase() : listViewApiName}`;
}
/**
 * Update the default values based on a server response.
 *
 * @param config getListUi config
 * @param serverResponse ListUiRepresentation from the server
 */
function addServerDefaults(config, records, listReference, context) {
    // If the request already contains the potential default information,
    // or the response does not come back with a default or comes back wih an empty array []
    if ((config.sortBy !== undefined && config.sortBy.length > 0) || records.sortBy === null) {
        return;
    }
    const key = buildDefaultsKey(listReference);
    const currentDefaults = context.get(key) || {};
    context.set(key, { ...currentDefaults, sortBy: records.sortBy });
}
/**
 * Returns default values observed on previous requests for a list.
 *
 * @param config getListUi config
 * @returns defaults from previous requests for this list, or {} if no defaults are known
 */
function getServerDefaults(config, context) {
    const listRef = getListReference(config, context);
    if (listRef === undefined) {
        return {};
    }
    const key = buildDefaultsKey(listRef);
    return context.get(key) || {};
}
// Logic to deal with fields on the list view. This would be reasonably straightforward
// except that the server sometimes adds 5 well-known fields to every record & nested
// record in its responses.
// hardcoded fields that the server adds
const DEFAULT_SERVER_FIELDS = [
    'CreatedDate',
    'Id',
    'LastModifiedById',
    'LastModifiedDate',
    'SystemModstamp',
];
/**
 * Adds default fields for every record referenced in a given field name. E.g. if field
 * is "Opportunity.Account.Name" then add default fields "Opportunity.CreatedDate",
 * "Opportunity.Id", ..., "Opportunity.Account.CreatedDate", "Opportunity.Account.Id", ... .
 *
 * @param field explicitly included field
 * @param defaultFields fields object to be updated with the fields that the server will
 *    implicitly add
 */
function addDefaultFields(field, defaultFields) {
    const fieldParts = field.split('.');
    for (let i = 1; i < fieldParts.length; ++i) {
        const fieldPrefix = fieldParts.slice(0, i).join('.');
        for (let j = 0; j < DEFAULT_SERVER_FIELDS.length; ++j) {
            defaultFields[`${fieldPrefix}.${DEFAULT_SERVER_FIELDS[j]}`] = true;
        }
    }
}
/**
 * Indicates if a RecordRepresentation contains a specified field.
 *
 * @param record record
 * @param field field to check for, split on '.'s, with the leading object api name omitted.
 *    E.g. if searching an Opportunity for "Opportunity.Account.Name" this parameter should
 *    be ['Account','Name'].
 */
function recordContainsField(record, field) {
    // make sure it looks like a record and the first piece of the field path has a value
    if (!record ||
        !record.fields ||
        !record.fields[field[0]] ||
        record.fields[field[0]].value === undefined) {
        return false;
    }
    // recurse if nested record
    else if (field.length > 1) {
        return recordContainsField(record.fields[field[0]].value, field.slice(1));
    }
    // found it
    return true;
}
function listFields(luvio, { fields = [], optionalFields = [], sortBy, }, listInfo) {
    const { displayColumns, listReference: { objectApiName }, } = listInfo;
    let fields_ = {}, optionalFields_ = {}, defaultFields_ = {};
    // all the fields in the list info are required
    for (let i = 0, len = displayColumns.length; i < len; ++i) {
        const qualifiedField = `${objectApiName}.${displayColumns[i].fieldApiName}`;
        fields_[qualifiedField] = true;
        addDefaultFields(qualifiedField, defaultFields_);
    }
    // required fields from the component
    for (let i = 0, len = fields.length; i < len; ++i) {
        const qualifiedField = fields[i].startsWith(`${objectApiName}.`)
            ? fields[i]
            : `${objectApiName}.${fields[i]}`;
        if (!fields_[qualifiedField]) {
            fields_[qualifiedField] = true;
            addDefaultFields(qualifiedField, defaultFields_);
        }
    }
    // optional fields from the component
    for (let i = 0, len = optionalFields.length; i < len; ++i) {
        const qualifiedField = optionalFields[i].startsWith(`${objectApiName}.`)
            ? optionalFields[i]
            : `${objectApiName}.${optionalFields[i]}`;
        if (!fields_[qualifiedField]) {
            optionalFields_[qualifiedField] = true;
            addDefaultFields(qualifiedField, defaultFields_);
        }
    }
    const key = keyBuilder$3B(luvio, {
        listViewId: listInfo.eTag,
        sortBy: sortBy || null,
        searchTerm: null,
        where: null,
        objectApiName: listInfo.listReference.objectApiName,
        listViewApiName: listInfo.listReference.listViewApiName,
    }) + '__fieldstatus';
    const node = luvio.getNode(key);
    const defaultServerFieldStatus = isGraphNode(node)
        ? node.retrieve()
        : { missingFields: { ...defaultFields_ } };
    return {
        getRecordSelectionFieldSets() {
            const optionalPlusDefaultFields = { ...optionalFields_ };
            const fields = keys(defaultFields_);
            for (let i = 0; i < fields.length; ++i) {
                const field = fields[i];
                if (!fields_[field] && !defaultServerFieldStatus.missingFields[field]) {
                    optionalPlusDefaultFields[field] = true;
                }
            }
            return [keys(fields_).sort(), keys(optionalPlusDefaultFields).sort()];
        },
        processRecords(records) {
            const { missingFields } = defaultServerFieldStatus;
            const fields = keys(missingFields);
            for (let i = 0; i < fields.length; ++i) {
                const field = fields[i], splitField = field.split('.').slice(1);
                for (let i = 0; i < records.length; ++i) {
                    if (recordContainsField(records[i], splitField)) {
                        delete missingFields[field];
                        break;
                    }
                }
            }
            luvio.storePublish(key, defaultServerFieldStatus);
            // snapshots do not subscribe to this key, so no need to broadcast
            return this;
        },
    };
}

let context$1;
function setContext$1(adapterContext) {
    context$1 = adapterContext;
}
function keyBuilder$3t(luvio, params) {
    // lists data can now be fetched using case insensitive urlParams therefore
    // also changing contextStore to use lowercase caching
    const query = {
        objectApiName: params.urlParams.objectApiName.toLowerCase(),
        listViewApiName: params.urlParams.listViewApiName.toLowerCase(),
    };
    // Fetch listReference from internal store to better ensure a cache hit regardless of listViewApiName or listViewId
    const listReference = getListReference(query, context$1);
    if (listReference !== undefined) {
        // Check and use any default values returned from the server (i.e. sortBy)
        const config = { ...params.urlParams, ...params.body };
        const defaults = getServerDefaults(config, context$1);
        // Use default values when sortBy is undefined or an empty Array []
        return keyBuilder$3B(luvio, {
            objectApiName: listReference.objectApiName.toLowerCase(),
            listViewApiName: listReference.listViewApiName
                ? listReference.listViewApiName.toLowerCase()
                : listReference.listViewApiName,
            // # removing listViewId from key only supporing getting records using api name
            // listViewId: listReference.id,
            searchTerm: params.body.searchTerm || null,
            where: params.body.where || null,
            sortBy: params.body.sortBy !== undefined && params.body.sortBy.length <= 0
                ? defaults.sortBy || params.body.sortBy || []
                : params.body.sortBy || defaults.sortBy || [],
        });
    }
    // If there are no matching entries in the store, then we haven't fetched any data for this list view yet.
    // Assuming that listViewId is empty and continue on
    return keyBuilder$3B(luvio, {
        objectApiName: params.urlParams.objectApiName.toLowerCase(),
        listViewApiName: params.urlParams.listViewApiName.toLowerCase(),
        // # removing listViewId from key only supporing getting records using api name
        // listViewId: '',
        searchTerm: params.body.searchTerm || null,
        where: params.body.where || null,
        sortBy: params.body.sortBy || [],
    });
}

function select$3i(luvio, params) {
    const { fields = [], optionalFields = [] } = params.body;
    return dynamicSelect$7({
        records: {
            name: 'records',
            kind: 'Link',
            fragment: {
                kind: 'Fragment',
                private: ['eTag', 'weakEtag'],
                selections: buildSelectionFromFields(fields, optionalFields),
                version: VERSION$2K
            },
        },
    }, createPaginationParams$4(params));
}
function ingestSuccess$U(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$3t(luvio, resourceParams);
    luvio.storeIngest(key, ingest$2f, body);
    const optionalFields = resourceParams.body.optionalFields;
    if (optionalFields && optionalFields.length > 0) {
        const normalized = body;
        markMissingOptionalFieldsOnRecords$1(luvio, normalized, optionalFields);
    }
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$3i(luvio, resourceParams),
        variables: {},
    }, snapshotRefresh);
    return snapshot;
}
function markMissingOptionalFieldsOnRecords$1(luvio, nomalized, optionalFields) {
    const records = nomalized.records;
    for (let i = 0; i < records.length; i++) {
        const record = records[i];
        const recordKey = record.__ref;
        if (recordKey !== undefined) {
            const node = luvio.getNode(recordKey);
            markMissingOptionalFields(node, optionalFields);
        }
    }
}

function createPaginationParams$4(params) {
    const { body } = params;
    return {
        token: body.pageToken,
        pageSize: body.pageSize === undefined ? 50 : body.pageSize
    };
}
function getResponseCacheKeys$13(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2i(storeKeyMap, luvio, response);
}
function ingestError$N(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$3t(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$E,
        namespace: keyPrefix,
        version: VERSION$2I,
        representationName: RepresentationType$Z
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$1b(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-records/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

/**
 * Examines a set of paginated data & metadata from an UnfulfilledSnapshot and computes a
 * pageToken and pageSize that will minimize the amount of data requested while still
 * satisfying the original request.
 *
 * @param config.name name of the field within data that contains the items
 * @param conifg.data paginated data/metadata from an UnfulfilledSnapshot
 * @param config.pageSize requested pageSize
 * @param config.pagination pagination data/functions from engine
 * @returns pageToken & pageSize to fill in the missing data
 */
function minimizeRequest(config) {
    // the only way to handle missing current or previous token is to ask for the full set of requested records
    if (!config.data || !config.data[config.name] || config.data.previousPageToken === undefined) {
        return {
            pageSize: config.pageSize,
            pageToken: config.pageToken,
        };
    }
    else {
        // compute the offset of the last record that was found
        const { offsetFromToken, tokenForAtMost, paginationMetadata } = config.pagination;
        const pageTokenOffset = offsetFromToken(paginationMetadata, config.data.currentPageToken);
        if (pageTokenOffset < 0) {
            return {
                pageSize: config.pageSize,
                pageToken: config.pageToken,
            };
        }
        const lastFoundOffset = pageTokenOffset + config.data[config.name].length;
        // backup to the nearest offset for which we have a token
        const [newToken, newOffset] = tokenForAtMost(paginationMetadata, lastFoundOffset);
        // recompute pageToken and pageSize for query based on new starting token
        return {
            pageSize: pageTokenOffset - newOffset + config.pageSize,
            pageToken: newToken,
        };
    }
}

function isFulfilledSnapshot(snapshot) {
    return snapshot.state === 'Fulfilled';
}
function isUnfulfilledSnapshot(snapshot) {
    return snapshot.state === 'Unfulfilled';
}
function isStaleSnapshot(snapshot) {
    return snapshot.state === 'Stale';
}
function isErrorSnapshot(snapshot) {
    return snapshot.state === 'Error';
}

const VERSION$2E = "3529c5f7d0eb2206c90a52c6d9f721d4";
function validate$1V(obj, path = 'ListViewSummaryRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_listUiUrl = obj.listUiUrl;
        const path_listUiUrl = path + '.listUiUrl';
        if (typeof obj_listUiUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_listUiUrl + '" (at "' + path_listUiUrl + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$V = 'ListViewSummaryRepresentation';
function keyBuilder$3s(luvio, config) {
    return keyPrefix + '::' + RepresentationType$V + ':' + (config.id === null ? '' : config.id);
}
function keyBuilderFromType$y(luvio, object) {
    const keyParams = {
        id: object.id
    };
    return keyBuilder$3s(luvio, keyParams);
}
function normalize$N(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3h = function ListViewSummaryRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2E,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'listUiUrl',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$14(existing, incoming) {
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_listUiUrl = existing.listUiUrl;
    const incoming_listUiUrl = incoming.listUiUrl;
    if (!(existing_listUiUrl === incoming_listUiUrl)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    return true;
}
const ingest$2b = function ListViewSummaryRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1V(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$y(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$N, "UiApi", VERSION$2E, RepresentationType$V, equals$14);
    return createLink$1(key);
};
function getTypeCacheKeys$2e(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$y(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$V,
        mergeable: false
    });
}

const VERSION$2D = "f4c79fe066da3558499d5ee88c72abcd";
function validate$1U(obj, path = 'ListViewSummaryCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_count = obj.count;
        const path_count = path + '.count';
        if (typeof obj_count !== 'number' || (typeof obj_count === 'number' && Math.floor(obj_count) !== obj_count)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_count + '" (at "' + path_count + '")');
        }
        const obj_currentPageToken = obj.currentPageToken;
        const path_currentPageToken = path + '.currentPageToken';
        if (typeof obj_currentPageToken !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
        }
        const obj_currentPageUrl = obj.currentPageUrl;
        const path_currentPageUrl = path + '.currentPageUrl';
        if (typeof obj_currentPageUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_lists = obj.lists;
        const path_lists = path + '.lists';
        if (!ArrayIsArray(obj_lists)) {
            return new TypeError('Expected "array" but received "' + typeof obj_lists + '" (at "' + path_lists + '")');
        }
        for (let i = 0; i < obj_lists.length; i++) {
            const obj_lists_item = obj_lists[i];
            const path_lists_item = path_lists + '[' + i + ']';
            if (typeof obj_lists_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_lists_item + '" (at "' + path_lists_item + '")');
            }
        }
        const obj_nextPageToken = obj.nextPageToken;
        const path_nextPageToken = path + '.nextPageToken';
        let obj_nextPageToken_union0 = null;
        const obj_nextPageToken_union0_error = (() => {
            if (typeof obj_nextPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union0_error != null) {
            obj_nextPageToken_union0 = obj_nextPageToken_union0_error.message;
        }
        let obj_nextPageToken_union1 = null;
        const obj_nextPageToken_union1_error = (() => {
            if (obj_nextPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union1_error != null) {
            obj_nextPageToken_union1 = obj_nextPageToken_union1_error.message;
        }
        if (obj_nextPageToken_union0 && obj_nextPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageToken + '")';
            message += '\n' + obj_nextPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageUrl = obj.nextPageUrl;
        const path_nextPageUrl = path + '.nextPageUrl';
        let obj_nextPageUrl_union0 = null;
        const obj_nextPageUrl_union0_error = (() => {
            if (typeof obj_nextPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union0_error != null) {
            obj_nextPageUrl_union0 = obj_nextPageUrl_union0_error.message;
        }
        let obj_nextPageUrl_union1 = null;
        const obj_nextPageUrl_union1_error = (() => {
            if (obj_nextPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union1_error != null) {
            obj_nextPageUrl_union1 = obj_nextPageUrl_union1_error.message;
        }
        if (obj_nextPageUrl_union0 && obj_nextPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageUrl + '")';
            message += '\n' + obj_nextPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_pageSize = obj.pageSize;
        const path_pageSize = path + '.pageSize';
        if (typeof obj_pageSize !== 'number' || (typeof obj_pageSize === 'number' && Math.floor(obj_pageSize) !== obj_pageSize)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_pageSize + '" (at "' + path_pageSize + '")');
        }
        const obj_previousPageToken = obj.previousPageToken;
        const path_previousPageToken = path + '.previousPageToken';
        let obj_previousPageToken_union0 = null;
        const obj_previousPageToken_union0_error = (() => {
            if (typeof obj_previousPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union0_error != null) {
            obj_previousPageToken_union0 = obj_previousPageToken_union0_error.message;
        }
        let obj_previousPageToken_union1 = null;
        const obj_previousPageToken_union1_error = (() => {
            if (obj_previousPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union1_error != null) {
            obj_previousPageToken_union1 = obj_previousPageToken_union1_error.message;
        }
        if (obj_previousPageToken_union0 && obj_previousPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageToken + '")';
            message += '\n' + obj_previousPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_previousPageUrl = obj.previousPageUrl;
        const path_previousPageUrl = path + '.previousPageUrl';
        let obj_previousPageUrl_union0 = null;
        const obj_previousPageUrl_union0_error = (() => {
            if (typeof obj_previousPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union0_error != null) {
            obj_previousPageUrl_union0 = obj_previousPageUrl_union0_error.message;
        }
        let obj_previousPageUrl_union1 = null;
        const obj_previousPageUrl_union1_error = (() => {
            if (obj_previousPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union1_error != null) {
            obj_previousPageUrl_union1 = obj_previousPageUrl_union1_error.message;
        }
        if (obj_previousPageUrl_union0 && obj_previousPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageUrl + '")';
            message += '\n' + obj_previousPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_queryString = obj.queryString;
        const path_queryString = path + '.queryString';
        let obj_queryString_union0 = null;
        const obj_queryString_union0_error = (() => {
            if (typeof obj_queryString !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_queryString + '" (at "' + path_queryString + '")');
            }
        })();
        if (obj_queryString_union0_error != null) {
            obj_queryString_union0 = obj_queryString_union0_error.message;
        }
        let obj_queryString_union1 = null;
        const obj_queryString_union1_error = (() => {
            if (obj_queryString !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_queryString + '" (at "' + path_queryString + '")');
            }
        })();
        if (obj_queryString_union1_error != null) {
            obj_queryString_union1 = obj_queryString_union1_error.message;
        }
        if (obj_queryString_union0 && obj_queryString_union1) {
            let message = 'Object doesn\'t match union (at "' + path_queryString + '")';
            message += '\n' + obj_queryString_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_queryString_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recentListsOnly = obj.recentListsOnly;
        const path_recentListsOnly = path + '.recentListsOnly';
        if (typeof obj_recentListsOnly !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_recentListsOnly + '" (at "' + path_recentListsOnly + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$U = 'ListViewSummaryCollectionRepresentation';
function keyBuilder$3r(luvio, config) {
    return keyPrefix + '::' + RepresentationType$U + ':' + config.objectApiName + ':' + (config.queryString === null ? '' : config.queryString) + ':' + config.recentListsOnly;
}
function keyBuilderFromType$x(luvio, object) {
    const keyParams = {
        objectApiName: object.objectApiName,
        queryString: object.queryString,
        recentListsOnly: object.recentListsOnly
    };
    return keyBuilder$3r(luvio, keyParams);
}
function normalize$M(input, existing, path, luvio, store, timestamp) {
    const input_lists = input.lists;
    const input_lists_id = path.fullPath + '__lists';
    for (let i = 0; i < input_lists.length; i++) {
        const input_lists_item = input_lists[i];
        let input_lists_item_id = input_lists_id + '__' + i;
        input_lists[i] = ingest$2b(input_lists_item, {
            fullPath: input_lists_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$3g = function ListViewSummaryCollectionRepresentationSelect(paginationParams) {
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: false,
        version: VERSION$2D,
        read: (source, reader) => {
            const sink = {};
            if (paginationParams === undefined) {
                return sink;
            }
            const metadata = reader.resolveMetadata(source, PAGINATION_VERSION$1);
            const currentPageMetadata = getPageMetadata$2(metadata, paginationParams);
            const { startOffset, endOffset, metadataProperties } = currentPageMetadata;
            if (metadata !== undefined && startOffset !== undefined && endOffset !== undefined) {
                const itemsProperty = 'lists';
                reader.enterPath(itemsProperty);
                const itemsSource = source[itemsProperty];
                const itemsSink = [];
                const allSink = [];
                for (let i = startOffset; i < endOffset; i++) {
                    reader.enterPath(i);
                    reader.readSingleLink(i, {
                        name: 'lists',
                        kind: 'Link',
                        fragment: select$3h()
                    }, itemsSource, allSink, i - startOffset);
                    if (reader.getIsDataMissing() === false) {
                        itemsSink[i - startOffset] = allSink[i - startOffset];
                    }
                    reader.exitPath();
                }
                reader.exitPath();
                reader.assignNonScalar(sink, itemsProperty, itemsSink);
            }
            reader.enterPath('currentPageToken');
            reader.readScalar('currentPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('nextPageToken');
            reader.readScalar('nextPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('previousPageToken');
            reader.readScalar('previousPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('pageSize');
            reader.readScalar('pageSize', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('count');
            reader.readScalar('count', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('objectApiName');
            reader.readScalar('objectApiName', source, sink);
            reader.exitPath();
            reader.enterPath('queryString');
            reader.readScalar('queryString', source, sink);
            reader.exitPath();
            reader.enterPath('recentListsOnly');
            reader.readScalar('recentListsOnly', source, sink);
            reader.exitPath();
            return sink;
        },
    };
};
function getPageMetadata$2(paginationMetadata, paginationParams) {
    const metadataProperties = {};
    const sink = {
        metadataProperties
    };
    if (paginationMetadata === undefined) {
        return sink;
    }
    const { token, pageSize } = paginationParams;
    const pageTokenAndOffset = getPageTokenAndOffset$1(paginationMetadata, token);
    if (pageTokenAndOffset === undefined) {
        return sink;
    }
    const [pageToken, startOffset] = pageTokenAndOffset;
    const endOffset = pageEndOffset$1(paginationMetadata, startOffset, pageSize);
    sink.startOffset = startOffset;
    sink.endOffset = endOffset;
    metadataProperties.currentPageToken = pageToken;
    const nextToken = getNextPageToken(paginationMetadata, endOffset);
    if (nextToken !== undefined) {
        metadataProperties.nextPageToken = nextToken;
    }
    const previousToken = getPreviousPageToken(paginationMetadata, startOffset, pageSize);
    if (previousToken !== undefined) {
        metadataProperties.previousPageToken = previousToken;
    }
    metadataProperties.pageSize = pageSize;
    metadataProperties.count = endOffset - startOffset;
    return sink;
}
function equals$13(existing, incoming) {
    const existing_recentListsOnly = existing.recentListsOnly;
    const incoming_recentListsOnly = incoming.recentListsOnly;
    if (!(existing_recentListsOnly === incoming_recentListsOnly)) {
        return false;
    }
    const existing_count = existing.count;
    const incoming_count = incoming.count;
    if (!(existing_count === incoming_count)) {
        return false;
    }
    const existing_pageSize = existing.pageSize;
    const incoming_pageSize = incoming.pageSize;
    if (!(existing_pageSize === incoming_pageSize)) {
        return false;
    }
    const existing_currentPageToken = existing.currentPageToken;
    const incoming_currentPageToken = incoming.currentPageToken;
    if (!(existing_currentPageToken === incoming_currentPageToken)) {
        return false;
    }
    const existing_currentPageUrl = existing.currentPageUrl;
    const incoming_currentPageUrl = incoming.currentPageUrl;
    if (!(existing_currentPageUrl === incoming_currentPageUrl)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_lists = existing.lists;
    const incoming_lists = incoming.lists;
    const equals_lists_items = equalsArray(existing_lists, incoming_lists, (existing_lists_item, incoming_lists_item) => {
        if (!(existing_lists_item.__ref === incoming_lists_item.__ref)) {
            return false;
        }
    });
    if (equals_lists_items === false) {
        return false;
    }
    const existing_nextPageToken = existing.nextPageToken;
    const incoming_nextPageToken = incoming.nextPageToken;
    if (!(existing_nextPageToken === incoming_nextPageToken)) {
        return false;
    }
    const existing_nextPageUrl = existing.nextPageUrl;
    const incoming_nextPageUrl = incoming.nextPageUrl;
    if (!(existing_nextPageUrl === incoming_nextPageUrl)) {
        return false;
    }
    const existing_previousPageToken = existing.previousPageToken;
    const incoming_previousPageToken = incoming.previousPageToken;
    if (!(existing_previousPageToken === incoming_previousPageToken)) {
        return false;
    }
    const existing_previousPageUrl = existing.previousPageUrl;
    const incoming_previousPageUrl = incoming.previousPageUrl;
    if (!(existing_previousPageUrl === incoming_previousPageUrl)) {
        return false;
    }
    const existing_queryString = existing.queryString;
    const incoming_queryString = incoming.queryString;
    if (!(existing_queryString === incoming_queryString)) {
        return false;
    }
    return true;
}
const ingest$2a = function ListViewSummaryCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1U(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$x(luvio, input);
    const existingRecord = store.readEntry(key);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    let incomingRecord = normalize$M(input, store.readEntry(key), {
        fullPath: key,
        parent: path.parent,
        propertyName: path.propertyName,
        ttl: ttlToUse
    }, luvio, store, timestamp);
    const paginationKey = key + '__pagination';
    const existingPaginationMetadata = store.readEntry(paginationKey);
    const { count: count, currentPageToken: token, nextPageToken: nextToken, pageSize: pageSize, previousPageToken: previousToken, lists: items } = input;
    const startOffset = existingPaginationMetadata === undefined ? 0 : offsetFromToken(existingPaginationMetadata, token);
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0 ||
            (existingRecord && existingRecord.lists && existingRecord.lists.length < startOffset)) {
            throw new RangeError('currentPageToken value ' + token + ' is not recognized');
        }
    }
    const incomingPaginationMetadata = {
        [token]: startOffset,
    };
    const nextOffset = startOffset + items.length;
    if (nextToken === null) {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    else {
        incomingPaginationMetadata[nextToken] = nextOffset;
        if (existingPaginationMetadata !== undefined) {
            const listEndOffset = existingPaginationMetadata.__END__;
            if (listEndOffset !== undefined && nextOffset >= listEndOffset) {
                incomingPaginationMetadata.__END__ = undefined;
            }
        }
    }
    if (previousToken !== null) {
        incomingPaginationMetadata[previousToken] = Math.max(startOffset - pageSize, 0);
    }
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    if (existingPaginationMetadata === undefined ||
        equalsObject(existingPaginationMetadata, mergedPaginationMetadata, (existingProp, incomingProp) => {
            return existingProp === incomingProp;
        }) === false) {
        luvio.storePublish(paginationKey, mergedPaginationMetadata);
        luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    }
    assignMetadataLink(incomingRecord, paginationKey);
    if (existingRecord !== undefined && existingRecord.lists !== undefined) {
        incomingRecord.lists = [
            ...existingRecord.lists.slice(0, startOffset),
            ...incomingRecord.lists,
            ...existingRecord.lists.slice(nextOffset),
        ];
    }
    if (existingRecord === undefined || equals$13(existingRecord, incomingRecord) === false) {
        luvio.storePublish(key, incomingRecord);
    }
    if (ttlToUse !== undefined) {
        const storeMetadataParams = {
            ttl: ttlToUse,
            namespace: "UiApi",
            version: VERSION$2D,
            representationName: RepresentationType$U,
            ingestionTimestamp: timestamp,
        };
        luvio.publishStoreMetadata(key, storeMetadataParams);
    }
    return createLink$1(key);
};
function getTypeCacheKeys$2d(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$x(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$U,
        mergeable: true
    });
    rootKeySet.set(rootKey + '__pagination', {
        namespace: keyPrefix,
        representationName: RepresentationType$U,
        mergeable: true
    });
    const input_lists_length = input.lists.length;
    for (let i = 0; i < input_lists_length; i++) {
        getTypeCacheKeys$2e(rootKeySet, luvio, input.lists[i]);
    }
}

function createPaginationParams$3(params) {
    const { queryParams } = params;
    return {
        token: queryParams.pageToken,
        pageSize: queryParams.pageSize === undefined ? 20 : queryParams.pageSize
    };
}
function select$3f(luvio, params) {
    const paginationParams = createPaginationParams$3(params);
    return select$3g(paginationParams);
}
function keyBuilder$3q(luvio, params) {
    return keyBuilder$3r(luvio, {
        objectApiName: params.urlParams.objectApiName,
        queryString: params.queryParams.q || null,
        recentListsOnly: params.queryParams.recentListsOnly || false
    });
}
function getResponseCacheKeys$12(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2d(storeKeyMap, luvio, response);
}
function ingestSuccess$T(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$3q(luvio, resourceParams);
    luvio.storeIngest(key, ingest$2a, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$3f(luvio, resourceParams),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$M(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$3q(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$1a(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-ui/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$Y = 'getListViewSummaryCollection';
const getListViewSummaryCollection_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('q', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('recentListsOnly', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
const getListViewSummaryCollection_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$Y, getListViewSummaryCollection_ConfigPropertyMetadata);
const createResourceParams$11 = /*#__PURE__*/ createResourceParams$k(getListViewSummaryCollection_ConfigPropertyMetadata);
function keyBuilder$3p(luvio, config) {
    const resourceParams = createResourceParams$11(config);
    return keyBuilder$3q(luvio, resourceParams);
}
function typeCheckConfig$16(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListViewSummaryCollection_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$19(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getListViewSummaryCollection_ConfigPropertyMetadata);
    const config = typeCheckConfig$16(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$L(luvio, config) {
    const resourceParams = createResourceParams$11(config);
    return select$3f(luvio, resourceParams);
}
function onFetchResponseSuccess$P(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$T(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$17(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$Q(luvio, config, resourceParams, response) {
    const snapshot = ingestError$M(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$17(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function prepareRequest$6(luvio, config, resourceParams, cacheSnapshot) {
    if (cacheSnapshot === undefined) {
        return createResourceRequest$1a(resourceParams);
    }
    const [newToken, newPageSize] = minimizePaginationParams$2(luvio, resourceParams, cacheSnapshot);
    if (newToken === undefined) {
        return createResourceRequest$1a(resourceParams);
    }
    return createResourceRequest$1a(createResourceParams$11({
        ...config,
        pageSize: newPageSize,
        pageToken: newToken
    }));
}
function minimizePaginationParams$2(luvio, resourceParams, cacheSnapshot) {
    const num = getCachedItemsNumber$3(cacheSnapshot);
    if (num > 0) {
        const paginationMetadata = getPaginationMetadata$3(luvio, resourceParams);
        if (paginationMetadata !== undefined) {
            const { token: configToken, pageSize: configPageSize } = createPaginationParams$3(resourceParams);
            const configTokenOffset = offsetFromToken(paginationMetadata, configToken);
            if (configTokenOffset < 0) {
                return [undefined, -1];
            }
            const maxOffset = configTokenOffset + num;
            const [token, offset] = tokenForAtMost(paginationMetadata, maxOffset);
            if (token !== undefined) {
                return [token, configPageSize - (offset - configTokenOffset)];
            }
        }
    }
    return [undefined, -1];
}
function getCachedItemsNumber$3(cacheSnapshot) {
    const { data } = cacheSnapshot;
    if (data === undefined || !data.lists) {
        return 0;
    }
    const items = data.lists;
    for (let i = 0; i < items.length; i++) {
        if (items[i] === undefined) {
            return i;
        }
    }
    return items.length;
}
function getPaginationMetadata$3(luvio, resourceParams) {
    const key = keyBuilder$3q(luvio, resourceParams);
    const selector = {
        recordId: key,
        node: {
            kind: 'Fragment',
            version: VERSION$2D,
            private: [],
            selections: [
                {
                    kind: 'Link',
                    name: '__metadata',
                    fragment: {
                        kind: 'Fragment',
                        version: PAGINATION_VERSION$1,
                        opaque: true,
                        private: [],
                    },
                },
            ],
        },
        variables: {}
    };
    const node = luvio.storeLookup(selector);
    if (node.state !== 'Fulfilled') {
        return;
    }
    return node.data.__metadata;
}
function buildNetworkSnapshot$17(luvio, config, options, cacheSnapshot) {
    const resourceParams = createResourceParams$11(config);
    const request = prepareRequest$6(luvio, config, resourceParams, cacheSnapshot);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$P(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$12(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$Q(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$T(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$17, undefined, true);
}
function buildCachedSnapshotCachePolicy$S(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$3p(luvio, config),
        node: adapterFragment$L(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$17(luvio, config, snapshotRefreshOptions)
    });
    if (isUnfulfilledSnapshot$1(cacheSnapshot) && cacheSnapshot.data !== undefined) {
        context.cacheSnapshot = cacheSnapshot;
    }
    return cacheSnapshot;
}
const getListViewSummaryCollectionAdapterFactory = (luvio) => function UiApi__getListViewSummaryCollection(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$19(untrustedConfig, getListViewSummaryCollection_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$S, buildNetworkSnapshotCachePolicy$T);
};

function createPaginationParams$2(params) {
    const { queryParams } = params;
    return {
        token: queryParams.pageToken,
        pageSize: queryParams.pageSize === undefined ? 50 : queryParams.pageSize
    };
}
function keyBuilder$3o(luvio, params) {
    return keyBuilder$3A(luvio, {
        listViewApiName: null,
        objectApiName: params.urlParams.objectApiName,
        type: "mru",
        sortBy: params.queryParams.sortBy || null
    });
}
function createResourceRequest$19(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/mru-list-ui/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$X = 'getMruListUi';
const getMruListUi_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('fields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sortBy', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
];
const getMruListUi_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$X, getMruListUi_ConfigPropertyMetadata);
const createResourceParams$10 = /*#__PURE__*/ createResourceParams$k(getMruListUi_ConfigPropertyMetadata);
function typeCheckConfig$15(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getMruListUi_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$18(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getMruListUi_ConfigPropertyMetadata);
    const config = typeCheckConfig$15(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

function createResourceRequest$18(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/mru-list-records/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

/**
 * Returns a BuildNetworkSnapshot function that always returns a Promise<ErrorSnapshot>.
 *
 * @param luvio Luvio instance
 * @param status status code for ErrorSnapshot, defaults to 400
 * @param statusText status text for ErrorSnapshot, defaults to 'data is not fetchable'
 * @returns BuildNetworkSnapshot that always returns Promise<ErrorSnapshot>
 */
function buildNotFetchableNetworkSnapshot(luvio, status = 400, statusText = 'data is not fetchable') {
    return (_context) => Promise.resolve(luvio.errorSnapshot({
        status,
        statusText,
        ok: false,
        body: undefined,
        headers: {},
        errorType: 'fetchResponse',
    }));
}

function isPromise(value) {
    return value.then !== undefined;
}

// eslint-disable-next-line @salesforce/lds/no-invalid-todo
// TODO RAML - this more properly goes in the generated resource files
const DEFAULT_PAGE_SIZE$1 = 50;
// make local copies of the adapter configs so we can ignore other getListUi config parameters to match
// lds222 behavior
const getMruListUi_ConfigPropertyNames_augmented = {
    ...getMruListUi_ConfigPropertyNames,
    parameters: {
        ...getMruListUi_ConfigPropertyNames.parameters,
        optional: [
            ...getMruListUi_ConfigPropertyNames.parameters.optional,
            'listViewApiName',
            'listViewId',
        ],
    },
};
function buildListUiFragment$1(config, fields) {
    const resourceParams = createResourceParams$10(config);
    const paginationParams = createPaginationParams$2(resourceParams);
    const recordSelectParams = {
        records: {
            name: 'records',
            kind: 'Link',
            fragment: {
                kind: 'Fragment',
                private: ['eTag', 'weakEtag'],
                selections: buildSelectionFromFields(...fields.getRecordSelectionFieldSets()),
                version: VERSION$2K,
            },
        },
    };
    const listRecordCollectionSelect = dynamicSelect$7(recordSelectParams, paginationParams);
    const listRecordCollectionSelectParams = {
        records: {
            name: 'records',
            kind: 'Link',
            fragment: listRecordCollectionSelect,
        },
    };
    return dynamicSelect$6(listRecordCollectionSelectParams, paginationParams);
}
function buildSnapshotRefresh_getMruListUi(luvio, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot_getMruListUi(luvio, config),
    };
}
function onResourceSuccess_getMruListUi(luvio, config, response) {
    const { body } = response;
    const listInfo = body.info;
    const listUiKey = keyBuilder$3A(luvio, {
        ...listInfo.listReference,
        sortBy: body.records.sortBy,
    });
    // grab relevant bits before ingest destroys the structure
    const fields = listFields(luvio, config, listInfo);
    fields.processRecords(body.records.records);
    // build the selector while the list info is still easily accessible
    const fragment = buildListUiFragment$1(config, fields);
    luvio.storeIngest(listUiKey, ingest$2e, body);
    const snapshot = luvio.storeLookup({
        recordId: listUiKey,
        node: fragment,
        variables: {},
    }, buildSnapshotRefresh_getMruListUi(luvio, config));
    return luvio.storeBroadcast().then(() => snapshot);
}
function onResourceError_getMruListUi(luvio, config, err) {
    return Promise.resolve(luvio.errorSnapshot(err, buildSnapshotRefresh_getMruListUi(luvio, config)));
}
function buildCachedSnapshot$8(luvio, storeLookup, config, listInfo, fields) {
    const listFields_ = fields || listFields(luvio, config, listInfo);
    const resourceParams = createResourceParams$10(config);
    const selector = {
        recordId: keyBuilder$3o(luvio, resourceParams),
        node: buildListUiFragment$1(config, listFields_),
        variables: {},
    };
    return storeLookup(selector, buildSnapshotRefresh_getMruListUi(luvio, config));
}
/**
 * Builds, sends, and processes the result of a mru-list-ui request, ignoring any cached
 * data for the list.
 *
 * @param luvio Luvio engine
 * @param config wire config
 */
function buildNetworkSnapshot_getMruListUi(luvio, config, dispatchOptions) {
    const params = createResourceParams$10(config);
    const request = createResourceRequest$19(params);
    return luvio.dispatchResourceRequest(request, dispatchOptions).then((response) => {
        const { body } = response;
        // response might have records.sortBy in csv format but keyBuilder/ingestion
        // functions expect it to be an array so coerce it here if needed
        const sortBy = body.records.sortBy;
        if (sortBy && typeof sortBy === 'string') {
            body.records.sortBy = sortBy.split(',');
        }
        return luvio.handleSuccessResponse(() => onResourceSuccess_getMruListUi(luvio, config, response), () => {
            const cache = new StoreKeyMap();
            getTypeCacheKeys$2h(cache, luvio, body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => onResourceError_getMruListUi(luvio, config, err));
    });
}
function prepareRequest_getMruListRecords(luvio, config, listInfo, snapshot) {
    const { fields, optionalFields, pageSize, pageToken, sortBy } = config;
    const queryParams = {
        fields,
        optionalFields,
        pageSize,
        pageToken,
        sortBy,
    };
    const request = createResourceRequest$18({
        urlParams: {
            objectApiName: config.objectApiName,
        },
        queryParams,
    });
    if (snapshot) {
        const paginationKey = paginationKeyBuilder(luvio, {
            listViewId: listInfo.eTag,
            searchTerm: null,
            where: null,
            sortBy: config.sortBy === undefined ? null : config.sortBy,
            objectApiName: listInfo.listReference.objectApiName.toLowerCase(),
            listViewApiName: listInfo.listReference.listViewApiName
                ? listInfo.listReference.listViewApiName.toLowerCase()
                : listInfo.listReference.listViewApiName,
        });
        const selector = {
            recordId: paginationKey,
            node: {
                kind: 'Fragment',
                private: [],
                opaque: true,
                version: PAGINATION_VERSION$1,
            },
            variables: {},
        };
        const metadataSnapshot = luvio.storeLookup(selector);
        const paginationMetadata = metadataSnapshot.state === 'Fulfilled' ? metadataSnapshot.data : {};
        // compute the minimum number of records we need to request
        const { pageSize, pageToken } = minimizeRequest({
            data: snapshot.data ? snapshot.data.records : null,
            name: 'records',
            pageSize: config.pageSize || DEFAULT_PAGE_SIZE$1,
            pageToken: config.pageToken,
            pagination: {
                offsetFromToken,
                tokenForAtMost,
                paginationMetadata,
            },
        });
        // update request, but don't harden default values unless they were already present
        if (pageSize !== DEFAULT_PAGE_SIZE$1 || request.queryParams.pageSize !== undefined) {
            request.queryParams.pageSize = pageSize;
        }
        if (pageToken !== undefined || request.queryParams.pageToken !== undefined) {
            request.queryParams.pageToken = pageToken;
        }
    }
    return request;
}
// Only call this function if you are certain the list view hasn't changed (ie:
// the listInfoEtag in the body is the same as the cached listInfo.eTag)
function onResourceSuccess_getMruListRecords(luvio, config, listInfo, response) {
    const { body } = response;
    const { listInfoETag } = body;
    const fields = listFields(luvio, config, listInfo).processRecords(body.records);
    luvio.storeIngest(keyBuilder$3B(luvio, {
        listViewId: listInfoETag,
        searchTerm: null,
        where: null,
        sortBy: body.sortBy,
        objectApiName: listInfo.listReference.objectApiName.toLowerCase(),
        listViewApiName: listInfo.listReference.listViewApiName
            ? listInfo.listReference.listViewApiName.toLowerCase()
            : listInfo.listReference.listViewApiName,
    }), ingest$2f, body);
    const snapshot = buildCachedSnapshot$8(luvio, luvio.storeLookup.bind(luvio), config, listInfo, fields);
    return luvio.storeBroadcast().then(() => snapshot);
}
function onResourceError_getMruListRecords(luvio, config, listInfo, err) {
    const errorSnapshot = luvio.errorSnapshot(err, buildSnapshotRefresh_getMruListUi(luvio, config));
    luvio.storeIngestError(keyBuilder$3A(luvio, {
        ...listInfo.listReference,
        sortBy: config.sortBy === undefined ? null : config.sortBy,
    }), errorSnapshot);
    return luvio.storeBroadcast().then(() => errorSnapshot);
}
function buildNetworkSnapshot_getMruListRecords(luvio, config, listInfo, dispatchOptions, snapshot) {
    const request = prepareRequest_getMruListRecords(luvio, config, listInfo, snapshot);
    return luvio
        .dispatchResourceRequest(request, dispatchOptions)
        .then((response) => {
        const { body } = response;
        // fall back to mru-list-ui if list view has changed
        if (body.listInfoETag !== listInfo.eTag) {
            return buildNetworkSnapshot_getMruListUi(luvio, config, dispatchOptions);
        }
        // response might have records.sortBy in csv format but keyBuilder/ingestion
        // functions expect it to be an array so coerce it here if needed
        const { sortBy } = body;
        if (sortBy && typeof sortBy === 'string') {
            body.sortBy = sortBy.split(',');
        }
        // else ingest
        return luvio.handleSuccessResponse(() => onResourceSuccess_getMruListRecords(luvio, config, listInfo, response), () => {
            const cache = new StoreKeyMap();
            getTypeCacheKeys$2i(cache, luvio, body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return onResourceError_getMruListRecords(luvio, config, listInfo, err);
        });
    });
}
function buildCachedListInfoSnapshot$1(context, storeLookup, luvio) {
    const { config } = context;
    // try to get a list reference and a list info for the list; this should come back
    // non-null if we have the list info cached
    return getListInfo(luvio, {
        id: null,
        listViewApiName: null,
        objectApiName: config.objectApiName.toLowerCase(),
        type: 'mru',
    }, storeLookup);
}
function buildCachedListUiSnapshot$1(context, storeLookup) {
    const { config, listInfo, luvio } = context;
    if (listInfo !== undefined) {
        context.listUi = buildCachedSnapshot$8(luvio, storeLookup, config, listInfo);
        return context.listUi;
    }
}
function buildNetworkListUiSnapshot$1(context, coercedAdapterRequestContext) {
    const { config, listInfo, listUi, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    // make the full list ui request if any of the following is true:
    //
    // - the list info was not found
    // - we couldn't build enough of the list ui to locate any record data
    // - we found the complete cached list ui; this is somewhat counterintuitive,
    //   but it happens when the cache policy has decided to refetch cached data
    if (!listInfo ||
        !listUi ||
        !listUi.data ||
        isFulfilledSnapshot(listUi) ||
        isStaleSnapshot(listUi)) {
        return buildNetworkSnapshot_getMruListUi(luvio, config, dispatchOptions);
    }
    // we *should* only be missing records and/or tokens at this point; send a list-records
    // request to fill them in
    return buildNetworkSnapshot_getMruListRecords(luvio, config, listInfo, dispatchOptions, listUi);
}
const factory$k = (luvio) => function getMruListUi(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$18(untrustedConfig, getMruListUi_ConfigPropertyNames_augmented);
    if (config === null) {
        return null;
    }
    const definedRequestContext = requestContext || {};
    // try to find a cached ListInfoRepresentation
    const listInfoPromiseOrSnapshot = luvio.applyCachePolicy(definedRequestContext, { config, luvio }, buildCachedListInfoSnapshot$1, buildNotFetchableNetworkSnapshot(luvio));
    // build the ListUiRepresentation from the cached ListInfoRepresentation (if any)
    const processListInfo = (listInfoSnapshot) => {
        const listInfo = isFulfilledSnapshot(listInfoSnapshot) || isStaleSnapshot(listInfoSnapshot)
            ? listInfoSnapshot.data
            : undefined;
        return luvio.applyCachePolicy(definedRequestContext, { config, listInfo, luvio }, buildCachedListUiSnapshot$1, buildNetworkListUiSnapshot$1);
    };
    return isPromise(listInfoPromiseOrSnapshot)
        ? listInfoPromiseOrSnapshot.then(processListInfo)
        : processListInfo(listInfoPromiseOrSnapshot);
};

const contextId$5 = `${keyPrefix}__getListUi`;
// eslint-disable-next-line @salesforce/lds/no-invalid-todo
// TODO RAML - this more properly goes in the generated resource files
const DEFAULT_PAGE_SIZE = 50;
// make local copies of the adapter configs so we can have them ignore each other's config parameters
// to match lds222 behavior
const getListUiByApiName_ConfigPropertyNames_augmented = {
    ...getListUiByApiName_ConfigPropertyNames,
    parameters: {
        ...getListUiByApiName_ConfigPropertyNames.parameters,
        optional: [...getListUiByApiName_ConfigPropertyNames.parameters.optional, 'listViewId'],
    },
};
const getListUiByListViewId_ConfigPropertyNames_augmented = {
    ...getListUiByListViewId_ConfigPropertyNames,
    parameters: {
        ...getListUiByListViewId_ConfigPropertyNames.parameters,
        optional: [
            ...getListUiByListViewId_ConfigPropertyNames.parameters.optional,
            'listViewApiName',
            'objectApiName',
        ],
    },
};
function getSortBy(config, context) {
    if (config.sortBy !== undefined) {
        return config.sortBy;
    }
    const defaults = getServerDefaults(config, context);
    if (defaults.sortBy !== undefined) {
        return defaults.sortBy;
    }
    return null;
}
function buildListUiFragment(config, context, fields) {
    const defaultedConfig = { ...getServerDefaults(config, context), ...config };
    let paginationParams;
    if (isGetListUiByListViewIdConfig(defaultedConfig)) {
        const resourceParams = createResourceParams$12(defaultedConfig);
        paginationParams = createPaginationParams$5(resourceParams);
    }
    else if (isGetListUiByApiNameConfig(defaultedConfig)) {
        const resourceParams = createResourceParams$13(defaultedConfig);
        paginationParams =
            createPaginationParams$6(resourceParams);
    }
    const recordSelectParams = {
        records: {
            name: 'records',
            kind: 'Link',
            fragment: {
                kind: 'Fragment',
                private: ['eTag', 'weakEtag'],
                selections: buildSelectionFromFields(...fields.getRecordSelectionFieldSets()),
                version: VERSION$2K,
            },
        },
    };
    const listRecordCollectionSelect = dynamicSelect$7(recordSelectParams, paginationParams);
    const listRecordCollectionSelectParams = {
        records: {
            name: 'records',
            kind: 'Link',
            fragment: listRecordCollectionSelect,
        },
    };
    return dynamicSelect$6(listRecordCollectionSelectParams, paginationParams);
}
function buildCachedSnapshot$7(luvio, storeLookup, context, config, listInfo, fields) {
    const listUiKey = keyBuilder$3A(luvio, {
        ...listInfo.listReference,
        sortBy: getSortBy(config, context),
    });
    // ok to pass luvio here since listFields() ignores TTL when getting cached field information
    const listFields_ = fields || listFields(luvio, config, listInfo);
    const selector = {
        recordId: listUiKey,
        node: buildListUiFragment(config, context, listFields_),
        variables: {},
    };
    return storeLookup(selector, buildSnapshotRefresh_getListUi(luvio, context, config));
}
function buildSnapshotRefresh_getListUi(luvio, context, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot_getListUi(luvio, context, config, undefined),
    };
}
function prepareRequest_getListUi(config) {
    const { fields, optionalFields, pageSize, pageToken, sortBy } = config;
    const queryParams = {
        fields,
        optionalFields,
        pageSize,
        pageToken,
        sortBy,
    };
    let request;
    if (isGetListUiByApiNameConfig(config)) {
        request = createResourceRequest$1e({
            urlParams: {
                listViewApiName: config.listViewApiName,
                objectApiName: config.objectApiName,
            },
            queryParams,
        });
    }
    else if (isGetListUiByListViewIdConfig(config)) {
        request = createResourceRequest$1d({
            urlParams: { listViewId: config.listViewId },
            queryParams,
        });
    }
    else {
        // eslint-disable-next-line @salesforce/lds/no-error-in-production
        throw new Error('unrecognized config');
    }
    return request;
}
function onResourceSuccess_getListUi(luvio, context, config, response) {
    const { body } = response, listInfo = body.info, { listReference } = listInfo;
    const listUiKey = keyBuilder$3A(luvio, {
        ...listReference,
        sortBy: body.records.sortBy,
    });
    // grab relevant bits before ingest destroys the structure
    const fields = listFields(luvio, config, listInfo);
    fields.processRecords(body.records.records);
    // remember the id/name of this list
    addListReference(listReference, context);
    // remember any default values that the server filled in
    addServerDefaults(config, body.records, listReference, context);
    // build the selector while the list info is still easily accessible
    const fragment = buildListUiFragment(config, context, fields);
    luvio.storeIngest(listUiKey, ingest$2e, body);
    const snapshot = luvio.storeLookup({
        recordId: listUiKey,
        node: fragment,
        variables: {},
    }, buildSnapshotRefresh_getListUi(luvio, context, config));
    return luvio.storeBroadcast().then(() => snapshot);
}
function onResourceError_getListUi(luvio, context, config, err) {
    return Promise.resolve(luvio.errorSnapshot(err, buildSnapshotRefresh_getListUi(luvio, context, config)));
}
/**
 * Builds, sends, and processes the result of a list-ui request, ignoring any cached
 * data for the list view.
 *
 * @param luvio Luvio engine
 * @param config wire config
 */
function buildNetworkSnapshot_getListUi(luvio, context, config, dispatchOptions) {
    const request = prepareRequest_getListUi(config);
    return luvio.dispatchResourceRequest(request, dispatchOptions).then((response) => {
        const { body } = response;
        // response might have records.sortBy in csv format but keyBuilder/ingestion
        // functions expect it to be an array so coerce it here if needed
        const sortBy = body.records.sortBy;
        if (sortBy && typeof sortBy === 'string') {
            body.records.sortBy = sortBy.split(',');
        }
        return luvio.handleSuccessResponse(() => onResourceSuccess_getListUi(luvio, context, config, response), () => {
            const cache = new StoreKeyMap();
            getTypeCacheKeys$2h(cache, luvio, body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return onResourceError_getListUi(luvio, context, config, err);
        });
    });
}
function prepareRequest_getListRecords(luvio, context, config, listInfo, snapshot) {
    const { fields, optionalFields, pageSize, pageToken, sortBy } = config;
    const queryParams = {
        fields,
        optionalFields,
        pageSize,
        pageToken,
        sortBy,
    };
    const body = {
        fields,
        optionalFields,
        pageSize,
        pageToken,
        sortBy,
    };
    let request;
    if (isGetListUiByApiNameConfig(config)) {
        request = createResourceRequest$1b({
            urlParams: {
                listViewApiName: config.listViewApiName,
                objectApiName: config.objectApiName,
            },
            body,
        });
    }
    else if (isGetListUiByListViewIdConfig(config)) {
        request = createResourceRequest$1c({
            urlParams: { listViewId: config.listViewId },
            queryParams,
        });
    }
    else {
        // eslint-disable-next-line @salesforce/lds/no-error-in-production
        throw new Error('how did MRU config get here?');
    }
    let releaseKeys = () => { };
    if (snapshot) {
        if (snapshot.state !== 'Error') {
            const { recordId, seenRecords } = snapshot;
            const snapshotRetainedIds = seenRecords === undefined ? [recordId] : [recordId, ...seenRecords.keysAsArray()];
            const snapshotKeys = [];
            snapshotRetainedIds.forEach((key) => snapshotKeys.push(key));
            releaseKeys = luvio.storeRetain(snapshotKeys);
        }
        const paginationKey = paginationKeyBuilder(luvio, {
            listViewId: listInfo.eTag,
            searchTerm: null,
            where: null,
            sortBy: getSortBy(config, context),
            objectApiName: listInfo.listReference.objectApiName.toLowerCase(),
            listViewApiName: listInfo.listReference.listViewApiName
                ? listInfo.listReference.listViewApiName.toLowerCase()
                : listInfo.listReference.listViewApiName,
        });
        const selector = {
            recordId: paginationKey,
            node: {
                kind: 'Fragment',
                private: [],
                opaque: true,
                version: PAGINATION_VERSION$1,
            },
            variables: {},
        };
        const metadataSnapshot = luvio.storeLookup(selector);
        const paginationMetadata = metadataSnapshot.state === 'Fulfilled' ? metadataSnapshot.data : {};
        // compute the minimum number of records we need to request
        const { pageSize, pageToken } = minimizeRequest({
            data: snapshot.data ? snapshot.data.records : null,
            name: 'records',
            pageSize: config.pageSize || DEFAULT_PAGE_SIZE,
            pageToken: config.pageToken,
            pagination: {
                offsetFromToken,
                tokenForAtMost,
                paginationMetadata,
            },
        });
        // update request, but don't harden default values unless they were already present
        if (pageSize !== DEFAULT_PAGE_SIZE || request.queryParams.pageSize !== undefined) {
            request.queryParams.pageSize = pageSize;
        }
        if (pageToken || request.queryParams.pageToken !== undefined) {
            request.queryParams.pageToken = pageToken;
        }
    }
    return {
        request: request,
        releaseKeys: releaseKeys,
    };
}
// Only call this function if you are certain the list view hasn't changed (ie:
// the listInfoEtag in the body is the same as the cached listInfo.eTag)
function onResourceSuccess_getListRecords(luvio, context, config, listInfo, response, releaseKeys) {
    const { body } = response;
    const { listInfoETag } = body;
    const fields = listFields(luvio, config, listInfo).processRecords(body.records);
    luvio.storeIngest(keyBuilder$3B(luvio, {
        listViewId: listInfoETag,
        searchTerm: null,
        where: null,
        sortBy: body.sortBy,
        objectApiName: listInfo.listReference.objectApiName.toLowerCase(),
        listViewApiName: listInfo.listReference.listViewApiName
            ? listInfo.listReference.listViewApiName.toLowerCase()
            : listInfo.listReference.listViewApiName,
    }), ingest$2f, body);
    const snapshot = buildCachedSnapshot$7(luvio, luvio.storeLookup.bind(luvio), context, config, listInfo, fields);
    releaseKeys();
    return luvio.storeBroadcast().then(() => snapshot);
}
function onResourceError_getListRecords(luvio, context, config, listInfo, err, releaseKeys) {
    const errorSnapshot = luvio.errorSnapshot(err);
    luvio.storeIngestError(keyBuilder$3A(luvio, {
        ...listInfo.listReference,
        sortBy: getSortBy(config, context),
    }), errorSnapshot);
    releaseKeys();
    return luvio.storeBroadcast().then(() => errorSnapshot);
}
function buildNetworkSnapshot_getListRecords(luvio, context, config, listInfo, dispatchOptions, snapshot) {
    const { request, releaseKeys } = prepareRequest_getListRecords(luvio, context, config, listInfo, snapshot);
    return luvio.dispatchResourceRequest(request).then((response) => {
        const { body } = response;
        // fall back to list-ui if list view has changed
        if (body.listInfoETag !== listInfo.eTag) {
            return buildNetworkSnapshot_getListUi(luvio, context, config, dispatchOptions);
        }
        // response might have records.sortBy in csv format but keyBuilder/ingestion
        // functions expect it to be an array so coerce it here if needed
        const { sortBy } = body;
        if (sortBy && typeof sortBy === 'string') {
            body.sortBy = sortBy.split(',');
        }
        // else ingest
        return luvio.handleSuccessResponse(() => onResourceSuccess_getListRecords(luvio, context, config, listInfo, response, releaseKeys), () => {
            const cache = new StoreKeyMap();
            getTypeCacheKeys$2i(cache, luvio, body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return onResourceError_getListRecords(luvio, context, config, listInfo, err, releaseKeys);
        });
    });
}
function buildCachedListInfoSnapshot(context, storeLookup, luvio) {
    const { adapterContext, config } = context;
    // try to get a list reference and a list info for the list; this should come back
    // non-null if we have the list info cached
    const listRef = getListReference(config, adapterContext);
    // no listRef means we can't even attempt to build a cached snapshot
    // so make a full list-ui request
    if (listRef === undefined) {
        return;
    }
    return getListInfo(luvio, listRef, storeLookup);
}
function buildCachedListUiSnapshot(context, storeLookup) {
    const { adapterContext, config, listInfo, luvio } = context;
    if (listInfo !== undefined) {
        context.listUi = buildCachedSnapshot$7(luvio, storeLookup, adapterContext, config, listInfo);
        return context.listUi;
    }
}
function buildNetworkListUiSnapshot(context, coercedAdapterRequestContext) {
    const { adapterContext, config, listInfo, listUi, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    // make the full list ui request if any of the following is true:
    //
    // - the list info was not found
    // - we couldn't build enough of the list ui to locate any record data
    // - we found the complete cached list ui; this is somewhat counterintuitive,
    //   but it happens when the cache policy has decided to refetch cached data
    if (!listInfo ||
        !listUi ||
        !listUi.data ||
        isFulfilledSnapshot(listUi) ||
        isStaleSnapshot(listUi)) {
        return buildNetworkSnapshot_getListUi(luvio, adapterContext, config, dispatchOptions);
    }
    // we *should* only be missing records and/or tokens at this point; send a list-records
    // request to fill them in
    return buildNetworkSnapshot_getListRecords(luvio, adapterContext, config, listInfo, dispatchOptions, listUi);
}
// functions to discern config variations
function isGetListUiByApiNameConfig(config) {
    return config.listViewApiName !== undefined;
}
function looksLikeGetListUiByApiNameConfig(untrustedConfig) {
    return (untrustedIsObject(untrustedConfig) &&
        untrustedConfig.objectApiName &&
        untrustedConfig.listViewApiName);
}
function isGetListUiByListViewIdConfig(config) {
    return !!config.listViewId;
}
function looksLikeGetListUiByListViewIdConfig(untrustedConfig) {
    return (untrustedIsObject(untrustedConfig) &&
        untrustedConfig.listViewId);
}
function looksLikeGetListViewSummaryCollectionConfig(untrustedConfig) {
    return (untrustedIsObject(untrustedConfig) &&
        untrustedConfig.objectApiName &&
        !untrustedConfig.listViewId &&
        !untrustedConfig.listViewApiName);
}
function looksLikeGetMruListUiConfig(untrustedConfig) {
    // the MRU symbol is a carryover hack from 222 and doesn't show up in any
    // of the generated config types, so we cast to any in order to check for it
    return untrustedIsObject(untrustedConfig) && untrustedConfig.listViewApiName === MRU;
}
function validateGetListUiConfig(untrustedConfig) {
    return looksLikeGetListUiByApiNameConfig(untrustedConfig)
        ? validateAdapterConfig$1b(untrustedConfig, getListUiByApiName_ConfigPropertyNames_augmented)
        : looksLikeGetListUiByListViewIdConfig(untrustedConfig)
            ? validateAdapterConfig$1a(untrustedConfig, getListUiByListViewId_ConfigPropertyNames_augmented)
            : null;
}
// the listViewApiName value to pass to getListUi() to request the MRU list
const MRU = Symbol.for('MRU');
const factory$j = (luvio) => {
    // adapter implementation for getListUiBy*
    const listUiAdapter = (untrustedConfig, adapterContext, requestContext) => {
        const config = validateGetListUiConfig(untrustedConfig);
        if (config === null) {
            return null;
        }
        const definedRequestContext = requestContext || {};
        // try to find a cached ListInfoRepresentation
        const listInfoPromiseOrSnapshot = luvio.applyCachePolicy(definedRequestContext, { adapterContext, config, luvio }, buildCachedListInfoSnapshot, buildNotFetchableNetworkSnapshot(luvio));
        // build the ListUiRepresentation from the cached ListInfoRepresentation (if any)
        const processListInfo = (listInfoSnapshot) => {
            const listInfo = isFulfilledSnapshot(listInfoSnapshot) || isStaleSnapshot(listInfoSnapshot)
                ? listInfoSnapshot.data
                : undefined;
            return luvio.applyCachePolicy(definedRequestContext, { adapterContext, config, listInfo, luvio }, buildCachedListUiSnapshot, buildNetworkListUiSnapshot);
        };
        return isPromise(listInfoPromiseOrSnapshot)
            ? listInfoPromiseOrSnapshot.then(processListInfo)
            : processListInfo(listInfoPromiseOrSnapshot);
    };
    let listViewSummaryCollectionAdapter = null;
    let mruAdapter = null;
    // delegate to various other adapter based on what config looks like; note that the adapters
    // we delegate to are responsible for returning refreshable results
    return luvio.withContext(function UiApi__custom_getListUi(untrustedConfig, adapterContext, requestContext) {
        // if the MRU symbol is there then just return the getMruListUi adapter
        if (looksLikeGetMruListUiConfig(untrustedConfig)) {
            if (mruAdapter === null) {
                mruAdapter = factory$k(luvio);
            }
            // the symbol in the listViewApiName is just a hack so we can recognize the request as MRU
            const mruConfig = { ...untrustedConfig };
            delete mruConfig.listViewApiName;
            return mruAdapter(mruConfig, requestContext);
        }
        // if config has objectApiName but no listViewId or listViewApiName then hand off
        // to listViewSummaryCollectionAdapter
        if (looksLikeGetListViewSummaryCollectionConfig(untrustedConfig)) {
            if (listViewSummaryCollectionAdapter === null) {
                listViewSummaryCollectionAdapter =
                    getListViewSummaryCollectionAdapterFactory(luvio);
            }
            return listViewSummaryCollectionAdapter(untrustedConfig, requestContext);
        }
        // see if config looks like a listViewId or listViewApiName request
        if (looksLikeGetListUiByApiNameConfig(untrustedConfig) ||
            looksLikeGetListUiByListViewIdConfig(untrustedConfig)) {
            return listUiAdapter(untrustedConfig, adapterContext, requestContext);
        }
        return null;
    }, { contextId: contextId$5 });
};

function validate$1T(obj, path = 'ChildRelationshipRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_childObjectApiName = obj.childObjectApiName;
        const path_childObjectApiName = path + '.childObjectApiName';
        if (typeof obj_childObjectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_childObjectApiName + '" (at "' + path_childObjectApiName + '")');
        }
        const obj_fieldName = obj.fieldName;
        const path_fieldName = path + '.fieldName';
        if (typeof obj_fieldName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldName + '" (at "' + path_fieldName + '")');
        }
        const obj_junctionIdListNames = obj.junctionIdListNames;
        const path_junctionIdListNames = path + '.junctionIdListNames';
        if (!ArrayIsArray(obj_junctionIdListNames)) {
            return new TypeError('Expected "array" but received "' + typeof obj_junctionIdListNames + '" (at "' + path_junctionIdListNames + '")');
        }
        for (let i = 0; i < obj_junctionIdListNames.length; i++) {
            const obj_junctionIdListNames_item = obj_junctionIdListNames[i];
            const path_junctionIdListNames_item = path_junctionIdListNames + '[' + i + ']';
            if (typeof obj_junctionIdListNames_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_junctionIdListNames_item + '" (at "' + path_junctionIdListNames_item + '")');
            }
        }
        const obj_junctionReferenceTo = obj.junctionReferenceTo;
        const path_junctionReferenceTo = path + '.junctionReferenceTo';
        if (!ArrayIsArray(obj_junctionReferenceTo)) {
            return new TypeError('Expected "array" but received "' + typeof obj_junctionReferenceTo + '" (at "' + path_junctionReferenceTo + '")');
        }
        for (let i = 0; i < obj_junctionReferenceTo.length; i++) {
            const obj_junctionReferenceTo_item = obj_junctionReferenceTo[i];
            const path_junctionReferenceTo_item = path_junctionReferenceTo + '[' + i + ']';
            if (typeof obj_junctionReferenceTo_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_junctionReferenceTo_item + '" (at "' + path_junctionReferenceTo_item + '")');
            }
        }
        const obj_relationshipName = obj.relationshipName;
        const path_relationshipName = path + '.relationshipName';
        if (typeof obj_relationshipName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_relationshipName + '" (at "' + path_relationshipName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1S(obj, path = 'FilteredLookupInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_controllingFields = obj.controllingFields;
        const path_controllingFields = path + '.controllingFields';
        if (!ArrayIsArray(obj_controllingFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_controllingFields + '" (at "' + path_controllingFields + '")');
        }
        for (let i = 0; i < obj_controllingFields.length; i++) {
            const obj_controllingFields_item = obj_controllingFields[i];
            const path_controllingFields_item = path_controllingFields + '[' + i + ']';
            if (typeof obj_controllingFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_controllingFields_item + '" (at "' + path_controllingFields_item + '")');
            }
        }
        const obj_dependent = obj.dependent;
        const path_dependent = path + '.dependent';
        if (typeof obj_dependent !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_dependent + '" (at "' + path_dependent + '")');
        }
        const obj_optionalFilter = obj.optionalFilter;
        const path_optionalFilter = path + '.optionalFilter';
        if (typeof obj_optionalFilter !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_optionalFilter + '" (at "' + path_optionalFilter + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1R(obj, path = 'ReferenceToInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_nameFields = obj.nameFields;
        const path_nameFields = path + '.nameFields';
        if (!ArrayIsArray(obj_nameFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_nameFields + '" (at "' + path_nameFields + '")');
        }
        for (let i = 0; i < obj_nameFields.length; i++) {
            const obj_nameFields_item = obj_nameFields[i];
            const path_nameFields_item = path_nameFields + '[' + i + ']';
            if (typeof obj_nameFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nameFields_item + '" (at "' + path_nameFields_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1Q(obj, path = 'FieldRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_calculated = obj.calculated;
        const path_calculated = path + '.calculated';
        if (typeof obj_calculated !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_calculated + '" (at "' + path_calculated + '")');
        }
        const obj_compound = obj.compound;
        const path_compound = path + '.compound';
        if (typeof obj_compound !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_compound + '" (at "' + path_compound + '")');
        }
        const obj_compoundComponentName = obj.compoundComponentName;
        const path_compoundComponentName = path + '.compoundComponentName';
        let obj_compoundComponentName_union0 = null;
        const obj_compoundComponentName_union0_error = (() => {
            if (typeof obj_compoundComponentName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_compoundComponentName + '" (at "' + path_compoundComponentName + '")');
            }
        })();
        if (obj_compoundComponentName_union0_error != null) {
            obj_compoundComponentName_union0 = obj_compoundComponentName_union0_error.message;
        }
        let obj_compoundComponentName_union1 = null;
        const obj_compoundComponentName_union1_error = (() => {
            if (obj_compoundComponentName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_compoundComponentName + '" (at "' + path_compoundComponentName + '")');
            }
        })();
        if (obj_compoundComponentName_union1_error != null) {
            obj_compoundComponentName_union1 = obj_compoundComponentName_union1_error.message;
        }
        if (obj_compoundComponentName_union0 && obj_compoundComponentName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_compoundComponentName + '")';
            message += '\n' + obj_compoundComponentName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_compoundComponentName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_compoundFieldName = obj.compoundFieldName;
        const path_compoundFieldName = path + '.compoundFieldName';
        let obj_compoundFieldName_union0 = null;
        const obj_compoundFieldName_union0_error = (() => {
            if (typeof obj_compoundFieldName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_compoundFieldName + '" (at "' + path_compoundFieldName + '")');
            }
        })();
        if (obj_compoundFieldName_union0_error != null) {
            obj_compoundFieldName_union0 = obj_compoundFieldName_union0_error.message;
        }
        let obj_compoundFieldName_union1 = null;
        const obj_compoundFieldName_union1_error = (() => {
            if (obj_compoundFieldName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_compoundFieldName + '" (at "' + path_compoundFieldName + '")');
            }
        })();
        if (obj_compoundFieldName_union1_error != null) {
            obj_compoundFieldName_union1 = obj_compoundFieldName_union1_error.message;
        }
        if (obj_compoundFieldName_union0 && obj_compoundFieldName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_compoundFieldName + '")';
            message += '\n' + obj_compoundFieldName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_compoundFieldName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_controllerName = obj.controllerName;
        const path_controllerName = path + '.controllerName';
        let obj_controllerName_union0 = null;
        const obj_controllerName_union0_error = (() => {
            if (typeof obj_controllerName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_controllerName + '" (at "' + path_controllerName + '")');
            }
        })();
        if (obj_controllerName_union0_error != null) {
            obj_controllerName_union0 = obj_controllerName_union0_error.message;
        }
        let obj_controllerName_union1 = null;
        const obj_controllerName_union1_error = (() => {
            if (obj_controllerName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_controllerName + '" (at "' + path_controllerName + '")');
            }
        })();
        if (obj_controllerName_union1_error != null) {
            obj_controllerName_union1 = obj_controllerName_union1_error.message;
        }
        if (obj_controllerName_union0 && obj_controllerName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_controllerName + '")';
            message += '\n' + obj_controllerName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_controllerName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_controllingFields = obj.controllingFields;
        const path_controllingFields = path + '.controllingFields';
        if (!ArrayIsArray(obj_controllingFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_controllingFields + '" (at "' + path_controllingFields + '")');
        }
        for (let i = 0; i < obj_controllingFields.length; i++) {
            const obj_controllingFields_item = obj_controllingFields[i];
            const path_controllingFields_item = path_controllingFields + '[' + i + ']';
            if (typeof obj_controllingFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_controllingFields_item + '" (at "' + path_controllingFields_item + '")');
            }
        }
        const obj_createable = obj.createable;
        const path_createable = path + '.createable';
        if (typeof obj_createable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_createable + '" (at "' + path_createable + '")');
        }
        const obj_custom = obj.custom;
        const path_custom = path + '.custom';
        if (typeof obj_custom !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_custom + '" (at "' + path_custom + '")');
        }
        const obj_dataType = obj.dataType;
        const path_dataType = path + '.dataType';
        if (typeof obj_dataType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
        }
        const obj_externalId = obj.externalId;
        const path_externalId = path + '.externalId';
        if (typeof obj_externalId !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_externalId + '" (at "' + path_externalId + '")');
        }
        const obj_extraTypeInfo = obj.extraTypeInfo;
        const path_extraTypeInfo = path + '.extraTypeInfo';
        let obj_extraTypeInfo_union0 = null;
        const obj_extraTypeInfo_union0_error = (() => {
            if (typeof obj_extraTypeInfo !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_extraTypeInfo + '" (at "' + path_extraTypeInfo + '")');
            }
        })();
        if (obj_extraTypeInfo_union0_error != null) {
            obj_extraTypeInfo_union0 = obj_extraTypeInfo_union0_error.message;
        }
        let obj_extraTypeInfo_union1 = null;
        const obj_extraTypeInfo_union1_error = (() => {
            if (obj_extraTypeInfo !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_extraTypeInfo + '" (at "' + path_extraTypeInfo + '")');
            }
        })();
        if (obj_extraTypeInfo_union1_error != null) {
            obj_extraTypeInfo_union1 = obj_extraTypeInfo_union1_error.message;
        }
        if (obj_extraTypeInfo_union0 && obj_extraTypeInfo_union1) {
            let message = 'Object doesn\'t match union (at "' + path_extraTypeInfo + '")';
            message += '\n' + obj_extraTypeInfo_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_extraTypeInfo_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_filterable = obj.filterable;
        const path_filterable = path + '.filterable';
        if (typeof obj_filterable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_filterable + '" (at "' + path_filterable + '")');
        }
        const obj_filteredLookupInfo = obj.filteredLookupInfo;
        const path_filteredLookupInfo = path + '.filteredLookupInfo';
        let obj_filteredLookupInfo_union0 = null;
        const obj_filteredLookupInfo_union0_error = (() => {
            const referencepath_filteredLookupInfoValidationError = validate$1S(obj_filteredLookupInfo, path_filteredLookupInfo);
            if (referencepath_filteredLookupInfoValidationError !== null) {
                let message = 'Object doesn\'t match FilteredLookupInfoRepresentation (at "' + path_filteredLookupInfo + '")\n';
                message += referencepath_filteredLookupInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_filteredLookupInfo_union0_error != null) {
            obj_filteredLookupInfo_union0 = obj_filteredLookupInfo_union0_error.message;
        }
        let obj_filteredLookupInfo_union1 = null;
        const obj_filteredLookupInfo_union1_error = (() => {
            if (obj_filteredLookupInfo !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_filteredLookupInfo + '" (at "' + path_filteredLookupInfo + '")');
            }
        })();
        if (obj_filteredLookupInfo_union1_error != null) {
            obj_filteredLookupInfo_union1 = obj_filteredLookupInfo_union1_error.message;
        }
        if (obj_filteredLookupInfo_union0 && obj_filteredLookupInfo_union1) {
            let message = 'Object doesn\'t match union (at "' + path_filteredLookupInfo + '")';
            message += '\n' + obj_filteredLookupInfo_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_filteredLookupInfo_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_highScaleNumber = obj.highScaleNumber;
        const path_highScaleNumber = path + '.highScaleNumber';
        if (typeof obj_highScaleNumber !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_highScaleNumber + '" (at "' + path_highScaleNumber + '")');
        }
        const obj_htmlFormatted = obj.htmlFormatted;
        const path_htmlFormatted = path + '.htmlFormatted';
        if (typeof obj_htmlFormatted !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_htmlFormatted + '" (at "' + path_htmlFormatted + '")');
        }
        const obj_inlineHelpText = obj.inlineHelpText;
        const path_inlineHelpText = path + '.inlineHelpText';
        let obj_inlineHelpText_union0 = null;
        const obj_inlineHelpText_union0_error = (() => {
            if (typeof obj_inlineHelpText !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_inlineHelpText + '" (at "' + path_inlineHelpText + '")');
            }
        })();
        if (obj_inlineHelpText_union0_error != null) {
            obj_inlineHelpText_union0 = obj_inlineHelpText_union0_error.message;
        }
        let obj_inlineHelpText_union1 = null;
        const obj_inlineHelpText_union1_error = (() => {
            if (obj_inlineHelpText !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_inlineHelpText + '" (at "' + path_inlineHelpText + '")');
            }
        })();
        if (obj_inlineHelpText_union1_error != null) {
            obj_inlineHelpText_union1 = obj_inlineHelpText_union1_error.message;
        }
        if (obj_inlineHelpText_union0 && obj_inlineHelpText_union1) {
            let message = 'Object doesn\'t match union (at "' + path_inlineHelpText + '")';
            message += '\n' + obj_inlineHelpText_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_inlineHelpText_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_length = obj.length;
        const path_length = path + '.length';
        if (typeof obj_length !== 'number' || (typeof obj_length === 'number' && Math.floor(obj_length) !== obj_length)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_length + '" (at "' + path_length + '")');
        }
        const obj_maskType = obj.maskType;
        const path_maskType = path + '.maskType';
        let obj_maskType_union0 = null;
        const obj_maskType_union0_error = (() => {
            if (typeof obj_maskType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_maskType + '" (at "' + path_maskType + '")');
            }
        })();
        if (obj_maskType_union0_error != null) {
            obj_maskType_union0 = obj_maskType_union0_error.message;
        }
        let obj_maskType_union1 = null;
        const obj_maskType_union1_error = (() => {
            if (obj_maskType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_maskType + '" (at "' + path_maskType + '")');
            }
        })();
        if (obj_maskType_union1_error != null) {
            obj_maskType_union1 = obj_maskType_union1_error.message;
        }
        if (obj_maskType_union0 && obj_maskType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_maskType + '")';
            message += '\n' + obj_maskType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_maskType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nameField = obj.nameField;
        const path_nameField = path + '.nameField';
        if (typeof obj_nameField !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_nameField + '" (at "' + path_nameField + '")');
        }
        const obj_polymorphicForeignKey = obj.polymorphicForeignKey;
        const path_polymorphicForeignKey = path + '.polymorphicForeignKey';
        if (typeof obj_polymorphicForeignKey !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_polymorphicForeignKey + '" (at "' + path_polymorphicForeignKey + '")');
        }
        const obj_precision = obj.precision;
        const path_precision = path + '.precision';
        if (typeof obj_precision !== 'number' || (typeof obj_precision === 'number' && Math.floor(obj_precision) !== obj_precision)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_precision + '" (at "' + path_precision + '")');
        }
        const obj_reference = obj.reference;
        const path_reference = path + '.reference';
        if (typeof obj_reference !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_reference + '" (at "' + path_reference + '")');
        }
        const obj_referenceTargetField = obj.referenceTargetField;
        const path_referenceTargetField = path + '.referenceTargetField';
        let obj_referenceTargetField_union0 = null;
        const obj_referenceTargetField_union0_error = (() => {
            if (typeof obj_referenceTargetField !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_referenceTargetField + '" (at "' + path_referenceTargetField + '")');
            }
        })();
        if (obj_referenceTargetField_union0_error != null) {
            obj_referenceTargetField_union0 = obj_referenceTargetField_union0_error.message;
        }
        let obj_referenceTargetField_union1 = null;
        const obj_referenceTargetField_union1_error = (() => {
            if (obj_referenceTargetField !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_referenceTargetField + '" (at "' + path_referenceTargetField + '")');
            }
        })();
        if (obj_referenceTargetField_union1_error != null) {
            obj_referenceTargetField_union1 = obj_referenceTargetField_union1_error.message;
        }
        if (obj_referenceTargetField_union0 && obj_referenceTargetField_union1) {
            let message = 'Object doesn\'t match union (at "' + path_referenceTargetField + '")';
            message += '\n' + obj_referenceTargetField_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_referenceTargetField_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_referenceToInfos = obj.referenceToInfos;
        const path_referenceToInfos = path + '.referenceToInfos';
        if (!ArrayIsArray(obj_referenceToInfos)) {
            return new TypeError('Expected "array" but received "' + typeof obj_referenceToInfos + '" (at "' + path_referenceToInfos + '")');
        }
        for (let i = 0; i < obj_referenceToInfos.length; i++) {
            const obj_referenceToInfos_item = obj_referenceToInfos[i];
            const path_referenceToInfos_item = path_referenceToInfos + '[' + i + ']';
            const referencepath_referenceToInfos_itemValidationError = validate$1R(obj_referenceToInfos_item, path_referenceToInfos_item);
            if (referencepath_referenceToInfos_itemValidationError !== null) {
                let message = 'Object doesn\'t match ReferenceToInfoRepresentation (at "' + path_referenceToInfos_item + '")\n';
                message += referencepath_referenceToInfos_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_relationshipName = obj.relationshipName;
        const path_relationshipName = path + '.relationshipName';
        let obj_relationshipName_union0 = null;
        const obj_relationshipName_union0_error = (() => {
            if (typeof obj_relationshipName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_relationshipName + '" (at "' + path_relationshipName + '")');
            }
        })();
        if (obj_relationshipName_union0_error != null) {
            obj_relationshipName_union0 = obj_relationshipName_union0_error.message;
        }
        let obj_relationshipName_union1 = null;
        const obj_relationshipName_union1_error = (() => {
            if (obj_relationshipName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_relationshipName + '" (at "' + path_relationshipName + '")');
            }
        })();
        if (obj_relationshipName_union1_error != null) {
            obj_relationshipName_union1 = obj_relationshipName_union1_error.message;
        }
        if (obj_relationshipName_union0 && obj_relationshipName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_relationshipName + '")';
            message += '\n' + obj_relationshipName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_relationshipName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_required = obj.required;
        const path_required = path + '.required';
        if (typeof obj_required !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_required + '" (at "' + path_required + '")');
        }
        const obj_scale = obj.scale;
        const path_scale = path + '.scale';
        if (typeof obj_scale !== 'number' || (typeof obj_scale === 'number' && Math.floor(obj_scale) !== obj_scale)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_scale + '" (at "' + path_scale + '")');
        }
        const obj_searchPrefilterable = obj.searchPrefilterable;
        const path_searchPrefilterable = path + '.searchPrefilterable';
        if (typeof obj_searchPrefilterable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_searchPrefilterable + '" (at "' + path_searchPrefilterable + '")');
        }
        const obj_sortable = obj.sortable;
        const path_sortable = path + '.sortable';
        if (typeof obj_sortable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_sortable + '" (at "' + path_sortable + '")');
        }
        const obj_unique = obj.unique;
        const path_unique = path + '.unique';
        if (typeof obj_unique !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_unique + '" (at "' + path_unique + '")');
        }
        const obj_updateable = obj.updateable;
        const path_updateable = path + '.updateable';
        if (typeof obj_updateable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_updateable + '" (at "' + path_updateable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1P(obj, path = 'ThemeInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_color = obj.color;
        const path_color = path + '.color';
        if (typeof obj_color !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_color + '" (at "' + path_color + '")');
        }
        const obj_iconUrl = obj.iconUrl;
        const path_iconUrl = path + '.iconUrl';
        let obj_iconUrl_union0 = null;
        const obj_iconUrl_union0_error = (() => {
            if (typeof obj_iconUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union0_error != null) {
            obj_iconUrl_union0 = obj_iconUrl_union0_error.message;
        }
        let obj_iconUrl_union1 = null;
        const obj_iconUrl_union1_error = (() => {
            if (obj_iconUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union1_error != null) {
            obj_iconUrl_union1 = obj_iconUrl_union1_error.message;
        }
        if (obj_iconUrl_union0 && obj_iconUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_iconUrl + '")';
            message += '\n' + obj_iconUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_iconUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$D = 900000;
const VERSION$2C = "2302a71ca718abb60353eb88dedb1c12";
function validate$1O(obj, path = 'ObjectInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_associateEntityType = obj.associateEntityType;
        const path_associateEntityType = path + '.associateEntityType';
        let obj_associateEntityType_union0 = null;
        const obj_associateEntityType_union0_error = (() => {
            if (typeof obj_associateEntityType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_associateEntityType + '" (at "' + path_associateEntityType + '")');
            }
        })();
        if (obj_associateEntityType_union0_error != null) {
            obj_associateEntityType_union0 = obj_associateEntityType_union0_error.message;
        }
        let obj_associateEntityType_union1 = null;
        const obj_associateEntityType_union1_error = (() => {
            if (obj_associateEntityType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_associateEntityType + '" (at "' + path_associateEntityType + '")');
            }
        })();
        if (obj_associateEntityType_union1_error != null) {
            obj_associateEntityType_union1 = obj_associateEntityType_union1_error.message;
        }
        if (obj_associateEntityType_union0 && obj_associateEntityType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_associateEntityType + '")';
            message += '\n' + obj_associateEntityType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_associateEntityType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_associateParentEntity = obj.associateParentEntity;
        const path_associateParentEntity = path + '.associateParentEntity';
        let obj_associateParentEntity_union0 = null;
        const obj_associateParentEntity_union0_error = (() => {
            if (typeof obj_associateParentEntity !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_associateParentEntity + '" (at "' + path_associateParentEntity + '")');
            }
        })();
        if (obj_associateParentEntity_union0_error != null) {
            obj_associateParentEntity_union0 = obj_associateParentEntity_union0_error.message;
        }
        let obj_associateParentEntity_union1 = null;
        const obj_associateParentEntity_union1_error = (() => {
            if (obj_associateParentEntity !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_associateParentEntity + '" (at "' + path_associateParentEntity + '")');
            }
        })();
        if (obj_associateParentEntity_union1_error != null) {
            obj_associateParentEntity_union1 = obj_associateParentEntity_union1_error.message;
        }
        if (obj_associateParentEntity_union0 && obj_associateParentEntity_union1) {
            let message = 'Object doesn\'t match union (at "' + path_associateParentEntity + '")';
            message += '\n' + obj_associateParentEntity_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_associateParentEntity_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_childRelationships = obj.childRelationships;
        const path_childRelationships = path + '.childRelationships';
        if (!ArrayIsArray(obj_childRelationships)) {
            return new TypeError('Expected "array" but received "' + typeof obj_childRelationships + '" (at "' + path_childRelationships + '")');
        }
        for (let i = 0; i < obj_childRelationships.length; i++) {
            const obj_childRelationships_item = obj_childRelationships[i];
            const path_childRelationships_item = path_childRelationships + '[' + i + ']';
            const referencepath_childRelationships_itemValidationError = validate$1T(obj_childRelationships_item, path_childRelationships_item);
            if (referencepath_childRelationships_itemValidationError !== null) {
                let message = 'Object doesn\'t match ChildRelationshipRepresentation (at "' + path_childRelationships_item + '")\n';
                message += referencepath_childRelationships_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_compactLayoutable = obj.compactLayoutable;
        const path_compactLayoutable = path + '.compactLayoutable';
        if (typeof obj_compactLayoutable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_compactLayoutable + '" (at "' + path_compactLayoutable + '")');
        }
        const obj_createable = obj.createable;
        const path_createable = path + '.createable';
        if (typeof obj_createable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_createable + '" (at "' + path_createable + '")');
        }
        const obj_custom = obj.custom;
        const path_custom = path + '.custom';
        if (typeof obj_custom !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_custom + '" (at "' + path_custom + '")');
        }
        const obj_defaultRecordTypeId = obj.defaultRecordTypeId;
        const path_defaultRecordTypeId = path + '.defaultRecordTypeId';
        let obj_defaultRecordTypeId_union0 = null;
        const obj_defaultRecordTypeId_union0_error = (() => {
            if (typeof obj_defaultRecordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_defaultRecordTypeId + '" (at "' + path_defaultRecordTypeId + '")');
            }
        })();
        if (obj_defaultRecordTypeId_union0_error != null) {
            obj_defaultRecordTypeId_union0 = obj_defaultRecordTypeId_union0_error.message;
        }
        let obj_defaultRecordTypeId_union1 = null;
        const obj_defaultRecordTypeId_union1_error = (() => {
            if (obj_defaultRecordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_defaultRecordTypeId + '" (at "' + path_defaultRecordTypeId + '")');
            }
        })();
        if (obj_defaultRecordTypeId_union1_error != null) {
            obj_defaultRecordTypeId_union1 = obj_defaultRecordTypeId_union1_error.message;
        }
        if (obj_defaultRecordTypeId_union0 && obj_defaultRecordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_defaultRecordTypeId + '")';
            message += '\n' + obj_defaultRecordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_defaultRecordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_deletable = obj.deletable;
        const path_deletable = path + '.deletable';
        if (typeof obj_deletable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_deletable + '" (at "' + path_deletable + '")');
        }
        const obj_dependentFields = obj.dependentFields;
        const path_dependentFields = path + '.dependentFields';
        if (typeof obj_dependentFields !== 'object' || ArrayIsArray(obj_dependentFields) || obj_dependentFields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_dependentFields + '" (at "' + path_dependentFields + '")');
        }
        const obj_dependentFields_keys = ObjectKeys(obj_dependentFields);
        for (let i = 0; i < obj_dependentFields_keys.length; i++) {
            const key = obj_dependentFields_keys[i];
            const obj_dependentFields_prop = obj_dependentFields[key];
            const path_dependentFields_prop = path_dependentFields + '["' + key + '"]';
            if (typeof obj_dependentFields_prop !== 'object' || ArrayIsArray(obj_dependentFields_prop) || obj_dependentFields_prop === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_dependentFields_prop + '" (at "' + path_dependentFields_prop + '")');
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_feedEnabled = obj.feedEnabled;
        const path_feedEnabled = path + '.feedEnabled';
        if (typeof obj_feedEnabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_feedEnabled + '" (at "' + path_feedEnabled + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            const referencepath_fields_propValidationError = validate$1Q(obj_fields_prop, path_fields_prop);
            if (referencepath_fields_propValidationError !== null) {
                let message = 'Object doesn\'t match FieldRepresentation (at "' + path_fields_prop + '")\n';
                message += referencepath_fields_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_keyPrefix = obj.keyPrefix;
        const path_keyPrefix = path + '.keyPrefix';
        let obj_keyPrefix_union0 = null;
        const obj_keyPrefix_union0_error = (() => {
            if (typeof obj_keyPrefix !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_keyPrefix + '" (at "' + path_keyPrefix + '")');
            }
        })();
        if (obj_keyPrefix_union0_error != null) {
            obj_keyPrefix_union0 = obj_keyPrefix_union0_error.message;
        }
        let obj_keyPrefix_union1 = null;
        const obj_keyPrefix_union1_error = (() => {
            if (obj_keyPrefix !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_keyPrefix + '" (at "' + path_keyPrefix + '")');
            }
        })();
        if (obj_keyPrefix_union1_error != null) {
            obj_keyPrefix_union1 = obj_keyPrefix_union1_error.message;
        }
        if (obj_keyPrefix_union0 && obj_keyPrefix_union1) {
            let message = 'Object doesn\'t match union (at "' + path_keyPrefix + '")';
            message += '\n' + obj_keyPrefix_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_keyPrefix_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_labelPlural = obj.labelPlural;
        const path_labelPlural = path + '.labelPlural';
        if (typeof obj_labelPlural !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_labelPlural + '" (at "' + path_labelPlural + '")');
        }
        const obj_layoutable = obj.layoutable;
        const path_layoutable = path + '.layoutable';
        if (typeof obj_layoutable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_layoutable + '" (at "' + path_layoutable + '")');
        }
        const obj_mruEnabled = obj.mruEnabled;
        const path_mruEnabled = path + '.mruEnabled';
        if (typeof obj_mruEnabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_mruEnabled + '" (at "' + path_mruEnabled + '")');
        }
        const obj_nameFields = obj.nameFields;
        const path_nameFields = path + '.nameFields';
        if (!ArrayIsArray(obj_nameFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_nameFields + '" (at "' + path_nameFields + '")');
        }
        for (let i = 0; i < obj_nameFields.length; i++) {
            const obj_nameFields_item = obj_nameFields[i];
            const path_nameFields_item = path_nameFields + '[' + i + ']';
            if (typeof obj_nameFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nameFields_item + '" (at "' + path_nameFields_item + '")');
            }
        }
        const obj_queryable = obj.queryable;
        const path_queryable = path + '.queryable';
        if (typeof obj_queryable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_queryable + '" (at "' + path_queryable + '")');
        }
        const obj_recordTypeInfos = obj.recordTypeInfos;
        const path_recordTypeInfos = path + '.recordTypeInfos';
        if (typeof obj_recordTypeInfos !== 'object' || ArrayIsArray(obj_recordTypeInfos) || obj_recordTypeInfos === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_recordTypeInfos + '" (at "' + path_recordTypeInfos + '")');
        }
        const obj_recordTypeInfos_keys = ObjectKeys(obj_recordTypeInfos);
        for (let i = 0; i < obj_recordTypeInfos_keys.length; i++) {
            const key = obj_recordTypeInfos_keys[i];
            const obj_recordTypeInfos_prop = obj_recordTypeInfos[key];
            const path_recordTypeInfos_prop = path_recordTypeInfos + '["' + key + '"]';
            const referencepath_recordTypeInfos_propValidationError = validate$21(obj_recordTypeInfos_prop, path_recordTypeInfos_prop);
            if (referencepath_recordTypeInfos_propValidationError !== null) {
                let message = 'Object doesn\'t match RecordTypeInfoRepresentation (at "' + path_recordTypeInfos_prop + '")\n';
                message += referencepath_recordTypeInfos_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_searchLayoutable = obj.searchLayoutable;
        const path_searchLayoutable = path + '.searchLayoutable';
        if (typeof obj_searchLayoutable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_searchLayoutable + '" (at "' + path_searchLayoutable + '")');
        }
        const obj_searchable = obj.searchable;
        const path_searchable = path + '.searchable';
        if (typeof obj_searchable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_searchable + '" (at "' + path_searchable + '")');
        }
        const obj_themeInfo = obj.themeInfo;
        const path_themeInfo = path + '.themeInfo';
        let obj_themeInfo_union0 = null;
        const obj_themeInfo_union0_error = (() => {
            const referencepath_themeInfoValidationError = validate$1P(obj_themeInfo, path_themeInfo);
            if (referencepath_themeInfoValidationError !== null) {
                let message = 'Object doesn\'t match ThemeInfoRepresentation (at "' + path_themeInfo + '")\n';
                message += referencepath_themeInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_themeInfo_union0_error != null) {
            obj_themeInfo_union0 = obj_themeInfo_union0_error.message;
        }
        let obj_themeInfo_union1 = null;
        const obj_themeInfo_union1_error = (() => {
            if (obj_themeInfo !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_themeInfo + '" (at "' + path_themeInfo + '")');
            }
        })();
        if (obj_themeInfo_union1_error != null) {
            obj_themeInfo_union1 = obj_themeInfo_union1_error.message;
        }
        if (obj_themeInfo_union0 && obj_themeInfo_union1) {
            let message = 'Object doesn\'t match union (at "' + path_themeInfo + '")';
            message += '\n' + obj_themeInfo_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_themeInfo_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_updateable = obj.updateable;
        const path_updateable = path + '.updateable';
        if (typeof obj_updateable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_updateable + '" (at "' + path_updateable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$T = 'ObjectInfoRepresentation';
function keyBuilder$3n(luvio, config) {
    return keyPrefix + '::' + RepresentationType$T + ':' + config.apiName;
}
function keyBuilderFromType$w(luvio, object) {
    const keyParams = {
        apiName: object.apiName
    };
    return keyBuilder$3n(luvio, keyParams);
}
function normalize$L(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3e = function ObjectInfoRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2C,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'associateEntityType',
                kind: 'Scalar'
            },
            {
                name: 'associateParentEntity',
                kind: 'Scalar'
            },
            {
                name: 'childRelationships',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'compactLayoutable',
                kind: 'Scalar'
            },
            {
                name: 'createable',
                kind: 'Scalar'
            },
            {
                name: 'custom',
                kind: 'Scalar'
            },
            {
                name: 'defaultRecordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'deletable',
                kind: 'Scalar'
            },
            {
                name: 'dependentFields',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'feedEnabled',
                kind: 'Scalar'
            },
            {
                name: 'fields',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'keyPrefix',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'labelPlural',
                kind: 'Scalar'
            },
            {
                name: 'layoutable',
                kind: 'Scalar'
            },
            {
                name: 'mruEnabled',
                kind: 'Scalar'
            },
            {
                name: 'nameFields',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'queryable',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeInfos',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'searchLayoutable',
                kind: 'Scalar'
            },
            {
                name: 'searchable',
                kind: 'Scalar'
            },
            {
                name: 'themeInfo',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'updateable',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$12(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$29 = function ObjectInfoRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1O(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$w(luvio, input);
    const ttlToUse = TTL$D;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$L, "UiApi", VERSION$2C, RepresentationType$T, equals$12);
    return createLink$1(key);
};
function getTypeCacheKeys$2c(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$w(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$T,
        mergeable: false
    });
}

function select$3d(luvio, params) {
    return select$3e();
}
function keyBuilder$3m(luvio, params) {
    return keyBuilder$3n(luvio, {
        apiName: params.urlParams.objectApiName
    });
}
function getResponseCacheKeys$11(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2c(storeKeyMap, luvio, response);
}
function ingestSuccess$S(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$3m(luvio, resourceParams);
    luvio.storeIngest(key, ingest$29, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$3d(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$L(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$3m(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$D,
        namespace: keyPrefix,
        version: VERSION$2C,
        representationName: RepresentationType$T
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$17(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/object-info/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$W = 'getObjectInfo';
const getObjectInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
];
const getObjectInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$W, getObjectInfo_ConfigPropertyMetadata);
const createResourceParams$$ = /*#__PURE__*/ createResourceParams$k(getObjectInfo_ConfigPropertyMetadata);
function keyBuilder$3l(luvio, config) {
    const resourceParams = createResourceParams$$(config);
    return keyBuilder$3m(luvio, resourceParams);
}
function typeCheckConfig$14(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getObjectInfo_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$17(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getObjectInfo_ConfigPropertyMetadata);
    const config = typeCheckConfig$14(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$K(luvio, config) {
    createResourceParams$$(config);
    return select$3d();
}
function onFetchResponseSuccess$O(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$S(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$16(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$P(luvio, config, resourceParams, response) {
    const snapshot = ingestError$L(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$16(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$16(luvio, config, options) {
    const resourceParams = createResourceParams$$(config);
    const request = createResourceRequest$17(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$O(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$11(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$P(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$S(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$16, undefined, false);
}
function buildCachedSnapshotCachePolicy$R(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$3l(luvio, config),
        node: adapterFragment$K(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$16(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getObjectInfoAdapterFactory = (luvio) => function UiApi__getObjectInfo(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$17(untrustedConfig, getObjectInfo_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$R, buildNetworkSnapshotCachePolicy$S);
};

function validate$1N(obj, path = 'RecordLayoutSaveOptionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_defaultValue = obj.defaultValue;
        const path_defaultValue = path + '.defaultValue';
        if (typeof obj_defaultValue !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_defaultValue + '" (at "' + path_defaultValue + '")');
        }
        const obj_isDisplayed = obj.isDisplayed;
        const path_isDisplayed = path + '.isDisplayed';
        if (typeof obj_isDisplayed !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isDisplayed + '" (at "' + path_isDisplayed + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_name = obj.name;
        const path_name = path + '.name';
        if (typeof obj_name !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_name + '" (at "' + path_name + '")');
        }
        const obj_restHeaderName = obj.restHeaderName;
        const path_restHeaderName = path + '.restHeaderName';
        if (typeof obj_restHeaderName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_restHeaderName + '" (at "' + path_restHeaderName + '")');
        }
        const obj_soapHeaderName = obj.soapHeaderName;
        const path_soapHeaderName = path + '.soapHeaderName';
        if (typeof obj_soapHeaderName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_soapHeaderName + '" (at "' + path_soapHeaderName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

var DiscriminatorValues$5;
(function (DiscriminatorValues) {
    DiscriminatorValues["EmptySpace"] = "EmptySpace";
    DiscriminatorValues["ReportChart"] = "ReportChart";
    DiscriminatorValues["VisualforcePage"] = "VisualforcePage";
    DiscriminatorValues["Canvas"] = "Canvas";
    DiscriminatorValues["Field"] = "Field";
    DiscriminatorValues["CustomLink"] = "CustomLink";
})(DiscriminatorValues$5 || (DiscriminatorValues$5 = {}));
function validate$1M(obj, path = 'AbstractRecordLayoutComponentRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        let obj_apiName_union0 = null;
        const obj_apiName_union0_error = (() => {
            if (typeof obj_apiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
            }
        })();
        if (obj_apiName_union0_error != null) {
            obj_apiName_union0 = obj_apiName_union0_error.message;
        }
        let obj_apiName_union1 = null;
        const obj_apiName_union1_error = (() => {
            if (obj_apiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
            }
        })();
        if (obj_apiName_union1_error != null) {
            obj_apiName_union1 = obj_apiName_union1_error.message;
        }
        if (obj_apiName_union0 && obj_apiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_apiName + '")';
            message += '\n' + obj_apiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_apiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_componentType = obj.componentType;
        const path_componentType = path + '.componentType';
        if (typeof obj_componentType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_componentType + '" (at "' + path_componentType + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1L(obj, path = 'RecordLayoutItemRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_editableForNew = obj.editableForNew;
        const path_editableForNew = path + '.editableForNew';
        if (typeof obj_editableForNew !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_editableForNew + '" (at "' + path_editableForNew + '")');
        }
        const obj_editableForUpdate = obj.editableForUpdate;
        const path_editableForUpdate = path + '.editableForUpdate';
        if (typeof obj_editableForUpdate !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_editableForUpdate + '" (at "' + path_editableForUpdate + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_layoutComponents = obj.layoutComponents;
        const path_layoutComponents = path + '.layoutComponents';
        if (!ArrayIsArray(obj_layoutComponents)) {
            return new TypeError('Expected "array" but received "' + typeof obj_layoutComponents + '" (at "' + path_layoutComponents + '")');
        }
        for (let i = 0; i < obj_layoutComponents.length; i++) {
            const obj_layoutComponents_item = obj_layoutComponents[i];
            const path_layoutComponents_item = path_layoutComponents + '[' + i + ']';
            const referencepath_layoutComponents_itemValidationError = validate$1M(obj_layoutComponents_item, path_layoutComponents_item);
            if (referencepath_layoutComponents_itemValidationError !== null) {
                let message = 'Object doesn\'t match AbstractRecordLayoutComponentRepresentation (at "' + path_layoutComponents_item + '")\n';
                message += referencepath_layoutComponents_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_lookupIdApiName = obj.lookupIdApiName;
        const path_lookupIdApiName = path + '.lookupIdApiName';
        let obj_lookupIdApiName_union0 = null;
        const obj_lookupIdApiName_union0_error = (() => {
            if (typeof obj_lookupIdApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lookupIdApiName + '" (at "' + path_lookupIdApiName + '")');
            }
        })();
        if (obj_lookupIdApiName_union0_error != null) {
            obj_lookupIdApiName_union0 = obj_lookupIdApiName_union0_error.message;
        }
        let obj_lookupIdApiName_union1 = null;
        const obj_lookupIdApiName_union1_error = (() => {
            if (obj_lookupIdApiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lookupIdApiName + '" (at "' + path_lookupIdApiName + '")');
            }
        })();
        if (obj_lookupIdApiName_union1_error != null) {
            obj_lookupIdApiName_union1 = obj_lookupIdApiName_union1_error.message;
        }
        if (obj_lookupIdApiName_union0 && obj_lookupIdApiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lookupIdApiName + '")';
            message += '\n' + obj_lookupIdApiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lookupIdApiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_required = obj.required;
        const path_required = path + '.required';
        if (typeof obj_required !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_required + '" (at "' + path_required + '")');
        }
        const obj_sortable = obj.sortable;
        const path_sortable = path + '.sortable';
        if (typeof obj_sortable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_sortable + '" (at "' + path_sortable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1K(obj, path = 'RecordLayoutRowRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_layoutItems = obj.layoutItems;
        const path_layoutItems = path + '.layoutItems';
        if (!ArrayIsArray(obj_layoutItems)) {
            return new TypeError('Expected "array" but received "' + typeof obj_layoutItems + '" (at "' + path_layoutItems + '")');
        }
        for (let i = 0; i < obj_layoutItems.length; i++) {
            const obj_layoutItems_item = obj_layoutItems[i];
            const path_layoutItems_item = path_layoutItems + '[' + i + ']';
            const referencepath_layoutItems_itemValidationError = validate$1L(obj_layoutItems_item, path_layoutItems_item);
            if (referencepath_layoutItems_itemValidationError !== null) {
                let message = 'Object doesn\'t match RecordLayoutItemRepresentation (at "' + path_layoutItems_item + '")\n';
                message += referencepath_layoutItems_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1J(obj, path = 'RecordLayoutSectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_collapsible = obj.collapsible;
        const path_collapsible = path + '.collapsible';
        if (typeof obj_collapsible !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_collapsible + '" (at "' + path_collapsible + '")');
        }
        const obj_columns = obj.columns;
        const path_columns = path + '.columns';
        if (typeof obj_columns !== 'number' || (typeof obj_columns === 'number' && Math.floor(obj_columns) !== obj_columns)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_columns + '" (at "' + path_columns + '")');
        }
        const obj_heading = obj.heading;
        const path_heading = path + '.heading';
        let obj_heading_union0 = null;
        const obj_heading_union0_error = (() => {
            if (typeof obj_heading !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_heading + '" (at "' + path_heading + '")');
            }
        })();
        if (obj_heading_union0_error != null) {
            obj_heading_union0 = obj_heading_union0_error.message;
        }
        let obj_heading_union1 = null;
        const obj_heading_union1_error = (() => {
            if (obj_heading !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_heading + '" (at "' + path_heading + '")');
            }
        })();
        if (obj_heading_union1_error != null) {
            obj_heading_union1 = obj_heading_union1_error.message;
        }
        if (obj_heading_union0 && obj_heading_union1) {
            let message = 'Object doesn\'t match union (at "' + path_heading + '")';
            message += '\n' + obj_heading_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_heading_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_layoutRows = obj.layoutRows;
        const path_layoutRows = path + '.layoutRows';
        if (!ArrayIsArray(obj_layoutRows)) {
            return new TypeError('Expected "array" but received "' + typeof obj_layoutRows + '" (at "' + path_layoutRows + '")');
        }
        for (let i = 0; i < obj_layoutRows.length; i++) {
            const obj_layoutRows_item = obj_layoutRows[i];
            const path_layoutRows_item = path_layoutRows + '[' + i + ']';
            const referencepath_layoutRows_itemValidationError = validate$1K(obj_layoutRows_item, path_layoutRows_item);
            if (referencepath_layoutRows_itemValidationError !== null) {
                let message = 'Object doesn\'t match RecordLayoutRowRepresentation (at "' + path_layoutRows_item + '")\n';
                message += referencepath_layoutRows_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_rows = obj.rows;
        const path_rows = path + '.rows';
        if (typeof obj_rows !== 'number' || (typeof obj_rows === 'number' && Math.floor(obj_rows) !== obj_rows)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_rows + '" (at "' + path_rows + '")');
        }
        const obj_useHeading = obj.useHeading;
        const path_useHeading = path + '.useHeading';
        if (typeof obj_useHeading !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_useHeading + '" (at "' + path_useHeading + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$C = 900000;
const VERSION$2B = "fb515e25a89ca1ec154dc865e72b913a";
function validate$1I(obj, path = 'RecordLayoutRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_layoutType = obj.layoutType;
        const path_layoutType = path + '.layoutType';
        if (typeof obj_layoutType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_layoutType + '" (at "' + path_layoutType + '")');
        }
        const obj_mode = obj.mode;
        const path_mode = path + '.mode';
        if (typeof obj_mode !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_mode + '" (at "' + path_mode + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        let obj_objectApiName_union0 = null;
        const obj_objectApiName_union0_error = (() => {
            if (typeof obj_objectApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
            }
        })();
        if (obj_objectApiName_union0_error != null) {
            obj_objectApiName_union0 = obj_objectApiName_union0_error.message;
        }
        let obj_objectApiName_union1 = null;
        const obj_objectApiName_union1_error = (() => {
            if (obj_objectApiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
            }
        })();
        if (obj_objectApiName_union1_error != null) {
            obj_objectApiName_union1 = obj_objectApiName_union1_error.message;
        }
        if (obj_objectApiName_union0 && obj_objectApiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_objectApiName + '")';
            message += '\n' + obj_objectApiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_objectApiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        let obj_recordTypeId_union0 = null;
        const obj_recordTypeId_union0_error = (() => {
            if (typeof obj_recordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union0_error != null) {
            obj_recordTypeId_union0 = obj_recordTypeId_union0_error.message;
        }
        let obj_recordTypeId_union1 = null;
        const obj_recordTypeId_union1_error = (() => {
            if (obj_recordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union1_error != null) {
            obj_recordTypeId_union1 = obj_recordTypeId_union1_error.message;
        }
        if (obj_recordTypeId_union0 && obj_recordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeId + '")';
            message += '\n' + obj_recordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_saveOptions = obj.saveOptions;
        const path_saveOptions = path + '.saveOptions';
        if (!ArrayIsArray(obj_saveOptions)) {
            return new TypeError('Expected "array" but received "' + typeof obj_saveOptions + '" (at "' + path_saveOptions + '")');
        }
        for (let i = 0; i < obj_saveOptions.length; i++) {
            const obj_saveOptions_item = obj_saveOptions[i];
            const path_saveOptions_item = path_saveOptions + '[' + i + ']';
            const referencepath_saveOptions_itemValidationError = validate$1N(obj_saveOptions_item, path_saveOptions_item);
            if (referencepath_saveOptions_itemValidationError !== null) {
                let message = 'Object doesn\'t match RecordLayoutSaveOptionRepresentation (at "' + path_saveOptions_item + '")\n';
                message += referencepath_saveOptions_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_sections = obj.sections;
        const path_sections = path + '.sections';
        if (!ArrayIsArray(obj_sections)) {
            return new TypeError('Expected "array" but received "' + typeof obj_sections + '" (at "' + path_sections + '")');
        }
        for (let i = 0; i < obj_sections.length; i++) {
            const obj_sections_item = obj_sections[i];
            const path_sections_item = path_sections + '[' + i + ']';
            const referencepath_sections_itemValidationError = validate$1J(obj_sections_item, path_sections_item);
            if (referencepath_sections_itemValidationError !== null) {
                let message = 'Object doesn\'t match RecordLayoutSectionRepresentation (at "' + path_sections_item + '")\n';
                message += referencepath_sections_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$S = 'RecordLayoutRepresentation';
function keyBuilder$3k(luvio, config) {
    return keyPrefix + '::' + RepresentationType$S + ':' + (config.recordTypeId === null ? '' : config.recordTypeId) + ':' + (config.objectApiName === null ? '' : config.objectApiName) + ':' + config.layoutType + ':' + config.mode;
}
function keyBuilderFromType$v(luvio, object) {
    const keyParams = {
        recordTypeId: object.recordTypeId,
        objectApiName: object.objectApiName,
        layoutType: object.layoutType,
        mode: object.mode
    };
    return keyBuilder$3k(luvio, keyParams);
}
function normalize$K(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3c = function RecordLayoutRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2B,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'layoutType',
                kind: 'Scalar'
            },
            {
                name: 'mode',
                kind: 'Scalar'
            },
            {
                name: 'objectApiName',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'saveOptions',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'sections',
                kind: 'Object',
                opaque: true
            }
        ]
    };
};
function equals$11(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$28 = function RecordLayoutRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1I(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$v(luvio, input);
    const ttlToUse = TTL$C;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$K, "UiApi", VERSION$2B, RepresentationType$S, equals$11);
    return createLink$1(key);
};
function getTypeCacheKeys$2b(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$v(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$S,
        mergeable: false
    });
}

/**
 * Returns the object API name.
 * @param value The value from which to get the object API name.
 * @returns The object API name.
 */
function getRecordId18Array(value) {
    const valueArray = isArray(value) ? value : [value];
    const array = [];
    for (let i = 0, len = valueArray.length; i < len; i += 1) {
        const item = valueArray[i];
        const apiName = getRecordId18(item);
        if (apiName === undefined) {
            return undefined;
        }
        push.call(array, apiName);
    }
    if (array.length === 0) {
        return undefined;
    }
    return dedupe(array).sort();
}

function validate$1H(obj, path = 'RecordLayoutSectionUserStateRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_collapsed = obj.collapsed;
        const path_collapsed = path + '.collapsed';
        if (typeof obj_collapsed !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_collapsed + '" (at "' + path_collapsed + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$B = 900000;
const VERSION$2A = "12123f1aca1b5a48303b1d099f9f5629";
function validate$1G(obj, path = 'RecordLayoutUserStateRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_layoutType = obj.layoutType;
        const path_layoutType = path + '.layoutType';
        if (typeof obj_layoutType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_layoutType + '" (at "' + path_layoutType + '")');
        }
        const obj_mode = obj.mode;
        const path_mode = path + '.mode';
        if (typeof obj_mode !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_mode + '" (at "' + path_mode + '")');
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        if (typeof obj_recordTypeId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
        }
        const obj_sectionUserStates = obj.sectionUserStates;
        const path_sectionUserStates = path + '.sectionUserStates';
        if (typeof obj_sectionUserStates !== 'object' || ArrayIsArray(obj_sectionUserStates) || obj_sectionUserStates === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_sectionUserStates + '" (at "' + path_sectionUserStates + '")');
        }
        const obj_sectionUserStates_keys = ObjectKeys(obj_sectionUserStates);
        for (let i = 0; i < obj_sectionUserStates_keys.length; i++) {
            const key = obj_sectionUserStates_keys[i];
            const obj_sectionUserStates_prop = obj_sectionUserStates[key];
            const path_sectionUserStates_prop = path_sectionUserStates + '["' + key + '"]';
            const referencepath_sectionUserStates_propValidationError = validate$1H(obj_sectionUserStates_prop, path_sectionUserStates_prop);
            if (referencepath_sectionUserStates_propValidationError !== null) {
                let message = 'Object doesn\'t match RecordLayoutSectionUserStateRepresentation (at "' + path_sectionUserStates_prop + '")\n';
                message += referencepath_sectionUserStates_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$R = 'RecordLayoutUserStateRepresentation';
function keyBuilder$3j(luvio, config) {
    return keyPrefix + '::' + RepresentationType$R + ':' + config.recordTypeId + ':' + config.apiName + ':' + config.layoutType + ':' + config.mode;
}
function keyBuilderFromType$u(luvio, object) {
    const keyParams = {
        recordTypeId: object.recordTypeId,
        apiName: object.apiName,
        layoutType: object.layoutType,
        mode: object.mode
    };
    return keyBuilder$3j(luvio, keyParams);
}
function normalize$J(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$3b = function RecordLayoutUserStateRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2A,
        private: [
            'apiName',
            'recordTypeId',
            'layoutType',
            'mode'
        ],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'sectionUserStates',
                kind: 'Object',
                opaque: true
            }
        ]
    };
};
function equals$10(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$27 = function RecordLayoutUserStateRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1G(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$u(luvio, input);
    const ttlToUse = TTL$B;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$J, "UiApi", VERSION$2A, RepresentationType$R, equals$10);
    return createLink$1(key);
};
function getTypeCacheKeys$2a(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$u(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$R,
        mergeable: false
    });
}

const TTL$A = 900000;
const VERSION$2z = "49cdd4bc235a6094c3559cc7735b3b6d";
function validate$1F(obj, path = 'RecordUiRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_layoutUserStates = obj.layoutUserStates;
        const path_layoutUserStates = path + '.layoutUserStates';
        if (typeof obj_layoutUserStates !== 'object' || ArrayIsArray(obj_layoutUserStates) || obj_layoutUserStates === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_layoutUserStates + '" (at "' + path_layoutUserStates + '")');
        }
        const obj_layoutUserStates_keys = ObjectKeys(obj_layoutUserStates);
        for (let i = 0; i < obj_layoutUserStates_keys.length; i++) {
            const key = obj_layoutUserStates_keys[i];
            const obj_layoutUserStates_prop = obj_layoutUserStates[key];
            const path_layoutUserStates_prop = path_layoutUserStates + '["' + key + '"]';
            if (typeof obj_layoutUserStates_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_layoutUserStates_prop + '" (at "' + path_layoutUserStates_prop + '")');
            }
        }
        const obj_layouts = obj.layouts;
        const path_layouts = path + '.layouts';
        if (typeof obj_layouts !== 'object' || ArrayIsArray(obj_layouts) || obj_layouts === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_layouts + '" (at "' + path_layouts + '")');
        }
        const obj_layouts_keys = ObjectKeys(obj_layouts);
        for (let i = 0; i < obj_layouts_keys.length; i++) {
            const key = obj_layouts_keys[i];
            const obj_layouts_prop = obj_layouts[key];
            const path_layouts_prop = path_layouts + '["' + key + '"]';
            if (typeof obj_layouts_prop !== 'object' || ArrayIsArray(obj_layouts_prop) || obj_layouts_prop === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_layouts_prop + '" (at "' + path_layouts_prop + '")');
            }
            const obj_layouts_prop_keys = ObjectKeys(obj_layouts_prop);
            for (let i = 0; i < obj_layouts_prop_keys.length; i++) {
                const key = obj_layouts_prop_keys[i];
                const obj_layouts_prop_prop = obj_layouts_prop[key];
                const path_layouts_prop_prop = path_layouts_prop + '["' + key + '"]';
                if (typeof obj_layouts_prop_prop !== 'object' || ArrayIsArray(obj_layouts_prop_prop) || obj_layouts_prop_prop === null) {
                    return new TypeError('Expected "object" but received "' + typeof obj_layouts_prop_prop + '" (at "' + path_layouts_prop_prop + '")');
                }
                const obj_layouts_prop_prop_keys = ObjectKeys(obj_layouts_prop_prop);
                for (let i = 0; i < obj_layouts_prop_prop_keys.length; i++) {
                    const key = obj_layouts_prop_prop_keys[i];
                    const obj_layouts_prop_prop_prop = obj_layouts_prop_prop[key];
                    const path_layouts_prop_prop_prop = path_layouts_prop_prop + '["' + key + '"]';
                    if (typeof obj_layouts_prop_prop_prop !== 'object' || ArrayIsArray(obj_layouts_prop_prop_prop) || obj_layouts_prop_prop_prop === null) {
                        return new TypeError('Expected "object" but received "' + typeof obj_layouts_prop_prop_prop + '" (at "' + path_layouts_prop_prop_prop + '")');
                    }
                    const obj_layouts_prop_prop_prop_keys = ObjectKeys(obj_layouts_prop_prop_prop);
                    for (let i = 0; i < obj_layouts_prop_prop_prop_keys.length; i++) {
                        const key = obj_layouts_prop_prop_prop_keys[i];
                        const obj_layouts_prop_prop_prop_prop = obj_layouts_prop_prop_prop[key];
                        const path_layouts_prop_prop_prop_prop = path_layouts_prop_prop_prop + '["' + key + '"]';
                        if (typeof obj_layouts_prop_prop_prop_prop !== 'object') {
                            return new TypeError('Expected "object" but received "' + typeof obj_layouts_prop_prop_prop_prop + '" (at "' + path_layouts_prop_prop_prop_prop + '")');
                        }
                    }
                }
            }
        }
        const obj_objectInfos = obj.objectInfos;
        const path_objectInfos = path + '.objectInfos';
        if (typeof obj_objectInfos !== 'object' || ArrayIsArray(obj_objectInfos) || obj_objectInfos === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_objectInfos + '" (at "' + path_objectInfos + '")');
        }
        const obj_objectInfos_keys = ObjectKeys(obj_objectInfos);
        for (let i = 0; i < obj_objectInfos_keys.length; i++) {
            const key = obj_objectInfos_keys[i];
            const obj_objectInfos_prop = obj_objectInfos[key];
            const path_objectInfos_prop = path_objectInfos + '["' + key + '"]';
            if (typeof obj_objectInfos_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_objectInfos_prop + '" (at "' + path_objectInfos_prop + '")');
            }
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (typeof obj_records !== 'object' || ArrayIsArray(obj_records) || obj_records === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
        const obj_records_keys = ObjectKeys(obj_records);
        for (let i = 0; i < obj_records_keys.length; i++) {
            const key = obj_records_keys[i];
            const obj_records_prop = obj_records[key];
            const path_records_prop = path_records + '["' + key + '"]';
            if (typeof obj_records_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_records_prop + '" (at "' + path_records_prop + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$Q = 'RecordUiRepresentation';
function normalize$I(input, existing, path, luvio, store, timestamp) {
    const input_layoutUserStates = input.layoutUserStates;
    const input_layoutUserStates_id = path.fullPath + '__layoutUserStates';
    const input_layoutUserStates_keys = Object.keys(input_layoutUserStates);
    const input_layoutUserStates_length = input_layoutUserStates_keys.length;
    for (let i = 0; i < input_layoutUserStates_length; i++) {
        const key = input_layoutUserStates_keys[i];
        const input_layoutUserStates_prop = input_layoutUserStates[key];
        const input_layoutUserStates_prop_id = input_layoutUserStates_id + '__' + key;
        input_layoutUserStates[key] = ingest$27(input_layoutUserStates_prop, {
            fullPath: input_layoutUserStates_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_layouts = input.layouts;
    const input_layouts_id = path.fullPath + '__layouts';
    const input_layouts_keys = Object.keys(input_layouts);
    const input_layouts_length = input_layouts_keys.length;
    for (let i = 0; i < input_layouts_length; i++) {
        const key = input_layouts_keys[i];
        const input_layouts_prop = input_layouts[key];
        const input_layouts_prop_id = input_layouts_id + '__' + key;
        const input_layouts_prop_keys = Object.keys(input_layouts_prop);
        const input_layouts_prop_length = input_layouts_prop_keys.length;
        for (let i = 0; i < input_layouts_prop_length; i++) {
            const key = input_layouts_prop_keys[i];
            const input_layouts_prop_prop = input_layouts_prop[key];
            const input_layouts_prop_prop_id = input_layouts_prop_id + '__' + key;
            const input_layouts_prop_prop_keys = Object.keys(input_layouts_prop_prop);
            const input_layouts_prop_prop_length = input_layouts_prop_prop_keys.length;
            for (let i = 0; i < input_layouts_prop_prop_length; i++) {
                const key = input_layouts_prop_prop_keys[i];
                const input_layouts_prop_prop_prop = input_layouts_prop_prop[key];
                const input_layouts_prop_prop_prop_id = input_layouts_prop_prop_id + '__' + key;
                const input_layouts_prop_prop_prop_keys = Object.keys(input_layouts_prop_prop_prop);
                const input_layouts_prop_prop_prop_length = input_layouts_prop_prop_prop_keys.length;
                for (let i = 0; i < input_layouts_prop_prop_prop_length; i++) {
                    const key = input_layouts_prop_prop_prop_keys[i];
                    const input_layouts_prop_prop_prop_prop = input_layouts_prop_prop_prop[key];
                    const input_layouts_prop_prop_prop_prop_id = input_layouts_prop_prop_prop_id + '__' + key;
                    input_layouts_prop_prop_prop[key] = ingest$28(input_layouts_prop_prop_prop_prop, {
                        fullPath: input_layouts_prop_prop_prop_prop_id,
                        propertyName: key,
                        parent: {
                            data: input,
                            key: path.fullPath,
                            existing: existing,
                        },
                        ttl: path.ttl
                    }, luvio, store, timestamp);
                }
            }
        }
    }
    const input_objectInfos = input.objectInfos;
    const input_objectInfos_id = path.fullPath + '__objectInfos';
    const input_objectInfos_keys = Object.keys(input_objectInfos);
    const input_objectInfos_length = input_objectInfos_keys.length;
    for (let i = 0; i < input_objectInfos_length; i++) {
        const key = input_objectInfos_keys[i];
        const input_objectInfos_prop = input_objectInfos[key];
        const input_objectInfos_prop_id = input_objectInfos_id + '__' + key;
        input_objectInfos[key] = ingest$29(input_objectInfos_prop, {
            fullPath: input_objectInfos_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_records = input.records;
    const input_records_id = path.fullPath + '__records';
    const input_records_keys = Object.keys(input_records);
    const input_records_length = input_records_keys.length;
    for (let i = 0; i < input_records_length; i++) {
        const key = input_records_keys[i];
        const input_records_prop = input_records[key];
        const input_records_prop_id = input_records_id + '__' + key;
        input_records[key] = ingest$2g(input_records_prop, {
            fullPath: input_records_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
function equals$$(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_layoutUserStates = existing.layoutUserStates;
    const incoming_layoutUserStates = incoming.layoutUserStates;
    const equals_layoutUserStates_props = equalsObject(existing_layoutUserStates, incoming_layoutUserStates, (existing_layoutUserStates_prop, incoming_layoutUserStates_prop) => {
        if (!(existing_layoutUserStates_prop.__ref === incoming_layoutUserStates_prop.__ref)) {
            return false;
        }
    });
    if (equals_layoutUserStates_props === false) {
        return false;
    }
    const existing_layouts = existing.layouts;
    const incoming_layouts = incoming.layouts;
    const equals_layouts_props = equalsObject(existing_layouts, incoming_layouts, (existing_layouts_prop, incoming_layouts_prop) => {
        const equals_layouts_props = equalsObject(existing_layouts_prop, incoming_layouts_prop, (existing_layouts_prop_prop, incoming_layouts_prop_prop) => {
            const equals_layouts_props = equalsObject(existing_layouts_prop_prop, incoming_layouts_prop_prop, (existing_layouts_prop_prop_prop, incoming_layouts_prop_prop_prop) => {
                const equals_layouts_props = equalsObject(existing_layouts_prop_prop_prop, incoming_layouts_prop_prop_prop, (existing_layouts_prop_prop_prop_prop, incoming_layouts_prop_prop_prop_prop) => {
                    if (!(existing_layouts_prop_prop_prop_prop.__ref === incoming_layouts_prop_prop_prop_prop.__ref)) {
                        return false;
                    }
                });
                if (equals_layouts_props === false) {
                    return false;
                }
            });
            if (equals_layouts_props === false) {
                return false;
            }
        });
        if (equals_layouts_props === false) {
            return false;
        }
    });
    if (equals_layouts_props === false) {
        return false;
    }
    const existing_objectInfos = existing.objectInfos;
    const incoming_objectInfos = incoming.objectInfos;
    const equals_objectInfos_props = equalsObject(existing_objectInfos, incoming_objectInfos, (existing_objectInfos_prop, incoming_objectInfos_prop) => {
        if (!(existing_objectInfos_prop.__ref === incoming_objectInfos_prop.__ref)) {
            return false;
        }
    });
    if (equals_objectInfos_props === false) {
        return false;
    }
    const existing_records = existing.records;
    const incoming_records = incoming.records;
    const equals_records_props = equalsObject(existing_records, incoming_records, (existing_records_prop, incoming_records_prop) => {
        if (!(existing_records_prop.__ref === incoming_records_prop.__ref)) {
            return false;
        }
    });
    if (equals_records_props === false) {
        return false;
    }
    return true;
}
const ingest$26 = function RecordUiRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1F(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$A;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$I, "UiApi", VERSION$2z, RepresentationType$Q, equals$$);
    return createLink$1(key);
};

function createResourceRequest$16(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/record-ui/' + config.urlParams.recordIds + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const getRecordUi_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('childRelationships', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('layoutTypes', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArrayTitleCase),
    generateParamConfigMetadata('modes', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('updateMru', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
function typeCheckConfig$13(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordUi_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$16(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordUi_ConfigPropertyMetadata);
    const config = typeCheckConfig$13(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const layoutSelections$2 = select$3c();
const objectInfoPathSelection = select$3e();
const layoutUserStatePathSelector = select$3b();
function buildRecordUiSelector(recordDefs, layoutTypes, modes, recordOptionalFields) {
    const layoutTypeSelections = [];
    for (let i = 0, len = layoutTypes.length; i < len; i += 1) {
        const layoutType = layoutTypes[i];
        const modeSelections = [];
        const sel = {
            kind: 'Object',
            name: layoutType,
            selections: modeSelections,
        };
        for (let m = 0; m < modes.length; m += 1) {
            const mode = modes[m];
            const modeSel = {
                kind: 'Link',
                name: mode,
                fragment: layoutSelections$2,
            };
            push.call(modeSelections, modeSel);
        }
        push.call(layoutTypeSelections, sel);
    }
    const recordLayoutSelections = [];
    const recordSelections = [];
    for (let i = 0, len = recordDefs.length; i < len; i += 1) {
        const { recordId, recordData } = recordDefs[i];
        push.call(recordLayoutSelections, {
            kind: 'Object',
            name: recordData.apiName,
            required: false,
            map: true,
            selections: layoutTypeSelections,
        });
        const optionalFields = recordOptionalFields[recordId];
        const fields = extractFields(recordData);
        push.call(recordSelections, {
            kind: 'Link',
            name: recordId,
            fragment: {
                kind: 'Fragment',
                selections: buildSelectionFromFields(fields, optionalFields),
                private: ['eTag', 'weakEtag'],
            },
        });
    }
    return {
        kind: 'Fragment',
        private: ['eTag'],
        version: VERSION$2z,
        selections: [
            {
                kind: 'Link',
                name: 'layoutUserStates',
                map: true,
                fragment: layoutUserStatePathSelector,
            },
            {
                kind: 'Object',
                name: 'layouts',
                selections: recordLayoutSelections,
            },
            {
                kind: 'Link',
                name: 'objectInfos',
                map: true,
                fragment: objectInfoPathSelection,
            },
            {
                name: 'records',
                kind: 'Object',
                selections: recordSelections,
            },
        ],
    };
}

const FIELD_ID = 'Id';
const FIELD_NAME = 'Name';
const COMPONENT_TYPE_FIELD = 'Field';
function isFieldAReferenceWithRelationshipName(objectInfo, fieldApiName) {
    const field = objectInfo.fields[fieldApiName];
    if (field === undefined) {
        return false;
    }
    // eslint-disable-next-line @salesforce/lds/no-invalid-todo
    // TODO - can reference===true and relationshipName===null?
    return field.reference === true && field.relationshipName !== null;
}
function getRelationshipName(objectInfo, fieldApiName) {
    // eslint-disable-next-line @salesforce/lds/no-invalid-todo
    // TODO RAML - fix typing so isFieldAReferenceWithRelationshipName enables calling this without `relationshipName!`
    return objectInfo.fields[fieldApiName].relationshipName;
}
function getNameField(objectInfo, fieldApiName) {
    // eslint-disable-next-line @salesforce/lds/no-invalid-todo
    // TODO - this logic is adopted from lds222. It searches
    // ObjectInfoRep.ReferenceToInfoRep[].nameFields[]:
    // 1. If any of the arrays are empty returns `Name`
    // 2. If `Name` is found in any array position then returns it
    // 3. If the field has more than 1 references(polymorphic), return `Name`
    // 4. Else returns ObjectInfoRep.ReferenceToInfoRep[0].nameFields[0]
    // Rationale for this is unclear and needs clarification.
    const referenceToInfos = objectInfo.fields[fieldApiName].referenceToInfos;
    if (referenceToInfos.length !== 1) {
        return FIELD_NAME;
    }
    const firstReferenceNameFields = referenceToInfos[0].nameFields;
    if (firstReferenceNameFields.length < 1) {
        return FIELD_NAME;
    }
    for (let a = 0, alen = referenceToInfos.length; a < alen; a++) {
        const nameFields = referenceToInfos[a].nameFields;
        for (let b = 0, blen = nameFields.length; b < blen; b++) {
            const nameField = nameFields[b];
            if (nameField === FIELD_NAME) {
                return nameField;
            }
        }
    }
    return firstReferenceNameFields[0];
}
function getQualifiedFieldApiNamesFromLayout(layout, objectInfo) {
    const qualifiedFieldNames = [];
    const qualifiedOptionalFieldNames = [];
    for (let a = 0, alen = layout.sections.length; a < alen; a++) {
        const section = layout.sections[a];
        for (let b = 0, blen = section.layoutRows.length; b < blen; b++) {
            const row = section.layoutRows[b];
            for (let c = 0, clen = row.layoutItems.length; c < clen; c++) {
                const item = row.layoutItems[c];
                for (let d = 0, dlen = item.layoutComponents.length; d < dlen; d++) {
                    const component = item.layoutComponents[d];
                    const { apiName } = component;
                    if (apiName && component.componentType === COMPONENT_TYPE_FIELD) {
                        if (isFieldAReferenceWithRelationshipName(objectInfo, apiName)) {
                            const relationshipFieldApiName = getRelationshipName(objectInfo, apiName);
                            // By default, include the "Id" field on spanning records that are on the layout.
                            qualifiedFieldNames.push(`${objectInfo.apiName}.${relationshipFieldApiName}.${FIELD_ID}`);
                            const nameField = getNameField(objectInfo, apiName);
                            // W-15692973: Name field of referenced entites are moved to optionalFields since the Name field may not exist for few entites.
                            qualifiedOptionalFieldNames.push(`${objectInfo.apiName}.${relationshipFieldApiName}.${nameField}`);
                        }
                        qualifiedFieldNames.push(`${objectInfo.apiName}.${component.apiName}`);
                    }
                }
            }
        }
    }
    return {
        fields: qualifiedFieldNames,
        optionalFields: qualifiedOptionalFieldNames,
    };
}

function getMissingRecordLookupFields(record, objectInfo) {
    const lookupFields = {};
    const { apiName, fields: recordFields } = record;
    const { fields: objectInfoFields } = objectInfo;
    const objectInfoFieldNames = keys(objectInfoFields);
    for (let i = 0, len = objectInfoFieldNames.length; i < len; i += 1) {
        const fieldName = objectInfoFieldNames[i];
        const field = objectInfoFields[fieldName];
        const { relationshipName } = field;
        if (relationshipName === null) {
            continue;
        }
        const recordFieldValue = recordFields[relationshipName];
        // Only interested in record fields that are present and that are null
        if (recordFieldValue === undefined || recordFieldValue.value !== null) {
            continue;
        }
        // Include the Id field. Ex: Opportunity.Account.Id, Opportunity.relation1__r.Id
        const idFieldName = `${apiName}.${relationshipName}.Id`;
        lookupFields[idFieldName] = true;
        const nameField = `${apiName}.${relationshipName}.${getNameField(objectInfo, fieldName)}`;
        lookupFields[nameField] = true;
    }
    return keys(lookupFields);
}
function getRecordUiMissingRecordLookupFields(recordUi) {
    const { records, objectInfos } = recordUi;
    const recordLookupFields = {};
    const recordIds = keys(records);
    for (let i = 0, len = recordIds.length; i < len; i += 1) {
        const recordId = recordIds[i];
        const recordData = records[recordId];
        const { apiName } = recordData;
        const objectInfo = objectInfos[apiName];
        recordLookupFields[recordId] = getMissingRecordLookupFields(recordData, objectInfo);
    }
    return recordLookupFields;
}

// Custom adapter config due to `unsupported` items
const GET_RECORDUI_ADAPTER_CONFIG = {
    displayName: 'getRecordUi',
    parameters: {
        required: ['recordIds', 'layoutTypes', 'modes'],
        optional: ['optionalFields'],
        unsupported: [
            'formFactor',
            'childRelationships',
            'pageSize',
            'updateMru',
        ],
    },
};
const RECORD_UI_ERROR_STORE_METADATA_PARAMS_VERSION = 'RECORD_UI_ERROR_STORE_METADATA_PARAMS_VERSION_1';
const RECORD_UI_ERROR_STORE_METADATA_PARAMS = {
    ttl: TTL$A,
    representationName: '',
    namespace: keyPrefix,
    version: RECORD_UI_ERROR_STORE_METADATA_PARAMS_VERSION,
};
const RECORD_UI_CACHED_SELECTOR_VERSION = 'RECORD_UI_CACHED_SELECTOR_VERSION_1';
function buildCachedSelectorKey(key) {
    return `${key}__selector`;
}
function eachLayout(recordUi, cb) {
    const { layouts } = recordUi;
    const layoutApiNames = keys(layouts);
    for (let a = 0, len = layoutApiNames.length; a < len; a += 1) {
        const apiName = layoutApiNames[a];
        const apiNameData = layouts[apiName];
        const recordTypeIds = keys(apiNameData);
        for (let b = 0, recordTypeIdsLen = recordTypeIds.length; b < recordTypeIdsLen; b += 1) {
            const recordTypeId = recordTypeIds[b];
            const recordTypeData = apiNameData[recordTypeId];
            const layoutTypes = keys(recordTypeData);
            for (let c = 0, layoutTypesLen = layoutTypes.length; c < layoutTypesLen; c += 1) {
                const layoutType = layoutTypes[c];
                const layoutTypeData = recordTypeData[layoutType];
                const modes = keys(layoutTypeData);
                for (let d = 0, modesLen = modes.length; d < modesLen; d += 1) {
                    const mode = modes[d];
                    const layout = layoutTypeData[mode];
                    cb(apiName, recordTypeId, layout);
                }
            }
        }
    }
}
function collectRecordDefs(resp, recordIds) {
    const recordDefs = [];
    for (let i = 0, len = recordIds.length; i < len; i += 1) {
        const recordId = recordIds[i];
        const recordData = resp.records[recordId];
        push.call(recordDefs, {
            recordId,
            recordData,
            recordTypeId: getRecordTypeId$2(recordData),
        });
    }
    return recordDefs;
}
function keyBuilder$3i(recordIds, layoutTypes, modes, optionalFields) {
    const joinedRecordIds = recordIds.sort().join(',');
    const joinedOptionalFields = optionalFields.sort().join(',');
    const joinedLayoutTypes = layoutTypes.sort().join(',');
    const joinedModes = modes.sort().join(',');
    return `${keyPrefix}::RecordUiRepresentation:${joinedRecordIds}:${joinedLayoutTypes}:${joinedModes}:${joinedOptionalFields}`;
}
function buildSnapshotRefresh$4(luvio, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot$15(luvio, config),
    };
}
function markRecordUiNulledOutLookupFields(recordLookupFields, recordNodes) {
    for (let i = 0, len = recordNodes.length; i < len; i++) {
        const recordId = recordNodes[i].data.id;
        if (recordLookupFields[recordId] !== undefined) {
            markNulledOutRequiredFields(recordNodes[i], recordLookupFields[recordId]);
        }
    }
}
function markRecordUiOptionalFields(optionalFields, recordLookupFields, recordNodes) {
    for (let i = 0, len = recordNodes.length; i < len; i++) {
        const recordId = recordNodes[i].data.id;
        if (optionalFields.length > 0 || recordLookupFields[recordId] !== undefined) {
            markMissingOptionalFields(recordNodes[i], [
                ...optionalFields,
                ...recordLookupFields[recordId],
            ]);
        }
    }
}
function prepareRequest$5(luvio, config) {
    const { recordIds, layoutTypes, modes, optionalFields } = config;
    const key = keyBuilder$3i(recordIds, layoutTypes, modes, optionalFields);
    const selectorKey = buildCachedSelectorKey(key);
    const resourceRequest = createResourceRequest$16({
        urlParams: {
            recordIds,
        },
        queryParams: {
            layoutTypes,
            modes,
            optionalFields: dedupe(optionalFields).sort(),
        },
    });
    return { key, selectorKey, resourceRequest };
}
// NOTE: getRecordUi is special and we can't use the generated getResponseCacheKeys
// (just like we don't use the generated ingestSuccess).  To get the cache keys
// we have to run ingest code and look at the resulting snapshot's seenRecords.
function getCacheKeys(keySet, luvio, config, key, originalResponseBody) {
    const { recordIds, layoutTypes, modes } = config;
    const responseBody = parse(stringify(originalResponseBody));
    eachLayout(responseBody, (apiName, recordTypeId, layout) => {
        if (layout.id === null) {
            return;
        }
        const layoutUserState = responseBody.layoutUserStates[layout.id];
        // Temporary hack since we can't match keys from getLayoutUserState response
        // to record ui's layout users states.
        if (layoutUserState === undefined) {
            return;
        }
        layoutUserState.apiName = apiName;
        layoutUserState.recordTypeId = recordTypeId;
        layoutUserState.mode = layout.mode;
        layoutUserState.layoutType = layout.layoutType;
    });
    const recordLookupFields = getRecordUiMissingRecordLookupFields(responseBody);
    const selPath = buildRecordUiSelector(collectRecordDefs(responseBody, recordIds), layoutTypes, modes, recordLookupFields);
    const sel = {
        recordId: key,
        node: selPath,
        variables: {},
    };
    luvio.storeIngest(key, ingest$26, responseBody);
    const snapshot = luvio.storeLookup(sel, buildSnapshotRefresh$4(luvio, config));
    if (snapshot.state === 'Error') {
        return new StoreKeyMap();
    }
    const keys = [...snapshot.seenRecords.keysAsArray(), snapshot.recordId];
    for (let i = 0, len = keys.length; i < len; i++) {
        const key = keys[i];
        const namespace = key.split('::')[0];
        const representationName = key.split('::')[1].split(':')[0];
        keySet.set(key, {
            namespace,
            representationName,
            mergeable: representationName === RepresentationType$_,
        });
    }
}
function onFetchResponseSuccess$N(luvio, config, selectorKey, key, responseBody) {
    const { recordIds, layoutTypes, modes, optionalFields } = config;
    // eslint-disable-next-line @salesforce/lds/no-invalid-todo
    // TODO fix API so we don't have to augment the response with request details in order
    // to support refresh. these are never emitted out per (private).
    eachLayout(responseBody, (apiName, recordTypeId, layout) => {
        if (layout.id === null) {
            return;
        }
        const layoutUserState = responseBody.layoutUserStates[layout.id];
        // Temporary hack since we can't match keys from getLayoutUserState response
        // to record ui's layout users states.
        if (layoutUserState === undefined) {
            return;
        }
        layoutUserState.apiName = apiName;
        layoutUserState.recordTypeId = recordTypeId;
        layoutUserState.mode = layout.mode;
        layoutUserState.layoutType = layout.layoutType;
    });
    const recordLookupFields = getRecordUiMissingRecordLookupFields(responseBody);
    const selPath = buildRecordUiSelector(collectRecordDefs(responseBody, recordIds), layoutTypes, modes, recordLookupFields);
    const sel = {
        recordId: key,
        node: selPath,
        variables: {},
    };
    luvio.storePublish(selectorKey, sel);
    luvio.storeIngest(key, ingest$26, responseBody);
    // During ingestion, only valid records are stored.
    const recordNodes = [];
    const validRecordIds = [];
    for (let i = 0, len = recordIds.length; i < len; i += 1) {
        const recordId = recordIds[i];
        const recordKey = keyBuilder$3E(luvio, { recordId });
        const node = luvio.getNode(recordKey);
        if (isGraphNode(node)) {
            recordNodes.push(node);
            validRecordIds.push(recordId);
        }
    }
    markRecordUiNulledOutLookupFields(recordLookupFields, recordNodes);
    markRecordUiOptionalFields(optionalFields, recordLookupFields, recordNodes);
    publishDependencies(luvio, validRecordIds, [key, selectorKey]);
    const snapshot = luvio.storeLookup(sel, buildSnapshotRefresh$4(luvio, config));
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$O(luvio, config, selectorKey, key, err) {
    const errorSnapshot = luvio.errorSnapshot(err, buildSnapshotRefresh$4(luvio, config));
    luvio.storeIngestError(key, errorSnapshot, RECORD_UI_ERROR_STORE_METADATA_PARAMS);
    return luvio.storeBroadcast().then(() => {
        if (err.errorType === 'fetchResponse') {
            const { status } = err;
            if (status === 404) {
                const sel = {
                    recordId: key,
                    node: {
                        kind: 'Fragment',
                        private: [],
                        opaque: true,
                        version: RECORD_UI_CACHED_SELECTOR_VERSION,
                    },
                    variables: {},
                };
                luvio.storePublish(selectorKey, sel);
                return luvio.storeLookup(sel, buildSnapshotRefresh$4(luvio, config));
            }
        }
        return errorSnapshot;
    });
}
function buildNetworkSnapshot$15(luvio, config, dispatchContext) {
    const { key, resourceRequest, selectorKey } = prepareRequest$5(luvio, config);
    return luvio
        .dispatchResourceRequest(resourceRequest, dispatchContext)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$N(luvio, config, selectorKey, key, response.body), () => {
            const cache = new StoreKeyMap();
            getCacheKeys(cache, luvio, config, key, response.body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return onFetchResponseError$O(luvio, config, selectorKey, key, err);
        });
    });
}
function publishDependencies(luvio, recordIds, depKeys) {
    for (let i = 0, len = recordIds.length; i < len; i += 1) {
        const recordDepKey = dependencyKeyBuilder({ recordId: recordIds[i] });
        const dependencies = create(null);
        for (let j = 0, len = depKeys.length; j < len; j++) {
            dependencies[depKeys[j]] = true;
        }
        const node = luvio.getNode(recordDepKey);
        if (isGraphNode(node)) {
            const recordDeps = node.retrieve();
            assign(dependencies, recordDeps);
        }
        luvio.storePublish(recordDepKey, dependencies);
    }
}
function buildCachedSelectorSnapshot(context, storeLookup) {
    const { config } = context;
    const { recordIds, layoutTypes, modes, optionalFields } = config;
    const key = keyBuilder$3i(recordIds, layoutTypes, modes, optionalFields);
    const cachedSelectorKey = buildCachedSelectorKey(key);
    return storeLookup({
        recordId: cachedSelectorKey,
        node: {
            kind: 'Fragment',
            private: [],
            opaque: true,
            version: RECORD_UI_CACHED_SELECTOR_VERSION,
        },
        variables: {},
    });
}
function buildCachedRecordUiRepresentationSnapshot(context, storeLookup) {
    const { config, luvio, selector } = context;
    // try to resolve RecordUiRepresentation selector if previous steps were able to find one
    if (selector !== undefined) {
        return storeLookup(selector, buildSnapshotRefresh$4(luvio, config));
    }
}
function buildNetworkRecordUiRepresentationSnapshot(context, coercedAdapterRequestContext) {
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$15(context.luvio, context.config, dispatchOptions);
}
function coerceConfigWithDefaults$3(untrustedConfig) {
    const config = validateAdapterConfig$16(untrustedConfig, GET_RECORDUI_ADAPTER_CONFIG);
    if (config === null) {
        return null;
    }
    const { layoutTypes, modes } = config;
    // custom config validation
    if (layoutTypes === undefined || modes === undefined) {
        return null;
    }
    return {
        ...config,
        layoutTypes: layoutTypes,
        modes: modes,
        optionalFields: config.optionalFields === undefined ? [] : config.optionalFields,
    };
}
const factory$i = (luvio) => function UiApi__getRecordUi(untrustedConfig, requestContext) {
    // standard config validation and coercion
    const config = coerceConfigWithDefaults$3(untrustedConfig);
    if (config === null) {
        return null;
    }
    const definedRequestContext = requestContext || {};
    const selectorPromiseOrSnapshot = luvio.applyCachePolicy(definedRequestContext, { config, luvio }, buildCachedSelectorSnapshot, buildNotFetchableNetworkSnapshot(luvio));
    const resolveSelector = (selectorSnapshot) => {
        const selector = isFulfilledSnapshot(selectorSnapshot) || isStaleSnapshot(selectorSnapshot)
            ? selectorSnapshot.data
            : undefined;
        return luvio.applyCachePolicy(definedRequestContext, { config, luvio, selector }, buildCachedRecordUiRepresentationSnapshot, buildNetworkRecordUiRepresentationSnapshot);
    };
    return isPromise(selectorPromiseOrSnapshot)
        ? selectorPromiseOrSnapshot.then(resolveSelector)
        : resolveSelector(selectorPromiseOrSnapshot);
};

var LayoutMode;
(function (LayoutMode) {
    LayoutMode["View"] = "View";
    LayoutMode["Edit"] = "Edit";
    LayoutMode["Create"] = "Create";
})(LayoutMode || (LayoutMode = {}));

const DEFAULT_MODE = LayoutMode.View;
const layoutSelections$1 = select$3c();
function buildSnapshotRefresh$3(luvio, config) {
    return {
        config,
        resolve: () => refresh$1(luvio, config),
    };
}
function buildObjectInfoSelectorKey(luvio, objectInfo) {
    const key = keyBuilderFromType$w(luvio, objectInfo);
    return `${key}__implicit__fields__selector`;
}
function refresh$1(luvio, config) {
    const { recordId, layoutTypes, modes: configModes, optionalFields: configOptionalFields, } = config;
    const modes = configModes === undefined ? [DEFAULT_MODE] : configModes;
    const optionalFields = configOptionalFields === undefined ? [] : configOptionalFields;
    const recordUiConfig = {
        recordIds: [recordId],
        layoutTypes,
        modes,
        optionalFields,
    };
    return buildNetworkSnapshot$15(luvio, recordUiConfig).then((snapshot) => {
        const refresh = buildSnapshotRefresh$3(luvio, config);
        if (isErrorSnapshot(snapshot)) {
            var recordKey = keyBuilder$3E(luvio, { recordId });
            luvio.storeIngestError(recordKey, snapshot);
            return luvio.storeBroadcast().then(() => luvio.errorSnapshot(snapshot.error, refresh));
        }
        if (isUnfulfilledSnapshot(snapshot)) {
            if (process.env.NODE_ENV !== 'production') {
                throw new Error(`RecordUi adapter resolved with a snapshot with missing data, missingPaths: ${keys(snapshot.missingPaths)}`);
            }
        }
        const { data } = snapshot;
        if (data === undefined) {
            // eslint-disable-next-line @salesforce/lds/no-error-in-production
            throw new Error(`RecordUi adapter resolved with a ${snapshot.state} snapshot with undefined data`);
        }
        const { layoutMap, objectInfo } = getLayoutMapAndObjectInfo(recordId, data);
        // W-15692973: Name field of referenced entites are moved to optionalFields since the Name field may not exist for few entites.
        const { fields, optionalFields: layoutOptionalFields } = getFieldsFromLayoutMap(layoutMap, objectInfo);
        const mergedOptionalFields = dedupe(optionalFields.concat(layoutOptionalFields));
        return buildCachedSnapshot$9(luvio, {
            recordId,
            fields,
            optionalFields: mergedOptionalFields,
        }, refresh);
    });
}
function getLayoutMapAndObjectInfo(recordId, data) {
    const { objectInfos, layouts, records } = data;
    const record = records[recordId];
    const { apiName } = record;
    const objectInfo = objectInfos[apiName];
    const recordTypeId = getRecordTypeId$2(record);
    // W-12697744 - Return an empty layoutMap if the entity does not have a layout.
    // Temp fix until we can mimic the server behavior for non-layoutable entities.
    let layoutMap = {};
    if (hasOwnProperty.call(layouts, apiName)) {
        layoutMap = layouts[apiName][recordTypeId] || {};
    }
    return {
        layoutMap,
        objectInfo,
    };
}
function getFields(recordId, data) {
    const { records } = data;
    const record = records[recordId];
    const { fields } = record;
    return fields;
}
function processRecordUiRepresentation(luvio, refresh, recordId, modes, snapshot, adapterContext, optionalFields) {
    if (isErrorSnapshot(snapshot)) {
        return luvio.errorSnapshot(snapshot.error, refresh);
    }
    if (isUnfulfilledSnapshot(snapshot)) {
        if (process.env.NODE_ENV !== 'production') {
            throw new Error(`RecordUi adapter resolved with a snapshot with missing data, missingPaths: ${keys(snapshot.missingPaths)}`);
        }
    }
    const { data } = snapshot;
    if (data === undefined) {
        // eslint-disable-next-line @salesforce/lds/no-error-in-production
        throw new Error(`RecordUi adapter resolved with a ${snapshot.state} snapshot with undefined data`);
    }
    const { layoutMap, objectInfo } = getLayoutMapAndObjectInfo(recordId, data);
    if (objectInfo.layoutable) {
        return getRecordForLayoutableEntities(luvio, refresh, recordId, layoutMap, objectInfo, optionalFields);
    }
    else {
        return getRecordForNonLayoutableEntities(luvio, adapterContext, refresh, recordId, objectInfo, optionalFields, getFields(recordId, data));
    }
}
const recordLayoutFragmentSelector = [
    {
        name: 'apiName',
        kind: 'Scalar',
    },
    {
        name: 'recordTypeId',
        kind: 'Scalar',
    },
];
function getFieldsFromLayoutMap(layoutMap, objectInfo) {
    let fields = [];
    let optionalFields = [];
    const layoutTypes = keys(layoutMap);
    for (let i = 0, layoutTypesLen = layoutTypes.length; i < layoutTypesLen; i += 1) {
        const layoutType = layoutTypes[i];
        const modesMap = layoutMap[layoutType];
        const modes = keys(modesMap);
        for (let m = 0, modesLen = modes.length; m < modesLen; m += 1) {
            const mode = modes[m];
            const { fields: modeFields, optionalFields: modeOptionalFields } = getQualifiedFieldApiNamesFromLayout(modesMap[mode], objectInfo);
            fields = fields.concat(modeFields);
            optionalFields = optionalFields.concat(modeOptionalFields);
        }
    }
    return {
        fields: dedupe(fields).sort(),
        optionalFields: dedupe(optionalFields).sort(),
    };
}
/**
 * W-14696113
 * Merge fields into optionalFields and return the whole list.
 * @param {string[]} fields
 * @param {string[]} [optionalFields]
 */
function convertToImplicitFields(fields, optionalFields) {
    const implicitFields = optionalFields || [];
    fields.forEach((field) => {
        if (!(field in implicitFields))
            implicitFields.push(field);
    });
    return implicitFields;
}
function getRecordForLayoutableEntities(luvio, refresh, recordId, layoutMap, objectInfo, configOptionalFields) {
    // W-15692973: Name field of referenced entites are moved to optionalFields since the Name field may not exist for few entites.
    let { fields, optionalFields } = getFieldsFromLayoutMap(layoutMap, objectInfo);
    if (configOptionalFields && configOptionalFields.length > 0) {
        optionalFields = dedupe(optionalFields.concat(configOptionalFields));
    }
    const implicitFields = convertToImplicitFields(fields, optionalFields);
    return getRecord$1(luvio, refresh, recordId, [], implicitFields);
}
function getRecordForNonLayoutableEntities(luvio, adapterContext, refresh, recordId, objectInfo, configOptionalFields, configFields) {
    const fields = keys(configFields ? configFields : {}).map((key) => `${objectInfo.apiName}.${key}`);
    // W-12697744
    // Set the implicit fields received from the server in adapter context
    // This ensures that the implicit fields are available when data is read locally or from durable store
    const selectorKey = buildObjectInfoSelectorKey(luvio, objectInfo);
    adapterContext.set(selectorKey, fields);
    return getRecord$1(luvio, refresh, recordId, fields, configOptionalFields);
}
function getRecord$1(luvio, refresh, recordId, fields, configOptionalFields) {
    const optionalFields = configOptionalFields === undefined ? [] : dedupe(configOptionalFields).sort();
    // We know what fields we need so delegate to getRecordByFields
    // This should be a cache hit because we just fetched the record-ui
    const recordSnapshotOrPromise = getRecordByFields(luvio, {
        recordId,
        fields,
        optionalFields,
    });
    // attach a record layout refresh
    if (isPromise(recordSnapshotOrPromise)) {
        recordSnapshotOrPromise.then((snapshot) => {
            snapshot.refresh = refresh;
            return snapshot;
        });
    }
    else {
        recordSnapshotOrPromise.refresh = refresh;
    }
    return recordSnapshotOrPromise;
}
function buildNetworkSnapshot$14(context, coercedAdapterRequestContext) {
    const { config, luvio, adapterContext } = context;
    const { recordId } = config;
    const optionalFields = config.optionalFields === undefined ? [] : dedupe(config.optionalFields).sort();
    const refresh = buildSnapshotRefresh$3(luvio, config);
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    // if we were able to map the layouts to a set of fields then use GetRecordFields
    // to send a request for just those fields
    if (context.fields !== undefined) {
        const recordConfig = {
            recordId,
            fields: context.fields,
            optionalFields,
        };
        return buildNetworkSnapshot$1b(luvio, recordConfig, 0, coercedAdapterRequestContext).then((snapshot) => {
            snapshot.refresh = refresh;
            return snapshot;
        });
    }
    // otherwise fallback to getRecordUi to get the full set of information
    const modes = config.modes !== undefined ? config.modes : [DEFAULT_MODE];
    const recordUiConfig = {
        recordIds: [recordId],
        layoutTypes: config.layoutTypes,
        modes,
        optionalFields,
    };
    return buildNetworkSnapshot$15(luvio, recordUiConfig, dispatchOptions).then((snapshot) => processRecordUiRepresentation(luvio, refresh, recordId, modes, snapshot, adapterContext, optionalFields));
}
function buildRecordLayoutCachedSnapshot(context, storeLookup) {
    const { config, luvio } = context;
    // get cached copy of the record
    const { recordId } = config;
    const storeKey = keyBuilder$3E(luvio, { recordId });
    const recordSnapshot = storeLookup({
        recordId: storeKey,
        node: {
            kind: 'Fragment',
            private: [],
            selections: recordLayoutFragmentSelector,
            version: VERSION$2K,
        },
        variables: {},
    });
    return recordSnapshot;
}
const buildObjectInfoCacheSnapshot = (recordLayoutSnapshot) => (context, storeLookup) => {
    const { luvio } = context;
    if (recordLayoutSnapshot.data === undefined) {
        return;
    }
    return buildCachedSnapshotCachePolicy$R({
        config: {
            objectApiName: recordLayoutSnapshot.data.apiName,
        },
        luvio,
    }, storeLookup);
};
const buildLayoutModeCacheSnapshot = (apiName, recordTypeId, layoutType, mode) => (context, storeLookup) => {
    const { luvio } = context;
    const key = keyBuilder$3k(luvio, {
        objectApiName: apiName,
        recordTypeId,
        layoutType,
        mode,
    });
    return storeLookup({
        recordId: key,
        node: layoutSelections$1,
        variables: {},
    });
};
/**
 * Create a snapshot that will only attempt to build from cache
 * These are intermediate lookups to check if the record is in the L2 cache
 * @param  {Luvio} luvio
 * @param  {GetRecordLayoutTypeConfig} config
 * @param  {BuildCachedSnapshot<BuildSnapshotContext>} cachedSnapshot
 */
function makeCacheOnlySnapshot(luvio, config, adapterContext, cachedSnapshot) {
    return luvio.applyCachePolicy(
    // Pass empty context so environment will use its default cache-policy
    {}, { config, luvio, adapterContext }, cachedSnapshot, 
    // disallow hitting the network by returning a gateway timeout
    () => {
        return new Promise((resolve) => {
            resolve({
                state: 'Error',
                data: undefined,
                error: {
                    body: undefined,
                    headers: {},
                    ok: false,
                    status: 504,
                    statusText: 'Gateway Timeout',
                    errorType: 'fetchResponse',
                },
            });
        });
    });
}
/**
 * Create a snapshot that will fail to pull from the cache
 * and force a network call
 * @param  {Luvio} luvio
 * @param  {GetRecordLayoutTypeConfig} config
 * @param  {AdapterRequestContext} requestContext?
 */
function makeNetworkOnlySnapshot(luvio, config, adapterContext, requestContext) {
    return luvio.applyCachePolicy(requestContext || {}, { config, luvio, adapterContext }, () => {
        return undefined;
    }, buildNetworkSnapshot$14);
}
function getRecordLayoutType(luvio, config, adapterContext, requestContext) {
    // lookup record layout
    const recordLayoutSnapshotOrPromise = makeCacheOnlySnapshot(luvio, config, adapterContext, buildRecordLayoutCachedSnapshot);
    return runSnapshotOrPromise(recordLayoutSnapshotOrPromise, (recordLayoutSnapshot) => {
        // if record layout is not fulfulled or data is undefined send to the network
        if (recordLayoutSnapshot.state === 'Unfulfilled' ||
            recordLayoutSnapshot.state === 'Error' ||
            recordLayoutSnapshot.data === undefined) {
            return makeNetworkOnlySnapshot(luvio, config, adapterContext, requestContext);
        }
        // lookup object info
        const objectInfoSnapshotOrPromise = makeCacheOnlySnapshot(luvio, config, adapterContext, buildObjectInfoCacheSnapshot(recordLayoutSnapshot));
        return runSnapshotOrPromise(objectInfoSnapshotOrPromise, (objectInfoSnapshot) => {
            const { layoutTypes } = config;
            // if not fulfilled or missing data send to the network
            if (objectInfoSnapshot.state === 'Unfulfilled' ||
                objectInfoSnapshot.state === 'Error' ||
                objectInfoSnapshot.data === undefined) {
                return makeNetworkOnlySnapshot(luvio, config, adapterContext, requestContext);
            }
            const { data: record } = recordLayoutSnapshot;
            // try to load all the requested layouts from cache
            const recordTypeId = getRecordTypeId$2(record);
            const modes = config.modes === undefined ? [DEFAULT_MODE] : config.modes;
            // W-12697744
            // For non-layoutable entities, use the implicit fields as the fields are not available in layout
            // Fields can be fetched from adapter context if the server request was already made at some point in the past
            // If fields are not available in adapter context, make a network request to get the data
            // and store implicit fields in adapter context.
            if (!objectInfoSnapshot.data.layoutable) {
                const selectorKey = buildObjectInfoSelectorKey(luvio, objectInfoSnapshot.data);
                const fields = adapterContext.get(selectorKey);
                if (!fields) {
                    return makeNetworkOnlySnapshot(luvio, config, adapterContext, requestContext);
                }
                return luvio.applyCachePolicy(requestContext || {}, { config, luvio, adapterContext }, buildRecordSnapshot(luvio, config, recordLayoutSnapshot, objectInfoSnapshot, fields), buildNetworkSnapshot$14);
            }
            return makeRecordLayoutMap(luvio, config, objectInfoSnapshot.data.apiName, recordTypeId, layoutTypes, modes, adapterContext).then((recordLayoutMap) => {
                return luvio.applyCachePolicy(requestContext || {}, { config, luvio, adapterContext }, buildRecordLayoutSnapshot(recordLayoutMap, luvio, config, recordLayoutSnapshot, objectInfoSnapshot), buildNetworkSnapshot$14);
            });
        });
    });
}
/**
 * Takes a views layout types and modes and returns the snapshot of a RecordLayoutRepresentation
 * for each type/mode
 * @param  {Luvio} luvio
 * @param  {GetRecordLayoutTypeConfig} config
 * @param  {string} apiName
 * @param  {string} recordTypeId
 * @param  {LayoutType[]} layoutTypes
 * @param  {LayoutMode[]} modes
 */
function makeRecordLayoutMap(luvio, config, apiName, recordTypeId, layoutTypes, modes, adapterContext) {
    let promises = [];
    for (let i = 0; i < layoutTypes.length; i++) {
        const layoutType = layoutTypes[i];
        for (let m = 0; m < modes.length; m++) {
            const mode = modes[m];
            promises.push(Promise.resolve(makeCacheOnlySnapshot(luvio, config, adapterContext, buildLayoutModeCacheSnapshot(apiName, recordTypeId, layoutType, mode))).then((snapshot) => {
                return {
                    layoutType,
                    mode,
                    snapshot,
                };
            }));
        }
    }
    return Promise.all(promises).then((containers) => {
        let wrapper = {
            isStale: false,
        };
        for (let i = 0; i < containers.length; i++) {
            const container = containers[i];
            if (container.snapshot.data === undefined) {
                wrapper.isStale = false;
                wrapper.layoutMap = undefined;
                //if we have no data then break and force the adapter to go to the network
                break;
            }
            if (wrapper.layoutMap === undefined) {
                wrapper.layoutMap = {};
            }
            const { layoutType, mode, snapshot } = container;
            if (wrapper.layoutMap[layoutType] === undefined) {
                wrapper.layoutMap = assign({}, wrapper.layoutMap, {
                    [layoutType]: {},
                });
            }
            if (snapshot.data !== undefined) {
                wrapper.layoutMap[layoutType][mode] = snapshot.data;
            }
            wrapper.isStale = wrapper.isStale || isStaleSnapshot(snapshot);
        }
        return wrapper;
    });
}
function buildRecordLayoutSnapshot(recordLayout, luvio, config, recordLayoutSnapshot, objectInfoSnapshot) {
    return (context, storeLookup) => {
        const { layoutMap, isStale: isRecordLayoutStale } = recordLayout;
        if (layoutMap === undefined || objectInfoSnapshot.data === undefined) {
            return;
        }
        // transform the layouts & object info into a set of fields
        // // W-15692973: Name field of referenced entites are moved to optionalFields since the Name field may not exist for few entites.
        const { fields, optionalFields } = getFieldsFromLayoutMap(layoutMap, objectInfoSnapshot.data);
        let configWithModifiedOptionalFields = {
            ...config,
            optionalFields: config.optionalFields
                ? dedupe(config.optionalFields.concat(optionalFields)).sort()
                : optionalFields,
        };
        return getRecordRepSnapshot(luvio, configWithModifiedOptionalFields, recordLayoutSnapshot, objectInfoSnapshot, fields, isRecordLayoutStale, context, storeLookup);
    };
}
function buildRecordSnapshot(luvio, config, recordLayoutSnapshot, objectInfoSnapshot, fields) {
    return (context, storeLookup) => {
        return getRecordRepSnapshot(luvio, config, recordLayoutSnapshot, objectInfoSnapshot, fields, false, context, storeLookup);
    };
}
function getRecordRepSnapshot(luvio, config, recordLayoutSnapshot, objectInfoSnapshot, fields, isRecordLayoutStale, context, storeLookup) {
    if (objectInfoSnapshot.data === undefined) {
        return;
    }
    // transform the layouts & object info into a set of fields
    const optionalFields = config.optionalFields === undefined ? [] : dedupe(config.optionalFields).sort();
    const { recordId } = config;
    // borrow GetRecordFields' logic to construct the RecordRepresentation with the necessary fields
    const sel = buildRecordSelector(luvio, recordId, fields, optionalFields);
    const recordRepSnapshot = storeLookup(sel, buildSnapshotRefresh$3(luvio, config));
    if (isFulfilledSnapshot(recordRepSnapshot)) {
        // mark snapshot as stale if any of the information used to construct it was stale
        if (isStaleSnapshot(objectInfoSnapshot) ||
            isStaleSnapshot(recordLayoutSnapshot) ||
            isRecordLayoutStale) {
            recordRepSnapshot.state = 'Stale';
        }
    }
    // allow buildNetworkSnapshot() to use GetRecordFields if we were just missing some fields in L1
    else if (isUnfulfilledSnapshot(recordRepSnapshot)) {
        context.fields = fields;
        // W-15692973: Update optionalFields of config as the optional fields may have Name fields of referenced entities
        context.config.optionalFields = optionalFields;
    }
    // return however much of the record we were able to find in L1; cache policy decides if we
    // should consult L2 or go to network
    return recordRepSnapshot;
}
function runSnapshotOrPromise(promiseOrSnapshot, next) {
    if (isPromise(promiseOrSnapshot)) {
        return promiseOrSnapshot.then(next);
    }
    else {
        return next(promiseOrSnapshot);
    }
}

const contextId$4 = `${keyPrefix}__${adapterName$$}`;
// Custom adapter config due to `unsupported` items
const GET_RECORD_ADAPTER_CONFIG = {
    displayName: 'getRecord',
    parameters: {
        required: ['recordId'],
        optional: ['fields', 'layoutTypes', 'modes', 'optionalFields'],
        unsupported: [
            'childRelationships',
            'pageSize',
            'updateMru',
        ],
    },
};
function hasLayoutTypes(config) {
    return 'layoutTypes' in config;
}
function hasFieldsOrOptionalFields(config) {
    return 'fields' in config || 'optionalFields' in config;
}
function createResourceRequestFromRepresentation(representation, optionalFields) {
    const config = {
        urlParams: {
            recordId: representation.id,
        },
        queryParams: {
            optionalFields,
        },
    };
    return createResourceRequest$1h(config);
}
// eslint-disable-next-line @salesforce/lds/no-invalid-todo
// TODO: this should probably be code generated in RecordRepresentation
function coerceKeyParams(config) {
    const coercedConfig = {};
    const recordId = getRecordId18(config.recordId);
    if (recordId !== undefined) {
        coercedConfig.recordId = recordId;
    }
    return coercedConfig;
}
const notifyChangeFactory = (luvio) => {
    return function getUiApiRecordsByRecordIdNotifyChange(configs) {
        const keys = configs.map((c) => keyBuilder$3E(luvio, coerceKeyParams(c)));
        luvio.getNotifyChangeStoreEntries(keys).then((entries) => {
            const responsePromises = [];
            for (let i = 0, len = entries.length; i < len; i++) {
                const { key, record } = entries[i];
                const node = luvio.wrapNormalizedGraphNode(record, key);
                const optionalFields = getTrackedFields(key, node, {
                    maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnNotifyChange(),
                    onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
                });
                const refreshRequest = createResourceRequestFromRepresentation(record, optionalFields);
                const existingWeakEtag = record.weakEtag;
                const fieldTrie = convertFieldsToTrie([], false);
                const optionalFieldTrie = convertFieldsToTrie(optionalFields, true);
                const dispatchResourceRequestPromise = luvio
                    .dispatchResourceRequest(refreshRequest)
                    .then((response) => {
                    return luvio.handleSuccessResponse(() => {
                        const { body } = response;
                        luvio.storeIngest(key, createFieldsIngestSuccess$3({
                            fields: fieldTrie,
                            optionalFields: optionalFieldTrie,
                            trackedFields: optionalFieldTrie,
                            serverRequestCount: 1,
                        }), body);
                        const result = luvio.storeBroadcast();
                        instrumentation.getRecordNotifyChangeNetworkResult(existingWeakEtag !== body.weakEtag);
                        return result;
                    }, () => {
                        const cache = new StoreKeyMap();
                        getTypeCacheKeys$2j(cache, luvio, response.body);
                        return cache;
                    });
                }, (error) => {
                    return luvio.handleErrorResponse(() => {
                        const errorSnapshot = luvio.errorSnapshot(error);
                        luvio.storeIngestError(key, errorSnapshot, RECORD_REPRESENTATION_ERROR_STORE_METADATA_PARAMS);
                        const result = luvio.storeBroadcast();
                        instrumentation.getRecordNotifyChangeNetworkResult(null, true);
                        return result.then(() => errorSnapshot);
                    });
                });
                responsePromises.push(dispatchResourceRequestPromise);
            }
            return Promise.all(responsePromises);
        });
    };
};
const factory$h = (luvio) => {
    return luvio.withContext(function getRecord(untrustedConfig, adapterContext, requestContext) {
        // standard config validation and coercion
        const config = validateAdapterConfig$1c(untrustedConfig, GET_RECORD_ADAPTER_CONFIG);
        if (config === null) {
            return null;
        }
        if (hasLayoutTypes(config)) {
            return getRecordLayoutType(luvio, config, adapterContext, requestContext);
        }
        else if (hasFieldsOrOptionalFields(config)) {
            return getRecordByFields(luvio, config, requestContext);
        }
        return null;
    }, { contextId: contextId$4 });
};

const VERSION$2w = "7e00c51105cbf56a79ab8fcddf462c1e";
function validate$1C(obj, path = 'QuickActionExecutionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_contextId = obj.contextId;
        const path_contextId = path + '.contextId';
        let obj_contextId_union0 = null;
        const obj_contextId_union0_error = (() => {
            if (typeof obj_contextId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_contextId + '" (at "' + path_contextId + '")');
            }
        })();
        if (obj_contextId_union0_error != null) {
            obj_contextId_union0 = obj_contextId_union0_error.message;
        }
        let obj_contextId_union1 = null;
        const obj_contextId_union1_error = (() => {
            if (obj_contextId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_contextId + '" (at "' + path_contextId + '")');
            }
        })();
        if (obj_contextId_union1_error != null) {
            obj_contextId_union1 = obj_contextId_union1_error.message;
        }
        if (obj_contextId_union0 && obj_contextId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_contextId + '")';
            message += '\n' + obj_contextId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_contextId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_feedItemId = obj.feedItemId;
        const path_feedItemId = path + '.feedItemId';
        let obj_feedItemId_union0 = null;
        const obj_feedItemId_union0_error = (() => {
            if (typeof obj_feedItemId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_feedItemId + '" (at "' + path_feedItemId + '")');
            }
        })();
        if (obj_feedItemId_union0_error != null) {
            obj_feedItemId_union0 = obj_feedItemId_union0_error.message;
        }
        let obj_feedItemId_union1 = null;
        const obj_feedItemId_union1_error = (() => {
            if (obj_feedItemId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_feedItemId + '" (at "' + path_feedItemId + '")');
            }
        })();
        if (obj_feedItemId_union1_error != null) {
            obj_feedItemId_union1 = obj_feedItemId_union1_error.message;
        }
        if (obj_feedItemId_union0 && obj_feedItemId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_feedItemId + '")';
            message += '\n' + obj_feedItemId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_feedItemId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_isCreated = obj.isCreated;
        const path_isCreated = path + '.isCreated';
        if (typeof obj_isCreated !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isCreated + '" (at "' + path_isCreated + '")');
        }
        const obj_isSuccess = obj.isSuccess;
        const path_isSuccess = path + '.isSuccess';
        if (typeof obj_isSuccess !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isSuccess + '" (at "' + path_isSuccess + '")');
        }
        const obj_successMessage = obj.successMessage;
        const path_successMessage = path + '.successMessage';
        if (typeof obj_successMessage !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_successMessage + '" (at "' + path_successMessage + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$N = 'QuickActionExecutionRepresentation';
function keyBuilder$3g(luvio, config) {
    return keyPrefix + '::' + RepresentationType$N + ':' + config.id;
}
function keyBuilderFromType$s(luvio, object) {
    const keyParams = {
        id: object.id
    };
    return keyBuilder$3g(luvio, keyParams);
}
function normalize$F(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$38 = function QuickActionExecutionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2w,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'contextId',
                kind: 'Scalar'
            },
            {
                name: 'feedItemId',
                kind: 'Scalar'
            },
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'isCreated',
                kind: 'Scalar'
            },
            {
                name: 'isSuccess',
                kind: 'Scalar'
            },
            {
                name: 'successMessage',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$Y(existing, incoming) {
    const existing_isCreated = existing.isCreated;
    const incoming_isCreated = incoming.isCreated;
    if (!(existing_isCreated === incoming_isCreated)) {
        return false;
    }
    const existing_isSuccess = existing.isSuccess;
    const incoming_isSuccess = incoming.isSuccess;
    if (!(existing_isSuccess === incoming_isSuccess)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_successMessage = existing.successMessage;
    const incoming_successMessage = incoming.successMessage;
    if (!(existing_successMessage === incoming_successMessage)) {
        return false;
    }
    const existing_contextId = existing.contextId;
    const incoming_contextId = incoming.contextId;
    if (!(existing_contextId === incoming_contextId)) {
        return false;
    }
    const existing_feedItemId = existing.feedItemId;
    const incoming_feedItemId = incoming.feedItemId;
    if (!(existing_feedItemId === incoming_feedItemId)) {
        return false;
    }
    return true;
}
const ingest$23 = function QuickActionExecutionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1C(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$s(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$F, "UiApi", VERSION$2w, RepresentationType$N, equals$Y);
    return createLink$1(key);
};
function getTypeCacheKeys$27(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$s(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$N,
        mergeable: false
    });
}

const TTL$z = 900000;
const VERSION$2v = "993b0a7bce6056c4f57ed300ec153d9c";
function validate$1B(obj, path = 'QuickActionDefaultsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_actionApiName = obj.actionApiName;
        const path_actionApiName = path + '.actionApiName';
        if (typeof obj_actionApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_actionApiName + '" (at "' + path_actionApiName + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            if (typeof obj_fields_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
            }
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$M = 'QuickActionDefaultsRepresentation';
function keyBuilder$3f(luvio, config) {
    return keyPrefix + '::' + RepresentationType$M + ':' + config.actionApiName;
}
function keyBuilderFromType$r(luvio, object) {
    const keyParams = {
        actionApiName: object.actionApiName
    };
    return keyBuilder$3f(luvio, keyParams);
}
function dynamicNormalize$4(ingestParams) {
    return function normalize_dynamic(input, existing, path, luvio, store, timestamp) {
        const input_fields = input.fields;
        const input_fields_id = path.fullPath + '__fields';
        const input_fields_keys = Object.keys(input_fields);
        const input_fields_length = input_fields_keys.length;
        for (let i = 0; i < input_fields_length; i++) {
            const key = input_fields_keys[i];
            const input_fields_prop = input_fields[key];
            const input_fields_prop_id = input_fields_id + '__' + key;
            input_fields[key] = ingestParams.fields(input_fields_prop, {
                fullPath: input_fields_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        return input;
    };
}
const dynamicSelect$5 = function dynamicQuickActionDefaultsRepresentationSelect(params) {
    const fieldsPathSelection = params.fields === undefined ? {
        name: 'fields',
        kind: 'Link',
        map: true,
        fragment: select$3r()
    } : params.fields;
    return {
        kind: 'Fragment',
        version: VERSION$2v,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'actionApiName',
                kind: 'Scalar'
            },
            fieldsPathSelection,
            {
                name: 'objectApiName',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$X(existing, incoming) {
    const existing_actionApiName = existing.actionApiName;
    const incoming_actionApiName = incoming.actionApiName;
    if (!(existing_actionApiName === incoming_actionApiName)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_props = equalsObject(existing_fields, incoming_fields, (existing_fields_prop, incoming_fields_prop) => {
        if (!(existing_fields_prop.__ref === incoming_fields_prop.__ref)) {
            return false;
        }
    });
    if (equals_fields_props === false) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$26(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$r(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$M,
        mergeable: false
    });
    const input_fields = input.fields;
    const input_fields_keys = ObjectKeys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        getTypeCacheKeys$2m(rootKeySet, luvio, input_fields[key], () => rootKey + "__fields" + "__" + key);
    }
}

function toSortedStringArrayAllowEmpty(value) {
    const valueArray = isArray(value) ? value : [value];
    if (valueArray.length === 0) {
        return valueArray;
    }
    return toSortedStringArray(valueArray);
}

var FormFactor;
(function (FormFactor) {
    FormFactor["Large"] = "Large";
    FormFactor["Medium"] = "Medium";
    FormFactor["Small"] = "Small";
})(FormFactor || (FormFactor = {}));

function coerceFormFactor(form) {
    if (form === FormFactor.Large || form === FormFactor.Medium || form === FormFactor.Small) {
        return form;
    }
    return undefined;
}

const VERSION$2u = "3f49d751896cf66e6e29788d8880e2cc";
function validate$1A(obj, path = 'PlatformActionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_actionListContext = obj.actionListContext;
        const path_actionListContext = path + '.actionListContext';
        if (typeof obj_actionListContext !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_actionListContext + '" (at "' + path_actionListContext + '")');
        }
        const obj_actionTarget = obj.actionTarget;
        const path_actionTarget = path + '.actionTarget';
        let obj_actionTarget_union0 = null;
        const obj_actionTarget_union0_error = (() => {
            if (typeof obj_actionTarget !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_actionTarget + '" (at "' + path_actionTarget + '")');
            }
        })();
        if (obj_actionTarget_union0_error != null) {
            obj_actionTarget_union0 = obj_actionTarget_union0_error.message;
        }
        let obj_actionTarget_union1 = null;
        const obj_actionTarget_union1_error = (() => {
            if (obj_actionTarget !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_actionTarget + '" (at "' + path_actionTarget + '")');
            }
        })();
        if (obj_actionTarget_union1_error != null) {
            obj_actionTarget_union1 = obj_actionTarget_union1_error.message;
        }
        if (obj_actionTarget_union0 && obj_actionTarget_union1) {
            let message = 'Object doesn\'t match union (at "' + path_actionTarget + '")';
            message += '\n' + obj_actionTarget_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_actionTarget_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_actionTargetType = obj.actionTargetType;
        const path_actionTargetType = path + '.actionTargetType';
        let obj_actionTargetType_union0 = null;
        const obj_actionTargetType_union0_error = (() => {
            if (typeof obj_actionTargetType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_actionTargetType + '" (at "' + path_actionTargetType + '")');
            }
        })();
        if (obj_actionTargetType_union0_error != null) {
            obj_actionTargetType_union0 = obj_actionTargetType_union0_error.message;
        }
        let obj_actionTargetType_union1 = null;
        const obj_actionTargetType_union1_error = (() => {
            if (obj_actionTargetType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_actionTargetType + '" (at "' + path_actionTargetType + '")');
            }
        })();
        if (obj_actionTargetType_union1_error != null) {
            obj_actionTargetType_union1 = obj_actionTargetType_union1_error.message;
        }
        if (obj_actionTargetType_union0 && obj_actionTargetType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_actionTargetType + '")';
            message += '\n' + obj_actionTargetType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_actionTargetType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_externalId = obj.externalId;
        const path_externalId = path + '.externalId';
        if (typeof obj_externalId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_externalId + '" (at "' + path_externalId + '")');
        }
        const obj_iconUrl = obj.iconUrl;
        const path_iconUrl = path + '.iconUrl';
        let obj_iconUrl_union0 = null;
        const obj_iconUrl_union0_error = (() => {
            if (typeof obj_iconUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union0_error != null) {
            obj_iconUrl_union0 = obj_iconUrl_union0_error.message;
        }
        let obj_iconUrl_union1 = null;
        const obj_iconUrl_union1_error = (() => {
            if (obj_iconUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union1_error != null) {
            obj_iconUrl_union1 = obj_iconUrl_union1_error.message;
        }
        if (obj_iconUrl_union0 && obj_iconUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_iconUrl + '")';
            message += '\n' + obj_iconUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_iconUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_isMassAction = obj.isMassAction;
        const path_isMassAction = path + '.isMassAction';
        if (typeof obj_isMassAction !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_isMassAction + '" (at "' + path_isMassAction + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_lwcComponent = obj.lwcComponent;
        const path_lwcComponent = path + '.lwcComponent';
        let obj_lwcComponent_union0 = null;
        const obj_lwcComponent_union0_error = (() => {
            if (typeof obj_lwcComponent !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lwcComponent + '" (at "' + path_lwcComponent + '")');
            }
        })();
        if (obj_lwcComponent_union0_error != null) {
            obj_lwcComponent_union0 = obj_lwcComponent_union0_error.message;
        }
        let obj_lwcComponent_union1 = null;
        const obj_lwcComponent_union1_error = (() => {
            if (obj_lwcComponent !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lwcComponent + '" (at "' + path_lwcComponent + '")');
            }
        })();
        if (obj_lwcComponent_union1_error != null) {
            obj_lwcComponent_union1 = obj_lwcComponent_union1_error.message;
        }
        if (obj_lwcComponent_union0 && obj_lwcComponent_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lwcComponent + '")';
            message += '\n' + obj_lwcComponent_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lwcComponent_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_primaryColor = obj.primaryColor;
        const path_primaryColor = path + '.primaryColor';
        let obj_primaryColor_union0 = null;
        const obj_primaryColor_union0_error = (() => {
            if (typeof obj_primaryColor !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_primaryColor + '" (at "' + path_primaryColor + '")');
            }
        })();
        if (obj_primaryColor_union0_error != null) {
            obj_primaryColor_union0 = obj_primaryColor_union0_error.message;
        }
        let obj_primaryColor_union1 = null;
        const obj_primaryColor_union1_error = (() => {
            if (obj_primaryColor !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_primaryColor + '" (at "' + path_primaryColor + '")');
            }
        })();
        if (obj_primaryColor_union1_error != null) {
            obj_primaryColor_union1 = obj_primaryColor_union1_error.message;
        }
        if (obj_primaryColor_union0 && obj_primaryColor_union1) {
            let message = 'Object doesn\'t match union (at "' + path_primaryColor + '")';
            message += '\n' + obj_primaryColor_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_primaryColor_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_relatedListRecordId = obj.relatedListRecordId;
        const path_relatedListRecordId = path + '.relatedListRecordId';
        let obj_relatedListRecordId_union0 = null;
        const obj_relatedListRecordId_union0_error = (() => {
            if (typeof obj_relatedListRecordId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_relatedListRecordId + '" (at "' + path_relatedListRecordId + '")');
            }
        })();
        if (obj_relatedListRecordId_union0_error != null) {
            obj_relatedListRecordId_union0 = obj_relatedListRecordId_union0_error.message;
        }
        let obj_relatedListRecordId_union1 = null;
        const obj_relatedListRecordId_union1_error = (() => {
            if (obj_relatedListRecordId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_relatedListRecordId + '" (at "' + path_relatedListRecordId + '")');
            }
        })();
        if (obj_relatedListRecordId_union1_error != null) {
            obj_relatedListRecordId_union1 = obj_relatedListRecordId_union1_error.message;
        }
        if (obj_relatedListRecordId_union0 && obj_relatedListRecordId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_relatedListRecordId + '")';
            message += '\n' + obj_relatedListRecordId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_relatedListRecordId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_relatedSourceObject = obj.relatedSourceObject;
        const path_relatedSourceObject = path + '.relatedSourceObject';
        let obj_relatedSourceObject_union0 = null;
        const obj_relatedSourceObject_union0_error = (() => {
            if (typeof obj_relatedSourceObject !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_relatedSourceObject + '" (at "' + path_relatedSourceObject + '")');
            }
        })();
        if (obj_relatedSourceObject_union0_error != null) {
            obj_relatedSourceObject_union0 = obj_relatedSourceObject_union0_error.message;
        }
        let obj_relatedSourceObject_union1 = null;
        const obj_relatedSourceObject_union1_error = (() => {
            if (obj_relatedSourceObject !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_relatedSourceObject + '" (at "' + path_relatedSourceObject + '")');
            }
        })();
        if (obj_relatedSourceObject_union1_error != null) {
            obj_relatedSourceObject_union1 = obj_relatedSourceObject_union1_error.message;
        }
        if (obj_relatedSourceObject_union0 && obj_relatedSourceObject_union1) {
            let message = 'Object doesn\'t match union (at "' + path_relatedSourceObject + '")';
            message += '\n' + obj_relatedSourceObject_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_relatedSourceObject_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_section = obj.section;
        const path_section = path + '.section';
        let obj_section_union0 = null;
        const obj_section_union0_error = (() => {
            if (typeof obj_section !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_section + '" (at "' + path_section + '")');
            }
        })();
        if (obj_section_union0_error != null) {
            obj_section_union0 = obj_section_union0_error.message;
        }
        let obj_section_union1 = null;
        const obj_section_union1_error = (() => {
            if (obj_section !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_section + '" (at "' + path_section + '")');
            }
        })();
        if (obj_section_union1_error != null) {
            obj_section_union1 = obj_section_union1_error.message;
        }
        if (obj_section_union0 && obj_section_union1) {
            let message = 'Object doesn\'t match union (at "' + path_section + '")';
            message += '\n' + obj_section_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_section_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_sourceObject = obj.sourceObject;
        const path_sourceObject = path + '.sourceObject';
        if (typeof obj_sourceObject !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_sourceObject + '" (at "' + path_sourceObject + '")');
        }
        const obj_subtype = obj.subtype;
        const path_subtype = path + '.subtype';
        let obj_subtype_union0 = null;
        const obj_subtype_union0_error = (() => {
            if (typeof obj_subtype !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_subtype + '" (at "' + path_subtype + '")');
            }
        })();
        if (obj_subtype_union0_error != null) {
            obj_subtype_union0 = obj_subtype_union0_error.message;
        }
        let obj_subtype_union1 = null;
        const obj_subtype_union1_error = (() => {
            if (obj_subtype !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_subtype + '" (at "' + path_subtype + '")');
            }
        })();
        if (obj_subtype_union1_error != null) {
            obj_subtype_union1 = obj_subtype_union1_error.message;
        }
        if (obj_subtype_union0 && obj_subtype_union1) {
            let message = 'Object doesn\'t match union (at "' + path_subtype + '")';
            message += '\n' + obj_subtype_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_subtype_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_targetObject = obj.targetObject;
        const path_targetObject = path + '.targetObject';
        let obj_targetObject_union0 = null;
        const obj_targetObject_union0_error = (() => {
            if (typeof obj_targetObject !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_targetObject + '" (at "' + path_targetObject + '")');
            }
        })();
        if (obj_targetObject_union0_error != null) {
            obj_targetObject_union0 = obj_targetObject_union0_error.message;
        }
        let obj_targetObject_union1 = null;
        const obj_targetObject_union1_error = (() => {
            if (obj_targetObject !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_targetObject + '" (at "' + path_targetObject + '")');
            }
        })();
        if (obj_targetObject_union1_error != null) {
            obj_targetObject_union1 = obj_targetObject_union1_error.message;
        }
        if (obj_targetObject_union0 && obj_targetObject_union1) {
            let message = 'Object doesn\'t match union (at "' + path_targetObject + '")';
            message += '\n' + obj_targetObject_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_targetObject_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_targetUrl = obj.targetUrl;
        const path_targetUrl = path + '.targetUrl';
        let obj_targetUrl_union0 = null;
        const obj_targetUrl_union0_error = (() => {
            if (typeof obj_targetUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_targetUrl + '" (at "' + path_targetUrl + '")');
            }
        })();
        if (obj_targetUrl_union0_error != null) {
            obj_targetUrl_union0 = obj_targetUrl_union0_error.message;
        }
        let obj_targetUrl_union1 = null;
        const obj_targetUrl_union1_error = (() => {
            if (obj_targetUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_targetUrl + '" (at "' + path_targetUrl + '")');
            }
        })();
        if (obj_targetUrl_union1_error != null) {
            obj_targetUrl_union1 = obj_targetUrl_union1_error.message;
        }
        if (obj_targetUrl_union0 && obj_targetUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_targetUrl + '")';
            message += '\n' + obj_targetUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_targetUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$L = 'PlatformActionRepresentation';
function keyBuilder$3e(luvio, config) {
    return keyPrefix + '::' + RepresentationType$L + ':' + config.externalId + ':' + (config.relatedSourceObject === null ? '' : config.relatedSourceObject) + ':' + (config.relatedListRecordId === null ? '' : config.relatedListRecordId);
}
function keyBuilderFromType$q(luvio, object) {
    const keyParams = {
        externalId: object.externalId,
        relatedSourceObject: object.relatedSourceObject,
        relatedListRecordId: object.relatedListRecordId
    };
    return keyBuilder$3e(luvio, keyParams);
}
function normalize$E(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$37 = function PlatformActionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2u,
        private: [
            'id'
        ],
        selections: [
            {
                name: 'actionListContext',
                kind: 'Scalar'
            },
            {
                name: 'actionTarget',
                kind: 'Scalar'
            },
            {
                name: 'actionTargetType',
                kind: 'Scalar'
            },
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'externalId',
                kind: 'Scalar'
            },
            {
                name: 'iconUrl',
                kind: 'Scalar'
            },
            {
                name: 'isMassAction',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'lwcComponent',
                kind: 'Scalar'
            },
            {
                name: 'primaryColor',
                kind: 'Scalar'
            },
            {
                name: 'relatedListRecordId',
                kind: 'Scalar'
            },
            {
                name: 'relatedSourceObject',
                kind: 'Scalar'
            },
            {
                name: 'section',
                kind: 'Scalar'
            },
            {
                name: 'sourceObject',
                kind: 'Scalar'
            },
            {
                name: 'subtype',
                kind: 'Scalar'
            },
            {
                name: 'targetObject',
                kind: 'Scalar'
            },
            {
                name: 'targetUrl',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$W(existing, incoming) {
    const existing_actionListContext = existing.actionListContext;
    const incoming_actionListContext = incoming.actionListContext;
    if (!(existing_actionListContext === incoming_actionListContext)) {
        return false;
    }
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_externalId = existing.externalId;
    const incoming_externalId = incoming.externalId;
    if (!(existing_externalId === incoming_externalId)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_isMassAction = existing.isMassAction;
    const incoming_isMassAction = incoming.isMassAction;
    if (!(existing_isMassAction === incoming_isMassAction)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_sourceObject = existing.sourceObject;
    const incoming_sourceObject = incoming.sourceObject;
    if (!(existing_sourceObject === incoming_sourceObject)) {
        return false;
    }
    const existing_type = existing.type;
    const incoming_type = incoming.type;
    if (!(existing_type === incoming_type)) {
        return false;
    }
    const existing_actionTarget = existing.actionTarget;
    const incoming_actionTarget = incoming.actionTarget;
    if (!(existing_actionTarget === incoming_actionTarget)) {
        return false;
    }
    const existing_actionTargetType = existing.actionTargetType;
    const incoming_actionTargetType = incoming.actionTargetType;
    if (!(existing_actionTargetType === incoming_actionTargetType)) {
        return false;
    }
    const existing_iconUrl = existing.iconUrl;
    const incoming_iconUrl = incoming.iconUrl;
    if (!(existing_iconUrl === incoming_iconUrl)) {
        return false;
    }
    const existing_lwcComponent = existing.lwcComponent;
    const incoming_lwcComponent = incoming.lwcComponent;
    if (!(existing_lwcComponent === incoming_lwcComponent)) {
        return false;
    }
    const existing_primaryColor = existing.primaryColor;
    const incoming_primaryColor = incoming.primaryColor;
    if (!(existing_primaryColor === incoming_primaryColor)) {
        return false;
    }
    const existing_relatedListRecordId = existing.relatedListRecordId;
    const incoming_relatedListRecordId = incoming.relatedListRecordId;
    if (!(existing_relatedListRecordId === incoming_relatedListRecordId)) {
        return false;
    }
    const existing_relatedSourceObject = existing.relatedSourceObject;
    const incoming_relatedSourceObject = incoming.relatedSourceObject;
    if (!(existing_relatedSourceObject === incoming_relatedSourceObject)) {
        return false;
    }
    const existing_section = existing.section;
    const incoming_section = incoming.section;
    if (!(existing_section === incoming_section)) {
        return false;
    }
    const existing_subtype = existing.subtype;
    const incoming_subtype = incoming.subtype;
    if (!(existing_subtype === incoming_subtype)) {
        return false;
    }
    const existing_targetObject = existing.targetObject;
    const incoming_targetObject = incoming.targetObject;
    if (!(existing_targetObject === incoming_targetObject)) {
        return false;
    }
    const existing_targetUrl = existing.targetUrl;
    const incoming_targetUrl = incoming.targetUrl;
    if (!(existing_targetUrl === incoming_targetUrl)) {
        return false;
    }
    return true;
}
const ingest$22 = function PlatformActionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1A(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$q(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$E, "UiApi", VERSION$2u, RepresentationType$L, equals$W);
    return createLink$1(key);
};
function getTypeCacheKeys$25(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$q(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$L,
        mergeable: false
    });
}

const VERSION$2t = "378d506f563a4bd724b322d440df33d1";
function validate$1z(obj, path = 'EntityActionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_actions = obj.actions;
        const path_actions = path + '.actions';
        if (!ArrayIsArray(obj_actions)) {
            return new TypeError('Expected "array" but received "' + typeof obj_actions + '" (at "' + path_actions + '")');
        }
        for (let i = 0; i < obj_actions.length; i++) {
            const obj_actions_item = obj_actions[i];
            const path_actions_item = path_actions + '[' + i + ']';
            if (typeof obj_actions_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_actions_item + '" (at "' + path_actions_item + '")');
            }
        }
        const obj_links = obj.links;
        const path_links = path + '.links';
        if (!ArrayIsArray(obj_links)) {
            return new TypeError('Expected "array" but received "' + typeof obj_links + '" (at "' + path_links + '")');
        }
        for (let i = 0; i < obj_links.length; i++) {
            const obj_links_item = obj_links[i];
            const path_links_item = path_links + '[' + i + ']';
            if (typeof obj_links_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_links_item + '" (at "' + path_links_item + '")');
            }
        }
        const obj_url = obj.url;
        const path_url = path + '.url';
        if (typeof obj_url !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_url + '" (at "' + path_url + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$K = 'EntityActionRepresentation';
function keyBuilder$3d(luvio, config) {
    return keyPrefix + '::' + RepresentationType$K + ':' + config.url;
}
function keyBuilderFromType$p(luvio, object) {
    const keyParams = {
        url: object.url
    };
    return keyBuilder$3d(luvio, keyParams);
}
function normalize$D(input, existing, path, luvio, store, timestamp) {
    const input_actions = input.actions;
    const input_actions_id = path.fullPath + '__actions';
    for (let i = 0; i < input_actions.length; i++) {
        const input_actions_item = input_actions[i];
        let input_actions_item_id = input_actions_id + '__' + i;
        input_actions[i] = ingest$22(input_actions_item, {
            fullPath: input_actions_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$36 = function EntityActionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2t,
        private: [
            'links',
            'url'
        ],
        selections: [
            {
                name: 'actions',
                kind: 'Link',
                plural: true,
                fragment: select$37()
            }
        ]
    };
};
function equals$V(existing, incoming) {
    const existing_url = existing.url;
    const incoming_url = incoming.url;
    if (!(existing_url === incoming_url)) {
        return false;
    }
    const existing_actions = existing.actions;
    const incoming_actions = incoming.actions;
    const equals_actions_items = equalsArray(existing_actions, incoming_actions, (existing_actions_item, incoming_actions_item) => {
        if (!(existing_actions_item.__ref === incoming_actions_item.__ref)) {
            return false;
        }
    });
    if (equals_actions_items === false) {
        return false;
    }
    const existing_links = existing.links;
    const incoming_links = incoming.links;
    const equals_links_items = equalsArray(existing_links, incoming_links, (existing_links_item, incoming_links_item) => {
        if (!(existing_links_item === incoming_links_item)) {
            return false;
        }
    });
    if (equals_links_items === false) {
        return false;
    }
    return true;
}
const ingest$21 = function EntityActionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1z(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$p(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$D, "UiApi", VERSION$2t, RepresentationType$K, equals$V);
    return createLink$1(key);
};
function getTypeCacheKeys$24(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$p(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$K,
        mergeable: false
    });
    const input_actions_length = input.actions.length;
    for (let i = 0; i < input_actions_length; i++) {
        getTypeCacheKeys$25(rootKeySet, luvio, input.actions[i]);
    }
}

const TTL$y = 300000;
const VERSION$2s = "e485d96c1402a9ca2f56e56485af0216";
function validate$1y(obj, path = 'ActionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_actions = obj.actions;
        const path_actions = path + '.actions';
        if (typeof obj_actions !== 'object' || ArrayIsArray(obj_actions) || obj_actions === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_actions + '" (at "' + path_actions + '")');
        }
        const obj_actions_keys = ObjectKeys(obj_actions);
        for (let i = 0; i < obj_actions_keys.length; i++) {
            const key = obj_actions_keys[i];
            const obj_actions_prop = obj_actions[key];
            const path_actions_prop = path_actions + '["' + key + '"]';
            if (typeof obj_actions_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_actions_prop + '" (at "' + path_actions_prop + '")');
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_url = obj.url;
        const path_url = path + '.url';
        if (typeof obj_url !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_url + '" (at "' + path_url + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$J = 'ActionRepresentation';
function normalize$C(input, existing, path, luvio, store, timestamp) {
    const input_actions = input.actions;
    const input_actions_id = path.fullPath + '__actions';
    const input_actions_keys = Object.keys(input_actions);
    const input_actions_length = input_actions_keys.length;
    for (let i = 0; i < input_actions_length; i++) {
        const key = input_actions_keys[i];
        const input_actions_prop = input_actions[key];
        const input_actions_prop_id = input_actions_id + '__' + key;
        input_actions[key] = ingest$21(input_actions_prop, {
            fullPath: input_actions_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$35 = function ActionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2s,
        private: [
            'eTag',
            'url'
        ],
        selections: [
            {
                name: 'actions',
                kind: 'Link',
                map: true,
                fragment: select$36()
            }
        ]
    };
};
function equals$U(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_url = existing.url;
    const incoming_url = incoming.url;
    if (!(existing_url === incoming_url)) {
        return false;
    }
    const existing_actions = existing.actions;
    const incoming_actions = incoming.actions;
    const equals_actions_props = equalsObject(existing_actions, incoming_actions, (existing_actions_prop, incoming_actions_prop) => {
        if (!(existing_actions_prop.__ref === incoming_actions_prop.__ref)) {
            return false;
        }
    });
    if (equals_actions_props === false) {
        return false;
    }
    return true;
}
const ingest$20 = function ActionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1y(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$y;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$C, "UiApi", VERSION$2s, RepresentationType$J, equals$U);
    return createLink$1(key);
};
function getTypeCacheKeys$23(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$J,
        mergeable: false
    });
    const input_actions = input.actions;
    const input_actions_keys = ObjectKeys(input_actions);
    const input_actions_length = input_actions_keys.length;
    for (let i = 0; i < input_actions_length; i++) {
        const key = input_actions_keys[i];
        getTypeCacheKeys$24(rootKeySet, luvio, input_actions[key]);
    }
}

function select$34(luvio, params) {
    return select$35();
}
function keyBuilder$3c(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'actionTypes:' + params.queryParams.actionTypes + ',' + 'apiNames:' + params.queryParams.apiNames + ',' + 'formFactor:' + params.queryParams.formFactor + ',' + 'retrievalMode:' + params.queryParams.retrievalMode + ',' + 'sections:' + params.queryParams.sections + ')';
}
function getResponseCacheKeys$10(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$3c(luvio, resourceParams));
}
function ingestSuccess$R(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$3c(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$34(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$K(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$3c(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$15(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/global',
        method: 'get',
        body: null,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$V = 'getGlobalActions';
const getGlobalActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionTypes', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('apiNames', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArrayAllowEmpty),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('retrievalMode', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sections', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getGlobalActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$V, getGlobalActions_ConfigPropertyMetadata);
const createResourceParams$_ = /*#__PURE__*/ createResourceParams$k(getGlobalActions_ConfigPropertyMetadata);
function keyBuilder$3b(luvio, config) {
    const resourceParams = createResourceParams$_(config);
    return keyBuilder$3c(luvio, resourceParams);
}
function typeCheckConfig$12(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getGlobalActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$15(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getGlobalActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$12(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$J(luvio, config) {
    createResourceParams$_(config);
    return select$34();
}
function onFetchResponseSuccess$M(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$R(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$13(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$N(luvio, config, resourceParams, response) {
    const snapshot = ingestError$K(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$13(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$13(luvio, config, options) {
    const resourceParams = createResourceParams$_(config);
    const request = createResourceRequest$15(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$M(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$10(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$N(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$R(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$13, undefined, false);
}
function buildCachedSnapshotCachePolicy$Q(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$3b(luvio, config),
        node: adapterFragment$J(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$13(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getGlobalActionsAdapterFactory = (luvio) => function UiApi__getGlobalActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$15(untrustedConfig, getGlobalActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$Q, buildNetworkSnapshotCachePolicy$R);
};

const TTL$x = 900000;
const VERSION$2r = "35f3eec8ce7f6001c6d5d17821b75bb9";
function validate$1x(obj, path = 'QuickActionLayoutRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_actionApiName = obj.actionApiName;
        const path_actionApiName = path + '.actionApiName';
        if (typeof obj_actionApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_actionApiName + '" (at "' + path_actionApiName + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_layout = obj.layout;
        const path_layout = path + '.layout';
        const referencepath_layoutValidationError = validate$1I(obj_layout, path_layout);
        if (referencepath_layoutValidationError !== null) {
            let message = 'Object doesn\'t match RecordLayoutRepresentation (at "' + path_layout + '")\n';
            message += referencepath_layoutValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$I = 'QuickActionLayoutRepresentation';
function keyBuilder$3a(luvio, config) {
    return keyPrefix + '::' + RepresentationType$I + ':' + config.actionApiName;
}
function keyBuilderFromType$o(luvio, object) {
    const keyParams = {
        actionApiName: object.actionApiName
    };
    return keyBuilder$3a(luvio, keyParams);
}
function normalize$B(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$33 = function QuickActionLayoutRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2r,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'actionApiName',
                kind: 'Scalar'
            },
            {
                name: 'layout',
                kind: 'Object',
                opaque: true
            }
        ]
    };
};
function equals$T(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$1$ = function QuickActionLayoutRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1x(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$o(luvio, input);
    const ttlToUse = TTL$x;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$B, "UiApi", VERSION$2r, RepresentationType$I, equals$T);
    return createLink$1(key);
};
function getTypeCacheKeys$22(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$o(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$I,
        mergeable: false
    });
}

function select$32(luvio, params) {
    return select$33();
}
function keyBuilder$39(luvio, params) {
    return keyBuilder$3a(luvio, {
        actionApiName: params.urlParams.actionApiName
    });
}
function getResponseCacheKeys$$(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$22(storeKeyMap, luvio, response);
}
function ingestSuccess$Q(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$39(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1$, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$32(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$J(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$39(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$x,
        namespace: keyPrefix,
        version: VERSION$2r,
        representationName: RepresentationType$I
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$14(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/layout/' + config.urlParams.actionApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$U = 'getQuickActionLayout';
const getQuickActionLayout_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getQuickActionLayout_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$U, getQuickActionLayout_ConfigPropertyMetadata);
const createResourceParams$Z = /*#__PURE__*/ createResourceParams$k(getQuickActionLayout_ConfigPropertyMetadata);
function keyBuilder$38(luvio, config) {
    const resourceParams = createResourceParams$Z(config);
    return keyBuilder$39(luvio, resourceParams);
}
function typeCheckConfig$11(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getQuickActionLayout_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$14(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$11(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$I(luvio, config) {
    createResourceParams$Z(config);
    return select$32();
}
function onFetchResponseSuccess$L(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$Q(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$12(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$M(luvio, config, resourceParams, response) {
    const snapshot = ingestError$J(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$12(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$12(luvio, config, options) {
    const resourceParams = createResourceParams$Z(config);
    const request = createResourceRequest$14(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$L(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$$(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$M(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$Q(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$12, undefined, false);
}
function buildCachedSnapshotCachePolicy$P(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$38(luvio, config),
        node: adapterFragment$I(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$12(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getQuickActionLayoutAdapterFactory = (luvio) => function UiApi__getQuickActionLayout(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$14(untrustedConfig, getQuickActionLayout_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$P, buildNetworkSnapshotCachePolicy$Q);
};

/**
 * Returns the object API name.
 * @param value The value from which to get the object API name.
 * @returns The object API name.
 */
function getObjectApiNamesArray(value) {
    const valueArray = isArray(value) ? value : [value];
    const array = [];
    for (let i = 0, len = valueArray.length; i < len; i += 1) {
        const item = valueArray[i];
        const apiName = getObjectApiName$1(item);
        if (apiName === undefined) {
            return undefined;
        }
        push.call(array, apiName);
    }
    if (array.length === 0) {
        return undefined;
    }
    return dedupe(array);
}

/**
 * Returns the object API name.
 * @param value The value from which to get the object API name.
 * @returns The object API name.
 */
function getSortedObjectApiNamesArray(value) {
    const unsortedArray = getObjectApiNamesArray(value);
    return unsortedArray === undefined ? undefined : unsortedArray.sort();
}

function select$31(luvio, params) {
    return select$35();
}
function keyBuilder$37(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'actionTypes:' + params.queryParams.actionTypes + ',' + 'formFactor:' + params.queryParams.formFactor + ',' + 'sections:' + params.queryParams.sections + ',' + 'objectApiNames:' + params.urlParams.objectApiNames + ')';
}
function getResponseCacheKeys$_(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$37(luvio, resourceParams));
}
function ingestSuccess$P(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$37(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$31(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$I(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$37(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$13(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/lookup/' + config.urlParams.objectApiNames + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$T = 'getLookupActions';
const getLookupActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiNames', true, 0 /* UrlParameter */, 0 /* String */, true, getSortedObjectApiNamesArray),
    generateParamConfigMetadata('actionTypes', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('sections', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getLookupActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$T, getLookupActions_ConfigPropertyMetadata);
const createResourceParams$Y = /*#__PURE__*/ createResourceParams$k(getLookupActions_ConfigPropertyMetadata);
function keyBuilder$36(luvio, config) {
    const resourceParams = createResourceParams$Y(config);
    return keyBuilder$37(luvio, resourceParams);
}
function typeCheckConfig$10(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getLookupActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$13(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getLookupActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$10(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$H(luvio, config) {
    createResourceParams$Y(config);
    return select$31();
}
function onFetchResponseSuccess$K(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$P(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$11(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$L(luvio, config, resourceParams, response) {
    const snapshot = ingestError$I(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$11(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$11(luvio, config, options) {
    const resourceParams = createResourceParams$Y(config);
    const request = createResourceRequest$13(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$K(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$_(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$L(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$P(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$11, undefined, false);
}
function buildCachedSnapshotCachePolicy$O(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$36(luvio, config),
        node: adapterFragment$H(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$11(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getLookupActionsAdapterFactory = (luvio) => function UiApi__getLookupActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$13(untrustedConfig, getLookupActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$O, buildNetworkSnapshotCachePolicy$P);
};

function select$30(luvio, params) {
    return select$35();
}
function keyBuilder$35(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'actionTypes:' + params.queryParams.actionTypes + ',' + 'formFactor:' + params.queryParams.formFactor + ',' + 'sections:' + params.queryParams.sections + ',' + 'objectApiName:' + params.urlParams.objectApiName + ')';
}
function getResponseCacheKeys$Z(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$35(luvio, resourceParams));
}
function ingestSuccess$O(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$35(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$30(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$H(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$35(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$12(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/object/' + config.urlParams.objectApiName + '/record-create',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$S = 'getObjectCreateActions';
const getObjectCreateActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('actionTypes', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('sections', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getObjectCreateActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$S, getObjectCreateActions_ConfigPropertyMetadata);
const createResourceParams$X = /*#__PURE__*/ createResourceParams$k(getObjectCreateActions_ConfigPropertyMetadata);
function keyBuilder$34(luvio, config) {
    const resourceParams = createResourceParams$X(config);
    return keyBuilder$35(luvio, resourceParams);
}
function typeCheckConfig$$(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getObjectCreateActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$12(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getObjectCreateActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$$(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$G(luvio, config) {
    createResourceParams$X(config);
    return select$30();
}
function onFetchResponseSuccess$J(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$O(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$10(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$K(luvio, config, resourceParams, response) {
    const snapshot = ingestError$H(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$10(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$10(luvio, config, options) {
    const resourceParams = createResourceParams$X(config);
    const request = createResourceRequest$12(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$J(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$Z(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$K(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$O(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$10, undefined, false);
}
function buildCachedSnapshotCachePolicy$N(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$34(luvio, config),
        node: adapterFragment$G(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$10(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getObjectCreateActionsAdapterFactory = (luvio) => function UiApi__getObjectCreateActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$12(untrustedConfig, getObjectCreateActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$N, buildNetworkSnapshotCachePolicy$O);
};

const VERSION$2q = "fecd80e9e24a1c1e75fd5395cd34ff2e";
function validate$1w(obj, path = 'ActionOverrideRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (typeof obj_records !== 'object' || ArrayIsArray(obj_records) || obj_records === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$H = 'ActionOverrideRepresentation';
function normalize$A(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2$ = function ActionOverrideRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2q,
        private: [],
        opaque: true
    };
};
function equals$S(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1_ = function ActionOverrideRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1w(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$A, "UiApi", VERSION$2q, RepresentationType$H, equals$S);
    return createLink$1(key);
};
function getTypeCacheKeys$21(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$H,
        mergeable: false
    });
}

function select$2_(luvio, params) {
    return select$2$();
}
function keyBuilder$33(luvio, params) {
    return keyPrefix + '::ActionOverrideRepresentation:(' + 'defaultTemplateOnly:' + params.queryParams.defaultTemplateOnly + ',' + 'type:' + params.queryParams.type + ',' + 'objectApiName:' + params.urlParams.objectApiName + ')';
}
function getResponseCacheKeys$Y(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$21(storeKeyMap, luvio, response, () => keyBuilder$33(luvio, resourceParams));
}
function ingestSuccess$N(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$33(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1_, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2_(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$G(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$33(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$11(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/overrides/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$R = 'getActionOverrides';
const getActionOverrides_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('defaultTemplateOnly', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('type', false, 1 /* QueryParameter */, 0 /* String */),
];
const getActionOverrides_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$R, getActionOverrides_ConfigPropertyMetadata);
const createResourceParams$W = /*#__PURE__*/ createResourceParams$k(getActionOverrides_ConfigPropertyMetadata);
function keyBuilder$32(luvio, config) {
    const resourceParams = createResourceParams$W(config);
    return keyBuilder$33(luvio, resourceParams);
}
function typeCheckConfig$_(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getActionOverrides_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$11(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getActionOverrides_ConfigPropertyMetadata);
    const config = typeCheckConfig$_(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$F(luvio, config) {
    createResourceParams$W(config);
    return select$2_();
}
function onFetchResponseSuccess$I(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$N(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$$(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$J(luvio, config, resourceParams, response) {
    const snapshot = ingestError$G(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$$(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$$(luvio, config, options) {
    const resourceParams = createResourceParams$W(config);
    const request = createResourceRequest$11(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$I(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$Y(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$J(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$N(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$$, undefined, false);
}
function buildCachedSnapshotCachePolicy$M(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$32(luvio, config),
        node: adapterFragment$F(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$$(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getActionOverridesAdapterFactory = (luvio) => function UiApi__getActionOverrides(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$11(untrustedConfig, getActionOverrides_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$M, buildNetworkSnapshotCachePolicy$N);
};

function validate$1v(obj, path = 'FormulaOverridesInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldNames = obj.fieldNames;
        const path_fieldNames = path + '.fieldNames';
        if (!ArrayIsArray(obj_fieldNames)) {
            return new TypeError('Expected "array" but received "' + typeof obj_fieldNames + '" (at "' + path_fieldNames + '")');
        }
        for (let i = 0; i < obj_fieldNames.length; i++) {
            const obj_fieldNames_item = obj_fieldNames[i];
            const path_fieldNames_item = path_fieldNames + '[' + i + ']';
            if (typeof obj_fieldNames_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_fieldNames_item + '" (at "' + path_fieldNames_item + '")');
            }
        }
        const obj_flexipageName = obj.flexipageName;
        const path_flexipageName = path + '.flexipageName';
        if (typeof obj_flexipageName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_flexipageName + '" (at "' + path_flexipageName + '")');
        }
        const obj_jsFormula = obj.jsFormula;
        const path_jsFormula = path + '.jsFormula';
        if (typeof obj_jsFormula !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_jsFormula + '" (at "' + path_jsFormula + '")');
        }
        const obj_order = obj.order;
        const path_order = path + '.order';
        if (typeof obj_order !== 'number' || (typeof obj_order === 'number' && Math.floor(obj_order) !== obj_order)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_order + '" (at "' + path_order + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$w = 300000;
const VERSION$2p = "c57b66c259b23683db7b763e132e8633";
function validate$1u(obj, path = 'FlexipageFormulaActivationRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_activations = obj.activations;
        const path_activations = path + '.activations';
        if (typeof obj_activations !== 'object' || ArrayIsArray(obj_activations) || obj_activations === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_activations + '" (at "' + path_activations + '")');
        }
        const obj_activations_keys = ObjectKeys(obj_activations);
        for (let i = 0; i < obj_activations_keys.length; i++) {
            const key = obj_activations_keys[i];
            const obj_activations_prop = obj_activations[key];
            const path_activations_prop = path_activations + '["' + key + '"]';
            if (!ArrayIsArray(obj_activations_prop)) {
                return new TypeError('Expected "array" but received "' + typeof obj_activations_prop + '" (at "' + path_activations_prop + '")');
            }
            for (let i = 0; i < obj_activations_prop.length; i++) {
                const obj_activations_prop_item = obj_activations_prop[i];
                const path_activations_prop_item = path_activations_prop + '[' + i + ']';
                const referencepath_activations_prop_itemValidationError = validate$1v(obj_activations_prop_item, path_activations_prop_item);
                if (referencepath_activations_prop_itemValidationError !== null) {
                    let message = 'Object doesn\'t match FormulaOverridesInfoRepresentation (at "' + path_activations_prop_item + '")\n';
                    message += referencepath_activations_prop_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                    return new TypeError(message);
                }
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$G = 'FlexipageFormulaActivationRepresentation';
function normalize$z(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2Z = function FlexipageFormulaActivationRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2p,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'activations',
                kind: 'Object',
                opaque: true
            }
        ]
    };
};
function equals$R(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1Z = function FlexipageFormulaActivationRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1u(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$w;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$z, "UiApi", VERSION$2p, RepresentationType$G, equals$R);
    return createLink$1(key);
};
function getTypeCacheKeys$20(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$G,
        mergeable: false
    });
}

function select$2Y(luvio, params) {
    return select$2Z();
}
function keyBuilder$31(luvio, params) {
    return keyPrefix + '::FlexipageFormulaActivationRepresentation:(' + 'application:' + params.queryParams.application + ',' + 'objectApiNames:' + params.queryParams.objectApiNames + ',' + 'actionFeature:' + params.urlParams.actionFeature + ')';
}
function getResponseCacheKeys$X(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$20(storeKeyMap, luvio, response, () => keyBuilder$31(luvio, resourceParams));
}
function ingestSuccess$M(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$31(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1Z, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2Y(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$F(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$31(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$w,
        namespace: keyPrefix,
        version: VERSION$2p,
        representationName: RepresentationType$G
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$10(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/formula-activation/' + config.urlParams.actionFeature + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$Q = 'getFlexipageFormulaOverrides';
const getFlexipageFormulaOverrides_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionFeature', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('application', false, 1 /* QueryParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('objectApiNames', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getFlexipageFormulaOverrides_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$Q, getFlexipageFormulaOverrides_ConfigPropertyMetadata);
const createResourceParams$V = /*#__PURE__*/ createResourceParams$k(getFlexipageFormulaOverrides_ConfigPropertyMetadata);
function keyBuilder$30(luvio, config) {
    const resourceParams = createResourceParams$V(config);
    return keyBuilder$31(luvio, resourceParams);
}
function typeCheckConfig$Z(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getFlexipageFormulaOverrides_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$10(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getFlexipageFormulaOverrides_ConfigPropertyMetadata);
    const config = typeCheckConfig$Z(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$E(luvio, config) {
    createResourceParams$V(config);
    return select$2Y();
}
function onFetchResponseSuccess$H(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$M(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$_(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$I(luvio, config, resourceParams, response) {
    const snapshot = ingestError$F(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$_(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$_(luvio, config, options) {
    const resourceParams = createResourceParams$V(config);
    const request = createResourceRequest$10(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$H(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$X(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$I(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$M(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$_, undefined, false);
}
function buildCachedSnapshotCachePolicy$L(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$30(luvio, config),
        node: adapterFragment$E(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$_(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getFlexipageFormulaOverridesAdapterFactory = (luvio) => function UiApi__getFlexipageFormulaOverrides(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$10(untrustedConfig, getFlexipageFormulaOverrides_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$L, buildNetworkSnapshotCachePolicy$M);
};

const VERSION$2o = "7a5e66e0b88fe9d2233bb3158931bfa6";
function validate$1t(obj, path = 'QuickActionColorRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_color = obj.color;
        const path_color = path + '.color';
        if (typeof obj_color !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_color + '" (at "' + path_color + '")');
        }
        const obj_colorContext = obj.colorContext;
        const path_colorContext = path + '.colorContext';
        if (typeof obj_colorContext !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_colorContext + '" (at "' + path_colorContext + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_theme = obj.theme;
        const path_theme = path + '.theme';
        if (typeof obj_theme !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_theme + '" (at "' + path_theme + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2X = function QuickActionColorRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2o,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'color',
                kind: 'Scalar'
            },
            {
                name: 'colorContext',
                kind: 'Scalar'
            },
            {
                name: 'theme',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$Q(existing, incoming) {
    const existing_color = existing.color;
    const incoming_color = incoming.color;
    if (!(existing_color === incoming_color)) {
        return false;
    }
    const existing_colorContext = existing.colorContext;
    const incoming_colorContext = incoming.colorContext;
    if (!(existing_colorContext === incoming_colorContext)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_theme = existing.theme;
    const incoming_theme = incoming.theme;
    if (!(existing_theme === incoming_theme)) {
        return false;
    }
    return true;
}

const VERSION$2n = "5e970aa7dacfdd8e0c6d5daca575ec58";
function validate$1s(obj, path = 'CopilotQuickActionInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_inputName = obj.inputName;
        const path_inputName = path + '.inputName';
        if (typeof obj_inputName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_inputName + '" (at "' + path_inputName + '")');
        }
        const obj_inputValue = obj.inputValue;
        const path_inputValue = path + '.inputValue';
        if (typeof obj_inputValue !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_inputValue + '" (at "' + path_inputValue + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2W = function CopilotQuickActionInputRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2n,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'inputName',
                kind: 'Scalar'
            },
            {
                name: 'inputValue',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$P(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_inputName = existing.inputName;
    const incoming_inputName = incoming.inputName;
    if (!(existing_inputName === incoming_inputName)) {
        return false;
    }
    const existing_inputValue = existing.inputValue;
    const incoming_inputValue = incoming.inputValue;
    if (!(existing_inputValue === incoming_inputValue)) {
        return false;
    }
    return true;
}

const VERSION$2m = "53b489083ef57944d2a9d4a2aaf72ec3";
function validate$1r(obj, path = 'QuickActionIconRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_contentType = obj.contentType;
        const path_contentType = path + '.contentType';
        if (typeof obj_contentType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_contentType + '" (at "' + path_contentType + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_height = obj.height;
        const path_height = path + '.height';
        if (typeof obj_height !== 'number' || (typeof obj_height === 'number' && Math.floor(obj_height) !== obj_height)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_height + '" (at "' + path_height + '")');
        }
        const obj_theme = obj.theme;
        const path_theme = path + '.theme';
        if (typeof obj_theme !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_theme + '" (at "' + path_theme + '")');
        }
        const obj_url = obj.url;
        const path_url = path + '.url';
        if (typeof obj_url !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_url + '" (at "' + path_url + '")');
        }
        const obj_width = obj.width;
        const path_width = path + '.width';
        if (typeof obj_width !== 'number' || (typeof obj_width === 'number' && Math.floor(obj_width) !== obj_width)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_width + '" (at "' + path_width + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2V = function QuickActionIconRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2m,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'contentType',
                kind: 'Scalar'
            },
            {
                name: 'height',
                kind: 'Scalar'
            },
            {
                name: 'theme',
                kind: 'Scalar'
            },
            {
                name: 'url',
                kind: 'Scalar'
            },
            {
                name: 'width',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$O(existing, incoming) {
    const existing_height = existing.height;
    const incoming_height = incoming.height;
    if (!(existing_height === incoming_height)) {
        return false;
    }
    const existing_width = existing.width;
    const incoming_width = incoming.width;
    if (!(existing_width === incoming_width)) {
        return false;
    }
    const existing_contentType = existing.contentType;
    const incoming_contentType = incoming.contentType;
    if (!(existing_contentType === incoming_contentType)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_theme = existing.theme;
    const incoming_theme = incoming.theme;
    if (!(existing_theme === incoming_theme)) {
        return false;
    }
    const existing_url = existing.url;
    const incoming_url = incoming.url;
    if (!(existing_url === incoming_url)) {
        return false;
    }
    return true;
}

const VERSION$2l = "bab371abd4de5aea22d80f5c9843eb8a";
function validate$1q(obj, path = 'CopilotQuickActionParameterRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_parameterName = obj.parameterName;
        const path_parameterName = path + '.parameterName';
        if (typeof obj_parameterName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_parameterName + '" (at "' + path_parameterName + '")');
        }
        const obj_parameterValue = obj.parameterValue;
        const path_parameterValue = path + '.parameterValue';
        if (typeof obj_parameterValue !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_parameterValue + '" (at "' + path_parameterValue + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2U = function CopilotQuickActionParameterRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2l,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'parameterName',
                kind: 'Scalar'
            },
            {
                name: 'parameterValue',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$N(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_parameterName = existing.parameterName;
    const incoming_parameterName = incoming.parameterName;
    if (!(existing_parameterName === incoming_parameterName)) {
        return false;
    }
    const existing_parameterValue = existing.parameterValue;
    const incoming_parameterValue = incoming.parameterValue;
    if (!(existing_parameterValue === incoming_parameterValue)) {
        return false;
    }
    return true;
}

const TTL$v = 900000;
const VERSION$2k = "463cd270460d626d438908f8348e6dc8";
function validate$1p(obj, path = 'QuickActionInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_accessLevelRequired = obj.accessLevelRequired;
        const path_accessLevelRequired = path + '.accessLevelRequired';
        let obj_accessLevelRequired_union0 = null;
        const obj_accessLevelRequired_union0_error = (() => {
            if (typeof obj_accessLevelRequired !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_accessLevelRequired + '" (at "' + path_accessLevelRequired + '")');
            }
        })();
        if (obj_accessLevelRequired_union0_error != null) {
            obj_accessLevelRequired_union0 = obj_accessLevelRequired_union0_error.message;
        }
        let obj_accessLevelRequired_union1 = null;
        const obj_accessLevelRequired_union1_error = (() => {
            if (obj_accessLevelRequired !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_accessLevelRequired + '" (at "' + path_accessLevelRequired + '")');
            }
        })();
        if (obj_accessLevelRequired_union1_error != null) {
            obj_accessLevelRequired_union1 = obj_accessLevelRequired_union1_error.message;
        }
        if (obj_accessLevelRequired_union0 && obj_accessLevelRequired_union1) {
            let message = 'Object doesn\'t match union (at "' + path_accessLevelRequired + '")';
            message += '\n' + obj_accessLevelRequired_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_accessLevelRequired_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_actionApiName = obj.actionApiName;
        const path_actionApiName = path + '.actionApiName';
        if (typeof obj_actionApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_actionApiName + '" (at "' + path_actionApiName + '")');
        }
        const obj_colors = obj.colors;
        const path_colors = path + '.colors';
        if (!ArrayIsArray(obj_colors)) {
            return new TypeError('Expected "array" but received "' + typeof obj_colors + '" (at "' + path_colors + '")');
        }
        for (let i = 0; i < obj_colors.length; i++) {
            const obj_colors_item = obj_colors[i];
            const path_colors_item = path_colors + '[' + i + ']';
            const referencepath_colors_itemValidationError = validate$1t(obj_colors_item, path_colors_item);
            if (referencepath_colors_itemValidationError !== null) {
                let message = 'Object doesn\'t match QuickActionColorRepresentation (at "' + path_colors_item + '")\n';
                message += referencepath_colors_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_copilotActionDescription = obj.copilotActionDescription;
        const path_copilotActionDescription = path + '.copilotActionDescription';
        let obj_copilotActionDescription_union0 = null;
        const obj_copilotActionDescription_union0_error = (() => {
            if (typeof obj_copilotActionDescription !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_copilotActionDescription + '" (at "' + path_copilotActionDescription + '")');
            }
        })();
        if (obj_copilotActionDescription_union0_error != null) {
            obj_copilotActionDescription_union0 = obj_copilotActionDescription_union0_error.message;
        }
        let obj_copilotActionDescription_union1 = null;
        const obj_copilotActionDescription_union1_error = (() => {
            if (obj_copilotActionDescription !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_copilotActionDescription + '" (at "' + path_copilotActionDescription + '")');
            }
        })();
        if (obj_copilotActionDescription_union1_error != null) {
            obj_copilotActionDescription_union1 = obj_copilotActionDescription_union1_error.message;
        }
        if (obj_copilotActionDescription_union0 && obj_copilotActionDescription_union1) {
            let message = 'Object doesn\'t match union (at "' + path_copilotActionDescription + '")';
            message += '\n' + obj_copilotActionDescription_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_copilotActionDescription_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_copilotInputs = obj.copilotInputs;
        const path_copilotInputs = path + '.copilotInputs';
        if (!ArrayIsArray(obj_copilotInputs)) {
            return new TypeError('Expected "array" but received "' + typeof obj_copilotInputs + '" (at "' + path_copilotInputs + '")');
        }
        for (let i = 0; i < obj_copilotInputs.length; i++) {
            const obj_copilotInputs_item = obj_copilotInputs[i];
            const path_copilotInputs_item = path_copilotInputs + '[' + i + ']';
            const referencepath_copilotInputs_itemValidationError = validate$1s(obj_copilotInputs_item, path_copilotInputs_item);
            if (referencepath_copilotInputs_itemValidationError !== null) {
                let message = 'Object doesn\'t match CopilotQuickActionInputRepresentation (at "' + path_copilotInputs_item + '")\n';
                message += referencepath_copilotInputs_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_copilotIntent = obj.copilotIntent;
        const path_copilotIntent = path + '.copilotIntent';
        let obj_copilotIntent_union0 = null;
        const obj_copilotIntent_union0_error = (() => {
            if (typeof obj_copilotIntent !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_copilotIntent + '" (at "' + path_copilotIntent + '")');
            }
        })();
        if (obj_copilotIntent_union0_error != null) {
            obj_copilotIntent_union0 = obj_copilotIntent_union0_error.message;
        }
        let obj_copilotIntent_union1 = null;
        const obj_copilotIntent_union1_error = (() => {
            if (obj_copilotIntent !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_copilotIntent + '" (at "' + path_copilotIntent + '")');
            }
        })();
        if (obj_copilotIntent_union1_error != null) {
            obj_copilotIntent_union1 = obj_copilotIntent_union1_error.message;
        }
        if (obj_copilotIntent_union0 && obj_copilotIntent_union1) {
            let message = 'Object doesn\'t match union (at "' + path_copilotIntent + '")';
            message += '\n' + obj_copilotIntent_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_copilotIntent_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_copilotPlanTemplateId = obj.copilotPlanTemplateId;
        const path_copilotPlanTemplateId = path + '.copilotPlanTemplateId';
        let obj_copilotPlanTemplateId_union0 = null;
        const obj_copilotPlanTemplateId_union0_error = (() => {
            if (typeof obj_copilotPlanTemplateId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_copilotPlanTemplateId + '" (at "' + path_copilotPlanTemplateId + '")');
            }
        })();
        if (obj_copilotPlanTemplateId_union0_error != null) {
            obj_copilotPlanTemplateId_union0 = obj_copilotPlanTemplateId_union0_error.message;
        }
        let obj_copilotPlanTemplateId_union1 = null;
        const obj_copilotPlanTemplateId_union1_error = (() => {
            if (obj_copilotPlanTemplateId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_copilotPlanTemplateId + '" (at "' + path_copilotPlanTemplateId + '")');
            }
        })();
        if (obj_copilotPlanTemplateId_union1_error != null) {
            obj_copilotPlanTemplateId_union1 = obj_copilotPlanTemplateId_union1_error.message;
        }
        if (obj_copilotPlanTemplateId_union0 && obj_copilotPlanTemplateId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_copilotPlanTemplateId + '")';
            message += '\n' + obj_copilotPlanTemplateId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_copilotPlanTemplateId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_copilotUserUtterance = obj.copilotUserUtterance;
        const path_copilotUserUtterance = path + '.copilotUserUtterance';
        let obj_copilotUserUtterance_union0 = null;
        const obj_copilotUserUtterance_union0_error = (() => {
            if (typeof obj_copilotUserUtterance !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_copilotUserUtterance + '" (at "' + path_copilotUserUtterance + '")');
            }
        })();
        if (obj_copilotUserUtterance_union0_error != null) {
            obj_copilotUserUtterance_union0 = obj_copilotUserUtterance_union0_error.message;
        }
        let obj_copilotUserUtterance_union1 = null;
        const obj_copilotUserUtterance_union1_error = (() => {
            if (obj_copilotUserUtterance !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_copilotUserUtterance + '" (at "' + path_copilotUserUtterance + '")');
            }
        })();
        if (obj_copilotUserUtterance_union1_error != null) {
            obj_copilotUserUtterance_union1 = obj_copilotUserUtterance_union1_error.message;
        }
        if (obj_copilotUserUtterance_union0 && obj_copilotUserUtterance_union1) {
            let message = 'Object doesn\'t match union (at "' + path_copilotUserUtterance + '")';
            message += '\n' + obj_copilotUserUtterance_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_copilotUserUtterance_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_flowDevName = obj.flowDevName;
        const path_flowDevName = path + '.flowDevName';
        let obj_flowDevName_union0 = null;
        const obj_flowDevName_union0_error = (() => {
            if (typeof obj_flowDevName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_flowDevName + '" (at "' + path_flowDevName + '")');
            }
        })();
        if (obj_flowDevName_union0_error != null) {
            obj_flowDevName_union0 = obj_flowDevName_union0_error.message;
        }
        let obj_flowDevName_union1 = null;
        const obj_flowDevName_union1_error = (() => {
            if (obj_flowDevName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_flowDevName + '" (at "' + path_flowDevName + '")');
            }
        })();
        if (obj_flowDevName_union1_error != null) {
            obj_flowDevName_union1 = obj_flowDevName_union1_error.message;
        }
        if (obj_flowDevName_union0 && obj_flowDevName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_flowDevName + '")';
            message += '\n' + obj_flowDevName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_flowDevName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_flowRecordIdVar = obj.flowRecordIdVar;
        const path_flowRecordIdVar = path + '.flowRecordIdVar';
        let obj_flowRecordIdVar_union0 = null;
        const obj_flowRecordIdVar_union0_error = (() => {
            if (typeof obj_flowRecordIdVar !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_flowRecordIdVar + '" (at "' + path_flowRecordIdVar + '")');
            }
        })();
        if (obj_flowRecordIdVar_union0_error != null) {
            obj_flowRecordIdVar_union0 = obj_flowRecordIdVar_union0_error.message;
        }
        let obj_flowRecordIdVar_union1 = null;
        const obj_flowRecordIdVar_union1_error = (() => {
            if (obj_flowRecordIdVar !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_flowRecordIdVar + '" (at "' + path_flowRecordIdVar + '")');
            }
        })();
        if (obj_flowRecordIdVar_union1_error != null) {
            obj_flowRecordIdVar_union1 = obj_flowRecordIdVar_union1_error.message;
        }
        if (obj_flowRecordIdVar_union0 && obj_flowRecordIdVar_union1) {
            let message = 'Object doesn\'t match union (at "' + path_flowRecordIdVar + '")';
            message += '\n' + obj_flowRecordIdVar_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_flowRecordIdVar_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_iconName = obj.iconName;
        const path_iconName = path + '.iconName';
        let obj_iconName_union0 = null;
        const obj_iconName_union0_error = (() => {
            if (typeof obj_iconName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_iconName + '" (at "' + path_iconName + '")');
            }
        })();
        if (obj_iconName_union0_error != null) {
            obj_iconName_union0 = obj_iconName_union0_error.message;
        }
        let obj_iconName_union1 = null;
        const obj_iconName_union1_error = (() => {
            if (obj_iconName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_iconName + '" (at "' + path_iconName + '")');
            }
        })();
        if (obj_iconName_union1_error != null) {
            obj_iconName_union1 = obj_iconName_union1_error.message;
        }
        if (obj_iconName_union0 && obj_iconName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_iconName + '")';
            message += '\n' + obj_iconName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_iconName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_icons = obj.icons;
        const path_icons = path + '.icons';
        if (!ArrayIsArray(obj_icons)) {
            return new TypeError('Expected "array" but received "' + typeof obj_icons + '" (at "' + path_icons + '")');
        }
        for (let i = 0; i < obj_icons.length; i++) {
            const obj_icons_item = obj_icons[i];
            const path_icons_item = path_icons + '[' + i + ']';
            const referencepath_icons_itemValidationError = validate$1r(obj_icons_item, path_icons_item);
            if (referencepath_icons_itemValidationError !== null) {
                let message = 'Object doesn\'t match QuickActionIconRepresentation (at "' + path_icons_item + '")\n';
                message += referencepath_icons_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_lightningComponentBundleName = obj.lightningComponentBundleName;
        const path_lightningComponentBundleName = path + '.lightningComponentBundleName';
        let obj_lightningComponentBundleName_union0 = null;
        const obj_lightningComponentBundleName_union0_error = (() => {
            if (typeof obj_lightningComponentBundleName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lightningComponentBundleName + '" (at "' + path_lightningComponentBundleName + '")');
            }
        })();
        if (obj_lightningComponentBundleName_union0_error != null) {
            obj_lightningComponentBundleName_union0 = obj_lightningComponentBundleName_union0_error.message;
        }
        let obj_lightningComponentBundleName_union1 = null;
        const obj_lightningComponentBundleName_union1_error = (() => {
            if (obj_lightningComponentBundleName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lightningComponentBundleName + '" (at "' + path_lightningComponentBundleName + '")');
            }
        })();
        if (obj_lightningComponentBundleName_union1_error != null) {
            obj_lightningComponentBundleName_union1 = obj_lightningComponentBundleName_union1_error.message;
        }
        if (obj_lightningComponentBundleName_union0 && obj_lightningComponentBundleName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lightningComponentBundleName + '")';
            message += '\n' + obj_lightningComponentBundleName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lightningComponentBundleName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_lightningComponentQualifiedName = obj.lightningComponentQualifiedName;
        const path_lightningComponentQualifiedName = path + '.lightningComponentQualifiedName';
        let obj_lightningComponentQualifiedName_union0 = null;
        const obj_lightningComponentQualifiedName_union0_error = (() => {
            if (typeof obj_lightningComponentQualifiedName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lightningComponentQualifiedName + '" (at "' + path_lightningComponentQualifiedName + '")');
            }
        })();
        if (obj_lightningComponentQualifiedName_union0_error != null) {
            obj_lightningComponentQualifiedName_union0 = obj_lightningComponentQualifiedName_union0_error.message;
        }
        let obj_lightningComponentQualifiedName_union1 = null;
        const obj_lightningComponentQualifiedName_union1_error = (() => {
            if (obj_lightningComponentQualifiedName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lightningComponentQualifiedName + '" (at "' + path_lightningComponentQualifiedName + '")');
            }
        })();
        if (obj_lightningComponentQualifiedName_union1_error != null) {
            obj_lightningComponentQualifiedName_union1 = obj_lightningComponentQualifiedName_union1_error.message;
        }
        if (obj_lightningComponentQualifiedName_union0 && obj_lightningComponentQualifiedName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lightningComponentQualifiedName + '")';
            message += '\n' + obj_lightningComponentQualifiedName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lightningComponentQualifiedName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_lightningWebComponentBundleName = obj.lightningWebComponentBundleName;
        const path_lightningWebComponentBundleName = path + '.lightningWebComponentBundleName';
        let obj_lightningWebComponentBundleName_union0 = null;
        const obj_lightningWebComponentBundleName_union0_error = (() => {
            if (typeof obj_lightningWebComponentBundleName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lightningWebComponentBundleName + '" (at "' + path_lightningWebComponentBundleName + '")');
            }
        })();
        if (obj_lightningWebComponentBundleName_union0_error != null) {
            obj_lightningWebComponentBundleName_union0 = obj_lightningWebComponentBundleName_union0_error.message;
        }
        let obj_lightningWebComponentBundleName_union1 = null;
        const obj_lightningWebComponentBundleName_union1_error = (() => {
            if (obj_lightningWebComponentBundleName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lightningWebComponentBundleName + '" (at "' + path_lightningWebComponentBundleName + '")');
            }
        })();
        if (obj_lightningWebComponentBundleName_union1_error != null) {
            obj_lightningWebComponentBundleName_union1 = obj_lightningWebComponentBundleName_union1_error.message;
        }
        if (obj_lightningWebComponentBundleName_union0 && obj_lightningWebComponentBundleName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lightningWebComponentBundleName + '")';
            message += '\n' + obj_lightningWebComponentBundleName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lightningWebComponentBundleName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_lightningWebComponentQualifiedName = obj.lightningWebComponentQualifiedName;
        const path_lightningWebComponentQualifiedName = path + '.lightningWebComponentQualifiedName';
        let obj_lightningWebComponentQualifiedName_union0 = null;
        const obj_lightningWebComponentQualifiedName_union0_error = (() => {
            if (typeof obj_lightningWebComponentQualifiedName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lightningWebComponentQualifiedName + '" (at "' + path_lightningWebComponentQualifiedName + '")');
            }
        })();
        if (obj_lightningWebComponentQualifiedName_union0_error != null) {
            obj_lightningWebComponentQualifiedName_union0 = obj_lightningWebComponentQualifiedName_union0_error.message;
        }
        let obj_lightningWebComponentQualifiedName_union1 = null;
        const obj_lightningWebComponentQualifiedName_union1_error = (() => {
            if (obj_lightningWebComponentQualifiedName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lightningWebComponentQualifiedName + '" (at "' + path_lightningWebComponentQualifiedName + '")');
            }
        })();
        if (obj_lightningWebComponentQualifiedName_union1_error != null) {
            obj_lightningWebComponentQualifiedName_union1 = obj_lightningWebComponentQualifiedName_union1_error.message;
        }
        if (obj_lightningWebComponentQualifiedName_union0 && obj_lightningWebComponentQualifiedName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lightningWebComponentQualifiedName + '")';
            message += '\n' + obj_lightningWebComponentQualifiedName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lightningWebComponentQualifiedName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_parameters = obj.parameters;
        const path_parameters = path + '.parameters';
        if (!ArrayIsArray(obj_parameters)) {
            return new TypeError('Expected "array" but received "' + typeof obj_parameters + '" (at "' + path_parameters + '")');
        }
        for (let i = 0; i < obj_parameters.length; i++) {
            const obj_parameters_item = obj_parameters[i];
            const path_parameters_item = path_parameters + '[' + i + ']';
            const referencepath_parameters_itemValidationError = validate$1q(obj_parameters_item, path_parameters_item);
            if (referencepath_parameters_itemValidationError !== null) {
                let message = 'Object doesn\'t match CopilotQuickActionParameterRepresentation (at "' + path_parameters_item + '")\n';
                message += referencepath_parameters_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_targetParentField = obj.targetParentField;
        const path_targetParentField = path + '.targetParentField';
        let obj_targetParentField_union0 = null;
        const obj_targetParentField_union0_error = (() => {
            if (typeof obj_targetParentField !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_targetParentField + '" (at "' + path_targetParentField + '")');
            }
        })();
        if (obj_targetParentField_union0_error != null) {
            obj_targetParentField_union0 = obj_targetParentField_union0_error.message;
        }
        let obj_targetParentField_union1 = null;
        const obj_targetParentField_union1_error = (() => {
            if (obj_targetParentField !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_targetParentField + '" (at "' + path_targetParentField + '")');
            }
        })();
        if (obj_targetParentField_union1_error != null) {
            obj_targetParentField_union1 = obj_targetParentField_union1_error.message;
        }
        if (obj_targetParentField_union0 && obj_targetParentField_union1) {
            let message = 'Object doesn\'t match union (at "' + path_targetParentField + '")';
            message += '\n' + obj_targetParentField_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_targetParentField_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_targetRecordTypeId = obj.targetRecordTypeId;
        const path_targetRecordTypeId = path + '.targetRecordTypeId';
        let obj_targetRecordTypeId_union0 = null;
        const obj_targetRecordTypeId_union0_error = (() => {
            if (typeof obj_targetRecordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_targetRecordTypeId + '" (at "' + path_targetRecordTypeId + '")');
            }
        })();
        if (obj_targetRecordTypeId_union0_error != null) {
            obj_targetRecordTypeId_union0 = obj_targetRecordTypeId_union0_error.message;
        }
        let obj_targetRecordTypeId_union1 = null;
        const obj_targetRecordTypeId_union1_error = (() => {
            if (obj_targetRecordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_targetRecordTypeId + '" (at "' + path_targetRecordTypeId + '")');
            }
        })();
        if (obj_targetRecordTypeId_union1_error != null) {
            obj_targetRecordTypeId_union1 = obj_targetRecordTypeId_union1_error.message;
        }
        if (obj_targetRecordTypeId_union0 && obj_targetRecordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_targetRecordTypeId + '")';
            message += '\n' + obj_targetRecordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_targetRecordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_targetSobjectType = obj.targetSobjectType;
        const path_targetSobjectType = path + '.targetSobjectType';
        let obj_targetSobjectType_union0 = null;
        const obj_targetSobjectType_union0_error = (() => {
            if (typeof obj_targetSobjectType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_targetSobjectType + '" (at "' + path_targetSobjectType + '")');
            }
        })();
        if (obj_targetSobjectType_union0_error != null) {
            obj_targetSobjectType_union0 = obj_targetSobjectType_union0_error.message;
        }
        let obj_targetSobjectType_union1 = null;
        const obj_targetSobjectType_union1_error = (() => {
            if (obj_targetSobjectType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_targetSobjectType + '" (at "' + path_targetSobjectType + '")');
            }
        })();
        if (obj_targetSobjectType_union1_error != null) {
            obj_targetSobjectType_union1 = obj_targetSobjectType_union1_error.message;
        }
        if (obj_targetSobjectType_union0 && obj_targetSobjectType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_targetSobjectType + '")';
            message += '\n' + obj_targetSobjectType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_targetSobjectType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$F = 'QuickActionInfoRepresentation';
function keyBuilder$2$(luvio, config) {
    return keyPrefix + '::' + RepresentationType$F + ':' + config.actionApiName;
}
function keyBuilderFromType$n(luvio, object) {
    const keyParams = {
        actionApiName: object.actionApiName
    };
    return keyBuilder$2$(luvio, keyParams);
}
function normalize$y(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2T = function QuickActionInfoRepresentationSelect() {
    const { selections: QuickActionColorRepresentation__selections, opaque: QuickActionColorRepresentation__opaque, } = select$2X();
    const { selections: CopilotQuickActionInputRepresentation__selections, opaque: CopilotQuickActionInputRepresentation__opaque, } = select$2W();
    const { selections: QuickActionIconRepresentation__selections, opaque: QuickActionIconRepresentation__opaque, } = select$2V();
    const { selections: CopilotQuickActionParameterRepresentation__selections, opaque: CopilotQuickActionParameterRepresentation__opaque, } = select$2U();
    return {
        kind: 'Fragment',
        version: VERSION$2k,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'accessLevelRequired',
                kind: 'Scalar'
            },
            {
                name: 'actionApiName',
                kind: 'Scalar'
            },
            {
                name: 'colors',
                kind: 'Object',
                plural: true,
                selections: QuickActionColorRepresentation__selections
            },
            {
                name: 'copilotActionDescription',
                kind: 'Scalar'
            },
            {
                name: 'copilotInputs',
                kind: 'Object',
                plural: true,
                selections: CopilotQuickActionInputRepresentation__selections
            },
            {
                name: 'copilotIntent',
                kind: 'Scalar'
            },
            {
                name: 'copilotPlanTemplateId',
                kind: 'Scalar'
            },
            {
                name: 'copilotUserUtterance',
                kind: 'Scalar'
            },
            {
                name: 'flowDevName',
                kind: 'Scalar'
            },
            {
                name: 'flowRecordIdVar',
                kind: 'Scalar'
            },
            {
                name: 'iconName',
                kind: 'Scalar'
            },
            {
                name: 'icons',
                kind: 'Object',
                plural: true,
                selections: QuickActionIconRepresentation__selections
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'lightningComponentBundleName',
                kind: 'Scalar'
            },
            {
                name: 'lightningComponentQualifiedName',
                kind: 'Scalar'
            },
            {
                name: 'lightningWebComponentBundleName',
                kind: 'Scalar'
            },
            {
                name: 'lightningWebComponentQualifiedName',
                kind: 'Scalar'
            },
            {
                name: 'parameters',
                kind: 'Object',
                plural: true,
                selections: CopilotQuickActionParameterRepresentation__selections
            },
            {
                name: 'targetParentField',
                kind: 'Scalar'
            },
            {
                name: 'targetRecordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'targetSobjectType',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$M(existing, incoming) {
    const existing_actionApiName = existing.actionApiName;
    const incoming_actionApiName = incoming.actionApiName;
    if (!(existing_actionApiName === incoming_actionApiName)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_type = existing.type;
    const incoming_type = incoming.type;
    if (!(existing_type === incoming_type)) {
        return false;
    }
    const existing_accessLevelRequired = existing.accessLevelRequired;
    const incoming_accessLevelRequired = incoming.accessLevelRequired;
    if (!(existing_accessLevelRequired === incoming_accessLevelRequired)) {
        return false;
    }
    const existing_colors = existing.colors;
    const incoming_colors = incoming.colors;
    const equals_colors_items = equalsArray(existing_colors, incoming_colors, (existing_colors_item, incoming_colors_item) => {
        if (!(equals$Q(existing_colors_item, incoming_colors_item))) {
            return false;
        }
    });
    if (equals_colors_items === false) {
        return false;
    }
    const existing_copilotActionDescription = existing.copilotActionDescription;
    const incoming_copilotActionDescription = incoming.copilotActionDescription;
    if (!(existing_copilotActionDescription === incoming_copilotActionDescription)) {
        return false;
    }
    const existing_copilotInputs = existing.copilotInputs;
    const incoming_copilotInputs = incoming.copilotInputs;
    const equals_copilotInputs_items = equalsArray(existing_copilotInputs, incoming_copilotInputs, (existing_copilotInputs_item, incoming_copilotInputs_item) => {
        if (!(equals$P(existing_copilotInputs_item, incoming_copilotInputs_item))) {
            return false;
        }
    });
    if (equals_copilotInputs_items === false) {
        return false;
    }
    const existing_copilotIntent = existing.copilotIntent;
    const incoming_copilotIntent = incoming.copilotIntent;
    if (!(existing_copilotIntent === incoming_copilotIntent)) {
        return false;
    }
    const existing_copilotPlanTemplateId = existing.copilotPlanTemplateId;
    const incoming_copilotPlanTemplateId = incoming.copilotPlanTemplateId;
    if (!(existing_copilotPlanTemplateId === incoming_copilotPlanTemplateId)) {
        return false;
    }
    const existing_copilotUserUtterance = existing.copilotUserUtterance;
    const incoming_copilotUserUtterance = incoming.copilotUserUtterance;
    if (!(existing_copilotUserUtterance === incoming_copilotUserUtterance)) {
        return false;
    }
    const existing_flowDevName = existing.flowDevName;
    const incoming_flowDevName = incoming.flowDevName;
    if (!(existing_flowDevName === incoming_flowDevName)) {
        return false;
    }
    const existing_flowRecordIdVar = existing.flowRecordIdVar;
    const incoming_flowRecordIdVar = incoming.flowRecordIdVar;
    if (!(existing_flowRecordIdVar === incoming_flowRecordIdVar)) {
        return false;
    }
    const existing_iconName = existing.iconName;
    const incoming_iconName = incoming.iconName;
    if (!(existing_iconName === incoming_iconName)) {
        return false;
    }
    const existing_icons = existing.icons;
    const incoming_icons = incoming.icons;
    const equals_icons_items = equalsArray(existing_icons, incoming_icons, (existing_icons_item, incoming_icons_item) => {
        if (!(equals$O(existing_icons_item, incoming_icons_item))) {
            return false;
        }
    });
    if (equals_icons_items === false) {
        return false;
    }
    const existing_lightningComponentBundleName = existing.lightningComponentBundleName;
    const incoming_lightningComponentBundleName = incoming.lightningComponentBundleName;
    if (!(existing_lightningComponentBundleName === incoming_lightningComponentBundleName)) {
        return false;
    }
    const existing_lightningComponentQualifiedName = existing.lightningComponentQualifiedName;
    const incoming_lightningComponentQualifiedName = incoming.lightningComponentQualifiedName;
    if (!(existing_lightningComponentQualifiedName === incoming_lightningComponentQualifiedName)) {
        return false;
    }
    const existing_lightningWebComponentBundleName = existing.lightningWebComponentBundleName;
    const incoming_lightningWebComponentBundleName = incoming.lightningWebComponentBundleName;
    if (!(existing_lightningWebComponentBundleName === incoming_lightningWebComponentBundleName)) {
        return false;
    }
    const existing_lightningWebComponentQualifiedName = existing.lightningWebComponentQualifiedName;
    const incoming_lightningWebComponentQualifiedName = incoming.lightningWebComponentQualifiedName;
    if (!(existing_lightningWebComponentQualifiedName === incoming_lightningWebComponentQualifiedName)) {
        return false;
    }
    const existing_parameters = existing.parameters;
    const incoming_parameters = incoming.parameters;
    const equals_parameters_items = equalsArray(existing_parameters, incoming_parameters, (existing_parameters_item, incoming_parameters_item) => {
        if (!(equals$N(existing_parameters_item, incoming_parameters_item))) {
            return false;
        }
    });
    if (equals_parameters_items === false) {
        return false;
    }
    const existing_targetParentField = existing.targetParentField;
    const incoming_targetParentField = incoming.targetParentField;
    if (!(existing_targetParentField === incoming_targetParentField)) {
        return false;
    }
    const existing_targetRecordTypeId = existing.targetRecordTypeId;
    const incoming_targetRecordTypeId = incoming.targetRecordTypeId;
    if (!(existing_targetRecordTypeId === incoming_targetRecordTypeId)) {
        return false;
    }
    const existing_targetSobjectType = existing.targetSobjectType;
    const incoming_targetSobjectType = incoming.targetSobjectType;
    if (!(existing_targetSobjectType === incoming_targetSobjectType)) {
        return false;
    }
    return true;
}
const ingest$1Y = function QuickActionInfoRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1p(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$n(luvio, input);
    const ttlToUse = TTL$v;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$y, "UiApi", VERSION$2k, RepresentationType$F, equals$M);
    return createLink$1(key);
};
function getTypeCacheKeys$1$(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$n(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$F,
        mergeable: false
    });
}

function select$2S(luvio, params) {
    return select$2T();
}
function keyBuilder$2_(luvio, params) {
    return keyBuilder$2$(luvio, {
        actionApiName: params.urlParams.actionApiName
    });
}
function getResponseCacheKeys$W(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1$(storeKeyMap, luvio, response);
}
function ingestSuccess$L(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2_(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1Y, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2S(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$E(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2_(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$v,
        namespace: keyPrefix,
        version: VERSION$2k,
        representationName: RepresentationType$F
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$$(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/quick-action-info/' + config.urlParams.actionApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$P = 'getQuickActionInfo';
const getQuickActionInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('parentRecordId', false, 1 /* QueryParameter */, 0 /* String */, false, getRecordId18),
];
const getQuickActionInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$P, getQuickActionInfo_ConfigPropertyMetadata);
const createResourceParams$U = /*#__PURE__*/ createResourceParams$k(getQuickActionInfo_ConfigPropertyMetadata);
function keyBuilder$2Z(luvio, config) {
    const resourceParams = createResourceParams$U(config);
    return keyBuilder$2_(luvio, resourceParams);
}
function typeCheckConfig$Y(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getQuickActionInfo_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$$(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getQuickActionInfo_ConfigPropertyMetadata);
    const config = typeCheckConfig$Y(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$D(luvio, config) {
    createResourceParams$U(config);
    return select$2S();
}
function onFetchResponseSuccess$G(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$L(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$Z(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$H(luvio, config, resourceParams, response) {
    const snapshot = ingestError$E(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$Z(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$Z(luvio, config, options) {
    const resourceParams = createResourceParams$U(config);
    const request = createResourceRequest$$(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$G(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$W(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$H(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$L(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$Z, undefined, false);
}
function buildCachedSnapshotCachePolicy$K(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2Z(luvio, config),
        node: adapterFragment$D(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$Z(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getQuickActionInfoAdapterFactory = (luvio) => function UiApi__getQuickActionInfo(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$$(untrustedConfig, getQuickActionInfo_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$K, buildNetworkSnapshotCachePolicy$L);
};

const QUICK_ACTION_DEFAULTS_STORE_METADATA_PARAMS = {
    ttl: TTL$z,
    namespace: keyPrefix,
    representationName: RepresentationType$M,
    version: VERSION$2v,
};
function merge$1(existing, incoming) {
    if (existing === undefined) {
        return incoming;
    }
    // Merge QuickActionDefaultsRepresentation field values together
    return {
        ...incoming,
        fields: {
            ...existing.fields,
            ...incoming.fields,
        },
    };
}
const dynamicIngest$4 = (ingestParams) => {
    return function QuickActionDefaultsRepresentationIngest(input, path, luvio, store, timestamp) {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$1B(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        const key = keyBuilderFromType$r(luvio, input);
        const existingRecord = store.readEntry(key);
        let incomingRecord = dynamicNormalize$4(ingestParams)(input, store.readEntry(key), {
            fullPath: key,
            parent: path.parent,
            propertyName: path.propertyName,
        }, luvio, store, timestamp);
        incomingRecord = merge$1(existingRecord, incomingRecord);
        if (existingRecord === undefined || equals$X(existingRecord, incomingRecord) === false) {
            luvio.storePublish(key, incomingRecord);
        }
        luvio.publishStoreMetadata(key, {
            ...QUICK_ACTION_DEFAULTS_STORE_METADATA_PARAMS,
            ingestionTimestamp: timestamp,
        });
        return createLink$1(key);
    };
};

function createFieldsIngest$4(params) {
    const { fields, optionalFields, trackedFields, recordConflictMap } = params;
    const ingest = dynamicIngest$4({
        fields: createFieldsIngestion(fields, optionalFields, recordConflictMap)
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$7(fields) {
    return dynamicSelect$5({
        fields: createPathSelection('fields', fields)
    });
}

function createFieldsIngestSuccess$2(params) {
    const { trackedFields } = params;
    const recordConflictMap = {
        conflicts: {},
        serverRequestCount: 1,
    };
    const ingest = createFieldsIngest$4({
        ...params,
        recordConflictMap,
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        resolveConflict(luvio, recordConflictMap);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$6(luvio, params) {
    const optionalFields = params.queryParams.optionalFields || [];
    const fields = [];
    const trie = convertRecordFieldsArrayToTrie(fields, optionalFields);
    return selectFields$7(trie);
}

function keyBuilder$2Y(luvio, params) {
    return keyBuilder$3f(luvio, {
        actionApiName: params.urlParams.actionApiName
    });
}
function getResponseCacheKeys$V(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$26(storeKeyMap, luvio, response);
}
function ingestError$D(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2Y(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$z,
        namespace: keyPrefix,
        version: VERSION$2v,
        representationName: RepresentationType$M
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$_(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/record-defaults/' + config.urlParams.actionApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

function adapterFragment$C(luvio, config) {
    const resourceParams = createResourceParams$T(config);
    return selectFields$6(luvio, resourceParams);
}
function buildNetworkSnapshot$Y(luvio, config, options) {
    const resourceParams = createResourceParams$T(config);
    const request = createResourceRequest$_(resourceParams);
    const key = keyBuilder$2X(luvio, config);
    const trackedFieldsConfig = {
        maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMiss(),
        onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly()
    };
    const optionalFieldsTrie = convertFieldsToTrie(getTrackedFields(key, luvio.getNode(key), trackedFieldsConfig, config.optionalFields));
    const fieldsTrie = BLANK_RECORD_FIELDS_TRIE;
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const ingest = createFieldsIngestSuccess$2({
                fields: fieldsTrie,
                optionalFields: optionalFieldsTrie,
                trackedFields: optionalFieldsTrie,
            });
            luvio.storeIngest(key, ingest, response.body);
            const snapshot = buildCachedSnapshot$6(luvio, config);
            if (process.env.NODE_ENV !== 'production') {
                if (snapshot.state !== 'Fulfilled') {
                    throw new Error('Invalid network response. Expected network response to result in Fulfilled snapshot');
                }
            }
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$V(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => {
            return onFetchResponseError$G(luvio, config, resourceParams, response);
        });
    });
}

const adapterName$O = 'getQuickActionDefaults';
const getQuickActionDefaults_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('parentRecordId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getQuickActionDefaults_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$O, getQuickActionDefaults_ConfigPropertyMetadata);
const createResourceParams$T = /*#__PURE__*/ createResourceParams$k(getQuickActionDefaults_ConfigPropertyMetadata);
function keyBuilder$2X(luvio, config) {
    const resourceParams = createResourceParams$T(config);
    return keyBuilder$2Y(luvio, resourceParams);
}
function typeCheckConfig$X(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getQuickActionDefaults_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$_(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getQuickActionDefaults_ConfigPropertyMetadata);
    const config = typeCheckConfig$X(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildCachedSnapshot$6(luvio, config) {
    const selector = {
        recordId: keyBuilder$2X(luvio, config),
        node: adapterFragment$C(luvio, config),
        variables: {},
    };
    return luvio.storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$Y(luvio, config, snapshotRefreshOptions)
    });
}
function onFetchResponseError$G(luvio, config, resourceParams, response) {
    const snapshot = ingestError$D(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$Y(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshotCachePolicy$K(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$Y, undefined, false);
}
function buildCachedSnapshotCachePolicy$J(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2X(luvio, config),
        node: adapterFragment$C(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$Y(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getQuickActionDefaultsAdapterFactory = (luvio) => function UiApi__getQuickActionDefaults(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$_(untrustedConfig, getQuickActionDefaults_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$J, buildNetworkSnapshotCachePolicy$K);
};

function select$2R(luvio, params) {
    return select$35();
}
function keyBuilder$2W(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'actionTypes:' + params.queryParams.actionTypes + ',' + 'apiNames:' + params.queryParams.apiNames + ',' + 'formFactor:' + params.queryParams.formFactor + ',' + 'retrievalMode:' + params.queryParams.retrievalMode + ',' + 'sections:' + params.queryParams.sections + ',' + 'recordIds:' + params.urlParams.recordIds + ')';
}
function getResponseCacheKeys$U(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$2W(luvio, resourceParams));
}
function ingestSuccess$K(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2W(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2R(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$C(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2W(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$Z(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/record/' + config.urlParams.recordIds + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$N = 'getRecordActions';
const oneOfConfigPropertiesIdentifier = [
    'sections',
    'apiNames'
];
const getRecordActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('actionTypes', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('apiNames', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArrayAllowEmpty),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('retrievalMode', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sections', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getRecordActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$N, getRecordActions_ConfigPropertyMetadata);
const createResourceParams$S = /*#__PURE__*/ createResourceParams$k(getRecordActions_ConfigPropertyMetadata);
function keyBuilder$2V(luvio, config) {
    const resourceParams = createResourceParams$S(config);
    return keyBuilder$2W(luvio, resourceParams);
}
function typeCheckConfig$W(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$Z(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames, oneOfConfigPropertiesIdentifier);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$W(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    if (config.sections === undefined &&
        config.apiNames === undefined) {
        return null;
    }
    return config;
}
function adapterFragment$B(luvio, config) {
    createResourceParams$S(config);
    return select$2R();
}
function onFetchResponseSuccess$F(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$K(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$X(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$F(luvio, config, resourceParams, response) {
    const snapshot = ingestError$C(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$X(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$X(luvio, config, options) {
    const resourceParams = createResourceParams$S(config);
    const request = createResourceRequest$Z(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$F(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$U(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$F(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$J(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$X, undefined, false);
}
function buildCachedSnapshotCachePolicy$I(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2V(luvio, config),
        node: adapterFragment$B(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$X(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRecordActionsAdapterFactory = (luvio) => function UiApi__getRecordActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$Z(untrustedConfig, getRecordActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$I, buildNetworkSnapshotCachePolicy$J);
};

function select$2Q(luvio, params) {
    return select$35();
}
function keyBuilder$2U(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'actionTypes:' + params.queryParams.actionTypes + ',' + 'formFactor:' + params.queryParams.formFactor + ',' + 'sections:' + params.queryParams.sections + ',' + 'recordIds:' + params.urlParams.recordIds + ')';
}
function getResponseCacheKeys$T(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$2U(luvio, resourceParams));
}
function ingestSuccess$J(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2U(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2Q(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$B(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2U(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$Y(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/record/' + config.urlParams.recordIds + '/record-edit',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$M = 'getRecordEditActions';
const getRecordEditActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('actionTypes', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('sections', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getRecordEditActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$M, getRecordEditActions_ConfigPropertyMetadata);
const createResourceParams$R = /*#__PURE__*/ createResourceParams$k(getRecordEditActions_ConfigPropertyMetadata);
function keyBuilder$2T(luvio, config) {
    const resourceParams = createResourceParams$R(config);
    return keyBuilder$2U(luvio, resourceParams);
}
function typeCheckConfig$V(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordEditActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$Y(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordEditActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$V(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$A(luvio, config) {
    createResourceParams$R(config);
    return select$2Q();
}
function onFetchResponseSuccess$E(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$J(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$W(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$E(luvio, config, resourceParams, response) {
    const snapshot = ingestError$B(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$W(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$W(luvio, config, options) {
    const resourceParams = createResourceParams$R(config);
    const request = createResourceRequest$Y(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$E(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$T(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$E(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$I(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$W, undefined, false);
}
function buildCachedSnapshotCachePolicy$H(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2T(luvio, config),
        node: adapterFragment$A(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$W(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRecordEditActionsAdapterFactory = (luvio) => function UiApi__getRecordEditActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$Y(untrustedConfig, getRecordEditActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$H, buildNetworkSnapshotCachePolicy$I);
};

function validate$1o(obj, path = 'ActionRelatedListSingleBatchInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        if (obj.actionTypes !== undefined) {
            const obj_actionTypes = obj.actionTypes;
            const path_actionTypes = path + '.actionTypes';
            if (!ArrayIsArray(obj_actionTypes)) {
                return new TypeError('Expected "array" but received "' + typeof obj_actionTypes + '" (at "' + path_actionTypes + '")');
            }
            for (let i = 0; i < obj_actionTypes.length; i++) {
                const obj_actionTypes_item = obj_actionTypes[i];
                const path_actionTypes_item = path_actionTypes + '[' + i + ']';
                if (typeof obj_actionTypes_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_actionTypes_item + '" (at "' + path_actionTypes_item + '")');
                }
            }
        }
        if (obj.apiNames !== undefined) {
            const obj_apiNames = obj.apiNames;
            const path_apiNames = path + '.apiNames';
            if (!ArrayIsArray(obj_apiNames)) {
                return new TypeError('Expected "array" but received "' + typeof obj_apiNames + '" (at "' + path_apiNames + '")');
            }
            for (let i = 0; i < obj_apiNames.length; i++) {
                const obj_apiNames_item = obj_apiNames[i];
                const path_apiNames_item = path_apiNames + '[' + i + ']';
                if (typeof obj_apiNames_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_apiNames_item + '" (at "' + path_apiNames_item + '")');
                }
            }
        }
        if (obj.formFactor !== undefined) {
            const obj_formFactor = obj.formFactor;
            const path_formFactor = path + '.formFactor';
            if (typeof obj_formFactor !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_formFactor + '" (at "' + path_formFactor + '")');
            }
        }
        const obj_relatedListId = obj.relatedListId;
        const path_relatedListId = path + '.relatedListId';
        if (typeof obj_relatedListId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_relatedListId + '" (at "' + path_relatedListId + '")');
        }
        if (obj.retrievalMode !== undefined) {
            const obj_retrievalMode = obj.retrievalMode;
            const path_retrievalMode = path + '.retrievalMode';
            if (typeof obj_retrievalMode !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_retrievalMode + '" (at "' + path_retrievalMode + '")');
            }
        }
        if (obj.sections !== undefined) {
            const obj_sections = obj.sections;
            const path_sections = path + '.sections';
            if (!ArrayIsArray(obj_sections)) {
                return new TypeError('Expected "array" but received "' + typeof obj_sections + '" (at "' + path_sections + '")');
            }
            for (let i = 0; i < obj_sections.length; i++) {
                const obj_sections_item = obj_sections[i];
                const path_sections_item = path_sections + '[' + i + ']';
                if (typeof obj_sections_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_sections_item + '" (at "' + path_sections_item + '")');
                }
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function select$2P(luvio, params) {
    return select$35();
}
function keyBuilder$2S(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'recordIds:' + params.urlParams.recordIds + ',' + 'relatedListId:' + params.urlParams.relatedListId + ',' + (params.body.actionTypes === undefined ? 'actionTypes' : 'actionTypes:' + params.body.actionTypes) + '::' + (params.body.apiNames === undefined ? 'apiNames' : 'apiNames:' + params.body.apiNames) + '::' + (params.body.formFactor === undefined ? 'formFactor' : 'formFactor:' + params.body.formFactor) + '::' + (params.body.retrievalMode === undefined ? 'retrievalMode' : 'retrievalMode:' + params.body.retrievalMode) + '::' + (params.body.sections === undefined ? 'sections' : 'sections:' + params.body.sections) + ')';
}
function getResponseCacheKeys$S(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$2S(luvio, resourceParams));
}
function ingestSuccess$I(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2S(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2P(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$A(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2S(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$X(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/record/' + config.urlParams.recordIds + '/related-list/' + config.urlParams.relatedListId + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

function createChildResourceParams$a(resourceParams) {
    const childConfigs = [];
    for (let index = 0, len = resourceParams.body.relatedListsActionParameters.length; index < len; index += 1) {
        let childConfig = {
            urlParams: {
                recordIds: resourceParams.urlParams.recordIds,
                relatedListId: resourceParams.body.relatedListsActionParameters[index].relatedListId,
            },
            body: {},
        };
        let relatedListConfig = resourceParams.body.relatedListsActionParameters[index];
        // manually populate childConfig body
        if ('apiNames' in relatedListConfig) {
            childConfig.body.apiNames = toSortedStringArrayAllowEmpty(relatedListConfig.apiNames);
        }
        if ('retrievalMode' in relatedListConfig) {
            childConfig.body.retrievalMode = relatedListConfig.retrievalMode;
        }
        if ('formFactor' in relatedListConfig) {
            childConfig.body.formFactor = coerceFormFactor(relatedListConfig.formFactor);
        }
        if ('actionTypes' in relatedListConfig) {
            childConfig.body.actionTypes = relatedListConfig.actionTypes;
        }
        if ('sections' in relatedListConfig) {
            childConfig.body.sections = toSortedStringArray(relatedListConfig.sections);
        }
        childConfigs.push(childConfig);
    }
    return childConfigs;
}

const nonCachedErrors$7 = ObjectCreate$1(null);
function createChildResourceParams$9(resourceParams) {
    return createChildResourceParams$a(resourceParams);
}
function selectChildResourceParams$7(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$2R(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$2S(luvio, childResource);
                const childFragment = select$2P();
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$7[childKey];
                            if (nonCachedError === undefined || nonCachedError.expiration < reader.getTimeStamp()) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$2O(luvio, resourceParams) {
    const childResources = createChildResourceParams$9(resourceParams);
    return selectChildResourceParams$7(luvio, childResources, resourceParams);
}
function keyBuilder$2R(luvio, params) {
    return keyPrefix + '::ActionRelatedListsCollectionBatchRepresentation:(' + 'recordIds:' + params.urlParams.recordIds + ',' + '[' + params.body.relatedListsActionParameters.map(element => (element.actionTypes === undefined ? 'relatedListsActionParameters.actionTypes' : 'relatedListsActionParameters.actionTypes:' + element.actionTypes) + '::' + (element.apiNames === undefined ? 'relatedListsActionParameters.apiNames' : 'relatedListsActionParameters.apiNames:' + element.apiNames) + '::' + (element.formFactor === undefined ? 'relatedListsActionParameters.formFactor' : 'relatedListsActionParameters.formFactor:' + element.formFactor) + '::' + 'relatedListsActionParameters.relatedListId:' + element.relatedListId + '::' + (element.retrievalMode === undefined ? 'relatedListsActionParameters.retrievalMode' : 'relatedListsActionParameters.retrievalMode:' + element.retrievalMode) + '::' + (element.sections === undefined ? 'relatedListsActionParameters.sections' : 'relatedListsActionParameters.sections:' + element.sections)).join(',') + ']' + ')';
}
function getResponseCacheKeys$R(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$9(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$S(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$2S(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$J,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$8(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$2S(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$I(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$A(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$7[childKey] = { expiration: now + TTL$y, response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$7[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$H(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$9(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$2R(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$8(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$2O(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$z(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2R(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$W(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/record/' + config.urlParams.recordIds + '/related-list/batch',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$L = 'getRelatedListsActions';
const getRelatedListsActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('relatedListsActionParameters', true, 2 /* Body */, 4 /* Unsupported */, true),
];
const getRelatedListsActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$L, getRelatedListsActions_ConfigPropertyMetadata);
const createResourceParams$Q = /*#__PURE__*/ createResourceParams$k(getRelatedListsActions_ConfigPropertyMetadata);
function keyBuilder$2Q(luvio, config) {
    const resourceParams = createResourceParams$Q(config);
    return keyBuilder$2R(luvio, resourceParams);
}
function typeCheckConfig$U(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListsActions_ConfigPropertyMetadata);
    const untrustedConfig_relatedListsActionParameters = untrustedConfig.relatedListsActionParameters;
    if (ArrayIsArray$1(untrustedConfig_relatedListsActionParameters)) {
        const untrustedConfig_relatedListsActionParameters_array = [];
        for (let i = 0, arrayLength = untrustedConfig_relatedListsActionParameters.length; i < arrayLength; i++) {
            const untrustedConfig_relatedListsActionParameters_item = untrustedConfig_relatedListsActionParameters[i];
            const referenceActionRelatedListSingleBatchInputRepresentationValidationError = validate$1o(untrustedConfig_relatedListsActionParameters_item);
            if (referenceActionRelatedListSingleBatchInputRepresentationValidationError === null) {
                untrustedConfig_relatedListsActionParameters_array.push(untrustedConfig_relatedListsActionParameters_item);
            }
        }
        config.relatedListsActionParameters = untrustedConfig_relatedListsActionParameters_array;
    }
    return config;
}
function validateAdapterConfig$X(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListsActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$U(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$z(luvio, config) {
    const resourceParams = createResourceParams$Q(config);
    return select$2O(luvio, resourceParams);
}
function onFetchResponseSuccess$D(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$H(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$V(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$D(luvio, config, resourceParams, response) {
    const snapshot = ingestError$z(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$V(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$V(luvio, config, options) {
    const resourceParams = createResourceParams$Q(config);
    const request = createResourceRequest$W(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$D(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$R(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$D(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$H(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$V, 'get', false);
}
function buildCachedSnapshotCachePolicy$G(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2Q(luvio, config),
        node: adapterFragment$z(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$V(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListsActionsAdapterFactory = (luvio) => function UiApi__getRelatedListsActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$X(untrustedConfig, getRelatedListsActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$G, buildNetworkSnapshotCachePolicy$H);
};

const adapterName$K = 'getRelatedListActions';
const getRelatedListActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('relatedListId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('actionTypes', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('apiNames', false, 2 /* Body */, 0 /* String */, true, toSortedStringArrayAllowEmpty),
    generateParamConfigMetadata('formFactor', false, 2 /* Body */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('retrievalMode', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('sections', false, 2 /* Body */, 0 /* String */, true, toSortedStringArray),
];
const getRelatedListActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$K, getRelatedListActions_ConfigPropertyMetadata);
const createResourceParams$P = /*#__PURE__*/ createResourceParams$k(getRelatedListActions_ConfigPropertyMetadata);
function keyBuilder$2P(luvio, config) {
    const resourceParams = createResourceParams$P(config);
    return keyBuilder$2S(luvio, resourceParams);
}
function typeCheckConfig$T(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$W(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$T(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$y(luvio, config) {
    createResourceParams$P(config);
    return select$2P();
}
function onFetchResponseSuccess$C(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$I(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$U(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$C(luvio, config, resourceParams, response) {
    const snapshot = ingestError$A(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$U(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$U(luvio, config, options) {
    const resourceParams = createResourceParams$P(config);
    const request = createResourceRequest$X(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$C(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$S(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$C(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$G(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$U, 'get', false);
}
function buildCachedSnapshotCachePolicy$F(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2P(luvio, config),
        node: adapterFragment$y(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$U(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListActionsAdapterFactory = (luvio) => function UiApi__getRelatedListActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$W(untrustedConfig, getRelatedListActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$F, buildNetworkSnapshotCachePolicy$G);
};

function select$2N(luvio, params) {
    return select$35();
}
function keyBuilder$2O(luvio, params) {
    return keyPrefix + '::ActionRepresentation:(' + 'actionTypes:' + params.queryParams.actionTypes + ',' + 'formFactor:' + params.queryParams.formFactor + ',' + 'sections:' + params.queryParams.sections + ',' + 'recordIds:' + params.urlParams.recordIds + ',' + 'relatedListRecordIds:' + params.urlParams.relatedListRecordIds + ')';
}
function getResponseCacheKeys$Q(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$23(storeKeyMap, luvio, response, () => keyBuilder$2O(luvio, resourceParams));
}
function ingestSuccess$G(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2O(luvio, resourceParams);
    luvio.storeIngest(key, ingest$20, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2N(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$y(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2O(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$y,
        namespace: keyPrefix,
        version: VERSION$2s,
        representationName: RepresentationType$J
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$V(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/record/' + config.urlParams.recordIds + '/related-list-record/' + config.urlParams.relatedListRecordIds + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$J = 'getRelatedListRecordActions';
const getRelatedListRecordActions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('relatedListRecordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('actionTypes', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('sections', false, 1 /* QueryParameter */, 0 /* String */, true, toSortedStringArray),
];
const getRelatedListRecordActions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$J, getRelatedListRecordActions_ConfigPropertyMetadata);
const createResourceParams$O = /*#__PURE__*/ createResourceParams$k(getRelatedListRecordActions_ConfigPropertyMetadata);
function keyBuilder$2N(luvio, config) {
    const resourceParams = createResourceParams$O(config);
    return keyBuilder$2O(luvio, resourceParams);
}
function typeCheckConfig$S(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListRecordActions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$V(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListRecordActions_ConfigPropertyMetadata);
    const config = typeCheckConfig$S(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$x(luvio, config) {
    createResourceParams$O(config);
    return select$2N();
}
function onFetchResponseSuccess$B(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$G(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$T(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$B(luvio, config, resourceParams, response) {
    const snapshot = ingestError$y(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$T(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$T(luvio, config, options) {
    const resourceParams = createResourceParams$O(config);
    const request = createResourceRequest$V(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$B(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$Q(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$B(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$F(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$T, undefined, false);
}
function buildCachedSnapshotCachePolicy$E(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2N(luvio, config),
        node: adapterFragment$x(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$T(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListRecordActionsAdapterFactory = (luvio) => function UiApi__getRelatedListRecordActions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$V(untrustedConfig, getRelatedListRecordActions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$E, buildNetworkSnapshotCachePolicy$F);
};

const VERSION$2j = "a316b3bba367f54adc67d7552ed7d36d";
function validate$1n(obj, path = 'PageReferenceRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_attributes = obj.attributes;
        const path_attributes = path + '.attributes';
        if (typeof obj_attributes !== 'object' || ArrayIsArray(obj_attributes) || obj_attributes === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_attributes + '" (at "' + path_attributes + '")');
        }
        const obj_attributes_keys = ObjectKeys(obj_attributes);
        for (let i = 0; i < obj_attributes_keys.length; i++) {
            const key = obj_attributes_keys[i];
            const obj_attributes_prop = obj_attributes[key];
            const path_attributes_prop = path_attributes + '["' + key + '"]';
            if (obj_attributes_prop === undefined) {
                return new TypeError('Expected "defined" but received "' + typeof obj_attributes_prop + '" (at "' + path_attributes_prop + '")');
            }
        }
        const obj_state = obj.state;
        const path_state = path + '.state';
        if (typeof obj_state !== 'object' || ArrayIsArray(obj_state) || obj_state === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_state + '" (at "' + path_state + '")');
        }
        const obj_state_keys = ObjectKeys(obj_state);
        for (let i = 0; i < obj_state_keys.length; i++) {
            const key = obj_state_keys[i];
            const obj_state_prop = obj_state[key];
            const path_state_prop = path_state + '["' + key + '"]';
            if (typeof obj_state_prop !== 'object' || ArrayIsArray(obj_state_prop) || obj_state_prop === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_state_prop + '" (at "' + path_state_prop + '")');
            }
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$E = 'PageReferenceRepresentation';
function normalize$x(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2M = function PageReferenceRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2j,
        private: [],
        opaque: true
    };
};
function equals$L(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1X = function PageReferenceRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1n(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$x, "UiApi", VERSION$2j, RepresentationType$E, equals$L);
    return createLink$1(key);
};
function getTypeCacheKeys$1_(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$E,
        mergeable: false
    });
}

const TTL$u = 120000;
const VERSION$2i = "09884ca5bf90ea4662092a4e48817081";
function validate$1m(obj, path = 'NavItemRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_availableInClassic = obj.availableInClassic;
        const path_availableInClassic = path + '.availableInClassic';
        if (typeof obj_availableInClassic !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_availableInClassic + '" (at "' + path_availableInClassic + '")');
        }
        const obj_availableInLightning = obj.availableInLightning;
        const path_availableInLightning = path + '.availableInLightning';
        if (typeof obj_availableInLightning !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_availableInLightning + '" (at "' + path_availableInLightning + '")');
        }
        const obj_color = obj.color;
        const path_color = path + '.color';
        if (typeof obj_color !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_color + '" (at "' + path_color + '")');
        }
        const obj_content = obj.content;
        const path_content = path + '.content';
        let obj_content_union0 = null;
        const obj_content_union0_error = (() => {
            if (typeof obj_content !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_content + '" (at "' + path_content + '")');
            }
        })();
        if (obj_content_union0_error != null) {
            obj_content_union0 = obj_content_union0_error.message;
        }
        let obj_content_union1 = null;
        const obj_content_union1_error = (() => {
            if (obj_content !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_content + '" (at "' + path_content + '")');
            }
        })();
        if (obj_content_union1_error != null) {
            obj_content_union1 = obj_content_union1_error.message;
        }
        if (obj_content_union0 && obj_content_union1) {
            let message = 'Object doesn\'t match union (at "' + path_content + '")';
            message += '\n' + obj_content_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_content_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_custom = obj.custom;
        const path_custom = path + '.custom';
        if (typeof obj_custom !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_custom + '" (at "' + path_custom + '")');
        }
        const obj_developerName = obj.developerName;
        const path_developerName = path + '.developerName';
        if (typeof obj_developerName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_developerName + '" (at "' + path_developerName + '")');
        }
        const obj_iconUrl = obj.iconUrl;
        const path_iconUrl = path + '.iconUrl';
        if (typeof obj_iconUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_itemType = obj.itemType;
        const path_itemType = path + '.itemType';
        if (typeof obj_itemType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_itemType + '" (at "' + path_itemType + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_objectLabel = obj.objectLabel;
        const path_objectLabel = path + '.objectLabel';
        let obj_objectLabel_union0 = null;
        const obj_objectLabel_union0_error = (() => {
            if (typeof obj_objectLabel !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectLabel + '" (at "' + path_objectLabel + '")');
            }
        })();
        if (obj_objectLabel_union0_error != null) {
            obj_objectLabel_union0 = obj_objectLabel_union0_error.message;
        }
        let obj_objectLabel_union1 = null;
        const obj_objectLabel_union1_error = (() => {
            if (obj_objectLabel !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_objectLabel + '" (at "' + path_objectLabel + '")');
            }
        })();
        if (obj_objectLabel_union1_error != null) {
            obj_objectLabel_union1 = obj_objectLabel_union1_error.message;
        }
        if (obj_objectLabel_union0 && obj_objectLabel_union1) {
            let message = 'Object doesn\'t match union (at "' + path_objectLabel + '")';
            message += '\n' + obj_objectLabel_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_objectLabel_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectLabelPlural = obj.objectLabelPlural;
        const path_objectLabelPlural = path + '.objectLabelPlural';
        let obj_objectLabelPlural_union0 = null;
        const obj_objectLabelPlural_union0_error = (() => {
            if (typeof obj_objectLabelPlural !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectLabelPlural + '" (at "' + path_objectLabelPlural + '")');
            }
        })();
        if (obj_objectLabelPlural_union0_error != null) {
            obj_objectLabelPlural_union0 = obj_objectLabelPlural_union0_error.message;
        }
        let obj_objectLabelPlural_union1 = null;
        const obj_objectLabelPlural_union1_error = (() => {
            if (obj_objectLabelPlural !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_objectLabelPlural + '" (at "' + path_objectLabelPlural + '")');
            }
        })();
        if (obj_objectLabelPlural_union1_error != null) {
            obj_objectLabelPlural_union1 = obj_objectLabelPlural_union1_error.message;
        }
        if (obj_objectLabelPlural_union0 && obj_objectLabelPlural_union1) {
            let message = 'Object doesn\'t match union (at "' + path_objectLabelPlural + '")';
            message += '\n' + obj_objectLabelPlural_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_objectLabelPlural_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_pageReference = obj.pageReference;
        const path_pageReference = path + '.pageReference';
        let obj_pageReference_union0 = null;
        const obj_pageReference_union0_error = (() => {
            if (typeof obj_pageReference !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_pageReference + '" (at "' + path_pageReference + '")');
            }
        })();
        if (obj_pageReference_union0_error != null) {
            obj_pageReference_union0 = obj_pageReference_union0_error.message;
        }
        let obj_pageReference_union1 = null;
        const obj_pageReference_union1_error = (() => {
            if (obj_pageReference !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_pageReference + '" (at "' + path_pageReference + '")');
            }
        })();
        if (obj_pageReference_union1_error != null) {
            obj_pageReference_union1 = obj_pageReference_union1_error.message;
        }
        if (obj_pageReference_union0 && obj_pageReference_union1) {
            let message = 'Object doesn\'t match union (at "' + path_pageReference + '")';
            message += '\n' + obj_pageReference_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_pageReference_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_standardType = obj.standardType;
        const path_standardType = path + '.standardType';
        let obj_standardType_union0 = null;
        const obj_standardType_union0_error = (() => {
            if (typeof obj_standardType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_standardType + '" (at "' + path_standardType + '")');
            }
        })();
        if (obj_standardType_union0_error != null) {
            obj_standardType_union0 = obj_standardType_union0_error.message;
        }
        let obj_standardType_union1 = null;
        const obj_standardType_union1_error = (() => {
            if (obj_standardType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_standardType + '" (at "' + path_standardType + '")');
            }
        })();
        if (obj_standardType_union1_error != null) {
            obj_standardType_union1 = obj_standardType_union1_error.message;
        }
        if (obj_standardType_union0 && obj_standardType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_standardType + '")';
            message += '\n' + obj_standardType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_standardType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$D = 'NavItemRepresentation';
function keyBuilder$2M(luvio, config) {
    return keyPrefix + '::' + RepresentationType$D + ':' + config.developerName;
}
function keyBuilderFromType$m(luvio, object) {
    const keyParams = {
        developerName: object.developerName
    };
    return keyBuilder$2M(luvio, keyParams);
}
function normalize$w(input, existing, path, luvio, store, timestamp) {
    const input_pageReference = input.pageReference;
    const input_pageReference_id = path.fullPath + '__pageReference';
    if (input_pageReference !== null && typeof input_pageReference === 'object') {
        input.pageReference = ingest$1X(input_pageReference, {
            fullPath: input_pageReference_id,
            propertyName: 'pageReference',
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$2L = function NavItemRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2i,
        private: [],
        selections: [
            {
                name: 'availableInClassic',
                kind: 'Scalar'
            },
            {
                name: 'availableInLightning',
                kind: 'Scalar'
            },
            {
                name: 'color',
                kind: 'Scalar'
            },
            {
                name: 'content',
                kind: 'Scalar'
            },
            {
                name: 'custom',
                kind: 'Scalar'
            },
            {
                name: 'developerName',
                kind: 'Scalar'
            },
            {
                name: 'iconUrl',
                kind: 'Scalar'
            },
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'itemType',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'objectApiName',
                kind: 'Scalar'
            },
            {
                name: 'objectLabel',
                kind: 'Scalar'
            },
            {
                name: 'objectLabelPlural',
                kind: 'Scalar'
            },
            {
                name: 'pageReference',
                kind: 'Link',
                nullable: true,
                fragment: select$2M()
            },
            {
                name: 'standardType',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$K(existing, incoming) {
    const existing_availableInClassic = existing.availableInClassic;
    const incoming_availableInClassic = incoming.availableInClassic;
    if (!(existing_availableInClassic === incoming_availableInClassic)) {
        return false;
    }
    const existing_availableInLightning = existing.availableInLightning;
    const incoming_availableInLightning = incoming.availableInLightning;
    if (!(existing_availableInLightning === incoming_availableInLightning)) {
        return false;
    }
    const existing_custom = existing.custom;
    const incoming_custom = incoming.custom;
    if (!(existing_custom === incoming_custom)) {
        return false;
    }
    const existing_color = existing.color;
    const incoming_color = incoming.color;
    if (!(existing_color === incoming_color)) {
        return false;
    }
    const existing_developerName = existing.developerName;
    const incoming_developerName = incoming.developerName;
    if (!(existing_developerName === incoming_developerName)) {
        return false;
    }
    const existing_iconUrl = existing.iconUrl;
    const incoming_iconUrl = incoming.iconUrl;
    if (!(existing_iconUrl === incoming_iconUrl)) {
        return false;
    }
    const existing_itemType = existing.itemType;
    const incoming_itemType = incoming.itemType;
    if (!(existing_itemType === incoming_itemType)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_content = existing.content;
    const incoming_content = incoming.content;
    if (!(existing_content === incoming_content)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_objectLabel = existing.objectLabel;
    const incoming_objectLabel = incoming.objectLabel;
    if (!(existing_objectLabel === incoming_objectLabel)) {
        return false;
    }
    const existing_objectLabelPlural = existing.objectLabelPlural;
    const incoming_objectLabelPlural = incoming.objectLabelPlural;
    if (!(existing_objectLabelPlural === incoming_objectLabelPlural)) {
        return false;
    }
    const existing_pageReference = existing.pageReference;
    const incoming_pageReference = incoming.pageReference;
    if (!(existing_pageReference === incoming_pageReference
        || (existing_pageReference != null &&
            incoming_pageReference != null &&
            existing_pageReference.__ref != null &&
            incoming_pageReference.__ref != null &&
            existing_pageReference.__ref === incoming_pageReference.__ref))) {
        return false;
    }
    const existing_standardType = existing.standardType;
    const incoming_standardType = incoming.standardType;
    if (!(existing_standardType === incoming_standardType)) {
        return false;
    }
    return true;
}
const ingest$1W = function NavItemRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1m(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$m(luvio, input);
    const ttlToUse = TTL$u;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$w, "UiApi", VERSION$2i, RepresentationType$D, equals$K);
    return createLink$1(key);
};
function getTypeCacheKeys$1Z(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$m(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$D,
        mergeable: false
    });
    if (input.pageReference !== null && typeof input.pageReference === 'object') {
        getTypeCacheKeys$1_(rootKeySet, luvio, input.pageReference, () => rootKey + "__" + "pageReference");
    }
}

const TTL$t = 300000;
const VERSION$2h = "0cd492fdd6c713c206ef53dc02175f71";
function validate$1l(obj, path = 'AppRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_appId = obj.appId;
        const path_appId = path + '.appId';
        let obj_appId_union0 = null;
        const obj_appId_union0_error = (() => {
            if (typeof obj_appId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_appId + '" (at "' + path_appId + '")');
            }
        })();
        if (obj_appId_union0_error != null) {
            obj_appId_union0 = obj_appId_union0_error.message;
        }
        let obj_appId_union1 = null;
        const obj_appId_union1_error = (() => {
            if (obj_appId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_appId + '" (at "' + path_appId + '")');
            }
        })();
        if (obj_appId_union1_error != null) {
            obj_appId_union1 = obj_appId_union1_error.message;
        }
        if (obj_appId_union0 && obj_appId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_appId + '")';
            message += '\n' + obj_appId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_appId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_description = obj.description;
        const path_description = path + '.description';
        let obj_description_union0 = null;
        const obj_description_union0_error = (() => {
            if (typeof obj_description !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_description + '" (at "' + path_description + '")');
            }
        })();
        if (obj_description_union0_error != null) {
            obj_description_union0 = obj_description_union0_error.message;
        }
        let obj_description_union1 = null;
        const obj_description_union1_error = (() => {
            if (obj_description !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_description + '" (at "' + path_description + '")');
            }
        })();
        if (obj_description_union1_error != null) {
            obj_description_union1 = obj_description_union1_error.message;
        }
        if (obj_description_union0 && obj_description_union1) {
            let message = 'Object doesn\'t match union (at "' + path_description + '")';
            message += '\n' + obj_description_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_description_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_developerName = obj.developerName;
        const path_developerName = path + '.developerName';
        if (typeof obj_developerName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_developerName + '" (at "' + path_developerName + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_formFactors = obj.formFactors;
        const path_formFactors = path + '.formFactors';
        if (!ArrayIsArray(obj_formFactors)) {
            return new TypeError('Expected "array" but received "' + typeof obj_formFactors + '" (at "' + path_formFactors + '")');
        }
        for (let i = 0; i < obj_formFactors.length; i++) {
            const obj_formFactors_item = obj_formFactors[i];
            const path_formFactors_item = path_formFactors + '[' + i + ']';
            if (typeof obj_formFactors_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_formFactors_item + '" (at "' + path_formFactors_item + '")');
            }
        }
        const obj_headerColor = obj.headerColor;
        const path_headerColor = path + '.headerColor';
        let obj_headerColor_union0 = null;
        const obj_headerColor_union0_error = (() => {
            if (typeof obj_headerColor !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_headerColor + '" (at "' + path_headerColor + '")');
            }
        })();
        if (obj_headerColor_union0_error != null) {
            obj_headerColor_union0 = obj_headerColor_union0_error.message;
        }
        let obj_headerColor_union1 = null;
        const obj_headerColor_union1_error = (() => {
            if (obj_headerColor !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_headerColor + '" (at "' + path_headerColor + '")');
            }
        })();
        if (obj_headerColor_union1_error != null) {
            obj_headerColor_union1 = obj_headerColor_union1_error.message;
        }
        if (obj_headerColor_union0 && obj_headerColor_union1) {
            let message = 'Object doesn\'t match union (at "' + path_headerColor + '")';
            message += '\n' + obj_headerColor_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_headerColor_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_iconUrl = obj.iconUrl;
        const path_iconUrl = path + '.iconUrl';
        let obj_iconUrl_union0 = null;
        const obj_iconUrl_union0_error = (() => {
            if (typeof obj_iconUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union0_error != null) {
            obj_iconUrl_union0 = obj_iconUrl_union0_error.message;
        }
        let obj_iconUrl_union1 = null;
        const obj_iconUrl_union1_error = (() => {
            if (obj_iconUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union1_error != null) {
            obj_iconUrl_union1 = obj_iconUrl_union1_error.message;
        }
        if (obj_iconUrl_union0 && obj_iconUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_iconUrl + '")';
            message += '\n' + obj_iconUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_iconUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_isNavAutoTempTabsDisabled = obj.isNavAutoTempTabsDisabled;
        const path_isNavAutoTempTabsDisabled = path + '.isNavAutoTempTabsDisabled';
        if (typeof obj_isNavAutoTempTabsDisabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isNavAutoTempTabsDisabled + '" (at "' + path_isNavAutoTempTabsDisabled + '")');
        }
        const obj_isNavPersonalizationDisabled = obj.isNavPersonalizationDisabled;
        const path_isNavPersonalizationDisabled = path + '.isNavPersonalizationDisabled';
        if (typeof obj_isNavPersonalizationDisabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isNavPersonalizationDisabled + '" (at "' + path_isNavPersonalizationDisabled + '")');
        }
        const obj_isNavTabPersistenceDisabled = obj.isNavTabPersistenceDisabled;
        const path_isNavTabPersistenceDisabled = path + '.isNavTabPersistenceDisabled';
        if (typeof obj_isNavTabPersistenceDisabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isNavTabPersistenceDisabled + '" (at "' + path_isNavTabPersistenceDisabled + '")');
        }
        const obj_isOmniPinnedViewEnabled = obj.isOmniPinnedViewEnabled;
        const path_isOmniPinnedViewEnabled = path + '.isOmniPinnedViewEnabled';
        if (typeof obj_isOmniPinnedViewEnabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isOmniPinnedViewEnabled + '" (at "' + path_isOmniPinnedViewEnabled + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_logoUrl = obj.logoUrl;
        const path_logoUrl = path + '.logoUrl';
        if (typeof obj_logoUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_logoUrl + '" (at "' + path_logoUrl + '")');
        }
        const obj_mobileStartUrl = obj.mobileStartUrl;
        const path_mobileStartUrl = path + '.mobileStartUrl';
        let obj_mobileStartUrl_union0 = null;
        const obj_mobileStartUrl_union0_error = (() => {
            if (typeof obj_mobileStartUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_mobileStartUrl + '" (at "' + path_mobileStartUrl + '")');
            }
        })();
        if (obj_mobileStartUrl_union0_error != null) {
            obj_mobileStartUrl_union0 = obj_mobileStartUrl_union0_error.message;
        }
        let obj_mobileStartUrl_union1 = null;
        const obj_mobileStartUrl_union1_error = (() => {
            if (obj_mobileStartUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_mobileStartUrl + '" (at "' + path_mobileStartUrl + '")');
            }
        })();
        if (obj_mobileStartUrl_union1_error != null) {
            obj_mobileStartUrl_union1 = obj_mobileStartUrl_union1_error.message;
        }
        if (obj_mobileStartUrl_union0 && obj_mobileStartUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_mobileStartUrl + '")';
            message += '\n' + obj_mobileStartUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_mobileStartUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_navItems = obj.navItems;
        const path_navItems = path + '.navItems';
        if (!ArrayIsArray(obj_navItems)) {
            return new TypeError('Expected "array" but received "' + typeof obj_navItems + '" (at "' + path_navItems + '")');
        }
        for (let i = 0; i < obj_navItems.length; i++) {
            const obj_navItems_item = obj_navItems[i];
            const path_navItems_item = path_navItems + '[' + i + ']';
            if (typeof obj_navItems_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_navItems_item + '" (at "' + path_navItems_item + '")');
            }
        }
        const obj_selected = obj.selected;
        const path_selected = path + '.selected';
        if (typeof obj_selected !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_selected + '" (at "' + path_selected + '")');
        }
        const obj_startUrl = obj.startUrl;
        const path_startUrl = path + '.startUrl';
        if (typeof obj_startUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_startUrl + '" (at "' + path_startUrl + '")');
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
        const obj_userNavItems = obj.userNavItems;
        const path_userNavItems = path + '.userNavItems';
        if (!ArrayIsArray(obj_userNavItems)) {
            return new TypeError('Expected "array" but received "' + typeof obj_userNavItems + '" (at "' + path_userNavItems + '")');
        }
        for (let i = 0; i < obj_userNavItems.length; i++) {
            const obj_userNavItems_item = obj_userNavItems[i];
            const path_userNavItems_item = path_userNavItems + '[' + i + ']';
            if (typeof obj_userNavItems_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_userNavItems_item + '" (at "' + path_userNavItems_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$C = 'AppRepresentation';
function keyBuilder$2L(luvio, config) {
    return keyPrefix + '::' + RepresentationType$C + ':' + (config.appId === null ? '' : config.appId);
}
function keyBuilderFromType$l(luvio, object) {
    const keyParams = {
        appId: object.appId
    };
    return keyBuilder$2L(luvio, keyParams);
}
function normalize$v(input, existing, path, luvio, store, timestamp) {
    const input_navItems = input.navItems;
    const input_navItems_id = path.fullPath + '__navItems';
    for (let i = 0; i < input_navItems.length; i++) {
        const input_navItems_item = input_navItems[i];
        let input_navItems_item_id = input_navItems_id + '__' + i;
        input_navItems[i] = ingest$1W(input_navItems_item, {
            fullPath: input_navItems_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_userNavItems = input.userNavItems;
    const input_userNavItems_id = path.fullPath + '__userNavItems';
    for (let i = 0; i < input_userNavItems.length; i++) {
        const input_userNavItems_item = input_userNavItems[i];
        let input_userNavItems_item_id = input_userNavItems_id + '__' + i;
        input_userNavItems[i] = ingest$1W(input_userNavItems_item, {
            fullPath: input_userNavItems_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$2K = function AppRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2h,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'appId',
                kind: 'Scalar'
            },
            {
                name: 'description',
                kind: 'Scalar'
            },
            {
                name: 'developerName',
                kind: 'Scalar'
            },
            {
                name: 'formFactors',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'headerColor',
                kind: 'Scalar'
            },
            {
                name: 'iconUrl',
                kind: 'Scalar'
            },
            {
                name: 'isNavAutoTempTabsDisabled',
                kind: 'Scalar'
            },
            {
                name: 'isNavPersonalizationDisabled',
                kind: 'Scalar'
            },
            {
                name: 'isNavTabPersistenceDisabled',
                kind: 'Scalar'
            },
            {
                name: 'isOmniPinnedViewEnabled',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'logoUrl',
                kind: 'Scalar'
            },
            {
                name: 'mobileStartUrl',
                kind: 'Scalar'
            },
            {
                name: 'navItems',
                kind: 'Link',
                plural: true,
                fragment: select$2L()
            },
            {
                name: 'selected',
                kind: 'Scalar'
            },
            {
                name: 'startUrl',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            },
            {
                name: 'userNavItems',
                kind: 'Link',
                plural: true,
                fragment: select$2L()
            }
        ]
    };
};
function equals$J(existing, incoming) {
    const existing_isNavAutoTempTabsDisabled = existing.isNavAutoTempTabsDisabled;
    const incoming_isNavAutoTempTabsDisabled = incoming.isNavAutoTempTabsDisabled;
    if (!(existing_isNavAutoTempTabsDisabled === incoming_isNavAutoTempTabsDisabled)) {
        return false;
    }
    const existing_isNavPersonalizationDisabled = existing.isNavPersonalizationDisabled;
    const incoming_isNavPersonalizationDisabled = incoming.isNavPersonalizationDisabled;
    if (!(existing_isNavPersonalizationDisabled === incoming_isNavPersonalizationDisabled)) {
        return false;
    }
    const existing_isNavTabPersistenceDisabled = existing.isNavTabPersistenceDisabled;
    const incoming_isNavTabPersistenceDisabled = incoming.isNavTabPersistenceDisabled;
    if (!(existing_isNavTabPersistenceDisabled === incoming_isNavTabPersistenceDisabled)) {
        return false;
    }
    const existing_isOmniPinnedViewEnabled = existing.isOmniPinnedViewEnabled;
    const incoming_isOmniPinnedViewEnabled = incoming.isOmniPinnedViewEnabled;
    if (!(existing_isOmniPinnedViewEnabled === incoming_isOmniPinnedViewEnabled)) {
        return false;
    }
    const existing_selected = existing.selected;
    const incoming_selected = incoming.selected;
    if (!(existing_selected === incoming_selected)) {
        return false;
    }
    const existing_developerName = existing.developerName;
    const incoming_developerName = incoming.developerName;
    if (!(existing_developerName === incoming_developerName)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_logoUrl = existing.logoUrl;
    const incoming_logoUrl = incoming.logoUrl;
    if (!(existing_logoUrl === incoming_logoUrl)) {
        return false;
    }
    const existing_startUrl = existing.startUrl;
    const incoming_startUrl = incoming.startUrl;
    if (!(existing_startUrl === incoming_startUrl)) {
        return false;
    }
    const existing_type = existing.type;
    const incoming_type = incoming.type;
    if (!(existing_type === incoming_type)) {
        return false;
    }
    const existing_appId = existing.appId;
    const incoming_appId = incoming.appId;
    if (!(existing_appId === incoming_appId)) {
        return false;
    }
    const existing_description = existing.description;
    const incoming_description = incoming.description;
    if (!(existing_description === incoming_description)) {
        return false;
    }
    const existing_formFactors = existing.formFactors;
    const incoming_formFactors = incoming.formFactors;
    const equals_formFactors_items = equalsArray(existing_formFactors, incoming_formFactors, (existing_formFactors_item, incoming_formFactors_item) => {
        if (!(existing_formFactors_item === incoming_formFactors_item)) {
            return false;
        }
    });
    if (equals_formFactors_items === false) {
        return false;
    }
    const existing_headerColor = existing.headerColor;
    const incoming_headerColor = incoming.headerColor;
    if (!(existing_headerColor === incoming_headerColor)) {
        return false;
    }
    const existing_iconUrl = existing.iconUrl;
    const incoming_iconUrl = incoming.iconUrl;
    if (!(existing_iconUrl === incoming_iconUrl)) {
        return false;
    }
    const existing_mobileStartUrl = existing.mobileStartUrl;
    const incoming_mobileStartUrl = incoming.mobileStartUrl;
    if (!(existing_mobileStartUrl === incoming_mobileStartUrl)) {
        return false;
    }
    const existing_navItems = existing.navItems;
    const incoming_navItems = incoming.navItems;
    const equals_navItems_items = equalsArray(existing_navItems, incoming_navItems, (existing_navItems_item, incoming_navItems_item) => {
        if (!(existing_navItems_item.__ref === incoming_navItems_item.__ref)) {
            return false;
        }
    });
    if (equals_navItems_items === false) {
        return false;
    }
    const existing_userNavItems = existing.userNavItems;
    const incoming_userNavItems = incoming.userNavItems;
    const equals_userNavItems_items = equalsArray(existing_userNavItems, incoming_userNavItems, (existing_userNavItems_item, incoming_userNavItems_item) => {
        if (!(existing_userNavItems_item.__ref === incoming_userNavItems_item.__ref)) {
            return false;
        }
    });
    if (equals_userNavItems_items === false) {
        return false;
    }
    return true;
}
const ingest$1V = function AppRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1l(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$l(luvio, input);
    const ttlToUse = TTL$t;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$v, "UiApi", VERSION$2h, RepresentationType$C, equals$J);
    return createLink$1(key);
};
function getTypeCacheKeys$1Y(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$l(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$C,
        mergeable: false
    });
    const input_navItems_length = input.navItems.length;
    for (let i = 0; i < input_navItems_length; i++) {
        getTypeCacheKeys$1Z(rootKeySet, luvio, input.navItems[i]);
    }
    const input_userNavItems_length = input.userNavItems.length;
    for (let i = 0; i < input_userNavItems_length; i++) {
        getTypeCacheKeys$1Z(rootKeySet, luvio, input.userNavItems[i]);
    }
}

const TTL$s = 300000;
const VERSION$2g = "a254babf0b6414315db7808a157fd9fc";
function validate$1k(obj, path = 'AppsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apps = obj.apps;
        const path_apps = path + '.apps';
        if (!ArrayIsArray(obj_apps)) {
            return new TypeError('Expected "array" but received "' + typeof obj_apps + '" (at "' + path_apps + '")');
        }
        for (let i = 0; i < obj_apps.length; i++) {
            const obj_apps_item = obj_apps[i];
            const path_apps_item = path_apps + '[' + i + ']';
            if (typeof obj_apps_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_apps_item + '" (at "' + path_apps_item + '")');
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$B = 'AppsRepresentation';
function normalize$u(input, existing, path, luvio, store, timestamp) {
    const input_apps = input.apps;
    const input_apps_id = path.fullPath + '__apps';
    for (let i = 0; i < input_apps.length; i++) {
        const input_apps_item = input_apps[i];
        let input_apps_item_id = input_apps_id + '__' + i;
        input_apps[i] = ingest$1V(input_apps_item, {
            fullPath: input_apps_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$2J = function AppsRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2g,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'apps',
                kind: 'Link',
                plural: true,
                fragment: select$2K()
            }
        ]
    };
};
function equals$I(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_apps = existing.apps;
    const incoming_apps = incoming.apps;
    const equals_apps_items = equalsArray(existing_apps, incoming_apps, (existing_apps_item, incoming_apps_item) => {
        if (!(existing_apps_item.__ref === incoming_apps_item.__ref)) {
            return false;
        }
    });
    if (equals_apps_items === false) {
        return false;
    }
    return true;
}
const ingest$1U = function AppsRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1k(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$s;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$u, "UiApi", VERSION$2g, RepresentationType$B, equals$I);
    return createLink$1(key);
};
function getTypeCacheKeys$1X(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$B,
        mergeable: false
    });
    const input_apps_length = input.apps.length;
    for (let i = 0; i < input_apps_length; i++) {
        getTypeCacheKeys$1Y(rootKeySet, luvio, input.apps[i]);
    }
}

function select$2I(luvio, params) {
    return select$2J();
}
function keyBuilder$2K(luvio, params) {
    return keyPrefix + '::AppsRepresentation:(' + 'formFactor:' + params.queryParams.formFactor + ',' + 'userCustomizations:' + params.queryParams.userCustomizations + ')';
}
function getResponseCacheKeys$P(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1X(storeKeyMap, luvio, response, () => keyBuilder$2K(luvio, resourceParams));
}
function ingestSuccess$F(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2K(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1U, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2I(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$x(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2K(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$s,
        namespace: keyPrefix,
        version: VERSION$2g,
        representationName: RepresentationType$B
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$U(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/apps',
        method: 'get',
        body: null,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$I = 'getAllApps';
const getAllApps_ConfigPropertyMetadata = [
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('userCustomizations', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
const getAllApps_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$I, getAllApps_ConfigPropertyMetadata);
const createResourceParams$N = /*#__PURE__*/ createResourceParams$k(getAllApps_ConfigPropertyMetadata);
function keyBuilder$2J(luvio, config) {
    const resourceParams = createResourceParams$N(config);
    return keyBuilder$2K(luvio, resourceParams);
}
function typeCheckConfig$R(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getAllApps_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$U(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getAllApps_ConfigPropertyMetadata);
    const config = typeCheckConfig$R(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$w(luvio, config) {
    createResourceParams$N(config);
    return select$2I();
}
function onFetchResponseSuccess$A(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$F(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$S(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$A(luvio, config, resourceParams, response) {
    const snapshot = ingestError$x(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$S(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$S(luvio, config, options) {
    const resourceParams = createResourceParams$N(config);
    const request = createResourceRequest$U(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$A(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$P(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$A(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$E(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$S, undefined, false);
}
function buildCachedSnapshotCachePolicy$D(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2J(luvio, config),
        node: adapterFragment$w(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$S(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getAllAppsAdapterFactory = (luvio) => function UiApi__getAllApps(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$U(untrustedConfig, getAllApps_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$D, buildNetworkSnapshotCachePolicy$E);
};

function select$2G(luvio, params) {
    return select$2K();
}
function keyBuilder$2G(luvio, params) {
    return keyBuilder$2L(luvio, {
        appId: params.urlParams.appId
    });
}
function getResponseCacheKeys$N(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1Y(storeKeyMap, luvio, response);
}
function ingestSuccess$D(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2G(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1V, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2G(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$v(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2G(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$t,
        namespace: keyPrefix,
        version: VERSION$2h,
        representationName: RepresentationType$C
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$S(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/apps/' + config.urlParams.appId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$G = 'getAppDetails';
const getAppDetails_ConfigPropertyMetadata = [
    generateParamConfigMetadata('appId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('userCustomizations', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
const getAppDetails_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$G, getAppDetails_ConfigPropertyMetadata);
const createResourceParams$L = /*#__PURE__*/ createResourceParams$k(getAppDetails_ConfigPropertyMetadata);
function keyBuilder$2F(luvio, config) {
    const resourceParams = createResourceParams$L(config);
    return keyBuilder$2G(luvio, resourceParams);
}
function typeCheckConfig$P(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getAppDetails_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$S(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getAppDetails_ConfigPropertyMetadata);
    const config = typeCheckConfig$P(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$u(luvio, config) {
    createResourceParams$L(config);
    return select$2G();
}
function onFetchResponseSuccess$y(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$D(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$Q(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$y(luvio, config, resourceParams, response) {
    const snapshot = ingestError$v(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$Q(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$Q(luvio, config, options) {
    const resourceParams = createResourceParams$L(config);
    const request = createResourceRequest$S(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$y(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$N(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$y(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$C(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$Q, undefined, false);
}
function buildCachedSnapshotCachePolicy$B(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2F(luvio, config),
        node: adapterFragment$u(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$Q(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getAppDetailsAdapterFactory = (luvio) => function UiApi__getAppDetails(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$S(untrustedConfig, getAppDetails_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$B, buildNetworkSnapshotCachePolicy$C);
};

const VERSION$2f = "f43ef90ffde4d488414d6868e3aaefcb";
function validate$1j(obj, path = 'DuplicateRuleFilterItemRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_field = obj.field;
        const path_field = path + '.field';
        if (typeof obj_field !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_field + '" (at "' + path_field + '")');
        }
        const obj_operation = obj.operation;
        const path_operation = path + '.operation';
        if (typeof obj_operation !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operation + '" (at "' + path_operation + '")');
        }
        const obj_sortOrder = obj.sortOrder;
        const path_sortOrder = path + '.sortOrder';
        if (typeof obj_sortOrder !== 'number' || (typeof obj_sortOrder === 'number' && Math.floor(obj_sortOrder) !== obj_sortOrder)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_sortOrder + '" (at "' + path_sortOrder + '")');
        }
        const obj_value = obj.value;
        const path_value = path + '.value';
        if (typeof obj_value !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_value + '" (at "' + path_value + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2F = function DuplicateRuleFilterItemRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2f,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'field',
                kind: 'Scalar'
            },
            {
                name: 'operation',
                kind: 'Scalar'
            },
            {
                name: 'sortOrder',
                kind: 'Scalar'
            },
            {
                name: 'value',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2e = "320d7f254687278f2763d4b490dab3fb";
function validate$1i(obj, path = 'DuplicateRuleFilterRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_booleanFilter = obj.booleanFilter;
        const path_booleanFilter = path + '.booleanFilter';
        if (typeof obj_booleanFilter !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_booleanFilter + '" (at "' + path_booleanFilter + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_filterItems = obj.filterItems;
        const path_filterItems = path + '.filterItems';
        if (!ArrayIsArray(obj_filterItems)) {
            return new TypeError('Expected "array" but received "' + typeof obj_filterItems + '" (at "' + path_filterItems + '")');
        }
        for (let i = 0; i < obj_filterItems.length; i++) {
            const obj_filterItems_item = obj_filterItems[i];
            const path_filterItems_item = path_filterItems + '[' + i + ']';
            const referencepath_filterItems_itemValidationError = validate$1j(obj_filterItems_item, path_filterItems_item);
            if (referencepath_filterItems_itemValidationError !== null) {
                let message = 'Object doesn\'t match DuplicateRuleFilterItemRepresentation (at "' + path_filterItems_item + '")\n';
                message += referencepath_filterItems_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_ruleCritera = obj.ruleCritera;
        const path_ruleCritera = path + '.ruleCritera';
        if (typeof obj_ruleCritera !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_ruleCritera + '" (at "' + path_ruleCritera + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2E = function DuplicateRuleFilterRepresentationSelect() {
    const { selections: DuplicateRuleFilterItemRepresentation__selections, opaque: DuplicateRuleFilterItemRepresentation__opaque, } = select$2F();
    return {
        kind: 'Fragment',
        version: VERSION$2e,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'booleanFilter',
                kind: 'Scalar'
            },
            {
                name: 'filterItems',
                kind: 'Object',
                plural: true,
                selections: DuplicateRuleFilterItemRepresentation__selections
            },
            {
                name: 'ruleCritera',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2d = "b47b44e260e5ce777beaf3da9088997c";
function validate$1h(obj, path = 'MatchRuleRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_matchEngine = obj.matchEngine;
        const path_matchEngine = path + '.matchEngine';
        if (typeof obj_matchEngine !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_matchEngine + '" (at "' + path_matchEngine + '")');
        }
        const obj_matchFields = obj.matchFields;
        const path_matchFields = path + '.matchFields';
        if (!ArrayIsArray(obj_matchFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_matchFields + '" (at "' + path_matchFields + '")');
        }
        for (let i = 0; i < obj_matchFields.length; i++) {
            const obj_matchFields_item = obj_matchFields[i];
            const path_matchFields_item = path_matchFields + '[' + i + ']';
            if (typeof obj_matchFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_matchFields_item + '" (at "' + path_matchFields_item + '")');
            }
        }
        const obj_name = obj.name;
        const path_name = path + '.name';
        if (typeof obj_name !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_name + '" (at "' + path_name + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2D = function MatchRuleRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2d,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'matchEngine',
                kind: 'Scalar'
            },
            {
                name: 'matchFields',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'name',
                kind: 'Scalar'
            },
            {
                name: 'objectApiName',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$2c = "bb83d7210bb1d7861b6188bc5f552617";
function validate$1g(obj, path = 'DuplicateRuleRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_actionOnInsert = obj.actionOnInsert;
        const path_actionOnInsert = path + '.actionOnInsert';
        if (typeof obj_actionOnInsert !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_actionOnInsert + '" (at "' + path_actionOnInsert + '")');
        }
        const obj_actionOnUpdate = obj.actionOnUpdate;
        const path_actionOnUpdate = path + '.actionOnUpdate';
        if (typeof obj_actionOnUpdate !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_actionOnUpdate + '" (at "' + path_actionOnUpdate + '")');
        }
        const obj_active = obj.active;
        const path_active = path + '.active';
        if (typeof obj_active !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_active + '" (at "' + path_active + '")');
        }
        const obj_duplicateRuleFilters = obj.duplicateRuleFilters;
        const path_duplicateRuleFilters = path + '.duplicateRuleFilters';
        if (!ArrayIsArray(obj_duplicateRuleFilters)) {
            return new TypeError('Expected "array" but received "' + typeof obj_duplicateRuleFilters + '" (at "' + path_duplicateRuleFilters + '")');
        }
        for (let i = 0; i < obj_duplicateRuleFilters.length; i++) {
            const obj_duplicateRuleFilters_item = obj_duplicateRuleFilters[i];
            const path_duplicateRuleFilters_item = path_duplicateRuleFilters + '[' + i + ']';
            const referencepath_duplicateRuleFilters_itemValidationError = validate$1i(obj_duplicateRuleFilters_item, path_duplicateRuleFilters_item);
            if (referencepath_duplicateRuleFilters_itemValidationError !== null) {
                let message = 'Object doesn\'t match DuplicateRuleFilterRepresentation (at "' + path_duplicateRuleFilters_item + '")\n';
                message += referencepath_duplicateRuleFilters_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_matchRules = obj.matchRules;
        const path_matchRules = path + '.matchRules';
        if (!ArrayIsArray(obj_matchRules)) {
            return new TypeError('Expected "array" but received "' + typeof obj_matchRules + '" (at "' + path_matchRules + '")');
        }
        for (let i = 0; i < obj_matchRules.length; i++) {
            const obj_matchRules_item = obj_matchRules[i];
            const path_matchRules_item = path_matchRules + '[' + i + ']';
            const referencepath_matchRules_itemValidationError = validate$1h(obj_matchRules_item, path_matchRules_item);
            if (referencepath_matchRules_itemValidationError !== null) {
                let message = 'Object doesn\'t match MatchRuleRepresentation (at "' + path_matchRules_item + '")\n';
                message += referencepath_matchRules_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_name = obj.name;
        const path_name = path + '.name';
        if (typeof obj_name !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_name + '" (at "' + path_name + '")');
        }
        const obj_operationsOnInsert = obj.operationsOnInsert;
        const path_operationsOnInsert = path + '.operationsOnInsert';
        if (!ArrayIsArray(obj_operationsOnInsert)) {
            return new TypeError('Expected "array" but received "' + typeof obj_operationsOnInsert + '" (at "' + path_operationsOnInsert + '")');
        }
        for (let i = 0; i < obj_operationsOnInsert.length; i++) {
            const obj_operationsOnInsert_item = obj_operationsOnInsert[i];
            const path_operationsOnInsert_item = path_operationsOnInsert + '[' + i + ']';
            if (typeof obj_operationsOnInsert_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_operationsOnInsert_item + '" (at "' + path_operationsOnInsert_item + '")');
            }
        }
        const obj_operationsOnUpdate = obj.operationsOnUpdate;
        const path_operationsOnUpdate = path + '.operationsOnUpdate';
        if (!ArrayIsArray(obj_operationsOnUpdate)) {
            return new TypeError('Expected "array" but received "' + typeof obj_operationsOnUpdate + '" (at "' + path_operationsOnUpdate + '")');
        }
        for (let i = 0; i < obj_operationsOnUpdate.length; i++) {
            const obj_operationsOnUpdate_item = obj_operationsOnUpdate[i];
            const path_operationsOnUpdate_item = path_operationsOnUpdate + '[' + i + ']';
            if (typeof obj_operationsOnUpdate_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_operationsOnUpdate_item + '" (at "' + path_operationsOnUpdate_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2C = function DuplicateRuleRepresentationSelect() {
    const { selections: DuplicateRuleFilterRepresentation__selections, opaque: DuplicateRuleFilterRepresentation__opaque, } = select$2E();
    const { selections: MatchRuleRepresentation__selections, opaque: MatchRuleRepresentation__opaque, } = select$2D();
    return {
        kind: 'Fragment',
        version: VERSION$2c,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'actionOnInsert',
                kind: 'Scalar'
            },
            {
                name: 'actionOnUpdate',
                kind: 'Scalar'
            },
            {
                name: 'active',
                kind: 'Scalar'
            },
            {
                name: 'duplicateRuleFilters',
                kind: 'Object',
                plural: true,
                selections: DuplicateRuleFilterRepresentation__selections
            },
            {
                name: 'matchRules',
                kind: 'Object',
                plural: true,
                selections: MatchRuleRepresentation__selections
            },
            {
                name: 'name',
                kind: 'Scalar'
            },
            {
                name: 'operationsOnInsert',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'operationsOnUpdate',
                kind: 'Scalar',
                plural: true
            }
        ]
    };
};

const TTL$r = 900000;
const VERSION$2b = "be27ee99dc0dc43a1f66b8fe98dc532c";
function validate$1f(obj, path = 'DuplicatesConfigurationRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_dedupeEnabled = obj.dedupeEnabled;
        const path_dedupeEnabled = path + '.dedupeEnabled';
        if (typeof obj_dedupeEnabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_dedupeEnabled + '" (at "' + path_dedupeEnabled + '")');
        }
        const obj_dedupeFields = obj.dedupeFields;
        const path_dedupeFields = path + '.dedupeFields';
        if (!ArrayIsArray(obj_dedupeFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_dedupeFields + '" (at "' + path_dedupeFields + '")');
        }
        for (let i = 0; i < obj_dedupeFields.length; i++) {
            const obj_dedupeFields_item = obj_dedupeFields[i];
            const path_dedupeFields_item = path_dedupeFields + '[' + i + ']';
            if (typeof obj_dedupeFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_dedupeFields_item + '" (at "' + path_dedupeFields_item + '")');
            }
        }
        const obj_duplicateRules = obj.duplicateRules;
        const path_duplicateRules = path + '.duplicateRules';
        if (!ArrayIsArray(obj_duplicateRules)) {
            return new TypeError('Expected "array" but received "' + typeof obj_duplicateRules + '" (at "' + path_duplicateRules + '")');
        }
        for (let i = 0; i < obj_duplicateRules.length; i++) {
            const obj_duplicateRules_item = obj_duplicateRules[i];
            const path_duplicateRules_item = path_duplicateRules + '[' + i + ']';
            const referencepath_duplicateRules_itemValidationError = validate$1g(obj_duplicateRules_item, path_duplicateRules_item);
            if (referencepath_duplicateRules_itemValidationError !== null) {
                let message = 'Object doesn\'t match DuplicateRuleRepresentation (at "' + path_duplicateRules_item + '")\n';
                message += referencepath_duplicateRules_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_predupeEnabled = obj.predupeEnabled;
        const path_predupeEnabled = path + '.predupeEnabled';
        if (typeof obj_predupeEnabled !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_predupeEnabled + '" (at "' + path_predupeEnabled + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$A = 'DuplicatesConfigurationRepresentation';
function normalize$t(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2B = function DuplicatesConfigurationRepresentationSelect() {
    const { selections: DuplicateRuleRepresentation__selections, opaque: DuplicateRuleRepresentation__opaque, } = select$2C();
    return {
        kind: 'Fragment',
        version: VERSION$2b,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'dedupeEnabled',
                kind: 'Scalar'
            },
            {
                name: 'dedupeFields',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'duplicateRules',
                kind: 'Object',
                plural: true,
                selections: DuplicateRuleRepresentation__selections
            },
            {
                name: 'predupeEnabled',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$H(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$1T = function DuplicatesConfigurationRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$1f(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$r;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$t, "UiApi", VERSION$2b, RepresentationType$A, equals$H);
    return createLink$1(key);
};
function getTypeCacheKeys$1W(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$A,
        mergeable: false
    });
}

function select$2A(luvio, params) {
    return select$2B();
}
function keyBuilder$2E(luvio, params) {
    return keyPrefix + '::DuplicatesConfigurationRepresentation:(' + 'recordTypeId:' + params.queryParams.recordTypeId + ',' + 'objectApiName:' + params.urlParams.objectApiName + ')';
}
function getResponseCacheKeys$M(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1W(storeKeyMap, luvio, response, () => keyBuilder$2E(luvio, resourceParams));
}
function ingestSuccess$C(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2E(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1T, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2A(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$u(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2E(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$r,
        namespace: keyPrefix,
        version: VERSION$2b,
        representationName: RepresentationType$A
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$R(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/duplicates/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$F = 'getDuplicateConfiguration';
const getDuplicateConfiguration_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getDuplicateConfiguration_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$F, getDuplicateConfiguration_ConfigPropertyMetadata);
const createResourceParams$K = /*#__PURE__*/ createResourceParams$k(getDuplicateConfiguration_ConfigPropertyMetadata);
function keyBuilder$2D(luvio, config) {
    const resourceParams = createResourceParams$K(config);
    return keyBuilder$2E(luvio, resourceParams);
}
function typeCheckConfig$O(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getDuplicateConfiguration_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$R(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$O(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$t(luvio, config) {
    createResourceParams$K(config);
    return select$2A();
}
function onFetchResponseSuccess$x(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$C(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$P(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$x(luvio, config, resourceParams, response) {
    const snapshot = ingestError$u(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$P(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$P(luvio, config, options) {
    const resourceParams = createResourceParams$K(config);
    const request = createResourceRequest$R(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$x(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$M(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$x(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$B(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$P, undefined, false);
}
function buildCachedSnapshotCachePolicy$A(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2D(luvio, config),
        node: adapterFragment$t(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$P(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getDuplicateConfigurationAdapterFactory = (luvio) => function UiApi__getDuplicateConfiguration(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$R(untrustedConfig, getDuplicateConfiguration_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$A, buildNetworkSnapshotCachePolicy$B);
};

var LayoutType;
(function (LayoutType) {
    LayoutType["Full"] = "Full";
    LayoutType["Compact"] = "Compact";
})(LayoutType || (LayoutType = {}));

function coerceLayoutType(value) {
    if (value === LayoutType.Full || value === LayoutType.Compact) {
        return value;
    }
    return undefined;
}

function coerceLayoutMode(value) {
    if (value === LayoutMode.Create || value === LayoutMode.Edit || value === LayoutMode.View) {
        return value;
    }
    return undefined;
}

function keyBuilder$2C(luvio, params) {
    return keyPrefix + '::RecordLayoutRepresentation:(' + 'formFactor:' + params.queryParams.formFactor + ',' + 'layoutType:' + params.queryParams.layoutType + ',' + 'mode:' + params.queryParams.mode + ',' + 'recordTypeId:' + params.queryParams.recordTypeId + ',' + 'objectApiName:' + params.urlParams.objectApiName + ')';
}
function getResponseCacheKeys$L(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2b(storeKeyMap, luvio, response);
}
function ingestError$t(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2C(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$C,
        namespace: keyPrefix,
        version: VERSION$2B,
        representationName: RepresentationType$S
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$Q(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/layout/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

function onFetchResponseSuccess$w(luvio, config, resourceParams, response) {
    const { body } = response;
    const key = keyBuilder$3k(luvio, config);
    luvio.storeIngest(key, ingest$28, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$3c(),
        variables: {},
    }, {
        config,
        resolve: () => buildNetworkSnapshot$O(luvio, config, snapshotRefreshOptions),
    });
    return luvio.storeBroadcast().then(() => snapshot);
}

// FYI stricter required set than RAML, matches lds222 behavior
const getLayout_ConfigPropertyNames = {
    displayName: 'getLayout',
    parameters: {
        required: ['objectApiName', 'layoutType', 'mode'],
        optional: ['recordTypeId'],
    },
};

function validateAdapterConfig$Q(untrustedConfig, _configPropertyNames) {
    const config = validateAdapterConfig$P(untrustedConfig, getLayout_ConfigPropertyNames);
    if (config === null) {
        return null;
    }
    let recordTypeId = config.recordTypeId;
    if (recordTypeId === undefined) {
        // must check untrusted bc config has been coerced
        if (untrustedConfig.recordTypeId !== null &&
            untrustedConfig.recordTypeId !== undefined) {
            return null;
        }
        recordTypeId = MAIN_RECORD_TYPE_ID;
    }
    // layoutType and mode are required during validation.
    // They will always be valid at this point.
    return {
        ...config,
        recordTypeId,
        layoutType: config.layoutType,
        mode: config.mode,
    };
}

function buildCachedSnapshotCachePolicy$z(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$3k(luvio, config),
        node: select$3c(),
        variables: {},
    };
    return storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$O(luvio, config, snapshotRefreshOptions),
    });
}

const getLayout_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('layoutType', false, 1 /* QueryParameter */, 0 /* String */, false, coerceLayoutType),
    generateParamConfigMetadata('mode', false, 1 /* QueryParameter */, 0 /* String */, false, coerceLayoutMode),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */, false, getRecordId18),
];
const createResourceParams$J = /*#__PURE__*/ createResourceParams$k(getLayout_ConfigPropertyMetadata);
function typeCheckConfig$N(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getLayout_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$P(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getLayout_ConfigPropertyMetadata);
    const config = typeCheckConfig$N(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function onFetchResponseError$w(luvio, config, resourceParams, response) {
    const snapshot = ingestError$t(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$O(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$O(luvio, config, options) {
    const resourceParams = createResourceParams$J(config);
    const request = createResourceRequest$Q(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$w(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$L(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$w(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$A(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$O, undefined, false);
}
const getLayoutAdapterFactory = (luvio) => function UiApi__getLayout(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$Q(untrustedConfig);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$z, buildNetworkSnapshotCachePolicy$A);
};

function getResponseCacheKeys$K(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2a(storeKeyMap, luvio, response);
}
function createResourceRequest$P(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/layout/' + config.urlParams.objectApiName + '/user-state',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

function buildCachedSnapshot$5(luvio, config) {
    const { objectApiName, recordTypeId, layoutType, mode } = config;
    const key = keyBuilder$3j(luvio, {
        apiName: objectApiName,
        recordTypeId,
        layoutType,
        mode,
    });
    const selector = {
        recordId: key,
        node: select$3b(),
        variables: {},
    };
    return luvio.storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$N(luvio, config),
    });
}

function buildNetworkSnapshot$N(luvio, config, context) {
    const { resourceParams, request, key } = prepareRequest$4(luvio, config);
    return luvio
        .dispatchResourceRequest(request, context)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            return onFetchResponseSuccess$v(luvio, config, key, response);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$K(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (error) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$v(luvio, config, key, error));
    });
}
function onFetchResponseSuccess$v(luvio, config, key, response) {
    const { body } = response;
    const { recordTypeId, layoutType, mode } = config;
    // Hack- adding in this params so record-ui will be able to use normed values.
    body.apiName = config.objectApiName;
    body.recordTypeId = recordTypeId;
    body.layoutType = layoutType;
    body.mode = mode;
    luvio.storeIngest(key, ingest$27, body);
    const snapshot = buildCachedSnapshot$5(luvio, config);
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$v(luvio, config, key, error) {
    const errorSnapshot = luvio.errorSnapshot(error, {
        config,
        resolve: () => buildNetworkSnapshot$N(luvio, config),
    });
    luvio.storeIngestError(key, errorSnapshot);
    return luvio.storeBroadcast().then(() => errorSnapshot);
}
function prepareRequest$4(luvio, config) {
    const { recordTypeId, layoutType, mode, objectApiName } = config;
    const key = keyBuilder$3j(luvio, {
        apiName: objectApiName,
        recordTypeId,
        layoutType,
        mode,
    });
    const resourceParams = {
        urlParams: { objectApiName: config.objectApiName },
        queryParams: {
            layoutType: config.layoutType,
            mode: config.mode,
            recordTypeId: config.recordTypeId,
        },
    };
    const request = createResourceRequest$P(resourceParams);
    return { resourceParams, request, key };
}

function validateAdapterConfig$O(untrustedConfig, configPropertyNames) {
    const config = validateAdapterConfig$N(untrustedConfig, configPropertyNames);
    if (config === null) {
        return null;
    }
    // recordTypeId is overridden to be required
    const recordTypeId = config.recordTypeId;
    const untrusted = untrustedConfig;
    let layoutType = config.layoutType;
    if (layoutType === undefined) {
        if (untrusted.layoutType === undefined) {
            layoutType = LayoutType.Full;
        }
        else {
            return null;
        }
    }
    let mode = config.mode;
    if (mode === undefined) {
        if (untrusted.mode === undefined) {
            mode = LayoutMode.View;
        }
        else {
            return null;
        }
    }
    return {
        ...config,
        recordTypeId,
        layoutType,
        mode,
    };
}

// FYI stricter required set than RAML defines, matches lds222 behavior
const getLayoutUserState_ConfigPropertyNames = {
    displayName: 'getLayoutUserState',
    parameters: {
        required: ['objectApiName', 'recordTypeId'],
        optional: ['formFactor', 'layoutType', 'mode'],
    },
};

function buildCachedSnapshotCachePolicy$y(context, storeLookup) {
    const { config, luvio } = context;
    const { objectApiName, recordTypeId, layoutType, mode } = config;
    const selector = {
        recordId: keyBuilder$3j(luvio, {
            apiName: objectApiName,
            recordTypeId,
            layoutType,
            mode,
        }),
        node: select$3b(),
        variables: {},
    };
    return storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$N(luvio, config),
    });
}

const getLayoutUserState_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('layoutType', false, 1 /* QueryParameter */, 0 /* String */, false, coerceLayoutType),
    generateParamConfigMetadata('mode', false, 1 /* QueryParameter */, 0 /* String */, false, coerceLayoutMode),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */, false, getRecordId18),
];
function typeCheckConfig$M(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getLayoutUserState_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$N(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getLayoutUserState_ConfigPropertyMetadata);
    const config = typeCheckConfig$M(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshotCachePolicy$z(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$N, undefined, false);
}
const getLayoutUserStateAdapterFactory = (luvio) => function UiApi__getLayoutUserState(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$O(untrustedConfig, getLayoutUserState_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$y, buildNetworkSnapshotCachePolicy$z);
};

function keyBuilder$2B(luvio, params) {
    return keyBuilder$3F(luvio, {
        objectApiName: params.urlParams.objectApiName.toLowerCase(),
        listViewApiName: params.urlParams.listViewApiName.toLowerCase(),
        type: params.urlParams.listViewApiName.toLowerCase() === '__recent' ? 'mru' : 'listView',
    });
}

function select$2z(luvio, params) {
    return select$3t();
}
function getResponseCacheKeys$J(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2n(storeKeyMap, luvio, response);
}
function ingestSuccess$B(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2B(luvio, resourceParams);
    luvio.storeIngest(key, ingest$2j, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2z(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$s(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2B(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$H,
        namespace: keyPrefix,
        version: VERSION$2O,
        representationName: RepresentationType$11
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$O(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-info/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

function createChildResourceParams$8(resourceParams) {
    const childConfigs = [];
    const { queryParams } = resourceParams;
    const { names = [] } = queryParams;
    for (const name of names) {
        const [objectApiName, listViewApiName] = name.split('.');
        childConfigs.push({
            urlParams: {
                objectApiName,
                listViewApiName,
            },
        });
    }
    return childConfigs;
}

const nonCachedErrors$6 = ObjectCreate$1(null);
function createChildResourceParams$7(resourceParams) {
    return createChildResourceParams$8(resourceParams);
}
function selectChildResourceParams$6(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$2A(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$2B(luvio, childResource);
                const childFragment = select$2z();
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$6[childKey];
                            if (nonCachedError === undefined || nonCachedError.expiration < reader.getTimeStamp()) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$2y(luvio, resourceParams) {
    const childResources = createChildResourceParams$7(resourceParams);
    return selectChildResourceParams$6(luvio, childResources, resourceParams);
}
function keyBuilder$2A(luvio, params) {
    return keyPrefix + '::ListInfoBatchRepresentation:(' + 'ids:' + params.queryParams.ids + ',' + 'names:' + params.queryParams.names + ')';
}
function getResponseCacheKeys$I(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$7(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$J(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$2B(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$11,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$7(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$2B(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$B(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$s(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$6[childKey] = { expiration: now + TTL$H, response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$6[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$A(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$7(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$2A(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$7(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$2y(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$r(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2A(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$N(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-info/batch',
        method: 'get',
        body: null,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$E = 'getListInfosByName';
const getListInfosByName_ConfigPropertyMetadata = [
    generateParamConfigMetadata('ids', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('names', false, 1 /* QueryParameter */, 0 /* String */, true),
];
const getListInfosByName_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$E, getListInfosByName_ConfigPropertyMetadata);
const createResourceParams$I = /*#__PURE__*/ createResourceParams$k(getListInfosByName_ConfigPropertyMetadata);
function keyBuilder$2z(luvio, config) {
    const resourceParams = createResourceParams$I(config);
    return keyBuilder$2A(luvio, resourceParams);
}
function typeCheckConfig$L(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListInfosByName_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$M(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$L(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$s(luvio, config) {
    const resourceParams = createResourceParams$I(config);
    return select$2y(luvio, resourceParams);
}
function onFetchResponseSuccess$u(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$A(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$M(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$u(luvio, config, resourceParams, response) {
    const snapshot = ingestError$r(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$M(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$M(luvio, config, options) {
    const resourceParams = createResourceParams$I(config);
    const request = createResourceRequest$N(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$u(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$I(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$u(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$y(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$M, undefined, false);
}
function buildCachedSnapshotCachePolicy$x(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2z(luvio, config),
        node: adapterFragment$s(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$M(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getListInfosByNameAdapterFactory = (luvio) => function UiApi__getListInfosByName(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$M(untrustedConfig, getListInfosByName_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$x, buildNetworkSnapshotCachePolicy$y);
};

function createPaginationParams$1(params) {
    const { queryParams } = params;
    return {
        token: queryParams.pageToken,
        pageSize: queryParams.pageSize === undefined ? 20 : queryParams.pageSize
    };
}
function select$2x(luvio, params) {
    const paginationParams = createPaginationParams$1(params);
    return select$3j(paginationParams);
}
function keyBuilder$2y(luvio, params) {
    return keyBuilder$3u(luvio, {
        objectApiName: params.urlParams.objectApiName,
        queryString: params.queryParams.q || null,
        recentListsOnly: params.queryParams.recentListsOnly || false
    });
}
function getResponseCacheKeys$H(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2f(storeKeyMap, luvio, response);
}
function ingestSuccess$z(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2y(luvio, resourceParams);
    luvio.storeIngest(key, ingest$2c, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2x(luvio, resourceParams),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$q(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2y(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$M(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-info/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$D = 'getListInfosByObjectName';
const getListInfosByObjectName_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('q', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('recentListsOnly', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
const getListInfosByObjectName_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$D, getListInfosByObjectName_ConfigPropertyMetadata);
const createResourceParams$H = /*#__PURE__*/ createResourceParams$k(getListInfosByObjectName_ConfigPropertyMetadata);
function keyBuilder$2x(luvio, config) {
    const resourceParams = createResourceParams$H(config);
    return keyBuilder$2y(luvio, resourceParams);
}
function typeCheckConfig$K(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListInfosByObjectName_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$L(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$K(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$r(luvio, config) {
    const resourceParams = createResourceParams$H(config);
    return select$2x(luvio, resourceParams);
}
function onFetchResponseSuccess$t(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$z(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$L(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$t(luvio, config, resourceParams, response) {
    const snapshot = ingestError$q(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$L(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function prepareRequest$3(luvio, config, resourceParams, cacheSnapshot) {
    if (cacheSnapshot === undefined) {
        return createResourceRequest$M(resourceParams);
    }
    const [newToken, newPageSize] = minimizePaginationParams$1(luvio, resourceParams, cacheSnapshot);
    if (newToken === undefined) {
        return createResourceRequest$M(resourceParams);
    }
    return createResourceRequest$M(createResourceParams$H({
        ...config,
        pageSize: newPageSize,
        pageToken: newToken
    }));
}
function minimizePaginationParams$1(luvio, resourceParams, cacheSnapshot) {
    const num = getCachedItemsNumber$2(cacheSnapshot);
    if (num > 0) {
        const paginationMetadata = getPaginationMetadata$2(luvio, resourceParams);
        if (paginationMetadata !== undefined) {
            const { token: configToken, pageSize: configPageSize } = createPaginationParams$1(resourceParams);
            const configTokenOffset = offsetFromToken(paginationMetadata, configToken);
            if (configTokenOffset < 0) {
                return [undefined, -1];
            }
            const maxOffset = configTokenOffset + num;
            const [token, offset] = tokenForAtMost(paginationMetadata, maxOffset);
            if (token !== undefined) {
                return [token, configPageSize - (offset - configTokenOffset)];
            }
        }
    }
    return [undefined, -1];
}
function getCachedItemsNumber$2(cacheSnapshot) {
    const { data } = cacheSnapshot;
    if (data === undefined || !data.lists) {
        return 0;
    }
    const items = data.lists;
    for (let i = 0; i < items.length; i++) {
        if (items[i] === undefined) {
            return i;
        }
    }
    return items.length;
}
function getPaginationMetadata$2(luvio, resourceParams) {
    const key = keyBuilder$2y(luvio, resourceParams);
    const selector = {
        recordId: key,
        node: {
            kind: 'Fragment',
            version: VERSION$2F,
            private: [],
            selections: [
                {
                    kind: 'Link',
                    name: '__metadata',
                    fragment: {
                        kind: 'Fragment',
                        version: PAGINATION_VERSION$1,
                        opaque: true,
                        private: [],
                    },
                },
            ],
        },
        variables: {}
    };
    const node = luvio.storeLookup(selector);
    if (node.state !== 'Fulfilled') {
        return;
    }
    return node.data.__metadata;
}
function buildNetworkSnapshot$L(luvio, config, options, cacheSnapshot) {
    const resourceParams = createResourceParams$H(config);
    const request = prepareRequest$3(luvio, config, resourceParams, cacheSnapshot);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$t(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$H(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$t(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$x(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$L, undefined, true);
}
function buildCachedSnapshotCachePolicy$w(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2x(luvio, config),
        node: adapterFragment$r(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$L(luvio, config, snapshotRefreshOptions)
    });
    if (isUnfulfilledSnapshot$1(cacheSnapshot) && cacheSnapshot.data !== undefined) {
        context.cacheSnapshot = cacheSnapshot;
    }
    return cacheSnapshot;
}
const getListInfosByObjectNameAdapterFactory = (luvio) => function UiApi__getListInfosByObjectName(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$L(untrustedConfig, getListInfosByObjectName_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$w, buildNetworkSnapshotCachePolicy$x);
};

function validate$1e(obj, path = 'ListFilterByInfoInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_operandLabels = obj.operandLabels;
        const path_operandLabels = path + '.operandLabels';
        if (!ArrayIsArray(obj_operandLabels)) {
            return new TypeError('Expected "array" but received "' + typeof obj_operandLabels + '" (at "' + path_operandLabels + '")');
        }
        for (let i = 0; i < obj_operandLabels.length; i++) {
            const obj_operandLabels_item = obj_operandLabels[i];
            const path_operandLabels_item = path_operandLabels + '[' + i + ']';
            if (typeof obj_operandLabels_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_operandLabels_item + '" (at "' + path_operandLabels_item + '")');
            }
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$1d(obj, path = 'ListScopeInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        if (obj.entityId !== undefined) {
            const obj_entityId = obj.entityId;
            const path_entityId = path + '.entityId';
            let obj_entityId_union0 = null;
            const obj_entityId_union0_error = (() => {
                if (typeof obj_entityId !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_entityId + '" (at "' + path_entityId + '")');
                }
            })();
            if (obj_entityId_union0_error != null) {
                obj_entityId_union0 = obj_entityId_union0_error.message;
            }
            let obj_entityId_union1 = null;
            const obj_entityId_union1_error = (() => {
                if (obj_entityId !== null) {
                    return new TypeError('Expected "null" but received "' + typeof obj_entityId + '" (at "' + path_entityId + '")');
                }
            })();
            if (obj_entityId_union1_error != null) {
                obj_entityId_union1 = obj_entityId_union1_error.message;
            }
            if (obj_entityId_union0 && obj_entityId_union1) {
                let message = 'Object doesn\'t match union (at "' + path_entityId + '")';
                message += '\n' + obj_entityId_union0.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_entityId_union1.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        if (obj.relatedEntityId !== undefined) {
            const obj_relatedEntityId = obj.relatedEntityId;
            const path_relatedEntityId = path + '.relatedEntityId';
            let obj_relatedEntityId_union0 = null;
            const obj_relatedEntityId_union0_error = (() => {
                if (typeof obj_relatedEntityId !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_relatedEntityId + '" (at "' + path_relatedEntityId + '")');
                }
            })();
            if (obj_relatedEntityId_union0_error != null) {
                obj_relatedEntityId_union0 = obj_relatedEntityId_union0_error.message;
            }
            let obj_relatedEntityId_union1 = null;
            const obj_relatedEntityId_union1_error = (() => {
                if (obj_relatedEntityId !== null) {
                    return new TypeError('Expected "null" but received "' + typeof obj_relatedEntityId + '" (at "' + path_relatedEntityId + '")');
                }
            })();
            if (obj_relatedEntityId_union1_error != null) {
                obj_relatedEntityId_union1 = obj_relatedEntityId_union1_error.message;
            }
            if (obj_relatedEntityId_union0 && obj_relatedEntityId_union1) {
                let message = 'Object doesn\'t match union (at "' + path_relatedEntityId + '")';
                message += '\n' + obj_relatedEntityId_union0.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_relatedEntityId_union1.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function select$2w(luvio, params) {
    return select$3t();
}
function getResponseCacheKeys$G(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2n(storeKeyMap, luvio, response);
}
function ingestSuccess$y(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType$F(luvio, body);
    luvio.storeIngest(key, ingest$2j, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2w(),
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function createResourceRequest$L(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-info/' + config.urlParams.objectApiName + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$C = 'createListInfo';
const createListInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('displayColumns', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('filterLogicString', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('filteredByInfo', false, 2 /* Body */, 4 /* Unsupported */, true),
    generateParamConfigMetadata('includeDefaultAttributes', false, 2 /* Body */, 1 /* Boolean */),
    generateParamConfigMetadata('label', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('listShares', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('listViewApiName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('scope', false, 2 /* Body */, 4 /* Unsupported */),
    generateParamConfigMetadata('visibility', false, 2 /* Body */, 0 /* String */),
];
const createListInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$C, createListInfo_ConfigPropertyMetadata);
const createResourceParams$G = /*#__PURE__*/ createResourceParams$k(createListInfo_ConfigPropertyMetadata);
function typeCheckConfig$J(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, createListInfo_ConfigPropertyMetadata);
    const untrustedConfig_filteredByInfo = untrustedConfig.filteredByInfo;
    if (ArrayIsArray$1(untrustedConfig_filteredByInfo)) {
        const untrustedConfig_filteredByInfo_array = [];
        for (let i = 0, arrayLength = untrustedConfig_filteredByInfo.length; i < arrayLength; i++) {
            const untrustedConfig_filteredByInfo_item = untrustedConfig_filteredByInfo[i];
            const referenceListFilterByInfoInputRepresentationValidationError = validate$1e(untrustedConfig_filteredByInfo_item);
            if (referenceListFilterByInfoInputRepresentationValidationError === null) {
                untrustedConfig_filteredByInfo_array.push(untrustedConfig_filteredByInfo_item);
            }
        }
        config.filteredByInfo = untrustedConfig_filteredByInfo_array;
    }
    const untrustedConfig_scope = untrustedConfig.scope;
    const referenceListScopeInputRepresentationValidationError = validate$1d(untrustedConfig_scope);
    if (referenceListScopeInputRepresentationValidationError === null) {
        config.scope = untrustedConfig_scope;
    }
    return config;
}
function validateAdapterConfig$K(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, createListInfo_ConfigPropertyMetadata);
    const config = typeCheckConfig$J(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$K(luvio, config, options) {
    const resourceParams = createResourceParams$G(config);
    const request = createResourceRequest$L(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$y(luvio, resourceParams, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$G(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const createListInfoAdapterFactory = (luvio) => {
    return function createListInfo(untrustedConfig) {
        const config = validateAdapterConfig$K(untrustedConfig, createListInfo_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            throw new Error('Invalid config for "createListInfo"');
        }
        return buildNetworkSnapshot$K(luvio, config);
    };
};

function keyBuilder$2w(luvio, params) {
    return keyBuilder$3F(luvio, {
        objectApiName: params.urlParams.objectApiName,
        listViewApiName: params.urlParams.listViewApiName,
        type: "listView"
    });
}
function getResponseCacheKeys$F(cacheKeyMap, luvio, resourceParams) {
    const key = keyBuilder$2w(luvio, resourceParams);
    cacheKeyMap.set(key, {
        namespace: keyPrefix,
        representationName: RepresentationType$11,
        mergeable: false
    });
}
function evictSuccess$1(luvio, resourceParams) {
    const key = keyBuilder$2w(luvio, resourceParams);
    luvio.storeEvict(key);
}
function createResourceRequest$K(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-info/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'delete',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$B = 'deleteListInfo';
const deleteListInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
];
const deleteListInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$B, deleteListInfo_ConfigPropertyMetadata);
const createResourceParams$F = /*#__PURE__*/ createResourceParams$k(deleteListInfo_ConfigPropertyMetadata);
function typeCheckConfig$I(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, deleteListInfo_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$J(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$I(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$J(luvio, config, options) {
    const resourceParams = createResourceParams$F(config);
    const request = createResourceRequest$K(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then(() => {
        return luvio.handleSuccessResponse(() => {
            evictSuccess$1(luvio, resourceParams);
            return luvio.storeBroadcast();
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$F(cache, luvio, resourceParams);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const deleteListInfoAdapterFactory = (luvio) => {
    return function UiApideleteListInfo(untrustedConfig) {
        const config = validateAdapterConfig$J(untrustedConfig, deleteListInfo_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            throw new Error(`Invalid config for "${adapterName$B}"`);
        }
        return buildNetworkSnapshot$J(luvio, config);
    };
};

const adapterName$A = 'getListInfoByName';
const getListInfoByName_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
];
const getListInfoByName_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$A, getListInfoByName_ConfigPropertyMetadata);
const createResourceParams$E = /*#__PURE__*/ createResourceParams$k(getListInfoByName_ConfigPropertyMetadata);
function keyBuilder$2v(luvio, config) {
    const resourceParams = createResourceParams$E(config);
    return keyBuilder$2B(luvio, resourceParams);
}
function typeCheckConfig$H(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListInfoByName_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$I(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$H(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$q(luvio, config) {
    createResourceParams$E(config);
    return select$2z();
}
function onFetchResponseSuccess$s(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$B(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$I(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$s(luvio, config, resourceParams, response) {
    const snapshot = ingestError$s(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$I(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$I(luvio, config, options) {
    const resourceParams = createResourceParams$E(config);
    const request = createResourceRequest$O(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$s(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$J(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$s(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$w(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$I, undefined, false);
}
function buildCachedSnapshotCachePolicy$v(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2v(luvio, config),
        node: adapterFragment$q(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$I(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getListInfoByNameAdapterFactory = (luvio) => function UiApi__getListInfoByName(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$I(untrustedConfig, getListInfoByName_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$v, buildNetworkSnapshotCachePolicy$w);
};

function select$2v(luvio, params) {
    return select$3t();
}
function getResponseCacheKeys$E(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2n(storeKeyMap, luvio, response);
}
function ingestSuccess$x(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType$F(luvio, body);
    luvio.storeIngest(key, ingest$2j, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2v(),
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function createResourceRequest$J(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-info/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'patch',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$z = 'updateListInfoByName';
const updateListInfoByName_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('displayColumns', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('filterLogicString', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('filteredByInfo', false, 2 /* Body */, 4 /* Unsupported */, true),
    generateParamConfigMetadata('label', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('listShares', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('scope', false, 2 /* Body */, 4 /* Unsupported */),
    generateParamConfigMetadata('visibility', false, 2 /* Body */, 0 /* String */),
];
const updateListInfoByName_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$z, updateListInfoByName_ConfigPropertyMetadata);
const createResourceParams$D = /*#__PURE__*/ createResourceParams$k(updateListInfoByName_ConfigPropertyMetadata);
function typeCheckConfig$G(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, updateListInfoByName_ConfigPropertyMetadata);
    const untrustedConfig_filteredByInfo = untrustedConfig.filteredByInfo;
    if (ArrayIsArray$1(untrustedConfig_filteredByInfo)) {
        const untrustedConfig_filteredByInfo_array = [];
        for (let i = 0, arrayLength = untrustedConfig_filteredByInfo.length; i < arrayLength; i++) {
            const untrustedConfig_filteredByInfo_item = untrustedConfig_filteredByInfo[i];
            const referenceListFilterByInfoInputRepresentationValidationError = validate$1e(untrustedConfig_filteredByInfo_item);
            if (referenceListFilterByInfoInputRepresentationValidationError === null) {
                untrustedConfig_filteredByInfo_array.push(untrustedConfig_filteredByInfo_item);
            }
        }
        config.filteredByInfo = untrustedConfig_filteredByInfo_array;
    }
    const untrustedConfig_scope = untrustedConfig.scope;
    const referenceListScopeInputRepresentationValidationError = validate$1d(untrustedConfig_scope);
    if (referenceListScopeInputRepresentationValidationError === null) {
        config.scope = untrustedConfig_scope;
    }
    return config;
}
function validateAdapterConfig$H(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, updateListInfoByName_ConfigPropertyMetadata);
    const config = typeCheckConfig$G(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$H(luvio, config, options) {
    const resourceParams = createResourceParams$D(config);
    const request = createResourceRequest$J(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$x(luvio, resourceParams, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$E(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const updateListInfoByNameAdapterFactory = (luvio) => {
    return function updateListInfoByName(untrustedConfig) {
        const config = validateAdapterConfig$H(untrustedConfig, updateListInfoByName_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            throw new Error('Invalid config for "updateListInfoByName"');
        }
        return buildNetworkSnapshot$H(luvio, config);
    };
};

const VERSION$2a = "d77026c4d63ef00cdd7e3d600687a40d";
function validate$1c(obj, path = 'ListObjectScopeAvailableEntityRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (typeof obj_id !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2u = function ListObjectScopeAvailableEntityRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$2a,
        private: [],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$G(existing, incoming) {
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    return true;
}

const VERSION$29 = "355933ff2970aa91a2848a94565e956a";
function validate$1b(obj, path = 'ListObjectScopeRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_availableEntities = obj.availableEntities;
        const path_availableEntities = path + '.availableEntities';
        if (!ArrayIsArray(obj_availableEntities)) {
            return new TypeError('Expected "array" but received "' + typeof obj_availableEntities + '" (at "' + path_availableEntities + '")');
        }
        for (let i = 0; i < obj_availableEntities.length; i++) {
            const obj_availableEntities_item = obj_availableEntities[i];
            const path_availableEntities_item = path_availableEntities + '[' + i + ']';
            const referencepath_availableEntities_itemValidationError = validate$1c(obj_availableEntities_item, path_availableEntities_item);
            if (referencepath_availableEntities_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListObjectScopeAvailableEntityRepresentation (at "' + path_availableEntities_item + '")\n';
                message += referencepath_availableEntities_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2t = function ListObjectScopeRepresentationSelect() {
    const { selections: ListObjectScopeAvailableEntityRepresentation__selections, opaque: ListObjectScopeAvailableEntityRepresentation__opaque, } = select$2u();
    return {
        kind: 'Fragment',
        version: VERSION$29,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'availableEntities',
                kind: 'Object',
                plural: true,
                selections: ListObjectScopeAvailableEntityRepresentation__selections
            },
            {
                name: 'label',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$F(existing, incoming) {
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_availableEntities = existing.availableEntities;
    const incoming_availableEntities = incoming.availableEntities;
    const equals_availableEntities_items = equalsArray(existing_availableEntities, incoming_availableEntities, (existing_availableEntities_item, incoming_availableEntities_item) => {
        if (!(equals$G(existing_availableEntities_item, incoming_availableEntities_item))) {
            return false;
        }
    });
    if (equals_availableEntities_items === false) {
        return false;
    }
    return true;
}

const VERSION$28 = "c5c08ea834378670c9d6f235ff71c9f0";
function validate$1a(obj, path = 'ListObjectPicklistValueRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2s = function ListObjectPicklistValueRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$28,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$E(existing, incoming) {
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    return true;
}

const VERSION$27 = "98f8e97eae828e3f165bd05c518ae8df";
function validate$19(obj, path = 'ListObjectQuickFilterOverridesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_dataType = obj.dataType;
        const path_dataType = path + '.dataType';
        let obj_dataType_union0 = null;
        const obj_dataType_union0_error = (() => {
            if (typeof obj_dataType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
            }
        })();
        if (obj_dataType_union0_error != null) {
            obj_dataType_union0 = obj_dataType_union0_error.message;
        }
        let obj_dataType_union1 = null;
        const obj_dataType_union1_error = (() => {
            if (obj_dataType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
            }
        })();
        if (obj_dataType_union1_error != null) {
            obj_dataType_union1 = obj_dataType_union1_error.message;
        }
        if (obj_dataType_union0 && obj_dataType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_dataType + '")';
            message += '\n' + obj_dataType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_dataType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_picklistValues = obj.picklistValues;
        const path_picklistValues = path + '.picklistValues';
        if (!ArrayIsArray(obj_picklistValues)) {
            return new TypeError('Expected "array" but received "' + typeof obj_picklistValues + '" (at "' + path_picklistValues + '")');
        }
        for (let i = 0; i < obj_picklistValues.length; i++) {
            const obj_picklistValues_item = obj_picklistValues[i];
            const path_picklistValues_item = path_picklistValues + '[' + i + ']';
            if (typeof obj_picklistValues_item !== 'object' || ArrayIsArray(obj_picklistValues_item) || obj_picklistValues_item === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_picklistValues_item + '" (at "' + path_picklistValues_item + '")');
            }
            const obj_picklistValues_item_keys = ObjectKeys(obj_picklistValues_item);
            for (let i = 0; i < obj_picklistValues_item_keys.length; i++) {
                const key = obj_picklistValues_item_keys[i];
                const obj_picklistValues_item_prop = obj_picklistValues_item[key];
                const path_picklistValues_item_prop = path_picklistValues_item + '["' + key + '"]';
                if (typeof obj_picklistValues_item_prop !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_picklistValues_item_prop + '" (at "' + path_picklistValues_item_prop + '")');
                }
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2r = function ListObjectQuickFilterOverridesRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$27,
        private: [],
        selections: [
            {
                name: 'dataType',
                kind: 'Scalar'
            },
            {
                name: 'picklistValues',
                kind: 'Scalar',
                map: true
            }
        ]
    };
};
function equals$D(existing, incoming) {
    const existing_dataType = existing.dataType;
    const incoming_dataType = incoming.dataType;
    if (!(existing_dataType === incoming_dataType)) {
        return false;
    }
    const existing_picklistValues = existing.picklistValues;
    const incoming_picklistValues = incoming.picklistValues;
    const equals_picklistValues_items = equalsArray(existing_picklistValues, incoming_picklistValues, (existing_picklistValues_item, incoming_picklistValues_item) => {
        const equals_picklistValues_items_props = equalsObject(existing_picklistValues_item, incoming_picklistValues_item, (existing_picklistValues_item_prop, incoming_picklistValues_item_prop) => {
            if (!(existing_picklistValues_item_prop === incoming_picklistValues_item_prop)) {
                return false;
            }
        });
        if (equals_picklistValues_items_props === false) {
            return false;
        }
    });
    if (equals_picklistValues_items === false) {
        return false;
    }
    return true;
}

const VERSION$26 = "394e8d71b515892fe366021cc2872dae";
function validate$18(obj, path = 'ListObjectColumnRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_alias = obj.alias;
        const path_alias = path + '.alias';
        if (typeof obj_alias !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_alias + '" (at "' + path_alias + '")');
        }
        const obj_dataType = obj.dataType;
        const path_dataType = path + '.dataType';
        let obj_dataType_union0 = null;
        const obj_dataType_union0_error = (() => {
            if (typeof obj_dataType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
            }
        })();
        if (obj_dataType_union0_error != null) {
            obj_dataType_union0 = obj_dataType_union0_error.message;
        }
        let obj_dataType_union1 = null;
        const obj_dataType_union1_error = (() => {
            if (obj_dataType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
            }
        })();
        if (obj_dataType_union1_error != null) {
            obj_dataType_union1 = obj_dataType_union1_error.message;
        }
        if (obj_dataType_union0 && obj_dataType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_dataType + '")';
            message += '\n' + obj_dataType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_dataType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_defaultFilterOperator = obj.defaultFilterOperator;
        const path_defaultFilterOperator = path + '.defaultFilterOperator';
        if (typeof obj_defaultFilterOperator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_defaultFilterOperator + '" (at "' + path_defaultFilterOperator + '")');
        }
        const obj_displayable = obj.displayable;
        const path_displayable = path + '.displayable';
        if (typeof obj_displayable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_displayable + '" (at "' + path_displayable + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_filterable = obj.filterable;
        const path_filterable = path + '.filterable';
        if (typeof obj_filterable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_filterable + '" (at "' + path_filterable + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_picklistValues = obj.picklistValues;
        const path_picklistValues = path + '.picklistValues';
        if (!ArrayIsArray(obj_picklistValues)) {
            return new TypeError('Expected "array" but received "' + typeof obj_picklistValues + '" (at "' + path_picklistValues + '")');
        }
        for (let i = 0; i < obj_picklistValues.length; i++) {
            const obj_picklistValues_item = obj_picklistValues[i];
            const path_picklistValues_item = path_picklistValues + '[' + i + ']';
            const referencepath_picklistValues_itemValidationError = validate$1a(obj_picklistValues_item, path_picklistValues_item);
            if (referencepath_picklistValues_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListObjectPicklistValueRepresentation (at "' + path_picklistValues_item + '")\n';
                message += referencepath_picklistValues_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_quickFilterOperator = obj.quickFilterOperator;
        const path_quickFilterOperator = path + '.quickFilterOperator';
        let obj_quickFilterOperator_union0 = null;
        const obj_quickFilterOperator_union0_error = (() => {
            if (typeof obj_quickFilterOperator !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_quickFilterOperator + '" (at "' + path_quickFilterOperator + '")');
            }
        })();
        if (obj_quickFilterOperator_union0_error != null) {
            obj_quickFilterOperator_union0 = obj_quickFilterOperator_union0_error.message;
        }
        let obj_quickFilterOperator_union1 = null;
        const obj_quickFilterOperator_union1_error = (() => {
            if (obj_quickFilterOperator !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_quickFilterOperator + '" (at "' + path_quickFilterOperator + '")');
            }
        })();
        if (obj_quickFilterOperator_union1_error != null) {
            obj_quickFilterOperator_union1 = obj_quickFilterOperator_union1_error.message;
        }
        if (obj_quickFilterOperator_union0 && obj_quickFilterOperator_union1) {
            let message = 'Object doesn\'t match union (at "' + path_quickFilterOperator + '")';
            message += '\n' + obj_quickFilterOperator_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_quickFilterOperator_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_quickFilterOverrides = obj.quickFilterOverrides;
        const path_quickFilterOverrides = path + '.quickFilterOverrides';
        let obj_quickFilterOverrides_union0 = null;
        const obj_quickFilterOverrides_union0_error = (() => {
            const referencepath_quickFilterOverridesValidationError = validate$19(obj_quickFilterOverrides, path_quickFilterOverrides);
            if (referencepath_quickFilterOverridesValidationError !== null) {
                let message = 'Object doesn\'t match ListObjectQuickFilterOverridesRepresentation (at "' + path_quickFilterOverrides + '")\n';
                message += referencepath_quickFilterOverridesValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_quickFilterOverrides_union0_error != null) {
            obj_quickFilterOverrides_union0 = obj_quickFilterOverrides_union0_error.message;
        }
        let obj_quickFilterOverrides_union1 = null;
        const obj_quickFilterOverrides_union1_error = (() => {
            if (obj_quickFilterOverrides !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_quickFilterOverrides + '" (at "' + path_quickFilterOverrides + '")');
            }
        })();
        if (obj_quickFilterOverrides_union1_error != null) {
            obj_quickFilterOverrides_union1 = obj_quickFilterOverrides_union1_error.message;
        }
        if (obj_quickFilterOverrides_union0 && obj_quickFilterOverrides_union1) {
            let message = 'Object doesn\'t match union (at "' + path_quickFilterOverrides + '")';
            message += '\n' + obj_quickFilterOverrides_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_quickFilterOverrides_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_quickFilterable = obj.quickFilterable;
        const path_quickFilterable = path + '.quickFilterable';
        if (typeof obj_quickFilterable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_quickFilterable + '" (at "' + path_quickFilterable + '")');
        }
        const obj_sortable = obj.sortable;
        const path_sortable = path + '.sortable';
        if (typeof obj_sortable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_sortable + '" (at "' + path_sortable + '")');
        }
        const obj_supportedFilterOperators = obj.supportedFilterOperators;
        const path_supportedFilterOperators = path + '.supportedFilterOperators';
        if (!ArrayIsArray(obj_supportedFilterOperators)) {
            return new TypeError('Expected "array" but received "' + typeof obj_supportedFilterOperators + '" (at "' + path_supportedFilterOperators + '")');
        }
        for (let i = 0; i < obj_supportedFilterOperators.length; i++) {
            const obj_supportedFilterOperators_item = obj_supportedFilterOperators[i];
            const path_supportedFilterOperators_item = path_supportedFilterOperators + '[' + i + ']';
            if (typeof obj_supportedFilterOperators_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_supportedFilterOperators_item + '" (at "' + path_supportedFilterOperators_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2q = function ListObjectColumnRepresentationSelect() {
    const { selections: ListObjectPicklistValueRepresentation__selections, opaque: ListObjectPicklistValueRepresentation__opaque, } = select$2s();
    const { selections: ListObjectQuickFilterOverridesRepresentation__selections, opaque: ListObjectQuickFilterOverridesRepresentation__opaque, } = select$2r();
    return {
        kind: 'Fragment',
        version: VERSION$26,
        private: [],
        selections: [
            {
                name: 'alias',
                kind: 'Scalar'
            },
            {
                name: 'dataType',
                kind: 'Scalar'
            },
            {
                name: 'defaultFilterOperator',
                kind: 'Scalar'
            },
            {
                name: 'displayable',
                kind: 'Scalar'
            },
            {
                name: 'fieldApiName',
                kind: 'Scalar'
            },
            {
                name: 'filterable',
                kind: 'Scalar'
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'picklistValues',
                kind: 'Object',
                plural: true,
                selections: ListObjectPicklistValueRepresentation__selections
            },
            {
                name: 'quickFilterOperator',
                kind: 'Scalar'
            },
            {
                name: 'quickFilterOverrides',
                kind: 'Object',
                nullable: true,
                selections: ListObjectQuickFilterOverridesRepresentation__selections
            },
            {
                name: 'quickFilterable',
                kind: 'Scalar'
            },
            {
                name: 'sortable',
                kind: 'Scalar'
            },
            {
                name: 'supportedFilterOperators',
                kind: 'Scalar',
                plural: true
            }
        ]
    };
};
function equals$C(existing, incoming) {
    const existing_displayable = existing.displayable;
    const incoming_displayable = incoming.displayable;
    if (!(existing_displayable === incoming_displayable)) {
        return false;
    }
    const existing_filterable = existing.filterable;
    const incoming_filterable = incoming.filterable;
    if (!(existing_filterable === incoming_filterable)) {
        return false;
    }
    const existing_quickFilterable = existing.quickFilterable;
    const incoming_quickFilterable = incoming.quickFilterable;
    if (!(existing_quickFilterable === incoming_quickFilterable)) {
        return false;
    }
    const existing_sortable = existing.sortable;
    const incoming_sortable = incoming.sortable;
    if (!(existing_sortable === incoming_sortable)) {
        return false;
    }
    const existing_alias = existing.alias;
    const incoming_alias = incoming.alias;
    if (!(existing_alias === incoming_alias)) {
        return false;
    }
    const existing_defaultFilterOperator = existing.defaultFilterOperator;
    const incoming_defaultFilterOperator = incoming.defaultFilterOperator;
    if (!(existing_defaultFilterOperator === incoming_defaultFilterOperator)) {
        return false;
    }
    const existing_fieldApiName = existing.fieldApiName;
    const incoming_fieldApiName = incoming.fieldApiName;
    if (!(existing_fieldApiName === incoming_fieldApiName)) {
        return false;
    }
    const existing_label = existing.label;
    const incoming_label = incoming.label;
    if (!(existing_label === incoming_label)) {
        return false;
    }
    const existing_dataType = existing.dataType;
    const incoming_dataType = incoming.dataType;
    if (!(existing_dataType === incoming_dataType)) {
        return false;
    }
    const existing_picklistValues = existing.picklistValues;
    const incoming_picklistValues = incoming.picklistValues;
    const equals_picklistValues_items = equalsArray(existing_picklistValues, incoming_picklistValues, (existing_picklistValues_item, incoming_picklistValues_item) => {
        if (!(equals$E(existing_picklistValues_item, incoming_picklistValues_item))) {
            return false;
        }
    });
    if (equals_picklistValues_items === false) {
        return false;
    }
    const existing_quickFilterOperator = existing.quickFilterOperator;
    const incoming_quickFilterOperator = incoming.quickFilterOperator;
    if (!(existing_quickFilterOperator === incoming_quickFilterOperator)) {
        return false;
    }
    const existing_quickFilterOverrides = existing.quickFilterOverrides;
    const incoming_quickFilterOverrides = incoming.quickFilterOverrides;
    if (!(existing_quickFilterOverrides === incoming_quickFilterOverrides
        || (existing_quickFilterOverrides != null &&
            incoming_quickFilterOverrides != null &&
            equals$D(existing_quickFilterOverrides, incoming_quickFilterOverrides)))) {
        return false;
    }
    const existing_supportedFilterOperators = existing.supportedFilterOperators;
    const incoming_supportedFilterOperators = incoming.supportedFilterOperators;
    const equals_supportedFilterOperators_items = equalsArray(existing_supportedFilterOperators, incoming_supportedFilterOperators, (existing_supportedFilterOperators_item, incoming_supportedFilterOperators_item) => {
        if (!(existing_supportedFilterOperators_item === incoming_supportedFilterOperators_item)) {
            return false;
        }
    });
    if (equals_supportedFilterOperators_items === false) {
        return false;
    }
    return true;
}

const TTL$q = 900000;
const VERSION$25 = "2405a0b25c2c00f82e88b600edc16387";
function validate$17(obj, path = 'ListObjectInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_availableScopes = obj.availableScopes;
        const path_availableScopes = path + '.availableScopes';
        if (!ArrayIsArray(obj_availableScopes)) {
            return new TypeError('Expected "array" but received "' + typeof obj_availableScopes + '" (at "' + path_availableScopes + '")');
        }
        for (let i = 0; i < obj_availableScopes.length; i++) {
            const obj_availableScopes_item = obj_availableScopes[i];
            const path_availableScopes_item = path_availableScopes + '[' + i + ']';
            const referencepath_availableScopes_itemValidationError = validate$1b(obj_availableScopes_item, path_availableScopes_item);
            if (referencepath_availableScopes_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListObjectScopeRepresentation (at "' + path_availableScopes_item + '")\n';
                message += referencepath_availableScopes_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_columns = obj.columns;
        const path_columns = path + '.columns';
        if (!ArrayIsArray(obj_columns)) {
            return new TypeError('Expected "array" but received "' + typeof obj_columns + '" (at "' + path_columns + '")');
        }
        for (let i = 0; i < obj_columns.length; i++) {
            const obj_columns_item = obj_columns[i];
            const path_columns_item = path_columns + '[' + i + ']';
            const referencepath_columns_itemValidationError = validate$18(obj_columns_item, path_columns_item);
            if (referencepath_columns_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListObjectColumnRepresentation (at "' + path_columns_item + '")\n';
                message += referencepath_columns_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_createable = obj.createable;
        const path_createable = path + '.createable';
        if (typeof obj_createable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_createable + '" (at "' + path_createable + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_publicOrSharedCreateable = obj.publicOrSharedCreateable;
        const path_publicOrSharedCreateable = path + '.publicOrSharedCreateable';
        if (typeof obj_publicOrSharedCreateable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_publicOrSharedCreateable + '" (at "' + path_publicOrSharedCreateable + '")');
        }
        const obj_relatedEntityApiName = obj.relatedEntityApiName;
        const path_relatedEntityApiName = path + '.relatedEntityApiName';
        let obj_relatedEntityApiName_union0 = null;
        const obj_relatedEntityApiName_union0_error = (() => {
            if (typeof obj_relatedEntityApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_relatedEntityApiName + '" (at "' + path_relatedEntityApiName + '")');
            }
        })();
        if (obj_relatedEntityApiName_union0_error != null) {
            obj_relatedEntityApiName_union0 = obj_relatedEntityApiName_union0_error.message;
        }
        let obj_relatedEntityApiName_union1 = null;
        const obj_relatedEntityApiName_union1_error = (() => {
            if (obj_relatedEntityApiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_relatedEntityApiName + '" (at "' + path_relatedEntityApiName + '")');
            }
        })();
        if (obj_relatedEntityApiName_union1_error != null) {
            obj_relatedEntityApiName_union1 = obj_relatedEntityApiName_union1_error.message;
        }
        if (obj_relatedEntityApiName_union0 && obj_relatedEntityApiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_relatedEntityApiName + '")';
            message += '\n' + obj_relatedEntityApiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_relatedEntityApiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$z = 'ListObjectInfoRepresentation';
function keyBuilder$2u(luvio, config) {
    return keyPrefix + '::' + RepresentationType$z + ':' + config.objectApiName;
}
function keyBuilderFromType$k(luvio, object) {
    const keyParams = {
        objectApiName: object.objectApiName
    };
    return keyBuilder$2u(luvio, keyParams);
}
function normalize$s(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2p = function ListObjectInfoRepresentationSelect() {
    const { selections: ListObjectScopeRepresentation__selections, opaque: ListObjectScopeRepresentation__opaque, } = select$2t();
    const { selections: ListObjectColumnRepresentation__selections, opaque: ListObjectColumnRepresentation__opaque, } = select$2q();
    return {
        kind: 'Fragment',
        version: VERSION$25,
        private: [],
        selections: [
            {
                name: 'availableScopes',
                kind: 'Object',
                plural: true,
                selections: ListObjectScopeRepresentation__selections
            },
            {
                name: 'columns',
                kind: 'Object',
                plural: true,
                selections: ListObjectColumnRepresentation__selections
            },
            {
                name: 'createable',
                kind: 'Scalar'
            },
            {
                name: 'objectApiName',
                kind: 'Scalar'
            },
            {
                name: 'publicOrSharedCreateable',
                kind: 'Scalar'
            },
            {
                name: 'relatedEntityApiName',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$B(existing, incoming) {
    const existing_createable = existing.createable;
    const incoming_createable = incoming.createable;
    if (!(existing_createable === incoming_createable)) {
        return false;
    }
    const existing_publicOrSharedCreateable = existing.publicOrSharedCreateable;
    const incoming_publicOrSharedCreateable = incoming.publicOrSharedCreateable;
    if (!(existing_publicOrSharedCreateable === incoming_publicOrSharedCreateable)) {
        return false;
    }
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_availableScopes = existing.availableScopes;
    const incoming_availableScopes = incoming.availableScopes;
    const equals_availableScopes_items = equalsArray(existing_availableScopes, incoming_availableScopes, (existing_availableScopes_item, incoming_availableScopes_item) => {
        if (!(equals$F(existing_availableScopes_item, incoming_availableScopes_item))) {
            return false;
        }
    });
    if (equals_availableScopes_items === false) {
        return false;
    }
    const existing_columns = existing.columns;
    const incoming_columns = incoming.columns;
    const equals_columns_items = equalsArray(existing_columns, incoming_columns, (existing_columns_item, incoming_columns_item) => {
        if (!(equals$C(existing_columns_item, incoming_columns_item))) {
            return false;
        }
    });
    if (equals_columns_items === false) {
        return false;
    }
    const existing_relatedEntityApiName = existing.relatedEntityApiName;
    const incoming_relatedEntityApiName = incoming.relatedEntityApiName;
    if (!(existing_relatedEntityApiName === incoming_relatedEntityApiName)) {
        return false;
    }
    return true;
}
const ingest$1S = function ListObjectInfoRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$17(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$k(luvio, input);
    const ttlToUse = TTL$q;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$s, "UiApi", VERSION$25, RepresentationType$z, equals$B);
    return createLink$1(key);
};
function getTypeCacheKeys$1V(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$k(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$z,
        mergeable: false
    });
}

function select$2o(luvio, params) {
    return select$2p();
}
function keyBuilder$2t(luvio, params) {
    return keyBuilder$2u(luvio, {
        objectApiName: params.urlParams.objectApiName
    });
}
function getResponseCacheKeys$D(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1V(storeKeyMap, luvio, response);
}
function ingestSuccess$w(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2t(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1S, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2o(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$p(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2t(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$q,
        namespace: keyPrefix,
        version: VERSION$25,
        representationName: RepresentationType$z
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$I(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-object-info/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$y = 'getListObjectInfo';
const getListObjectInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
];
const getListObjectInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$y, getListObjectInfo_ConfigPropertyMetadata);
const createResourceParams$C = /*#__PURE__*/ createResourceParams$k(getListObjectInfo_ConfigPropertyMetadata);
function keyBuilder$2s(luvio, config) {
    const resourceParams = createResourceParams$C(config);
    return keyBuilder$2t(luvio, resourceParams);
}
function typeCheckConfig$F(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListObjectInfo_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$G(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getListObjectInfo_ConfigPropertyMetadata);
    const config = typeCheckConfig$F(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$p(luvio, config) {
    createResourceParams$C(config);
    return select$2o();
}
function onFetchResponseSuccess$r(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$w(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$G(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$r(luvio, config, resourceParams, response) {
    const snapshot = ingestError$p(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$G(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$G(luvio, config, options) {
    const resourceParams = createResourceParams$C(config);
    const request = createResourceRequest$I(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$r(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$D(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$r(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$v(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$G, undefined, false);
}
function buildCachedSnapshotCachePolicy$u(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2s(luvio, config),
        node: adapterFragment$p(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$G(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getListObjectInfoAdapterFactory = (luvio) => function UiApi__getListObjectInfo(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$G(untrustedConfig, getListObjectInfo_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$u, buildNetworkSnapshotCachePolicy$v);
};

const TTL$p = 900000;
const VERSION$24 = "458d4a6a30201e422e8daec5fcb03845";
function validate$16(obj, path = 'ListPreferencesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_columnWidths = obj.columnWidths;
        const path_columnWidths = path + '.columnWidths';
        if (typeof obj_columnWidths !== 'object' || ArrayIsArray(obj_columnWidths) || obj_columnWidths === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_columnWidths + '" (at "' + path_columnWidths + '")');
        }
        const obj_columnWidths_keys = ObjectKeys(obj_columnWidths);
        for (let i = 0; i < obj_columnWidths_keys.length; i++) {
            const key = obj_columnWidths_keys[i];
            const obj_columnWidths_prop = obj_columnWidths[key];
            const path_columnWidths_prop = path_columnWidths + '["' + key + '"]';
            if (typeof obj_columnWidths_prop !== 'number' || (typeof obj_columnWidths_prop === 'number' && Math.floor(obj_columnWidths_prop) !== obj_columnWidths_prop)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_columnWidths_prop + '" (at "' + path_columnWidths_prop + '")');
            }
        }
        const obj_columnWrap = obj.columnWrap;
        const path_columnWrap = path + '.columnWrap';
        if (typeof obj_columnWrap !== 'object' || ArrayIsArray(obj_columnWrap) || obj_columnWrap === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_columnWrap + '" (at "' + path_columnWrap + '")');
        }
        const obj_columnWrap_keys = ObjectKeys(obj_columnWrap);
        for (let i = 0; i < obj_columnWrap_keys.length; i++) {
            const key = obj_columnWrap_keys[i];
            const obj_columnWrap_prop = obj_columnWrap[key];
            const path_columnWrap_prop = path_columnWrap + '["' + key + '"]';
            if (typeof obj_columnWrap_prop !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_columnWrap_prop + '" (at "' + path_columnWrap_prop + '")');
            }
        }
        const obj_listReference = obj.listReference;
        const path_listReference = path + '.listReference';
        const referencepath_listReferenceValidationError = validate$2d(obj_listReference, path_listReference);
        if (referencepath_listReferenceValidationError !== null) {
            let message = 'Object doesn\'t match ListReferenceRepresentation (at "' + path_listReference + '")\n';
            message += referencepath_listReferenceValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_orderedBy = obj.orderedBy;
        const path_orderedBy = path + '.orderedBy';
        if (!ArrayIsArray(obj_orderedBy)) {
            return new TypeError('Expected "array" but received "' + typeof obj_orderedBy + '" (at "' + path_orderedBy + '")');
        }
        for (let i = 0; i < obj_orderedBy.length; i++) {
            const obj_orderedBy_item = obj_orderedBy[i];
            const path_orderedBy_item = path_orderedBy + '[' + i + ']';
            const referencepath_orderedBy_itemValidationError = validate$2a(obj_orderedBy_item, path_orderedBy_item);
            if (referencepath_orderedBy_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListOrderByInfoRepresentation (at "' + path_orderedBy_item + '")\n';
                message += referencepath_orderedBy_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$y = 'ListPreferencesRepresentation';
function keyBuilder$2r(luvio, config) {
    return keyPrefix + '::' + RepresentationType$y + ':' + config.objectApiName + ':' + (config.listViewApiName === null ? '' : config.listViewApiName);
}
function keyBuilderFromType$j(luvio, object) {
    const keyParams = {
        objectApiName: object.listReference.objectApiName,
        listViewApiName: object.listReference.listViewApiName
    };
    return keyBuilder$2r(luvio, keyParams);
}
function normalize$r(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2n = function ListPreferencesRepresentationSelect() {
    const { selections: ListReferenceRepresentation__selections, opaque: ListReferenceRepresentation__opaque, } = select$3C();
    const { selections: ListOrderByInfoRepresentation__selections, opaque: ListOrderByInfoRepresentation__opaque, } = select$3z();
    return {
        kind: 'Fragment',
        version: VERSION$24,
        private: [],
        selections: [
            {
                name: 'columnWidths',
                kind: 'Scalar',
                map: true
            },
            {
                name: 'columnWrap',
                kind: 'Scalar',
                map: true
            },
            {
                name: 'listReference',
                kind: 'Object',
                selections: ListReferenceRepresentation__selections
            },
            {
                name: 'orderedBy',
                kind: 'Object',
                plural: true,
                selections: ListOrderByInfoRepresentation__selections
            }
        ]
    };
};
function equals$A(existing, incoming) {
    const existing_columnWidths = existing.columnWidths;
    const incoming_columnWidths = incoming.columnWidths;
    const equals_columnWidths_props = equalsObject(existing_columnWidths, incoming_columnWidths, (existing_columnWidths_prop, incoming_columnWidths_prop) => {
        if (!(existing_columnWidths_prop === incoming_columnWidths_prop)) {
            return false;
        }
    });
    if (equals_columnWidths_props === false) {
        return false;
    }
    const existing_columnWrap = existing.columnWrap;
    const incoming_columnWrap = incoming.columnWrap;
    const equals_columnWrap_props = equalsObject(existing_columnWrap, incoming_columnWrap, (existing_columnWrap_prop, incoming_columnWrap_prop) => {
        if (!(existing_columnWrap_prop === incoming_columnWrap_prop)) {
            return false;
        }
    });
    if (equals_columnWrap_props === false) {
        return false;
    }
    const existing_listReference = existing.listReference;
    const incoming_listReference = incoming.listReference;
    if (!(equals$1g(existing_listReference, incoming_listReference))) {
        return false;
    }
    const existing_orderedBy = existing.orderedBy;
    const incoming_orderedBy = incoming.orderedBy;
    const equals_orderedBy_items = equalsArray(existing_orderedBy, incoming_orderedBy, (existing_orderedBy_item, incoming_orderedBy_item) => {
        if (!(equals$1f(existing_orderedBy_item, incoming_orderedBy_item))) {
            return false;
        }
    });
    if (equals_orderedBy_items === false) {
        return false;
    }
    return true;
}
const ingest$1R = function ListPreferencesRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$16(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$j(luvio, input);
    const ttlToUse = TTL$p;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$r, "UiApi", VERSION$24, RepresentationType$y, equals$A);
    return createLink$1(key);
};
function getTypeCacheKeys$1U(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$j(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$y,
        mergeable: false
    });
}

function select$2m(luvio, params) {
    return select$2n();
}
function keyBuilder$2q(luvio, params) {
    return keyBuilder$2r(luvio, {
        objectApiName: params.urlParams.objectApiName,
        listViewApiName: params.urlParams.listViewApiName
    });
}
function getResponseCacheKeys$C(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1U(storeKeyMap, luvio, response);
}
function ingestSuccess$v(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2q(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1R, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2m(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$o(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2q(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$p,
        namespace: keyPrefix,
        version: VERSION$24,
        representationName: RepresentationType$y
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$H(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-preferences/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$x = 'getListPreferences';
const getListPreferences_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
];
const getListPreferences_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$x, getListPreferences_ConfigPropertyMetadata);
const createResourceParams$B = /*#__PURE__*/ createResourceParams$k(getListPreferences_ConfigPropertyMetadata);
function keyBuilder$2p(luvio, config) {
    const resourceParams = createResourceParams$B(config);
    return keyBuilder$2q(luvio, resourceParams);
}
function typeCheckConfig$E(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListPreferences_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$F(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getListPreferences_ConfigPropertyMetadata);
    const config = typeCheckConfig$E(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$o(luvio, config) {
    createResourceParams$B(config);
    return select$2m();
}
function onFetchResponseSuccess$q(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$v(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$F(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$q(luvio, config, resourceParams, response) {
    const snapshot = ingestError$o(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$F(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$F(luvio, config, options) {
    const resourceParams = createResourceParams$B(config);
    const request = createResourceRequest$H(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$q(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$C(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$q(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$u(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$F, undefined, false);
}
function buildCachedSnapshotCachePolicy$t(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2p(luvio, config),
        node: adapterFragment$o(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$F(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getListPreferencesAdapterFactory = (luvio) => function UiApi__getListPreferences(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$F(untrustedConfig, getListPreferences_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$t, buildNetworkSnapshotCachePolicy$u);
};

function validate$15(obj, path = 'ListOrderedByInfoInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_isAscending = obj.isAscending;
        const path_isAscending = path + '.isAscending';
        if (typeof obj_isAscending !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_isAscending + '" (at "' + path_isAscending + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function select$2l(luvio, params) {
    return select$2n();
}
function getResponseCacheKeys$B(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1U(storeKeyMap, luvio, response);
}
function ingestSuccess$u(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType$j(luvio, body);
    luvio.storeIngest(key, ingest$1R, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2l(),
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function createResourceRequest$G(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/list-preferences/' + config.urlParams.objectApiName + '/' + config.urlParams.listViewApiName + '',
        method: 'patch',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$w = 'updateListPreferences';
const updateListPreferences_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('columnWidths', false, 2 /* Body */, 4 /* Unsupported */),
    generateParamConfigMetadata('columnWrap', false, 2 /* Body */, 4 /* Unsupported */),
    generateParamConfigMetadata('orderedBy', false, 2 /* Body */, 4 /* Unsupported */, true),
];
const updateListPreferences_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$w, updateListPreferences_ConfigPropertyMetadata);
const createResourceParams$A = /*#__PURE__*/ createResourceParams$k(updateListPreferences_ConfigPropertyMetadata);
function typeCheckConfig$D(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, updateListPreferences_ConfigPropertyMetadata);
    const untrustedConfig_columnWidths = untrustedConfig.columnWidths;
    if (untrustedIsObject(untrustedConfig_columnWidths)) {
        const untrustedConfig_columnWidths_object = {};
        const untrustedConfig_columnWidths_keys = Object.keys(untrustedConfig_columnWidths);
        for (let i = 0, arrayLength = untrustedConfig_columnWidths_keys.length; i < arrayLength; i++) {
            const key = untrustedConfig_columnWidths_keys[i];
            const untrustedConfig_columnWidths_prop = untrustedConfig_columnWidths[key];
            if (typeof untrustedConfig_columnWidths_prop === 'number' && Math.floor(untrustedConfig_columnWidths_prop) === untrustedConfig_columnWidths_prop) {
                if (untrustedConfig_columnWidths_object !== undefined) {
                    untrustedConfig_columnWidths_object[key] = untrustedConfig_columnWidths_prop;
                }
            }
        }
        if (untrustedConfig_columnWidths_object !== undefined && Object.keys(untrustedConfig_columnWidths_object).length >= 0) {
            config.columnWidths = untrustedConfig_columnWidths_object;
        }
    }
    const untrustedConfig_columnWrap = untrustedConfig.columnWrap;
    if (untrustedIsObject(untrustedConfig_columnWrap)) {
        const untrustedConfig_columnWrap_object = {};
        const untrustedConfig_columnWrap_keys = Object.keys(untrustedConfig_columnWrap);
        for (let i = 0, arrayLength = untrustedConfig_columnWrap_keys.length; i < arrayLength; i++) {
            const key = untrustedConfig_columnWrap_keys[i];
            const untrustedConfig_columnWrap_prop = untrustedConfig_columnWrap[key];
            if (typeof untrustedConfig_columnWrap_prop === 'boolean') {
                if (untrustedConfig_columnWrap_object !== undefined) {
                    untrustedConfig_columnWrap_object[key] = untrustedConfig_columnWrap_prop;
                }
            }
        }
        if (untrustedConfig_columnWrap_object !== undefined && Object.keys(untrustedConfig_columnWrap_object).length >= 0) {
            config.columnWrap = untrustedConfig_columnWrap_object;
        }
    }
    const untrustedConfig_orderedBy = untrustedConfig.orderedBy;
    if (ArrayIsArray$1(untrustedConfig_orderedBy)) {
        const untrustedConfig_orderedBy_array = [];
        for (let i = 0, arrayLength = untrustedConfig_orderedBy.length; i < arrayLength; i++) {
            const untrustedConfig_orderedBy_item = untrustedConfig_orderedBy[i];
            const referenceListOrderedByInfoInputRepresentationValidationError = validate$15(untrustedConfig_orderedBy_item);
            if (referenceListOrderedByInfoInputRepresentationValidationError === null) {
                untrustedConfig_orderedBy_array.push(untrustedConfig_orderedBy_item);
            }
        }
        config.orderedBy = untrustedConfig_orderedBy_array;
    }
    return config;
}
function validateAdapterConfig$E(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, updateListPreferences_ConfigPropertyMetadata);
    const config = typeCheckConfig$D(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$E(luvio, config, options) {
    const resourceParams = createResourceParams$A(config);
    const request = createResourceRequest$G(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$u(luvio, resourceParams, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$B(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const updateListPreferencesAdapterFactory = (luvio) => {
    return function updateListPreferences(untrustedConfig) {
        const config = validateAdapterConfig$E(untrustedConfig, updateListPreferences_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            throw new Error('Invalid config for "updateListPreferences"');
        }
        return buildNetworkSnapshot$E(luvio, config);
    };
};

const TTL$o = 120000;
const VERSION$23 = "756779d0d7e137dd72c743544afbad82";
function validate$14(obj, path = 'NavItemsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_currentPageUrl = obj.currentPageUrl;
        const path_currentPageUrl = path + '.currentPageUrl';
        if (typeof obj_currentPageUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_navItems = obj.navItems;
        const path_navItems = path + '.navItems';
        if (!ArrayIsArray(obj_navItems)) {
            return new TypeError('Expected "array" but received "' + typeof obj_navItems + '" (at "' + path_navItems + '")');
        }
        for (let i = 0; i < obj_navItems.length; i++) {
            const obj_navItems_item = obj_navItems[i];
            const path_navItems_item = path_navItems + '[' + i + ']';
            if (typeof obj_navItems_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_navItems_item + '" (at "' + path_navItems_item + '")');
            }
        }
        const obj_nextPageUrl = obj.nextPageUrl;
        const path_nextPageUrl = path + '.nextPageUrl';
        let obj_nextPageUrl_union0 = null;
        const obj_nextPageUrl_union0_error = (() => {
            if (typeof obj_nextPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union0_error != null) {
            obj_nextPageUrl_union0 = obj_nextPageUrl_union0_error.message;
        }
        let obj_nextPageUrl_union1 = null;
        const obj_nextPageUrl_union1_error = (() => {
            if (obj_nextPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union1_error != null) {
            obj_nextPageUrl_union1 = obj_nextPageUrl_union1_error.message;
        }
        if (obj_nextPageUrl_union0 && obj_nextPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageUrl + '")';
            message += '\n' + obj_nextPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$x = 'NavItemsRepresentation';
function normalize$q(input, existing, path, luvio, store, timestamp) {
    const input_navItems = input.navItems;
    const input_navItems_id = path.fullPath + '__navItems';
    for (let i = 0; i < input_navItems.length; i++) {
        const input_navItems_item = input_navItems[i];
        let input_navItems_item_id = input_navItems_id + '__' + i;
        input_navItems[i] = ingest$1W(input_navItems_item, {
            fullPath: input_navItems_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$2k = function NavItemsRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$23,
        private: [
            'eTag',
            'currentPageUrl'
        ],
        selections: [
            {
                name: 'navItems',
                kind: 'Link',
                plural: true,
                fragment: select$2L()
            },
            {
                name: 'nextPageUrl',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$z(existing, incoming) {
    const existing_currentPageUrl = existing.currentPageUrl;
    const incoming_currentPageUrl = incoming.currentPageUrl;
    if (!(existing_currentPageUrl === incoming_currentPageUrl)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_navItems = existing.navItems;
    const incoming_navItems = incoming.navItems;
    const equals_navItems_items = equalsArray(existing_navItems, incoming_navItems, (existing_navItems_item, incoming_navItems_item) => {
        if (!(existing_navItems_item.__ref === incoming_navItems_item.__ref)) {
            return false;
        }
    });
    if (equals_navItems_items === false) {
        return false;
    }
    const existing_nextPageUrl = existing.nextPageUrl;
    const incoming_nextPageUrl = incoming.nextPageUrl;
    if (!(existing_nextPageUrl === incoming_nextPageUrl)) {
        return false;
    }
    return true;
}
const ingest$1Q = function NavItemsRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$14(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$o;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$q, "UiApi", VERSION$23, RepresentationType$x, equals$z);
    return createLink$1(key);
};
function getTypeCacheKeys$1T(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$x,
        mergeable: false
    });
    const input_navItems_length = input.navItems.length;
    for (let i = 0; i < input_navItems_length; i++) {
        getTypeCacheKeys$1Z(rootKeySet, luvio, input.navItems[i]);
    }
}

function select$2j(luvio, params) {
    return select$2k();
}
function keyBuilder$2o(luvio, params) {
    return keyPrefix + '::NavItemsRepresentation:(' + 'formFactor:' + params.queryParams.formFactor + ',' + 'navItemNames:' + params.queryParams.navItemNames + ',' + 'page:' + params.queryParams.page + ',' + 'pageSize:' + params.queryParams.pageSize + ')';
}
function getResponseCacheKeys$A(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1T(storeKeyMap, luvio, response, () => keyBuilder$2o(luvio, resourceParams));
}
function ingestSuccess$t(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2o(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1Q, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$2j(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$n(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2o(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$o,
        namespace: keyPrefix,
        version: VERSION$23,
        representationName: RepresentationType$x
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$F(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/nav-items',
        method: 'get',
        body: null,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$v = 'getNavItems';
const getNavItems_ConfigPropertyMetadata = [
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('navItemNames', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('page', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
];
const getNavItems_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$v, getNavItems_ConfigPropertyMetadata);
const createResourceParams$z = /*#__PURE__*/ createResourceParams$k(getNavItems_ConfigPropertyMetadata);
function keyBuilder$2n(luvio, config) {
    const resourceParams = createResourceParams$z(config);
    return keyBuilder$2o(luvio, resourceParams);
}
function typeCheckConfig$C(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getNavItems_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$D(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getNavItems_ConfigPropertyMetadata);
    const config = typeCheckConfig$C(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$n(luvio, config) {
    createResourceParams$z(config);
    return select$2j();
}
function onFetchResponseSuccess$p(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$t(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$D(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$p(luvio, config, resourceParams, response) {
    const snapshot = ingestError$n(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$D(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$D(luvio, config, options) {
    const resourceParams = createResourceParams$z(config);
    const request = createResourceRequest$F(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$p(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$A(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$p(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$t(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$D, undefined, false);
}
function buildCachedSnapshotCachePolicy$s(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2n(luvio, config),
        node: adapterFragment$n(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$D(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getNavItemsAdapterFactory = (luvio) => function UiApi__getNavItems(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$D(untrustedConfig, getNavItems_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$s, buildNetworkSnapshotCachePolicy$t);
};

const nonCachedErrors$5 = ObjectCreate$1(null);
function createChildResourceParams$6(resourceParams) {
    let childConfigs = [];
    for (let index = 0, len = resourceParams.urlParams.objectApiNames.length; index < len; index += 1) {
        const item = resourceParams.urlParams.objectApiNames[index];
        childConfigs.push({
            urlParams: {
                objectApiName: item
            }
        });
    }
    return childConfigs;
}
function selectChildResourceParams$5(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$2k(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$3m(luvio, childResource);
                const childFragment = select$3d();
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$5[childKey];
                            if (nonCachedError === undefined || nonCachedError.expiration < reader.getTimeStamp()) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$2h(luvio, resourceParams) {
    const childResources = createChildResourceParams$6(resourceParams);
    return selectChildResourceParams$5(luvio, childResources, resourceParams);
}
function keyBuilder$2k(luvio, params) {
    return keyPrefix + '::SimplifiedBatchRepresentation:(' + 'objectApiNames:' + params.urlParams.objectApiNames + ')';
}
function getResponseCacheKeys$y(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$6(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$11(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$3m(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$T,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$6(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$3m(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$S(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$L(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$5[childKey] = { expiration: now + TTL$D, response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$5[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$r(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$6(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$2k(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$6(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$2h(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$l(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2k(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$D(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/object-info/batch/' + config.urlParams.objectApiNames + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$t = 'getObjectInfos';
const getObjectInfos_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiNames', true, 0 /* UrlParameter */, 0 /* String */, true, getObjectApiNamesArray),
];
const getObjectInfos_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$t, getObjectInfos_ConfigPropertyMetadata);
const createResourceParams$x = /*#__PURE__*/ createResourceParams$k(getObjectInfos_ConfigPropertyMetadata);
function keyBuilder$2j(luvio, config) {
    const resourceParams = createResourceParams$x(config);
    return keyBuilder$2k(luvio, resourceParams);
}
function typeCheckConfig$A(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getObjectInfos_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$B(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getObjectInfos_ConfigPropertyMetadata);
    const config = typeCheckConfig$A(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$l(luvio, config) {
    const resourceParams = createResourceParams$x(config);
    return select$2h(luvio, resourceParams);
}
function onFetchResponseSuccess$n(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$r(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$B(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$n(luvio, config, resourceParams, response) {
    const snapshot = ingestError$l(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$B(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$B(luvio, config, options) {
    const resourceParams = createResourceParams$x(config);
    const request = createResourceRequest$D(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$n(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$y(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$n(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$r(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$B, undefined, false);
}
function buildCachedSnapshotCachePolicy$q(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2j(luvio, config),
        node: adapterFragment$l(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$B(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getObjectInfosAdapterFactory = (luvio) => function UiApi__getObjectInfos(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$B(untrustedConfig, getObjectInfos_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$q, buildNetworkSnapshotCachePolicy$r);
};

const VERSION$22 = "8d2032474fb4fabdf5cf4042be9042d9";
const select$2g = function WorkStepPicklistValueAttributeRepresentationSelect() {
    const { selections: AbstractPicklistValueAttributesRepresentationSelections } = select$2c();
    return {
        kind: 'Fragment',
        version: VERSION$22,
        private: [],
        selections: [
            ...AbstractPicklistValueAttributesRepresentationSelections,
            {
                name: 'sortOrder',
                kind: 'Scalar'
            },
            {
                name: 'statusCode',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$21 = "53893a219bc0888951863f2c44317cac";
const select$2f = function LeadStatusPicklistValueAttributesRepresentationSelect() {
    const { selections: AbstractPicklistValueAttributesRepresentationSelections } = select$2c();
    return {
        kind: 'Fragment',
        version: VERSION$21,
        private: [],
        selections: [
            ...AbstractPicklistValueAttributesRepresentationSelections,
            {
                name: 'converted',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$20 = "4818ec07c499784571c1012ead048c88";
const select$2e = function CaseStatusPicklistValueAttributesRepresentationSelect() {
    const { selections: AbstractPicklistValueAttributesRepresentationSelections } = select$2c();
    return {
        kind: 'Fragment',
        version: VERSION$20,
        private: [],
        selections: [
            ...AbstractPicklistValueAttributesRepresentationSelections,
            {
                name: 'closed',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$1$ = "950a3a962acbd2b128f937d4c055a9d6";
const select$2d = function OpportunityStagePicklistValueAttributesRepresentationSelect() {
    const { selections: AbstractPicklistValueAttributesRepresentationSelections } = select$2c();
    return {
        kind: 'Fragment',
        version: VERSION$1$,
        private: [],
        selections: [
            ...AbstractPicklistValueAttributesRepresentationSelections,
            {
                name: 'closed',
                kind: 'Scalar'
            },
            {
                name: 'defaultProbability',
                kind: 'Scalar'
            },
            {
                name: 'forecastCategoryName',
                kind: 'Scalar'
            },
            {
                name: 'won',
                kind: 'Scalar'
            }
        ]
    };
};

var DiscriminatorValues$4;
(function (DiscriminatorValues) {
    DiscriminatorValues["WorkStepStatus"] = "WorkStepStatus";
    DiscriminatorValues["LeadStatus"] = "LeadStatus";
    DiscriminatorValues["CaseStatus"] = "CaseStatus";
    DiscriminatorValues["OpportunityStage"] = "OpportunityStage";
})(DiscriminatorValues$4 || (DiscriminatorValues$4 = {}));
const VERSION$1_ = "bd523b2343366edfc25a2dbee2c4e986";
function validate$13(obj, path = 'AbstractPicklistValueAttributesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_picklistAtrributesValueType = obj.picklistAtrributesValueType;
        const path_picklistAtrributesValueType = path + '.picklistAtrributesValueType';
        if (typeof obj_picklistAtrributesValueType !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_picklistAtrributesValueType + '" (at "' + path_picklistAtrributesValueType + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const selectChildren$2 = function AbstractPicklistValueAttributesRepresentationSelectChildren() {
    const workStepPicklistValueAttributeRepresentationSelections = select$2g();
    const leadStatusPicklistValueAttributesRepresentationSelections = select$2f();
    const caseStatusPicklistValueAttributesRepresentationSelections = select$2e();
    const opportunityStagePicklistValueAttributesRepresentationSelections = select$2d();
    return {
        kind: 'Fragment',
        union: true,
        discriminator: 'picklistAtrributesValueType',
        unionSelections: {
            [DiscriminatorValues$4.WorkStepStatus]: workStepPicklistValueAttributeRepresentationSelections,
            [DiscriminatorValues$4.LeadStatus]: leadStatusPicklistValueAttributesRepresentationSelections,
            [DiscriminatorValues$4.CaseStatus]: caseStatusPicklistValueAttributesRepresentationSelections,
            [DiscriminatorValues$4.OpportunityStage]: opportunityStagePicklistValueAttributesRepresentationSelections
        }
    };
};
const select$2c = function AbstractPicklistValueAttributesRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1_,
        private: [],
        selections: [
            {
                name: 'picklistAtrributesValueType',
                kind: 'Scalar'
            }
        ]
    };
};

const VERSION$1Z = "9e2a16a80378487f557124c771201cf9";
function validate$12(obj, path = 'PicklistValueRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_attributes = obj.attributes;
        const path_attributes = path + '.attributes';
        let obj_attributes_union0 = null;
        const obj_attributes_union0_error = (() => {
            const referencepath_attributesValidationError = validate$13(obj_attributes, path_attributes);
            if (referencepath_attributesValidationError !== null) {
                let message = 'Object doesn\'t match AbstractPicklistValueAttributesRepresentation (at "' + path_attributes + '")\n';
                message += referencepath_attributesValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_attributes_union0_error != null) {
            obj_attributes_union0 = obj_attributes_union0_error.message;
        }
        let obj_attributes_union1 = null;
        const obj_attributes_union1_error = (() => {
            if (obj_attributes !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_attributes + '" (at "' + path_attributes + '")');
            }
        })();
        if (obj_attributes_union1_error != null) {
            obj_attributes_union1 = obj_attributes_union1_error.message;
        }
        if (obj_attributes_union0 && obj_attributes_union1) {
            let message = 'Object doesn\'t match union (at "' + path_attributes + '")';
            message += '\n' + obj_attributes_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_attributes_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_validFor = obj.validFor;
        const path_validFor = path + '.validFor';
        if (!ArrayIsArray(obj_validFor)) {
            return new TypeError('Expected "array" but received "' + typeof obj_validFor + '" (at "' + path_validFor + '")');
        }
        for (let i = 0; i < obj_validFor.length; i++) {
            const obj_validFor_item = obj_validFor[i];
            const path_validFor_item = path_validFor + '[' + i + ']';
            if (typeof obj_validFor_item !== 'number' || (typeof obj_validFor_item === 'number' && Math.floor(obj_validFor_item) !== obj_validFor_item)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_validFor_item + '" (at "' + path_validFor_item + '")');
            }
        }
        const obj_value = obj.value;
        const path_value = path + '.value';
        if (typeof obj_value !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_value + '" (at "' + path_value + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$2b = function PicklistValueRepresentationSelect() {
    const AbstractPicklistValueAttributesRepresentation__unionSelections = selectChildren$2();
    return {
        kind: 'Fragment',
        version: VERSION$1Z,
        private: [],
        selections: [
            {
                kind: 'Object',
                name: 'attributes',
                discriminator: AbstractPicklistValueAttributesRepresentation__unionSelections.discriminator,
                union: true,
                nullable: true,
                unionSelections: AbstractPicklistValueAttributesRepresentation__unionSelections.unionSelections
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'validFor',
                kind: 'Scalar',
                plural: true
            },
            {
                name: 'value',
                kind: 'Scalar'
            }
        ]
    };
};

const keyBuilderFromType$i = 
/**
 * custom key builder from PicklistValuesRepresentation.
 * PicklistValuesRepresentation.url is like /services/data/v65.0/ui-api/object-info/Account/picklist-values/012000000000000AAA/AccountSource
 * the '/services/data/v65.0' in the key make trouble for new js release.  so chop it off make the key unique cross releases.
 */
function PicklistValuesRepresentationKeyBuilderFromType(luvio, object) {
    const id = object.url.substring(object.url.indexOf('/ui-api'));
    return keyBuilder$2i(luvio, { id });
};

const TTL$n = 900000;
const VERSION$1Y = "0a361a49370acb4c6a31721a2057649a";
function validate$11(obj, path = 'PicklistValuesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_controllerValues = obj.controllerValues;
        const path_controllerValues = path + '.controllerValues';
        if (typeof obj_controllerValues !== 'object' || ArrayIsArray(obj_controllerValues) || obj_controllerValues === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_controllerValues + '" (at "' + path_controllerValues + '")');
        }
        const obj_controllerValues_keys = ObjectKeys(obj_controllerValues);
        for (let i = 0; i < obj_controllerValues_keys.length; i++) {
            const key = obj_controllerValues_keys[i];
            const obj_controllerValues_prop = obj_controllerValues[key];
            const path_controllerValues_prop = path_controllerValues + '["' + key + '"]';
            if (typeof obj_controllerValues_prop !== 'number' || (typeof obj_controllerValues_prop === 'number' && Math.floor(obj_controllerValues_prop) !== obj_controllerValues_prop)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_controllerValues_prop + '" (at "' + path_controllerValues_prop + '")');
            }
        }
        const obj_defaultValue = obj.defaultValue;
        const path_defaultValue = path + '.defaultValue';
        let obj_defaultValue_union0 = null;
        const obj_defaultValue_union0_error = (() => {
            const referencepath_defaultValueValidationError = validate$12(obj_defaultValue, path_defaultValue);
            if (referencepath_defaultValueValidationError !== null) {
                let message = 'Object doesn\'t match PicklistValueRepresentation (at "' + path_defaultValue + '")\n';
                message += referencepath_defaultValueValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_defaultValue_union0_error != null) {
            obj_defaultValue_union0 = obj_defaultValue_union0_error.message;
        }
        let obj_defaultValue_union1 = null;
        const obj_defaultValue_union1_error = (() => {
            if (obj_defaultValue !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_defaultValue + '" (at "' + path_defaultValue + '")');
            }
        })();
        if (obj_defaultValue_union1_error != null) {
            obj_defaultValue_union1 = obj_defaultValue_union1_error.message;
        }
        if (obj_defaultValue_union0 && obj_defaultValue_union1) {
            let message = 'Object doesn\'t match union (at "' + path_defaultValue + '")';
            message += '\n' + obj_defaultValue_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_defaultValue_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_url = obj.url;
        const path_url = path + '.url';
        if (typeof obj_url !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_url + '" (at "' + path_url + '")');
        }
        const obj_values = obj.values;
        const path_values = path + '.values';
        if (!ArrayIsArray(obj_values)) {
            return new TypeError('Expected "array" but received "' + typeof obj_values + '" (at "' + path_values + '")');
        }
        for (let i = 0; i < obj_values.length; i++) {
            const obj_values_item = obj_values[i];
            const path_values_item = path_values + '[' + i + ']';
            const referencepath_values_itemValidationError = validate$12(obj_values_item, path_values_item);
            if (referencepath_values_itemValidationError !== null) {
                let message = 'Object doesn\'t match PicklistValueRepresentation (at "' + path_values_item + '")\n';
                message += referencepath_values_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$w = 'PicklistValuesRepresentation';
function keyBuilder$2i(luvio, config) {
    return keyPrefix + '::' + RepresentationType$w + ':' + config.id;
}
function normalize$p(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$2a = function PicklistValuesRepresentationSelect() {
    const { selections: PicklistValueRepresentation__selections, opaque: PicklistValueRepresentation__opaque, } = select$2b();
    return {
        kind: 'Fragment',
        version: VERSION$1Y,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'controllerValues',
                kind: 'Scalar',
                map: true
            },
            {
                name: 'defaultValue',
                kind: 'Object',
                nullable: true,
                selections: PicklistValueRepresentation__selections
            },
            {
                name: 'url',
                kind: 'Scalar'
            },
            {
                name: 'values',
                kind: 'Object',
                plural: true,
                selections: PicklistValueRepresentation__selections
            }
        ]
    };
};
function equals$y(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$1P = function PicklistValuesRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$11(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$i(luvio, input);
    const ttlToUse = TTL$n;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$p, "UiApi", VERSION$1Y, RepresentationType$w, equals$y);
    return createLink$1(key);
};
function getTypeCacheKeys$1S(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$i(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$w,
        mergeable: false
    });
}

const TTL$m = 900000;
const VERSION$1X = "ec03b0f6da287c949d1ccaa904ddbfd3";
function validate$10(obj, path = 'PicklistValuesCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_picklistFieldValues = obj.picklistFieldValues;
        const path_picklistFieldValues = path + '.picklistFieldValues';
        if (typeof obj_picklistFieldValues !== 'object' || ArrayIsArray(obj_picklistFieldValues) || obj_picklistFieldValues === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_picklistFieldValues + '" (at "' + path_picklistFieldValues + '")');
        }
        const obj_picklistFieldValues_keys = ObjectKeys(obj_picklistFieldValues);
        for (let i = 0; i < obj_picklistFieldValues_keys.length; i++) {
            const key = obj_picklistFieldValues_keys[i];
            const obj_picklistFieldValues_prop = obj_picklistFieldValues[key];
            const path_picklistFieldValues_prop = path_picklistFieldValues + '["' + key + '"]';
            if (typeof obj_picklistFieldValues_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_picklistFieldValues_prop + '" (at "' + path_picklistFieldValues_prop + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$v = 'PicklistValuesCollectionRepresentation';
function normalize$o(input, existing, path, luvio, store, timestamp) {
    const input_picklistFieldValues = input.picklistFieldValues;
    const input_picklistFieldValues_id = path.fullPath + '__picklistFieldValues';
    const input_picklistFieldValues_keys = Object.keys(input_picklistFieldValues);
    const input_picklistFieldValues_length = input_picklistFieldValues_keys.length;
    for (let i = 0; i < input_picklistFieldValues_length; i++) {
        const key = input_picklistFieldValues_keys[i];
        const input_picklistFieldValues_prop = input_picklistFieldValues[key];
        const input_picklistFieldValues_prop_id = input_picklistFieldValues_id + '__' + key;
        input_picklistFieldValues[key] = ingest$1P(input_picklistFieldValues_prop, {
            fullPath: input_picklistFieldValues_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
const select$29 = function PicklistValuesCollectionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1X,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'picklistFieldValues',
                kind: 'Link',
                map: true,
                fragment: select$2a()
            }
        ]
    };
};
function equals$x(existing, incoming) {
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_picklistFieldValues = existing.picklistFieldValues;
    const incoming_picklistFieldValues = incoming.picklistFieldValues;
    const equals_picklistFieldValues_props = equalsObject(existing_picklistFieldValues, incoming_picklistFieldValues, (existing_picklistFieldValues_prop, incoming_picklistFieldValues_prop) => {
        if (!(existing_picklistFieldValues_prop.__ref === incoming_picklistFieldValues_prop.__ref)) {
            return false;
        }
    });
    if (equals_picklistFieldValues_props === false) {
        return false;
    }
    return true;
}
const ingest$1O = function PicklistValuesCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$10(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$m;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$o, "UiApi", VERSION$1X, RepresentationType$v, equals$x);
    return createLink$1(key);
};
function getTypeCacheKeys$1R(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$v,
        mergeable: false
    });
    const input_picklistFieldValues = input.picklistFieldValues;
    const input_picklistFieldValues_keys = ObjectKeys(input_picklistFieldValues);
    const input_picklistFieldValues_length = input_picklistFieldValues_keys.length;
    for (let i = 0; i < input_picklistFieldValues_length; i++) {
        const key = input_picklistFieldValues_keys[i];
        getTypeCacheKeys$1S(rootKeySet, luvio, input_picklistFieldValues[key]);
    }
}

function select$28(luvio, params) {
    return select$29();
}
function keyBuilder$2h(luvio, params) {
    return keyPrefix + '::PicklistValuesCollectionRepresentation:(' + 'objectApiName:' + params.urlParams.objectApiName + ',' + 'recordTypeId:' + params.urlParams.recordTypeId + ')';
}
function getResponseCacheKeys$x(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1R(storeKeyMap, luvio, response, () => keyBuilder$2h(luvio, resourceParams));
}
function ingestSuccess$q(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2h(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1O, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$28(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$k(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2h(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$m,
        namespace: keyPrefix,
        version: VERSION$1X,
        representationName: RepresentationType$v
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$C(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/object-info/' + config.urlParams.objectApiName + '/picklist-values/' + config.urlParams.recordTypeId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$s = 'getPicklistValuesByRecordType';
const getPicklistValuesByRecordType_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('recordTypeId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
];
const getPicklistValuesByRecordType_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$s, getPicklistValuesByRecordType_ConfigPropertyMetadata);
const createResourceParams$w = /*#__PURE__*/ createResourceParams$k(getPicklistValuesByRecordType_ConfigPropertyMetadata);
function keyBuilder$2g(luvio, config) {
    const resourceParams = createResourceParams$w(config);
    return keyBuilder$2h(luvio, resourceParams);
}
function typeCheckConfig$z(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getPicklistValuesByRecordType_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$A(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getPicklistValuesByRecordType_ConfigPropertyMetadata);
    const config = typeCheckConfig$z(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$k(luvio, config) {
    createResourceParams$w(config);
    return select$28();
}
function onFetchResponseSuccess$m(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$q(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$A(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$m(luvio, config, resourceParams, response) {
    const snapshot = ingestError$k(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$A(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$A(luvio, config, options) {
    const resourceParams = createResourceParams$w(config);
    const request = createResourceRequest$C(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$m(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$x(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$m(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$q(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$A, undefined, false);
}
function buildCachedSnapshotCachePolicy$p(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2g(luvio, config),
        node: adapterFragment$k(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$A(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getPicklistValuesByRecordTypeAdapterFactory = (luvio) => function UiApi__getPicklistValuesByRecordType(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$A(untrustedConfig, getPicklistValuesByRecordType_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$p, buildNetworkSnapshotCachePolicy$q);
};

function getResponseCacheKeys$w(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1S(storeKeyMap, luvio, response);
}
function createResourceRequest$B(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/object-info/' + config.urlParams.objectApiName + '/picklist-values/' + config.urlParams.recordTypeId + '/' + config.urlParams.fieldApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const path = select$2a().selections;
function buildSnapshotRefresh$2(luvio, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot$z(luvio, config),
    };
}
function buildRequestAndKey(luvio, config) {
    const { recordTypeId, fieldApiName } = config;
    const fieldNames = getFieldId(fieldApiName);
    const resourceParams = {
        urlParams: {
            objectApiName: fieldNames.objectApiName,
            fieldApiName: fieldNames.fieldApiName,
            recordTypeId,
        },
    };
    const request = createResourceRequest$B(resourceParams);
    const key = keyBuilder$2i(luvio, { id: request.basePath });
    return { resourceParams, request, key };
}
function onResponseSuccess$1(luvio, config, key, response) {
    const { body } = response;
    luvio.storeIngest(key, ingest$1P, body);
    const snapshot = buildCachedSnapshot$4(luvio, config);
    return luvio.storeBroadcast().then(() => snapshot);
}
function onResponseError(luvio, config, key, err) {
    const errorSnapshot = luvio.errorSnapshot(err, buildSnapshotRefresh$2(luvio, config));
    luvio.storeIngestError(key, errorSnapshot);
    return luvio.storeBroadcast().then(() => errorSnapshot);
}
function buildNetworkSnapshot$z(luvio, config, options) {
    const { resourceParams, request, key } = buildRequestAndKey(luvio, config);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        return luvio.handleSuccessResponse(() => {
            return onResponseSuccess$1(luvio, config, key, response);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$w(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return onResponseError(luvio, config, key, err);
        });
    });
}
function buildCachedSnapshot$4(luvio, config) {
    const fieldNames = getFieldId(config.fieldApiName);
    const request = createResourceRequest$B({
        urlParams: {
            objectApiName: fieldNames.objectApiName,
            fieldApiName: fieldNames.fieldApiName,
            recordTypeId: config.recordTypeId,
        },
    });
    const key = keyBuilder$2i(luvio, { id: request.basePath });
    return luvio.storeLookup({
        recordId: key,
        node: {
            kind: 'Fragment',
            private: ['eTag'],
            selections: path,
            version: VERSION$1Y,
        },
        variables: {},
    }, buildSnapshotRefresh$2(luvio, config));
}
function buildNetworkSnapshotCachePolicy$p(context, coercedAdapterRequestContext) {
    const { config, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$z(luvio, config, dispatchOptions);
}
function buildCachedSnapshotCachePolicy$o(context, storeLookup) {
    const { config, luvio } = context;
    const fieldNames = getFieldId(config.fieldApiName);
    const { basePath } = createResourceRequest$B({
        urlParams: {
            objectApiName: fieldNames.objectApiName,
            fieldApiName: fieldNames.fieldApiName,
            recordTypeId: config.recordTypeId,
        },
    });
    const key = keyBuilder$2i(luvio, { id: basePath });
    return storeLookup({
        recordId: key,
        node: {
            kind: 'Fragment',
            private: ['eTag'],
            selections: path,
            version: VERSION$1Y,
        },
        variables: {},
    }, buildSnapshotRefresh$2(luvio, config));
}
const picklistValuesConfigPropertyNames = {
    displayName: 'getPicklistValues',
    parameters: {
        required: ['recordTypeId', 'fieldApiName'],
        optional: [],
    },
};
function validateAdapterConfig$z(untrusted, _config) {
    return validateAdapterConfig$y(untrusted, picklistValuesConfigPropertyNames);
}

const getPicklistValues_ConfigPropertyMetadata = [
    generateParamConfigMetadata('fieldApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getFieldApiName),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('recordTypeId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
];
function typeCheckConfig$y(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getPicklistValues_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$y(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getPicklistValues_ConfigPropertyMetadata);
    const config = typeCheckConfig$y(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
const getPicklistValuesAdapterFactory = (luvio) => function UiApi__getPicklistValues(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$z(untrustedConfig);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$o, buildNetworkSnapshotCachePolicy$p);
};

function select$27(luvio, params) {
    return select$3c();
}
function keyBuilder$2f(luvio, params) {
    return keyBuilder$3k(luvio, {
        layoutType: "PathAssistant",
        objectApiName: params.urlParams.objectApiName,
        layoutOverride: params.queryParams.layoutOverride,
        recordTypeId: params.queryParams.recordTypeId || "",
        mode: params.queryParams.mode || "",
        formFactor: params.queryParams.formFactor || ""
    });
}
function getResponseCacheKeys$v(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2b(storeKeyMap, luvio, response);
}
function ingestSuccess$p(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2f(luvio, resourceParams);
    luvio.storeIngest(key, ingest$28, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$27(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$j(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2f(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$C,
        namespace: keyPrefix,
        version: VERSION$2B,
        representationName: RepresentationType$S
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$A(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/path/layout/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$r = 'getPathLayout';
const getPathLayout_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('layoutOverride', true, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('mode', false, 1 /* QueryParameter */, 0 /* String */, false, coerceLayoutMode),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */, false, getRecordId18),
];
const getPathLayout_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$r, getPathLayout_ConfigPropertyMetadata);
const createResourceParams$v = /*#__PURE__*/ createResourceParams$k(getPathLayout_ConfigPropertyMetadata);
function keyBuilder$2e(luvio, config) {
    const resourceParams = createResourceParams$v(config);
    return keyBuilder$2f(luvio, resourceParams);
}
function typeCheckConfig$x(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getPathLayout_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$x(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getPathLayout_ConfigPropertyMetadata);
    const config = typeCheckConfig$x(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$j(luvio, config) {
    createResourceParams$v(config);
    return select$27();
}
function onFetchResponseSuccess$l(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$p(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$y(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$l(luvio, config, resourceParams, response) {
    const snapshot = ingestError$j(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$y(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$y(luvio, config, options) {
    const resourceParams = createResourceParams$v(config);
    const request = createResourceRequest$A(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$l(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$v(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$l(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$o(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$y, undefined, false);
}
function buildCachedSnapshotCachePolicy$n(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2e(luvio, config),
        node: adapterFragment$j(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$y(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getPathLayoutAdapterFactory = (luvio) => function UiApi__getPathLayout(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$x(untrustedConfig, getPathLayout_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$n, buildNetworkSnapshotCachePolicy$o);
};

function validate$$(obj, path = 'MatchResultInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_recordIds = obj.recordIds;
        const path_recordIds = path + '.recordIds';
        if (!ArrayIsArray(obj_recordIds)) {
            return new TypeError('Expected "array" but received "' + typeof obj_recordIds + '" (at "' + path_recordIds + '")');
        }
        for (let i = 0; i < obj_recordIds.length; i++) {
            const obj_recordIds_item = obj_recordIds[i];
            const path_recordIds_item = path_recordIds + '[' + i + ']';
            if (typeof obj_recordIds_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordIds_item + '" (at "' + path_recordIds_item + '")');
            }
        }
        const obj_rule = obj.rule;
        const path_rule = path + '.rule';
        if (typeof obj_rule !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_rule + '" (at "' + path_rule + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$_(obj, path = 'DuplicateResultInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_allowSave = obj.allowSave;
        const path_allowSave = path + '.allowSave';
        if (typeof obj_allowSave !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_allowSave + '" (at "' + path_allowSave + '")');
        }
        const obj_matchResultInfo = obj.matchResultInfo;
        const path_matchResultInfo = path + '.matchResultInfo';
        const referencepath_matchResultInfoValidationError = validate$$(obj_matchResultInfo, path_matchResultInfo);
        if (referencepath_matchResultInfoValidationError !== null) {
            let message = 'Object doesn\'t match MatchResultInfoRepresentation (at "' + path_matchResultInfo + '")\n';
            message += referencepath_matchResultInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_rule = obj.rule;
        const path_rule = path + '.rule';
        if (typeof obj_rule !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_rule + '" (at "' + path_rule + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$Z(obj, path = 'MatchRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_duplicateResultInfos = obj.duplicateResultInfos;
        const path_duplicateResultInfos = path + '.duplicateResultInfos';
        if (typeof obj_duplicateResultInfos !== 'object' || ArrayIsArray(obj_duplicateResultInfos) || obj_duplicateResultInfos === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_duplicateResultInfos + '" (at "' + path_duplicateResultInfos + '")');
        }
        const obj_duplicateResultInfos_keys = ObjectKeys(obj_duplicateResultInfos);
        for (let i = 0; i < obj_duplicateResultInfos_keys.length; i++) {
            const key = obj_duplicateResultInfos_keys[i];
            const obj_duplicateResultInfos_prop = obj_duplicateResultInfos[key];
            const path_duplicateResultInfos_prop = path_duplicateResultInfos + '["' + key + '"]';
            const referencepath_duplicateResultInfos_propValidationError = validate$_(obj_duplicateResultInfos_prop, path_duplicateResultInfos_prop);
            if (referencepath_duplicateResultInfos_propValidationError !== null) {
                let message = 'Object doesn\'t match DuplicateResultInfoRepresentation (at "' + path_duplicateResultInfos_prop + '")\n';
                message += referencepath_duplicateResultInfos_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_recordIds = obj.recordIds;
        const path_recordIds = path + '.recordIds';
        if (!ArrayIsArray(obj_recordIds)) {
            return new TypeError('Expected "array" but received "' + typeof obj_recordIds + '" (at "' + path_recordIds + '")');
        }
        for (let i = 0; i < obj_recordIds.length; i++) {
            const obj_recordIds_item = obj_recordIds[i];
            const path_recordIds_item = path_recordIds + '[' + i + ']';
            if (typeof obj_recordIds_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordIds_item + '" (at "' + path_recordIds_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$l = 30000;
const VERSION$1W = "583c38564fa15ce0fb3dd2807be1bdc6";
function validate$Y(obj, path = 'DuplicatesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_allowSave = obj.allowSave;
        const path_allowSave = path + '.allowSave';
        if (typeof obj_allowSave !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_allowSave + '" (at "' + path_allowSave + '")');
        }
        const obj_duplicateError = obj.duplicateError;
        const path_duplicateError = path + '.duplicateError';
        if (typeof obj_duplicateError !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_duplicateError + '" (at "' + path_duplicateError + '")');
        }
        const obj_duplicateRules = obj.duplicateRules;
        const path_duplicateRules = path + '.duplicateRules';
        if (!ArrayIsArray(obj_duplicateRules)) {
            return new TypeError('Expected "array" but received "' + typeof obj_duplicateRules + '" (at "' + path_duplicateRules + '")');
        }
        for (let i = 0; i < obj_duplicateRules.length; i++) {
            const obj_duplicateRules_item = obj_duplicateRules[i];
            const path_duplicateRules_item = path_duplicateRules + '[' + i + ']';
            if (typeof obj_duplicateRules_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_duplicateRules_item + '" (at "' + path_duplicateRules_item + '")');
            }
        }
        const obj_matches = obj.matches;
        const path_matches = path + '.matches';
        if (!ArrayIsArray(obj_matches)) {
            return new TypeError('Expected "array" but received "' + typeof obj_matches + '" (at "' + path_matches + '")');
        }
        for (let i = 0; i < obj_matches.length; i++) {
            const obj_matches_item = obj_matches[i];
            const path_matches_item = path_matches + '[' + i + ']';
            const referencepath_matches_itemValidationError = validate$Z(obj_matches_item, path_matches_item);
            if (referencepath_matches_itemValidationError !== null) {
                let message = 'Object doesn\'t match MatchRepresentation (at "' + path_matches_item + '")\n';
                message += referencepath_matches_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$u = 'DuplicatesRepresentation';
function normalize$n(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$26 = function DuplicatesRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1W,
        private: [],
        opaque: true
    };
};
function equals$w(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1N = function DuplicatesRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$Y(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$l;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$n, "UiApi", VERSION$1W, RepresentationType$u, equals$w);
    return createLink$1(key);
};
function getTypeCacheKeys$1Q(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$u,
        mergeable: false
    });
}

function select$25(luvio, params) {
    return select$26();
}
function keyBuilder$2d(luvio, params) {
    return keyPrefix + '::DuplicatesRepresentation:(' + (params.body.allowSaveOnDuplicate === undefined ? 'allowSaveOnDuplicate' : 'allowSaveOnDuplicate:' + params.body.allowSaveOnDuplicate) + '::' + (params.body.apiName === undefined ? 'apiName' : 'apiName:' + params.body.apiName) + '::' + stableJSONStringify(params.body.fields) + ')';
}
function getResponseCacheKeys$u(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1Q(storeKeyMap, luvio, response, () => keyBuilder$2d(luvio, resourceParams));
}
function ingestSuccess$o(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$2d(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1N, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$25(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$i(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2d(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$l,
        namespace: keyPrefix,
        version: VERSION$1W,
        representationName: RepresentationType$u
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$z(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/predupe',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$q = 'getDuplicates';
const getDuplicates_ConfigPropertyMetadata = [
    generateParamConfigMetadata('allowSaveOnDuplicate', false, 2 /* Body */, 1 /* Boolean */),
    generateParamConfigMetadata('apiName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('fields', true, 2 /* Body */, 4 /* Unsupported */),
];
const getDuplicates_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$q, getDuplicates_ConfigPropertyMetadata);
const createResourceParams$u = /*#__PURE__*/ createResourceParams$k(getDuplicates_ConfigPropertyMetadata);
function keyBuilder$2c(luvio, config) {
    const resourceParams = createResourceParams$u(config);
    return keyBuilder$2d(luvio, resourceParams);
}
function typeCheckConfig$w(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getDuplicates_ConfigPropertyMetadata);
    const untrustedConfig_fields = untrustedConfig.fields;
    if (untrustedIsObject(untrustedConfig_fields)) {
        const untrustedConfig_fields_object = {};
        const untrustedConfig_fields_keys = Object.keys(untrustedConfig_fields);
        for (let i = 0, arrayLength = untrustedConfig_fields_keys.length; i < arrayLength; i++) {
            const key = untrustedConfig_fields_keys[i];
            const untrustedConfig_fields_prop = untrustedConfig_fields[key];
            if (typeof untrustedConfig_fields_prop === 'string') {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
            if (typeof untrustedConfig_fields_prop === 'number') {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
            if (untrustedConfig_fields_prop === null) {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
            if (typeof untrustedConfig_fields_prop === 'boolean') {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
        }
        if (untrustedConfig_fields_object !== undefined && Object.keys(untrustedConfig_fields_object).length >= 0) {
            config.fields = untrustedConfig_fields_object;
        }
    }
    return config;
}
function validateAdapterConfig$w(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$w(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$i(luvio, config) {
    createResourceParams$u(config);
    return select$25();
}
function onFetchResponseSuccess$k(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$o(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$x(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$k(luvio, config, resourceParams, response) {
    const snapshot = ingestError$i(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$x(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$x(luvio, config, options) {
    const resourceParams = createResourceParams$u(config);
    const request = createResourceRequest$z(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$k(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$u(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$k(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$n(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$x, 'get', false);
}
function buildCachedSnapshotCachePolicy$m(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$2c(luvio, config),
        node: adapterFragment$i(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$x(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getDuplicatesAdapterFactory = (luvio) => function UiApi__getDuplicates(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$w(untrustedConfig, getDuplicates_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$m, buildNetworkSnapshotCachePolicy$n);
};

const VERSION$1V = "e84b9ffdeff97c411bd111f2a55cf6b6";
function validate$X(obj, path = 'PhotoMetadataRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_companyBluemasterId = obj.companyBluemasterId;
        const path_companyBluemasterId = path + '.companyBluemasterId';
        let obj_companyBluemasterId_union0 = null;
        const obj_companyBluemasterId_union0_error = (() => {
            if (typeof obj_companyBluemasterId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_companyBluemasterId + '" (at "' + path_companyBluemasterId + '")');
            }
        })();
        if (obj_companyBluemasterId_union0_error != null) {
            obj_companyBluemasterId_union0 = obj_companyBluemasterId_union0_error.message;
        }
        let obj_companyBluemasterId_union1 = null;
        const obj_companyBluemasterId_union1_error = (() => {
            if (obj_companyBluemasterId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_companyBluemasterId + '" (at "' + path_companyBluemasterId + '")');
            }
        })();
        if (obj_companyBluemasterId_union1_error != null) {
            obj_companyBluemasterId_union1 = obj_companyBluemasterId_union1_error.message;
        }
        if (obj_companyBluemasterId_union0 && obj_companyBluemasterId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_companyBluemasterId + '")';
            message += '\n' + obj_companyBluemasterId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_companyBluemasterId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_responseId = obj.responseId;
        const path_responseId = path + '.responseId';
        let obj_responseId_union0 = null;
        const obj_responseId_union0_error = (() => {
            if (typeof obj_responseId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_responseId + '" (at "' + path_responseId + '")');
            }
        })();
        if (obj_responseId_union0_error != null) {
            obj_responseId_union0 = obj_responseId_union0_error.message;
        }
        let obj_responseId_union1 = null;
        const obj_responseId_union1_error = (() => {
            if (obj_responseId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_responseId + '" (at "' + path_responseId + '")');
            }
        })();
        if (obj_responseId_union1_error != null) {
            obj_responseId_union1 = obj_responseId_union1_error.message;
        }
        if (obj_responseId_union0 && obj_responseId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_responseId + '")';
            message += '\n' + obj_responseId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_responseId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$24 = function PhotoMetadataRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1V,
        private: [],
        selections: [
            {
                name: 'companyBluemasterId',
                kind: 'Scalar'
            },
            {
                name: 'responseId',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$v(existing, incoming) {
    const existing_companyBluemasterId = existing.companyBluemasterId;
    const incoming_companyBluemasterId = incoming.companyBluemasterId;
    if (!(existing_companyBluemasterId === incoming_companyBluemasterId)) {
        return false;
    }
    const existing_responseId = existing.responseId;
    const incoming_responseId = incoming.responseId;
    if (!(existing_responseId === incoming_responseId)) {
        return false;
    }
    return true;
}

const VERSION$1U = "34ffbf4219dfebb707bff609c2d5bf89";
function validate$W(obj, path = 'PhotoRecordAvatarRepresentation') {
    const validateAbstractRecordAvatarRepresentation_validateError = validate$U(obj, path);
    if (validateAbstractRecordAvatarRepresentation_validateError !== null) {
        return validateAbstractRecordAvatarRepresentation_validateError;
    }
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_backgroundColor = obj.backgroundColor;
        const path_backgroundColor = path + '.backgroundColor';
        let obj_backgroundColor_union0 = null;
        const obj_backgroundColor_union0_error = (() => {
            if (typeof obj_backgroundColor !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_backgroundColor + '" (at "' + path_backgroundColor + '")');
            }
        })();
        if (obj_backgroundColor_union0_error != null) {
            obj_backgroundColor_union0 = obj_backgroundColor_union0_error.message;
        }
        let obj_backgroundColor_union1 = null;
        const obj_backgroundColor_union1_error = (() => {
            if (obj_backgroundColor !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_backgroundColor + '" (at "' + path_backgroundColor + '")');
            }
        })();
        if (obj_backgroundColor_union1_error != null) {
            obj_backgroundColor_union1 = obj_backgroundColor_union1_error.message;
        }
        if (obj_backgroundColor_union0 && obj_backgroundColor_union1) {
            let message = 'Object doesn\'t match union (at "' + path_backgroundColor + '")';
            message += '\n' + obj_backgroundColor_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_backgroundColor_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_height = obj.height;
        const path_height = path + '.height';
        let obj_height_union0 = null;
        const obj_height_union0_error = (() => {
            if (typeof obj_height !== 'number' || (typeof obj_height === 'number' && Math.floor(obj_height) !== obj_height)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_height + '" (at "' + path_height + '")');
            }
        })();
        if (obj_height_union0_error != null) {
            obj_height_union0 = obj_height_union0_error.message;
        }
        let obj_height_union1 = null;
        const obj_height_union1_error = (() => {
            if (obj_height !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_height + '" (at "' + path_height + '")');
            }
        })();
        if (obj_height_union1_error != null) {
            obj_height_union1 = obj_height_union1_error.message;
        }
        if (obj_height_union0 && obj_height_union1) {
            let message = 'Object doesn\'t match union (at "' + path_height + '")';
            message += '\n' + obj_height_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_height_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_photoMetadata = obj.photoMetadata;
        const path_photoMetadata = path + '.photoMetadata';
        const referencepath_photoMetadataValidationError = validate$X(obj_photoMetadata, path_photoMetadata);
        if (referencepath_photoMetadataValidationError !== null) {
            let message = 'Object doesn\'t match PhotoMetadataRepresentation (at "' + path_photoMetadata + '")\n';
            message += referencepath_photoMetadataValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_photoUrl = obj.photoUrl;
        const path_photoUrl = path + '.photoUrl';
        if (typeof obj_photoUrl !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_photoUrl + '" (at "' + path_photoUrl + '")');
        }
        const obj_provider = obj.provider;
        const path_provider = path + '.provider';
        let obj_provider_union0 = null;
        const obj_provider_union0_error = (() => {
            if (typeof obj_provider !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_provider + '" (at "' + path_provider + '")');
            }
        })();
        if (obj_provider_union0_error != null) {
            obj_provider_union0 = obj_provider_union0_error.message;
        }
        let obj_provider_union1 = null;
        const obj_provider_union1_error = (() => {
            if (obj_provider !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_provider + '" (at "' + path_provider + '")');
            }
        })();
        if (obj_provider_union1_error != null) {
            obj_provider_union1 = obj_provider_union1_error.message;
        }
        if (obj_provider_union0 && obj_provider_union1) {
            let message = 'Object doesn\'t match union (at "' + path_provider + '")';
            message += '\n' + obj_provider_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_provider_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_width = obj.width;
        const path_width = path + '.width';
        let obj_width_union0 = null;
        const obj_width_union0_error = (() => {
            if (typeof obj_width !== 'number' || (typeof obj_width === 'number' && Math.floor(obj_width) !== obj_width)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_width + '" (at "' + path_width + '")');
            }
        })();
        if (obj_width_union0_error != null) {
            obj_width_union0 = obj_width_union0_error.message;
        }
        let obj_width_union1 = null;
        const obj_width_union1_error = (() => {
            if (obj_width !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_width + '" (at "' + path_width + '")');
            }
        })();
        if (obj_width_union1_error != null) {
            obj_width_union1 = obj_width_union1_error.message;
        }
        if (obj_width_union0 && obj_width_union1) {
            let message = 'Object doesn\'t match union (at "' + path_width + '")';
            message += '\n' + obj_width_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_width_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$t = 'PhotoRecordAvatarRepresentation';
function keyBuilderFromType$h(luvio, object) {
    return keyBuilderFromType$f(luvio, object);
}
function normalize$m(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$23 = function PhotoRecordAvatarRepresentationSelect() {
    const { selections: AbstractRecordAvatarRepresentationSelections } = select$21();
    const { selections: PhotoMetadataRepresentation__selections, opaque: PhotoMetadataRepresentation__opaque, } = select$24();
    return {
        kind: 'Fragment',
        version: VERSION$1U,
        private: [
            'eTag'
        ],
        selections: [
            ...AbstractRecordAvatarRepresentationSelections,
            {
                name: 'backgroundColor',
                kind: 'Scalar'
            },
            {
                name: 'height',
                kind: 'Scalar'
            },
            {
                name: 'photoMetadata',
                kind: 'Object',
                selections: PhotoMetadataRepresentation__selections
            },
            {
                name: 'photoUrl',
                kind: 'Scalar'
            },
            {
                name: 'provider',
                kind: 'Scalar'
            },
            {
                name: 'width',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$u(existing, incoming) {
    if (equals$s(existing, incoming) === false) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_photoUrl = existing.photoUrl;
    const incoming_photoUrl = incoming.photoUrl;
    if (!(existing_photoUrl === incoming_photoUrl)) {
        return false;
    }
    const existing_backgroundColor = existing.backgroundColor;
    const incoming_backgroundColor = incoming.backgroundColor;
    if (!(existing_backgroundColor === incoming_backgroundColor)) {
        return false;
    }
    const existing_height = existing.height;
    const incoming_height = incoming.height;
    if (!(existing_height === incoming_height)) {
        return false;
    }
    const existing_photoMetadata = existing.photoMetadata;
    const incoming_photoMetadata = incoming.photoMetadata;
    if (!(equals$v(existing_photoMetadata, incoming_photoMetadata))) {
        return false;
    }
    const existing_provider = existing.provider;
    const incoming_provider = incoming.provider;
    if (!(existing_provider === incoming_provider)) {
        return false;
    }
    const existing_width = existing.width;
    const incoming_width = incoming.width;
    if (!(existing_width === incoming_width)) {
        return false;
    }
    return true;
}
const ingest$1M = function PhotoRecordAvatarRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$W(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$h(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$m, "UiApi", VERSION$1U, RepresentationType$t, equals$u);
    return createLink$1(key);
};
function getTypeCacheKeys$1P(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$h(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$r,
        mergeable: false
    });
}

const VERSION$1T = "cbdcf69a6568d41ccf155b2b95ec70e7";
function validate$V(obj, path = 'ThemeRecordAvatarRepresentation') {
    const validateAbstractRecordAvatarRepresentation_validateError = validate$U(obj, path);
    if (validateAbstractRecordAvatarRepresentation_validateError !== null) {
        return validateAbstractRecordAvatarRepresentation_validateError;
    }
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_backgroundColor = obj.backgroundColor;
        const path_backgroundColor = path + '.backgroundColor';
        let obj_backgroundColor_union0 = null;
        const obj_backgroundColor_union0_error = (() => {
            if (typeof obj_backgroundColor !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_backgroundColor + '" (at "' + path_backgroundColor + '")');
            }
        })();
        if (obj_backgroundColor_union0_error != null) {
            obj_backgroundColor_union0 = obj_backgroundColor_union0_error.message;
        }
        let obj_backgroundColor_union1 = null;
        const obj_backgroundColor_union1_error = (() => {
            if (obj_backgroundColor !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_backgroundColor + '" (at "' + path_backgroundColor + '")');
            }
        })();
        if (obj_backgroundColor_union1_error != null) {
            obj_backgroundColor_union1 = obj_backgroundColor_union1_error.message;
        }
        if (obj_backgroundColor_union0 && obj_backgroundColor_union1) {
            let message = 'Object doesn\'t match union (at "' + path_backgroundColor + '")';
            message += '\n' + obj_backgroundColor_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_backgroundColor_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_iconUrl = obj.iconUrl;
        const path_iconUrl = path + '.iconUrl';
        let obj_iconUrl_union0 = null;
        const obj_iconUrl_union0_error = (() => {
            if (typeof obj_iconUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union0_error != null) {
            obj_iconUrl_union0 = obj_iconUrl_union0_error.message;
        }
        let obj_iconUrl_union1 = null;
        const obj_iconUrl_union1_error = (() => {
            if (obj_iconUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_iconUrl + '" (at "' + path_iconUrl + '")');
            }
        })();
        if (obj_iconUrl_union1_error != null) {
            obj_iconUrl_union1 = obj_iconUrl_union1_error.message;
        }
        if (obj_iconUrl_union0 && obj_iconUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_iconUrl + '")';
            message += '\n' + obj_iconUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_iconUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$s = 'ThemeRecordAvatarRepresentation';
function keyBuilderFromType$g(luvio, object) {
    return keyBuilderFromType$f(luvio, object);
}
function normalize$l(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$22 = function ThemeRecordAvatarRepresentationSelect() {
    const { selections: AbstractRecordAvatarRepresentationSelections } = select$21();
    return {
        kind: 'Fragment',
        version: VERSION$1T,
        private: [
            'eTag'
        ],
        selections: [
            ...AbstractRecordAvatarRepresentationSelections,
            {
                name: 'backgroundColor',
                kind: 'Scalar'
            },
            {
                name: 'iconUrl',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$t(existing, incoming) {
    if (equals$s(existing, incoming) === false) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_backgroundColor = existing.backgroundColor;
    const incoming_backgroundColor = incoming.backgroundColor;
    if (!(existing_backgroundColor === incoming_backgroundColor)) {
        return false;
    }
    const existing_iconUrl = existing.iconUrl;
    const incoming_iconUrl = incoming.iconUrl;
    if (!(existing_iconUrl === incoming_iconUrl)) {
        return false;
    }
    return true;
}
const ingest$1L = function ThemeRecordAvatarRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$V(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$g(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$l, "UiApi", VERSION$1T, RepresentationType$s, equals$t);
    return createLink$1(key);
};
function getTypeCacheKeys$1O(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$g(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$r,
        mergeable: false
    });
}

var DiscriminatorValues$3;
(function (DiscriminatorValues) {
    DiscriminatorValues["Photo"] = "Photo";
    DiscriminatorValues["Theme"] = "Theme";
})(DiscriminatorValues$3 || (DiscriminatorValues$3 = {}));
const VERSION$1S = "bc05f8d1c708b6c41c9bf2fe6001aff0";
function validate$U(obj, path = 'AbstractRecordAvatarRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_recordId = obj.recordId;
        const path_recordId = path + '.recordId';
        if (typeof obj_recordId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_recordId + '" (at "' + path_recordId + '")');
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$r = 'AbstractRecordAvatarRepresentation';
function keyBuilder$2b(luvio, config) {
    return keyPrefix + '::' + RepresentationType$r + ':' + config.recordId;
}
function keyBuilderFromType$f(luvio, object) {
    const keyParams = {
        recordId: object.recordId
    };
    return keyBuilder$2b(luvio, keyParams);
}
const selectChildren$1 = function AbstractRecordAvatarRepresentationSelectChildren() {
    const photoRecordAvatarRepresentationSelections = select$23();
    const themeRecordAvatarRepresentationSelections = select$22();
    return {
        kind: 'Fragment',
        union: true,
        discriminator: 'type',
        unionSelections: {
            [DiscriminatorValues$3.Photo]: photoRecordAvatarRepresentationSelections,
            [DiscriminatorValues$3.Theme]: themeRecordAvatarRepresentationSelections
        }
    };
};
const select$21 = function AbstractRecordAvatarRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1S,
        private: [],
        selections: [
            {
                name: 'recordId',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$s(existing, incoming) {
    const existing_recordId = existing.recordId;
    const incoming_recordId = incoming.recordId;
    if (!(existing_recordId === incoming_recordId)) {
        return false;
    }
    const existing_type = existing.type;
    const incoming_type = incoming.type;
    if (!(existing_type === incoming_type)) {
        return false;
    }
    return true;
}
const discriminatorIngest$1 = function AbstractRecordAvatarRepresentationDiscriminatorIngest(input, path, luvio, store, timestamp) {
    const discriminatorValue = input.type;
    if (discriminatorValue === 'Photo') {
        return ingest$1M(input, path, luvio, store, timestamp);
    }
    if (discriminatorValue === 'Theme') {
        return ingest$1L(input, path, luvio, store, timestamp);
    }
    throw new Error(`Invalid discriminatorValue "${discriminatorValue}". Expected one of "Photo","Theme"`);
};
function getTypeCacheKeys$1N(rootKeySet, luvio, input, fullPathFactory) {
    const discriminatorValue = input.type;
    if (discriminatorValue === 'Photo') {
        return getTypeCacheKeys$1P(rootKeySet, luvio, input);
    }
    if (discriminatorValue === 'Theme') {
        return getTypeCacheKeys$1O(rootKeySet, luvio, input);
    }
    throw new Error(`Invalid discriminatorValue "${discriminatorValue}". Expected one of "Photo","Theme"`);
}

const VERSION$1R = "f531b44760afecfce89990fe5a5c5c29";
function validate$T(obj, path = 'RecordAvatarBatchRepresentation') {
    const validateAbstractRecordAvatarBatchRepresentation_validateError = validate$P(obj, path);
    if (validateAbstractRecordAvatarBatchRepresentation_validateError !== null) {
        return validateAbstractRecordAvatarBatchRepresentation_validateError;
    }
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_result = obj.result;
        const path_result = path + '.result';
        if (typeof obj_result !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_result + '" (at "' + path_result + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$q = 'RecordAvatarBatchRepresentation';
function normalize$k(input, existing, path, luvio, store, timestamp) {
    const input_result = input.result;
    const input_result_id = path.fullPath + '__result';
    input.result = discriminatorIngest$1(input_result, {
        fullPath: input_result_id,
        propertyName: 'result',
        parent: {
            data: input,
            key: path.fullPath,
            existing: existing,
        },
        ttl: path.ttl
    }, luvio, store, timestamp);
    return input;
}
const select$20 = function RecordAvatarBatchRepresentationSelect() {
    const { selections: AbstractRecordAvatarBatchRepresentationSelections } = select$1Y();
    return {
        kind: 'Fragment',
        version: VERSION$1R,
        private: [],
        selections: [
            ...AbstractRecordAvatarBatchRepresentationSelections,
            {
                kind: 'Link',
                name: 'result',
                fragment: selectChildren$1()
            }
        ]
    };
};
function equals$r(existing, incoming) {
    if (equals$n(existing, incoming) === false) {
        return false;
    }
    const existing_result = existing.result;
    const incoming_result = incoming.result;
    if (!(existing_result.__ref === incoming_result.__ref)) {
        return false;
    }
    return true;
}
const ingest$1K = function RecordAvatarBatchRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$T(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$k;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$k, "UiApi", VERSION$1R, RepresentationType$q, equals$r);
    return createLink$1(key);
};
function getTypeCacheKeys$1M(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$q,
        mergeable: false
    });
}

const VERSION$1Q = "17c20b01167dba9a81452e60bb28b2f2";
function validate$S(obj, path = 'ErrorSingleRecordAvatarRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_errorCode = obj.errorCode;
        const path_errorCode = path + '.errorCode';
        if (typeof obj_errorCode !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_errorCode + '" (at "' + path_errorCode + '")');
        }
        const obj_message = obj.message;
        const path_message = path + '.message';
        if (typeof obj_message !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_message + '" (at "' + path_message + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$1$ = function ErrorSingleRecordAvatarRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1Q,
        private: [],
        selections: [
            {
                name: 'errorCode',
                kind: 'Scalar'
            },
            {
                name: 'message',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$q(existing, incoming) {
    const existing_errorCode = existing.errorCode;
    const incoming_errorCode = incoming.errorCode;
    if (!(existing_errorCode === incoming_errorCode)) {
        return false;
    }
    const existing_message = existing.message;
    const incoming_message = incoming.message;
    if (!(existing_message === incoming_message)) {
        return false;
    }
    return true;
}

const VERSION$1P = "e7fac2bfb5cdacc5e0f15c350f9ab8fe";
function validate$R(obj, path = 'ErrorBadRequestRecordAvatarBatchRepresentation') {
    const validateAbstractRecordAvatarBatchRepresentation_validateError = validate$P(obj, path);
    if (validateAbstractRecordAvatarBatchRepresentation_validateError !== null) {
        return validateAbstractRecordAvatarBatchRepresentation_validateError;
    }
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_result = obj.result;
        const path_result = path + '.result';
        if (!ArrayIsArray(obj_result)) {
            return new TypeError('Expected "array" but received "' + typeof obj_result + '" (at "' + path_result + '")');
        }
        for (let i = 0; i < obj_result.length; i++) {
            const obj_result_item = obj_result[i];
            const path_result_item = path_result + '[' + i + ']';
            const referencepath_result_itemValidationError = validate$S(obj_result_item, path_result_item);
            if (referencepath_result_itemValidationError !== null) {
                let message = 'Object doesn\'t match ErrorSingleRecordAvatarRepresentation (at "' + path_result_item + '")\n';
                message += referencepath_result_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$p = 'ErrorBadRequestRecordAvatarBatchRepresentation';
function normalize$j(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1_ = function ErrorBadRequestRecordAvatarBatchRepresentationSelect() {
    const { selections: AbstractRecordAvatarBatchRepresentationSelections } = select$1Y();
    const { selections: ErrorSingleRecordAvatarRepresentation__selections, opaque: ErrorSingleRecordAvatarRepresentation__opaque, } = select$1$();
    return {
        kind: 'Fragment',
        version: VERSION$1P,
        private: [],
        selections: [
            ...AbstractRecordAvatarBatchRepresentationSelections,
            {
                name: 'result',
                kind: 'Object',
                plural: true,
                selections: ErrorSingleRecordAvatarRepresentation__selections
            }
        ]
    };
};
function equals$p(existing, incoming) {
    if (equals$n(existing, incoming) === false) {
        return false;
    }
    const existing_result = existing.result;
    const incoming_result = incoming.result;
    const equals_result_items = equalsArray(existing_result, incoming_result, (existing_result_item, incoming_result_item) => {
        if (!(equals$q(existing_result_item, incoming_result_item))) {
            return false;
        }
    });
    if (equals_result_items === false) {
        return false;
    }
    return true;
}
const ingest$1J = function ErrorBadRequestRecordAvatarBatchRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$R(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$k;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$j, "UiApi", VERSION$1P, RepresentationType$p, equals$p);
    return createLink$1(key);
};
function getTypeCacheKeys$1L(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$p,
        mergeable: false
    });
}

const VERSION$1O = "cbd48fd50d0db463fdff7b1834c22bed";
function validate$Q(obj, path = 'ErrorRecordAvatarBatchRepresentation') {
    const validateAbstractRecordAvatarBatchRepresentation_validateError = validate$P(obj, path);
    if (validateAbstractRecordAvatarBatchRepresentation_validateError !== null) {
        return validateAbstractRecordAvatarBatchRepresentation_validateError;
    }
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_result = obj.result;
        const path_result = path + '.result';
        if (!ArrayIsArray(obj_result)) {
            return new TypeError('Expected "array" but received "' + typeof obj_result + '" (at "' + path_result + '")');
        }
        for (let i = 0; i < obj_result.length; i++) {
            const obj_result_item = obj_result[i];
            const path_result_item = path_result + '[' + i + ']';
            const referencepath_result_itemValidationError = validate$S(obj_result_item, path_result_item);
            if (referencepath_result_itemValidationError !== null) {
                let message = 'Object doesn\'t match ErrorSingleRecordAvatarRepresentation (at "' + path_result_item + '")\n';
                message += referencepath_result_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$o = 'ErrorRecordAvatarBatchRepresentation';
function normalize$i(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1Z = function ErrorRecordAvatarBatchRepresentationSelect() {
    const { selections: AbstractRecordAvatarBatchRepresentationSelections } = select$1Y();
    const { selections: ErrorSingleRecordAvatarRepresentation__selections, opaque: ErrorSingleRecordAvatarRepresentation__opaque, } = select$1$();
    return {
        kind: 'Fragment',
        version: VERSION$1O,
        private: [],
        selections: [
            ...AbstractRecordAvatarBatchRepresentationSelections,
            {
                name: 'result',
                kind: 'Object',
                plural: true,
                selections: ErrorSingleRecordAvatarRepresentation__selections
            }
        ]
    };
};
function equals$o(existing, incoming) {
    if (equals$n(existing, incoming) === false) {
        return false;
    }
    const existing_result = existing.result;
    const incoming_result = incoming.result;
    const equals_result_items = equalsArray(existing_result, incoming_result, (existing_result_item, incoming_result_item) => {
        if (!(equals$q(existing_result_item, incoming_result_item))) {
            return false;
        }
    });
    if (equals_result_items === false) {
        return false;
    }
    return true;
}
const ingest$1I = function ErrorRecordAvatarBatchRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$Q(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$k;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$i, "UiApi", VERSION$1O, RepresentationType$o, equals$o);
    return createLink$1(key);
};
function getTypeCacheKeys$1K(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$o,
        mergeable: false
    });
}

const DiscriminatorValues$2 = {
    '200': 200,
    '400': 400,
    '404': 404
};
const TTL$k = 300000;
const VERSION$1N = "8956293536e94d5ec63b274b61033d2c";
function validate$P(obj, path = 'AbstractRecordAvatarBatchRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_statusCode = obj.statusCode;
        const path_statusCode = path + '.statusCode';
        if (typeof obj_statusCode !== 'number') {
            return new TypeError('Expected "number" but received "' + typeof obj_statusCode + '" (at "' + path_statusCode + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const selectChildren = function AbstractRecordAvatarBatchRepresentationSelectChildren() {
    const recordAvatarBatchRepresentationSelections = select$20();
    const errorBadRequestRecordAvatarBatchRepresentationSelections = select$1_();
    const errorRecordAvatarBatchRepresentationSelections = select$1Z();
    return {
        kind: 'Fragment',
        union: true,
        discriminator: 'statusCode',
        unionSelections: {
            [DiscriminatorValues$2['200']]: recordAvatarBatchRepresentationSelections,
            [DiscriminatorValues$2['400']]: errorBadRequestRecordAvatarBatchRepresentationSelections,
            [DiscriminatorValues$2['404']]: errorRecordAvatarBatchRepresentationSelections
        }
    };
};
const select$1Y = function AbstractRecordAvatarBatchRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1N,
        private: [],
        selections: [
            {
                name: 'statusCode',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$n(existing, incoming) {
    const existing_statusCode = existing.statusCode;
    const incoming_statusCode = incoming.statusCode;
    if (!(existing_statusCode === incoming_statusCode)) {
        return false;
    }
    return true;
}
const discriminatorIngest = function AbstractRecordAvatarBatchRepresentationDiscriminatorIngest(input, path, luvio, store, timestamp) {
    const discriminatorValue = input.statusCode;
    if (discriminatorValue === 200) {
        return ingest$1K(input, path, luvio, store, timestamp);
    }
    if (discriminatorValue === 400) {
        return ingest$1J(input, path, luvio, store, timestamp);
    }
    if (discriminatorValue === 404) {
        return ingest$1I(input, path, luvio, store, timestamp);
    }
    throw new Error(`Invalid discriminatorValue "${discriminatorValue}". Expected one of "200","400","404"`);
};
function getTypeCacheKeys$1J(rootKeySet, luvio, input, fullPathFactory) {
    const discriminatorValue = input.statusCode;
    if (discriminatorValue === 200) {
        return getTypeCacheKeys$1M(rootKeySet, luvio, input, fullPathFactory);
    }
    if (discriminatorValue === 400) {
        return getTypeCacheKeys$1L(rootKeySet, luvio, input, fullPathFactory);
    }
    if (discriminatorValue === 404) {
        return getTypeCacheKeys$1K(rootKeySet, luvio, input, fullPathFactory);
    }
    throw new Error(`Invalid discriminatorValue "${discriminatorValue}". Expected one of "200","400","404"`);
}

const TTL$j = 3600000;
const VERSION$1M = "c44c049fa6ad7cf7e932c0aab9107d86";
function validate$O(obj, path = 'RecordAvatarBulkMapRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_keys = ObjectKeys(obj);
        for (let i = 0; i < obj_keys.length; i++) {
            const key = obj_keys[i];
            const obj_prop = obj[key];
            const path_prop = path + '["' + key + '"]';
            if (typeof obj_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_prop + '" (at "' + path_prop + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$n = 'RecordAvatarBulkMapRepresentation';
function normalize$h(input, existing, path, luvio, store, timestamp) {
    const input_keys = Object.keys(input);
    const input_length = input_keys.length;
    for (let i = 0; i < input_length; i++) {
        const key = input_keys[i];
        const input_prop = input[key];
        const input_prop_id = path.fullPath + '__' + key;
        input[key] = discriminatorIngest(input_prop, {
            fullPath: input_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
function equals$m(existing, incoming) {
    const equals_props = equalsObject(existing, incoming, (existing_prop, incoming_prop) => {
        if (!(existing_prop.__ref === incoming_prop.__ref)) {
            return false;
        }
    });
    if (equals_props === false) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$1I(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$n,
        mergeable: false
    });
    const input__root = input;
    const input__root_keys = ObjectKeys(input__root);
    const input__root_length = input__root_keys.length;
    for (let i = 0; i < input__root_length; i++) {
        const key = input__root_keys[i];
        getTypeCacheKeys$1J(rootKeySet, luvio, input__root[key], () => rootKey + "__" + key);
    }
}

function merge(existing, incoming, _luvio, _path) {
    if (existing === undefined) {
        return incoming;
    }
    // Merge RecordRepresentation field values together
    return {
        ...existing,
        ...incoming,
    };
}

const ingest$1H = function RecordAvatarBulkMapRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$O(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const existingRecord = store.readEntry(key);
    let incomingRecord = normalize$h(input, store.readEntry(key), {
        fullPath: key,
        propertyName: path.propertyName,
        parent: path.parent,
    }, luvio, store, timestamp);
    incomingRecord = merge(existingRecord, incomingRecord);
    if (existingRecord === undefined || equals$m(existingRecord, incomingRecord) === false) {
        luvio.storePublish(key, incomingRecord);
    }
    return createLink$1(key);
};

function keyBuilder$2a(luvio, params) {
    return keyPrefix + '::RecordAvatarBulkMapRepresentation:(' + 'formFactor:' + params.queryParams.formFactor + ',' + 'recordIds:' + params.urlParams.recordIds + ')';
}
function getResponseCacheKeys$t(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1I(storeKeyMap, luvio, response, () => keyBuilder$2a(luvio, resourceParams));
}
function ingestError$h(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2a(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$j,
        namespace: keyPrefix,
        version: VERSION$1M,
        representationName: RepresentationType$n
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$y(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/record-avatars/batch/' + config.urlParams.recordIds + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

// All of the avatars are ingested into
// the same top level object
const KEY = `${keyPrefix}RecordAvatarsBulk`;
function recordAvatarsFragment(recordIds) {
    return {
        kind: 'Fragment',
        private: [],
        version: VERSION$1N,
        selections: recordIds.map((recordId) => {
            return {
                kind: 'Link',
                name: recordId,
                fragment: selectChildren(),
            };
        }),
    };
}
function recordAvatarsSelector(recordIds) {
    return {
        recordId: KEY,
        node: recordAvatarsFragment(recordIds),
        variables: {},
    };
}

function buildCachedSnapshot$3(luvio, config) {
    const { recordIds } = config;
    return luvio.storeLookup(recordAvatarsSelector(recordIds), {
        config,
        resolve: () => buildNetworkSnapshot$w(luvio, config, snapshotRefreshOptions),
    });
}

// Track in-flight xrequests so we known when to send out our fake response
const IN_FLIGHT_REQUESTS = new Set();
function getRecordIdsFlightStatus(recordIds, in_flight_requests) {
    const recordIdsNotInFlight = [], recordIdsInFlight = [];
    // Split up the given record ids into in flight ids and not in flight ids
    recordIds.forEach((id) => {
        if (in_flight_requests.has(id)) {
            recordIdsInFlight.push(id);
        }
        else {
            recordIdsNotInFlight.push(id);
        }
    });
    return {
        recordIdsInFlight: recordIdsInFlight,
        recordIdsNotInFlight: recordIdsNotInFlight,
    };
}
/**
 * TODO [W-8318817]: For 230/232 this is a workaround to not having inflight
 * deduping for record avatars.
 * The way it works is we ingest a fake response while the real response is pending.
 * So for example a component wires record avatars for ABC, while that request is in flight
 * another component wires record avatars AB. The wire for ABC will send off a request for data
 * Then AB comes in and will resolve the fake response instead of sending off a request.
 * Then once the real response comes back it will be ingested and it will notify both AB and ABC.
 * We will resolve this in 234 using real inflight deduping
 */
function ingestFakeResponse(luvio, recordIds) {
    const formatted = reduce.call(recordIds, (accum, recordId) => {
        accum[recordId] = {
            statusCode: 200,
            result: {
                backgroundColor: null,
                eTag: '',
                height: null,
                photoMetadata: { companyBluemasterId: null, responseId: null },
                photoUrl: '',
                provider: null,
                recordId: '',
                type: 'Photo',
                width: null,
            },
        };
        return accum;
    }, {});
    luvio.storeIngest(KEY, ingest$1H, formatted);
}
function isRecordAvatarBulkMapRepresentation(response) {
    return response.body.hasErrors === undefined;
}
function buildNetworkSnapshot$w(luvio, config, options) {
    const { uncachedRecordIds, recordIds } = config;
    const { recordIdsInFlight, recordIdsNotInFlight } = getRecordIdsFlightStatus(uncachedRecordIds || recordIds, // If uncached records were specified, only get those, otherwise, get all of them
    IN_FLIGHT_REQUESTS);
    // For any remaining record ids send off the real request and add it to the in flight request list
    let luvioResponse;
    if (recordIdsNotInFlight.length > 0) {
        recordIdsNotInFlight.forEach((id) => IN_FLIGHT_REQUESTS.add(id));
        const resourceParams = createResourceParams$t({ ...config, recordIds: recordIdsNotInFlight });
        const request = createResourceRequest$y(resourceParams);
        luvioResponse = luvio
            .dispatchResourceRequest(request, options)
            .then((response) => {
            recordIdsNotInFlight.forEach((id) => IN_FLIGHT_REQUESTS.delete(id));
            // the selector passed to dispatchResourceRequest requests the data already formatted so the response
            // can either be a RecordAvatarBulkRepresentation or a RecordAvatarBulkMapRepresentation
            let formatted;
            if (isRecordAvatarBulkMapRepresentation(response)) {
                formatted = response.body;
            }
            else {
                formatted = response.body.results.reduce((seed, avatar, index) => {
                    const recordId = recordIdsNotInFlight[index];
                    seed[recordId] = avatar;
                    return seed;
                }, {});
            }
            response.body = formatted;
            return luvio.handleSuccessResponse(() => onFetchResponseSuccess$j(luvio, config, resourceParams, response), () => {
                const cache = new StoreKeyMap();
                getResponseCacheKeys$t(cache, luvio, resourceParams, formatted);
                return cache;
            });
        }, (err) => {
            recordIdsNotInFlight.forEach((id) => IN_FLIGHT_REQUESTS.delete(id));
            return luvio.handleErrorResponse(() => {
                return onFetchResponseError$j(luvio, config, resourceParams, err);
            });
        });
    }
    // For any currently in flight record ids lets emit a fake response
    if (recordIdsInFlight.length > 0) {
        ingestFakeResponse(luvio, recordIdsInFlight);
    }
    return luvioResponse ? luvioResponse : Promise.resolve(buildCachedSnapshot$3(luvio, config));
}

function ingestSuccess$n(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    luvio.storeIngest(KEY, ingest$1H, body);
    let recordIds = [];
    if (snapshotRefresh) {
        recordIds = snapshotRefresh.config.recordIds;
    }
    const snapshot = luvio.storeLookup(recordAvatarsSelector(recordIds), snapshotRefresh);
    if (snapshot.state === 'Pending') {
        luvio.resolvePendingSnapshot(snapshot).then(() => {
            ingestSuccess$n(luvio, resourceParams, response, snapshotRefresh);
        });
        return Promise.resolve(snapshot);
    }
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    return Promise.resolve(snapshot);
}

function buildCachedSnapshotCachePolicy$l(context, storeLookup) {
    const { luvio, config } = context;
    const { recordIds } = config;
    const cachedSnapshot = storeLookup(recordAvatarsSelector(recordIds), {
        config,
        resolve: () => buildNetworkSnapshot$w(luvio, config, snapshotRefreshOptions),
    });
    // if the L1 lookup had some data but not all, then put the cache keys that were
    // missing onto the context, so buildNetworkSnapshotCachePolicy only requests
    // the missing records
    if (cachedSnapshot.data !== undefined && isUnfulfilledSnapshot(cachedSnapshot)) {
        context.uncachedRecordIds = cachedSnapshot.missingPaths.keysAsArray().sort();
    }
    return cachedSnapshot;
}

function buildNetworkSnapshotCachePolicy$m(context, coercedAdapterRequestContext) {
    const { luvio, config, uncachedRecordIds } = context;
    if (uncachedRecordIds !== undefined) {
        config.uncachedRecordIds = uncachedRecordIds;
    }
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$w(luvio, config, dispatchOptions);
}

const adapterName$p = 'getRecordAvatars';
const getRecordAvatars_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordIds', true, 0 /* UrlParameter */, 0 /* String */, true, getRecordId18Array),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
];
const getRecordAvatars_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$p, getRecordAvatars_ConfigPropertyMetadata);
const createResourceParams$t = /*#__PURE__*/ createResourceParams$k(getRecordAvatars_ConfigPropertyMetadata);
function typeCheckConfig$v(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordAvatars_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$v(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordAvatars_ConfigPropertyMetadata);
    const config = typeCheckConfig$v(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function onFetchResponseSuccess$j(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$n(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$w(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$j(luvio, config, resourceParams, response) {
    const snapshot = ingestError$h(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$w(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
const getRecordAvatarsAdapterFactory = (luvio) => function UiApi__getRecordAvatars(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$v(untrustedConfig, getRecordAvatars_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$l, buildNetworkSnapshotCachePolicy$m);
};

function coerceRecordId18Array(value) {
    const valueArray = ArrayIsArray$1(value) ? value : [value];
    const ret = [];
    for (let i = 0, len = valueArray.length; i < len; i += 1) {
        const item = valueArray[i];
        const coerced = getRecordId18(item);
        if (coerced === undefined) {
            return undefined;
        }
        ArrayPrototypePush.call(ret, coerced);
    }
    if (ret.length === 0) {
        return undefined;
    }
    return dedupe(ret);
}
function coerceConfig(config) {
    const coercedRecordsConfig = { records: [] };
    const { records: recordsFromConfig = [] } = config;
    for (let index = 0, len = recordsFromConfig.length; index < len; index += 1) {
        const coeredRecordConfig = {};
        const currentRecordBatch = recordsFromConfig[index];
        if (untrustedIsObject(currentRecordBatch)) {
            const recordIds = coerceRecordId18Array(currentRecordBatch.recordIds);
            if (recordIds !== undefined) {
                coeredRecordConfig.recordIds = recordIds;
            }
            const optionalFields = getFieldApiNamesArray(currentRecordBatch.optionalFields);
            if (optionalFields !== undefined) {
                coeredRecordConfig.optionalFields = optionalFields;
            }
            const fields = getFieldApiNamesArray(currentRecordBatch.fields);
            if (fields !== undefined) {
                coeredRecordConfig.fields = fields;
            }
            ArrayPrototypePush.call(coercedRecordsConfig.records, coeredRecordConfig);
        }
    }
    return coercedRecordsConfig;
}
function typeCheckConfig$u(untrustedConfig) {
    const coercedConfig = coerceConfig(untrustedConfig);
    const config = {};
    const untrustedConfigs_records = coercedConfig.records;
    if (ArrayIsArray$1(untrustedConfigs_records)) {
        const records = [];
        for (let index = 0, len = untrustedConfigs_records.length; index < len; index += 1) {
            const output_recordEntityConfig = {};
            const untrustedConfig_recordEntity = untrustedConfigs_records[index];
            if (untrustedIsObject(untrustedConfig_recordEntity)) {
                const { recordIds: untrustedConfig_recordIds, fields: untrustedConfig_fields, optionalFields: untrustedConfig_optionalFields, } = untrustedConfig_recordEntity;
                if (ArrayIsArray$1(untrustedConfig_recordIds)) {
                    const untrustedConfig_recordIds_array = [];
                    for (let i = 0, arrayLength = untrustedConfig_recordIds.length; i < arrayLength; i++) {
                        const untrustedConfig_recordIds_item = untrustedConfig_recordIds[i];
                        if (typeof untrustedConfig_recordIds_item === 'string') {
                            ArrayPrototypePush.call(untrustedConfig_recordIds_array, untrustedConfig_recordIds_item);
                        }
                    }
                    output_recordEntityConfig.recordIds = untrustedConfig_recordIds_array;
                }
                if (!output_recordEntityConfig.recordIds ||
                    output_recordEntityConfig.recordIds.length === 0) {
                    continue;
                }
                if (ArrayIsArray$1(untrustedConfig_fields)) {
                    const untrustedConfig_fields_array = [];
                    for (let i = 0, arrayLength = untrustedConfig_fields.length; i < arrayLength; i++) {
                        const untrustedConfig_fields_item = untrustedConfig_fields[i];
                        if (typeof untrustedConfig_fields_item === 'string') {
                            ArrayPrototypePush.call(untrustedConfig_fields_array, untrustedConfig_fields_item);
                        }
                    }
                    output_recordEntityConfig.fields = untrustedConfig_fields_array;
                }
                if (ArrayIsArray$1(untrustedConfig_optionalFields)) {
                    const untrustedConfig_optionalFields_array = [];
                    for (let i = 0, arrayLength = untrustedConfig_optionalFields.length; i < arrayLength; i++) {
                        const untrustedConfig_optionalFields_item = untrustedConfig_optionalFields[i];
                        if (typeof untrustedConfig_optionalFields_item === 'string') {
                            ArrayPrototypePush.call(untrustedConfig_optionalFields_array, untrustedConfig_optionalFields_item);
                        }
                    }
                    output_recordEntityConfig.optionalFields = untrustedConfig_optionalFields_array;
                }
                if (output_recordEntityConfig.fields === undefined &&
                    output_recordEntityConfig.optionalFields === undefined) {
                    continue;
                }
                ArrayPrototypePush.call(records, output_recordEntityConfig);
            }
        }
        if (records.length > 0) {
            assign(config, { records });
        }
    }
    return config;
}

function checkFieldsObjectApiName(fields, apiName) {
    return fields.every((field) => splitQualifiedFieldApiName(field)[0] === apiName);
}
function getObjectApiName(fields, optionalFields) {
    if (fields === undefined || fields.length === 0) {
        return splitQualifiedFieldApiName(optionalFields[0])[0];
    }
    return splitQualifiedFieldApiName(fields[0])[0];
}
function validateFieldsObjectApiName(config) {
    const configRecords = config.records;
    for (let index = 0, len = configRecords.length; index < len; index++) {
        const { fields: configRecordEntityFields, optionalFields: configRecordEntityOptionalFields, } = configRecords[index];
        const objectApiName = getObjectApiName(configRecordEntityFields, configRecordEntityOptionalFields);
        // check objectApiName of fields and optionalFields array
        if ((configRecordEntityFields !== undefined &&
            !checkFieldsObjectApiName(configRecordEntityFields, objectApiName)) ||
            (configRecordEntityOptionalFields !== undefined &&
                !checkFieldsObjectApiName(configRecordEntityOptionalFields, objectApiName))) {
            throw new TypeError(`all specified fields and optional fields must be of same ObjectApiName as ${objectApiName}`);
        }
    }
}
function validateAdapterConfig$u(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$u(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    // If we have made it this far, our config is validated and type checked
    if (process.env.NODE_ENV !== 'production') {
        validateFieldsObjectApiName(config);
    }
    return config;
}

const getRecords_ConfigPropertyNames = {
    displayName: 'getRecords',
    parameters: {
        required: ['records'],
        optional: [],
    },
};

function keyBuilder$29(luvio, config) {
    const resourceParams = createResourceParams$14(config);
    return keyBuilder$3C(luvio, resourceParams);
}
function onFetchResponseError$i(luvio, config, resourceParams, response) {
    const snapshot = ingestError$Q(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$v(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$v(luvio, config, options) {
    const resourceParams = createResourceParams$14(config);
    const request = createResourceRequest$1f(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$S(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$16(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$i(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$l(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$v, undefined, false);
}
function buildCachedSnapshotCachePolicy$k(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$29(luvio, config),
        node: adapterFragment$O(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$v(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRecordsAdapterFactory = (luvio) => function UiApi__getRecords(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$u(untrustedConfig, getRecords_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$k, buildNetworkSnapshotCachePolicy$l);
};

const VERSION$1L = "4d4bdab5d0aa61e365e766409c9b3a31";
function validate$N(obj, path = 'RelatedListReferenceRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        let obj_id_union0 = null;
        const obj_id_union0_error = (() => {
            if (typeof obj_id !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union0_error != null) {
            obj_id_union0 = obj_id_union0_error.message;
        }
        let obj_id_union1 = null;
        const obj_id_union1_error = (() => {
            if (obj_id !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
            }
        })();
        if (obj_id_union1_error != null) {
            obj_id_union1 = obj_id_union1_error.message;
        }
        if (obj_id_union0 && obj_id_union1) {
            let message = 'Object doesn\'t match union (at "' + path_id + '")';
            message += '\n' + obj_id_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_id_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_inContextOfRecordId = obj.inContextOfRecordId;
        const path_inContextOfRecordId = path + '.inContextOfRecordId';
        let obj_inContextOfRecordId_union0 = null;
        const obj_inContextOfRecordId_union0_error = (() => {
            if (typeof obj_inContextOfRecordId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_inContextOfRecordId + '" (at "' + path_inContextOfRecordId + '")');
            }
        })();
        if (obj_inContextOfRecordId_union0_error != null) {
            obj_inContextOfRecordId_union0 = obj_inContextOfRecordId_union0_error.message;
        }
        let obj_inContextOfRecordId_union1 = null;
        const obj_inContextOfRecordId_union1_error = (() => {
            if (obj_inContextOfRecordId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_inContextOfRecordId + '" (at "' + path_inContextOfRecordId + '")');
            }
        })();
        if (obj_inContextOfRecordId_union1_error != null) {
            obj_inContextOfRecordId_union1 = obj_inContextOfRecordId_union1_error.message;
        }
        if (obj_inContextOfRecordId_union0 && obj_inContextOfRecordId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_inContextOfRecordId + '")';
            message += '\n' + obj_inContextOfRecordId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_inContextOfRecordId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_listViewApiName = obj.listViewApiName;
        const path_listViewApiName = path + '.listViewApiName';
        let obj_listViewApiName_union0 = null;
        const obj_listViewApiName_union0_error = (() => {
            if (typeof obj_listViewApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_listViewApiName + '" (at "' + path_listViewApiName + '")');
            }
        })();
        if (obj_listViewApiName_union0_error != null) {
            obj_listViewApiName_union0 = obj_listViewApiName_union0_error.message;
        }
        let obj_listViewApiName_union1 = null;
        const obj_listViewApiName_union1_error = (() => {
            if (obj_listViewApiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_listViewApiName + '" (at "' + path_listViewApiName + '")');
            }
        })();
        if (obj_listViewApiName_union1_error != null) {
            obj_listViewApiName_union1 = obj_listViewApiName_union1_error.message;
        }
        if (obj_listViewApiName_union0 && obj_listViewApiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_listViewApiName + '")';
            message += '\n' + obj_listViewApiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_listViewApiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        let obj_objectApiName_union0 = null;
        const obj_objectApiName_union0_error = (() => {
            if (typeof obj_objectApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
            }
        })();
        if (obj_objectApiName_union0_error != null) {
            obj_objectApiName_union0 = obj_objectApiName_union0_error.message;
        }
        let obj_objectApiName_union1 = null;
        const obj_objectApiName_union1_error = (() => {
            if (obj_objectApiName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
            }
        })();
        if (obj_objectApiName_union1_error != null) {
            obj_objectApiName_union1 = obj_objectApiName_union1_error.message;
        }
        if (obj_objectApiName_union0 && obj_objectApiName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_objectApiName + '")';
            message += '\n' + obj_objectApiName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_objectApiName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_parentObjectApiName = obj.parentObjectApiName;
        const path_parentObjectApiName = path + '.parentObjectApiName';
        if (typeof obj_parentObjectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_parentObjectApiName + '" (at "' + path_parentObjectApiName + '")');
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        let obj_recordTypeId_union0 = null;
        const obj_recordTypeId_union0_error = (() => {
            if (typeof obj_recordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union0_error != null) {
            obj_recordTypeId_union0 = obj_recordTypeId_union0_error.message;
        }
        let obj_recordTypeId_union1 = null;
        const obj_recordTypeId_union1_error = (() => {
            if (obj_recordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union1_error != null) {
            obj_recordTypeId_union1 = obj_recordTypeId_union1_error.message;
        }
        if (obj_recordTypeId_union0 && obj_recordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeId + '")';
            message += '\n' + obj_recordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_relatedListId = obj.relatedListId;
        const path_relatedListId = path + '.relatedListId';
        if (typeof obj_relatedListId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_relatedListId + '" (at "' + path_relatedListId + '")');
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const select$1X = function RelatedListReferenceRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1L,
        private: [],
        selections: [
            {
                name: 'id',
                kind: 'Scalar'
            },
            {
                name: 'inContextOfRecordId',
                kind: 'Scalar'
            },
            {
                name: 'listViewApiName',
                kind: 'Scalar'
            },
            {
                name: 'objectApiName',
                kind: 'Scalar'
            },
            {
                name: 'parentObjectApiName',
                kind: 'Scalar'
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'relatedListId',
                kind: 'Scalar'
            },
            {
                name: 'type',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$l(existing, incoming) {
    const existing_parentObjectApiName = existing.parentObjectApiName;
    const incoming_parentObjectApiName = incoming.parentObjectApiName;
    if (!(existing_parentObjectApiName === incoming_parentObjectApiName)) {
        return false;
    }
    const existing_relatedListId = existing.relatedListId;
    const incoming_relatedListId = incoming.relatedListId;
    if (!(existing_relatedListId === incoming_relatedListId)) {
        return false;
    }
    const existing_type = existing.type;
    const incoming_type = incoming.type;
    if (!(existing_type === incoming_type)) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_inContextOfRecordId = existing.inContextOfRecordId;
    const incoming_inContextOfRecordId = incoming.inContextOfRecordId;
    if (!(existing_inContextOfRecordId === incoming_inContextOfRecordId)) {
        return false;
    }
    const existing_listViewApiName = existing.listViewApiName;
    const incoming_listViewApiName = incoming.listViewApiName;
    if (!(existing_listViewApiName === incoming_listViewApiName)) {
        return false;
    }
    const existing_objectApiName = existing.objectApiName;
    const incoming_objectApiName = incoming.objectApiName;
    if (!(existing_objectApiName === incoming_objectApiName)) {
        return false;
    }
    const existing_recordTypeId = existing.recordTypeId;
    const incoming_recordTypeId = incoming.recordTypeId;
    if (!(existing_recordTypeId === incoming_recordTypeId)) {
        return false;
    }
    return true;
}

const VERSION$1K = "836b875b519813e7499efc62f0f1d04b";
function validate$M(obj, path = 'RelatedListRecordCountRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_count = obj.count;
        const path_count = path + '.count';
        let obj_count_union0 = null;
        const obj_count_union0_error = (() => {
            if (typeof obj_count !== 'number' || (typeof obj_count === 'number' && Math.floor(obj_count) !== obj_count)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_count + '" (at "' + path_count + '")');
            }
        })();
        if (obj_count_union0_error != null) {
            obj_count_union0 = obj_count_union0_error.message;
        }
        let obj_count_union1 = null;
        const obj_count_union1_error = (() => {
            if (obj_count !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_count + '" (at "' + path_count + '")');
            }
        })();
        if (obj_count_union1_error != null) {
            obj_count_union1 = obj_count_union1_error.message;
        }
        if (obj_count_union0 && obj_count_union1) {
            let message = 'Object doesn\'t match union (at "' + path_count + '")';
            message += '\n' + obj_count_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_count_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_hasMore = obj.hasMore;
        const path_hasMore = path + '.hasMore';
        if (typeof obj_hasMore !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_hasMore + '" (at "' + path_hasMore + '")');
        }
        const obj_listReference = obj.listReference;
        const path_listReference = path + '.listReference';
        const referencepath_listReferenceValidationError = validate$N(obj_listReference, path_listReference);
        if (referencepath_listReferenceValidationError !== null) {
            let message = 'Object doesn\'t match RelatedListReferenceRepresentation (at "' + path_listReference + '")\n';
            message += referencepath_listReferenceValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$m = 'RelatedListRecordCountRepresentation';
function keyBuilder$28(luvio, config) {
    return keyPrefix + '::' + RepresentationType$m + ':' + (config.parentRecordId === null ? '' : config.parentRecordId) + ':' + config.relatedListId;
}
function keyBuilderFromType$e(luvio, object) {
    const keyParams = {
        parentRecordId: object.listReference.inContextOfRecordId,
        relatedListId: object.listReference.relatedListId
    };
    return keyBuilder$28(luvio, keyParams);
}
function normalize$g(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1W = function RelatedListRecordCountRepresentationSelect() {
    const { selections: RelatedListReferenceRepresentation__selections, opaque: RelatedListReferenceRepresentation__opaque, } = select$1X();
    return {
        kind: 'Fragment',
        version: VERSION$1K,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'count',
                kind: 'Scalar'
            },
            {
                name: 'hasMore',
                kind: 'Scalar'
            },
            {
                name: 'listReference',
                kind: 'Object',
                selections: RelatedListReferenceRepresentation__selections
            }
        ]
    };
};
function equals$k(existing, incoming) {
    const existing_hasMore = existing.hasMore;
    const incoming_hasMore = incoming.hasMore;
    if (!(existing_hasMore === incoming_hasMore)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_count = existing.count;
    const incoming_count = incoming.count;
    if (!(existing_count === incoming_count)) {
        return false;
    }
    const existing_listReference = existing.listReference;
    const incoming_listReference = incoming.listReference;
    if (!(equals$l(existing_listReference, incoming_listReference))) {
        return false;
    }
    return true;
}
const ingest$1G = function RelatedListRecordCountRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$M(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$e(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$g, "UiApi", VERSION$1K, RepresentationType$m, equals$k);
    return createLink$1(key);
};
function getTypeCacheKeys$1H(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$e(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$m,
        mergeable: false
    });
}

function select$1V(luvio, params) {
    return select$1W();
}
function keyBuilder$27(luvio, params) {
    return keyBuilder$28(luvio, {
        parentRecordId: params.urlParams.parentRecordId,
        relatedListId: params.urlParams.relatedListId
    });
}
function getResponseCacheKeys$s(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1H(storeKeyMap, luvio, response);
}
function ingestSuccess$m(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$27(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1G, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1V(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$g(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$27(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$x(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-count/' + config.urlParams.parentRecordId + '/' + config.urlParams.relatedListId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const nonCachedErrors$4 = ObjectCreate$1(null);
function createChildResourceParams$5(resourceParams) {
    let childConfigs = [];
    for (let index = 0, len = resourceParams.urlParams.relatedListNames.length; index < len; index += 1) {
        const item = resourceParams.urlParams.relatedListNames[index];
        childConfigs.push({
            urlParams: {
                parentRecordId: resourceParams.urlParams.parentRecordId,
                relatedListId: item
            },
            queryParams: {
                maxCount: resourceParams.queryParams.maxCount
            }
        });
    }
    return childConfigs;
}
function selectChildResourceParams$4(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$26(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$27(luvio, childResource);
                const childFragment = select$1V();
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$4[childKey];
                            if (nonCachedError === undefined) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$1U(luvio, resourceParams) {
    const childResources = createChildResourceParams$5(resourceParams);
    return selectChildResourceParams$4(luvio, childResources, resourceParams);
}
function keyBuilder$26(luvio, params) {
    return keyPrefix + '::BatchRelatedListCountRepresentation:(' + 'maxCount:' + params.queryParams.maxCount + ',' + 'parentRecordId:' + params.urlParams.parentRecordId + ',' + 'relatedListNames:' + params.urlParams.relatedListNames + ')';
}
function getResponseCacheKeys$r(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$5(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$s(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$27(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$m,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$5(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$27(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$m(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$g(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$4[childKey] = { response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$4[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$l(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$5(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$26(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$5(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$1U(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$f(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$26(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$w(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-count/batch/' + config.urlParams.parentRecordId + '/' + config.urlParams.relatedListNames + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$o = 'getRelatedListsCount';
const getRelatedListsCount_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentRecordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('relatedListNames', true, 0 /* UrlParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('maxCount', false, 1 /* QueryParameter */, 3 /* Integer */),
];
const getRelatedListsCount_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$o, getRelatedListsCount_ConfigPropertyMetadata);
const createResourceParams$s = /*#__PURE__*/ createResourceParams$k(getRelatedListsCount_ConfigPropertyMetadata);
function keyBuilder$25(luvio, config) {
    const resourceParams = createResourceParams$s(config);
    return keyBuilder$26(luvio, resourceParams);
}
function typeCheckConfig$t(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListsCount_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$t(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListsCount_ConfigPropertyMetadata);
    const config = typeCheckConfig$t(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$h(luvio, config) {
    const resourceParams = createResourceParams$s(config);
    return select$1U(luvio, resourceParams);
}
function onFetchResponseSuccess$i(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$l(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$u(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$h(luvio, config, resourceParams, response) {
    const snapshot = ingestError$f(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$u(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$u(luvio, config, options) {
    const resourceParams = createResourceParams$s(config);
    const request = createResourceRequest$w(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$i(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$r(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$h(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$k(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$u, undefined, false);
}
function buildCachedSnapshotCachePolicy$j(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$25(luvio, config),
        node: adapterFragment$h(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$u(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListsCountAdapterFactory = (luvio) => function UiApi__getRelatedListsCount(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$t(untrustedConfig, getRelatedListsCount_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$j, buildNetworkSnapshotCachePolicy$k);
};

const adapterName$n = 'getRelatedListCount';
const getRelatedListCount_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentRecordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('relatedListId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('maxCount', false, 1 /* QueryParameter */, 3 /* Integer */),
];
const getRelatedListCount_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$n, getRelatedListCount_ConfigPropertyMetadata);
const createResourceParams$r = /*#__PURE__*/ createResourceParams$k(getRelatedListCount_ConfigPropertyMetadata);
function keyBuilder$24(luvio, config) {
    const resourceParams = createResourceParams$r(config);
    return keyBuilder$27(luvio, resourceParams);
}
function typeCheckConfig$s(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListCount_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$s(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListCount_ConfigPropertyMetadata);
    const config = typeCheckConfig$s(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$g(luvio, config) {
    createResourceParams$r(config);
    return select$1V();
}
function onFetchResponseSuccess$h(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$m(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$t(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$g(luvio, config, resourceParams, response) {
    const snapshot = ingestError$g(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$t(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$t(luvio, config, options) {
    const resourceParams = createResourceParams$r(config);
    const request = createResourceRequest$x(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$h(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$s(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$g(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$j(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$t, undefined, false);
}
function buildCachedSnapshotCachePolicy$i(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$24(luvio, config),
        node: adapterFragment$g(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$t(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListCountAdapterFactory = (luvio) => function UiApi__getRelatedListCount(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$s(untrustedConfig, getRelatedListCount_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$i, buildNetworkSnapshotCachePolicy$j);
};

function validate$L(obj, path = 'RelatedListColumnRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_dataType = obj.dataType;
        const path_dataType = path + '.dataType';
        let obj_dataType_union0 = null;
        const obj_dataType_union0_error = (() => {
            if (typeof obj_dataType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
            }
        })();
        if (obj_dataType_union0_error != null) {
            obj_dataType_union0 = obj_dataType_union0_error.message;
        }
        let obj_dataType_union1 = null;
        const obj_dataType_union1_error = (() => {
            if (obj_dataType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_dataType + '" (at "' + path_dataType + '")');
            }
        })();
        if (obj_dataType_union1_error != null) {
            obj_dataType_union1 = obj_dataType_union1_error.message;
        }
        if (obj_dataType_union0 && obj_dataType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_dataType + '")';
            message += '\n' + obj_dataType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_dataType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_filterable = obj.filterable;
        const path_filterable = path + '.filterable';
        let obj_filterable_union0 = null;
        const obj_filterable_union0_error = (() => {
            if (typeof obj_filterable !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_filterable + '" (at "' + path_filterable + '")');
            }
        })();
        if (obj_filterable_union0_error != null) {
            obj_filterable_union0 = obj_filterable_union0_error.message;
        }
        let obj_filterable_union1 = null;
        const obj_filterable_union1_error = (() => {
            if (obj_filterable !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_filterable + '" (at "' + path_filterable + '")');
            }
        })();
        if (obj_filterable_union1_error != null) {
            obj_filterable_union1 = obj_filterable_union1_error.message;
        }
        if (obj_filterable_union0 && obj_filterable_union1) {
            let message = 'Object doesn\'t match union (at "' + path_filterable + '")';
            message += '\n' + obj_filterable_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_filterable_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_lookupId = obj.lookupId;
        const path_lookupId = path + '.lookupId';
        let obj_lookupId_union0 = null;
        const obj_lookupId_union0_error = (() => {
            if (typeof obj_lookupId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lookupId + '" (at "' + path_lookupId + '")');
            }
        })();
        if (obj_lookupId_union0_error != null) {
            obj_lookupId_union0 = obj_lookupId_union0_error.message;
        }
        let obj_lookupId_union1 = null;
        const obj_lookupId_union1_error = (() => {
            if (obj_lookupId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lookupId + '" (at "' + path_lookupId + '")');
            }
        })();
        if (obj_lookupId_union1_error != null) {
            obj_lookupId_union1 = obj_lookupId_union1_error.message;
        }
        if (obj_lookupId_union0 && obj_lookupId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lookupId + '")';
            message += '\n' + obj_lookupId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lookupId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_picklistValues = obj.picklistValues;
        const path_picklistValues = path + '.picklistValues';
        if (!ArrayIsArray(obj_picklistValues)) {
            return new TypeError('Expected "array" but received "' + typeof obj_picklistValues + '" (at "' + path_picklistValues + '")');
        }
        for (let i = 0; i < obj_picklistValues.length; i++) {
            const obj_picklistValues_item = obj_picklistValues[i];
            const path_picklistValues_item = path_picklistValues + '[' + i + ']';
            if (typeof obj_picklistValues_item !== 'object' || ArrayIsArray(obj_picklistValues_item) || obj_picklistValues_item === null) {
                return new TypeError('Expected "object" but received "' + typeof obj_picklistValues_item + '" (at "' + path_picklistValues_item + '")');
            }
            const obj_picklistValues_item_keys = ObjectKeys(obj_picklistValues_item);
            for (let i = 0; i < obj_picklistValues_item_keys.length; i++) {
                const key = obj_picklistValues_item_keys[i];
                const obj_picklistValues_item_prop = obj_picklistValues_item[key];
                const path_picklistValues_item_prop = path_picklistValues_item + '["' + key + '"]';
                if (typeof obj_picklistValues_item_prop !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_picklistValues_item_prop + '" (at "' + path_picklistValues_item_prop + '")');
                }
            }
        }
        const obj_quickFilterOperator = obj.quickFilterOperator;
        const path_quickFilterOperator = path + '.quickFilterOperator';
        let obj_quickFilterOperator_union0 = null;
        const obj_quickFilterOperator_union0_error = (() => {
            if (typeof obj_quickFilterOperator !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_quickFilterOperator + '" (at "' + path_quickFilterOperator + '")');
            }
        })();
        if (obj_quickFilterOperator_union0_error != null) {
            obj_quickFilterOperator_union0 = obj_quickFilterOperator_union0_error.message;
        }
        let obj_quickFilterOperator_union1 = null;
        const obj_quickFilterOperator_union1_error = (() => {
            if (obj_quickFilterOperator !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_quickFilterOperator + '" (at "' + path_quickFilterOperator + '")');
            }
        })();
        if (obj_quickFilterOperator_union1_error != null) {
            obj_quickFilterOperator_union1 = obj_quickFilterOperator_union1_error.message;
        }
        if (obj_quickFilterOperator_union0 && obj_quickFilterOperator_union1) {
            let message = 'Object doesn\'t match union (at "' + path_quickFilterOperator + '")';
            message += '\n' + obj_quickFilterOperator_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_quickFilterOperator_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_sortable = obj.sortable;
        const path_sortable = path + '.sortable';
        if (typeof obj_sortable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_sortable + '" (at "' + path_sortable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$i = 900000;
const VERSION$1J = "c977d65d153a2b4e888ddd45fb083248";
function validate$K(obj, path = 'RelatedListInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_cloneable = obj.cloneable;
        const path_cloneable = path + '.cloneable';
        if (typeof obj_cloneable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_cloneable + '" (at "' + path_cloneable + '")');
        }
        const obj_createable = obj.createable;
        const path_createable = path + '.createable';
        if (typeof obj_createable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_createable + '" (at "' + path_createable + '")');
        }
        const obj_deletable = obj.deletable;
        const path_deletable = path + '.deletable';
        if (typeof obj_deletable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_deletable + '" (at "' + path_deletable + '")');
        }
        const obj_displayColumns = obj.displayColumns;
        const path_displayColumns = path + '.displayColumns';
        if (!ArrayIsArray(obj_displayColumns)) {
            return new TypeError('Expected "array" but received "' + typeof obj_displayColumns + '" (at "' + path_displayColumns + '")');
        }
        for (let i = 0; i < obj_displayColumns.length; i++) {
            const obj_displayColumns_item = obj_displayColumns[i];
            const path_displayColumns_item = path_displayColumns + '[' + i + ']';
            const referencepath_displayColumns_itemValidationError = validate$L(obj_displayColumns_item, path_displayColumns_item);
            if (referencepath_displayColumns_itemValidationError !== null) {
                let message = 'Object doesn\'t match RelatedListColumnRepresentation (at "' + path_displayColumns_item + '")\n';
                message += referencepath_displayColumns_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (!ArrayIsArray(obj_fields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        for (let i = 0; i < obj_fields.length; i++) {
            const obj_fields_item = obj_fields[i];
            const path_fields_item = path_fields + '[' + i + ']';
            if (typeof obj_fields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_fields_item + '" (at "' + path_fields_item + '")');
            }
        }
        const obj_filterLogicString = obj.filterLogicString;
        const path_filterLogicString = path + '.filterLogicString';
        if (typeof obj_filterLogicString !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_filterLogicString + '" (at "' + path_filterLogicString + '")');
        }
        const obj_filterable = obj.filterable;
        const path_filterable = path + '.filterable';
        if (typeof obj_filterable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_filterable + '" (at "' + path_filterable + '")');
        }
        const obj_filteredByInfo = obj.filteredByInfo;
        const path_filteredByInfo = path + '.filteredByInfo';
        if (!ArrayIsArray(obj_filteredByInfo)) {
            return new TypeError('Expected "array" but received "' + typeof obj_filteredByInfo + '" (at "' + path_filteredByInfo + '")');
        }
        for (let i = 0; i < obj_filteredByInfo.length; i++) {
            const obj_filteredByInfo_item = obj_filteredByInfo[i];
            const path_filteredByInfo_item = path_filteredByInfo + '[' + i + ']';
            const referencepath_filteredByInfo_itemValidationError = validate$2f(obj_filteredByInfo_item, path_filteredByInfo_item);
            if (referencepath_filteredByInfo_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListFilterByInfoRepresentation (at "' + path_filteredByInfo_item + '")\n';
                message += referencepath_filteredByInfo_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_listReference = obj.listReference;
        const path_listReference = path + '.listReference';
        const referencepath_listReferenceValidationError = validate$N(obj_listReference, path_listReference);
        if (referencepath_listReferenceValidationError !== null) {
            let message = 'Object doesn\'t match RelatedListReferenceRepresentation (at "' + path_listReference + '")\n';
            message += referencepath_listReferenceValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiNames = obj.objectApiNames;
        const path_objectApiNames = path + '.objectApiNames';
        if (!ArrayIsArray(obj_objectApiNames)) {
            return new TypeError('Expected "array" but received "' + typeof obj_objectApiNames + '" (at "' + path_objectApiNames + '")');
        }
        for (let i = 0; i < obj_objectApiNames.length; i++) {
            const obj_objectApiNames_item = obj_objectApiNames[i];
            const path_objectApiNames_item = path_objectApiNames + '[' + i + ']';
            if (typeof obj_objectApiNames_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectApiNames_item + '" (at "' + path_objectApiNames_item + '")');
            }
        }
        const obj_optionalFields = obj.optionalFields;
        const path_optionalFields = path + '.optionalFields';
        if (!ArrayIsArray(obj_optionalFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_optionalFields + '" (at "' + path_optionalFields + '")');
        }
        for (let i = 0; i < obj_optionalFields.length; i++) {
            const obj_optionalFields_item = obj_optionalFields[i];
            const path_optionalFields_item = path_optionalFields + '[' + i + ']';
            if (typeof obj_optionalFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_optionalFields_item + '" (at "' + path_optionalFields_item + '")');
            }
        }
        const obj_orderedByInfo = obj.orderedByInfo;
        const path_orderedByInfo = path + '.orderedByInfo';
        if (!ArrayIsArray(obj_orderedByInfo)) {
            return new TypeError('Expected "array" but received "' + typeof obj_orderedByInfo + '" (at "' + path_orderedByInfo + '")');
        }
        for (let i = 0; i < obj_orderedByInfo.length; i++) {
            const obj_orderedByInfo_item = obj_orderedByInfo[i];
            const path_orderedByInfo_item = path_orderedByInfo + '[' + i + ']';
            const referencepath_orderedByInfo_itemValidationError = validate$2a(obj_orderedByInfo_item, path_orderedByInfo_item);
            if (referencepath_orderedByInfo_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListOrderByInfoRepresentation (at "' + path_orderedByInfo_item + '")\n';
                message += referencepath_orderedByInfo_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_restrictColumnsToLayout = obj.restrictColumnsToLayout;
        const path_restrictColumnsToLayout = path + '.restrictColumnsToLayout';
        if (typeof obj_restrictColumnsToLayout !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_restrictColumnsToLayout + '" (at "' + path_restrictColumnsToLayout + '")');
        }
        const obj_updateable = obj.updateable;
        const path_updateable = path + '.updateable';
        if (typeof obj_updateable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_updateable + '" (at "' + path_updateable + '")');
        }
        const obj_userPreferences = obj.userPreferences;
        const path_userPreferences = path + '.userPreferences';
        const referencepath_userPreferencesValidationError = validate$26(obj_userPreferences, path_userPreferences);
        if (referencepath_userPreferencesValidationError !== null) {
            let message = 'Object doesn\'t match ListUserPreferenceRepresentation (at "' + path_userPreferences + '")\n';
            message += referencepath_userPreferencesValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_visibility = obj.visibility;
        const path_visibility = path + '.visibility';
        if (typeof obj_visibility !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_visibility + '" (at "' + path_visibility + '")');
        }
        const obj_visibilityEditable = obj.visibilityEditable;
        const path_visibilityEditable = path + '.visibilityEditable';
        if (typeof obj_visibilityEditable !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_visibilityEditable + '" (at "' + path_visibilityEditable + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$l = 'RelatedListInfoRepresentation';
function keyBuilder$23(luvio, config) {
    return keyPrefix + '::' + RepresentationType$l + ':' + config.relatedListId + ':' + config.parentObjectApiName + ':' + config.restrictColumnsToLayout + ':' + (config.recordTypeId === null ? '' : config.recordTypeId) + ':' + '[' + config.optionalFields.join(',') + ']' + ':' + '[' + config.fields.join(',') + ']';
}
function keyBuilderFromType$d(luvio, object) {
    const keyParams = {
        relatedListId: object.listReference.relatedListId,
        parentObjectApiName: object.listReference.parentObjectApiName,
        restrictColumnsToLayout: object.restrictColumnsToLayout,
        recordTypeId: object.listReference.recordTypeId,
        optionalFields: object.optionalFields,
        fields: object.fields
    };
    return keyBuilder$23(luvio, keyParams);
}
function normalize$f(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1T = function RelatedListInfoRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1J,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'cloneable',
                kind: 'Scalar'
            },
            {
                name: 'createable',
                kind: 'Scalar'
            },
            {
                name: 'deletable',
                kind: 'Scalar'
            },
            {
                name: 'displayColumns',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'fieldApiName',
                kind: 'Scalar'
            },
            {
                name: 'fields',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'filterLogicString',
                kind: 'Scalar'
            },
            {
                name: 'filterable',
                kind: 'Scalar'
            },
            {
                name: 'filteredByInfo',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'label',
                kind: 'Scalar'
            },
            {
                name: 'listReference',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'objectApiNames',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'optionalFields',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'orderedByInfo',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'restrictColumnsToLayout',
                kind: 'Scalar'
            },
            {
                name: 'updateable',
                kind: 'Scalar'
            },
            {
                name: 'userPreferences',
                kind: 'Object',
                opaque: true
            },
            {
                name: 'visibility',
                kind: 'Scalar'
            },
            {
                name: 'visibilityEditable',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$j(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$1F = function RelatedListInfoRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$K(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$d(luvio, input);
    const ttlToUse = TTL$i;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$f, "UiApi", VERSION$1J, RepresentationType$l, equals$j);
    return createLink$1(key);
};
function getTypeCacheKeys$1G(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$d(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$l,
        mergeable: false
    });
}

function select$1S(luvio, params) {
    return select$1T();
}
function keyBuilder$22(luvio, params) {
    return keyBuilder$23(luvio, {
        relatedListId: params.urlParams.relatedListId,
        parentObjectApiName: params.urlParams.parentObjectApiName,
        restrictColumnsToLayout: params.queryParams.restrictColumnsToLayout === false ? false : true,
        recordTypeId: params.queryParams.recordTypeId || null,
        optionalFields: params.queryParams.optionalFields || [],
        fields: params.queryParams.fields || []
    });
}
function getResponseCacheKeys$q(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1G(storeKeyMap, luvio, response);
}
function ingestSuccess$k(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$22(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1F, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1S(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$e(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$22(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$i,
        namespace: keyPrefix,
        version: VERSION$1J,
        representationName: RepresentationType$l
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$v(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-info/' + config.urlParams.parentObjectApiName + '/' + config.urlParams.relatedListId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

function createChildResourceParams$4(resourceParams) {
    const childConfigs = [];
    for (let index = 0, len = resourceParams.urlParams.relatedListNames.length; index < len; index += 1) {
        const item = resourceParams.urlParams.relatedListNames[index];
        childConfigs.push({
            urlParams: {
                parentObjectApiName: resourceParams.urlParams.parentObjectApiName,
                relatedListId: item,
            },
            queryParams: {
                recordTypeId: resourceParams.queryParams.recordTypeId,
                fields: [],
                optionalFields: [],
                restrictColumnsToLayout: true,
            },
        });
    }
    return childConfigs;
}

const nonCachedErrors$3 = ObjectCreate$1(null);
function selectChildResourceParams$3(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$21(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$22(luvio, childResource);
                const childFragment = select$1S();
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$3[childKey];
                            if (nonCachedError === undefined || nonCachedError.expiration < reader.getTimeStamp()) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$1R(luvio, resourceParams) {
    const childResources = createChildResourceParams$4(resourceParams);
    return selectChildResourceParams$3(luvio, childResources, resourceParams);
}
function keyBuilder$21(luvio, params) {
    return keyPrefix + '::RelatedListInfoBatchRepresentation:(' + 'recordTypeId:' + params.queryParams.recordTypeId + ',' + 'parentObjectApiName:' + params.urlParams.parentObjectApiName + ',' + 'relatedListNames:' + params.urlParams.relatedListNames + ')';
}
function getResponseCacheKeys$p(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$4(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$q(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$22(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$l,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$4(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$22(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$k(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$e(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$3[childKey] = { expiration: now + TTL$i, response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$3[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$j(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$4(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$21(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$4(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$1R(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$d(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$21(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$u(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-info/batch/' + config.urlParams.parentObjectApiName + '/' + config.urlParams.relatedListNames + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$m = 'getRelatedListInfoBatch';
const getRelatedListInfoBatch_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentObjectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('relatedListNames', true, 0 /* UrlParameter */, 0 /* String */, true),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getRelatedListInfoBatch_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$m, getRelatedListInfoBatch_ConfigPropertyMetadata);
const createResourceParams$q = /*#__PURE__*/ createResourceParams$k(getRelatedListInfoBatch_ConfigPropertyMetadata);
function keyBuilder$20(luvio, config) {
    const resourceParams = createResourceParams$q(config);
    return keyBuilder$21(luvio, resourceParams);
}
function typeCheckConfig$r(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListInfoBatch_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$r(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListInfoBatch_ConfigPropertyMetadata);
    const config = typeCheckConfig$r(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$f(luvio, config) {
    const resourceParams = createResourceParams$q(config);
    return select$1R(luvio, resourceParams);
}
function onFetchResponseSuccess$g(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$j(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$s(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$f(luvio, config, resourceParams, response) {
    const snapshot = ingestError$d(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$s(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$s(luvio, config, options) {
    const resourceParams = createResourceParams$q(config);
    const request = createResourceRequest$u(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$g(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$p(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$f(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$i(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$s, undefined, false);
}
function buildCachedSnapshotCachePolicy$h(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$20(luvio, config),
        node: adapterFragment$f(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$s(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListInfoBatchAdapterFactory = (luvio) => function UiApi__getRelatedListInfoBatch(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$r(untrustedConfig, getRelatedListInfoBatch_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$h, buildNetworkSnapshotCachePolicy$i);
};

function validate$J(obj, path = 'RelatedListSummaryInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_entityLabel = obj.entityLabel;
        const path_entityLabel = path + '.entityLabel';
        if (typeof obj_entityLabel !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_entityLabel + '" (at "' + path_entityLabel + '")');
        }
        const obj_entityPluralLabel = obj.entityPluralLabel;
        const path_entityPluralLabel = path + '.entityPluralLabel';
        if (typeof obj_entityPluralLabel !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_entityPluralLabel + '" (at "' + path_entityPluralLabel + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_keyPrefix = obj.keyPrefix;
        const path_keyPrefix = path + '.keyPrefix';
        let obj_keyPrefix_union0 = null;
        const obj_keyPrefix_union0_error = (() => {
            if (typeof obj_keyPrefix !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_keyPrefix + '" (at "' + path_keyPrefix + '")');
            }
        })();
        if (obj_keyPrefix_union0_error != null) {
            obj_keyPrefix_union0 = obj_keyPrefix_union0_error.message;
        }
        let obj_keyPrefix_union1 = null;
        const obj_keyPrefix_union1_error = (() => {
            if (obj_keyPrefix !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_keyPrefix + '" (at "' + path_keyPrefix + '")');
            }
        })();
        if (obj_keyPrefix_union1_error != null) {
            obj_keyPrefix_union1 = obj_keyPrefix_union1_error.message;
        }
        if (obj_keyPrefix_union0 && obj_keyPrefix_union1) {
            let message = 'Object doesn\'t match union (at "' + path_keyPrefix + '")';
            message += '\n' + obj_keyPrefix_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_keyPrefix_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_parentFieldApiName = obj.parentFieldApiName;
        const path_parentFieldApiName = path + '.parentFieldApiName';
        if (typeof obj_parentFieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_parentFieldApiName + '" (at "' + path_parentFieldApiName + '")');
        }
        const obj_relatedListId = obj.relatedListId;
        const path_relatedListId = path + '.relatedListId';
        if (typeof obj_relatedListId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_relatedListId + '" (at "' + path_relatedListId + '")');
        }
        const obj_relatedListInfoUrl = obj.relatedListInfoUrl;
        const path_relatedListInfoUrl = path + '.relatedListInfoUrl';
        let obj_relatedListInfoUrl_union0 = null;
        const obj_relatedListInfoUrl_union0_error = (() => {
            if (typeof obj_relatedListInfoUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_relatedListInfoUrl + '" (at "' + path_relatedListInfoUrl + '")');
            }
        })();
        if (obj_relatedListInfoUrl_union0_error != null) {
            obj_relatedListInfoUrl_union0 = obj_relatedListInfoUrl_union0_error.message;
        }
        let obj_relatedListInfoUrl_union1 = null;
        const obj_relatedListInfoUrl_union1_error = (() => {
            if (obj_relatedListInfoUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_relatedListInfoUrl + '" (at "' + path_relatedListInfoUrl + '")');
            }
        })();
        if (obj_relatedListInfoUrl_union1_error != null) {
            obj_relatedListInfoUrl_union1 = obj_relatedListInfoUrl_union1_error.message;
        }
        if (obj_relatedListInfoUrl_union0 && obj_relatedListInfoUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_relatedListInfoUrl + '")';
            message += '\n' + obj_relatedListInfoUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_relatedListInfoUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_themeInfo = obj.themeInfo;
        const path_themeInfo = path + '.themeInfo';
        let obj_themeInfo_union0 = null;
        const obj_themeInfo_union0_error = (() => {
            const referencepath_themeInfoValidationError = validate$1P(obj_themeInfo, path_themeInfo);
            if (referencepath_themeInfoValidationError !== null) {
                let message = 'Object doesn\'t match ThemeInfoRepresentation (at "' + path_themeInfo + '")\n';
                message += referencepath_themeInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_themeInfo_union0_error != null) {
            obj_themeInfo_union0 = obj_themeInfo_union0_error.message;
        }
        let obj_themeInfo_union1 = null;
        const obj_themeInfo_union1_error = (() => {
            if (obj_themeInfo !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_themeInfo + '" (at "' + path_themeInfo + '")');
            }
        })();
        if (obj_themeInfo_union1_error != null) {
            obj_themeInfo_union1 = obj_themeInfo_union1_error.message;
        }
        if (obj_themeInfo_union0 && obj_themeInfo_union1) {
            let message = 'Object doesn\'t match union (at "' + path_themeInfo + '")';
            message += '\n' + obj_themeInfo_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_themeInfo_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_uiApiEnabledLayout = obj.uiApiEnabledLayout;
        const path_uiApiEnabledLayout = path + '.uiApiEnabledLayout';
        if (typeof obj_uiApiEnabledLayout !== 'boolean') {
            return new TypeError('Expected "boolean" but received "' + typeof obj_uiApiEnabledLayout + '" (at "' + path_uiApiEnabledLayout + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

const VERSION$1I = "2a1722afba0e1ee52d6b7b0a25ccd9f4";
function validate$I(obj, path = 'RelatedListSummaryInfoCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_parentObjectApiName = obj.parentObjectApiName;
        const path_parentObjectApiName = path + '.parentObjectApiName';
        if (typeof obj_parentObjectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_parentObjectApiName + '" (at "' + path_parentObjectApiName + '")');
        }
        const obj_parentRecordTypeId = obj.parentRecordTypeId;
        const path_parentRecordTypeId = path + '.parentRecordTypeId';
        let obj_parentRecordTypeId_union0 = null;
        const obj_parentRecordTypeId_union0_error = (() => {
            if (typeof obj_parentRecordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_parentRecordTypeId + '" (at "' + path_parentRecordTypeId + '")');
            }
        })();
        if (obj_parentRecordTypeId_union0_error != null) {
            obj_parentRecordTypeId_union0 = obj_parentRecordTypeId_union0_error.message;
        }
        let obj_parentRecordTypeId_union1 = null;
        const obj_parentRecordTypeId_union1_error = (() => {
            if (obj_parentRecordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_parentRecordTypeId + '" (at "' + path_parentRecordTypeId + '")');
            }
        })();
        if (obj_parentRecordTypeId_union1_error != null) {
            obj_parentRecordTypeId_union1 = obj_parentRecordTypeId_union1_error.message;
        }
        if (obj_parentRecordTypeId_union0 && obj_parentRecordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_parentRecordTypeId + '")';
            message += '\n' + obj_parentRecordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_parentRecordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_relatedLists = obj.relatedLists;
        const path_relatedLists = path + '.relatedLists';
        if (!ArrayIsArray(obj_relatedLists)) {
            return new TypeError('Expected "array" but received "' + typeof obj_relatedLists + '" (at "' + path_relatedLists + '")');
        }
        for (let i = 0; i < obj_relatedLists.length; i++) {
            const obj_relatedLists_item = obj_relatedLists[i];
            const path_relatedLists_item = path_relatedLists + '[' + i + ']';
            const referencepath_relatedLists_itemValidationError = validate$J(obj_relatedLists_item, path_relatedLists_item);
            if (referencepath_relatedLists_itemValidationError !== null) {
                let message = 'Object doesn\'t match RelatedListSummaryInfoRepresentation (at "' + path_relatedLists_item + '")\n';
                message += referencepath_relatedLists_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$k = 'RelatedListSummaryInfoCollectionRepresentation';
function keyBuilder$1$(luvio, config) {
    return keyPrefix + '::' + RepresentationType$k + ':' + config.parentObjectApiName + ':' + (config.recordTypeId === null ? '' : config.recordTypeId);
}
function keyBuilderFromType$c(luvio, object) {
    const keyParams = {
        parentObjectApiName: object.parentObjectApiName,
        recordTypeId: object.parentRecordTypeId
    };
    return keyBuilder$1$(luvio, keyParams);
}
function normalize$e(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1Q = function RelatedListSummaryInfoCollectionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1I,
        private: [
            'eTag'
        ],
        selections: [
            {
                name: 'parentObjectApiName',
                kind: 'Scalar'
            },
            {
                name: 'parentRecordTypeId',
                kind: 'Scalar'
            },
            {
                name: 'relatedLists',
                kind: 'Object',
                opaque: true
            }
        ]
    };
};
function equals$i(existing, incoming) {
    if (existing.eTag !== incoming.eTag) {
        return false;
    }
    return true;
}
const ingest$1E = function RelatedListSummaryInfoCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$I(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$c(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$e, "UiApi", VERSION$1I, RepresentationType$k, equals$i);
    return createLink$1(key);
};
function getTypeCacheKeys$1F(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$c(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$k,
        mergeable: false
    });
}

function select$1P(luvio, params) {
    return select$1Q();
}
function keyBuilder$1_(luvio, params) {
    return keyBuilder$1$(luvio, {
        parentObjectApiName: params.urlParams.parentObjectApiName,
        recordTypeId: params.queryParams.recordTypeId || null
    });
}
function getResponseCacheKeys$o(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1F(storeKeyMap, luvio, response);
}
function ingestSuccess$i(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1_(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1E, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1P(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$c(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1_(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$t(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-info/' + config.urlParams.parentObjectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$l = 'getRelatedListsInfo';
const getRelatedListsInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentObjectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getRelatedListsInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$l, getRelatedListsInfo_ConfigPropertyMetadata);
const createResourceParams$p = /*#__PURE__*/ createResourceParams$k(getRelatedListsInfo_ConfigPropertyMetadata);
function keyBuilder$1Z(luvio, config) {
    const resourceParams = createResourceParams$p(config);
    return keyBuilder$1_(luvio, resourceParams);
}
function typeCheckConfig$q(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListsInfo_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$q(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListsInfo_ConfigPropertyMetadata);
    const config = typeCheckConfig$q(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$e(luvio, config) {
    createResourceParams$p(config);
    return select$1P();
}
function onFetchResponseSuccess$f(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$i(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$r(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$e(luvio, config, resourceParams, response) {
    const snapshot = ingestError$c(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$r(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$r(luvio, config, options) {
    const resourceParams = createResourceParams$p(config);
    const request = createResourceRequest$t(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$f(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$o(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$e(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$h(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$r, undefined, false);
}
function buildCachedSnapshotCachePolicy$g(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1Z(luvio, config),
        node: adapterFragment$e(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$r(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListsInfoAdapterFactory = (luvio) => function UiApi__getRelatedListsInfo(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$q(untrustedConfig, getRelatedListsInfo_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$g, buildNetworkSnapshotCachePolicy$h);
};

const adapterName$k = 'getRelatedListInfo';
const getRelatedListInfo_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentObjectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('relatedListId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('fields', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('restrictColumnsToLayout', false, 1 /* QueryParameter */, 1 /* Boolean */),
];
const getRelatedListInfo_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$k, getRelatedListInfo_ConfigPropertyMetadata);
const createResourceParams$o = /*#__PURE__*/ createResourceParams$k(getRelatedListInfo_ConfigPropertyMetadata);
function keyBuilder$1Y(luvio, config) {
    const resourceParams = createResourceParams$o(config);
    return keyBuilder$22(luvio, resourceParams);
}
function typeCheckConfig$p(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListInfo_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$p(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListInfo_ConfigPropertyMetadata);
    const config = typeCheckConfig$p(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$d(luvio, config) {
    createResourceParams$o(config);
    return select$1S();
}
function onFetchResponseSuccess$e(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$k(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$q(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$d(luvio, config, resourceParams, response) {
    const snapshot = ingestError$e(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$q(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$q(luvio, config, options) {
    const resourceParams = createResourceParams$o(config);
    const request = createResourceRequest$v(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$e(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$q(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$d(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$g(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$q, undefined, false);
}
function buildCachedSnapshotCachePolicy$f(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1Y(luvio, config),
        node: adapterFragment$d(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$q(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListInfoAdapterFactory = (luvio) => function UiApi__getRelatedListInfo(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$p(untrustedConfig, getRelatedListInfo_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$f, buildNetworkSnapshotCachePolicy$g);
};

const TTL$h = 900000;
const VERSION$1H = "094cdf8e3e1f07fca02c4e51e14c528e";
function validate$G(obj, path = 'RelatedListUserPreferencesRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_columnWidths = obj.columnWidths;
        const path_columnWidths = path + '.columnWidths';
        if (typeof obj_columnWidths !== 'object' || ArrayIsArray(obj_columnWidths) || obj_columnWidths === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_columnWidths + '" (at "' + path_columnWidths + '")');
        }
        const obj_columnWidths_keys = ObjectKeys(obj_columnWidths);
        for (let i = 0; i < obj_columnWidths_keys.length; i++) {
            const key = obj_columnWidths_keys[i];
            const obj_columnWidths_prop = obj_columnWidths[key];
            const path_columnWidths_prop = path_columnWidths + '["' + key + '"]';
            if (typeof obj_columnWidths_prop !== 'number' || (typeof obj_columnWidths_prop === 'number' && Math.floor(obj_columnWidths_prop) !== obj_columnWidths_prop)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_columnWidths_prop + '" (at "' + path_columnWidths_prop + '")');
            }
        }
        const obj_columnWrap = obj.columnWrap;
        const path_columnWrap = path + '.columnWrap';
        if (typeof obj_columnWrap !== 'object' || ArrayIsArray(obj_columnWrap) || obj_columnWrap === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_columnWrap + '" (at "' + path_columnWrap + '")');
        }
        const obj_columnWrap_keys = ObjectKeys(obj_columnWrap);
        for (let i = 0; i < obj_columnWrap_keys.length; i++) {
            const key = obj_columnWrap_keys[i];
            const obj_columnWrap_prop = obj_columnWrap[key];
            const path_columnWrap_prop = path_columnWrap + '["' + key + '"]';
            if (typeof obj_columnWrap_prop !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_columnWrap_prop + '" (at "' + path_columnWrap_prop + '")');
            }
        }
        const obj_orderedBy = obj.orderedBy;
        const path_orderedBy = path + '.orderedBy';
        if (!ArrayIsArray(obj_orderedBy)) {
            return new TypeError('Expected "array" but received "' + typeof obj_orderedBy + '" (at "' + path_orderedBy + '")');
        }
        for (let i = 0; i < obj_orderedBy.length; i++) {
            const obj_orderedBy_item = obj_orderedBy[i];
            const path_orderedBy_item = path_orderedBy + '[' + i + ']';
            const referencepath_orderedBy_itemValidationError = validate$2a(obj_orderedBy_item, path_orderedBy_item);
            if (referencepath_orderedBy_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListOrderByInfoRepresentation (at "' + path_orderedBy_item + '")\n';
                message += referencepath_orderedBy_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_preferencesId = obj.preferencesId;
        const path_preferencesId = path + '.preferencesId';
        if (typeof obj_preferencesId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_preferencesId + '" (at "' + path_preferencesId + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$j = 'RelatedListUserPreferencesRepresentation';
function keyBuilder$1X(luvio, config) {
    return keyPrefix + '::' + RepresentationType$j + ':' + config.preferencesId;
}
function keyBuilderFromType$b(luvio, object) {
    const keyParams = {
        preferencesId: object.preferencesId
    };
    return keyBuilder$1X(luvio, keyParams);
}
function normalize$d(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1N = function RelatedListUserPreferencesRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1H,
        private: [],
        opaque: true
    };
};
function equals$h(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1D = function RelatedListUserPreferencesRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$G(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$b(luvio, input);
    const ttlToUse = TTL$h;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$d, "UiApi", VERSION$1H, RepresentationType$j, equals$h);
    return createLink$1(key);
};
function getTypeCacheKeys$1E(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$b(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$j,
        mergeable: false
    });
}

function select$1M(luvio, params) {
    return select$1N();
}
function keyBuilder$1W(luvio, params) {
    return keyBuilder$1X(luvio, {
        preferencesId: params.urlParams.preferencesId
    });
}
function getResponseCacheKeys$m(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1E(storeKeyMap, luvio, response);
}
function ingestSuccess$g(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1W(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1D, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1M(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$b(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1W(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$h,
        namespace: keyPrefix,
        version: VERSION$1H,
        representationName: RepresentationType$j
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$r(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-preferences/' + config.urlParams.preferencesId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const nonCachedErrors$2 = ObjectCreate$1(null);
function createChildResourceParams$3(resourceParams) {
    let childConfigs = [];
    for (let index = 0, len = resourceParams.urlParams.preferencesIds.length; index < len; index += 1) {
        const item = resourceParams.urlParams.preferencesIds[index];
        childConfigs.push({
            urlParams: {
                preferencesId: item
            }
        });
    }
    return childConfigs;
}
function selectChildResourceParams$2(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$1V(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$1W(luvio, childResource);
                const childFragment = select$1M();
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$2[childKey];
                            if (nonCachedError === undefined || nonCachedError.expiration < reader.getTimeStamp()) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$1L(luvio, resourceParams) {
    const childResources = createChildResourceParams$3(resourceParams);
    return selectChildResourceParams$2(luvio, childResources, resourceParams);
}
function keyBuilder$1V(luvio, params) {
    return keyPrefix + '::RelatedListPreferencesBatchRepresentation:(' + 'preferencesIds:' + params.urlParams.preferencesIds + ')';
}
function getResponseCacheKeys$l(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$3(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$m(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$1W(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$j,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$3(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$1W(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$g(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$b(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$2[childKey] = { expiration: now + TTL$h, response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$2[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$f(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$3(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$1V(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$3(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$1L(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$a(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1V(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$q(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-preferences/batch/' + config.urlParams.preferencesIds + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$i = 'getRelatedListPreferencesBatch';
const getRelatedListPreferencesBatch_ConfigPropertyMetadata = [
    generateParamConfigMetadata('preferencesIds', true, 0 /* UrlParameter */, 0 /* String */, true),
];
const getRelatedListPreferencesBatch_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$i, getRelatedListPreferencesBatch_ConfigPropertyMetadata);
const createResourceParams$m = /*#__PURE__*/ createResourceParams$k(getRelatedListPreferencesBatch_ConfigPropertyMetadata);
function keyBuilder$1U(luvio, config) {
    const resourceParams = createResourceParams$m(config);
    return keyBuilder$1V(luvio, resourceParams);
}
function typeCheckConfig$n(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListPreferencesBatch_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$n(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$n(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$c(luvio, config) {
    const resourceParams = createResourceParams$m(config);
    return select$1L(luvio, resourceParams);
}
function onFetchResponseSuccess$d(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$f(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$o(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$c(luvio, config, resourceParams, response) {
    const snapshot = ingestError$a(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$o(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$o(luvio, config, options) {
    const resourceParams = createResourceParams$m(config);
    const request = createResourceRequest$q(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$d(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$l(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$c(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$f(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$o, undefined, false);
}
function buildCachedSnapshotCachePolicy$e(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1U(luvio, config),
        node: adapterFragment$c(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$o(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListPreferencesBatchAdapterFactory = (luvio) => function UiApi__getRelatedListPreferencesBatch(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$n(untrustedConfig, getRelatedListPreferencesBatch_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$e, buildNetworkSnapshotCachePolicy$f);
};

const adapterName$h = 'getRelatedListPreferences';
const getRelatedListPreferences_ConfigPropertyMetadata = [
    generateParamConfigMetadata('preferencesId', true, 0 /* UrlParameter */, 0 /* String */),
];
const getRelatedListPreferences_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$h, getRelatedListPreferences_ConfigPropertyMetadata);
const createResourceParams$l = /*#__PURE__*/ createResourceParams$k(getRelatedListPreferences_ConfigPropertyMetadata);
function keyBuilder$1T(luvio, config) {
    const resourceParams = createResourceParams$l(config);
    return keyBuilder$1W(luvio, resourceParams);
}
function typeCheckConfig$m(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListPreferences_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$m(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$m(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$b(luvio, config) {
    createResourceParams$l(config);
    return select$1M();
}
function onFetchResponseSuccess$c(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$g(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$n(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$b(luvio, config, resourceParams, response) {
    const snapshot = ingestError$b(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$n(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$n(luvio, config, options) {
    const resourceParams = createResourceParams$l(config);
    const request = createResourceRequest$r(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$c(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$m(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$b(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$e(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$n, undefined, false);
}
function buildCachedSnapshotCachePolicy$d(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1T(luvio, config),
        node: adapterFragment$b(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$n(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListPreferencesAdapterFactory = (luvio) => function UiApi__getRelatedListPreferences(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$m(untrustedConfig, getRelatedListPreferences_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$d, buildNetworkSnapshotCachePolicy$e);
};

function validate$F(obj, path = 'RelatedListRecordsSingleBatchInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        if (obj.fields !== undefined) {
            const obj_fields = obj.fields;
            const path_fields = path + '.fields';
            if (!ArrayIsArray(obj_fields)) {
                return new TypeError('Expected "array" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
            }
            for (let i = 0; i < obj_fields.length; i++) {
                const obj_fields_item = obj_fields[i];
                const path_fields_item = path_fields + '[' + i + ']';
                if (typeof obj_fields_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_fields_item + '" (at "' + path_fields_item + '")');
                }
            }
        }
        if (obj.optionalFields !== undefined) {
            const obj_optionalFields = obj.optionalFields;
            const path_optionalFields = path + '.optionalFields';
            if (!ArrayIsArray(obj_optionalFields)) {
                return new TypeError('Expected "array" but received "' + typeof obj_optionalFields + '" (at "' + path_optionalFields + '")');
            }
            for (let i = 0; i < obj_optionalFields.length; i++) {
                const obj_optionalFields_item = obj_optionalFields[i];
                const path_optionalFields_item = path_optionalFields + '[' + i + ']';
                if (typeof obj_optionalFields_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_optionalFields_item + '" (at "' + path_optionalFields_item + '")');
                }
            }
        }
        if (obj.pageSize !== undefined) {
            const obj_pageSize = obj.pageSize;
            const path_pageSize = path + '.pageSize';
            if (typeof obj_pageSize !== 'number' || (typeof obj_pageSize === 'number' && Math.floor(obj_pageSize) !== obj_pageSize)) {
                return new TypeError('Expected "integer" but received "' + typeof obj_pageSize + '" (at "' + path_pageSize + '")');
            }
        }
        const obj_relatedListId = obj.relatedListId;
        const path_relatedListId = path + '.relatedListId';
        if (typeof obj_relatedListId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_relatedListId + '" (at "' + path_relatedListId + '")');
        }
        if (obj.sortBy !== undefined) {
            const obj_sortBy = obj.sortBy;
            const path_sortBy = path + '.sortBy';
            if (!ArrayIsArray(obj_sortBy)) {
                return new TypeError('Expected "array" but received "' + typeof obj_sortBy + '" (at "' + path_sortBy + '")');
            }
            for (let i = 0; i < obj_sortBy.length; i++) {
                const obj_sortBy_item = obj_sortBy[i];
                const path_sortBy_item = path_sortBy + '[' + i + ']';
                if (typeof obj_sortBy_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_sortBy_item + '" (at "' + path_sortBy_item + '")');
                }
            }
        }
        if (obj.where !== undefined) {
            const obj_where = obj.where;
            const path_where = path + '.where';
            if (typeof obj_where !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_where + '" (at "' + path_where + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

const PAGINATION_VERSION = "0.111.0";
const paginationStoreMetadata = {
    ttl: Number.MAX_SAFE_INTEGER,
    namespace: '',
    representationName: '',
    version: PAGINATION_VERSION
};
function clientTokenFromOffset(paginationMetadata, offset) {
    const { clientTokens } = paginationMetadata;
    const tokens = Object.keys(clientTokens);
    for (let i = 0, len = tokens.length; i < len; i++) {
        const token = tokens[i];
        if (clientTokens[token] === offset) {
            return token;
        }
    }
}
function offsetFromClientToken(paginationMetadata, token) {
    if (token === undefined) {
        return 0;
    }
    const offset = paginationMetadata.clientTokens[token];
    return offset === undefined ? -1 : offset;
}
function offsetFromServerToken(paginationMetadata, token) {
    return paginationMetadata.serverTokens.indexOf(token);
}
function getNextPageTokens(paginationMetadata, pageEndOffset) {
    const listEndOffset = paginationMetadata.__END__;
    if (listEndOffset === undefined || pageEndOffset < listEndOffset) {
        const clientToken = clientTokenFromOffset(paginationMetadata, pageEndOffset);
        if (clientToken === undefined) {
            return {
                clientToken: undefined,
                serverToken: undefined,
            };
        }
        return {
            clientToken,
            serverToken: paginationMetadata.serverTokens[pageEndOffset] || undefined
        };
    }
    return {
        clientToken: null,
        serverToken: null,
    };
}
function getPreviousClientToken(paginationMetadata, startOffset, pageSize) {
    const previousPageOffset = startOffset - pageSize;
    if (previousPageOffset < 0) {
        return null;
    }
    const listEndOffset = paginationMetadata.__END__;
    if (listEndOffset !== undefined && previousPageOffset >= listEndOffset) {
        return;
    }
    return clientTokenFromOffset(paginationMetadata, previousPageOffset);
}
function pageEndOffset(paginationMetadata, startOffset, pageSize) {
    const listEndOffset = paginationMetadata.__END__;
    const nextPageOffset = startOffset + pageSize;
    if (listEndOffset === undefined) {
        return nextPageOffset;
    }
    return Math.min(nextPageOffset, listEndOffset);
}
function getPageTokenAndOffset(paginationMetadata, token) {
    if (token === undefined) {
        const startToken = clientTokenFromOffset(paginationMetadata, 0);
        if (startToken === undefined) {
            return;
        }
        return [startToken, 0];
    }
    const startOffset = offsetFromClientToken(paginationMetadata, token);
    if (startOffset < 0) {
        return;
    }
    return [token, startOffset];
}
function serverTokenForAtMost(paginationMetadata, maxOffset) {
    const { serverTokens } = paginationMetadata;
    for (let i = maxOffset; i >= 0; i--) {
        const token = serverTokens[i];
        if (token !== undefined) {
            return [token, i];
        }
    }
    return [undefined, -1];
}
function generateClientToken(offset) {
    return base64Encode('client:' + offset);
}
// Code lifted from https://github.com/MaxArt2501/base64-js/blob/master/base64.js
// base64 character set, plus padding character (=)
const b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
function base64Encode(input) {
    let bitmap, a, b, c;
    let result = "", i = 0;
    const rest = input.length % 3; // To determine the final padding
    for (; i < input.length;) {
        if ((a = input.charCodeAt(i++)) > 255
            || (b = input.charCodeAt(i++)) > 255
            || (c = input.charCodeAt(i++)) > 255) {
            throw new TypeError("Failed base64ToAscii encoding: The string to be encoded contains characters outside of the Latin1 range. " + input);
        }
        bitmap = (a << 16) | (b << 8) | c;
        result += b64.charAt(bitmap >> 18 & 63) + b64.charAt(bitmap >> 12 & 63)
            + b64.charAt(bitmap >> 6 & 63) + b64.charAt(bitmap & 63);
    }
    // If there's need of padding, replace the last 'A's with equal signs
    return rest ? result.slice(0, rest - 3) + "===".substring(rest) : result;
}
function addTokens(paginationMetadata, offset, clientToken, serverToken) {
    paginationMetadata.clientTokens[clientToken] = offset;
    if (serverToken !== undefined) {
        paginationMetadata.serverTokens[offset] = serverToken;
    }
}
function mergeMetadata(existingMetadata, incomingMetadata) {
    if (existingMetadata === undefined) {
        return incomingMetadata;
    }
    const clientTokens = {
        ...existingMetadata.clientTokens,
        ...incomingMetadata.clientTokens
    };
    const serverTokens = mergeArrays(existingMetadata.serverTokens, incomingMetadata.serverTokens);
    const __END__ = incomingMetadata.hasOwnProperty('__END__') ? incomingMetadata.__END__ : existingMetadata.__END__;
    return {
        clientTokens,
        serverTokens,
        __END__
    };
}
function mergeArrays(existing, incoming) {
    const merged = [];
    const len = Math.max(existing.length, incoming.length);
    for (let i = 0; i < len; i++) {
        const value = incoming[i] || existing[i];
        merged.push(value);
    }
    return merged;
}
function equalsMetadata(existingMetadata, incomingMetadata) {
    const { clientTokens: existingClientTokens, serverTokens: existingServerTokens, __END__: existingListEnd } = existingMetadata;
    const { clientTokens: incomingClientTokens, serverTokens: incomingServerTokens, __END__: incomingListEnd } = incomingMetadata;
    const existingKeys = Object.keys(existingClientTokens);
    const incomingKeys = Object.keys(incomingClientTokens);
    const clientTokensLength = existingKeys.length;
    if (clientTokensLength !== incomingKeys.length) {
        return false;
    }
    for (let i = 0; i < clientTokensLength; i++) {
        const key = existingKeys[i];
        if (existingClientTokens[key] !== incomingClientTokens[key]) {
            return false;
        }
    }
    const serverTokensLength = existingServerTokens.length;
    if (serverTokensLength !== incomingServerTokens.length) {
        return false;
    }
    for (let i = 0; i < serverTokensLength; i++) {
        if (existingServerTokens[i] !== incomingServerTokens[i]) {
            return false;
        }
    }
    return existingListEnd === incomingListEnd;
}

const TTL$g = 30000;
const VERSION$1G = "62467c27c19349b70c9db2a8d9d591d9";
function validate$E(obj, path = 'RelatedListRecordCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_count = obj.count;
        const path_count = path + '.count';
        if (typeof obj_count !== 'number' || (typeof obj_count === 'number' && Math.floor(obj_count) !== obj_count)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_count + '" (at "' + path_count + '")');
        }
        const obj_currentPageToken = obj.currentPageToken;
        const path_currentPageToken = path + '.currentPageToken';
        if (typeof obj_currentPageToken !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
        }
        const obj_currentPageUrl = obj.currentPageUrl;
        const path_currentPageUrl = path + '.currentPageUrl';
        let obj_currentPageUrl_union0 = null;
        const obj_currentPageUrl_union0_error = (() => {
            if (typeof obj_currentPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
            }
        })();
        if (obj_currentPageUrl_union0_error != null) {
            obj_currentPageUrl_union0 = obj_currentPageUrl_union0_error.message;
        }
        let obj_currentPageUrl_union1 = null;
        const obj_currentPageUrl_union1_error = (() => {
            if (obj_currentPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_currentPageUrl + '" (at "' + path_currentPageUrl + '")');
            }
        })();
        if (obj_currentPageUrl_union1_error != null) {
            obj_currentPageUrl_union1 = obj_currentPageUrl_union1_error.message;
        }
        if (obj_currentPageUrl_union0 && obj_currentPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_currentPageUrl + '")';
            message += '\n' + obj_currentPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_currentPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (!ArrayIsArray(obj_fields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        for (let i = 0; i < obj_fields.length; i++) {
            const obj_fields_item = obj_fields[i];
            const path_fields_item = path_fields + '[' + i + ']';
            if (typeof obj_fields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_fields_item + '" (at "' + path_fields_item + '")');
            }
        }
        const obj_listInfoETag = obj.listInfoETag;
        const path_listInfoETag = path + '.listInfoETag';
        let obj_listInfoETag_union0 = null;
        const obj_listInfoETag_union0_error = (() => {
            if (typeof obj_listInfoETag !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_listInfoETag + '" (at "' + path_listInfoETag + '")');
            }
        })();
        if (obj_listInfoETag_union0_error != null) {
            obj_listInfoETag_union0 = obj_listInfoETag_union0_error.message;
        }
        let obj_listInfoETag_union1 = null;
        const obj_listInfoETag_union1_error = (() => {
            if (obj_listInfoETag !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_listInfoETag + '" (at "' + path_listInfoETag + '")');
            }
        })();
        if (obj_listInfoETag_union1_error != null) {
            obj_listInfoETag_union1 = obj_listInfoETag_union1_error.message;
        }
        if (obj_listInfoETag_union0 && obj_listInfoETag_union1) {
            let message = 'Object doesn\'t match union (at "' + path_listInfoETag + '")';
            message += '\n' + obj_listInfoETag_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_listInfoETag_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_listReference = obj.listReference;
        const path_listReference = path + '.listReference';
        const referencepath_listReferenceValidationError = validate$N(obj_listReference, path_listReference);
        if (referencepath_listReferenceValidationError !== null) {
            let message = 'Object doesn\'t match RelatedListReferenceRepresentation (at "' + path_listReference + '")\n';
            message += referencepath_listReferenceValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageToken = obj.nextPageToken;
        const path_nextPageToken = path + '.nextPageToken';
        let obj_nextPageToken_union0 = null;
        const obj_nextPageToken_union0_error = (() => {
            if (typeof obj_nextPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union0_error != null) {
            obj_nextPageToken_union0 = obj_nextPageToken_union0_error.message;
        }
        let obj_nextPageToken_union1 = null;
        const obj_nextPageToken_union1_error = (() => {
            if (obj_nextPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union1_error != null) {
            obj_nextPageToken_union1 = obj_nextPageToken_union1_error.message;
        }
        if (obj_nextPageToken_union0 && obj_nextPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageToken + '")';
            message += '\n' + obj_nextPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageUrl = obj.nextPageUrl;
        const path_nextPageUrl = path + '.nextPageUrl';
        let obj_nextPageUrl_union0 = null;
        const obj_nextPageUrl_union0_error = (() => {
            if (typeof obj_nextPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union0_error != null) {
            obj_nextPageUrl_union0 = obj_nextPageUrl_union0_error.message;
        }
        let obj_nextPageUrl_union1 = null;
        const obj_nextPageUrl_union1_error = (() => {
            if (obj_nextPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageUrl + '" (at "' + path_nextPageUrl + '")');
            }
        })();
        if (obj_nextPageUrl_union1_error != null) {
            obj_nextPageUrl_union1 = obj_nextPageUrl_union1_error.message;
        }
        if (obj_nextPageUrl_union0 && obj_nextPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageUrl + '")';
            message += '\n' + obj_nextPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_optionalFields = obj.optionalFields;
        const path_optionalFields = path + '.optionalFields';
        if (!ArrayIsArray(obj_optionalFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_optionalFields + '" (at "' + path_optionalFields + '")');
        }
        for (let i = 0; i < obj_optionalFields.length; i++) {
            const obj_optionalFields_item = obj_optionalFields[i];
            const path_optionalFields_item = path_optionalFields + '[' + i + ']';
            if (typeof obj_optionalFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_optionalFields_item + '" (at "' + path_optionalFields_item + '")');
            }
        }
        const obj_pageSize = obj.pageSize;
        const path_pageSize = path + '.pageSize';
        if (typeof obj_pageSize !== 'number' || (typeof obj_pageSize === 'number' && Math.floor(obj_pageSize) !== obj_pageSize)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_pageSize + '" (at "' + path_pageSize + '")');
        }
        const obj_previousPageToken = obj.previousPageToken;
        const path_previousPageToken = path + '.previousPageToken';
        let obj_previousPageToken_union0 = null;
        const obj_previousPageToken_union0_error = (() => {
            if (typeof obj_previousPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union0_error != null) {
            obj_previousPageToken_union0 = obj_previousPageToken_union0_error.message;
        }
        let obj_previousPageToken_union1 = null;
        const obj_previousPageToken_union1_error = (() => {
            if (obj_previousPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union1_error != null) {
            obj_previousPageToken_union1 = obj_previousPageToken_union1_error.message;
        }
        if (obj_previousPageToken_union0 && obj_previousPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageToken + '")';
            message += '\n' + obj_previousPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_previousPageUrl = obj.previousPageUrl;
        const path_previousPageUrl = path + '.previousPageUrl';
        let obj_previousPageUrl_union0 = null;
        const obj_previousPageUrl_union0_error = (() => {
            if (typeof obj_previousPageUrl !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union0_error != null) {
            obj_previousPageUrl_union0 = obj_previousPageUrl_union0_error.message;
        }
        let obj_previousPageUrl_union1 = null;
        const obj_previousPageUrl_union1_error = (() => {
            if (obj_previousPageUrl !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageUrl + '" (at "' + path_previousPageUrl + '")');
            }
        })();
        if (obj_previousPageUrl_union1_error != null) {
            obj_previousPageUrl_union1 = obj_previousPageUrl_union1_error.message;
        }
        if (obj_previousPageUrl_union0 && obj_previousPageUrl_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageUrl + '")';
            message += '\n' + obj_previousPageUrl_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageUrl_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (!ArrayIsArray(obj_records)) {
            return new TypeError('Expected "array" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
        for (let i = 0; i < obj_records.length; i++) {
            const obj_records_item = obj_records[i];
            const path_records_item = path_records + '[' + i + ']';
            if (typeof obj_records_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_records_item + '" (at "' + path_records_item + '")');
            }
        }
        const obj_sortBy = obj.sortBy;
        const path_sortBy = path + '.sortBy';
        if (!ArrayIsArray(obj_sortBy)) {
            return new TypeError('Expected "array" but received "' + typeof obj_sortBy + '" (at "' + path_sortBy + '")');
        }
        for (let i = 0; i < obj_sortBy.length; i++) {
            const obj_sortBy_item = obj_sortBy[i];
            const path_sortBy_item = path_sortBy + '[' + i + ']';
            if (typeof obj_sortBy_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_sortBy_item + '" (at "' + path_sortBy_item + '")');
            }
        }
        const obj_where = obj.where;
        const path_where = path + '.where';
        let obj_where_union0 = null;
        const obj_where_union0_error = (() => {
            if (typeof obj_where !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_where + '" (at "' + path_where + '")');
            }
        })();
        if (obj_where_union0_error != null) {
            obj_where_union0 = obj_where_union0_error.message;
        }
        let obj_where_union1 = null;
        const obj_where_union1_error = (() => {
            if (obj_where !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_where + '" (at "' + path_where + '")');
            }
        })();
        if (obj_where_union1_error != null) {
            obj_where_union1 = obj_where_union1_error.message;
        }
        if (obj_where_union0 && obj_where_union1) {
            let message = 'Object doesn\'t match union (at "' + path_where + '")';
            message += '\n' + obj_where_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_where_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$i = 'RelatedListRecordCollectionRepresentation';
function keyBuilder$1S(luvio, config) {
    return keyPrefix + '::' + RepresentationType$i + ':' + '[' + config.sortBy.join(',') + ']' + ':' + (config.parentRecordId === null ? '' : config.parentRecordId) + ':' + config.relatedListId + ':' + (config.where === null ? '' : config.where);
}
function keyBuilderFromType$a(luvio, object) {
    const keyParams = {
        sortBy: object.sortBy,
        parentRecordId: object.listReference.inContextOfRecordId,
        relatedListId: object.listReference.relatedListId,
        where: object.where
    };
    return keyBuilder$1S(luvio, keyParams);
}
function normalize$c(input, existing, path, luvio, store, timestamp) {
    const input_records = input.records;
    const input_records_id = path.fullPath + '__records';
    for (let i = 0; i < input_records.length; i++) {
        const input_records_item = input_records[i];
        let input_records_item_id = input_records_id + '__' + i;
        input_records[i] = ingest$2g(input_records_item, {
            fullPath: input_records_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
function getPageMetadata$1(paginationMetadata, paginationParams) {
    const metadataProperties = {};
    const sink = {
        metadataProperties
    };
    if (paginationMetadata === undefined) {
        return sink;
    }
    const { token, pageSize } = paginationParams;
    const pageTokenAndOffset = getPageTokenAndOffset(paginationMetadata, token);
    if (pageTokenAndOffset === undefined) {
        return sink;
    }
    const [pageToken, startOffset] = pageTokenAndOffset;
    const endOffset = pageEndOffset(paginationMetadata, startOffset, pageSize);
    sink.startOffset = startOffset;
    sink.endOffset = endOffset;
    metadataProperties.currentPageToken = pageToken;
    const { clientToken: nextClientToken, serverToken: nextServerToken } = getNextPageTokens(paginationMetadata, endOffset);
    if (nextClientToken !== undefined) {
        metadataProperties.nextPageToken = nextClientToken;
        sink.nextServerToken = nextServerToken;
    }
    const previousClientToken = getPreviousClientToken(paginationMetadata, startOffset, pageSize);
    if (previousClientToken !== undefined) {
        metadataProperties.previousPageToken = previousClientToken;
    }
    metadataProperties.pageSize = pageSize;
    metadataProperties.count = endOffset - startOffset;
    return sink;
}
const dynamicSelect$4 = function dynamicRelatedListRecordCollectionRepresentationSelect(params, paginationParams) {
    const recordsPathSelection = params.records === undefined ? {
        name: 'records',
        kind: 'Link',
        fragment: select$3p()
    } : params.records;
    const { selections: RelatedListReferenceRepresentation__selections, opaque: RelatedListReferenceRepresentation__opaque, } = select$1X();
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: false,
        version: VERSION$1G,
        read: (source, reader) => {
            const sink = {};
            if (paginationParams === undefined) {
                return sink;
            }
            const metadata = reader.resolveMetadata(source, PAGINATION_VERSION);
            const currentPageMetadata = getPageMetadata$1(metadata, paginationParams);
            const { startOffset, endOffset, metadataProperties } = currentPageMetadata;
            if (metadata !== undefined && startOffset !== undefined && endOffset !== undefined) {
                const itemsProperty = 'records';
                reader.enterPath(itemsProperty);
                const itemsSource = source[itemsProperty];
                const itemsSink = [];
                const allSink = [];
                for (let i = startOffset; i < endOffset; i++) {
                    reader.enterPath(i);
                    reader.readSingleLink(i, recordsPathSelection, itemsSource, allSink, i - startOffset);
                    if (reader.getIsDataMissing() === false) {
                        itemsSink[i - startOffset] = allSink[i - startOffset];
                    }
                    reader.exitPath();
                }
                reader.exitPath();
                reader.assignNonScalar(sink, itemsProperty, itemsSink);
            }
            reader.enterPath('currentPageToken');
            reader.readScalar('currentPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('nextPageToken');
            reader.readScalar('nextPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('previousPageToken');
            reader.readScalar('previousPageToken', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('pageSize');
            reader.readScalar('pageSize', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('count');
            reader.readScalar('count', metadataProperties, sink);
            reader.exitPath();
            reader.enterPath('fields');
            reader.readScalarPlural('fields', source, sink, true);
            reader.exitPath();
            reader.enterPath('listInfoETag');
            reader.readScalar('listInfoETag', source, sink);
            reader.exitPath();
            reader.enterPath('listReference');
            reader.readObject('listReference', {
                name: 'listReference',
                kind: 'Object',
                selections: RelatedListReferenceRepresentation__selections
            }, source, sink);
            reader.exitPath();
            reader.enterPath('optionalFields');
            reader.readScalarPlural('optionalFields', source, sink, true);
            reader.exitPath();
            reader.enterPath('sortBy');
            reader.readScalarPlural('sortBy', source, sink, true);
            reader.exitPath();
            reader.enterPath('where');
            reader.readScalar('where', source, sink);
            reader.exitPath();
            return sink;
        },
    };
};
function equals$g(existing, incoming) {
    const existing_count = existing.count;
    const incoming_count = incoming.count;
    if (!(existing_count === incoming_count)) {
        return false;
    }
    const existing_pageSize = existing.pageSize;
    const incoming_pageSize = incoming.pageSize;
    if (!(existing_pageSize === incoming_pageSize)) {
        return false;
    }
    const existing_currentPageToken = existing.currentPageToken;
    const incoming_currentPageToken = incoming.currentPageToken;
    if (!(existing_currentPageToken === incoming_currentPageToken)) {
        return false;
    }
    const existing_currentPageUrl = existing.currentPageUrl;
    const incoming_currentPageUrl = incoming.currentPageUrl;
    if (!(existing_currentPageUrl === incoming_currentPageUrl)) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_items = equalsArray(existing_fields, incoming_fields, (existing_fields_item, incoming_fields_item) => {
        if (!(existing_fields_item === incoming_fields_item)) {
            return false;
        }
    });
    if (equals_fields_items === false) {
        return false;
    }
    const existing_listInfoETag = existing.listInfoETag;
    const incoming_listInfoETag = incoming.listInfoETag;
    if (!(existing_listInfoETag === incoming_listInfoETag)) {
        return false;
    }
    const existing_listReference = existing.listReference;
    const incoming_listReference = incoming.listReference;
    if (!(equals$l(existing_listReference, incoming_listReference))) {
        return false;
    }
    const existing_nextPageToken = existing.nextPageToken;
    const incoming_nextPageToken = incoming.nextPageToken;
    if (!(existing_nextPageToken === incoming_nextPageToken)) {
        return false;
    }
    const existing_nextPageUrl = existing.nextPageUrl;
    const incoming_nextPageUrl = incoming.nextPageUrl;
    if (!(existing_nextPageUrl === incoming_nextPageUrl)) {
        return false;
    }
    const existing_optionalFields = existing.optionalFields;
    const incoming_optionalFields = incoming.optionalFields;
    const equals_optionalFields_items = equalsArray(existing_optionalFields, incoming_optionalFields, (existing_optionalFields_item, incoming_optionalFields_item) => {
        if (!(existing_optionalFields_item === incoming_optionalFields_item)) {
            return false;
        }
    });
    if (equals_optionalFields_items === false) {
        return false;
    }
    const existing_previousPageToken = existing.previousPageToken;
    const incoming_previousPageToken = incoming.previousPageToken;
    if (!(existing_previousPageToken === incoming_previousPageToken)) {
        return false;
    }
    const existing_previousPageUrl = existing.previousPageUrl;
    const incoming_previousPageUrl = incoming.previousPageUrl;
    if (!(existing_previousPageUrl === incoming_previousPageUrl)) {
        return false;
    }
    const existing_records = existing.records;
    const incoming_records = incoming.records;
    const equals_records_items = equalsArray(existing_records, incoming_records, (existing_records_item, incoming_records_item) => {
        if (!(existing_records_item.__ref === incoming_records_item.__ref)) {
            return false;
        }
    });
    if (equals_records_items === false) {
        return false;
    }
    const existing_sortBy = existing.sortBy;
    const incoming_sortBy = incoming.sortBy;
    const equals_sortBy_items = equalsArray(existing_sortBy, incoming_sortBy, (existing_sortBy_item, incoming_sortBy_item) => {
        if (!(existing_sortBy_item === incoming_sortBy_item)) {
            return false;
        }
    });
    if (equals_sortBy_items === false) {
        return false;
    }
    const existing_where = existing.where;
    const incoming_where = incoming.where;
    if (!(existing_where === incoming_where)) {
        return false;
    }
    return true;
}
const ingest$1C = function RelatedListRecordCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$E(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$a(luvio, input);
    const existingRecord = store.readEntry(key);
    const ttlToUse = TTL$g;
    let incomingRecord = normalize$c(input, store.readEntry(key), {
        fullPath: key,
        parent: path.parent,
        propertyName: path.propertyName,
        ttl: ttlToUse
    }, luvio, store, timestamp);
    const paginationKey = key + '__pagination';
    const existingPaginationMetadata = store.readEntry(paginationKey);
    const { count: count, currentPageToken: token, nextPageToken: nextToken, pageSize: pageSize, previousPageToken: previousToken, records: items } = input;
    const startOffset = existingPaginationMetadata === undefined ? 0 : offsetFromServerToken(existingPaginationMetadata, token);
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0 ||
            (existingRecord && existingRecord.records && existingRecord.records.length < startOffset)) {
            throw new RangeError('currentPageToken value ' + token + ' is not recognized');
        }
    }
    const incomingPaginationMetadata = {
        clientTokens: {},
        serverTokens: []
    };
    const pageClientToken = generateClientToken(startOffset);
    addTokens(incomingPaginationMetadata, startOffset, pageClientToken, token);
    const nextOffset = startOffset + items.length;
    if (nextToken === null) {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    else {
        const nextClientToken = generateClientToken(nextOffset);
        addTokens(incomingPaginationMetadata, nextOffset, nextClientToken, nextToken);
        if (existingPaginationMetadata !== undefined) {
            const listEndOffset = existingPaginationMetadata.__END__;
            if (listEndOffset !== undefined && nextOffset >= listEndOffset) {
                incomingPaginationMetadata.__END__ = undefined;
            }
        }
    }
    if (previousToken !== null) {
        const previousOffset = Math.max(startOffset - pageSize, 0);
        const previousClientToken = generateClientToken(previousOffset);
        addTokens(incomingPaginationMetadata, previousOffset, previousClientToken, previousToken);
    }
    for (let i = startOffset + 1; i < nextOffset; i++) {
        const itemClientToken = generateClientToken(i);
        addTokens(incomingPaginationMetadata, i, itemClientToken);
    }
    const mergedPaginationMetadata = mergeMetadata(existingPaginationMetadata, incomingPaginationMetadata);
    if (existingPaginationMetadata === undefined ||
        equalsMetadata(existingPaginationMetadata, mergedPaginationMetadata) === false) {
        luvio.storePublish(paginationKey, mergedPaginationMetadata);
        luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata);
    }
    assignMetadataLink(incomingRecord, paginationKey);
    if (existingRecord !== undefined && existingRecord.records !== undefined) {
        incomingRecord.records = [
            ...existingRecord.records.slice(0, startOffset),
            ...incomingRecord.records,
            ...existingRecord.records.slice(nextOffset),
        ];
    }
    if (existingRecord === undefined || equals$g(existingRecord, incomingRecord) === false) {
        luvio.storePublish(key, incomingRecord);
    }
    {
        const storeMetadataParams = {
            ttl: ttlToUse,
            namespace: "UiApi",
            version: VERSION$1G,
            representationName: RepresentationType$i,
            ingestionTimestamp: timestamp,
        };
        luvio.publishStoreMetadata(key, storeMetadataParams);
    }
    return createLink$1(key);
};
function getTypeCacheKeys$1D(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$a(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$i,
        mergeable: true
    });
    rootKeySet.set(rootKey + '__pagination', {
        namespace: keyPrefix,
        representationName: RepresentationType$i,
        mergeable: true
    });
    const input_records_length = input.records.length;
    for (let i = 0; i < input_records_length; i++) {
        getTypeCacheKeys$2l(rootKeySet, luvio, input.records[i]);
    }
}

function select$1J(luvio, params) {
    const { fields = [], optionalFields = [] } = params.body;
    return dynamicSelect$4({
        records: {
            name: 'records',
            kind: 'Link',
            fragment: {
                kind: 'Fragment',
                private: ['eTag', 'weakEtag'],
                selections: buildSelectionFromFields(fields, optionalFields),
                version: VERSION$2K
            },
        },
    }, createPaginationParams(params));
}
function ingestSuccess$d(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1R(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1C, body);
    const optionalFields = resourceParams.body.optionalFields;
    if (optionalFields && optionalFields.length > 0) {
        const normalized = body;
        markMissingOptionalFieldsOnRecords(luvio, normalized, optionalFields);
    }
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1J(luvio, resourceParams),
        variables: {},
    }, snapshotRefresh);
    return snapshot;
}
function markMissingOptionalFieldsOnRecords(luvio, nomalized, optionalFields) {
    const records = nomalized.records;
    for (let i = 0; i < records.length; i++) {
        const record = records[i];
        const recordKey = record.__ref;
        if (recordKey !== undefined) {
            const node = luvio.getNode(recordKey);
            markMissingOptionalFields(node, optionalFields);
        }
    }
}

function createPaginationParams(params) {
    const { body } = params;
    return {
        token: body.pageToken,
        pageSize: body.pageSize === undefined ? 50 : body.pageSize
    };
}
function keyBuilder$1R(luvio, params) {
    return keyBuilder$1S(luvio, {
        sortBy: params.body.sortBy || [],
        where: params.body.where || null,
        parentRecordId: params.urlParams.parentRecordId,
        relatedListId: params.urlParams.relatedListId
    });
}
function getResponseCacheKeys$j(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1D(storeKeyMap, luvio, response);
}
function ingestError$9(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1R(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$g,
        namespace: keyPrefix,
        version: VERSION$1G,
        representationName: RepresentationType$i
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$o(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-records/' + config.urlParams.parentRecordId + '/' + config.urlParams.relatedListId + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

function createChildResourceParams$2(resourceParams) {
    const childConfigs = [];
    for (let index = 0, len = resourceParams.body.relatedListParameters.length; index < len; index += 1) {
        let childConfig = {
            urlParams: {
                parentRecordId: resourceParams.urlParams.parentRecordId,
                relatedListId: resourceParams.body.relatedListParameters[index].relatedListId,
            },
            body: {},
        };
        let relatedListConfig = resourceParams.body.relatedListParameters[index];
        // manually populate childConfig body
        if ('fields' in relatedListConfig) {
            childConfig.body.fields = relatedListConfig.fields;
        }
        if ('optionalFields' in relatedListConfig) {
            childConfig.body.optionalFields = relatedListConfig.optionalFields;
        }
        if ('pageSize' in relatedListConfig) {
            childConfig.body.pageSize = relatedListConfig.pageSize;
        }
        if ('sortBy' in relatedListConfig) {
            childConfig.body.sortBy = relatedListConfig.sortBy;
        }
        if ('where' in relatedListConfig) {
            childConfig.body.where = relatedListConfig.where;
        }
        childConfigs.push(childConfig);
    }
    return childConfigs;
}

const nonCachedErrors$1 = ObjectCreate$1(null);
function createChildResourceParams$1(resourceParams) {
    return createChildResourceParams$2(resourceParams);
}
function selectChildResourceParams$1(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$1Q(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error && result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$1R(luvio, childResource);
                const childFragment = select$1J(luvio, childResource);
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error':
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors$1[childKey];
                            if (nonCachedError === undefined || nonCachedError.expiration < reader.getTimeStamp()) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            reader.exitPath();
            return sink;
        }
    };
}
function select$1I(luvio, resourceParams) {
    const childResources = createChildResourceParams$1(resourceParams);
    return selectChildResourceParams$1(luvio, childResources, resourceParams);
}
function keyBuilder$1Q(luvio, params) {
    return keyPrefix + '::RelatedListRecordCollectionBatchRepresentation:(' + 'parentRecordId:' + params.urlParams.parentRecordId + ',' + '[' + params.body.relatedListParameters.map(element => (element.fields === undefined ? 'relatedListParameters.fields' : 'relatedListParameters.fields:' + element.fields) + '::' + (element.optionalFields === undefined ? 'relatedListParameters.optionalFields' : 'relatedListParameters.optionalFields:' + element.optionalFields) + '::' + (element.pageSize === undefined ? 'relatedListParameters.pageSize' : 'relatedListParameters.pageSize:' + element.pageSize) + '::' + 'relatedListParameters.relatedListId:' + element.relatedListId + '::' + (element.sortBy === undefined ? 'relatedListParameters.sortBy' : 'relatedListParameters.sortBy:' + element.sortBy) + '::' + (element.where === undefined ? 'relatedListParameters.where' : 'relatedListParameters.where:' + element.where)).join(',') + ']' + ')';
}
function getResponseCacheKeys$i(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams$1(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$j(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$1R(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$i,
                mergeable: false
            });
        }
    }
}
function ingestSuccessChildResourceParams$2(luvio, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    const now = Date.now();
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$1R(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$d(luvio, childResourceParams, childResponse);
            if (childSnapshot.state === "Stale") {
                snapshotState = "Stale";
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse'
            };
            ingestError$9(luvio, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            deepFreeze(childValue);
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors$1[childKey] = { expiration: now + TTL$g, response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors$1[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses
    };
    deepFreeze(childSnapshotData);
    return { childSnapshotData: childSnapshotData, seenRecords, snapshotState };
}
function ingestSuccess$c(luvio, resourceParams, response, snapshotRefresh) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams$1(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' + childResourceParamsArray.length + ' items, received ' + childEnvelopes.length);
        }
    }
    const key = keyBuilder$1Q(luvio, resourceParams);
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$2(luvio, childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$1I(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {}
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$8(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1Q(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}
function createResourceRequest$n(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/related-list-records/batch/' + config.urlParams.parentRecordId + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$f = 'getRelatedListRecordsBatch';
const getRelatedListRecordsBatch_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentRecordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('relatedListParameters', true, 2 /* Body */, 4 /* Unsupported */, true),
];
const getRelatedListRecordsBatch_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$f, getRelatedListRecordsBatch_ConfigPropertyMetadata);
const createResourceParams$j = /*#__PURE__*/ createResourceParams$k(getRelatedListRecordsBatch_ConfigPropertyMetadata);
function keyBuilder$1P(luvio, config) {
    const resourceParams = createResourceParams$j(config);
    return keyBuilder$1Q(luvio, resourceParams);
}
function typeCheckConfig$k(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListRecordsBatch_ConfigPropertyMetadata);
    const untrustedConfig_relatedListParameters = untrustedConfig.relatedListParameters;
    if (ArrayIsArray$1(untrustedConfig_relatedListParameters)) {
        const untrustedConfig_relatedListParameters_array = [];
        for (let i = 0, arrayLength = untrustedConfig_relatedListParameters.length; i < arrayLength; i++) {
            const untrustedConfig_relatedListParameters_item = untrustedConfig_relatedListParameters[i];
            const referenceRelatedListRecordsSingleBatchInputRepresentationValidationError = validate$F(untrustedConfig_relatedListParameters_item);
            if (referenceRelatedListRecordsSingleBatchInputRepresentationValidationError === null) {
                untrustedConfig_relatedListParameters_array.push(untrustedConfig_relatedListParameters_item);
            }
        }
        config.relatedListParameters = untrustedConfig_relatedListParameters_array;
    }
    return config;
}
function validateAdapterConfig$k(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListRecordsBatch_ConfigPropertyMetadata);
    const config = typeCheckConfig$k(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$a(luvio, config) {
    const resourceParams = createResourceParams$j(config);
    return select$1I(luvio, resourceParams);
}
function onFetchResponseSuccess$b(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$c(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$l(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$a(luvio, config, resourceParams, response) {
    const snapshot = ingestError$8(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$l(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$l(luvio, config, options) {
    const resourceParams = createResourceParams$j(config);
    const request = createResourceRequest$n(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$b(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$i(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$a(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$d(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$l, 'get', false);
}
function buildCachedSnapshotCachePolicy$c(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1P(luvio, config),
        node: adapterFragment$a(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$l(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getRelatedListRecordsBatchAdapterFactory = (luvio) => function UiApi__getRelatedListRecordsBatch(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$k(untrustedConfig, getRelatedListRecordsBatch_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$c, buildNetworkSnapshotCachePolicy$d);
};

const adapterName$e = 'getRelatedListRecords';
const getRelatedListRecords_ConfigPropertyMetadata = [
    generateParamConfigMetadata('parentRecordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('relatedListId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('fields', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('optionalFields', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('pageSize', false, 2 /* Body */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('sortBy', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('where', false, 2 /* Body */, 0 /* String */),
];
const getRelatedListRecords_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$e, getRelatedListRecords_ConfigPropertyMetadata);
const createResourceParams$i = /*#__PURE__*/ createResourceParams$k(getRelatedListRecords_ConfigPropertyMetadata);
function keyBuilder$1O(luvio, config) {
    const resourceParams = createResourceParams$i(config);
    return keyBuilder$1R(luvio, resourceParams);
}
function typeCheckConfig$j(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRelatedListRecords_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$j(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRelatedListRecords_ConfigPropertyMetadata);
    const config = typeCheckConfig$j(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$9(luvio, config) {
    const resourceParams = createResourceParams$i(config);
    return select$1J(luvio, resourceParams);
}
function onFetchResponseSuccess$a(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$d(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$k(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$9(luvio, config, resourceParams, response) {
    const snapshot = ingestError$9(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$k(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function prepareRequest$2(luvio, config, resourceParams, cacheSnapshot) {
    const paginationMetadata = getPaginationMetadata$1(luvio, resourceParams);
    if (paginationMetadata === undefined) {
        return createResourceRequest$o(resourceParams);
    }
    const { token: configToken, pageSize: configPageSize } = createPaginationParams(resourceParams);
    const configTokenOffset = offsetFromClientToken(paginationMetadata, configToken);
    const maxOffset = getRequestMaxOffset(configTokenOffset, cacheSnapshot);
    const [serverToken, serverTokenOffset] = serverTokenForAtMost(paginationMetadata, maxOffset);
    return createResourceRequest$o(createResourceParams$i({
        ...config,
        pageSize: configPageSize - (serverTokenOffset - configTokenOffset),
        pageToken: serverToken
    }));
}
function getRequestMaxOffset(configTokenOffset, cacheSnapshot) {
    if (cacheSnapshot === undefined) {
        return configTokenOffset;
    }
    const num = getCachedItemsNumber$1(cacheSnapshot);
    if (num <= 0) {
        return configTokenOffset;
    }
    return Math.max(configTokenOffset + num, configTokenOffset);
}
function getCachedItemsNumber$1(cacheSnapshot) {
    const { data } = cacheSnapshot;
    if (data === undefined || !data.records) {
        return 0;
    }
    const items = data.records;
    for (let i = 0; i < items.length; i++) {
        if (items[i] === undefined) {
            return i;
        }
    }
    return items.length;
}
function getPaginationMetadata$1(luvio, resourceParams) {
    const key = keyBuilder$1R(luvio, resourceParams);
    const selector = {
        recordId: key,
        node: {
            kind: 'Fragment',
            version: VERSION$1G,
            private: [],
            selections: [
                {
                    kind: 'Link',
                    name: '__metadata',
                    fragment: {
                        kind: 'Fragment',
                        version: PAGINATION_VERSION,
                        opaque: true,
                        private: [],
                    },
                },
            ],
        },
        variables: {}
    };
    const node = luvio.storeLookup(selector);
    if (node.state !== 'Fulfilled') {
        return;
    }
    return node.data.__metadata;
}
function buildNetworkSnapshot$k(luvio, config, options, cacheSnapshot) {
    const resourceParams = createResourceParams$i(config);
    const request = prepareRequest$2(luvio, config, resourceParams, cacheSnapshot);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$a(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$j(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$9(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$c(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$k, 'get', true);
}
function buildCachedSnapshotCachePolicy$b(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1O(luvio, config),
        node: adapterFragment$9(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$k(luvio, config, snapshotRefreshOptions)
    });
    if (isUnfulfilledSnapshot$1(cacheSnapshot) && cacheSnapshot.data !== undefined) {
        context.cacheSnapshot = cacheSnapshot;
    }
    return cacheSnapshot;
}
const getRelatedListRecordsAdapterFactory = (luvio) => function UiApi__getRelatedListRecords(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$j(untrustedConfig, getRelatedListRecords_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$b, buildNetworkSnapshotCachePolicy$c);
};

function validate$D(obj, path = 'SearchFilterOptionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_value = obj.value;
        const path_value = path + '.value';
        if (typeof obj_value !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_value + '" (at "' + path_value + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

var DiscriminatorValues$1;
(function (DiscriminatorValues) {
    DiscriminatorValues["Text"] = "Text";
    DiscriminatorValues["Select"] = "Select";
})(DiscriminatorValues$1 || (DiscriminatorValues$1 = {}));

var DiscriminatorValues;
(function (DiscriminatorValues) {
    DiscriminatorValues["Field"] = "Field";
    DiscriminatorValues["DataCategory"] = "DataCategory";
})(DiscriminatorValues || (DiscriminatorValues = {}));
function validate$C(obj, path = 'SearchFilterDefinitionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_defaultValues = obj.defaultValues;
        const path_defaultValues = path + '.defaultValues';
        if (!ArrayIsArray(obj_defaultValues)) {
            return new TypeError('Expected "array" but received "' + typeof obj_defaultValues + '" (at "' + path_defaultValues + '")');
        }
        for (let i = 0; i < obj_defaultValues.length; i++) {
            const obj_defaultValues_item = obj_defaultValues[i];
            const path_defaultValues_item = path_defaultValues + '[' + i + ']';
            if (typeof obj_defaultValues_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_defaultValues_item + '" (at "' + path_defaultValues_item + '")');
            }
        }
        if (obj.helpMessage !== undefined) {
            const obj_helpMessage = obj.helpMessage;
            const path_helpMessage = path + '.helpMessage';
            let obj_helpMessage_union0 = null;
            const obj_helpMessage_union0_error = (() => {
                if (typeof obj_helpMessage !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_helpMessage + '" (at "' + path_helpMessage + '")');
                }
            })();
            if (obj_helpMessage_union0_error != null) {
                obj_helpMessage_union0 = obj_helpMessage_union0_error.message;
            }
            let obj_helpMessage_union1 = null;
            const obj_helpMessage_union1_error = (() => {
                if (obj_helpMessage !== null) {
                    return new TypeError('Expected "null" but received "' + typeof obj_helpMessage + '" (at "' + path_helpMessage + '")');
                }
            })();
            if (obj_helpMessage_union1_error != null) {
                obj_helpMessage_union1 = obj_helpMessage_union1_error.message;
            }
            if (obj_helpMessage_union0 && obj_helpMessage_union1) {
                let message = 'Object doesn\'t match union (at "' + path_helpMessage + '")';
                message += '\n' + obj_helpMessage_union0.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_helpMessage_union1.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_type = obj.type;
        const path_type = path + '.type';
        if (typeof obj_type !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_type + '" (at "' + path_type + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$f = 30000;
const VERSION$1F = "46ebae6041d5914e45bd2ccc6cb89271";
function validate$B(obj, path = 'SearchFilterMetadataCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_channel = obj.channel;
        const path_channel = path + '.channel';
        let obj_channel_union0 = null;
        const obj_channel_union0_error = (() => {
            if (typeof obj_channel !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_channel + '" (at "' + path_channel + '")');
            }
        })();
        if (obj_channel_union0_error != null) {
            obj_channel_union0 = obj_channel_union0_error.message;
        }
        let obj_channel_union1 = null;
        const obj_channel_union1_error = (() => {
            if (obj_channel !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_channel + '" (at "' + path_channel + '")');
            }
        })();
        if (obj_channel_union1_error != null) {
            obj_channel_union1 = obj_channel_union1_error.message;
        }
        if (obj_channel_union0 && obj_channel_union1) {
            let message = 'Object doesn\'t match union (at "' + path_channel + '")';
            message += '\n' + obj_channel_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_channel_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_configurationName = obj.configurationName;
        const path_configurationName = path + '.configurationName';
        let obj_configurationName_union0 = null;
        const obj_configurationName_union0_error = (() => {
            if (typeof obj_configurationName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_configurationName + '" (at "' + path_configurationName + '")');
            }
        })();
        if (obj_configurationName_union0_error != null) {
            obj_configurationName_union0 = obj_configurationName_union0_error.message;
        }
        let obj_configurationName_union1 = null;
        const obj_configurationName_union1_error = (() => {
            if (obj_configurationName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_configurationName + '" (at "' + path_configurationName + '")');
            }
        })();
        if (obj_configurationName_union1_error != null) {
            obj_configurationName_union1 = obj_configurationName_union1_error.message;
        }
        if (obj_configurationName_union0 && obj_configurationName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_configurationName + '")';
            message += '\n' + obj_configurationName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_configurationName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_filters = obj.filters;
        const path_filters = path + '.filters';
        if (!ArrayIsArray(obj_filters)) {
            return new TypeError('Expected "array" but received "' + typeof obj_filters + '" (at "' + path_filters + '")');
        }
        for (let i = 0; i < obj_filters.length; i++) {
            const obj_filters_item = obj_filters[i];
            const path_filters_item = path_filters + '[' + i + ']';
            const referencepath_filters_itemValidationError = validate$C(obj_filters_item, path_filters_item);
            if (referencepath_filters_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchFilterDefinitionRepresentation (at "' + path_filters_item + '")\n';
                message += referencepath_filters_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$h = 'SearchFilterMetadataCollectionRepresentation';
function keyBuilder$1N(luvio, config) {
    return keyPrefix + '::' + RepresentationType$h + ':' + config.objectApiName + ':' + (config.channel === null ? '' : config.channel) + ':' + (config.configurationName === null ? '' : config.configurationName);
}
function keyBuilderFromType$9(luvio, object) {
    const keyParams = {
        objectApiName: object.objectApiName,
        channel: object.channel,
        configurationName: object.configurationName
    };
    return keyBuilder$1N(luvio, keyParams);
}
function normalize$b(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1H = function SearchFilterMetadataCollectionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1F,
        private: [],
        opaque: true
    };
};
function equals$f(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1B = function SearchFilterMetadataCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$B(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$9(luvio, input);
    const ttlToUse = TTL$f;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$b, "UiApi", VERSION$1F, RepresentationType$h, equals$f);
    return createLink$1(key);
};
function getTypeCacheKeys$1C(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$9(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$h,
        mergeable: false
    });
}

function select$1G(luvio, params) {
    return select$1H();
}
function keyBuilder$1M(luvio, params) {
    return keyBuilder$1N(luvio, {
        objectApiName: params.urlParams.objectApiName,
        channel: params.queryParams.channel || null,
        configurationName: params.queryParams.configurationName || null
    });
}
function getResponseCacheKeys$h(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1C(storeKeyMap, luvio, response);
}
function ingestSuccess$b(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1M(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1B, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1G(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$7(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1M(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$f,
        namespace: keyPrefix,
        version: VERSION$1F,
        representationName: RepresentationType$h
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$m(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/search-info/' + config.urlParams.objectApiName + '/filters',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$d = 'getSearchFilterMetadata';
const getSearchFilterMetadata_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('channel', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('configurationName', false, 1 /* QueryParameter */, 0 /* String */),
];
const getSearchFilterMetadata_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$d, getSearchFilterMetadata_ConfigPropertyMetadata);
const createResourceParams$h = /*#__PURE__*/ createResourceParams$k(getSearchFilterMetadata_ConfigPropertyMetadata);
function keyBuilder$1L(luvio, config) {
    const resourceParams = createResourceParams$h(config);
    return keyBuilder$1M(luvio, resourceParams);
}
function typeCheckConfig$i(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getSearchFilterMetadata_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$i(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$i(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$8(luvio, config) {
    createResourceParams$h(config);
    return select$1G();
}
function onFetchResponseSuccess$9(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$b(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$j(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$8(luvio, config, resourceParams, response) {
    const snapshot = ingestError$7(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$j(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$j(luvio, config, options) {
    const resourceParams = createResourceParams$h(config);
    const request = createResourceRequest$m(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$9(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$h(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$8(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$b(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$j, undefined, false);
}
function buildCachedSnapshotCachePolicy$a(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1L(luvio, config),
        node: adapterFragment$8(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$j(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getSearchFilterMetadataAdapterFactory = (luvio) => function UiApi__getSearchFilterMetadata(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$i(untrustedConfig, getSearchFilterMetadata_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$a, buildNetworkSnapshotCachePolicy$b);
};

const TTL$e = 30000;
const VERSION$1E = "8d851a8d9abf0a061a8ad81d4cbb83bc";
function validate$A(obj, path = 'SearchFilterOptionCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_filterApiName = obj.filterApiName;
        const path_filterApiName = path + '.filterApiName';
        if (typeof obj_filterApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_filterApiName + '" (at "' + path_filterApiName + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_options = obj.options;
        const path_options = path + '.options';
        if (!ArrayIsArray(obj_options)) {
            return new TypeError('Expected "array" but received "' + typeof obj_options + '" (at "' + path_options + '")');
        }
        for (let i = 0; i < obj_options.length; i++) {
            const obj_options_item = obj_options[i];
            const path_options_item = path_options + '[' + i + ']';
            const referencepath_options_itemValidationError = validate$D(obj_options_item, path_options_item);
            if (referencepath_options_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchFilterOptionRepresentation (at "' + path_options_item + '")\n';
                message += referencepath_options_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_query = obj.query;
        const path_query = path + '.query';
        let obj_query_union0 = null;
        const obj_query_union0_error = (() => {
            if (typeof obj_query !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_query + '" (at "' + path_query + '")');
            }
        })();
        if (obj_query_union0_error != null) {
            obj_query_union0 = obj_query_union0_error.message;
        }
        let obj_query_union1 = null;
        const obj_query_union1_error = (() => {
            if (obj_query !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_query + '" (at "' + path_query + '")');
            }
        })();
        if (obj_query_union1_error != null) {
            obj_query_union1 = obj_query_union1_error.message;
        }
        if (obj_query_union0 && obj_query_union1) {
            let message = 'Object doesn\'t match union (at "' + path_query + '")';
            message += '\n' + obj_query_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_query_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$g = 'SearchFilterOptionCollectionRepresentation';
function keyBuilder$1K(luvio, config) {
    return keyPrefix + '::' + RepresentationType$g + ':' + (config.query === null ? '' : config.query) + ':' + config.objectApiName + ':' + config.filterApiName;
}
function keyBuilderFromType$8(luvio, object) {
    const keyParams = {
        query: object.query,
        objectApiName: object.objectApiName,
        filterApiName: object.filterApiName
    };
    return keyBuilder$1K(luvio, keyParams);
}
function normalize$a(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1F = function SearchFilterOptionCollectionRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1E,
        private: [],
        opaque: true
    };
};
function equals$e(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1A = function SearchFilterOptionCollectionRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$A(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$8(luvio, input);
    const ttlToUse = TTL$e;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$a, "UiApi", VERSION$1E, RepresentationType$g, equals$e);
    return createLink$1(key);
};
function getTypeCacheKeys$1B(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$8(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$g,
        mergeable: false
    });
}

function select$1E(luvio, params) {
    return select$1F();
}
function keyBuilder$1J(luvio, params) {
    return keyBuilder$1K(luvio, {
        query: params.queryParams.q || null,
        objectApiName: params.urlParams.objectApiName,
        filterApiName: params.urlParams.filterApiName
    });
}
function getResponseCacheKeys$g(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1B(storeKeyMap, luvio, response);
}
function ingestSuccess$a(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1J(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1A, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1E(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$6(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1J(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$e,
        namespace: keyPrefix,
        version: VERSION$1E,
        representationName: RepresentationType$g
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$l(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/search-info/' + config.urlParams.objectApiName + '/filters/' + config.urlParams.filterApiName + '/options',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$c = 'getSearchFilterOptions';
const getSearchFilterOptions_ConfigPropertyMetadata = [
    generateParamConfigMetadata('filterApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('q', false, 1 /* QueryParameter */, 0 /* String */),
];
const getSearchFilterOptions_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$c, getSearchFilterOptions_ConfigPropertyMetadata);
const createResourceParams$g = /*#__PURE__*/ createResourceParams$k(getSearchFilterOptions_ConfigPropertyMetadata);
function keyBuilder$1I(luvio, config) {
    const resourceParams = createResourceParams$g(config);
    return keyBuilder$1J(luvio, resourceParams);
}
function typeCheckConfig$h(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getSearchFilterOptions_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$h(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$h(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$7(luvio, config) {
    createResourceParams$g(config);
    return select$1E();
}
function onFetchResponseSuccess$8(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$a(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$i(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$7(luvio, config, resourceParams, response) {
    const snapshot = ingestError$6(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$i(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$i(luvio, config, options) {
    const resourceParams = createResourceParams$g(config);
    const request = createResourceRequest$l(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$8(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$g(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$7(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$a(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$i, undefined, false);
}
function buildCachedSnapshotCachePolicy$9(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1I(luvio, config),
        node: adapterFragment$7(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$i(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getSearchFilterOptionsAdapterFactory = (luvio) => function UiApi__getSearchFilterOptions(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$h(untrustedConfig, getSearchFilterOptions_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$9, buildNetworkSnapshotCachePolicy$a);
};

function validate$z(obj, path = 'AdvancedLookupDisplayInfoFieldRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldPath = obj.fieldPath;
        const path_fieldPath = path + '.fieldPath';
        if (typeof obj_fieldPath !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldPath + '" (at "' + path_fieldPath + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$y(obj, path = 'AdvancedLookupDisplayInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (!ArrayIsArray(obj_fields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        for (let i = 0; i < obj_fields.length; i++) {
            const obj_fields_item = obj_fields[i];
            const path_fields_item = path_fields + '[' + i + ']';
            const referencepath_fields_itemValidationError = validate$z(obj_fields_item, path_fields_item);
            if (referencepath_fields_itemValidationError !== null) {
                let message = 'Object doesn\'t match AdvancedLookupDisplayInfoFieldRepresentation (at "' + path_fields_item + '")\n';
                message += referencepath_fields_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$x(obj, path = 'AdvancedLookupInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_displayInfo = obj.displayInfo;
        const path_displayInfo = path + '.displayInfo';
        const referencepath_displayInfoValidationError = validate$y(obj_displayInfo, path_displayInfo);
        if (referencepath_displayInfoValidationError !== null) {
            let message = 'Object doesn\'t match AdvancedLookupDisplayInfoRepresentation (at "' + path_displayInfo + '")\n';
            message += referencepath_displayInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$w(obj, path = 'DisplayLayoutRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_displayFields = obj.displayFields;
        const path_displayFields = path + '.displayFields';
        if (!ArrayIsArray(obj_displayFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_displayFields + '" (at "' + path_displayFields + '")');
        }
        for (let i = 0; i < obj_displayFields.length; i++) {
            const obj_displayFields_item = obj_displayFields[i];
            const path_displayFields_item = path_displayFields + '[' + i + ']';
            if (typeof obj_displayFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_displayFields_item + '" (at "' + path_displayFields_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$v(obj, path = 'MatchingInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_matchingFields = obj.matchingFields;
        const path_matchingFields = path + '.matchingFields';
        if (!ArrayIsArray(obj_matchingFields)) {
            return new TypeError('Expected "array" but received "' + typeof obj_matchingFields + '" (at "' + path_matchingFields + '")');
        }
        for (let i = 0; i < obj_matchingFields.length; i++) {
            const obj_matchingFields_item = obj_matchingFields[i];
            const path_matchingFields_item = path_matchingFields + '[' + i + ']';
            if (typeof obj_matchingFields_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_matchingFields_item + '" (at "' + path_matchingFields_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$u(obj, path = 'DisplayLayoutAndMatchingInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_displayLayout = obj.displayLayout;
        const path_displayLayout = path + '.displayLayout';
        const referencepath_displayLayoutValidationError = validate$w(obj_displayLayout, path_displayLayout);
        if (referencepath_displayLayoutValidationError !== null) {
            let message = 'Object doesn\'t match DisplayLayoutRepresentation (at "' + path_displayLayout + '")\n';
            message += referencepath_displayLayoutValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_matchingInfo = obj.matchingInfo;
        const path_matchingInfo = path + '.matchingInfo';
        const referencepath_matchingInfoValidationError = validate$v(obj_matchingInfo, path_matchingInfo);
        if (referencepath_matchingInfoValidationError !== null) {
            let message = 'Object doesn\'t match MatchingInfoRepresentation (at "' + path_matchingInfo + '")\n';
            message += referencepath_matchingInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$t(obj, path = 'LookupMetadataTargetInfoRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_advancedLookupInfo = obj.advancedLookupInfo;
        const path_advancedLookupInfo = path + '.advancedLookupInfo';
        const referencepath_advancedLookupInfoValidationError = validate$x(obj_advancedLookupInfo, path_advancedLookupInfo);
        if (referencepath_advancedLookupInfoValidationError !== null) {
            let message = 'Object doesn\'t match AdvancedLookupInfoRepresentation (at "' + path_advancedLookupInfo + '")\n';
            message += referencepath_advancedLookupInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_suggestionsInfo = obj.suggestionsInfo;
        const path_suggestionsInfo = path + '.suggestionsInfo';
        const referencepath_suggestionsInfoValidationError = validate$u(obj_suggestionsInfo, path_suggestionsInfo);
        if (referencepath_suggestionsInfoValidationError !== null) {
            let message = 'Object doesn\'t match DisplayLayoutAndMatchingInfoRepresentation (at "' + path_suggestionsInfo + '")\n';
            message += referencepath_suggestionsInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$d = 30000;
const VERSION$1D = "ab99b79a5e8a78e051ec92b39d76a6bd";
function validate$s(obj, path = 'LookupMetadataRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldApiName = obj.fieldApiName;
        const path_fieldApiName = path + '.fieldApiName';
        if (typeof obj_fieldApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldApiName + '" (at "' + path_fieldApiName + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_secondaryField = obj.secondaryField;
        const path_secondaryField = path + '.secondaryField';
        let obj_secondaryField_union0 = null;
        const obj_secondaryField_union0_error = (() => {
            if (typeof obj_secondaryField !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_secondaryField + '" (at "' + path_secondaryField + '")');
            }
        })();
        if (obj_secondaryField_union0_error != null) {
            obj_secondaryField_union0 = obj_secondaryField_union0_error.message;
        }
        let obj_secondaryField_union1 = null;
        const obj_secondaryField_union1_error = (() => {
            if (obj_secondaryField !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_secondaryField + '" (at "' + path_secondaryField + '")');
            }
        })();
        if (obj_secondaryField_union1_error != null) {
            obj_secondaryField_union1 = obj_secondaryField_union1_error.message;
        }
        if (obj_secondaryField_union0 && obj_secondaryField_union1) {
            let message = 'Object doesn\'t match union (at "' + path_secondaryField + '")';
            message += '\n' + obj_secondaryField_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_secondaryField_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_targetInfo = obj.targetInfo;
        const path_targetInfo = path + '.targetInfo';
        if (typeof obj_targetInfo !== 'object' || ArrayIsArray(obj_targetInfo) || obj_targetInfo === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_targetInfo + '" (at "' + path_targetInfo + '")');
        }
        const obj_targetInfo_keys = ObjectKeys(obj_targetInfo);
        for (let i = 0; i < obj_targetInfo_keys.length; i++) {
            const key = obj_targetInfo_keys[i];
            const obj_targetInfo_prop = obj_targetInfo[key];
            const path_targetInfo_prop = path_targetInfo + '["' + key + '"]';
            const referencepath_targetInfo_propValidationError = validate$t(obj_targetInfo_prop, path_targetInfo_prop);
            if (referencepath_targetInfo_propValidationError !== null) {
                let message = 'Object doesn\'t match LookupMetadataTargetInfoRepresentation (at "' + path_targetInfo_prop + '")\n';
                message += referencepath_targetInfo_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$f = 'LookupMetadataRepresentation';
function keyBuilder$1H(luvio, config) {
    return keyPrefix + '::' + RepresentationType$f + ':' + config.objectApiName + ':' + config.fieldApiName;
}
function keyBuilderFromType$7(luvio, object) {
    const keyParams = {
        objectApiName: object.objectApiName,
        fieldApiName: object.fieldApiName
    };
    return keyBuilder$1H(luvio, keyParams);
}
function normalize$9(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1D = function LookupMetadataRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1D,
        private: [],
        opaque: true
    };
};
function equals$d(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1z = function LookupMetadataRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$s(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$7(luvio, input);
    const ttlToUse = TTL$d;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$9, "UiApi", VERSION$1D, RepresentationType$f, equals$d);
    return createLink$1(key);
};
function getTypeCacheKeys$1A(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$7(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$f,
        mergeable: false
    });
}

function select$1C(luvio, params) {
    return select$1D();
}
function keyBuilder$1G(luvio, params) {
    return keyBuilder$1H(luvio, {
        objectApiName: params.urlParams.objectApiName,
        fieldApiName: params.urlParams.fieldApiName
    });
}
function getResponseCacheKeys$f(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1A(storeKeyMap, luvio, response);
}
function ingestSuccess$9(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1G(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1z, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1C(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$5(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1G(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$d,
        namespace: keyPrefix,
        version: VERSION$1D,
        representationName: RepresentationType$f
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$k(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/search-info/' + config.urlParams.objectApiName + '/lookup/' + config.urlParams.fieldApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$b = 'getLookupMetadata';
const getLookupMetadata_ConfigPropertyMetadata = [
    generateParamConfigMetadata('fieldApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
];
const getLookupMetadata_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$b, getLookupMetadata_ConfigPropertyMetadata);
const createResourceParams$f = /*#__PURE__*/ createResourceParams$k(getLookupMetadata_ConfigPropertyMetadata);
function keyBuilder$1F(luvio, config) {
    const resourceParams = createResourceParams$f(config);
    return keyBuilder$1G(luvio, resourceParams);
}
function typeCheckConfig$g(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getLookupMetadata_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$g(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$g(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$6(luvio, config) {
    createResourceParams$f(config);
    return select$1C();
}
function onFetchResponseSuccess$7(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$9(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$h(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$6(luvio, config, resourceParams, response) {
    const snapshot = ingestError$5(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$h(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$h(luvio, config, options) {
    const resourceParams = createResourceParams$f(config);
    const request = createResourceRequest$k(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$7(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$f(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$6(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$9(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$h, undefined, false);
}
function buildCachedSnapshotCachePolicy$8(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1F(luvio, config),
        node: adapterFragment$6(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$h(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getLookupMetadataAdapterFactory = (luvio) => function UiApi__getLookupMetadata(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$g(untrustedConfig, getLookupMetadata_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$8, buildNetworkSnapshotCachePolicy$9);
};

function validate$r(obj, path = 'SearchDataCategoryInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_groupName = obj.groupName;
        const path_groupName = path + '.groupName';
        if (typeof obj_groupName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_groupName + '" (at "' + path_groupName + '")');
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
        const obj_values = obj.values;
        const path_values = path + '.values';
        if (!ArrayIsArray(obj_values)) {
            return new TypeError('Expected "array" but received "' + typeof obj_values + '" (at "' + path_values + '")');
        }
        for (let i = 0; i < obj_values.length; i++) {
            const obj_values_item = obj_values[i];
            const path_values_item = path_values + '[' + i + ']';
            if (typeof obj_values_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$q(obj, path = 'SearchFilterInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldPath = obj.fieldPath;
        const path_fieldPath = path + '.fieldPath';
        if (typeof obj_fieldPath !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldPath + '" (at "' + path_fieldPath + '")');
        }
        if (obj.objectApiName !== undefined) {
            const obj_objectApiName = obj.objectApiName;
            const path_objectApiName = path + '.objectApiName';
            if (typeof obj_objectApiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
            }
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
        const obj_values = obj.values;
        const path_values = path + '.values';
        if (!ArrayIsArray(obj_values)) {
            return new TypeError('Expected "array" but received "' + typeof obj_values + '" (at "' + path_values + '")');
        }
        for (let i = 0; i < obj_values.length; i++) {
            const obj_values_item = obj_values[i];
            const path_values_item = path_values + '[' + i + ']';
            let obj_values_item_union0 = null;
            const obj_values_item_union0_error = (() => {
                if (typeof obj_values_item !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
                }
            })();
            if (obj_values_item_union0_error != null) {
                obj_values_item_union0 = obj_values_item_union0_error.message;
            }
            let obj_values_item_union1 = null;
            const obj_values_item_union1_error = (() => {
                if (typeof obj_values_item !== 'number') {
                    return new TypeError('Expected "number" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
                }
            })();
            if (obj_values_item_union1_error != null) {
                obj_values_item_union1 = obj_values_item_union1_error.message;
            }
            let obj_values_item_union2 = null;
            const obj_values_item_union2_error = (() => {
                if (typeof obj_values_item !== 'boolean') {
                    return new TypeError('Expected "boolean" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
                }
            })();
            if (obj_values_item_union2_error != null) {
                obj_values_item_union2 = obj_values_item_union2_error.message;
            }
            if (obj_values_item_union0 && obj_values_item_union1 && obj_values_item_union2) {
                let message = 'Object doesn\'t match union (at "' + path_values_item + '")';
                message += '\n' + obj_values_item_union0.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_values_item_union1.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_values_item_union2.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$p(obj, path = 'SearchObjectOptionsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_dataCategories = obj.dataCategories;
        const path_dataCategories = path + '.dataCategories';
        if (!ArrayIsArray(obj_dataCategories)) {
            return new TypeError('Expected "array" but received "' + typeof obj_dataCategories + '" (at "' + path_dataCategories + '")');
        }
        for (let i = 0; i < obj_dataCategories.length; i++) {
            const obj_dataCategories_item = obj_dataCategories[i];
            const path_dataCategories_item = path_dataCategories + '[' + i + ']';
            const referencepath_dataCategories_itemValidationError = validate$r(obj_dataCategories_item, path_dataCategories_item);
            if (referencepath_dataCategories_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchDataCategoryInputRepresentation (at "' + path_dataCategories_item + '")\n';
                message += referencepath_dataCategories_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_filters = obj.filters;
        const path_filters = path + '.filters';
        if (!ArrayIsArray(obj_filters)) {
            return new TypeError('Expected "array" but received "' + typeof obj_filters + '" (at "' + path_filters + '")');
        }
        for (let i = 0; i < obj_filters.length; i++) {
            const obj_filters_item = obj_filters[i];
            const path_filters_item = path_filters + '[' + i + ']';
            const referencepath_filters_itemValidationError = validate$q(obj_filters_item, path_filters_item);
            if (referencepath_filters_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchFilterInputRepresentation (at "' + path_filters_item + '")\n';
                message += referencepath_filters_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$o(obj, path = 'ErrorMessageRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_errorCode = obj.errorCode;
        const path_errorCode = path + '.errorCode';
        if (typeof obj_errorCode !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_errorCode + '" (at "' + path_errorCode + '")');
        }
        const obj_message = obj.message;
        const path_message = path + '.message';
        let obj_message_union0 = null;
        const obj_message_union0_error = (() => {
            if (typeof obj_message !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_message + '" (at "' + path_message + '")');
            }
        })();
        if (obj_message_union0_error != null) {
            obj_message_union0 = obj_message_union0_error.message;
        }
        let obj_message_union1 = null;
        const obj_message_union1_error = (() => {
            if (obj_message !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_message + '" (at "' + path_message + '")');
            }
        })();
        if (obj_message_union1_error != null) {
            obj_message_union1 = obj_message_union1_error.message;
        }
        if (obj_message_union0 && obj_message_union1) {
            let message = 'Object doesn\'t match union (at "' + path_message + '")';
            message += '\n' + obj_message_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_message_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$n(obj, path = 'SearchResultCollectionRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_currentPageToken = obj.currentPageToken;
        const path_currentPageToken = path + '.currentPageToken';
        if (typeof obj_currentPageToken !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_currentPageToken + '" (at "' + path_currentPageToken + '")');
        }
        const obj_error = obj.error;
        const path_error = path + '.error';
        let obj_error_union0 = null;
        const obj_error_union0_error = (() => {
            const referencepath_errorValidationError = validate$o(obj_error, path_error);
            if (referencepath_errorValidationError !== null) {
                let message = 'Object doesn\'t match ErrorMessageRepresentation (at "' + path_error + '")\n';
                message += referencepath_errorValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_error_union0_error != null) {
            obj_error_union0 = obj_error_union0_error.message;
        }
        let obj_error_union1 = null;
        const obj_error_union1_error = (() => {
            if (obj_error !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_error + '" (at "' + path_error + '")');
            }
        })();
        if (obj_error_union1_error != null) {
            obj_error_union1 = obj_error_union1_error.message;
        }
        if (obj_error_union0 && obj_error_union1) {
            let message = 'Object doesn\'t match union (at "' + path_error + '")';
            message += '\n' + obj_error_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_error_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nextPageToken = obj.nextPageToken;
        const path_nextPageToken = path + '.nextPageToken';
        let obj_nextPageToken_union0 = null;
        const obj_nextPageToken_union0_error = (() => {
            if (typeof obj_nextPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union0_error != null) {
            obj_nextPageToken_union0 = obj_nextPageToken_union0_error.message;
        }
        let obj_nextPageToken_union1 = null;
        const obj_nextPageToken_union1_error = (() => {
            if (obj_nextPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nextPageToken + '" (at "' + path_nextPageToken + '")');
            }
        })();
        if (obj_nextPageToken_union1_error != null) {
            obj_nextPageToken_union1 = obj_nextPageToken_union1_error.message;
        }
        if (obj_nextPageToken_union0 && obj_nextPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nextPageToken + '")';
            message += '\n' + obj_nextPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nextPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_orderBy = obj.orderBy;
        const path_orderBy = path + '.orderBy';
        if (!ArrayIsArray(obj_orderBy)) {
            return new TypeError('Expected "array" but received "' + typeof obj_orderBy + '" (at "' + path_orderBy + '")');
        }
        for (let i = 0; i < obj_orderBy.length; i++) {
            const obj_orderBy_item = obj_orderBy[i];
            const path_orderBy_item = path_orderBy + '[' + i + ']';
            const referencepath_orderBy_itemValidationError = validate$2a(obj_orderBy_item, path_orderBy_item);
            if (referencepath_orderBy_itemValidationError !== null) {
                let message = 'Object doesn\'t match ListOrderByInfoRepresentation (at "' + path_orderBy_item + '")\n';
                message += referencepath_orderBy_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_pageSize = obj.pageSize;
        const path_pageSize = path + '.pageSize';
        if (typeof obj_pageSize !== 'number' || (typeof obj_pageSize === 'number' && Math.floor(obj_pageSize) !== obj_pageSize)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_pageSize + '" (at "' + path_pageSize + '")');
        }
        const obj_previousPageToken = obj.previousPageToken;
        const path_previousPageToken = path + '.previousPageToken';
        let obj_previousPageToken_union0 = null;
        const obj_previousPageToken_union0_error = (() => {
            if (typeof obj_previousPageToken !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union0_error != null) {
            obj_previousPageToken_union0 = obj_previousPageToken_union0_error.message;
        }
        let obj_previousPageToken_union1 = null;
        const obj_previousPageToken_union1_error = (() => {
            if (obj_previousPageToken !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_previousPageToken + '" (at "' + path_previousPageToken + '")');
            }
        })();
        if (obj_previousPageToken_union1_error != null) {
            obj_previousPageToken_union1 = obj_previousPageToken_union1_error.message;
        }
        if (obj_previousPageToken_union0 && obj_previousPageToken_union1) {
            let message = 'Object doesn\'t match union (at "' + path_previousPageToken + '")';
            message += '\n' + obj_previousPageToken_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_previousPageToken_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (!ArrayIsArray(obj_records)) {
            return new TypeError('Expected "array" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
        for (let i = 0; i < obj_records.length; i++) {
            const obj_records_item = obj_records[i];
            const path_records_item = path_records + '[' + i + ']';
            if (typeof obj_records_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_records_item + '" (at "' + path_records_item + '")');
            }
        }
        const obj_relatedObjectApiNames = obj.relatedObjectApiNames;
        const path_relatedObjectApiNames = path + '.relatedObjectApiNames';
        if (!ArrayIsArray(obj_relatedObjectApiNames)) {
            return new TypeError('Expected "array" but received "' + typeof obj_relatedObjectApiNames + '" (at "' + path_relatedObjectApiNames + '")');
        }
        for (let i = 0; i < obj_relatedObjectApiNames.length; i++) {
            const obj_relatedObjectApiNames_item = obj_relatedObjectApiNames[i];
            const path_relatedObjectApiNames_item = path_relatedObjectApiNames + '[' + i + ']';
            if (typeof obj_relatedObjectApiNames_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_relatedObjectApiNames_item + '" (at "' + path_relatedObjectApiNames_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$m(obj, path = 'KeywordSearchResultsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_results = obj.results;
        const path_results = path + '.results';
        if (!ArrayIsArray(obj_results)) {
            return new TypeError('Expected "array" but received "' + typeof obj_results + '" (at "' + path_results + '")');
        }
        for (let i = 0; i < obj_results.length; i++) {
            const obj_results_item = obj_results[i];
            const path_results_item = path_results + '[' + i + ']';
            if (typeof obj_results_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_results_item + '" (at "' + path_results_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$l(obj, path = 'NLSResultsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_hasNoResult = obj.hasNoResult;
        const path_hasNoResult = path + '.hasNoResult';
        let obj_hasNoResult_union0 = null;
        const obj_hasNoResult_union0_error = (() => {
            if (typeof obj_hasNoResult !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_hasNoResult + '" (at "' + path_hasNoResult + '")');
            }
        })();
        if (obj_hasNoResult_union0_error != null) {
            obj_hasNoResult_union0 = obj_hasNoResult_union0_error.message;
        }
        let obj_hasNoResult_union1 = null;
        const obj_hasNoResult_union1_error = (() => {
            if (obj_hasNoResult !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_hasNoResult + '" (at "' + path_hasNoResult + '")');
            }
        })();
        if (obj_hasNoResult_union1_error != null) {
            obj_hasNoResult_union1 = obj_hasNoResult_union1_error.message;
        }
        if (obj_hasNoResult_union0 && obj_hasNoResult_union1) {
            let message = 'Object doesn\'t match union (at "' + path_hasNoResult + '")';
            message += '\n' + obj_hasNoResult_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_hasNoResult_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_results = obj.results;
        const path_results = path + '.results';
        if (typeof obj_results !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_results + '" (at "' + path_results + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$k(obj, path = 'AppliedSearchFilterOutputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldPath = obj.fieldPath;
        const path_fieldPath = path + '.fieldPath';
        if (typeof obj_fieldPath !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldPath + '" (at "' + path_fieldPath + '")');
        }
        const obj_label = obj.label;
        const path_label = path + '.label';
        if (typeof obj_label !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_label + '" (at "' + path_label + '")');
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
        const obj_values = obj.values;
        const path_values = path + '.values';
        if (!ArrayIsArray(obj_values)) {
            return new TypeError('Expected "array" but received "' + typeof obj_values + '" (at "' + path_values + '")');
        }
        for (let i = 0; i < obj_values.length; i++) {
            const obj_values_item = obj_values[i];
            const path_values_item = path_values + '[' + i + ']';
            if (typeof obj_values_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$j(obj, path = 'SearchAnswersRecordRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_objectApiLabel = obj.objectApiLabel;
        const path_objectApiLabel = path + '.objectApiLabel';
        if (typeof obj_objectApiLabel !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiLabel + '" (at "' + path_objectApiLabel + '")');
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_recordId = obj.recordId;
        const path_recordId = path + '.recordId';
        if (typeof obj_recordId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_recordId + '" (at "' + path_recordId + '")');
        }
        const obj_title = obj.title;
        const path_title = path + '.title';
        if (typeof obj_title !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_title + '" (at "' + path_title + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$i(obj, path = 'SearchAnswersResultRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_answerType = obj.answerType;
        const path_answerType = path + '.answerType';
        let obj_answerType_union0 = null;
        const obj_answerType_union0_error = (() => {
            if (typeof obj_answerType !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_answerType + '" (at "' + path_answerType + '")');
            }
        })();
        if (obj_answerType_union0_error != null) {
            obj_answerType_union0 = obj_answerType_union0_error.message;
        }
        let obj_answerType_union1 = null;
        const obj_answerType_union1_error = (() => {
            if (obj_answerType !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_answerType + '" (at "' + path_answerType + '")');
            }
        })();
        if (obj_answerType_union1_error != null) {
            obj_answerType_union1 = obj_answerType_union1_error.message;
        }
        if (obj_answerType_union0 && obj_answerType_union1) {
            let message = 'Object doesn\'t match union (at "' + path_answerType + '")';
            message += '\n' + obj_answerType_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_answerType_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_appliedFilters = obj.appliedFilters;
        const path_appliedFilters = path + '.appliedFilters';
        if (!ArrayIsArray(obj_appliedFilters)) {
            return new TypeError('Expected "array" but received "' + typeof obj_appliedFilters + '" (at "' + path_appliedFilters + '")');
        }
        for (let i = 0; i < obj_appliedFilters.length; i++) {
            const obj_appliedFilters_item = obj_appliedFilters[i];
            const path_appliedFilters_item = path_appliedFilters + '[' + i + ']';
            const referencepath_appliedFilters_itemValidationError = validate$k(obj_appliedFilters_item, path_appliedFilters_item);
            if (referencepath_appliedFilters_itemValidationError !== null) {
                let message = 'Object doesn\'t match AppliedSearchFilterOutputRepresentation (at "' + path_appliedFilters_item + '")\n';
                message += referencepath_appliedFilters_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_llmGenerationId = obj.llmGenerationId;
        const path_llmGenerationId = path + '.llmGenerationId';
        let obj_llmGenerationId_union0 = null;
        const obj_llmGenerationId_union0_error = (() => {
            if (typeof obj_llmGenerationId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_llmGenerationId + '" (at "' + path_llmGenerationId + '")');
            }
        })();
        if (obj_llmGenerationId_union0_error != null) {
            obj_llmGenerationId_union0 = obj_llmGenerationId_union0_error.message;
        }
        let obj_llmGenerationId_union1 = null;
        const obj_llmGenerationId_union1_error = (() => {
            if (obj_llmGenerationId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_llmGenerationId + '" (at "' + path_llmGenerationId + '")');
            }
        })();
        if (obj_llmGenerationId_union1_error != null) {
            obj_llmGenerationId_union1 = obj_llmGenerationId_union1_error.message;
        }
        if (obj_llmGenerationId_union0 && obj_llmGenerationId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_llmGenerationId + '")';
            message += '\n' + obj_llmGenerationId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_llmGenerationId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_passage = obj.passage;
        const path_passage = path + '.passage';
        let obj_passage_union0 = null;
        const obj_passage_union0_error = (() => {
            if (typeof obj_passage !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_passage + '" (at "' + path_passage + '")');
            }
        })();
        if (obj_passage_union0_error != null) {
            obj_passage_union0 = obj_passage_union0_error.message;
        }
        let obj_passage_union1 = null;
        const obj_passage_union1_error = (() => {
            if (obj_passage !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_passage + '" (at "' + path_passage + '")');
            }
        })();
        if (obj_passage_union1_error != null) {
            obj_passage_union1 = obj_passage_union1_error.message;
        }
        if (obj_passage_union0 && obj_passage_union1) {
            let message = 'Object doesn\'t match union (at "' + path_passage + '")';
            message += '\n' + obj_passage_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_passage_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_records = obj.records;
        const path_records = path + '.records';
        if (!ArrayIsArray(obj_records)) {
            return new TypeError('Expected "array" but received "' + typeof obj_records + '" (at "' + path_records + '")');
        }
        for (let i = 0; i < obj_records.length; i++) {
            const obj_records_item = obj_records[i];
            const path_records_item = path_records + '[' + i + ']';
            const referencepath_records_itemValidationError = validate$j(obj_records_item, path_records_item);
            if (referencepath_records_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchAnswersRecordRepresentation (at "' + path_records_item + '")\n';
                message += referencepath_records_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$h(obj, path = 'SearchDataCategoryOutputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_groupName = obj.groupName;
        const path_groupName = path + '.groupName';
        if (typeof obj_groupName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_groupName + '" (at "' + path_groupName + '")');
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
        const obj_values = obj.values;
        const path_values = path + '.values';
        if (!ArrayIsArray(obj_values)) {
            return new TypeError('Expected "array" but received "' + typeof obj_values + '" (at "' + path_values + '")');
        }
        for (let i = 0; i < obj_values.length; i++) {
            const obj_values_item = obj_values[i];
            const path_values_item = path_values + '[' + i + ']';
            if (typeof obj_values_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$g(obj, path = 'SearchFilterOutputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_fieldPath = obj.fieldPath;
        const path_fieldPath = path + '.fieldPath';
        if (typeof obj_fieldPath !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_fieldPath + '" (at "' + path_fieldPath + '")');
        }
        const obj_operator = obj.operator;
        const path_operator = path + '.operator';
        if (typeof obj_operator !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_operator + '" (at "' + path_operator + '")');
        }
        const obj_values = obj.values;
        const path_values = path + '.values';
        if (!ArrayIsArray(obj_values)) {
            return new TypeError('Expected "array" but received "' + typeof obj_values + '" (at "' + path_values + '")');
        }
        for (let i = 0; i < obj_values.length; i++) {
            const obj_values_item = obj_values[i];
            const path_values_item = path_values + '[' + i + ']';
            if (typeof obj_values_item !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_values_item + '" (at "' + path_values_item + '")');
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

function validate$f(obj, path = 'SearchObjectOptionsOutputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_dataCategories = obj.dataCategories;
        const path_dataCategories = path + '.dataCategories';
        if (!ArrayIsArray(obj_dataCategories)) {
            return new TypeError('Expected "array" but received "' + typeof obj_dataCategories + '" (at "' + path_dataCategories + '")');
        }
        for (let i = 0; i < obj_dataCategories.length; i++) {
            const obj_dataCategories_item = obj_dataCategories[i];
            const path_dataCategories_item = path_dataCategories + '[' + i + ']';
            const referencepath_dataCategories_itemValidationError = validate$h(obj_dataCategories_item, path_dataCategories_item);
            if (referencepath_dataCategories_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchDataCategoryOutputRepresentation (at "' + path_dataCategories_item + '")\n';
                message += referencepath_dataCategories_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
        const obj_filters = obj.filters;
        const path_filters = path + '.filters';
        if (!ArrayIsArray(obj_filters)) {
            return new TypeError('Expected "array" but received "' + typeof obj_filters + '" (at "' + path_filters + '")');
        }
        for (let i = 0; i < obj_filters.length; i++) {
            const obj_filters_item = obj_filters[i];
            const path_filters_item = path_filters + '[' + i + ']';
            const referencepath_filters_itemValidationError = validate$g(obj_filters_item, path_filters_item);
            if (referencepath_filters_itemValidationError !== null) {
                let message = 'Object doesn\'t match SearchFilterOutputRepresentation (at "' + path_filters_item + '")\n';
                message += referencepath_filters_itemValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

const TTL$c = 200;
const VERSION$1C = "94ab97aedd252f899c90d52d898e547f";
function validate$e(obj, path = 'SearchResultsSummaryRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_configurationName = obj.configurationName;
        const path_configurationName = path + '.configurationName';
        let obj_configurationName_union0 = null;
        const obj_configurationName_union0_error = (() => {
            if (typeof obj_configurationName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_configurationName + '" (at "' + path_configurationName + '")');
            }
        })();
        if (obj_configurationName_union0_error != null) {
            obj_configurationName_union0 = obj_configurationName_union0_error.message;
        }
        let obj_configurationName_union1 = null;
        const obj_configurationName_union1_error = (() => {
            if (obj_configurationName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_configurationName + '" (at "' + path_configurationName + '")');
            }
        })();
        if (obj_configurationName_union1_error != null) {
            obj_configurationName_union1 = obj_configurationName_union1_error.message;
        }
        if (obj_configurationName_union0 && obj_configurationName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_configurationName + '")';
            message += '\n' + obj_configurationName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_configurationName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_keywordSearchResults = obj.keywordSearchResults;
        const path_keywordSearchResults = path + '.keywordSearchResults';
        let obj_keywordSearchResults_union0 = null;
        const obj_keywordSearchResults_union0_error = (() => {
            const referencepath_keywordSearchResultsValidationError = validate$m(obj_keywordSearchResults, path_keywordSearchResults);
            if (referencepath_keywordSearchResultsValidationError !== null) {
                let message = 'Object doesn\'t match KeywordSearchResultsRepresentation (at "' + path_keywordSearchResults + '")\n';
                message += referencepath_keywordSearchResultsValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_keywordSearchResults_union0_error != null) {
            obj_keywordSearchResults_union0 = obj_keywordSearchResults_union0_error.message;
        }
        let obj_keywordSearchResults_union1 = null;
        const obj_keywordSearchResults_union1_error = (() => {
            if (obj_keywordSearchResults !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_keywordSearchResults + '" (at "' + path_keywordSearchResults + '")');
            }
        })();
        if (obj_keywordSearchResults_union1_error != null) {
            obj_keywordSearchResults_union1 = obj_keywordSearchResults_union1_error.message;
        }
        if (obj_keywordSearchResults_union0 && obj_keywordSearchResults_union1) {
            let message = 'Object doesn\'t match union (at "' + path_keywordSearchResults + '")';
            message += '\n' + obj_keywordSearchResults_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_keywordSearchResults_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_nlsResults = obj.nlsResults;
        const path_nlsResults = path + '.nlsResults';
        let obj_nlsResults_union0 = null;
        const obj_nlsResults_union0_error = (() => {
            const referencepath_nlsResultsValidationError = validate$l(obj_nlsResults, path_nlsResults);
            if (referencepath_nlsResultsValidationError !== null) {
                let message = 'Object doesn\'t match NLSResultsRepresentation (at "' + path_nlsResults + '")\n';
                message += referencepath_nlsResultsValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_nlsResults_union0_error != null) {
            obj_nlsResults_union0 = obj_nlsResults_union0_error.message;
        }
        let obj_nlsResults_union1 = null;
        const obj_nlsResults_union1_error = (() => {
            if (obj_nlsResults !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_nlsResults + '" (at "' + path_nlsResults + '")');
            }
        })();
        if (obj_nlsResults_union1_error != null) {
            obj_nlsResults_union1 = obj_nlsResults_union1_error.message;
        }
        if (obj_nlsResults_union0 && obj_nlsResults_union1) {
            let message = 'Object doesn\'t match union (at "' + path_nlsResults + '")';
            message += '\n' + obj_nlsResults_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_nlsResults_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_qnaResult = obj.qnaResult;
        const path_qnaResult = path + '.qnaResult';
        let obj_qnaResult_union0 = null;
        const obj_qnaResult_union0_error = (() => {
            const referencepath_qnaResultValidationError = validate$i(obj_qnaResult, path_qnaResult);
            if (referencepath_qnaResultValidationError !== null) {
                let message = 'Object doesn\'t match SearchAnswersResultRepresentation (at "' + path_qnaResult + '")\n';
                message += referencepath_qnaResultValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_qnaResult_union0_error != null) {
            obj_qnaResult_union0 = obj_qnaResult_union0_error.message;
        }
        let obj_qnaResult_union1 = null;
        const obj_qnaResult_union1_error = (() => {
            if (obj_qnaResult !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_qnaResult + '" (at "' + path_qnaResult + '")');
            }
        })();
        if (obj_qnaResult_union1_error != null) {
            obj_qnaResult_union1 = obj_qnaResult_union1_error.message;
        }
        if (obj_qnaResult_union0 && obj_qnaResult_union1) {
            let message = 'Object doesn\'t match union (at "' + path_qnaResult + '")';
            message += '\n' + obj_qnaResult_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_qnaResult_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_query = obj.query;
        const path_query = path + '.query';
        if (typeof obj_query !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_query + '" (at "' + path_query + '")');
        }
        const obj_queryId = obj.queryId;
        const path_queryId = path + '.queryId';
        if (typeof obj_queryId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_queryId + '" (at "' + path_queryId + '")');
        }
        const obj_searchObjectOptions = obj.searchObjectOptions;
        const path_searchObjectOptions = path + '.searchObjectOptions';
        if (typeof obj_searchObjectOptions !== 'object' || ArrayIsArray(obj_searchObjectOptions) || obj_searchObjectOptions === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_searchObjectOptions + '" (at "' + path_searchObjectOptions + '")');
        }
        const obj_searchObjectOptions_keys = ObjectKeys(obj_searchObjectOptions);
        for (let i = 0; i < obj_searchObjectOptions_keys.length; i++) {
            const key = obj_searchObjectOptions_keys[i];
            const obj_searchObjectOptions_prop = obj_searchObjectOptions[key];
            const path_searchObjectOptions_prop = path_searchObjectOptions + '["' + key + '"]';
            const referencepath_searchObjectOptions_propValidationError = validate$f(obj_searchObjectOptions_prop, path_searchObjectOptions_prop);
            if (referencepath_searchObjectOptions_propValidationError !== null) {
                let message = 'Object doesn\'t match SearchObjectOptionsOutputRepresentation (at "' + path_searchObjectOptions_prop + '")\n';
                message += referencepath_searchObjectOptions_propValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$e = 'SearchResultsSummaryRepresentation';
function keyBuilder$1E(luvio, config) {
    return keyPrefix + '::' + RepresentationType$e + ':' + config.query + ':' + (config.configurationName === null ? '' : config.configurationName);
}
function keyBuilderFromType$6(luvio, object) {
    const keyParams = {
        query: object.query,
        configurationName: object.configurationName
    };
    return keyBuilder$1E(luvio, keyParams);
}
function normalize$8(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1B = function SearchResultsSummaryRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1C,
        private: [],
        opaque: true
    };
};
function equals$c(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1y = function SearchResultsSummaryRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$e(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$6(luvio, input);
    const ttlToUse = TTL$c;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$8, "UiApi", VERSION$1C, RepresentationType$e, equals$c);
    return createLink$1(key);
};
function getTypeCacheKeys$1z(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$6(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$e,
        mergeable: false
    });
}

function select$1A(luvio, params) {
    return select$1B();
}
function keyBuilder$1D(luvio, params) {
    return keyBuilder$1E(luvio, {
        query: params.queryParams.q,
        configurationName: params.body.configurationName || null
    });
}
function getResponseCacheKeys$e(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1z(storeKeyMap, luvio, response);
}
function ingestSuccess$8(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1D(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1y, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1A(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$4(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1D(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$c,
        namespace: keyPrefix,
        version: VERSION$1C,
        representationName: RepresentationType$e
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$j(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/search/results',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$a = 'getSearchResults';
const getSearchResults_ConfigPropertyMetadata = [
    generateParamConfigMetadata('q', true, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('answerTypes', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('configurationName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('objectApiNames', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('searchObjectOptions', false, 2 /* Body */, 4 /* Unsupported */),
];
const getSearchResults_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$a, getSearchResults_ConfigPropertyMetadata);
const createResourceParams$e = /*#__PURE__*/ createResourceParams$k(getSearchResults_ConfigPropertyMetadata);
function keyBuilder$1C(luvio, config) {
    const resourceParams = createResourceParams$e(config);
    return keyBuilder$1D(luvio, resourceParams);
}
function typeCheckConfig$f(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getSearchResults_ConfigPropertyMetadata);
    const untrustedConfig_searchObjectOptions = untrustedConfig.searchObjectOptions;
    if (untrustedIsObject(untrustedConfig_searchObjectOptions)) {
        const untrustedConfig_searchObjectOptions_object = {};
        const untrustedConfig_searchObjectOptions_keys = Object.keys(untrustedConfig_searchObjectOptions);
        for (let i = 0, arrayLength = untrustedConfig_searchObjectOptions_keys.length; i < arrayLength; i++) {
            const key = untrustedConfig_searchObjectOptions_keys[i];
            const untrustedConfig_searchObjectOptions_prop = untrustedConfig_searchObjectOptions[key];
            const referenceSearchObjectOptionsRepresentationValidationError = validate$p(untrustedConfig_searchObjectOptions_prop);
            if (referenceSearchObjectOptionsRepresentationValidationError === null) {
                if (untrustedConfig_searchObjectOptions_object !== undefined) {
                    untrustedConfig_searchObjectOptions_object[key] = untrustedConfig_searchObjectOptions_prop;
                }
            }
        }
        if (untrustedConfig_searchObjectOptions_object !== undefined && Object.keys(untrustedConfig_searchObjectOptions_object).length >= 0) {
            config.searchObjectOptions = untrustedConfig_searchObjectOptions_object;
        }
    }
    return config;
}
function validateAdapterConfig$f(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$f(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$5(luvio, config) {
    createResourceParams$e(config);
    return select$1A();
}
function onFetchResponseSuccess$6(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$8(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$g(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$5(luvio, config, resourceParams, response) {
    const snapshot = ingestError$4(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$g(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$g(luvio, config, options) {
    const resourceParams = createResourceParams$e(config);
    const request = createResourceRequest$j(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$6(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$e(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$5(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$8(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$g, 'get', false);
}
function buildCachedSnapshotCachePolicy$7(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1C(luvio, config),
        node: adapterFragment$5(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$g(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getSearchResultsAdapterFactory = (luvio) => function UiApi__getSearchResults(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$f(untrustedConfig, getSearchResults_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$7, buildNetworkSnapshotCachePolicy$8);
};

const TTL$b = 200;
const VERSION$1B = "3102453bf10ea449d9665914d5f5febf";
function validate$d(obj, path = 'KeywordSearchResultsSummaryRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_configurationName = obj.configurationName;
        const path_configurationName = path + '.configurationName';
        let obj_configurationName_union0 = null;
        const obj_configurationName_union0_error = (() => {
            if (typeof obj_configurationName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_configurationName + '" (at "' + path_configurationName + '")');
            }
        })();
        if (obj_configurationName_union0_error != null) {
            obj_configurationName_union0 = obj_configurationName_union0_error.message;
        }
        let obj_configurationName_union1 = null;
        const obj_configurationName_union1_error = (() => {
            if (obj_configurationName !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_configurationName + '" (at "' + path_configurationName + '")');
            }
        })();
        if (obj_configurationName_union1_error != null) {
            obj_configurationName_union1 = obj_configurationName_union1_error.message;
        }
        if (obj_configurationName_union0 && obj_configurationName_union1) {
            let message = 'Object doesn\'t match union (at "' + path_configurationName + '")';
            message += '\n' + obj_configurationName_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_configurationName_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_keywordSearchResult = obj.keywordSearchResult;
        const path_keywordSearchResult = path + '.keywordSearchResult';
        const referencepath_keywordSearchResultValidationError = validate$n(obj_keywordSearchResult, path_keywordSearchResult);
        if (referencepath_keywordSearchResultValidationError !== null) {
            let message = 'Object doesn\'t match SearchResultCollectionRepresentation (at "' + path_keywordSearchResult + '")\n';
            message += referencepath_keywordSearchResultValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectApiName = obj.objectApiName;
        const path_objectApiName = path + '.objectApiName';
        if (typeof obj_objectApiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_objectApiName + '" (at "' + path_objectApiName + '")');
        }
        const obj_query = obj.query;
        const path_query = path + '.query';
        if (typeof obj_query !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_query + '" (at "' + path_query + '")');
        }
        const obj_queryId = obj.queryId;
        const path_queryId = path + '.queryId';
        if (typeof obj_queryId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_queryId + '" (at "' + path_queryId + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$d = 'KeywordSearchResultsSummaryRepresentation';
function keyBuilder$1B(luvio, config) {
    return keyPrefix + '::' + RepresentationType$d + ':' + config.query + ':' + config.objectApiName + ':' + (config.configurationName === null ? '' : config.configurationName);
}
function keyBuilderFromType$5(luvio, object) {
    const keyParams = {
        query: object.query,
        objectApiName: object.objectApiName,
        configurationName: object.configurationName
    };
    return keyBuilder$1B(luvio, keyParams);
}
function normalize$7(input, existing, path, luvio, store, timestamp) {
    return input;
}
const select$1z = function KeywordSearchResultsSummaryRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$1B,
        private: [],
        opaque: true
    };
};
function equals$b(existing, incoming) {
    if (JSONStringify(incoming) !== JSONStringify(existing)) {
        return false;
    }
    return true;
}
const ingest$1x = function KeywordSearchResultsSummaryRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$d(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType$5(luvio, input);
    const ttlToUse = TTL$b;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$7, "UiApi", VERSION$1B, RepresentationType$d, equals$b);
    return createLink$1(key);
};
function getTypeCacheKeys$1y(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$5(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$d,
        mergeable: false
    });
}

function select$1y(luvio, params) {
    return select$1z();
}
function keyBuilder$1A(luvio, params) {
    return keyBuilder$1B(luvio, {
        query: params.queryParams.q,
        objectApiName: params.queryParams.objectApiName,
        configurationName: params.body.configurationName || null
    });
}
function getResponseCacheKeys$d(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1y(storeKeyMap, luvio, response);
}
function ingestSuccess$7(luvio, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$1A(luvio, resourceParams);
    luvio.storeIngest(key, ingest$1x, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$1y(),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function ingestError$3(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$1A(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$b,
        namespace: keyPrefix,
        version: VERSION$1B,
        representationName: RepresentationType$d
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$i(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/search/results/keyword',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$9 = 'getKeywordSearchResults';
const getKeywordSearchResults_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('q', true, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('configurationName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('dataCategories', false, 2 /* Body */, 4 /* Unsupported */, true),
    generateParamConfigMetadata('filters', false, 2 /* Body */, 4 /* Unsupported */, true),
    generateParamConfigMetadata('pageSize', false, 2 /* Body */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('sortBy', false, 2 /* Body */, 0 /* String */),
];
const getKeywordSearchResults_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$9, getKeywordSearchResults_ConfigPropertyMetadata);
const createResourceParams$d = /*#__PURE__*/ createResourceParams$k(getKeywordSearchResults_ConfigPropertyMetadata);
function keyBuilder$1z(luvio, config) {
    const resourceParams = createResourceParams$d(config);
    return keyBuilder$1A(luvio, resourceParams);
}
function typeCheckConfig$e(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getKeywordSearchResults_ConfigPropertyMetadata);
    const untrustedConfig_dataCategories = untrustedConfig.dataCategories;
    if (ArrayIsArray$1(untrustedConfig_dataCategories)) {
        const untrustedConfig_dataCategories_array = [];
        for (let i = 0, arrayLength = untrustedConfig_dataCategories.length; i < arrayLength; i++) {
            const untrustedConfig_dataCategories_item = untrustedConfig_dataCategories[i];
            const referenceSearchDataCategoryInputRepresentationValidationError = validate$r(untrustedConfig_dataCategories_item);
            if (referenceSearchDataCategoryInputRepresentationValidationError === null) {
                untrustedConfig_dataCategories_array.push(untrustedConfig_dataCategories_item);
            }
        }
        config.dataCategories = untrustedConfig_dataCategories_array;
    }
    const untrustedConfig_filters = untrustedConfig.filters;
    if (ArrayIsArray$1(untrustedConfig_filters)) {
        const untrustedConfig_filters_array = [];
        for (let i = 0, arrayLength = untrustedConfig_filters.length; i < arrayLength; i++) {
            const untrustedConfig_filters_item = untrustedConfig_filters[i];
            const referenceSearchFilterInputRepresentationValidationError = validate$q(untrustedConfig_filters_item);
            if (referenceSearchFilterInputRepresentationValidationError === null) {
                untrustedConfig_filters_array.push(untrustedConfig_filters_item);
            }
        }
        config.filters = untrustedConfig_filters_array;
    }
    return config;
}
function validateAdapterConfig$e(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$e(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$4(luvio, config) {
    createResourceParams$d(config);
    return select$1y();
}
function onFetchResponseSuccess$5(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$7(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$f(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$4(luvio, config, resourceParams, response) {
    const snapshot = ingestError$3(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$f(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$f(luvio, config, options) {
    const resourceParams = createResourceParams$d(config);
    const request = createResourceRequest$i(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$5(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$d(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$4(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$7(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$f, 'get', false);
}
function buildCachedSnapshotCachePolicy$6(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$1z(luvio, config),
        node: adapterFragment$4(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$f(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}
const getKeywordSearchResultsAdapterFactory = (luvio) => function UiApi__getKeywordSearchResults(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$e(untrustedConfig, getKeywordSearchResults_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    return luvio.applyCachePolicy((requestContext || {}), { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$6, buildNetworkSnapshotCachePolicy$7);
};

const name$1k = 'DoubleValue';
const VERSION$1A = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1y(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1k(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1w(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1y(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1l,
        isMappedFieldImport: isMappedField$1k,
        getFieldType: getFieldType$1C,
        ingestFieldByType: ingestFieldByType$1k,
        createLink: createLink,
        mergeData: mergeData$1k,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DoubleValue",
            version: VERSION$1A,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1k(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1C(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1l(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Double': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1l(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1k, getFieldType$1C, selectType$1l, parentRecordId);
}
function buildSelectionForNode$1l(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1l, buildSelectionForField$1l, parentRecordId);
}
function select$1x(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1l, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1x(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1y(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1k
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1C(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Double'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1k(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Double': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1k(field) {
    return false;
}
function isFragmentApplicable$1l(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "DoubleValue", typename);
}
function getInContextFragmentType$1C(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1j = 'LongValue';
const VERSION$1z = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1x(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1j(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1v(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1x(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1k,
        isMappedFieldImport: isMappedField$1j,
        getFieldType: getFieldType$1B,
        ingestFieldByType: ingestFieldByType$1j,
        createLink: createLink,
        mergeData: mergeData$1j,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LongValue",
            version: VERSION$1z,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1j(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1B(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1k(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Long': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1k(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1j, getFieldType$1B, selectType$1k, parentRecordId);
}
function buildSelectionForNode$1k(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1k, buildSelectionForField$1k, parentRecordId);
}
function select$1w(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1k, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1w(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1x(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1j
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1B(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1j(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Long': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1j(field) {
    return false;
}
function isFragmentApplicable$1k(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LongValue", typename);
}
function getInContextFragmentType$1B(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1i = 'PercentValue';
const VERSION$1y = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1w(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1i(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1u(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1w(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1j,
        isMappedFieldImport: isMappedField$1i,
        getFieldType: getFieldType$1A,
        ingestFieldByType: ingestFieldByType$1i,
        createLink: createLink,
        mergeData: mergeData$1i,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PercentValue",
            version: VERSION$1y,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1i(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1A(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1j(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Percent': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1j(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1i, getFieldType$1A, selectType$1j, parentRecordId);
}
function buildSelectionForNode$1j(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1j, buildSelectionForField$1j, parentRecordId);
}
function select$1v(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1j, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1v(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1w(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1i
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1A(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Percent'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1i(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Percent': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1i(field) {
    return false;
}
function isFragmentApplicable$1j(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "PercentValue", typename);
}
function getInContextFragmentType$1A(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1h = 'PercentAggregate';
const VERSION$1x = '93eeee533428dc68b3aa891a971948e4';
function keyBuilder$1v(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1h(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1t(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1v(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1i,
        isMappedFieldImport: isMappedField$1h,
        getFieldType: getFieldType$1z,
        ingestFieldByType: ingestFieldByType$1h,
        createLink: createLink,
        mergeData: mergeData$1h,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PercentAggregate",
            version: VERSION$1x,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1h(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1z(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1i(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Percent': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'PercentValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1y, select$1v, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1i(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1h, getFieldType$1z, selectType$1i, parentRecordId);
}
function buildSelectionForNode$1i(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1i, buildSelectionForField$1i, parentRecordId);
}
function select$1u(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1i, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$1f() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$1u(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1v(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1h
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$1i) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$1z(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'PercentValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1v(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'PercentValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1v(cacheKeySink, requestedField, minState);
                break;
            }
            case 'PercentValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1v(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$1z(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Percent'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'PercentValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'PercentValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'PercentValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1h(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Percent': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'PercentValue': {
            const PercentValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1u(requestedField, PercentValueState);
            break;
        }
    }
}
function isMappedField$1h(field) {
    return false;
}
function isFragmentApplicable$1i(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "PercentAggregate", typename);
}
function getInContextFragmentType$1z(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1g = 'IntValue';
const VERSION$1w = 'b71c94771478332a178ced0d71731d38';
function keyBuilder$1u(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1g(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1s(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1u(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1h,
        isMappedFieldImport: isMappedField$1g,
        getFieldType: getFieldType$1y,
        ingestFieldByType: ingestFieldByType$1g,
        createLink: createLink,
        mergeData: mergeData$1g,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "IntValue",
            version: VERSION$1w,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1g(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1y(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1h(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1h(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1g, getFieldType$1y, selectType$1h, parentRecordId);
}
function buildSelectionForNode$1h(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1h, buildSelectionForField$1h, parentRecordId);
}
function select$1t(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1h, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1t(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1u(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1g
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1y(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1g(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1g(field) {
    return false;
}
function isFragmentApplicable$1h(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "IntValue", typename);
}
function getInContextFragmentType$1y(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1f = 'StringValue';
const VERSION$1v = 'fb807e77bb467eb909318cafa7f3b4e0';
function keyBuilder$1t(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1f(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1r(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1t(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1g,
        isMappedFieldImport: isMappedField$1f,
        getFieldType: getFieldType$1x,
        ingestFieldByType: ingestFieldByType$1f,
        createLink: createLink,
        mergeData: mergeData$1f,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "StringValue",
            version: VERSION$1v,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1f(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1x(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1g(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1g(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1f, getFieldType$1x, selectType$1g, parentRecordId);
}
function buildSelectionForNode$1g(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1g, buildSelectionForField$1g, parentRecordId);
}
function select$1s(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1g, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1s(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1t(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1f
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1x(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1f(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1f(field) {
    return false;
}
function isFragmentApplicable$1g(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "StringValue", typename);
}
function getInContextFragmentType$1x(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1e = 'StringAggregate';
const VERSION$1u = 'cd64595f5647f1e5a246a7ac13aa702e';
function keyBuilder$1s(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1e(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1q(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1s(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1f,
        isMappedFieldImport: isMappedField$1e,
        getFieldType: getFieldType$1w,
        ingestFieldByType: ingestFieldByType$1e,
        createLink: createLink,
        mergeData: mergeData$1e,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "StringAggregate",
            version: VERSION$1u,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1e(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1w(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1f(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'StringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1v, select$1s, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1f(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1e, getFieldType$1w, selectType$1f, parentRecordId);
}
function buildSelectionForNode$1f(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1f, buildSelectionForField$1f, parentRecordId);
}
function select$1r(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1f, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$1e() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$1r(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1s(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1e
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$1f) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$1w(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'StringValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'StringValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$1w(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1e(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'StringValue': {
            const StringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1r(requestedField, StringValueState);
            break;
        }
    }
}
function isMappedField$1e(field) {
    return false;
}
function isFragmentApplicable$1f(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "StringAggregate", typename);
}
function getInContextFragmentType$1w(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1d = 'IDValue';
const VERSION$1t = '8a917704b9367cdf6322663e4a5f015c';
function keyBuilder$1r(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1d(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1p(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1r(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1e,
        isMappedFieldImport: isMappedField$1d,
        getFieldType: getFieldType$1v,
        ingestFieldByType: ingestFieldByType$1d,
        createLink: createLink,
        mergeData: mergeData$1d,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "IDValue",
            version: VERSION$1t,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1d(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1v(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1e(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1e(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1d, getFieldType$1v, selectType$1e, parentRecordId);
}
function buildSelectionForNode$1e(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1e, buildSelectionForField$1e, parentRecordId);
}
function select$1q(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1e, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1q(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1r(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1d
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1v(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1d(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1d(field) {
    return false;
}
function isFragmentApplicable$1e(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "IDValue", typename);
}
function getInContextFragmentType$1v(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1c = 'DateTimeValue';
const VERSION$1s = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1q(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1c(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1o(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1q(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1d,
        isMappedFieldImport: isMappedField$1c,
        getFieldType: getFieldType$1u,
        ingestFieldByType: ingestFieldByType$1c,
        createLink: createLink,
        mergeData: mergeData$1c,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DateTimeValue",
            version: VERSION$1s,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1c(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1u(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1d(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DateTime': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1d(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1c, getFieldType$1u, selectType$1d, parentRecordId);
}
function buildSelectionForNode$1d(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1d, buildSelectionForField$1d, parentRecordId);
}
function select$1p(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1d, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1p(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1q(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1c
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1u(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'DateTime'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1c(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DateTime': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1c(field) {
    return false;
}
function isFragmentApplicable$1d(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "DateTimeValue", typename);
}
function getInContextFragmentType$1u(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1b = 'BooleanValue';
const VERSION$1r = '8b2f0c1b2312044b6c60b3bf506054a2';
function keyBuilder$1p(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1b(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1n(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1p(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1c,
        isMappedFieldImport: isMappedField$1b,
        getFieldType: getFieldType$1t,
        ingestFieldByType: ingestFieldByType$1b,
        createLink: createLink,
        mergeData: mergeData$1b,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "BooleanValue",
            version: VERSION$1r,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1b(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1t(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1c(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1c(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1b, getFieldType$1t, selectType$1c, parentRecordId);
}
function buildSelectionForNode$1c(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1c, buildSelectionForField$1c, parentRecordId);
}
function select$1o(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1c, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1o(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1p(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1b
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1t(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1b(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1b(field) {
    return false;
}
function isFragmentApplicable$1c(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "BooleanValue", typename);
}
function getInContextFragmentType$1t(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1a = 'TimeValue';
const VERSION$1q = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1o(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1a(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1m(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1o(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1b,
        isMappedFieldImport: isMappedField$1a,
        getFieldType: getFieldType$1s,
        ingestFieldByType: ingestFieldByType$1a,
        createLink: createLink,
        mergeData: mergeData$1a,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "TimeValue",
            version: VERSION$1q,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1a(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1s(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1b(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Time': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1b(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1a, getFieldType$1s, selectType$1b, parentRecordId);
}
function buildSelectionForNode$1b(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1b, buildSelectionForField$1b, parentRecordId);
}
function select$1n(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1b, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1n(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1o(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1a
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1s(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Time'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1a(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Time': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$1a(field) {
    return false;
}
function isFragmentApplicable$1b(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "TimeValue", typename);
}
function getInContextFragmentType$1s(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$19 = 'DateValue';
const VERSION$1p = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1n(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$19(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1l(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1n(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1a,
        isMappedFieldImport: isMappedField$19,
        getFieldType: getFieldType$1r,
        ingestFieldByType: ingestFieldByType$19,
        createLink: createLink,
        mergeData: mergeData$19,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DateValue",
            version: VERSION$1p,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$19(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1r(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1a(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Date': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1a(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$19, getFieldType$1r, selectType$1a, parentRecordId);
}
function buildSelectionForNode$1a(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1a, buildSelectionForField$1a, parentRecordId);
}
function select$1m(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1a, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1m(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1n(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$19
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1r(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Date'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$19(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Date': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$19(field) {
    return false;
}
function isFragmentApplicable$1a(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "DateValue", typename);
}
function getInContextFragmentType$1r(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$18 = 'TextAreaValue';
const VERSION$1o = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1m(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$18(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1k(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1m(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$19,
        isMappedFieldImport: isMappedField$18,
        getFieldType: getFieldType$1q,
        ingestFieldByType: ingestFieldByType$18,
        createLink: createLink,
        mergeData: mergeData$18,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "TextAreaValue",
            version: VERSION$1o,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$18(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1q(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$19(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'TextArea': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$19(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$18, getFieldType$1q, selectType$19, parentRecordId);
}
function buildSelectionForNode$19(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$19, buildSelectionForField$19, parentRecordId);
}
function select$1l(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$19, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1l(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1m(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$18
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1q(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'TextArea'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$18(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'TextArea': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$18(field) {
    return false;
}
function isFragmentApplicable$19(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "TextAreaValue", typename);
}
function getInContextFragmentType$1q(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$17 = 'LongTextAreaValue';
const VERSION$1n = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1l(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$17(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1j(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1l(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$18,
        isMappedFieldImport: isMappedField$17,
        getFieldType: getFieldType$1p,
        ingestFieldByType: ingestFieldByType$17,
        createLink: createLink,
        mergeData: mergeData$17,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LongTextAreaValue",
            version: VERSION$1n,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$17(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1p(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$18(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongTextArea': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$18(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$17, getFieldType$1p, selectType$18, parentRecordId);
}
function buildSelectionForNode$18(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$18, buildSelectionForField$18, parentRecordId);
}
function select$1k(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$18, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1k(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1l(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$17
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1p(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'LongTextArea'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$17(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongTextArea': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$17(field) {
    return false;
}
function isFragmentApplicable$18(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LongTextAreaValue", typename);
}
function getInContextFragmentType$1p(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$16 = 'RichTextAreaValue';
const VERSION$1m = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1k(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$16(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1i(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1k(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$17,
        isMappedFieldImport: isMappedField$16,
        getFieldType: getFieldType$1o,
        ingestFieldByType: ingestFieldByType$16,
        createLink: createLink,
        mergeData: mergeData$16,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RichTextAreaValue",
            version: VERSION$1m,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$16(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1o(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$17(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RichTextArea': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$17(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$16, getFieldType$1o, selectType$17, parentRecordId);
}
function buildSelectionForNode$17(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$17, buildSelectionForField$17, parentRecordId);
}
function select$1j(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$17, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1j(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1k(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$16
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1o(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'RichTextArea'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$16(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RichTextArea': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$16(field) {
    return false;
}
function isFragmentApplicable$17(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "RichTextAreaValue", typename);
}
function getInContextFragmentType$1o(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$15 = 'PhoneNumberValue';
const VERSION$1l = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1j(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$15(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1h(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1j(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$16,
        isMappedFieldImport: isMappedField$15,
        getFieldType: getFieldType$1n,
        ingestFieldByType: ingestFieldByType$15,
        createLink: createLink,
        mergeData: mergeData$15,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PhoneNumberValue",
            version: VERSION$1l,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$15(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1n(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$16(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'PhoneNumber': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$16(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$15, getFieldType$1n, selectType$16, parentRecordId);
}
function buildSelectionForNode$16(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$16, buildSelectionForField$16, parentRecordId);
}
function select$1i(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$16, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1i(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1j(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$15
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1n(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'PhoneNumber'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$15(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'PhoneNumber': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$15(field) {
    return false;
}
function isFragmentApplicable$16(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "PhoneNumberValue", typename);
}
function getInContextFragmentType$1n(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$14 = 'EmailValue';
const VERSION$1k = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1i(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$14(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1g(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1i(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$15,
        isMappedFieldImport: isMappedField$14,
        getFieldType: getFieldType$1m,
        ingestFieldByType: ingestFieldByType$14,
        createLink: createLink,
        mergeData: mergeData$14,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "EmailValue",
            version: VERSION$1k,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$14(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1m(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$15(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Email': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$15(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$14, getFieldType$1m, selectType$15, parentRecordId);
}
function buildSelectionForNode$15(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$15, buildSelectionForField$15, parentRecordId);
}
function select$1h(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$15, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1h(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1i(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$14
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1m(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Email'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$14(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Email': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$14(field) {
    return false;
}
function isFragmentApplicable$15(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "EmailValue", typename);
}
function getInContextFragmentType$1m(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$13 = 'UrlValue';
const VERSION$1j = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1h(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$13(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1f(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1h(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$14,
        isMappedFieldImport: isMappedField$13,
        getFieldType: getFieldType$1l,
        ingestFieldByType: ingestFieldByType$13,
        createLink: createLink,
        mergeData: mergeData$13,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "UrlValue",
            version: VERSION$1j,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$13(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1l(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$14(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Url': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$14(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$13, getFieldType$1l, selectType$14, parentRecordId);
}
function buildSelectionForNode$14(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$14, buildSelectionForField$14, parentRecordId);
}
function select$1g(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$14, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1g(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1h(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$13
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1l(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Url'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$13(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Url': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$13(field) {
    return false;
}
function isFragmentApplicable$14(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "UrlValue", typename);
}
function getInContextFragmentType$1l(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$12 = 'EncryptedStringValue';
const VERSION$1i = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1g(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$12(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1e(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1g(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$13,
        isMappedFieldImport: isMappedField$12,
        getFieldType: getFieldType$1k,
        ingestFieldByType: ingestFieldByType$12,
        createLink: createLink,
        mergeData: mergeData$12,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "EncryptedStringValue",
            version: VERSION$1i,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$12(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1k(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$13(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'EncryptedString': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$13(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$12, getFieldType$1k, selectType$13, parentRecordId);
}
function buildSelectionForNode$13(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$13, buildSelectionForField$13, parentRecordId);
}
function select$1f(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$13, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1f(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1g(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$12
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1k(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'EncryptedString'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$12(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'EncryptedString': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$12(field) {
    return false;
}
function isFragmentApplicable$13(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "EncryptedStringValue", typename);
}
function getInContextFragmentType$1k(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$11 = 'CurrencyValue';
const VERSION$1h = 'de69c0962aca45d77a5bc8d9abacfa7d';
function keyBuilder$1f(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$11(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1d(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1f(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$12,
        isMappedFieldImport: isMappedField$11,
        getFieldType: getFieldType$1j,
        ingestFieldByType: ingestFieldByType$11,
        createLink: createLink,
        mergeData: mergeData$11,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "CurrencyValue",
            version: VERSION$1h,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$11(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1j(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$12(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Currency': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$12(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$11, getFieldType$1j, selectType$12, parentRecordId);
}
function buildSelectionForNode$12(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$12, buildSelectionForField$12, parentRecordId);
}
function select$1e(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$12, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1e(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1f(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$11
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1j(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Currency'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$11(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Currency': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$11(field) {
    return false;
}
function isFragmentApplicable$12(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "CurrencyValue", typename);
}
function getInContextFragmentType$1j(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$10 = 'LongitudeValue';
const VERSION$1g = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1e(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$10(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1c(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1e(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$11,
        isMappedFieldImport: isMappedField$10,
        getFieldType: getFieldType$1i,
        ingestFieldByType: ingestFieldByType$10,
        createLink: createLink,
        mergeData: mergeData$10,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LongitudeValue",
            version: VERSION$1g,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$10(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1i(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$11(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Longitude': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$11(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$10, getFieldType$1i, selectType$11, parentRecordId);
}
function buildSelectionForNode$11(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$11, buildSelectionForField$11, parentRecordId);
}
function select$1d(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$11, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1d(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1e(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$10
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1i(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Longitude'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$10(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Longitude': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$10(field) {
    return false;
}
function isFragmentApplicable$11(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LongitudeValue", typename);
}
function getInContextFragmentType$1i(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$$ = 'LatitudeValue';
const VERSION$1f = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1d(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$$(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1b(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1d(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$10,
        isMappedFieldImport: isMappedField$$,
        getFieldType: getFieldType$1h,
        ingestFieldByType: ingestFieldByType$$,
        createLink: createLink,
        mergeData: mergeData$$,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LatitudeValue",
            version: VERSION$1f,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$$(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1h(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$10(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Latitude': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$10(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$$, getFieldType$1h, selectType$10, parentRecordId);
}
function buildSelectionForNode$10(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$10, buildSelectionForField$10, parentRecordId);
}
function select$1c(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$10, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1c(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1d(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$$
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1h(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Latitude'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$$(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Latitude': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$$(field) {
    return false;
}
function isFragmentApplicable$10(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LatitudeValue", typename);
}
function getInContextFragmentType$1h(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$_ = 'PicklistValue';
const VERSION$1e = '38d383da02de0256f51de5000c01e745';
function keyBuilder$1c(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$_(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$1a(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1c(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$$,
        isMappedFieldImport: isMappedField$_,
        getFieldType: getFieldType$1g,
        ingestFieldByType: ingestFieldByType$_,
        createLink: createLink,
        mergeData: mergeData$_,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PicklistValue",
            version: VERSION$1e,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$_(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1g(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$$(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Picklist': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$$(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$_, getFieldType$1g, selectType$$, parentRecordId);
}
function buildSelectionForNode$$(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$$, buildSelectionForField$$, parentRecordId);
}
function select$1b(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$$, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1b(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1c(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$_
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1g(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Picklist'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$_(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Picklist': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$_(field) {
    return false;
}
function isFragmentApplicable$$(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "PicklistValue", typename);
}
function getInContextFragmentType$1g(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$Z = 'MultiPicklistValue';
const VERSION$1d = '38d383da02de0256f51de5000c01e745';
function keyBuilder$1b(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$Z(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$19(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1b(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$_,
        isMappedFieldImport: isMappedField$Z,
        getFieldType: getFieldType$1f,
        ingestFieldByType: ingestFieldByType$Z,
        createLink: createLink,
        mergeData: mergeData$Z,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "MultiPicklistValue",
            version: VERSION$1d,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$Z(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1f(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$_(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'MultiPicklist': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$_(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$Z, getFieldType$1f, selectType$_, parentRecordId);
}
function buildSelectionForNode$_(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$_, buildSelectionForField$_, parentRecordId);
}
function select$1a(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$_, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$1a(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1b(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$Z
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1f(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'MultiPicklist'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$Z(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'MultiPicklist': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$Z(field) {
    return false;
}
function isFragmentApplicable$_(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "MultiPicklistValue", typename);
}
function getInContextFragmentType$1f(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$Y = 'Base64Value';
const VERSION$1c = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$1a(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$Y(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$18(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$1a(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$Z,
        isMappedFieldImport: isMappedField$Y,
        getFieldType: getFieldType$1e,
        ingestFieldByType: ingestFieldByType$Y,
        createLink: createLink,
        mergeData: mergeData$Y,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Base64Value",
            version: VERSION$1c,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$Y(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1e(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$Z(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Base64': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$Z(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$Y, getFieldType$1e, selectType$Z, parentRecordId);
}
function buildSelectionForNode$Z(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$Z, buildSelectionForField$Z, parentRecordId);
}
function select$19(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$Z, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$19(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$1a(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$Y
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1e(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Base64'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$Y(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Base64': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$Y(field) {
    return false;
}
function isFragmentApplicable$Z(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "Base64Value", typename);
}
function getInContextFragmentType$1e(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$X = 'JSONValue';
const VERSION$1b = 'c8376c7f60ac5cfe1c2c1521cd4664fa';
function keyBuilder$19(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$X(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$17(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$19(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$Y,
        isMappedFieldImport: isMappedField$X,
        getFieldType: getFieldType$1d,
        ingestFieldByType: ingestFieldByType$X,
        createLink: createLink,
        mergeData: mergeData$X,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "JSONValue",
            version: VERSION$1b,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$X(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1d(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$Y(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'JSON': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$Y(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$X, getFieldType$1d, selectType$Y, parentRecordId);
}
function buildSelectionForNode$Y(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$Y, buildSelectionForField$Y, parentRecordId);
}
function select$18(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$Y, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$18(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$19(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$X
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$1d(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'JSON'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$X(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'JSON': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$X(field) {
    return false;
}
function isFragmentApplicable$Y(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "JSONValue", typename);
}
function getInContextFragmentType$1d(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

// union versions are undefined, selected versions must be specific to the type being selected
const VERSION$1a = undefined;
function ingest$16(astNode, state) {
    const typename = state.data.__typename;
    // Delegate key building and ingest to concrete implementations
    if (typename === 'RecordRepresentation') {
        return ingest$13(astNode, state);
    }
}
function select$17(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const typename = source.data.__typename;
        if (typename === 'RecordRepresentation') {
            return select$14(field, variables, fragments)(source, reader, parentRecordId);
        }
    };
}
function getTypeCacheKeys$17(cacheKeys, astNode, state) {
    const { path, data } = state;
    const typename = data.__typename;
    if (typename === 'RecordRepresentation') {
        return getTypeCacheKeys$13(cacheKeys, astNode, state);
    }
}
function getInContextFragmentType$1c(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}
function getFieldType$1c(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getMinimumSelections$1d() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}

// Naive impl for pulling out a directive value from a field node. Doesn't handle multiple stacked
// directives or need for multiple argument values.
function getDirectiveArgumentValue(field, directiveName, argumentName) {
    if (field.directives) {
        const targetedDirective = field.directives.find((directiveNode) => directiveNode.name.value === directiveName);
        if (targetedDirective && targetedDirective.arguments) {
            const targetedArgument = targetedDirective.arguments.find((arg) => arg.name.value === argumentName);
            if (targetedArgument) {
                return targetedArgument.value;
            }
        }
    }
}

function getFieldNameOverride(field) {
    const categoryValue = getDirectiveArgumentValue(field, 'category', 'name');
    if (categoryValue) {
        // Only string types are allowed for metaschema category directives.
        if (categoryValue.kind === 'StringValue') {
            return categoryValue.value;
        }
    }
}

function getFieldType$1b(field) {
    const fieldNameOverride = getFieldNameOverride(field);
    const fieldName = fieldNameOverride || field.name.value;
    switch (fieldName) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'IntValue': {
            return {
                isArray: false,
                typename: 'IntValue',
            };
        }
        case 'StringValue': {
            return {
                isArray: false,
                typename: 'StringValue',
            };
        }
        case 'BooleanValue': {
            return {
                isArray: false,
                typename: 'BooleanValue',
            };
        }
        case 'IDValue': {
            return {
                isArray: false,
                typename: 'IDValue',
            };
        }
        case 'DateTimeValue': {
            return {
                isArray: false,
                typename: 'DateTimeValue',
            };
        }
        case 'TimeValue': {
            return {
                isArray: false,
                typename: 'TimeValue',
            };
        }
        case 'DateValue': {
            return {
                isArray: false,
                typename: 'DateValue',
            };
        }
        case 'TextAreaValue': {
            return {
                isArray: false,
                typename: 'TextAreaValue',
            };
        }
        case 'LongTextAreaValue': {
            return {
                isArray: false,
                typename: 'LongTextAreaValue',
            };
        }
        case 'RichTextAreaValue': {
            return {
                isArray: false,
                typename: 'RichTextAreaValue',
            };
        }
        case 'PhoneNumberValue': {
            return {
                isArray: false,
                typename: 'PhoneNumberValue',
            };
        }
        case 'EmailValue': {
            return {
                isArray: false,
                typename: 'EmailValue',
            };
        }
        case 'UrlValue': {
            return {
                isArray: false,
                typename: 'UrlValue',
            };
        }
        case 'EncryptedStringValue': {
            return {
                isArray: false,
                typename: 'EncryptedStringValue',
            };
        }
        case 'CurrencyValue': {
            return {
                isArray: false,
                typename: 'CurrencyValue',
            };
        }
        case 'LongitudeValue': {
            return {
                isArray: false,
                typename: 'LongitudeValue',
            };
        }
        case 'LatitudeValue': {
            return {
                isArray: false,
                typename: 'LatitudeValue',
            };
        }
        case 'PicklistValue': {
            return {
                isArray: false,
                typename: 'PicklistValue',
            };
        }
        case 'MultiPicklistValue': {
            return {
                isArray: false,
                typename: 'MultiPicklistValue',
            };
        }
        case 'LongValue': {
            return {
                isArray: false,
                typename: 'LongValue',
            };
        }
        case 'DoubleValue': {
            return {
                isArray: false,
                typename: 'DoubleValue',
            };
        }
        case 'PercentValue': {
            return {
                isArray: false,
                typename: 'PercentValue',
            };
        }
        case 'Base64Value': {
            return {
                isArray: false,
                typename: 'Base64Value',
            };
        }
        case 'JSONValue': {
            return {
                isArray: false,
                typename: 'JSONValue',
            };
        }
        default:
            return undefined;
    }
}

function selectFieldValue(reader, fieldData, sel, variables, fragments, sink, key, selectFunction, __typename, hasFormat, hasLabel, parentRecordId) {
    const gqlFieldValue = readFieldValueAndConvertToGQL(reader, fieldData, hasFormat, hasLabel, __typename);
    if (gqlFieldValue !== undefined) {
        const result = selectFunction(sel, variables, fragments)({ data: gqlFieldValue }, reader, parentRecordId);
        reader.assignNonScalar(sink, key, result);
    }
    else {
        reader.markMissing();
    }
}
function readFieldValueAndConvertToGQL(reader, fieldData, hasFormat, hasLabel, __typename) {
    // Some top level fields aren't currently ingested as fields also, so if there's no ref, try returning the data.
    if (fieldData.__ref !== undefined) {
        const FieldValueLink = reader.read({
            recordId: fieldData.__ref,
            node: {
                kind: 'Fragment',
                private: [],
                opaque: true,
                version: VERSION$2M,
            },
            variables: {},
        });
        reader.markSeenId(fieldData.__ref);
        if (FieldValueLink.data !== undefined) {
            const result = convertFieldValueRepresentationToGraphQL(FieldValueLink.data, hasFormat, hasLabel, __typename);
            return result;
        }
        else {
            reader.markMissingLink(fieldData.__ref);
        }
    }
    else {
        return fieldData;
    }
}
function convertFieldValueRepresentationToGraphQL(source, hasFormat, hasLabel, __typename) {
    const fieldValue = {
        ...source,
        __typename,
    };
    // Apparently these fields === displayValue.
    if (hasFormat) {
        fieldValue['format'] = source.displayValue;
    }
    if (hasLabel) {
        fieldValue['label'] = source.displayValue;
    }
    return fieldValue;
}
// Helper function to deal with all 'regular' Record fields
function selectScalarFieldValueRepresentation(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    const fieldValueConfig = {
        hasFormat: false,
        hasLabel: false,
    };
    switch (typename) {
        // All fields which extend FieldValue, and no other special fields (format etc)
        case 'BooleanValue':
            fieldValueConfig.selectFunction = select$1o;
            break;
        case 'LongitudeValue':
            fieldValueConfig.selectFunction = select$1d;
            break;
        case 'UrlValue':
            fieldValueConfig.selectFunction = select$1g;
            break;
        case 'RichTextAreaValue':
            fieldValueConfig.selectFunction = select$1j;
            break;
        case 'LongTextAreaValue':
            fieldValueConfig.selectFunction = select$1k;
            break;
        case 'LatitudeValue':
            fieldValueConfig.selectFunction = select$1c;
            break;
        case 'PhoneNumberValue':
            fieldValueConfig.selectFunction = select$1i;
            break;
        case 'TextAreaValue':
            fieldValueConfig.selectFunction = select$1l;
            break;
        case 'IDValue':
            fieldValueConfig.selectFunction = select$1q;
            break;
        case 'JSONValue':
            fieldValueConfig.selectFunction = select$18;
            break;
        case 'EmailValue':
            fieldValueConfig.selectFunction = select$1h;
            break;
        case 'Base64Value':
            fieldValueConfig.selectFunction = select$19;
            break;
        case 'EncryptedStringValue':
            fieldValueConfig.selectFunction = select$1f;
            break;
        // All field values which support 'label'
        case 'StringValue':
            fieldValueConfig.selectFunction = select$1s;
            fieldValueConfig.hasLabel = true;
            break;
        case 'PicklistValue':
            fieldValueConfig.selectFunction = select$1b;
            fieldValueConfig.hasLabel = true;
            break;
        case 'MultiPicklistValue':
            fieldValueConfig.selectFunction = select$1a;
            fieldValueConfig.hasLabel = true;
            break;
        // All field values that support "format" field.
        case 'IntValue':
            fieldValueConfig.selectFunction = select$1t;
            fieldValueConfig.hasFormat = true;
            break;
        case 'DateValue':
            fieldValueConfig.selectFunction = select$1m;
            fieldValueConfig.hasFormat = true;
            break;
        case 'DoubleValue':
            fieldValueConfig.selectFunction = select$1x;
            fieldValueConfig.hasFormat = true;
            break;
        case 'TimeValue':
            fieldValueConfig.selectFunction = select$1n;
            fieldValueConfig.hasFormat = true;
            break;
        case 'DateTimeValue':
            fieldValueConfig.selectFunction = select$1p;
            fieldValueConfig.hasFormat = true;
            break;
        case 'LongValue':
            fieldValueConfig.selectFunction = select$1w;
            fieldValueConfig.hasFormat = true;
            break;
        case 'PercentValue':
            fieldValueConfig.selectFunction = select$1v;
            fieldValueConfig.hasFormat = true;
            break;
        case 'CurrencyValue':
            fieldValueConfig.selectFunction = select$1e;
            fieldValueConfig.hasFormat = true;
            break;
    }
    if (fieldValueConfig.selectFunction !== undefined) {
        selectFieldValue(reader, fieldData, sel, variables, fragments, sink, key, fieldValueConfig.selectFunction, typename, fieldValueConfig.hasFormat, fieldValueConfig.hasLabel, parentRecordId);
    }
}

function selectType$X(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'Boolean':
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        default:
            // If you didn't match any of the above, you're a FieldValueRepresentation (scalar Record field)
            selectScalarFieldValueRepresentation(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId);
    }
    return sink;
}

const name$W = 'CompoundField';
const VERSION$19 = 'e90ed8ffd8c8bf518fa166be5c214253';
function keyBuilder$18(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$W(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$15(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$18(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$X,
        isMappedFieldImport: isMappedField$W,
        getFieldType: getFieldType$1b,
        ingestFieldByType: ingestFieldByType$W,
        createLink: createLink,
        mergeData: mergeData$W,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "CompoundField",
            version: VERSION$19,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$W(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$1b(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function buildSelectionForField$X(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$W, getFieldType$1b, selectType$X, parentRecordId);
}
function buildSelectionForNode$X(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$X, buildSelectionForField$X, parentRecordId);
}
function select$16(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$X, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$1c() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$16(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$18(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$W
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$X) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$1b(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'IntValue': {
                const IntValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, IntValueState);
                break;
            }
            case 'StringValue': {
                const StringValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, StringValueState);
                break;
            }
            case 'BooleanValue': {
                const BooleanValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1o(cacheKeySink, requestedField, BooleanValueState);
                break;
            }
            case 'IDValue': {
                const IDValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, IDValueState);
                break;
            }
            case 'DateTimeValue': {
                const DateTimeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, DateTimeValueState);
                break;
            }
            case 'TimeValue': {
                const TimeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1n(cacheKeySink, requestedField, TimeValueState);
                break;
            }
            case 'DateValue': {
                const DateValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1m(cacheKeySink, requestedField, DateValueState);
                break;
            }
            case 'TextAreaValue': {
                const TextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1l(cacheKeySink, requestedField, TextAreaValueState);
                break;
            }
            case 'LongTextAreaValue': {
                const LongTextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1k(cacheKeySink, requestedField, LongTextAreaValueState);
                break;
            }
            case 'RichTextAreaValue': {
                const RichTextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1j(cacheKeySink, requestedField, RichTextAreaValueState);
                break;
            }
            case 'PhoneNumberValue': {
                const PhoneNumberValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1i(cacheKeySink, requestedField, PhoneNumberValueState);
                break;
            }
            case 'EmailValue': {
                const EmailValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1h(cacheKeySink, requestedField, EmailValueState);
                break;
            }
            case 'UrlValue': {
                const UrlValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1g(cacheKeySink, requestedField, UrlValueState);
                break;
            }
            case 'EncryptedStringValue': {
                const EncryptedStringValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1f(cacheKeySink, requestedField, EncryptedStringValueState);
                break;
            }
            case 'CurrencyValue': {
                const CurrencyValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1e(cacheKeySink, requestedField, CurrencyValueState);
                break;
            }
            case 'LongitudeValue': {
                const LongitudeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1d(cacheKeySink, requestedField, LongitudeValueState);
                break;
            }
            case 'LatitudeValue': {
                const LatitudeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1c(cacheKeySink, requestedField, LatitudeValueState);
                break;
            }
            case 'PicklistValue': {
                const PicklistValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1b(cacheKeySink, requestedField, PicklistValueState);
                break;
            }
            case 'MultiPicklistValue': {
                const MultiPicklistValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1a(cacheKeySink, requestedField, MultiPicklistValueState);
                break;
            }
            case 'LongValue': {
                const LongValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, LongValueState);
                break;
            }
            case 'DoubleValue': {
                const DoubleValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, DoubleValueState);
                break;
            }
            case 'PercentValue': {
                const PercentValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1v(cacheKeySink, requestedField, PercentValueState);
                break;
            }
            case 'Base64Value': {
                const Base64ValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$19(cacheKeySink, requestedField, Base64ValueState);
                break;
            }
            case 'JSONValue': {
                const JSONValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$18(cacheKeySink, requestedField, JSONValueState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function ingestFieldByType$W(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'StringValue': {
            const StringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1r(requestedField, StringValueState);
            break;
        }
        case 'BooleanValue': {
            const BooleanValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1n(requestedField, BooleanValueState);
            break;
        }
        case 'IDValue': {
            const IDValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1p(requestedField, IDValueState);
            break;
        }
        case 'DateTimeValue': {
            const DateTimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1o(requestedField, DateTimeValueState);
            break;
        }
        case 'TimeValue': {
            const TimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1m(requestedField, TimeValueState);
            break;
        }
        case 'DateValue': {
            const DateValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1l(requestedField, DateValueState);
            break;
        }
        case 'TextAreaValue': {
            const TextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1k(requestedField, TextAreaValueState);
            break;
        }
        case 'LongTextAreaValue': {
            const LongTextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1j(requestedField, LongTextAreaValueState);
            break;
        }
        case 'RichTextAreaValue': {
            const RichTextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1i(requestedField, RichTextAreaValueState);
            break;
        }
        case 'PhoneNumberValue': {
            const PhoneNumberValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1h(requestedField, PhoneNumberValueState);
            break;
        }
        case 'EmailValue': {
            const EmailValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1g(requestedField, EmailValueState);
            break;
        }
        case 'UrlValue': {
            const UrlValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1f(requestedField, UrlValueState);
            break;
        }
        case 'EncryptedStringValue': {
            const EncryptedStringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1e(requestedField, EncryptedStringValueState);
            break;
        }
        case 'CurrencyValue': {
            const CurrencyValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1d(requestedField, CurrencyValueState);
            break;
        }
        case 'LongitudeValue': {
            const LongitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1c(requestedField, LongitudeValueState);
            break;
        }
        case 'LatitudeValue': {
            const LatitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1b(requestedField, LatitudeValueState);
            break;
        }
        case 'PicklistValue': {
            const PicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1a(requestedField, PicklistValueState);
            break;
        }
        case 'MultiPicklistValue': {
            const MultiPicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$19(requestedField, MultiPicklistValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'PercentValue': {
            const PercentValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1u(requestedField, PercentValueState);
            break;
        }
        case 'Base64Value': {
            const Base64ValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$18(requestedField, Base64ValueState);
            break;
        }
        case 'JSONValue': {
            const JSONValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$17(requestedField, JSONValueState);
            break;
        }
    }
}
function isMappedField$W(field) {
    return false;
}
function isFragmentApplicable$X(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "CompoundField", typename);
}
function getInContextFragmentType$1b(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getFieldType$1a(field) {
    const fieldNameOverride = getFieldNameOverride(field);
    const fieldName = fieldNameOverride || field.name.value;
    switch (fieldName) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'String': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'Long': {
            return {
                isArray: false,
                typename: 'Long',
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID',
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'WeakEtag': {
            return {
                isArray: false,
                typename: 'Long',
            };
        }
        case 'DisplayValue': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'LastModifiedById': {
            return {
                isArray: false,
                typename: 'IDValue',
            };
        }
        case 'LastModifiedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue',
            };
        }
        case 'SystemModstamp': {
            return {
                isArray: false,
                typename: 'DateTimeValue',
            };
        }
        case 'RecordTypeId': {
            return {
                isArray: false,
                typename: 'IDValue',
            };
        }
        case 'IntValue': {
            return {
                isArray: false,
                typename: 'IntValue',
            };
        }
        case 'StringValue': {
            return {
                isArray: false,
                typename: 'StringValue',
            };
        }
        case 'BooleanValue': {
            return {
                isArray: false,
                typename: 'BooleanValue',
            };
        }
        case 'IDValue': {
            return {
                isArray: false,
                typename: 'IDValue',
            };
        }
        case 'DateTimeValue': {
            return {
                isArray: false,
                typename: 'DateTimeValue',
            };
        }
        case 'TimeValue': {
            return {
                isArray: false,
                typename: 'TimeValue',
            };
        }
        case 'DateValue': {
            return {
                isArray: false,
                typename: 'DateValue',
            };
        }
        case 'TextAreaValue': {
            return {
                isArray: false,
                typename: 'TextAreaValue',
            };
        }
        case 'LongTextAreaValue': {
            return {
                isArray: false,
                typename: 'LongTextAreaValue',
            };
        }
        case 'RichTextAreaValue': {
            return {
                isArray: false,
                typename: 'RichTextAreaValue',
            };
        }
        case 'PhoneNumberValue': {
            return {
                isArray: false,
                typename: 'PhoneNumberValue',
            };
        }
        case 'EmailValue': {
            return {
                isArray: false,
                typename: 'EmailValue',
            };
        }
        case 'UrlValue': {
            return {
                isArray: false,
                typename: 'UrlValue',
            };
        }
        case 'EncryptedStringValue': {
            return {
                isArray: false,
                typename: 'EncryptedStringValue',
            };
        }
        case 'CurrencyValue': {
            return {
                isArray: false,
                typename: 'CurrencyValue',
            };
        }
        case 'LongitudeValue': {
            return {
                isArray: false,
                typename: 'LongitudeValue',
            };
        }
        case 'LatitudeValue': {
            return {
                isArray: false,
                typename: 'LatitudeValue',
            };
        }
        case 'PicklistValue': {
            return {
                isArray: false,
                typename: 'PicklistValue',
            };
        }
        case 'MultiPicklistValue': {
            return {
                isArray: false,
                typename: 'MultiPicklistValue',
            };
        }
        case 'LongValue': {
            return {
                isArray: false,
                typename: 'LongValue',
            };
        }
        case 'DoubleValue': {
            return {
                isArray: false,
                typename: 'DoubleValue',
            };
        }
        case 'PercentValue': {
            return {
                isArray: false,
                typename: 'PercentValue',
            };
        }
        case 'Base64Value': {
            return {
                isArray: false,
                typename: 'Base64Value',
            };
        }
        case 'JSONValue': {
            return {
                isArray: false,
                typename: 'JSONValue',
            };
        }
        case 'polymorphicParentRelationship':
        case 'parentRelationship': {
            return {
                isArray: false,
                typename: 'RecordRepresentation',
            };
        }
        case 'childRelationship': {
            return {
                isArray: false,
                typename: 'RecordConnection',
            };
        }
        case 'CompoundField': {
            return {
                isArray: false,
                typename: 'CompoundField',
            };
        }
        // We don't know what type "value" is, so return a JSONValue type that uses an unknown for the value field.
        default:
            return {
                isArray: false,
                typename: 'JSONValue',
            };
    }
}

function isMappedField$V(field) {
    if (field.name.value === '__typename' || field.name.value === 'DisplayValue') {
        return false;
    }
    const fieldType = getFieldType$1a(field);
    // Map everything that isn't typename, DisplayValue or connection/childRelationship field.
    return fieldType !== undefined && fieldType.typename !== 'RecordConnection';
}

const SPECIAL_NORMALIZED_RECORD_FIELDS = ['ApiName', 'RecordTypeId', 'WeakEtag', 'DisplayValue'];
function convertGraphQLToRaml(astNode, state) {
    const { data } = state;
    const requestedFields = astNode.selectionSet
        ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$V)
        : new Map();
    const { fieldsBag, trie } = createFieldsBagAndTrie(data, requestedFields, state);
    const recordRepresentation = {
        apiName: data.ApiName,
        eTag: '',
        lastModifiedById: data.LastModifiedById.value,
        lastModifiedDate: data.LastModifiedDate.value,
        systemModstamp: data.SystemModstamp.value,
        recordTypeId: data.ldsRecordTypeId === null ? null : data.ldsRecordTypeId.value,
        recordTypeInfo: null,
        childRelationships: {},
        id: data.Id,
        weakEtag: data.WeakEtag,
        fields: fieldsBag,
    };
    return {
        recordRepresentation,
        fieldsTrie: trie,
    };
}
function createFieldsBagAndTrie(data, requestedFields, parentState) {
    const fieldsBag = ObjectCreate(null);
    const trieChildren = ObjectCreate(null);
    const trie = {
        name: data.ApiName,
        children: trieChildren,
    };
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // We got a field back we didn't ask for. Skip.
        }
        const fieldType = getFieldType$1a(requestedField);
        const fieldName = requestedField.name.value;
        // Skip any standard fields on RecordRep not in the fieldsBag, introspection fields, or unknown typed fields
        if (shouldSkipFieldNormalization(fieldName)) {
            continue;
        }
        // Id is special and _can_ be requested as a field in REST
        if (fieldName === 'Id') {
            fieldsBag[fieldName] = {
                value: fieldData !== undefined ? fieldData : null,
                displayValue: null,
            };
            trieChildren[fieldName] = {
                name: fieldName,
                children: {},
            };
            continue;
        }
        switch (fieldType.typename) {
            case 'RecordConnection':
                continue; // Let the 'normal' graphql handling get us into the Records in childRelationships.
            case 'PolymorphicParentRelationship':
            case 'RecordRepresentation': {
                // Spanning record field
                const state = {
                    ...parentState,
                    data: fieldData,
                };
                const { value, trie } = formatSpanningField(requestedField, state);
                trieChildren[fieldName] = trie;
                fieldsBag[fieldName] = value;
                break;
            }
            case 'CompoundField':
                // The REST endpoint has no concept of compound fields in a RecordRep, so best we can
                // do is just setting all the subfields as fields in the trie.
                Object.keys(fieldData).forEach((subfieldKey) => {
                    if (!shouldSkipFieldNormalization(subfieldKey)) {
                        setFieldsOnTrie(subfieldKey, fieldData[subfieldKey], fieldsBag, trieChildren);
                    }
                });
                break; // We don't set the top level compound field in the fieldTrie - There's no way to synthesize a Compound Field in REST responses w/o a lookaside table
            default:
                // Not a spanning field or a childRelationship - Do the standard field mapping.
                // There are edgecases where we don't inject all fields properly, which leads to an ingestion error
                // for FieldValueRepresentation. Default to null here to stop from throwing.
                setFieldsOnTrie(fieldName, fieldData, fieldsBag, trieChildren);
        }
    }
    return { fieldsBag, trie };
}
function setFieldsOnTrie(fieldName, fieldData, fieldsBag, trieChildren) {
    fieldsBag[fieldName] = {
        value: fieldData.value !== undefined ? fieldData.value : null,
        displayValue: getFieldDisplayValue(fieldData),
    };
    trieChildren[fieldName] = {
        name: fieldName,
        children: {},
    };
}
function shouldSkipFieldNormalization(fieldName) {
    return fieldName === '__typename' || SPECIAL_NORMALIZED_RECORD_FIELDS.includes(fieldName);
}
// GraphQL exposes field-specific properties of label/format depending on types. These values are
// always equal to the "normalized" displayValue field.
function getFieldDisplayValue(fieldData) {
    const displayValue = fieldData.label || fieldData.format || fieldData.displayValue;
    return displayValue !== undefined ? displayValue : null;
}
function formatSpanningField(field, state) {
    if (state.data === null) {
        return {
            value: {
                displayValue: null,
                value: null,
            },
            trie: convertAstToTrie(field, state.fragments),
        };
    }
    const { recordRepresentation, fieldsTrie } = convertGraphQLToRaml(field, state);
    return {
        trie: fieldsTrie,
        value: {
            displayValue: state.data.DisplayValue,
            value: recordRepresentation,
        },
    };
}
function convertAstToTrie(astNode, fragments) {
    const children = {};
    const trie = {
        name: astNode.name.value,
        children,
    };
    const requestedFields = astNode.selectionSet
        ? getRequestedFieldsForType('', // data is null here
        astNode.selectionSet, fragments, isFragmentApplicable$V)
        : new Map();
    for (const [, field] of requestedFields) {
        const fieldType = getFieldType$1a(field);
        const fieldName = field.name.value;
        //Untyped field - Adapter forwarded request?
        if (fieldType === undefined) {
            children[fieldName] = {
                name: fieldName,
                children: {},
            };
            continue;
        }
        if (shouldSkipFieldNormalization(field.name.value)) {
            continue;
        }
        switch (fieldType.typename) {
            case 'RecordConnection':
                continue; // Let the 'normal' graphql handling get us into the Records in childRelationships.
            case 'PolymorphicParentRelationship':
            case 'RecordRepresentation':
                // Spanning record field
                children[fieldName] = convertAstToTrie(field, fragments);
                break;
            default:
                // Does a Compound field need something special here?
                children[fieldName] = {
                    name: fieldName,
                    children: {},
                };
        }
    }
    return trie;
}

function ingestMappedData(astNode, state) {
    const ramlSink = convertGraphQLToRaml(astNode, state);
    // partially ingest with RAML
    const { recordRepresentation, fieldsTrie } = ramlSink;
    const ingestRecord = createIngestRecordWithFields(fieldsTrie, {
        name: recordRepresentation.apiName,
        children: {},
    });
    return ingestRecord(recordRepresentation, state.path, state.luvio, state.store, state.timestamp);
}

// Most of these could be generated off of GraphQL interface 'FieldValue'. Then just add RecordRepresentation and PolymorphicParentRelationship
const FieldValueTypes = [
    'IntValue',
    'StringValue',
    'BooleanValue',
    'IDValue',
    'DateTimeValue',
    'TimeValue',
    'DateValue',
    'TextAreaValue',
    'LongTextAreaValue',
    'RichTextAreaValue',
    'PhoneNumberValue',
    'EmailValue',
    'UrlValue',
    'EncryptedStringValue',
    'CurrencyValue',
    'LongitudeValue',
    'LatitudeValue',
    'PicklistValue',
    'MultiPicklistValue',
    'LongValue',
    'DoubleValue',
    'PercentValue',
    'Base64Value',
    'JSONValue',
    'RecordRepresentation',
    'PolymorphicParentRelationship',
];
/*
    This function is responsible for the inverse mapping of RecordRepresentation data into
    GraphQL shaped data.
*/
function getFieldData$V(source, sel, variables) {
    switch (sel.name.value) {
        // Scalar GraphQL fields (don't need to format, just return primitive values)
        case 'Id': {
            return source.__linkedData ? source.__linkedData['id'] : undefined;
        }
        case 'ApiName': {
            return source.__linkedData ? source.__linkedData['apiName'] : undefined;
        }
        case 'WeakEtag': {
            return source.__linkedData ? source.__linkedData['weakEtag'] : undefined;
        }
        // RecordTypeId is not requestable as field data in REST
        case 'RecordTypeId': {
            const topLevelKey = 'recordTypeId';
            const graphQLValue = source.__linkedData
                ? {
                    __typename: 'IDValue',
                    displayValue: null,
                    value: source.__linkedData[topLevelKey],
                }
                : undefined;
            return graphQLValue;
        }
        default: {
            const fieldType = getFieldType$1a(sel);
            if (fieldType !== undefined && FieldValueTypes.includes(fieldType.typename)) {
                // This is a FieldValue we need to unwind from the RecordRep
                return source.__linkedData ? source.__linkedData.fields[sel.name.value] : undefined;
            }
            if (fieldType !== undefined && fieldType.typename === 'CompoundField') {
                // Return the full fields array and let the sub selections sort out which ones we want
                return source.__linkedData ? source.__linkedData.fields : undefined;
            }
            // Everything else must be a 'regular' gql field (DisplayValue)
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}

function selectType$W(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'Long':
        case 'ID':
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'PolymorphicParentRelationship':
        case 'RecordRepresentation': {
            const spanningFieldLink = resolveLink(reader, fieldData, VERSION$2M);
            const resolvedSpanningFieldValue = spanningFieldLink && spanningFieldLink.data;
            const fieldDataRef = fieldData.__ref;
            if (resolvedSpanningFieldValue !== undefined) {
                reader.markSeenId(fieldDataRef);
                const { value: spanningFieldResult } = resolvedSpanningFieldValue;
                // Handle null values - graphql will return it at the field level, not return nested { value: null }
                if (spanningFieldResult === null || typeof spanningFieldResult !== 'object') {
                    reader.assignScalar(key, sink, null);
                    break;
                }
                // Synthesize what a NormalizedGraphQL Record looks like, then recurse to our select function to get the value.
                const syntheticNormalizedGraphQLRecord = {
                    data: {
                        DisplayValue: resolvedSpanningFieldValue.displayValue,
                    },
                    __link: spanningFieldResult,
                };
                const result = select$14(sel, variables, fragments)(syntheticNormalizedGraphQLRecord, reader, parentRecordId);
                if (result) {
                    reader.assignNonScalar(sink, key, result);
                }
                else {
                    reader.markMissingLink(fieldData.__ref);
                    reader.markMissing();
                }
            }
            else {
                if (fieldDataRef !== undefined) {
                    reader.markMissingLink(fieldDataRef);
                }
                reader.markMissing();
            }
            break;
        }
        case 'RecordConnection':
            selectType$U(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId); // No need to copy paste this case (lots of pagination code).
            break;
        case 'CompoundField': {
            const syntheticResult = select$16(sel, variables, fragments)({
                data: fieldData,
            }, reader, parentRecordId);
            reader.assignNonScalar(sink, key, syntheticResult);
            break;
        }
        default:
            selectScalarFieldValueRepresentation(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId);
    }
    return sink;
}

const API_NAMESPACE = 'UiApi';
const RECORD_REPRESENTATION_NAME = 'RecordRepresentation';
const RECORD_VIEW_ENTITY_REPRESENTATION_NAME = 'RecordViewEntityRepresentation';
const RECORD_ID_PREFIX = `${API_NAMESPACE}::${RECORD_REPRESENTATION_NAME}:`;
const RECORD_VIEW_ENTITY_ID_PREFIX = `${API_NAMESPACE}::${RECORD_VIEW_ENTITY_REPRESENTATION_NAME}:Name:`;
const RECORD_FIELDS_KEY_JUNCTION = '__fields__';
function extractRecordIdFromStoreKey(key) {
    if (key === undefined ||
        (key.indexOf(RECORD_ID_PREFIX) === -1 && key.indexOf(RECORD_VIEW_ENTITY_ID_PREFIX) === -1)) {
        return undefined;
    }
    const parts = key.split(':');
    return parts[parts.length - 1].split('_')[0];
}
function isStoreKeyRecordViewEntity(key) {
    return (key.indexOf(RECORD_VIEW_ENTITY_ID_PREFIX) > -1 &&
        key.indexOf(RECORD_FIELDS_KEY_JUNCTION) === -1);
}
function buildRecordRepKeyFromId(recordId) {
    return `${API_NAMESPACE}::${RECORD_REPRESENTATION_NAME}:${recordId}`;
}

function attachMappedData(source, reader) {
    let linkedRecordId = source.__link && source.__link.__ref;
    if (linkedRecordId !== undefined) {
        // Discovered with: W-13612372
        // We get wrong apiName for polymorphic parents when using getRecord
        // this breaks store links as it essentially creates wrong keys
        // also worth noting that polymorphic parents from REST are stored in RecordViewEntityRepresentation
        // while polymorphic parents from graphql are just another RecordRepresentation
        // Here, if we see a RecordViewEntityRepresentation key then we point it to
        // corresponding RecordRepresentation key that graphql adapter can understand
        // this essentially also means that polymorphic parent Records can be inconsistent
        // across graphql and getRecord snapshots and subsequent requests from getRecord
        // to fetch the referenced parent record directly
        // This behavior is at parity with getRecord wire adapter
        if (isStoreKeyRecordViewEntity(linkedRecordId)) {
            const recordId = extractRecordIdFromStoreKey(linkedRecordId);
            if (recordId) {
                linkedRecordId = buildRecordRepKeyFromId(recordId);
            }
        }
        reader.markSeenId(linkedRecordId);
        const linkedData = reader.read({
            recordId: linkedRecordId,
            node: {
                kind: 'Fragment',
                private: [],
                opaque: true,
                version: VERSION$2K,
            },
            variables: {},
        });
        if (linkedData.data !== undefined) {
            /// The Metaschema doesn't let us actually know that Account is a RecordRepresentation statically. This override will let us set __typename
            // so that we can properly use fragments that are on Account
            return {
                ...source,
                data: {
                    ...source.data,
                    __typename: linkedData.data.apiName,
                },
                __linkedData: linkedData.data,
            };
        }
        else {
            reader.markMissingLink(linkedRecordId);
        }
    }
    return source;
}

const name$V = 'PageInfo';
const VERSION$18 = '20ddc8913f009fcc648e94136b42e794';
function keyBuilder$17(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$V(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$14(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$17(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$W,
        isMappedFieldImport: isMappedField$U,
        getFieldType: getFieldType$19,
        ingestFieldByType: ingestFieldByType$V,
        createLink: createLink,
        mergeData: mergeData$V,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PageInfo",
            version: VERSION$18,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$U(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$19(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$V(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$W(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$U, getFieldType$19, selectType$V, parentRecordId);
}
function buildSelectionForNode$W(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$W, buildSelectionForField$W, parentRecordId);
}
function select$15(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$W, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$1b() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$15(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$17(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$V
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$19(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'hasNextPage': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'hasPreviousPage': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'startCursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'endCursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$V(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$U(field) {
    return false;
}
function isFragmentApplicable$W(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "PageInfo", typename);
}
function getInContextFragmentType$1a(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getTypeCacheKeys$14(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$14(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$S,
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet
        ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$T)
        : new Map();
    for (const fieldDataKey of ObjectKeys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$17(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordEdge': {
                const fullPath__edges = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: `${fullPath__edges}__${i}`,
                        },
                        data: item,
                    };
                    getTypeCacheKeys$12(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'PageInfo': {
                // I guess just return the pagination key here since pagination replaced it.
                cacheKeySink.set(`${rootKey}__pagination`, {
                    namespace: keyPrefix,
                    mergeable: true,
                    representationName: name$V,
                });
                break;
            }
        }
    }
}

function getMappedTypeCacheKeys(cache, astNode, state) {
    const convertedData = convertGraphQLToRaml(astNode, state);
    getTypeCacheKeys$2l(cache, state.luvio, convertedData.recordRepresentation);
}

const name$U = 'RecordRepresentation';
const VERSION$17 = 'b09728109b666bc69fe789bade015193';
function keyBuilder$16(luvio, path, data) {
    return keyPrefix + '::uiapi::' + name$U + '::' + data.Id; // We're missing idNodeName in the actual key here...
}
function mergeData$U(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
        __link: ObjectAssign({}, existingData["__link"] || {}, newData["__link"] || {}),
    };
}
function ingest$13(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$16(luvio, path, data);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$V,
        isMappedFieldImport: isMappedField$V,
        getFieldType: getFieldType$1a,
        ingestFieldByType: ingestFieldByType$U,
        ramlIngestBlock(newData) {
            newData["__link"] = ingestMappedData(astNode, state);
        },
        createLink: createLink,
        mergeData: mergeData$U,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordRepresentation",
            version: VERSION$17,
            ingestionTimestamp: timestamp,
        },
    });
}
function selectType$U(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Long': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'IDValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1t, select$1q, false);
            break;
        }
        case 'DateTimeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1s, select$1p, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'StringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1v, select$1s, false);
            break;
        }
        case 'BooleanValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1r, select$1o, false);
            break;
        }
        case 'TimeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1q, select$1n, false);
            break;
        }
        case 'DateValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1p, select$1m, false);
            break;
        }
        case 'TextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1o, select$1l, false);
            break;
        }
        case 'LongTextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1n, select$1k, false);
            break;
        }
        case 'RichTextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1m, select$1j, false);
            break;
        }
        case 'PhoneNumberValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1l, select$1i, false);
            break;
        }
        case 'EmailValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1k, select$1h, false);
            break;
        }
        case 'UrlValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1j, select$1g, false);
            break;
        }
        case 'EncryptedStringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1i, select$1f, false);
            break;
        }
        case 'CurrencyValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1h, select$1e, false);
            break;
        }
        case 'LongitudeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1g, select$1d, false);
            break;
        }
        case 'LatitudeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1f, select$1c, false);
            break;
        }
        case 'PicklistValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1e, select$1b, false);
            break;
        }
        case 'MultiPicklistValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1d, select$1a, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'PercentValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1y, select$1v, false);
            break;
        }
        case 'Base64Value': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1c, select$19, false);
            break;
        }
        case 'JSONValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1b, select$18, false);
            break;
        }
        case 'RecordRepresentation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$17, select$14, false);
            break;
        }
        case 'PolymorphicParentRelationship': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1a, select$17, false);
            break;
        }
        case 'RecordConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$15, select$12, true);
            break;
        }
        case 'CompoundField': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$19, select$16, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$V(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$V, getFieldType$1a, selectType$W, parentRecordId);
}
function buildSelectionForNode$V(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$V, buildSelectionForField$V, parentRecordId);
}
function select$14(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        source = attachMappedData(source, reader);
        reader.markSeenId(parentRecordId); // Always mark identifiable types as seen
        return selectCalculateSink(sink, field, buildSelectionForNode$V, source, reader, variables, fragments, parentRecordId);
    };
}
function getTypeCacheKeys$13(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$16(luvio, path, data);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$U
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$V) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$1a(requestedField);
        if (isMappedField$V(requestedField) || fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'IDValue': {
                const LastModifiedByIdState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, LastModifiedByIdState);
                break;
            }
            case 'DateTimeValue': {
                const LastModifiedDateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, LastModifiedDateState);
                break;
            }
            case 'DateTimeValue': {
                const SystemModstampState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, SystemModstampState);
                break;
            }
            case 'IDValue': {
                const RecordTypeIdState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, RecordTypeIdState);
                break;
            }
            case 'IntValue': {
                const IntValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, IntValueState);
                break;
            }
            case 'StringValue': {
                const StringValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, StringValueState);
                break;
            }
            case 'BooleanValue': {
                const BooleanValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1o(cacheKeySink, requestedField, BooleanValueState);
                break;
            }
            case 'IDValue': {
                const IDValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, IDValueState);
                break;
            }
            case 'DateTimeValue': {
                const DateTimeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, DateTimeValueState);
                break;
            }
            case 'TimeValue': {
                const TimeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1n(cacheKeySink, requestedField, TimeValueState);
                break;
            }
            case 'DateValue': {
                const DateValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1m(cacheKeySink, requestedField, DateValueState);
                break;
            }
            case 'TextAreaValue': {
                const TextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1l(cacheKeySink, requestedField, TextAreaValueState);
                break;
            }
            case 'LongTextAreaValue': {
                const LongTextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1k(cacheKeySink, requestedField, LongTextAreaValueState);
                break;
            }
            case 'RichTextAreaValue': {
                const RichTextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1j(cacheKeySink, requestedField, RichTextAreaValueState);
                break;
            }
            case 'PhoneNumberValue': {
                const PhoneNumberValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1i(cacheKeySink, requestedField, PhoneNumberValueState);
                break;
            }
            case 'EmailValue': {
                const EmailValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1h(cacheKeySink, requestedField, EmailValueState);
                break;
            }
            case 'UrlValue': {
                const UrlValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1g(cacheKeySink, requestedField, UrlValueState);
                break;
            }
            case 'EncryptedStringValue': {
                const EncryptedStringValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1f(cacheKeySink, requestedField, EncryptedStringValueState);
                break;
            }
            case 'CurrencyValue': {
                const CurrencyValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1e(cacheKeySink, requestedField, CurrencyValueState);
                break;
            }
            case 'LongitudeValue': {
                const LongitudeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1d(cacheKeySink, requestedField, LongitudeValueState);
                break;
            }
            case 'LatitudeValue': {
                const LatitudeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1c(cacheKeySink, requestedField, LatitudeValueState);
                break;
            }
            case 'PicklistValue': {
                const PicklistValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1b(cacheKeySink, requestedField, PicklistValueState);
                break;
            }
            case 'MultiPicklistValue': {
                const MultiPicklistValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1a(cacheKeySink, requestedField, MultiPicklistValueState);
                break;
            }
            case 'LongValue': {
                const LongValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, LongValueState);
                break;
            }
            case 'DoubleValue': {
                const DoubleValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, DoubleValueState);
                break;
            }
            case 'PercentValue': {
                const PercentValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1v(cacheKeySink, requestedField, PercentValueState);
                break;
            }
            case 'Base64Value': {
                const Base64ValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$19(cacheKeySink, requestedField, Base64ValueState);
                break;
            }
            case 'JSONValue': {
                const JSONValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$18(cacheKeySink, requestedField, JSONValueState);
                break;
            }
            case 'RecordRepresentation': {
                const parentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$13(cacheKeySink, requestedField, parentRelationshipState);
                break;
            }
            case 'PolymorphicParentRelationship': {
                const polymorphicParentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$17(cacheKeySink, requestedField, polymorphicParentRelationshipState);
                break;
            }
            case 'RecordConnection': {
                const childRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$14(cacheKeySink, requestedField, childRelationshipState);
                break;
            }
            case 'CompoundField': {
                const CompoundFieldState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$16(cacheKeySink, requestedField, CompoundFieldState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
    getMappedTypeCacheKeys(cacheKeySink, astNode, state);
}
function ingestFieldByType$U(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Long': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'IDValue': {
            const IDValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1p(requestedField, IDValueState);
            break;
        }
        case 'DateTimeValue': {
            const DateTimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1o(requestedField, DateTimeValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'StringValue': {
            const StringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1r(requestedField, StringValueState);
            break;
        }
        case 'BooleanValue': {
            const BooleanValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1n(requestedField, BooleanValueState);
            break;
        }
        case 'TimeValue': {
            const TimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1m(requestedField, TimeValueState);
            break;
        }
        case 'DateValue': {
            const DateValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1l(requestedField, DateValueState);
            break;
        }
        case 'TextAreaValue': {
            const TextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1k(requestedField, TextAreaValueState);
            break;
        }
        case 'LongTextAreaValue': {
            const LongTextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1j(requestedField, LongTextAreaValueState);
            break;
        }
        case 'RichTextAreaValue': {
            const RichTextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1i(requestedField, RichTextAreaValueState);
            break;
        }
        case 'PhoneNumberValue': {
            const PhoneNumberValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1h(requestedField, PhoneNumberValueState);
            break;
        }
        case 'EmailValue': {
            const EmailValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1g(requestedField, EmailValueState);
            break;
        }
        case 'UrlValue': {
            const UrlValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1f(requestedField, UrlValueState);
            break;
        }
        case 'EncryptedStringValue': {
            const EncryptedStringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1e(requestedField, EncryptedStringValueState);
            break;
        }
        case 'CurrencyValue': {
            const CurrencyValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1d(requestedField, CurrencyValueState);
            break;
        }
        case 'LongitudeValue': {
            const LongitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1c(requestedField, LongitudeValueState);
            break;
        }
        case 'LatitudeValue': {
            const LatitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1b(requestedField, LatitudeValueState);
            break;
        }
        case 'PicklistValue': {
            const PicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1a(requestedField, PicklistValueState);
            break;
        }
        case 'MultiPicklistValue': {
            const MultiPicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$19(requestedField, MultiPicklistValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'PercentValue': {
            const PercentValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1u(requestedField, PercentValueState);
            break;
        }
        case 'Base64Value': {
            const Base64ValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$18(requestedField, Base64ValueState);
            break;
        }
        case 'JSONValue': {
            const JSONValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$17(requestedField, JSONValueState);
            break;
        }
        case 'RecordRepresentation': {
            const RecordRepresentationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$13(requestedField, RecordRepresentationState);
            break;
        }
        case 'PolymorphicParentRelationship': {
            const PolymorphicParentRelationshipState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$16(requestedField, PolymorphicParentRelationshipState);
            break;
        }
        case 'RecordConnection': {
            const RecordConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$11(requestedField, RecordConnectionState);
            break;
        }
        case 'CompoundField': {
            const CompoundFieldState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$15(requestedField, CompoundFieldState);
            break;
        }
    }
}
function isFragmentApplicable$V(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["Record"], "RecordRepresentation", typename);
}

const name$T = 'RecordEdge';
const VERSION$16 = 'cdc691302ae1cc80680781cd1ae1f830';
function keyBuilder$15(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$T(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$12(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$15(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$U,
        isMappedFieldImport: isMappedField$T,
        getFieldType: getFieldType$18,
        ingestFieldByType: ingestFieldByType$T,
        createLink: createLink,
        mergeData: mergeData$T,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordEdge",
            version: VERSION$16,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$T(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$18(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$T(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordRepresentation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$17, select$14, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$U(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$T, getFieldType$18, selectType$T, parentRecordId);
}
function buildSelectionForNode$U(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$U, buildSelectionForField$U, parentRecordId);
}
function select$13(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$U, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$1a() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$12(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$15(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$T
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$U) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$18(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordRepresentation': {
                const nodeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$13(cacheKeySink, requestedField, nodeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$18(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'RecordRepresentation'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$T(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordRepresentation': {
            const RecordRepresentationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$13(requestedField, RecordRepresentationState);
            break;
        }
    }
}
function isMappedField$T(field) {
    return false;
}
function isFragmentApplicable$U(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordEdge", typename);
}
function getInContextFragmentType$19(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$S = 'RecordConnection';
const TTL$a = 300000;
const VERSION$15 = '5109e37c62e492243c1e6ae3967b0655';
function keyBuilder$14(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$S(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingestPaginationMetadata$4(astNode, state, key, sink, existingData) {
    var _a;
    const { data, luvio, store } = state;
    const { pageInfo: { hasNextPage, startCursor }, edges } = data;
    if (!hasGraphQLPaginationFields(data))
        return;
    const paginationKey = key + '__pagination';
    let existingPaginationMetadata = store.readEntry(paginationKey);
    const afterArg = (_a = astNode.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', state.variables);
    const isPagedResponse = afterArgVal !== undefined &&
        existingPaginationMetadata !== undefined &&
        existingPaginationMetadata[afterArgVal] !== undefined;
    if (isPagedResponse && existingPaginationMetadata !== undefined) {
        const previousPageOffset = existingPaginationMetadata[afterArgVal];
        existingPaginationMetadata = {
            ...existingPaginationMetadata,
            [startCursor]: previousPageOffset + 1,
        };
    }
    // Default to a zero offset if we're not a result of a paged request
    const startOffset = isPagedResponse && existingPaginationMetadata !== undefined
        ? offsetFromToken(existingPaginationMetadata, startCursor)
        : 0;
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0) {
            throw new RangeError(`Cursor value ${startCursor} is not recognized. Cannot ingest the returned page. After: ${afterArgVal}`);
        }
    }
    const nextOffset = startOffset + edges.length;
    const incomingPaginationMetadata = createPaginationMetadata(startOffset, edges);
    if (hasNextPage) {
        // we know the data extends beyond NEXT_OFFSET_IDENTIFIER; clean up any old __END__ markers that say the data ended at/before that offset
        if (existingPaginationMetadata !== undefined && existingPaginationMetadata.__END__ !== undefined && existingPaginationMetadata.__END__ <= nextOffset) {
            existingPaginationMetadata = { ...existingPaginationMetadata };
            delete existingPaginationMetadata.__END__;
        }
    }
    else {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    // store the updated pagination metadata
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    luvio.storePublish(paginationKey, mergedPaginationMetadata);
    luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    // merge the edges
    assignMetadataLink(sink, paginationKey);
    const existingRecord = existingData ? existingData.data : undefined;
    if (existingRecord !== undefined && existingRecord.edges !== undefined) {
        sink.edges = [
            ...existingRecord.edges.slice(0, startOffset),
            ...sink.edges,
            ...existingRecord.edges.slice(nextOffset),
        ];
    }
}
function ingest$11(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$14(luvio, path);
    return ingestCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$T,
        isMappedFieldImport: isMappedField$S,
        getFieldType: getFieldType$17,
        ingestFieldByType: ingestFieldByType$S,
        hasGraphQLPaginationFields: hasGraphQLPaginationFields,
        ingestPaginatedMetadata: ingestPaginationMetadata$4,
        createLink: createLink,
        mergeData: mergeData$S,
        storeMetadataParams: {
            ttl: TTL$a,
            namespace: keyPrefix,
            representationName: "RecordConnection",
            version: VERSION$15,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$S(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$17(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$S(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordEdge': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$16, select$13, false);
            break;
        }
        case 'PageInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$18, select$15, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$T(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, true, getFieldData$S, getFieldType$17, selectType$S, parentRecordId);
}
function buildSelectionForNode$T(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$T, buildSelectionForField$T, parentRecordId);
}
function select$12(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$T, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$19() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        ...getCursorConnectionMinimumSelections()
    ];
    return minimumSelections;
}
function getFieldType$17(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'RecordEdge'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageResultCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$S(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordEdge': {
            const RecordEdgeState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$12(requestedField, RecordEdgeState);
            break;
        }
        case 'PageInfo': {
            const PageInfoState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$14(requestedField, PageInfoState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$S(field) {
    return false;
}
function isFragmentApplicable$T(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordConnection", typename);
}
function getInContextFragmentType$18(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getFieldType$16(field) {
    const fieldNameOverride = getFieldNameOverride(field);
    const fieldName = fieldNameOverride || field.name.value;
    switch (fieldName) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'recordQuery': {
            return {
                isArray: false,
                typename: 'RecordConnection',
            };
        }
        default:
            return undefined;
    }
}

const name$R = 'RecordQuery';
const VERSION$14 = 'd70ec31adc7f4433b8c72751ce23f70e';
function keyBuilder$13(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$R(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$10(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$13(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$S,
        isMappedFieldImport: isMappedField$R,
        getFieldType: getFieldType$16,
        ingestFieldByType: ingestFieldByType$R,
        createLink: createLink,
        mergeData: mergeData$R,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordQuery",
            version: VERSION$14,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$R(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$16(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$R(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$15, select$12, true);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$S(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$R, getFieldType$16, selectType$R, parentRecordId);
}
function buildSelectionForNode$S(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$S, buildSelectionForField$S, parentRecordId);
}
function select$11(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$S, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$18() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$11(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$13(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$R
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$S) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$16(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordConnection': {
                const recordQueryState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$14(cacheKeySink, requestedField, recordQueryState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function ingestFieldByType$R(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordConnection': {
            const RecordConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$11(requestedField, RecordConnectionState);
            break;
        }
    }
}
function isMappedField$R(field) {
    return false;
}
function isFragmentApplicable$S(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordQuery", typename);
}
function getInContextFragmentType$17(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$Q = 'BooleanAggregate';
const VERSION$13 = 'a96004b34f561f154adc6a319a403141';
function keyBuilder$12(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$Q(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$$(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$12(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$R,
        isMappedFieldImport: isMappedField$Q,
        getFieldType: getFieldType$15,
        ingestFieldByType: ingestFieldByType$Q,
        createLink: createLink,
        mergeData: mergeData$Q,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "BooleanAggregate",
            version: VERSION$13,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$Q(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$15(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$Q(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$R(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$Q, getFieldType$15, selectType$Q, parentRecordId);
}
function buildSelectionForNode$R(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$R, buildSelectionForField$R, parentRecordId);
}
function select$10(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$R, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$17() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$10(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$12(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$Q
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$R) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$15(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$15(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$Q(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
    }
}
function isMappedField$Q(field) {
    return false;
}
function isFragmentApplicable$R(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "BooleanAggregate", typename);
}
function getInContextFragmentType$16(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$P = 'CurrencyAggregate';
const VERSION$12 = '82bb7a8eff45156d0ba4c1a51c17b9cc';
function keyBuilder$11(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$P(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$_(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$11(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$Q,
        isMappedFieldImport: isMappedField$P,
        getFieldType: getFieldType$14,
        ingestFieldByType: ingestFieldByType$P,
        createLink: createLink,
        mergeData: mergeData$P,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "CurrencyAggregate",
            version: VERSION$12,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$P(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$14(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$P(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Currency': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'CurrencyValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1h, select$1e, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$Q(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$P, getFieldType$14, selectType$P, parentRecordId);
}
function buildSelectionForNode$Q(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$Q, buildSelectionForField$Q, parentRecordId);
}
function select$$(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$Q, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$16() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$$(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$11(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$P
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$Q) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$14(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'CurrencyValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1e(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'CurrencyValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1e(cacheKeySink, requestedField, minState);
                break;
            }
            case 'CurrencyValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1e(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$14(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Currency'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'CurrencyValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'CurrencyValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'CurrencyValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$P(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Currency': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'CurrencyValue': {
            const CurrencyValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1d(requestedField, CurrencyValueState);
            break;
        }
    }
}
function isMappedField$P(field) {
    return false;
}
function isFragmentApplicable$Q(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "CurrencyAggregate", typename);
}
function getInContextFragmentType$15(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$O = 'DateFunctionAggregation';
const VERSION$11 = '2c6f28f604b988d12fd3fc208129d978';
function keyBuilder$10(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$O(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$Z(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$10(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$P,
        isMappedFieldImport: isMappedField$O,
        getFieldType: getFieldType$13,
        ingestFieldByType: ingestFieldByType$O,
        createLink: createLink,
        mergeData: mergeData$O,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DateFunctionAggregation",
            version: VERSION$11,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$O(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$13(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$O(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Long': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$P(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$O, getFieldType$13, selectType$O, parentRecordId);
}
function buildSelectionForNode$P(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$P, buildSelectionForField$P, parentRecordId);
}
function select$_(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$P, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$15() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$_(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$10(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$O
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$13(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$O(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Long': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$O(field) {
    return false;
}
function isFragmentApplicable$P(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "DateFunctionAggregation", typename);
}
function getInContextFragmentType$14(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$N = 'DateAggregate';
const VERSION$10 = '68225d7e1cf6bfa91f2ed998c9433705';
function keyBuilder$$(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$N(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$Y(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$$(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$O,
        isMappedFieldImport: isMappedField$N,
        getFieldType: getFieldType$12,
        ingestFieldByType: ingestFieldByType$N,
        createLink: createLink,
        mergeData: mergeData$N,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DateAggregate",
            version: VERSION$10,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$N(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$12(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$N(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Date': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DateFunctionAggregation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$11, select$_, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'DateValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1p, select$1m, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$O(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$N, getFieldType$12, selectType$N, parentRecordId);
}
function buildSelectionForNode$O(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$O, buildSelectionForField$O, parentRecordId);
}
function select$Z(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$O, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$14() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$Z(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$$(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$N
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$O) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$12(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DateFunctionAggregation': {
                const calendarMonthState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, calendarMonthState);
                break;
            }
            case 'DateFunctionAggregation': {
                const calendarQuarterState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, calendarQuarterState);
                break;
            }
            case 'DateFunctionAggregation': {
                const calendarYearState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, calendarYearState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'DateFunctionAggregation': {
                const dayInMonthState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, dayInMonthState);
                break;
            }
            case 'DateFunctionAggregation': {
                const dayInWeekState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, dayInWeekState);
                break;
            }
            case 'DateFunctionAggregation': {
                const dayInYearState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, dayInYearState);
                break;
            }
            case 'DateFunctionAggregation': {
                const fiscalMonthState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, fiscalMonthState);
                break;
            }
            case 'DateFunctionAggregation': {
                const fiscalQuarterState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, fiscalQuarterState);
                break;
            }
            case 'DateFunctionAggregation': {
                const fiscalYearState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, fiscalYearState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'DateValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1m(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'DateValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1m(cacheKeySink, requestedField, minState);
                break;
            }
            case 'DateFunctionAggregation': {
                const weekInMonthState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, weekInMonthState);
                break;
            }
            case 'DateFunctionAggregation': {
                const weekInYearState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, weekInYearState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$12(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Date'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'calendarMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'calendarQuarter': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'calendarYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'dayInMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'dayInWeek': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'dayInYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'fiscalMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'fiscalQuarter': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'fiscalYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'DateValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'DateValue'
            };
        }
        case 'weekInMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'weekInYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$N(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Date': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DateFunctionAggregation': {
            const DateFunctionAggregationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$Z(requestedField, DateFunctionAggregationState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'DateValue': {
            const DateValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1l(requestedField, DateValueState);
            break;
        }
    }
}
function isMappedField$N(field) {
    return false;
}
function isFragmentApplicable$O(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "DateAggregate", typename);
}
function getInContextFragmentType$13(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$M = 'DoubleAggregate';
const VERSION$$ = '9eaf724d0ea753527e8dda845d53b97c';
function keyBuilder$_(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$M(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$X(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$_(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$N,
        isMappedFieldImport: isMappedField$M,
        getFieldType: getFieldType$11,
        ingestFieldByType: ingestFieldByType$M,
        createLink: createLink,
        mergeData: mergeData$M,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DoubleAggregate",
            version: VERSION$$,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$M(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$11(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$M(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Double': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$N(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$M, getFieldType$11, selectType$M, parentRecordId);
}
function buildSelectionForNode$N(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$N, buildSelectionForField$N, parentRecordId);
}
function select$Y(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$N, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$13() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$Y(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$_(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$M
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$N) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$11(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'DoubleValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'DoubleValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, minState);
                break;
            }
            case 'DoubleValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$11(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Double'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$M(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Double': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
    }
}
function isMappedField$M(field) {
    return false;
}
function isFragmentApplicable$N(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "DoubleAggregate", typename);
}
function getInContextFragmentType$12(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$L = 'EmailAggregate';
const VERSION$_ = '2fd9198eb04453fc55c73879b3690167';
function keyBuilder$Z(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$L(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$W(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$Z(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$M,
        isMappedFieldImport: isMappedField$L,
        getFieldType: getFieldType$10,
        ingestFieldByType: ingestFieldByType$L,
        createLink: createLink,
        mergeData: mergeData$L,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "EmailAggregate",
            version: VERSION$_,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$L(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$10(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$L(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Email': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'EmailValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1k, select$1h, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$M(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$L, getFieldType$10, selectType$L, parentRecordId);
}
function buildSelectionForNode$M(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$M, buildSelectionForField$M, parentRecordId);
}
function select$X(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$M, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$12() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$X(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$Z(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$L
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$M) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$10(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'EmailValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1h(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'EmailValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1h(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$10(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Email'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'EmailValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'EmailValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$L(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Email': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'EmailValue': {
            const EmailValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1g(requestedField, EmailValueState);
            break;
        }
    }
}
function isMappedField$L(field) {
    return false;
}
function isFragmentApplicable$M(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "EmailAggregate", typename);
}
function getInContextFragmentType$11(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$K = 'IDAggregate';
const VERSION$Z = '04d4a6d94a5e17c6259f9bf6f8429b01';
function keyBuilder$Y(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$K(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$V(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$Y(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$L,
        isMappedFieldImport: isMappedField$K,
        getFieldType: getFieldType$$,
        ingestFieldByType: ingestFieldByType$K,
        createLink: createLink,
        mergeData: mergeData$K,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "IDAggregate",
            version: VERSION$Z,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$K(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$$(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$K(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'IDValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1t, select$1q, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$L(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$K, getFieldType$$, selectType$K, parentRecordId);
}
function buildSelectionForNode$L(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$L, buildSelectionForField$L, parentRecordId);
}
function select$W(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$L, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$11() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$W(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$Y(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$K
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$L) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$$(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'IDValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'IDValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$$(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$K(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'IDValue': {
            const IDValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1p(requestedField, IDValueState);
            break;
        }
    }
}
function isMappedField$K(field) {
    return false;
}
function isFragmentApplicable$L(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "IDAggregate", typename);
}
function getInContextFragmentType$10(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$J = 'IntAggregate';
const VERSION$Y = '2c4f51740c702f1652c59934464a1df8';
function keyBuilder$X(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$J(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$U(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$X(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$K,
        isMappedFieldImport: isMappedField$J,
        getFieldType: getFieldType$_,
        ingestFieldByType: ingestFieldByType$J,
        createLink: createLink,
        mergeData: mergeData$J,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "IntAggregate",
            version: VERSION$Y,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$J(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$_(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$J(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$K(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$J, getFieldType$_, selectType$J, parentRecordId);
}
function buildSelectionForNode$K(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$K, buildSelectionForField$K, parentRecordId);
}
function select$V(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$K, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$10() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$V(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$X(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$J
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$K) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$_(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'IntValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'IntValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, minState);
                break;
            }
            case 'LongValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$_(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$J(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
    }
}
function isMappedField$J(field) {
    return false;
}
function isFragmentApplicable$K(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "IntAggregate", typename);
}
function getInContextFragmentType$$(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$I = 'LatitudeAggregate';
const VERSION$X = '878c5527afbaf070dd32e3a728cc6763';
function keyBuilder$W(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$I(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$T(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$W(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$J,
        isMappedFieldImport: isMappedField$I,
        getFieldType: getFieldType$Z,
        ingestFieldByType: ingestFieldByType$I,
        createLink: createLink,
        mergeData: mergeData$I,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LatitudeAggregate",
            version: VERSION$X,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$I(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$Z(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$I(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Latitude': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'LatitudeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1f, select$1c, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$J(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$I, getFieldType$Z, selectType$I, parentRecordId);
}
function buildSelectionForNode$J(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$J, buildSelectionForField$J, parentRecordId);
}
function select$U(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$J, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$$() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$U(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$W(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$I
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$J) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$Z(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'LatitudeValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1c(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'LatitudeValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1c(cacheKeySink, requestedField, minState);
                break;
            }
            case 'DoubleValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$Z(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Latitude'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'LatitudeValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'LatitudeValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$I(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Latitude': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'LatitudeValue': {
            const LatitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1b(requestedField, LatitudeValueState);
            break;
        }
    }
}
function isMappedField$I(field) {
    return false;
}
function isFragmentApplicable$J(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LatitudeAggregate", typename);
}
function getInContextFragmentType$_(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$H = 'LongitudeAggregate';
const VERSION$W = 'cfdbd03df0a9ea2e17eb1e0d76e264ec';
function keyBuilder$V(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$H(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$S(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$V(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$I,
        isMappedFieldImport: isMappedField$H,
        getFieldType: getFieldType$Y,
        ingestFieldByType: ingestFieldByType$H,
        createLink: createLink,
        mergeData: mergeData$H,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LongitudeAggregate",
            version: VERSION$W,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$H(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$Y(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$H(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Longitude': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'LongitudeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1g, select$1d, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$I(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$H, getFieldType$Y, selectType$H, parentRecordId);
}
function buildSelectionForNode$I(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$I, buildSelectionForField$I, parentRecordId);
}
function select$T(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$I, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$_() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$T(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$V(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$H
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$I) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$Y(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'LongitudeValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1d(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'LongitudeValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1d(cacheKeySink, requestedField, minState);
                break;
            }
            case 'DoubleValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$Y(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Longitude'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'LongitudeValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'LongitudeValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$H(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Longitude': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'LongitudeValue': {
            const LongitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1c(requestedField, LongitudeValueState);
            break;
        }
    }
}
function isMappedField$H(field) {
    return false;
}
function isFragmentApplicable$I(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LongitudeAggregate", typename);
}
function getInContextFragmentType$Z(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$G = 'LongAggregate';
const VERSION$V = 'bf2f24661739e3152f29a37117900a46';
function keyBuilder$U(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$G(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$R(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$U(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$H,
        isMappedFieldImport: isMappedField$G,
        getFieldType: getFieldType$X,
        ingestFieldByType: ingestFieldByType$G,
        createLink: createLink,
        mergeData: mergeData$G,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "LongAggregate",
            version: VERSION$V,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$G(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$X(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$G(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Long': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$H(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$G, getFieldType$X, selectType$G, parentRecordId);
}
function buildSelectionForNode$H(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$H, buildSelectionForField$H, parentRecordId);
}
function select$S(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$H, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$Z() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$S(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$U(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$G
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$H) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$X(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DoubleValue': {
                const avgState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, avgState);
                break;
            }
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'LongValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'LongValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, minState);
                break;
            }
            case 'LongValue': {
                const sumState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, sumState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$X(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'avg': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'sum': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$G(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Long': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
    }
}
function isMappedField$G(field) {
    return false;
}
function isFragmentApplicable$H(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "LongAggregate", typename);
}
function getInContextFragmentType$Y(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$F = 'PhoneNumberAggregate';
const VERSION$U = '76e66d2f1d15218325d6a0601056ac0d';
function keyBuilder$T(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$F(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$Q(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$T(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$G,
        isMappedFieldImport: isMappedField$F,
        getFieldType: getFieldType$W,
        ingestFieldByType: ingestFieldByType$F,
        createLink: createLink,
        mergeData: mergeData$F,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PhoneNumberAggregate",
            version: VERSION$U,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$F(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$W(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$F(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'PhoneNumber': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'PhoneNumberValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1l, select$1i, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$G(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$F, getFieldType$W, selectType$F, parentRecordId);
}
function buildSelectionForNode$G(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$G, buildSelectionForField$G, parentRecordId);
}
function select$R(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$G, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$Y() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$R(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$T(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$F
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$G) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$W(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'PhoneNumberValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1i(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'PhoneNumberValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1i(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$W(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'PhoneNumber'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'PhoneNumberValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'PhoneNumberValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$F(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'PhoneNumber': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'PhoneNumberValue': {
            const PhoneNumberValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1h(requestedField, PhoneNumberValueState);
            break;
        }
    }
}
function isMappedField$F(field) {
    return false;
}
function isFragmentApplicable$G(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "PhoneNumberAggregate", typename);
}
function getInContextFragmentType$X(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$E = 'PicklistAggregate';
const VERSION$T = '6827470c6c52cedec0dfda19734a6b79';
function keyBuilder$S(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$E(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$P(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$S(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$F,
        isMappedFieldImport: isMappedField$E,
        getFieldType: getFieldType$V,
        ingestFieldByType: ingestFieldByType$E,
        createLink: createLink,
        mergeData: mergeData$E,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "PicklistAggregate",
            version: VERSION$T,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$E(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$V(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$E(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Picklist': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'PicklistValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1e, select$1b, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$F(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$E, getFieldType$V, selectType$E, parentRecordId);
}
function buildSelectionForNode$F(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$F, buildSelectionForField$F, parentRecordId);
}
function select$Q(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$F, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$X() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$Q(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$S(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$E
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$F) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$V(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'PicklistValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1b(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'PicklistValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1b(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$V(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Picklist'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'PicklistValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'PicklistValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$E(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Picklist': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'PicklistValue': {
            const PicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1a(requestedField, PicklistValueState);
            break;
        }
    }
}
function isMappedField$E(field) {
    return false;
}
function isFragmentApplicable$F(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "PicklistAggregate", typename);
}
function getInContextFragmentType$W(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$D = 'TextAreaAggregate';
const VERSION$S = 'a384df278f789db87864f6345adb134a';
function keyBuilder$R(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$D(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$O(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$R(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$E,
        isMappedFieldImport: isMappedField$D,
        getFieldType: getFieldType$U,
        ingestFieldByType: ingestFieldByType$D,
        createLink: createLink,
        mergeData: mergeData$D,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "TextAreaAggregate",
            version: VERSION$S,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$D(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$U(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$D(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'TextArea': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'TextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1o, select$1l, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$E(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$D, getFieldType$U, selectType$D, parentRecordId);
}
function buildSelectionForNode$E(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$E, buildSelectionForField$E, parentRecordId);
}
function select$P(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$E, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$W() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$P(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$R(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$D
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$E) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$U(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'TextAreaValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1l(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'TextAreaValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1l(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$U(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'TextArea'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'TextAreaValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'TextAreaValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$D(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'TextArea': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'TextAreaValue': {
            const TextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1k(requestedField, TextAreaValueState);
            break;
        }
    }
}
function isMappedField$D(field) {
    return false;
}
function isFragmentApplicable$E(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "TextAreaAggregate", typename);
}
function getInContextFragmentType$V(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$C = 'TimeAggregate';
const VERSION$R = '4b7aa92a66dc688b7e2e15c3976dd8ff';
function keyBuilder$Q(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$C(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$N(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$Q(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$D,
        isMappedFieldImport: isMappedField$C,
        getFieldType: getFieldType$T,
        ingestFieldByType: ingestFieldByType$C,
        createLink: createLink,
        mergeData: mergeData$C,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "TimeAggregate",
            version: VERSION$R,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$C(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$T(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$C(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Time': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DateFunctionAggregation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$11, select$_, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$D(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$C, getFieldType$T, selectType$C, parentRecordId);
}
function buildSelectionForNode$D(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$D, buildSelectionForField$D, parentRecordId);
}
function select$O(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$D, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$V() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$O(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$Q(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$C
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$D) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$T(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'DateFunctionAggregation': {
                const hourInDayState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$_(cacheKeySink, requestedField, hourInDayState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$T(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Time'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'hourInDay': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$C(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Time': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DateFunctionAggregation': {
            const DateFunctionAggregationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$Z(requestedField, DateFunctionAggregationState);
            break;
        }
    }
}
function isMappedField$C(field) {
    return false;
}
function isFragmentApplicable$D(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "TimeAggregate", typename);
}
function getInContextFragmentType$U(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$B = 'UrlAggregate';
const VERSION$Q = 'a20a03966540b9046db5f606e20e3c8a';
function keyBuilder$P(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$B(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$M(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$P(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$C,
        isMappedFieldImport: isMappedField$B,
        getFieldType: getFieldType$S,
        ingestFieldByType: ingestFieldByType$B,
        createLink: createLink,
        mergeData: mergeData$B,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "UrlAggregate",
            version: VERSION$Q,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$B(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$S(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$B(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Url': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'UrlValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1j, select$1g, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$C(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$B, getFieldType$S, selectType$B, parentRecordId);
}
function buildSelectionForNode$C(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$C, buildSelectionForField$C, parentRecordId);
}
function select$N(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$C, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$U() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$N(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$P(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$B
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$C) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$S(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'LongValue': {
                const countState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countState);
                break;
            }
            case 'LongValue': {
                const countDistinctState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, countDistinctState);
                break;
            }
            case 'IntValue': {
                const groupingState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, groupingState);
                break;
            }
            case 'UrlValue': {
                const maxState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1g(cacheKeySink, requestedField, maxState);
                break;
            }
            case 'UrlValue': {
                const minState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1g(cacheKeySink, requestedField, minState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$S(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Url'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'grouping': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'UrlValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'UrlValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$B(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Url': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'UrlValue': {
            const UrlValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1f(requestedField, UrlValueState);
            break;
        }
    }
}
function isMappedField$B(field) {
    return false;
}
function isFragmentApplicable$C(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["FieldValue"], "UrlAggregate", typename);
}
function getInContextFragmentType$T(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

// union versions are undefined, selected versions must be specific to the type being selected
const VERSION$P = undefined;
function ingest$L(astNode, state) {
    const typename = state.data.__typename;
    // Delegate key building and ingest to concrete implementations
    if (typename === 'RecordAggregate') {
        return ingest$K(astNode, state);
    }
}
function select$M(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const typename = source.data.__typename;
        if (typename === 'RecordAggregate') {
            return select$L(field, variables, fragments)(source, reader, parentRecordId);
        }
    };
}
function getTypeCacheKeys$M(cacheKeys, astNode, state) {
    const { path, data } = state;
    const typename = data.__typename;
    if (typename === 'RecordAggregate') {
        return getTypeCacheKeys$L(cacheKeys, astNode, state);
    }
}
function getInContextFragmentType$S(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}
function getFieldType$R(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getMinimumSelections$T() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}

function getFieldType$Q(field) {
    const fieldNameOverride = getFieldNameOverride(field);
    const fieldName = fieldNameOverride || field.name.value;
    switch (fieldName) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'BooleanAggregate': {
            return {
                isArray: false,
                typename: 'BooleanAggregate',
            };
        }
        case 'CurrencyAggregate': {
            return {
                isArray: false,
                typename: 'CurrencyAggregate',
            };
        }
        case 'DateAggregate': {
            return {
                isArray: false,
                typename: 'DateAggregate',
            };
        }
        case 'DoubleAggregate': {
            return {
                isArray: false,
                typename: 'DoubleAggregate',
            };
        }
        case 'EmailAggregate': {
            return {
                isArray: false,
                typename: 'EmailAggregate',
            };
        }
        case 'IDAggregate': {
            return {
                isArray: false,
                typename: 'IDAggregate',
            };
        }
        case 'IntAggregate': {
            return {
                isArray: false,
                typename: 'IntAggregate',
            };
        }
        case 'LatitudeAggregate': {
            return {
                isArray: false,
                typename: 'LatitudeAggregate',
            };
        }
        case 'LongitudeAggregate': {
            return {
                isArray: false,
                typename: 'LongitudeAggregate',
            };
        }
        case 'LongAggregate': {
            return {
                isArray: false,
                typename: 'LongAggregate',
            };
        }
        case 'PercentAggregate': {
            return {
                isArray: false,
                typename: 'PercentAggregate',
            };
        }
        case 'PhoneNumberAggregate': {
            return {
                isArray: false,
                typename: 'PhoneNumberAggregate',
            };
        }
        case 'PicklistAggregate': {
            return {
                isArray: false,
                typename: 'PicklistAggregate',
            };
        }
        case 'StringAggregate': {
            return {
                isArray: false,
                typename: 'StringAggregate',
            };
        }
        case 'TextAreaAggregate': {
            return {
                isArray: false,
                typename: 'TextAreaAggregate',
            };
        }
        case 'TimeAggregate': {
            return {
                isArray: false,
                typename: 'TimeAggregate',
            };
        }
        case 'UrlAggregate': {
            return {
                isArray: false,
                typename: 'UrlAggregate',
            };
        }
        default:
            return undefined;
    }
}

const name$A = 'RecordAggregate';
const VERSION$O = 'e7f09bcfe3bdaa58b46bbe4d724d0231';
function keyBuilder$O(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$A(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$K(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$O(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$B,
        isMappedFieldImport: isMappedField$A,
        getFieldType: getFieldType$Q,
        ingestFieldByType: ingestFieldByType$A,
        createLink: createLink,
        mergeData: mergeData$A,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordAggregate",
            version: VERSION$O,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$A(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$Q(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$A(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'BooleanAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$13, select$10, false);
            break;
        }
        case 'CurrencyAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$12, select$$, false);
            break;
        }
        case 'DateAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$10, select$Z, false);
            break;
        }
        case 'DoubleAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$$, select$Y, false);
            break;
        }
        case 'EmailAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$_, select$X, false);
            break;
        }
        case 'IDAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$Z, select$W, false);
            break;
        }
        case 'IntAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$Y, select$V, false);
            break;
        }
        case 'LatitudeAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$X, select$U, false);
            break;
        }
        case 'LongitudeAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$W, select$T, false);
            break;
        }
        case 'LongAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$V, select$S, false);
            break;
        }
        case 'PercentAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1x, select$1u, false);
            break;
        }
        case 'PhoneNumberAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$U, select$R, false);
            break;
        }
        case 'PicklistAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$T, select$Q, false);
            break;
        }
        case 'StringAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1u, select$1r, false);
            break;
        }
        case 'TextAreaAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$S, select$P, false);
            break;
        }
        case 'TimeAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$R, select$O, false);
            break;
        }
        case 'UrlAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$Q, select$N, false);
            break;
        }
        case 'RecordAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$O, select$L, false);
            break;
        }
        case 'PolymorphicAggregateParentRelationship': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$P, select$M, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$B(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$A, getFieldType$Q, selectType$A, parentRecordId);
}
function buildSelectionForNode$B(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$B, buildSelectionForField$B, parentRecordId);
}
function select$L(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$B, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$S() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$L(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$O(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$A
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$B) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$Q(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'BooleanAggregate': {
                const BooleanAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$10(cacheKeySink, requestedField, BooleanAggregateState);
                break;
            }
            case 'CurrencyAggregate': {
                const CurrencyAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$$(cacheKeySink, requestedField, CurrencyAggregateState);
                break;
            }
            case 'DateAggregate': {
                const DateAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$Z(cacheKeySink, requestedField, DateAggregateState);
                break;
            }
            case 'DoubleAggregate': {
                const DoubleAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$Y(cacheKeySink, requestedField, DoubleAggregateState);
                break;
            }
            case 'EmailAggregate': {
                const EmailAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$X(cacheKeySink, requestedField, EmailAggregateState);
                break;
            }
            case 'IDAggregate': {
                const IDAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$W(cacheKeySink, requestedField, IDAggregateState);
                break;
            }
            case 'IntAggregate': {
                const IntAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$V(cacheKeySink, requestedField, IntAggregateState);
                break;
            }
            case 'LatitudeAggregate': {
                const LatitudeAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$U(cacheKeySink, requestedField, LatitudeAggregateState);
                break;
            }
            case 'LongitudeAggregate': {
                const LongitudeAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$T(cacheKeySink, requestedField, LongitudeAggregateState);
                break;
            }
            case 'LongAggregate': {
                const LongAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$S(cacheKeySink, requestedField, LongAggregateState);
                break;
            }
            case 'PercentAggregate': {
                const PercentAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1u(cacheKeySink, requestedField, PercentAggregateState);
                break;
            }
            case 'PhoneNumberAggregate': {
                const PhoneNumberAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$R(cacheKeySink, requestedField, PhoneNumberAggregateState);
                break;
            }
            case 'PicklistAggregate': {
                const PicklistAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$Q(cacheKeySink, requestedField, PicklistAggregateState);
                break;
            }
            case 'StringAggregate': {
                const StringAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1r(cacheKeySink, requestedField, StringAggregateState);
                break;
            }
            case 'TextAreaAggregate': {
                const TextAreaAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$P(cacheKeySink, requestedField, TextAreaAggregateState);
                break;
            }
            case 'TimeAggregate': {
                const TimeAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$O(cacheKeySink, requestedField, TimeAggregateState);
                break;
            }
            case 'UrlAggregate': {
                const UrlAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$N(cacheKeySink, requestedField, UrlAggregateState);
                break;
            }
            case 'RecordAggregate': {
                const parentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$L(cacheKeySink, requestedField, parentRelationshipState);
                break;
            }
            case 'PolymorphicAggregateParentRelationship': {
                const polymorphicParentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$M(cacheKeySink, requestedField, polymorphicParentRelationshipState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function ingestFieldByType$A(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'BooleanAggregate': {
            const BooleanAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$$(requestedField, BooleanAggregateState);
            break;
        }
        case 'CurrencyAggregate': {
            const CurrencyAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$_(requestedField, CurrencyAggregateState);
            break;
        }
        case 'DateAggregate': {
            const DateAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$Y(requestedField, DateAggregateState);
            break;
        }
        case 'DoubleAggregate': {
            const DoubleAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$X(requestedField, DoubleAggregateState);
            break;
        }
        case 'EmailAggregate': {
            const EmailAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$W(requestedField, EmailAggregateState);
            break;
        }
        case 'IDAggregate': {
            const IDAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$V(requestedField, IDAggregateState);
            break;
        }
        case 'IntAggregate': {
            const IntAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$U(requestedField, IntAggregateState);
            break;
        }
        case 'LatitudeAggregate': {
            const LatitudeAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$T(requestedField, LatitudeAggregateState);
            break;
        }
        case 'LongitudeAggregate': {
            const LongitudeAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$S(requestedField, LongitudeAggregateState);
            break;
        }
        case 'LongAggregate': {
            const LongAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$R(requestedField, LongAggregateState);
            break;
        }
        case 'PercentAggregate': {
            const PercentAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1t(requestedField, PercentAggregateState);
            break;
        }
        case 'PhoneNumberAggregate': {
            const PhoneNumberAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$Q(requestedField, PhoneNumberAggregateState);
            break;
        }
        case 'PicklistAggregate': {
            const PicklistAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$P(requestedField, PicklistAggregateState);
            break;
        }
        case 'StringAggregate': {
            const StringAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1q(requestedField, StringAggregateState);
            break;
        }
        case 'TextAreaAggregate': {
            const TextAreaAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$O(requestedField, TextAreaAggregateState);
            break;
        }
        case 'TimeAggregate': {
            const TimeAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$N(requestedField, TimeAggregateState);
            break;
        }
        case 'UrlAggregate': {
            const UrlAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$M(requestedField, UrlAggregateState);
            break;
        }
        case 'RecordAggregate': {
            const RecordAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$K(requestedField, RecordAggregateState);
            break;
        }
        case 'PolymorphicAggregateParentRelationship': {
            const PolymorphicAggregateParentRelationshipState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$L(requestedField, PolymorphicAggregateParentRelationshipState);
            break;
        }
    }
}
function isMappedField$A(field) {
    return false;
}
function isFragmentApplicable$B(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordAggregate", typename);
}
function getInContextFragmentType$R(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$z = 'RecordResult';
const VERSION$N = 'ac087a39402cb113c9336a78e228b27a';
function keyBuilder$N(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$z(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$J(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$N(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$A,
        isMappedFieldImport: isMappedField$z,
        getFieldType: getFieldType$P,
        ingestFieldByType: ingestFieldByType$z,
        createLink: createLink,
        mergeData: mergeData$z,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordResult",
            version: VERSION$N,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$z(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$P(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$z(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$O, select$L, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$A(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$z, getFieldType$P, selectType$z, parentRecordId);
}
function buildSelectionForNode$A(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$A, buildSelectionForField$A, parentRecordId);
}
function select$K(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$A, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$R() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$K(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$N(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$z
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$A) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$P(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordAggregate': {
                const aggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$L(cacheKeySink, requestedField, aggregateState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$P(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'aggregate': {
            return {
                isArray: false,
                typename: 'RecordAggregate'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$z(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordAggregate': {
            const RecordAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$K(requestedField, RecordAggregateState);
            break;
        }
    }
}
function isMappedField$z(field) {
    return false;
}
function isFragmentApplicable$A(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordResult", typename);
}
function getInContextFragmentType$Q(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$y = 'RecordAggregateEdge';
const VERSION$M = '109073e1a791313cf074cd8b4d78a64d';
function keyBuilder$M(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$y(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$I(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$M(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$z,
        isMappedFieldImport: isMappedField$y,
        getFieldType: getFieldType$O,
        ingestFieldByType: ingestFieldByType$y,
        createLink: createLink,
        mergeData: mergeData$y,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordAggregateEdge",
            version: VERSION$M,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$y(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$O(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$y(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordResult': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$N, select$K, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$z(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$y, getFieldType$O, selectType$y, parentRecordId);
}
function buildSelectionForNode$z(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$z, buildSelectionForField$z, parentRecordId);
}
function select$J(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$z, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$Q() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$J(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$M(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$y
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$z) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$O(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordResult': {
                const nodeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$K(cacheKeySink, requestedField, nodeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$O(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'RecordResult'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$y(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordResult': {
            const RecordResultState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$J(requestedField, RecordResultState);
            break;
        }
    }
}
function isMappedField$y(field) {
    return false;
}
function isFragmentApplicable$z(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordAggregateEdge", typename);
}
function getInContextFragmentType$P(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$x = 'RecordAggregateConnection';
const VERSION$L = '75a36f5de3df006f439ccc8e04bdca0f';
function keyBuilder$L(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$x(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingestPaginationMetadata$3(astNode, state, key, sink, existingData) {
    var _a;
    const { data, luvio, store } = state;
    const { pageInfo: { hasNextPage, startCursor }, edges } = data;
    if (!hasGraphQLPaginationFields(data))
        return;
    const paginationKey = key + '__pagination';
    let existingPaginationMetadata = store.readEntry(paginationKey);
    const afterArg = (_a = astNode.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', state.variables);
    const isPagedResponse = afterArgVal !== undefined &&
        existingPaginationMetadata !== undefined &&
        existingPaginationMetadata[afterArgVal] !== undefined;
    if (isPagedResponse && existingPaginationMetadata !== undefined) {
        const previousPageOffset = existingPaginationMetadata[afterArgVal];
        existingPaginationMetadata = {
            ...existingPaginationMetadata,
            [startCursor]: previousPageOffset + 1,
        };
    }
    // Default to a zero offset if we're not a result of a paged request
    const startOffset = isPagedResponse && existingPaginationMetadata !== undefined
        ? offsetFromToken(existingPaginationMetadata, startCursor)
        : 0;
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0) {
            throw new RangeError(`Cursor value ${startCursor} is not recognized. Cannot ingest the returned page. After: ${afterArgVal}`);
        }
    }
    const nextOffset = startOffset + edges.length;
    const incomingPaginationMetadata = createPaginationMetadata(startOffset, edges);
    if (hasNextPage) {
        // we know the data extends beyond NEXT_OFFSET_IDENTIFIER; clean up any old __END__ markers that say the data ended at/before that offset
        if (existingPaginationMetadata !== undefined && existingPaginationMetadata.__END__ !== undefined && existingPaginationMetadata.__END__ <= nextOffset) {
            existingPaginationMetadata = { ...existingPaginationMetadata };
            delete existingPaginationMetadata.__END__;
        }
    }
    else {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    // store the updated pagination metadata
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    luvio.storePublish(paginationKey, mergedPaginationMetadata);
    luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    // merge the edges
    assignMetadataLink(sink, paginationKey);
    const existingRecord = existingData ? existingData.data : undefined;
    if (existingRecord !== undefined && existingRecord.edges !== undefined) {
        sink.edges = [
            ...existingRecord.edges.slice(0, startOffset),
            ...sink.edges,
            ...existingRecord.edges.slice(nextOffset),
        ];
    }
}
function ingest$H(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$L(luvio, path);
    return ingestCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$y,
        isMappedFieldImport: isMappedField$x,
        getFieldType: getFieldType$N,
        ingestFieldByType: ingestFieldByType$x,
        hasGraphQLPaginationFields: hasGraphQLPaginationFields,
        ingestPaginatedMetadata: ingestPaginationMetadata$3,
        createLink: createLink,
        mergeData: mergeData$x,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordAggregateConnection",
            version: VERSION$L,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$x(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$N(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$x(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordAggregateEdge': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$M, select$J, false);
            break;
        }
        case 'PageInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$18, select$15, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$y(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, true, getFieldData$x, getFieldType$N, selectType$x, parentRecordId);
}
function buildSelectionForNode$y(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$y, buildSelectionForField$y, parentRecordId);
}
function select$I(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$y, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$P() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        ...getCursorConnectionMinimumSelections()
    ];
    return minimumSelections;
}
function getTypeCacheKeys$I(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$L(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$x
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$y) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$N(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordAggregateEdge': {
                const fullPath__edges = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__edges + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$J(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'PageInfo': {
                const pageInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$15(cacheKeySink, requestedField, pageInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$N(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'RecordAggregateEdge'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$x(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordAggregateEdge': {
            const RecordAggregateEdgeState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$I(requestedField, RecordAggregateEdgeState);
            break;
        }
        case 'PageInfo': {
            const PageInfoState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$14(requestedField, PageInfoState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$x(field) {
    return false;
}
function isFragmentApplicable$y(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordAggregateConnection", typename);
}
function getInContextFragmentType$O(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getFieldType$M(field) {
    const fieldNameOverride = getFieldNameOverride(field);
    const fieldName = fieldNameOverride || field.name.value;
    switch (fieldName) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        case 'recordQueryAggregate': {
            return {
                isArray: false,
                typename: 'RecordAggregateConnection',
            };
        }
        default:
            return undefined;
    }
}

const name$w = 'RecordQueryAggregate';
const VERSION$K = 'd872f40fe358effab12441976e95ca71';
function keyBuilder$K(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$w(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$G(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$K(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$x,
        isMappedFieldImport: isMappedField$w,
        getFieldType: getFieldType$M,
        ingestFieldByType: ingestFieldByType$w,
        createLink: createLink,
        mergeData: mergeData$w,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordQueryAggregate",
            version: VERSION$K,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$w(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$M(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$w(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordAggregateConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$L, select$I, true);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$x(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$w, getFieldType$M, selectType$w, parentRecordId);
}
function buildSelectionForNode$x(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$x, buildSelectionForField$x, parentRecordId);
}
function select$H(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$x, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$O() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$H(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$K(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$w
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$x) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$M(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordAggregateConnection': {
                const recordQueryAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$I(cacheKeySink, requestedField, recordQueryAggregateState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function ingestFieldByType$w(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordAggregateConnection': {
            const RecordAggregateConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$H(requestedField, RecordAggregateConnectionState);
            break;
        }
    }
}
function isMappedField$w(field) {
    return false;
}
function isFragmentApplicable$x(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordQueryAggregate", typename);
}
function getInContextFragmentType$N(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$v = 'ChildRelationship';
const VERSION$J = '46fe6575916bfa87eabb7a722b4a9016';
function keyBuilder$J(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$v(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$F(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$J(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$w,
        isMappedFieldImport: isMappedField$v,
        getFieldType: getFieldType$L,
        ingestFieldByType: ingestFieldByType$v,
        createLink: createLink,
        mergeData: mergeData$v,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "ChildRelationship",
            version: VERSION$J,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$v(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$L(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$v(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ObjectInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$C, select$z, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$w(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$v, getFieldType$L, selectType$v, parentRecordId);
}
function buildSelectionForNode$w(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$w, buildSelectionForField$w, parentRecordId);
}
function select$G(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$w, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$N() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$G(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$J(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$v
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$w) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$L(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'ObjectInfo': {
                const objectInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$z(cacheKeySink, requestedField, objectInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$L(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'childObjectApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'fieldName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'junctionIdListNames': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        case 'junctionReferenceTo': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        case 'relationshipName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'objectInfo': {
            return {
                isArray: false,
                typename: 'ObjectInfo'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$v(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ObjectInfo': {
            const ObjectInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$y(requestedField, ObjectInfoState);
            break;
        }
    }
}
function isMappedField$v(field) {
    return false;
}
function isFragmentApplicable$w(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "ChildRelationship", typename);
}
function getInContextFragmentType$M(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$u = 'DependentField';
const VERSION$I = 'f1e7745a7b6da3e3a08e24836f72104f';
function keyBuilder$I(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$u(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$E(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$I(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$v,
        isMappedFieldImport: isMappedField$u,
        getFieldType: getFieldType$K,
        ingestFieldByType: ingestFieldByType$u,
        createLink: createLink,
        mergeData: mergeData$u,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "DependentField",
            version: VERSION$I,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$u(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$K(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$u(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$v(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$u, getFieldType$K, selectType$u, parentRecordId);
}
function buildSelectionForNode$v(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$v, buildSelectionForField$v, parentRecordId);
}
function select$F(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$v, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$M() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$F(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$I(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$u
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$K(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'controllingField': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'dependentFields': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$u(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$u(field) {
    return false;
}
function isFragmentApplicable$v(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "DependentField", typename);
}
function getInContextFragmentType$L(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$t = 'FilteredLookupInfo';
const VERSION$H = '5925c1087557ba234bfee8acff9cffb8';
function keyBuilder$H(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$t(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$D(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$H(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$u,
        isMappedFieldImport: isMappedField$t,
        getFieldType: getFieldType$J,
        ingestFieldByType: ingestFieldByType$t,
        createLink: createLink,
        mergeData: mergeData$t,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "FilteredLookupInfo",
            version: VERSION$H,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$t(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$J(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$t(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$u(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$t, getFieldType$J, selectType$t, parentRecordId);
}
function buildSelectionForNode$u(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$u, buildSelectionForField$u, parentRecordId);
}
function select$E(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$u, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$L() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$E(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$H(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$t
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$J(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'controllingFields': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        case 'dependent': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'optionalFilter': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$t(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$t(field) {
    return false;
}
function isFragmentApplicable$u(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "FilteredLookupInfo", typename);
}
function getInContextFragmentType$K(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$s = 'ReferenceToInfo';
const VERSION$G = '604954665bd0a8e8093001263b6d94ee';
function keyBuilder$G(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$s(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$C(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$G(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$t,
        isMappedFieldImport: isMappedField$s,
        getFieldType: getFieldType$I,
        ingestFieldByType: ingestFieldByType$s,
        createLink: createLink,
        mergeData: mergeData$s,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "ReferenceToInfo",
            version: VERSION$G,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$s(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$I(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$s(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ObjectInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$C, select$z, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$t(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$s, getFieldType$I, selectType$s, parentRecordId);
}
function buildSelectionForNode$t(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$t, buildSelectionForField$t, parentRecordId);
}
function select$D(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$t, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$K() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$D(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$G(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$s
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$t) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$I(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'ObjectInfo': {
                const objectInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$z(cacheKeySink, requestedField, objectInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$I(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'nameFields': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        case 'objectInfo': {
            return {
                isArray: false,
                typename: 'ObjectInfo'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$s(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ObjectInfo': {
            const ObjectInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$y(requestedField, ObjectInfoState);
            break;
        }
    }
}
function isMappedField$s(field) {
    return false;
}
function isFragmentApplicable$t(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "ReferenceToInfo", typename);
}
function getInContextFragmentType$J(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$r = 'Field';
const VERSION$F = '85f2db3fa3aa538eb507ff991c47f227';
function keyBuilder$F(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$r(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$B(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$F(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$s,
        isMappedFieldImport: isMappedField$r,
        getFieldType: getFieldType$H,
        ingestFieldByType: ingestFieldByType$r,
        createLink: createLink,
        mergeData: mergeData$r,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Field",
            version: VERSION$F,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$r(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$H(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$r(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DataType': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'FieldExtraTypeInfo': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'FilteredLookupInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$H, select$E, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ReferenceToInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$G, select$D, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$s(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$r, getFieldType$H, selectType$r, parentRecordId);
}
function buildSelectionForNode$s(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$s, buildSelectionForField$s, parentRecordId);
}
function select$C(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$s, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$J() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$C(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$F(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$r
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$s) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$H(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'FilteredLookupInfo': {
                const filteredLookupInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$E(cacheKeySink, requestedField, filteredLookupInfoState);
                break;
            }
            case 'ReferenceToInfo': {
                const fullPath__referenceToInfos = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__referenceToInfos + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$D(cacheKeySink, requestedField, itemState);
                }
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$H(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'calculated': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'compound': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'compoundComponentName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'compoundFieldName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'controllerName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'controllingFields': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        case 'createable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'custom': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'dataType': {
            return {
                isArray: false,
                typename: 'DataType'
            };
        }
        case 'extraTypeInfo': {
            return {
                isArray: false,
                typename: 'FieldExtraTypeInfo'
            };
        }
        case 'filterable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'filteredLookupInfo': {
            return {
                isArray: false,
                typename: 'FilteredLookupInfo'
            };
        }
        case 'highScaleNumber': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'htmlFormatted': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'inlineHelpText': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'nameField': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'polymorphicForeignKey': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'precision': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'reference': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'referenceTargetField': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'referenceToInfos': {
            return {
                isArray: true,
                typename: 'ReferenceToInfo'
            };
        }
        case 'relationshipName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'required': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'scale': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'searchPrefilterable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'sortable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'updateable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$r(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DataType': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'FieldExtraTypeInfo': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'FilteredLookupInfo': {
            const FilteredLookupInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$D(requestedField, FilteredLookupInfoState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ReferenceToInfo': {
            const ReferenceToInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$C(requestedField, ReferenceToInfoState);
            break;
        }
    }
}
function isMappedField$r(field) {
    return false;
}
function isFragmentApplicable$s(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Field", typename);
}
function getInContextFragmentType$I(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$q = 'RecordTypeInfo';
const VERSION$E = '4b9c8f1341c2ab9e0dddddaa0935dd09';
function keyBuilder$E(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$q(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$A(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$E(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$r,
        isMappedFieldImport: isMappedField$q,
        getFieldType: getFieldType$G,
        ingestFieldByType: ingestFieldByType$q,
        createLink: createLink,
        mergeData: mergeData$q,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "RecordTypeInfo",
            version: VERSION$E,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$q(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$G(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$q(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$r(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$q, getFieldType$G, selectType$q, parentRecordId);
}
function buildSelectionForNode$r(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$r, buildSelectionForField$r, parentRecordId);
}
function select$B(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$r, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$I() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$B(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$E(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$q
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$G(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'available': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'defaultRecordTypeMapping': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'master': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'name': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'recordTypeId': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$q(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$q(field) {
    return false;
}
function isFragmentApplicable$r(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RecordTypeInfo", typename);
}
function getInContextFragmentType$H(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$p = 'ThemeInfo';
const VERSION$D = '8d51202c3e44b6e99239b346b483a513';
function keyBuilder$D(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$p(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$z(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$D(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$q,
        isMappedFieldImport: isMappedField$p,
        getFieldType: getFieldType$F,
        ingestFieldByType: ingestFieldByType$p,
        createLink: createLink,
        mergeData: mergeData$p,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "ThemeInfo",
            version: VERSION$D,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$p(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$F(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$p(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$q(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$p, getFieldType$F, selectType$p, parentRecordId);
}
function buildSelectionForNode$q(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$q, buildSelectionForField$q, parentRecordId);
}
function select$A(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$q, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$H() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$A(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$D(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$p
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$F(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'color': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'iconUrl': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$p(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$p(field) {
    return false;
}
function isFragmentApplicable$q(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "ThemeInfo", typename);
}
function getInContextFragmentType$G(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$o = 'ObjectInfo';
const TTL$9 = 900000;
const VERSION$C = 'f8f627dd56d42a39e1a6642edb039182';
function keyBuilder$C(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$o(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$y(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$C(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$p,
        isMappedFieldImport: isMappedField$o,
        getFieldType: getFieldType$E,
        ingestFieldByType: ingestFieldByType$o,
        createLink: createLink,
        mergeData: mergeData$o,
        storeMetadataParams: {
            ttl: TTL$9,
            namespace: keyPrefix,
            representationName: "ObjectInfo",
            version: VERSION$C,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$o(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$E(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$o(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ChildRelationship': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$J, select$G, false);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'DependentField': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$I, select$F, false);
            break;
        }
        case 'Field': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$F, select$C, false);
            break;
        }
        case 'RecordTypeInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$E, select$B, false);
            break;
        }
        case 'ThemeInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$D, select$A, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$p(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$o, getFieldType$E, selectType$o, parentRecordId);
}
function buildSelectionForNode$p(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$p, buildSelectionForField$p, parentRecordId);
}
function select$z(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$p, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$G() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$z(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$C(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$o
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$p) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$E(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'ChildRelationship': {
                const fullPath__childRelationships = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__childRelationships + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$G(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'DependentField': {
                const fullPath__dependentFields = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__dependentFields + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$F(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'Field': {
                const fullPath__fields = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__fields + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$C(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'RecordTypeInfo': {
                const fullPath__recordTypeInfos = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__recordTypeInfos + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$B(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'ThemeInfo': {
                const themeInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$A(cacheKeySink, requestedField, themeInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$E(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'childRelationships': {
            return {
                isArray: true,
                typename: 'ChildRelationship'
            };
        }
        case 'createable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'custom': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'defaultRecordTypeId': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'deletable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'dependentFields': {
            return {
                isArray: true,
                typename: 'DependentField'
            };
        }
        case 'feedEnabled': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'fields': {
            return {
                isArray: true,
                typename: 'Field'
            };
        }
        case 'keyPrefix': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'labelPlural': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'layoutable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'mruEnabled': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'nameFields': {
            return {
                isArray: true,
                typename: 'String'
            };
        }
        case 'queryable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'recordTypeInfos': {
            return {
                isArray: true,
                typename: 'RecordTypeInfo'
            };
        }
        case 'searchable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'themeInfo': {
            return {
                isArray: false,
                typename: 'ThemeInfo'
            };
        }
        case 'updateable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        case 'locale': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$o(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ChildRelationship': {
            const ChildRelationshipState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$F(requestedField, ChildRelationshipState);
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'DependentField': {
            const DependentFieldState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$E(requestedField, DependentFieldState);
            break;
        }
        case 'Field': {
            const FieldState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$B(requestedField, FieldState);
            break;
        }
        case 'RecordTypeInfo': {
            const RecordTypeInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$A(requestedField, RecordTypeInfoState);
            break;
        }
        case 'ThemeInfo': {
            const ThemeInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$z(requestedField, ThemeInfoState);
            break;
        }
    }
}
function isMappedField$o(field) {
    return false;
}
function isFragmentApplicable$p(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "ObjectInfo", typename);
}
function getInContextFragmentType$F(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$n = 'ListColumn';
const VERSION$B = '8dcf4b78cc643cef31522376d8fa2ec9';
function keyBuilder$B(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$n(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$x(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$B(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$o,
        isMappedFieldImport: isMappedField$n,
        getFieldType: getFieldType$D,
        ingestFieldByType: ingestFieldByType$n,
        createLink: createLink,
        mergeData: mergeData$n,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "ListColumn",
            version: VERSION$B,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$n(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$D(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$n(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Boolean': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$o(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$n, getFieldType$D, selectType$n, parentRecordId);
}
function buildSelectionForNode$o(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$o, buildSelectionForField$o, parentRecordId);
}
function select$y(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$o, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$F() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$y(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$B(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$n
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$D(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'fieldApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'lookupId': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'sortable': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$n(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Boolean': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$n(field) {
    return false;
}
function isFragmentApplicable$o(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "ListColumn", typename);
}
function getInContextFragmentType$E(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$m = 'ListOrder';
const VERSION$A = '242dbbb2adb06b43073f9f14f401250a';
function keyBuilder$A(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$m(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$w(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$A(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$n,
        isMappedFieldImport: isMappedField$m,
        getFieldType: getFieldType$C,
        ingestFieldByType: ingestFieldByType$m,
        createLink: createLink,
        mergeData: mergeData$m,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "ListOrder",
            version: VERSION$A,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$m(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$C(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$m(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ResultOrder': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$n(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$m, getFieldType$C, selectType$m, parentRecordId);
}
function buildSelectionForNode$n(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$n, buildSelectionForField$n, parentRecordId);
}
function select$x(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$n, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$E() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$x(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$A(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$m
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$C(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'fieldApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'sortDirection': {
            return {
                isArray: false,
                typename: 'ResultOrder'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$m(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ResultOrder': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$m(field) {
    return false;
}
function isFragmentApplicable$n(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "ListOrder", typename);
}
function getInContextFragmentType$D(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$l = 'RelatedListInfo';
const TTL$8 = 900000;
const VERSION$z = 'a32cfdff7ca8dcf4b534b3491fef2019';
function keyBuilder$z(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$l(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$v(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$z(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$m,
        isMappedFieldImport: isMappedField$l,
        getFieldType: getFieldType$B,
        ingestFieldByType: ingestFieldByType$l,
        createLink: createLink,
        mergeData: mergeData$l,
        storeMetadataParams: {
            ttl: TTL$8,
            namespace: keyPrefix,
            representationName: "RelatedListInfo",
            version: VERSION$z,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$l(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$B(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$l(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ListColumn': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$B, select$y, false);
            break;
        }
        case 'ListOrder': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$A, select$x, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$m(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$l, getFieldType$B, selectType$l, parentRecordId);
}
function buildSelectionForNode$m(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$m, buildSelectionForField$m, parentRecordId);
}
function select$w(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$m, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$D() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$w(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$z(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$l
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$m) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$B(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'ListColumn': {
                const fullPath__displayColumns = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__displayColumns + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$y(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'ListOrder': {
                const fullPath__orderedByInfo = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__orderedByInfo + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$x(cacheKeySink, requestedField, itemState);
                }
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$B(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'childApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'relatedListName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'label': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'displayColumns': {
            return {
                isArray: true,
                typename: 'ListColumn'
            };
        }
        case 'orderedByInfo': {
            return {
                isArray: true,
                typename: 'ListOrder'
            };
        }
        case 'parentApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'fieldApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$l(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ListColumn': {
            const ListColumnState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$x(requestedField, ListColumnState);
            break;
        }
        case 'ListOrder': {
            const ListOrderState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$w(requestedField, ListOrderState);
            break;
        }
    }
}
function isMappedField$l(field) {
    return false;
}
function isFragmentApplicable$m(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "RelatedListInfo", typename);
}
function getInContextFragmentType$C(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$k = 'UIAPI';
const VERSION$y = '86b00be929ac510584ee034a4df812ea';
function keyBuilder$y(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$k(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$u(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$y(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$l,
        isMappedFieldImport: isMappedField$k,
        getFieldType: getFieldType$A,
        ingestFieldByType: ingestFieldByType$k,
        createLink: createLink,
        mergeData: mergeData$k,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "UIAPI",
            version: VERSION$y,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$k(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$A(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$k(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'RecordQuery': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$14, select$11, false);
            break;
        }
        case 'RecordQueryAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$K, select$H, false);
            break;
        }
        case 'ObjectInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$C, select$z, false);
            break;
        }
        case 'RelatedListInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$z, select$w, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$l(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$k, getFieldType$A, selectType$k, parentRecordId);
}
function buildSelectionForNode$l(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$l, buildSelectionForField$l, parentRecordId);
}
function select$v(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$l, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$C() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$v(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$y(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$k
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$l) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$A(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'RecordQuery': {
                const queryState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$11(cacheKeySink, requestedField, queryState);
                break;
            }
            case 'RecordQueryAggregate': {
                const aggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$H(cacheKeySink, requestedField, aggregateState);
                break;
            }
            case 'ObjectInfo': {
                const fullPath__objectInfos = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__objectInfos + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$z(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'RelatedListInfo': {
                const relatedListByNameState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$w(cacheKeySink, requestedField, relatedListByNameState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$A(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'query': {
            return {
                isArray: false,
                typename: 'RecordQuery'
            };
        }
        case 'aggregate': {
            return {
                isArray: false,
                typename: 'RecordQueryAggregate'
            };
        }
        case 'objectInfos': {
            return {
                isArray: true,
                typename: 'ObjectInfo'
            };
        }
        case 'relatedListByName': {
            return {
                isArray: false,
                typename: 'RelatedListInfo'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$k(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'RecordQuery': {
            const RecordQueryState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$10(requestedField, RecordQueryState);
            break;
        }
        case 'RecordQueryAggregate': {
            const RecordQueryAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$G(requestedField, RecordQueryAggregateState);
            break;
        }
        case 'ObjectInfo': {
            const ObjectInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$y(requestedField, ObjectInfoState);
            break;
        }
        case 'RelatedListInfo': {
            const RelatedListInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$v(requestedField, RelatedListInfoState);
            break;
        }
    }
}
function isMappedField$k(field) {
    return false;
}
function isFragmentApplicable$l(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "UIAPI", typename);
}
function getInContextFragmentType$B(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$j = 'Analytics__AnalyticsWorkspace';
const VERSION$x = 'bffae11a51f3d226cfaab2a3f3d33ac7';
function keyBuilder$x(luvio, path, data) {
    return keyPrefix + '::uiapi::' + name$j + '::' + data.Id; // We're missing idNodeName in the actual key here...
}
function mergeData$j(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$t(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$x(luvio, path, data);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$k,
        isMappedFieldImport: isMappedField$j,
        getFieldType: getFieldType$z,
        ingestFieldByType: ingestFieldByType$j,
        createLink: createLink,
        mergeData: mergeData$j,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsWorkspace",
            version: VERSION$x,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$j(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$z(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$j(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'StringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1v, select$1s, false);
            break;
        }
        case 'IDValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1t, select$1q, false);
            break;
        }
        case 'DateTimeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1s, select$1p, false);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$k(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$j, getFieldType$z, selectType$j, parentRecordId);
}
function buildSelectionForNode$k(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$k, buildSelectionForField$k, parentRecordId);
}
function select$u(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        reader.markSeenId(parentRecordId); // Always mark identifiable types as seen
        return selectCalculateSink(sink, field, buildSelectionForNode$k, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$B() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$u(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$x(luvio, path, data);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$j
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$k) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$z(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'StringValue': {
                const MasterLabelState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, MasterLabelState);
                break;
            }
            case 'StringValue': {
                const DeveloperNameState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, DeveloperNameState);
                break;
            }
            case 'IDValue': {
                const CreatedByIdState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, CreatedByIdState);
                break;
            }
            case 'IDValue': {
                const LastModifiedByIdState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, LastModifiedByIdState);
                break;
            }
            case 'DateTimeValue': {
                const LastModifiedDateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, LastModifiedDateState);
                break;
            }
            case 'DateTimeValue': {
                const CreatedDateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, CreatedDateState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$z(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'MasterLabel': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'DeveloperName': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'CreatedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'CreatedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$j(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'StringValue': {
            const StringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1r(requestedField, StringValueState);
            break;
        }
        case 'IDValue': {
            const IDValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1p(requestedField, IDValueState);
            break;
        }
        case 'DateTimeValue': {
            const DateTimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1o(requestedField, DateTimeValueState);
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$j(field) {
    return false;
}
function isFragmentApplicable$k(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__AnalyticsWorkspace", typename);
}
function getInContextFragmentType$A(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$i = 'Analytics__AnalyticsWorkspaceAsset';
const VERSION$w = 'c6366c7948f7ccacf02a144c87c0ca0b';
function keyBuilder$w(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$i(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$s(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$w(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$j,
        isMappedFieldImport: isMappedField$i,
        getFieldType: getFieldType$y,
        ingestFieldByType: ingestFieldByType$i,
        createLink: createLink,
        mergeData: mergeData$i,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsWorkspaceAsset",
            version: VERSION$w,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$i(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$y(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$i(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsWorkspace': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$x, select$u, false);
            break;
        }
        case 'PicklistValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1e, select$1b, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$j(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$i, getFieldType$y, selectType$i, parentRecordId);
}
function buildSelectionForNode$j(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$j, buildSelectionForField$j, parentRecordId);
}
function select$t(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$j, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$A() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$t(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$w(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$i
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$j) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$y(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Analytics__AnalyticsWorkspace': {
                const AnalyticsWorkspaceState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$u(cacheKeySink, requestedField, AnalyticsWorkspaceState);
                break;
            }
            case 'PicklistValue': {
                const AssetUsageTypeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1b(cacheKeySink, requestedField, AssetUsageTypeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$y(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'AnalyticsWorkspace': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsWorkspace'
            };
        }
        case 'AssetUsageType': {
            return {
                isArray: false,
                typename: 'PicklistValue'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$i(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsWorkspace': {
            const Analytics__AnalyticsWorkspaceState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$t(requestedField, Analytics__AnalyticsWorkspaceState);
            break;
        }
        case 'PicklistValue': {
            const PicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1a(requestedField, PicklistValueState);
            break;
        }
    }
}
function isMappedField$i(field) {
    return false;
}
function isFragmentApplicable$j(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__AnalyticsWorkspaceAsset", typename);
}
function getInContextFragmentType$z(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$h = 'Analytics__AnalyticsWorkspaceAssetEdge';
const VERSION$v = '59034a43231eec177f353ead0325ff66';
function keyBuilder$v(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$h(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$r(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$v(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$i,
        isMappedFieldImport: isMappedField$h,
        getFieldType: getFieldType$x,
        ingestFieldByType: ingestFieldByType$h,
        createLink: createLink,
        mergeData: mergeData$h,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsWorkspaceAssetEdge",
            version: VERSION$v,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$h(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$x(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$h(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsWorkspaceAsset': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$w, select$t, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$i(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$h, getFieldType$x, selectType$h, parentRecordId);
}
function buildSelectionForNode$i(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$i, buildSelectionForField$i, parentRecordId);
}
function select$s(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$i, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$z() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$s(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$v(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$h
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$i) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$x(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Analytics__AnalyticsWorkspaceAsset': {
                const nodeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$t(cacheKeySink, requestedField, nodeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$x(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsWorkspaceAsset'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$h(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsWorkspaceAsset': {
            const Analytics__AnalyticsWorkspaceAssetState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$s(requestedField, Analytics__AnalyticsWorkspaceAssetState);
            break;
        }
    }
}
function isMappedField$h(field) {
    return false;
}
function isFragmentApplicable$i(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__AnalyticsWorkspaceAssetEdge", typename);
}
function getInContextFragmentType$y(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$g = 'Analytics__AnalyticsWorkspaceAssetConnection';
const VERSION$u = '4fcd92af2b51c70cd9164767a7f28c75';
function keyBuilder$u(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$g(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingestPaginationMetadata$2(astNode, state, key, sink, existingData) {
    var _a;
    const { data, luvio, store } = state;
    const { pageInfo: { hasNextPage, startCursor }, edges } = data;
    if (!hasGraphQLPaginationFields(data))
        return;
    const paginationKey = key + '__pagination';
    let existingPaginationMetadata = store.readEntry(paginationKey);
    const afterArg = (_a = astNode.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', state.variables);
    const isPagedResponse = afterArgVal !== undefined &&
        existingPaginationMetadata !== undefined &&
        existingPaginationMetadata[afterArgVal] !== undefined;
    if (isPagedResponse && existingPaginationMetadata !== undefined) {
        const previousPageOffset = existingPaginationMetadata[afterArgVal];
        existingPaginationMetadata = {
            ...existingPaginationMetadata,
            [startCursor]: previousPageOffset + 1,
        };
    }
    // Default to a zero offset if we're not a result of a paged request
    const startOffset = isPagedResponse && existingPaginationMetadata !== undefined
        ? offsetFromToken(existingPaginationMetadata, startCursor)
        : 0;
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0) {
            throw new RangeError(`Cursor value ${startCursor} is not recognized. Cannot ingest the returned page. After: ${afterArgVal}`);
        }
    }
    const nextOffset = startOffset + edges.length;
    const incomingPaginationMetadata = createPaginationMetadata(startOffset, edges);
    if (hasNextPage) {
        // we know the data extends beyond NEXT_OFFSET_IDENTIFIER; clean up any old __END__ markers that say the data ended at/before that offset
        if (existingPaginationMetadata !== undefined && existingPaginationMetadata.__END__ !== undefined && existingPaginationMetadata.__END__ <= nextOffset) {
            existingPaginationMetadata = { ...existingPaginationMetadata };
            delete existingPaginationMetadata.__END__;
        }
    }
    else {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    // store the updated pagination metadata
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    luvio.storePublish(paginationKey, mergedPaginationMetadata);
    luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    // merge the edges
    assignMetadataLink(sink, paginationKey);
    const existingRecord = existingData ? existingData.data : undefined;
    if (existingRecord !== undefined && existingRecord.edges !== undefined) {
        sink.edges = [
            ...existingRecord.edges.slice(0, startOffset),
            ...sink.edges,
            ...existingRecord.edges.slice(nextOffset),
        ];
    }
}
function ingest$q(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$u(luvio, path);
    return ingestCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$h,
        isMappedFieldImport: isMappedField$g,
        getFieldType: getFieldType$w,
        ingestFieldByType: ingestFieldByType$g,
        hasGraphQLPaginationFields: hasGraphQLPaginationFields,
        ingestPaginatedMetadata: ingestPaginationMetadata$2,
        createLink: createLink,
        mergeData: mergeData$g,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsWorkspaceAssetConnection",
            version: VERSION$u,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$g(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$w(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$g(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsWorkspaceAssetEdge': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$v, select$s, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'PageInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$18, select$15, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$h(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, true, getFieldData$g, getFieldType$w, selectType$g, parentRecordId);
}
function buildSelectionForNode$h(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$h, buildSelectionForField$h, parentRecordId);
}
function select$r(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$h, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$y() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        ...getCursorConnectionMinimumSelections()
    ];
    return minimumSelections;
}
function getTypeCacheKeys$r(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$u(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$g
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$h) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$w(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Analytics__AnalyticsWorkspaceAssetEdge': {
                const fullPath__edges = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__edges + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$s(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'PageInfo': {
                const pageInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$15(cacheKeySink, requestedField, pageInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$w(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'Analytics__AnalyticsWorkspaceAssetEdge'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageResultCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$g(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsWorkspaceAssetEdge': {
            const Analytics__AnalyticsWorkspaceAssetEdgeState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$r(requestedField, Analytics__AnalyticsWorkspaceAssetEdgeState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'PageInfo': {
            const PageInfoState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$14(requestedField, PageInfoState);
            break;
        }
    }
}
function isMappedField$g(field) {
    return false;
}
function isFragmentApplicable$h(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__AnalyticsWorkspaceAssetConnection", typename);
}
function getInContextFragmentType$x(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$f = 'Analytics__AnalyticsRepresentation';
const VERSION$t = '9719233aaf1a82094e73b9cbd55205ac';
function keyBuilder$t(luvio, path, data) {
    return keyPrefix + '::uiapi::' + name$f + '::' + data.Id; // We're missing idNodeName in the actual key here...
}
function mergeData$f(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$p(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$t(luvio, path, data);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$g,
        isMappedFieldImport: isMappedField$f,
        getFieldType: getFieldType$v,
        ingestFieldByType: ingestFieldByType$f,
        createLink: createLink,
        mergeData: mergeData$f,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsRepresentation",
            version: VERSION$t,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$f(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$v(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$f(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'StringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1v, select$1s, false);
            break;
        }
        case 'IDValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1t, select$1q, false);
            break;
        }
        case 'DateTimeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1s, select$1p, false);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsWorkspaceAssetConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$u, select$r, true);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$g(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$f, getFieldType$v, selectType$f, parentRecordId);
}
function buildSelectionForNode$g(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$g, buildSelectionForField$g, parentRecordId);
}
function select$q(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        reader.markSeenId(parentRecordId); // Always mark identifiable types as seen
        return selectCalculateSink(sink, field, buildSelectionForNode$g, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$x() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$q(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$t(luvio, path, data);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$f
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$g) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$v(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'StringValue': {
                const MasterLabelState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, MasterLabelState);
                break;
            }
            case 'StringValue': {
                const DeveloperNameState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, DeveloperNameState);
                break;
            }
            case 'IDValue': {
                const CreatedByIdState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, CreatedByIdState);
                break;
            }
            case 'IDValue': {
                const LastModifiedByIdState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, LastModifiedByIdState);
                break;
            }
            case 'DateTimeValue': {
                const LastModifiedDateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, LastModifiedDateState);
                break;
            }
            case 'DateTimeValue': {
                const CreatedDateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, CreatedDateState);
                break;
            }
            case 'StringValue': {
                const NamespacePrefixState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, NamespacePrefixState);
                break;
            }
            case 'Analytics__AnalyticsWorkspaceAssetConnection': {
                const AnalyticsWorkspacesState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$r(cacheKeySink, requestedField, AnalyticsWorkspacesState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$v(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'MasterLabel': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'DeveloperName': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'CreatedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'CreatedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'NamespacePrefix': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'AnalyticsWorkspaces': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsWorkspaceAssetConnection'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$f(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'StringValue': {
            const StringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1r(requestedField, StringValueState);
            break;
        }
        case 'IDValue': {
            const IDValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1p(requestedField, IDValueState);
            break;
        }
        case 'DateTimeValue': {
            const DateTimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1o(requestedField, DateTimeValueState);
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsWorkspaceAssetConnection': {
            const Analytics__AnalyticsWorkspaceAssetConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$q(requestedField, Analytics__AnalyticsWorkspaceAssetConnectionState);
            break;
        }
    }
}
function isMappedField$f(field) {
    return false;
}
function isFragmentApplicable$g(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, ["Analytics__AnalyticsRepresentationInterface"], "Analytics__AnalyticsRepresentation", typename);
}
function getInContextFragmentType$w(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$e = 'Analytics__AnalyticsEdge';
const VERSION$s = 'bd0e530079262569dd4afee14b0d9b8e';
function keyBuilder$s(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$e(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$o(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$s(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$f,
        isMappedFieldImport: isMappedField$e,
        getFieldType: getFieldType$u,
        ingestFieldByType: ingestFieldByType$e,
        createLink: createLink,
        mergeData: mergeData$e,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsEdge",
            version: VERSION$s,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$e(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$u(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$e(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsRepresentation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$t, select$q, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$f(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$e, getFieldType$u, selectType$e, parentRecordId);
}
function buildSelectionForNode$f(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$f, buildSelectionForField$f, parentRecordId);
}
function select$p(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$f, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$w() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$p(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$s(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$e
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$f) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$u(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Analytics__AnalyticsRepresentation': {
                const nodeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$q(cacheKeySink, requestedField, nodeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$u(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsRepresentation'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$e(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsRepresentation': {
            const Analytics__AnalyticsRepresentationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$p(requestedField, Analytics__AnalyticsRepresentationState);
            break;
        }
    }
}
function isMappedField$e(field) {
    return false;
}
function isFragmentApplicable$f(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__AnalyticsEdge", typename);
}
function getInContextFragmentType$v(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$d = 'Analytics__AnalyticsBrowse';
const VERSION$r = 'de14944e92aa0e199e1e9f72d8d98551';
function keyBuilder$r(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$d(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$n(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$r(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$e,
        isMappedFieldImport: isMappedField$d,
        getFieldType: getFieldType$t,
        ingestFieldByType: ingestFieldByType$d,
        createLink: createLink,
        mergeData: mergeData$d,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__AnalyticsBrowse",
            version: VERSION$r,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$d(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$t(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$d(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsEdge': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$s, select$p, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'PageInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$18, select$15, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$e(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$d, getFieldType$t, selectType$d, parentRecordId);
}
function buildSelectionForNode$e(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$e, buildSelectionForField$e, parentRecordId);
}
function select$o(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$e, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$v() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$o(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$r(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$d
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$e) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$t(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Analytics__AnalyticsEdge': {
                const fullPath__edges = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__edges + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$p(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'PageInfo': {
                const pageInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$15(cacheKeySink, requestedField, pageInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$t(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'Analytics__AnalyticsEdge'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageResultCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$d(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsEdge': {
            const Analytics__AnalyticsEdgeState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$o(requestedField, Analytics__AnalyticsEdgeState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'PageInfo': {
            const PageInfoState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$14(requestedField, PageInfoState);
            break;
        }
    }
}
function isMappedField$d(field) {
    return false;
}
function isFragmentApplicable$e(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__AnalyticsBrowse", typename);
}
function getInContextFragmentType$u(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$c = 'Analytics__Analytics';
const VERSION$q = '7eb7858d53c6e210d35985480d71a083';
function keyBuilder$q(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$c(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$m(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$q(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$d,
        isMappedFieldImport: isMappedField$c,
        getFieldType: getFieldType$s,
        ingestFieldByType: ingestFieldByType$c,
        createLink: createLink,
        mergeData: mergeData$c,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Analytics__Analytics",
            version: VERSION$q,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$c(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$s(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$c(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Analytics__AnalyticsBrowse': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$r, select$o, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$d(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$c, getFieldType$s, selectType$c, parentRecordId);
}
function buildSelectionForNode$d(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$d, buildSelectionForField$d, parentRecordId);
}
function select$n(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$d, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$u() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$n(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$q(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$c
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$d) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$s(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Analytics__AnalyticsBrowse': {
                const browseState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$o(cacheKeySink, requestedField, browseState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$s(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'browse': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsBrowse'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$c(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Analytics__AnalyticsBrowse': {
            const Analytics__AnalyticsBrowseState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$n(requestedField, Analytics__AnalyticsBrowseState);
            break;
        }
    }
}
function isMappedField$c(field) {
    return false;
}
function isFragmentApplicable$d(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Analytics__Analytics", typename);
}
function getInContextFragmentType$t(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$b = 'SObject__Field';
const VERSION$p = '7acebed646d199da2d0c03c956162517';
function keyBuilder$p(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$b(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$l(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$p(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$c,
        isMappedFieldImport: isMappedField$b,
        getFieldType: getFieldType$r,
        ingestFieldByType: ingestFieldByType$b,
        createLink: createLink,
        mergeData: mergeData$b,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "SObject__Field",
            version: VERSION$p,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$b(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$r(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$b(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$c(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$b, getFieldType$r, selectType$b, parentRecordId);
}
function buildSelectionForNode$c(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$c, buildSelectionForField$c, parentRecordId);
}
function select$m(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$c, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$t() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$m(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$p(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$b
    });
    // get all cache keys from normalizable fields
    // Deal with mapped types' cache keys
}
function getFieldType$r(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'name': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$b(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$b(field) {
    return false;
}
function isFragmentApplicable$c(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "SObject__Field", typename);
}
function getInContextFragmentType$s(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

// union versions are undefined, selected versions must be specific to the type being selected
const VERSION$o = undefined;
function ingest$k(astNode, state) {
    const typename = state.data.__typename;
    // Delegate key building and ingest to concrete implementations
    if (typename === 'Setup__EntityRepresentation') {
        return ingest$j(astNode, state);
    }
}
function select$l(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const typename = source.data.__typename;
        if (typename === 'Setup__EntityRepresentation') {
            return select$k(field, variables, fragments)(source, reader, parentRecordId);
        }
    };
}
function getTypeCacheKeys$l(cacheKeys, astNode, state) {
    const { path, data } = state;
    const typename = data.__typename;
    if (typename === 'Setup__EntityRepresentation') {
        return getTypeCacheKeys$k(cacheKeys, astNode, state);
    }
}
function getInContextFragmentType$r(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}
function getFieldType$q(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getMinimumSelections$s() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}

const name$a = 'Setup__EntityRepresentation';
const VERSION$n = 'd45b0eb67ba6273ed9b38313a773b15a';
function keyBuilder$o(luvio, path, data) {
    return keyPrefix + '::uiapi::' + name$a + '::' + data.Id; // We're missing idNodeName in the actual key here...
}
function mergeData$a(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$j(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$o(luvio, path, data);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$b,
        isMappedFieldImport: isMappedField$a,
        getFieldType: getFieldType$p,
        ingestFieldByType: ingestFieldByType$a,
        createLink: createLink,
        mergeData: mergeData$a,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__EntityRepresentation",
            version: VERSION$n,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$a(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$p(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$a(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'ID': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'IntValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1w, select$1t, false);
            break;
        }
        case 'StringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1v, select$1s, false);
            break;
        }
        case 'BooleanValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1r, select$1o, false);
            break;
        }
        case 'IDValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1t, select$1q, false);
            break;
        }
        case 'DateTimeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1s, select$1p, false);
            break;
        }
        case 'TimeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1q, select$1n, false);
            break;
        }
        case 'DateValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1p, select$1m, false);
            break;
        }
        case 'TextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1o, select$1l, false);
            break;
        }
        case 'LongTextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1n, select$1k, false);
            break;
        }
        case 'RichTextAreaValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1m, select$1j, false);
            break;
        }
        case 'PhoneNumberValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1l, select$1i, false);
            break;
        }
        case 'EmailValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1k, select$1h, false);
            break;
        }
        case 'UrlValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1j, select$1g, false);
            break;
        }
        case 'EncryptedStringValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1i, select$1f, false);
            break;
        }
        case 'SObject__Field': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$p, select$m, false);
            break;
        }
        case 'CurrencyValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1h, select$1e, false);
            break;
        }
        case 'LongitudeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1g, select$1d, false);
            break;
        }
        case 'LatitudeValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1f, select$1c, false);
            break;
        }
        case 'PicklistValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1e, select$1b, false);
            break;
        }
        case 'MultiPicklistValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1d, select$1a, false);
            break;
        }
        case 'LongValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1z, select$1w, false);
            break;
        }
        case 'DoubleValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1A, select$1x, false);
            break;
        }
        case 'PercentValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1y, select$1v, false);
            break;
        }
        case 'Base64Value': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1c, select$19, false);
            break;
        }
        case 'JSONValue': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1b, select$18, false);
            break;
        }
        case 'Setup__EntityRepresentation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$n, select$k, false);
            break;
        }
        case 'Setup__SetupPolymorphicParentRelationship': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$o, select$l, false);
            break;
        }
        case 'Setup__SetupConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$l, select$i, true);
            break;
        }
        case 'CompoundField': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$19, select$16, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$b(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$a, getFieldType$p, selectType$a, parentRecordId);
}
function buildSelectionForNode$b(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$b, buildSelectionForField$b, parentRecordId);
}
function select$k(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        reader.markSeenId(parentRecordId); // Always mark identifiable types as seen
        return selectCalculateSink(sink, field, buildSelectionForNode$b, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$r() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$k(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$o(luvio, path, data);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$a
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$b) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$p(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'IntValue': {
                const IntValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1t(cacheKeySink, requestedField, IntValueState);
                break;
            }
            case 'StringValue': {
                const StringValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1s(cacheKeySink, requestedField, StringValueState);
                break;
            }
            case 'BooleanValue': {
                const BooleanValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1o(cacheKeySink, requestedField, BooleanValueState);
                break;
            }
            case 'IDValue': {
                const IDValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1q(cacheKeySink, requestedField, IDValueState);
                break;
            }
            case 'DateTimeValue': {
                const DateTimeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1p(cacheKeySink, requestedField, DateTimeValueState);
                break;
            }
            case 'TimeValue': {
                const TimeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1n(cacheKeySink, requestedField, TimeValueState);
                break;
            }
            case 'DateValue': {
                const DateValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1m(cacheKeySink, requestedField, DateValueState);
                break;
            }
            case 'TextAreaValue': {
                const TextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1l(cacheKeySink, requestedField, TextAreaValueState);
                break;
            }
            case 'LongTextAreaValue': {
                const LongTextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1k(cacheKeySink, requestedField, LongTextAreaValueState);
                break;
            }
            case 'RichTextAreaValue': {
                const RichTextAreaValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1j(cacheKeySink, requestedField, RichTextAreaValueState);
                break;
            }
            case 'PhoneNumberValue': {
                const PhoneNumberValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1i(cacheKeySink, requestedField, PhoneNumberValueState);
                break;
            }
            case 'EmailValue': {
                const EmailValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1h(cacheKeySink, requestedField, EmailValueState);
                break;
            }
            case 'UrlValue': {
                const UrlValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1g(cacheKeySink, requestedField, UrlValueState);
                break;
            }
            case 'EncryptedStringValue': {
                const EncryptedStringValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1f(cacheKeySink, requestedField, EncryptedStringValueState);
                break;
            }
            case 'SObject__Field': {
                const fullPath__Entity__fields = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__Entity__fields + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$m(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'CurrencyValue': {
                const CurrencyValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1e(cacheKeySink, requestedField, CurrencyValueState);
                break;
            }
            case 'LongitudeValue': {
                const LongitudeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1d(cacheKeySink, requestedField, LongitudeValueState);
                break;
            }
            case 'LatitudeValue': {
                const LatitudeValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1c(cacheKeySink, requestedField, LatitudeValueState);
                break;
            }
            case 'PicklistValue': {
                const PicklistValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1b(cacheKeySink, requestedField, PicklistValueState);
                break;
            }
            case 'MultiPicklistValue': {
                const MultiPicklistValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1a(cacheKeySink, requestedField, MultiPicklistValueState);
                break;
            }
            case 'LongValue': {
                const LongValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1w(cacheKeySink, requestedField, LongValueState);
                break;
            }
            case 'DoubleValue': {
                const DoubleValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1x(cacheKeySink, requestedField, DoubleValueState);
                break;
            }
            case 'PercentValue': {
                const PercentValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1v(cacheKeySink, requestedField, PercentValueState);
                break;
            }
            case 'Base64Value': {
                const Base64ValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$19(cacheKeySink, requestedField, Base64ValueState);
                break;
            }
            case 'JSONValue': {
                const JSONValueState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$18(cacheKeySink, requestedField, JSONValueState);
                break;
            }
            case 'Setup__EntityRepresentation': {
                const parentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$k(cacheKeySink, requestedField, parentRelationshipState);
                break;
            }
            case 'Setup__SetupPolymorphicParentRelationship': {
                const polymorphicParentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$l(cacheKeySink, requestedField, polymorphicParentRelationshipState);
                break;
            }
            case 'Setup__SetupConnection': {
                const childRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$i(cacheKeySink, requestedField, childRelationshipState);
                break;
            }
            case 'CompoundField': {
                const CompoundFieldState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$16(cacheKeySink, requestedField, CompoundFieldState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$p(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'IntValue': {
            return {
                isArray: false,
                typename: 'IntValue'
            };
        }
        case 'StringValue': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'BooleanValue': {
            return {
                isArray: false,
                typename: 'BooleanValue'
            };
        }
        case 'IDValue': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'DateTimeValue': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'TimeValue': {
            return {
                isArray: false,
                typename: 'TimeValue'
            };
        }
        case 'DateValue': {
            return {
                isArray: false,
                typename: 'DateValue'
            };
        }
        case 'TextAreaValue': {
            return {
                isArray: false,
                typename: 'TextAreaValue'
            };
        }
        case 'LongTextAreaValue': {
            return {
                isArray: false,
                typename: 'LongTextAreaValue'
            };
        }
        case 'RichTextAreaValue': {
            return {
                isArray: false,
                typename: 'RichTextAreaValue'
            };
        }
        case 'PhoneNumberValue': {
            return {
                isArray: false,
                typename: 'PhoneNumberValue'
            };
        }
        case 'EmailValue': {
            return {
                isArray: false,
                typename: 'EmailValue'
            };
        }
        case 'UrlValue': {
            return {
                isArray: false,
                typename: 'UrlValue'
            };
        }
        case 'EncryptedStringValue': {
            return {
                isArray: false,
                typename: 'EncryptedStringValue'
            };
        }
        case 'Entity__fields': {
            return {
                isArray: true,
                typename: 'SObject__Field'
            };
        }
        case 'CurrencyValue': {
            return {
                isArray: false,
                typename: 'CurrencyValue'
            };
        }
        case 'LongitudeValue': {
            return {
                isArray: false,
                typename: 'LongitudeValue'
            };
        }
        case 'LatitudeValue': {
            return {
                isArray: false,
                typename: 'LatitudeValue'
            };
        }
        case 'PicklistValue': {
            return {
                isArray: false,
                typename: 'PicklistValue'
            };
        }
        case 'MultiPicklistValue': {
            return {
                isArray: false,
                typename: 'MultiPicklistValue'
            };
        }
        case 'LongValue': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'DoubleValue': {
            return {
                isArray: false,
                typename: 'DoubleValue'
            };
        }
        case 'PercentValue': {
            return {
                isArray: false,
                typename: 'PercentValue'
            };
        }
        case 'Base64Value': {
            return {
                isArray: false,
                typename: 'Base64Value'
            };
        }
        case 'JSONValue': {
            return {
                isArray: false,
                typename: 'JSONValue'
            };
        }
        case 'parentRelationship': {
            return {
                isArray: false,
                typename: 'Setup__EntityRepresentation'
            };
        }
        case 'polymorphicParentRelationship': {
            return {
                isArray: false,
                typename: 'Setup__SetupPolymorphicParentRelationship'
            };
        }
        case 'childRelationship': {
            return {
                isArray: false,
                typename: 'Setup__SetupConnection'
            };
        }
        case 'CompoundField': {
            return {
                isArray: false,
                typename: 'CompoundField'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$a(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'ID': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'IntValue': {
            const IntValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1s(requestedField, IntValueState);
            break;
        }
        case 'StringValue': {
            const StringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1r(requestedField, StringValueState);
            break;
        }
        case 'BooleanValue': {
            const BooleanValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1n(requestedField, BooleanValueState);
            break;
        }
        case 'IDValue': {
            const IDValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1p(requestedField, IDValueState);
            break;
        }
        case 'DateTimeValue': {
            const DateTimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1o(requestedField, DateTimeValueState);
            break;
        }
        case 'TimeValue': {
            const TimeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1m(requestedField, TimeValueState);
            break;
        }
        case 'DateValue': {
            const DateValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1l(requestedField, DateValueState);
            break;
        }
        case 'TextAreaValue': {
            const TextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1k(requestedField, TextAreaValueState);
            break;
        }
        case 'LongTextAreaValue': {
            const LongTextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1j(requestedField, LongTextAreaValueState);
            break;
        }
        case 'RichTextAreaValue': {
            const RichTextAreaValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1i(requestedField, RichTextAreaValueState);
            break;
        }
        case 'PhoneNumberValue': {
            const PhoneNumberValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1h(requestedField, PhoneNumberValueState);
            break;
        }
        case 'EmailValue': {
            const EmailValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1g(requestedField, EmailValueState);
            break;
        }
        case 'UrlValue': {
            const UrlValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1f(requestedField, UrlValueState);
            break;
        }
        case 'EncryptedStringValue': {
            const EncryptedStringValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1e(requestedField, EncryptedStringValueState);
            break;
        }
        case 'SObject__Field': {
            const SObject__FieldState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$l(requestedField, SObject__FieldState);
            break;
        }
        case 'CurrencyValue': {
            const CurrencyValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1d(requestedField, CurrencyValueState);
            break;
        }
        case 'LongitudeValue': {
            const LongitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1c(requestedField, LongitudeValueState);
            break;
        }
        case 'LatitudeValue': {
            const LatitudeValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1b(requestedField, LatitudeValueState);
            break;
        }
        case 'PicklistValue': {
            const PicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1a(requestedField, PicklistValueState);
            break;
        }
        case 'MultiPicklistValue': {
            const MultiPicklistValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$19(requestedField, MultiPicklistValueState);
            break;
        }
        case 'LongValue': {
            const LongValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1v(requestedField, LongValueState);
            break;
        }
        case 'DoubleValue': {
            const DoubleValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1w(requestedField, DoubleValueState);
            break;
        }
        case 'PercentValue': {
            const PercentValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1u(requestedField, PercentValueState);
            break;
        }
        case 'Base64Value': {
            const Base64ValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$18(requestedField, Base64ValueState);
            break;
        }
        case 'JSONValue': {
            const JSONValueState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$17(requestedField, JSONValueState);
            break;
        }
        case 'Setup__EntityRepresentation': {
            const Setup__EntityRepresentationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$j(requestedField, Setup__EntityRepresentationState);
            break;
        }
        case 'Setup__SetupPolymorphicParentRelationship': {
            const Setup__SetupPolymorphicParentRelationshipState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$k(requestedField, Setup__SetupPolymorphicParentRelationshipState);
            break;
        }
        case 'Setup__SetupConnection': {
            const Setup__SetupConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$h(requestedField, Setup__SetupConnectionState);
            break;
        }
        case 'CompoundField': {
            const CompoundFieldState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$15(requestedField, CompoundFieldState);
            break;
        }
    }
}
function isMappedField$a(field) {
    return false;
}
function isFragmentApplicable$b(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__EntityRepresentation", typename);
}
function getInContextFragmentType$q(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$9 = 'Setup__SetupEdge';
const VERSION$m = 'b62a2103dcf86300f0ad705b5694328e';
function keyBuilder$n(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$9(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$i(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$n(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$a,
        isMappedFieldImport: isMappedField$9,
        getFieldType: getFieldType$o,
        ingestFieldByType: ingestFieldByType$9,
        createLink: createLink,
        mergeData: mergeData$9,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupEdge",
            version: VERSION$m,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$9(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$o(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$9(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__EntityRepresentation': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$n, select$k, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$a(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$9, getFieldType$o, selectType$9, parentRecordId);
}
function buildSelectionForNode$a(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$a, buildSelectionForField$a, parentRecordId);
}
function select$j(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$a, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$q() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$j(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$n(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$9
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$a) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$o(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__EntityRepresentation': {
                const nodeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$k(cacheKeySink, requestedField, nodeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$o(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'Setup__EntityRepresentation'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$9(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__EntityRepresentation': {
            const Setup__EntityRepresentationState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$j(requestedField, Setup__EntityRepresentationState);
            break;
        }
    }
}
function isMappedField$9(field) {
    return false;
}
function isFragmentApplicable$a(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupEdge", typename);
}
function getInContextFragmentType$p(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$8 = 'Setup__SetupConnection';
const VERSION$l = '02512d7a4706e18f377cc4abf3094579';
function keyBuilder$m(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$8(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingestPaginationMetadata$1(astNode, state, key, sink, existingData) {
    var _a;
    const { data, luvio, store } = state;
    const { pageInfo: { hasNextPage, startCursor }, edges } = data;
    if (!hasGraphQLPaginationFields(data))
        return;
    const paginationKey = key + '__pagination';
    let existingPaginationMetadata = store.readEntry(paginationKey);
    const afterArg = (_a = astNode.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', state.variables);
    const isPagedResponse = afterArgVal !== undefined &&
        existingPaginationMetadata !== undefined &&
        existingPaginationMetadata[afterArgVal] !== undefined;
    if (isPagedResponse && existingPaginationMetadata !== undefined) {
        const previousPageOffset = existingPaginationMetadata[afterArgVal];
        existingPaginationMetadata = {
            ...existingPaginationMetadata,
            [startCursor]: previousPageOffset + 1,
        };
    }
    // Default to a zero offset if we're not a result of a paged request
    const startOffset = isPagedResponse && existingPaginationMetadata !== undefined
        ? offsetFromToken(existingPaginationMetadata, startCursor)
        : 0;
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0) {
            throw new RangeError(`Cursor value ${startCursor} is not recognized. Cannot ingest the returned page. After: ${afterArgVal}`);
        }
    }
    const nextOffset = startOffset + edges.length;
    const incomingPaginationMetadata = createPaginationMetadata(startOffset, edges);
    if (hasNextPage) {
        // we know the data extends beyond NEXT_OFFSET_IDENTIFIER; clean up any old __END__ markers that say the data ended at/before that offset
        if (existingPaginationMetadata !== undefined && existingPaginationMetadata.__END__ !== undefined && existingPaginationMetadata.__END__ <= nextOffset) {
            existingPaginationMetadata = { ...existingPaginationMetadata };
            delete existingPaginationMetadata.__END__;
        }
    }
    else {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    // store the updated pagination metadata
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    luvio.storePublish(paginationKey, mergedPaginationMetadata);
    luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    // merge the edges
    assignMetadataLink(sink, paginationKey);
    const existingRecord = existingData ? existingData.data : undefined;
    if (existingRecord !== undefined && existingRecord.edges !== undefined) {
        sink.edges = [
            ...existingRecord.edges.slice(0, startOffset),
            ...sink.edges,
            ...existingRecord.edges.slice(nextOffset),
        ];
    }
}
function ingest$h(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$m(luvio, path);
    return ingestCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$9,
        isMappedFieldImport: isMappedField$8,
        getFieldType: getFieldType$n,
        ingestFieldByType: ingestFieldByType$8,
        hasGraphQLPaginationFields: hasGraphQLPaginationFields,
        ingestPaginatedMetadata: ingestPaginationMetadata$1,
        createLink: createLink,
        mergeData: mergeData$8,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupConnection",
            version: VERSION$l,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$8(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$n(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$8(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupEdge': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$m, select$j, false);
            break;
        }
        case 'PageInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$18, select$15, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$9(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, true, getFieldData$8, getFieldType$n, selectType$8, parentRecordId);
}
function buildSelectionForNode$9(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$9, buildSelectionForField$9, parentRecordId);
}
function select$i(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$9, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$p() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        ...getCursorConnectionMinimumSelections()
    ];
    return minimumSelections;
}
function getTypeCacheKeys$i(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$m(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$8
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$9) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$n(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupEdge': {
                const fullPath__edges = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__edges + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$j(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'PageInfo': {
                const pageInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$15(cacheKeySink, requestedField, pageInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$n(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'Setup__SetupEdge'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageResultCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$8(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupEdge': {
            const Setup__SetupEdgeState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$i(requestedField, Setup__SetupEdgeState);
            break;
        }
        case 'PageInfo': {
            const PageInfoState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$14(requestedField, PageInfoState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$8(field) {
    return false;
}
function isFragmentApplicable$9(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupConnection", typename);
}
function getInContextFragmentType$o(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getFieldType$m(field) {
    const fieldName = field.name.value;
    switch (fieldName) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        /*
            Probably should use metaschema like the RecordQuery graphql-artifact,
            but this type only has one generic type possible. Also, the most likely scenario for future schema additions,
            is UISDK adding the the parent of this type, Setup__Setup. Therefore, we will just default to SetupConnection so that
            query authors don't need to use metaschema when there's no real decision point in this type.
         */
        default:
            return {
                isArray: false,
                typename: 'Setup__SetupConnection',
            };
    }
}

const name$7 = 'Setup__SetupQuery';
const VERSION$k = 'b89cccb5fa5e458d89dc1137af3e5469';
function keyBuilder$l(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$7(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$g(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$l(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$8,
        isMappedFieldImport: isMappedField$7,
        getFieldType: getFieldType$m,
        ingestFieldByType: ingestFieldByType$7,
        createLink: createLink,
        mergeData: mergeData$7,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupQuery",
            version: VERSION$k,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$7(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$m(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$7(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$l, select$i, true);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$8(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$7, getFieldType$m, selectType$7, parentRecordId);
}
function buildSelectionForNode$8(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$8, buildSelectionForField$8, parentRecordId);
}
function select$h(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$8, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$o() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$h(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$l(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$7
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$8) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$m(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupConnection': {
                const recordQueryState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$i(cacheKeySink, requestedField, recordQueryState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function ingestFieldByType$7(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupConnection': {
            const Setup__SetupConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$h(requestedField, Setup__SetupConnectionState);
            break;
        }
    }
}
function isMappedField$7(field) {
    return false;
}
function isFragmentApplicable$8(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupQuery", typename);
}
function getInContextFragmentType$n(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

// union versions are undefined, selected versions must be specific to the type being selected
const VERSION$j = undefined;
function ingest$f(astNode, state) {
    const typename = state.data.__typename;
    // Delegate key building and ingest to concrete implementations
    if (typename === 'Setup__SetupRecordAggregate') {
        return ingest$e(astNode, state);
    }
}
function select$g(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const typename = source.data.__typename;
        if (typename === 'Setup__SetupRecordAggregate') {
            return select$f(field, variables, fragments)(source, reader, parentRecordId);
        }
    };
}
function getTypeCacheKeys$g(cacheKeys, astNode, state) {
    const { path, data } = state;
    const typename = data.__typename;
    if (typename === 'Setup__SetupRecordAggregate') {
        return getTypeCacheKeys$f(cacheKeys, astNode, state);
    }
}
function getInContextFragmentType$m(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}
function getFieldType$l(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getMinimumSelections$n() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}

const name$6 = 'Setup__SetupRecordAggregate';
const VERSION$i = '47f8092aa692ffc3ab1a2cc12b530dbc';
function keyBuilder$k(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$6(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$e(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$k(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$7,
        isMappedFieldImport: isMappedField$6,
        getFieldType: getFieldType$k,
        ingestFieldByType: ingestFieldByType$6,
        createLink: createLink,
        mergeData: mergeData$6,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupRecordAggregate",
            version: VERSION$i,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$6(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$k(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$6(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'BooleanAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$13, select$10, false);
            break;
        }
        case 'CurrencyAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$12, select$$, false);
            break;
        }
        case 'DateAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$10, select$Z, false);
            break;
        }
        case 'DoubleAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$$, select$Y, false);
            break;
        }
        case 'EmailAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$_, select$X, false);
            break;
        }
        case 'IDAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$Z, select$W, false);
            break;
        }
        case 'IntAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$Y, select$V, false);
            break;
        }
        case 'LatitudeAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$X, select$U, false);
            break;
        }
        case 'LongAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$V, select$S, false);
            break;
        }
        case 'LongitudeAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$W, select$T, false);
            break;
        }
        case 'PercentAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1x, select$1u, false);
            break;
        }
        case 'PhoneNumberAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$U, select$R, false);
            break;
        }
        case 'PicklistAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$T, select$Q, false);
            break;
        }
        case 'StringAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$1u, select$1r, false);
            break;
        }
        case 'TextAreaAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$S, select$P, false);
            break;
        }
        case 'TimeAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$R, select$O, false);
            break;
        }
        case 'UrlAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$Q, select$N, false);
            break;
        }
        case 'Setup__SetupRecordAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$i, select$f, false);
            break;
        }
        case 'Setup__SetupPolymorphicAggregateParentRelationship': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$j, select$g, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$7(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$6, getFieldType$k, selectType$6, parentRecordId);
}
function buildSelectionForNode$7(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$7, buildSelectionForField$7, parentRecordId);
}
function select$f(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$7, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$m() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$f(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$k(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$6
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$7) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$k(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'BooleanAggregate': {
                const BooleanAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$10(cacheKeySink, requestedField, BooleanAggregateState);
                break;
            }
            case 'CurrencyAggregate': {
                const CurrencyAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$$(cacheKeySink, requestedField, CurrencyAggregateState);
                break;
            }
            case 'DateAggregate': {
                const DateAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$Z(cacheKeySink, requestedField, DateAggregateState);
                break;
            }
            case 'DoubleAggregate': {
                const DoubleAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$Y(cacheKeySink, requestedField, DoubleAggregateState);
                break;
            }
            case 'EmailAggregate': {
                const EmailAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$X(cacheKeySink, requestedField, EmailAggregateState);
                break;
            }
            case 'IDAggregate': {
                const IDAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$W(cacheKeySink, requestedField, IDAggregateState);
                break;
            }
            case 'IntAggregate': {
                const IntAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$V(cacheKeySink, requestedField, IntAggregateState);
                break;
            }
            case 'LatitudeAggregate': {
                const LatitudeAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$U(cacheKeySink, requestedField, LatitudeAggregateState);
                break;
            }
            case 'LongAggregate': {
                const LongAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$S(cacheKeySink, requestedField, LongAggregateState);
                break;
            }
            case 'LongitudeAggregate': {
                const LongitudeAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$T(cacheKeySink, requestedField, LongitudeAggregateState);
                break;
            }
            case 'PercentAggregate': {
                const PercentAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1u(cacheKeySink, requestedField, PercentAggregateState);
                break;
            }
            case 'PhoneNumberAggregate': {
                const PhoneNumberAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$R(cacheKeySink, requestedField, PhoneNumberAggregateState);
                break;
            }
            case 'PicklistAggregate': {
                const PicklistAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$Q(cacheKeySink, requestedField, PicklistAggregateState);
                break;
            }
            case 'StringAggregate': {
                const StringAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$1r(cacheKeySink, requestedField, StringAggregateState);
                break;
            }
            case 'TextAreaAggregate': {
                const TextAreaAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$P(cacheKeySink, requestedField, TextAreaAggregateState);
                break;
            }
            case 'TimeAggregate': {
                const TimeAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$O(cacheKeySink, requestedField, TimeAggregateState);
                break;
            }
            case 'UrlAggregate': {
                const UrlAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$N(cacheKeySink, requestedField, UrlAggregateState);
                break;
            }
            case 'Setup__SetupRecordAggregate': {
                const parentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$f(cacheKeySink, requestedField, parentRelationshipState);
                break;
            }
            case 'Setup__SetupPolymorphicAggregateParentRelationship': {
                const polymorphicParentRelationshipState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$g(cacheKeySink, requestedField, polymorphicParentRelationshipState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$k(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'BooleanAggregate': {
            return {
                isArray: false,
                typename: 'BooleanAggregate'
            };
        }
        case 'CurrencyAggregate': {
            return {
                isArray: false,
                typename: 'CurrencyAggregate'
            };
        }
        case 'DateAggregate': {
            return {
                isArray: false,
                typename: 'DateAggregate'
            };
        }
        case 'DoubleAggregate': {
            return {
                isArray: false,
                typename: 'DoubleAggregate'
            };
        }
        case 'EmailAggregate': {
            return {
                isArray: false,
                typename: 'EmailAggregate'
            };
        }
        case 'IDAggregate': {
            return {
                isArray: false,
                typename: 'IDAggregate'
            };
        }
        case 'IntAggregate': {
            return {
                isArray: false,
                typename: 'IntAggregate'
            };
        }
        case 'LatitudeAggregate': {
            return {
                isArray: false,
                typename: 'LatitudeAggregate'
            };
        }
        case 'LongAggregate': {
            return {
                isArray: false,
                typename: 'LongAggregate'
            };
        }
        case 'LongitudeAggregate': {
            return {
                isArray: false,
                typename: 'LongitudeAggregate'
            };
        }
        case 'PercentAggregate': {
            return {
                isArray: false,
                typename: 'PercentAggregate'
            };
        }
        case 'PhoneNumberAggregate': {
            return {
                isArray: false,
                typename: 'PhoneNumberAggregate'
            };
        }
        case 'PicklistAggregate': {
            return {
                isArray: false,
                typename: 'PicklistAggregate'
            };
        }
        case 'StringAggregate': {
            return {
                isArray: false,
                typename: 'StringAggregate'
            };
        }
        case 'TextAreaAggregate': {
            return {
                isArray: false,
                typename: 'TextAreaAggregate'
            };
        }
        case 'TimeAggregate': {
            return {
                isArray: false,
                typename: 'TimeAggregate'
            };
        }
        case 'UrlAggregate': {
            return {
                isArray: false,
                typename: 'UrlAggregate'
            };
        }
        case 'parentRelationship': {
            return {
                isArray: false,
                typename: 'Setup__SetupRecordAggregate'
            };
        }
        case 'polymorphicParentRelationship': {
            return {
                isArray: false,
                typename: 'Setup__SetupPolymorphicAggregateParentRelationship'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$6(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'BooleanAggregate': {
            const BooleanAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$$(requestedField, BooleanAggregateState);
            break;
        }
        case 'CurrencyAggregate': {
            const CurrencyAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$_(requestedField, CurrencyAggregateState);
            break;
        }
        case 'DateAggregate': {
            const DateAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$Y(requestedField, DateAggregateState);
            break;
        }
        case 'DoubleAggregate': {
            const DoubleAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$X(requestedField, DoubleAggregateState);
            break;
        }
        case 'EmailAggregate': {
            const EmailAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$W(requestedField, EmailAggregateState);
            break;
        }
        case 'IDAggregate': {
            const IDAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$V(requestedField, IDAggregateState);
            break;
        }
        case 'IntAggregate': {
            const IntAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$U(requestedField, IntAggregateState);
            break;
        }
        case 'LatitudeAggregate': {
            const LatitudeAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$T(requestedField, LatitudeAggregateState);
            break;
        }
        case 'LongAggregate': {
            const LongAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$R(requestedField, LongAggregateState);
            break;
        }
        case 'LongitudeAggregate': {
            const LongitudeAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$S(requestedField, LongitudeAggregateState);
            break;
        }
        case 'PercentAggregate': {
            const PercentAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1t(requestedField, PercentAggregateState);
            break;
        }
        case 'PhoneNumberAggregate': {
            const PhoneNumberAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$Q(requestedField, PhoneNumberAggregateState);
            break;
        }
        case 'PicklistAggregate': {
            const PicklistAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$P(requestedField, PicklistAggregateState);
            break;
        }
        case 'StringAggregate': {
            const StringAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$1q(requestedField, StringAggregateState);
            break;
        }
        case 'TextAreaAggregate': {
            const TextAreaAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$O(requestedField, TextAreaAggregateState);
            break;
        }
        case 'TimeAggregate': {
            const TimeAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$N(requestedField, TimeAggregateState);
            break;
        }
        case 'UrlAggregate': {
            const UrlAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$M(requestedField, UrlAggregateState);
            break;
        }
        case 'Setup__SetupRecordAggregate': {
            const Setup__SetupRecordAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$e(requestedField, Setup__SetupRecordAggregateState);
            break;
        }
        case 'Setup__SetupPolymorphicAggregateParentRelationship': {
            const Setup__SetupPolymorphicAggregateParentRelationshipState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$f(requestedField, Setup__SetupPolymorphicAggregateParentRelationshipState);
            break;
        }
    }
}
function isMappedField$6(field) {
    return false;
}
function isFragmentApplicable$7(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupRecordAggregate", typename);
}
function getInContextFragmentType$l(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$5 = 'Setup__SetupRecordResult';
const VERSION$h = 'daf27c777ff084ce986416e1b392ccf8';
function keyBuilder$j(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$5(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$d(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$j(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$6,
        isMappedFieldImport: isMappedField$5,
        getFieldType: getFieldType$j,
        ingestFieldByType: ingestFieldByType$5,
        createLink: createLink,
        mergeData: mergeData$5,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupRecordResult",
            version: VERSION$h,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$5(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$j(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$5(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupRecordAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$i, select$f, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$6(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$5, getFieldType$j, selectType$5, parentRecordId);
}
function buildSelectionForNode$6(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$6, buildSelectionForField$6, parentRecordId);
}
function select$e(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$6, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$l() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$e(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$j(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$5
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$6) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$j(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupRecordAggregate': {
                const aggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$f(cacheKeySink, requestedField, aggregateState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$j(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'aggregate': {
            return {
                isArray: false,
                typename: 'Setup__SetupRecordAggregate'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$5(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupRecordAggregate': {
            const Setup__SetupRecordAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$e(requestedField, Setup__SetupRecordAggregateState);
            break;
        }
    }
}
function isMappedField$5(field) {
    return false;
}
function isFragmentApplicable$6(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupRecordResult", typename);
}
function getInContextFragmentType$k(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$4 = 'Setup__SetupAggregateEdge';
const VERSION$g = '3c03396221f538e0cd1d5076c30cd8ee';
function keyBuilder$i(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$4(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$c(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$i(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$5,
        isMappedFieldImport: isMappedField$4,
        getFieldType: getFieldType$i,
        ingestFieldByType: ingestFieldByType$4,
        createLink: createLink,
        mergeData: mergeData$4,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupAggregateEdge",
            version: VERSION$g,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$4(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$i(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$4(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupRecordResult': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$h, select$e, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$5(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$4, getFieldType$i, selectType$4, parentRecordId);
}
function buildSelectionForNode$5(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$5, buildSelectionForField$5, parentRecordId);
}
function select$d(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$5, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$k() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$d(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$i(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$4
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$5) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$i(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupRecordResult': {
                const nodeState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$e(cacheKeySink, requestedField, nodeState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$i(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'Setup__SetupRecordResult'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$4(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupRecordResult': {
            const Setup__SetupRecordResultState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$d(requestedField, Setup__SetupRecordResultState);
            break;
        }
    }
}
function isMappedField$4(field) {
    return false;
}
function isFragmentApplicable$5(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupAggregateEdge", typename);
}
function getInContextFragmentType$j(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$3 = 'Setup__SetupAggregateConnection';
const VERSION$f = '25dc88119696a06f67107fb7fd7f1aba';
function keyBuilder$h(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$3(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingestPaginationMetadata(astNode, state, key, sink, existingData) {
    var _a;
    const { data, luvio, store } = state;
    const { pageInfo: { hasNextPage, startCursor }, edges } = data;
    if (!hasGraphQLPaginationFields(data))
        return;
    const paginationKey = key + '__pagination';
    let existingPaginationMetadata = store.readEntry(paginationKey);
    const afterArg = (_a = astNode.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', state.variables);
    const isPagedResponse = afterArgVal !== undefined &&
        existingPaginationMetadata !== undefined &&
        existingPaginationMetadata[afterArgVal] !== undefined;
    if (isPagedResponse && existingPaginationMetadata !== undefined) {
        const previousPageOffset = existingPaginationMetadata[afterArgVal];
        existingPaginationMetadata = {
            ...existingPaginationMetadata,
            [startCursor]: previousPageOffset + 1,
        };
    }
    // Default to a zero offset if we're not a result of a paged request
    const startOffset = isPagedResponse && existingPaginationMetadata !== undefined
        ? offsetFromToken(existingPaginationMetadata, startCursor)
        : 0;
    if (process.env.NODE_ENV !== 'production') {
        if (startOffset < 0) {
            throw new RangeError(`Cursor value ${startCursor} is not recognized. Cannot ingest the returned page. After: ${afterArgVal}`);
        }
    }
    const nextOffset = startOffset + edges.length;
    const incomingPaginationMetadata = createPaginationMetadata(startOffset, edges);
    if (hasNextPage) {
        // we know the data extends beyond NEXT_OFFSET_IDENTIFIER; clean up any old __END__ markers that say the data ended at/before that offset
        if (existingPaginationMetadata !== undefined && existingPaginationMetadata.__END__ !== undefined && existingPaginationMetadata.__END__ <= nextOffset) {
            existingPaginationMetadata = { ...existingPaginationMetadata };
            delete existingPaginationMetadata.__END__;
        }
    }
    else {
        incomingPaginationMetadata.__END__ = nextOffset;
    }
    // store the updated pagination metadata
    const mergedPaginationMetadata = mergeMetadata$1(existingPaginationMetadata, incomingPaginationMetadata);
    luvio.storePublish(paginationKey, mergedPaginationMetadata);
    luvio.publishStoreMetadata(paginationKey, paginationStoreMetadata$1);
    // merge the edges
    assignMetadataLink(sink, paginationKey);
    const existingRecord = existingData ? existingData.data : undefined;
    if (existingRecord !== undefined && existingRecord.edges !== undefined) {
        sink.edges = [
            ...existingRecord.edges.slice(0, startOffset),
            ...sink.edges,
            ...existingRecord.edges.slice(nextOffset),
        ];
    }
}
function ingest$b(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$h(luvio, path);
    return ingestCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$4,
        isMappedFieldImport: isMappedField$3,
        getFieldType: getFieldType$h,
        ingestFieldByType: ingestFieldByType$3,
        hasGraphQLPaginationFields: hasGraphQLPaginationFields,
        ingestPaginatedMetadata: ingestPaginationMetadata,
        createLink: createLink,
        mergeData: mergeData$3,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupAggregateConnection",
            version: VERSION$f,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$3(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$h(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$3(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupAggregateEdge': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$g, select$d, false);
            break;
        }
        case 'PageInfo': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$18, select$15, false);
            break;
        }
        case 'Int': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$4(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, true, getFieldData$3, getFieldType$h, selectType$3, parentRecordId);
}
function buildSelectionForNode$4(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$4, buildSelectionForField$4, parentRecordId);
}
function select$c(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$4, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$j() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        ...getCursorConnectionMinimumSelections()
    ];
    return minimumSelections;
}
function getTypeCacheKeys$c(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$h(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$3
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$4) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$h(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupAggregateEdge': {
                const fullPath__edges = `${rootKey}__${fieldKey}`;
                for (let i = 0, len = fieldData.length; i < len; i++) {
                    const item = fieldData[i];
                    const itemState = {
                        ...state,
                        path: {
                            parent: null,
                            propertyName: null,
                            fullPath: fullPath__edges + '__' + i
                        },
                        data: item,
                    };
                    getTypeCacheKeys$d(cacheKeySink, requestedField, itemState);
                }
                break;
            }
            case 'PageInfo': {
                const pageInfoState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$15(cacheKeySink, requestedField, pageInfoState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$h(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'Setup__SetupAggregateEdge'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$3(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupAggregateEdge': {
            const Setup__SetupAggregateEdgeState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$c(requestedField, Setup__SetupAggregateEdgeState);
            break;
        }
        case 'PageInfo': {
            const PageInfoState = buildFieldState(state, fullPathKey, fieldData);
            sink[fieldKey] = ingest$14(requestedField, PageInfoState);
            break;
        }
        case 'Int': {
            sink[fieldKey] = fieldData;
            break;
        }
    }
}
function isMappedField$3(field) {
    return false;
}
function isFragmentApplicable$4(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupAggregateConnection", typename);
}
function getInContextFragmentType$i(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getFieldType$g(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String',
            };
        }
        default: {
            return {
                isArray: false,
                typename: 'Setup__SetupAggregateConnection',
            };
        }
    }
}

const name$2 = 'Setup__SetupQueryAggregate';
const VERSION$e = '0592284764c8e58016880e291b9ffb64';
function keyBuilder$g(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$2(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$a(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$g(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$3,
        isMappedFieldImport: isMappedField$2,
        getFieldType: getFieldType$g,
        ingestFieldByType: ingestFieldByType$2,
        createLink: createLink,
        mergeData: mergeData$2,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Setup__SetupQueryAggregate",
            version: VERSION$e,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$2(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$g(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$2(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupAggregateConnection': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$f, select$c, true);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$3(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$2, getFieldType$g, selectType$2, parentRecordId);
}
function buildSelectionForNode$3(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$3, buildSelectionForField$3, parentRecordId);
}
function select$b(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$3, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$i() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$b(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$g(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$2
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$3) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$g(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupAggregateConnection': {
                const recordQueryAggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$c(cacheKeySink, requestedField, recordQueryAggregateState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function ingestFieldByType$2(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupAggregateConnection': {
            const Setup__SetupAggregateConnectionState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$b(requestedField, Setup__SetupAggregateConnectionState);
            break;
        }
    }
}
function isMappedField$2(field) {
    return false;
}
function isFragmentApplicable$3(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__SetupQueryAggregate", typename);
}
function getInContextFragmentType$h(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name$1 = 'Setup__Setup';
const TTL$7 = 1000;
const VERSION$d = 'c9078f9a3c6d16c68179c4007365a008';
function keyBuilder$f(luvio, path, data) {
    return path.fullPath;
}
// W-12356727: To be switched over once path.fullPath is a structured key.
// export function keyBuilder_StructuredKey(luvio: $64$luvio_engine_Luvio, path: $64$luvio_engine_IngestPath, data: PartialInterface): $64$luvio_engine_NormalizedKeyMetadata {
//     return path.fullPath;
// }
function mergeData$1(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$9(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$f(luvio, path);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$2,
        isMappedFieldImport: isMappedField$1,
        getFieldType: getFieldType$f,
        ingestFieldByType: ingestFieldByType$1,
        createLink: createLink,
        mergeData: mergeData$1,
        storeMetadataParams: {
            ttl: TTL$7,
            namespace: keyPrefix,
            representationName: "Setup__Setup",
            version: VERSION$d,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData$1(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$f(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType$1(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'Setup__SetupQuery': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$k, select$h, false);
            break;
        }
        case 'Setup__SetupQueryAggregate': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$e, select$b, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$2(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData$1, getFieldType$f, selectType$1, parentRecordId);
}
function buildSelectionForNode$2(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$2, buildSelectionForField$2, parentRecordId);
}
function select$a(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$2, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$h() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getTypeCacheKeys$a(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$f(luvio, path);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name$1
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$2) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$f(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'Setup__SetupQuery': {
                const queryState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$h(cacheKeySink, requestedField, queryState);
                break;
            }
            case 'Setup__SetupQueryAggregate': {
                const aggregateState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$b(cacheKeySink, requestedField, aggregateState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$f(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'query': {
            return {
                isArray: false,
                typename: 'Setup__SetupQuery'
            };
        }
        case 'aggregate': {
            return {
                isArray: false,
                typename: 'Setup__SetupQueryAggregate'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType$1(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'Setup__SetupQuery': {
            const Setup__SetupQueryState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$g(requestedField, Setup__SetupQueryState);
            break;
        }
        case 'Setup__SetupQueryAggregate': {
            const Setup__SetupQueryAggregateState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$a(requestedField, Setup__SetupQueryAggregateState);
            break;
        }
    }
}
function isMappedField$1(field) {
    return false;
}
function isFragmentApplicable$2(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Setup__Setup", typename);
}
function getInContextFragmentType$g(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const name = 'Query';
const VERSION$c = '54885a089621803969258215830eec97';
function keyBuilder$e(luvio, operationNode, variables, fragmentMap) {
    return buildQueryTypeStringKey({
        luvio,
        keyPrefix: keyPrefix,
        schemaName: 'uiapi',
        queryTypeName: name,
        operationNode,
        variables,
        fragmentMap
    });
}
function mergeData(existingData, newData) {
    return {
        data: ObjectAssign({}, existingData["data"], newData["data"]),
    };
}
function ingest$8(astNode, state) {
    const { path, data, timestamp, luvio } = state;
    const key = keyBuilder$e(luvio, astNode, state.variables, state.fragments);
    return ingestNonCursorConnectionType(astNode, state, {
        key,
        isFragmentApplicable: isFragmentApplicable$1,
        isMappedFieldImport: isMappedField,
        getFieldType: getFieldType$e,
        ingestFieldByType,
        createLink: createLink,
        mergeData,
        storeMetadataParams: {
            ttl: TTL$6,
            namespace: keyPrefix,
            representationName: "Query",
            version: VERSION$c,
            ingestionTimestamp: timestamp,
        },
    });
}
function getFieldData(source, sel, variables) {
    switch (sel.name.value) {
        default: {
            const fieldType = getFieldType$e(sel);
            const fieldKey = getSerializedKeyForField(sel, variables, fieldType);
            return source.data[fieldKey];
        }
    }
}
function selectType(typename, sel, fieldData, reader, key, sink, variables, fragments, parentRecordId) {
    if (fieldData === null) {
        reader.markSeenId(parentRecordId);
        reader.assignScalar(key, sink, fieldData);
        return sink;
    }
    switch (typename) {
        case 'String': {
            reader.markSeenId(parentRecordId);
            reader.assignScalar(key, sink, fieldData);
            break;
        }
        case 'UIAPI': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$y, select$v, false);
            break;
        }
        case 'Analytics__Analytics': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$q, select$n, false);
            break;
        }
        case 'Setup__Setup': {
            selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, VERSION$d, select$a, false);
            break;
        }
    }
    return sink;
}
function buildSelectionForField$1(source, reader, sel, variables, fragments, parentRecordId) {
    return buildSelectionForField(source, reader, sel, variables, fragments, false, getFieldData, getFieldType$e, selectType, parentRecordId);
}
function buildSelectionForNode$1(source, reader, parentNode, selectionNode, variables, fragments, parentRecordId) {
    return buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable$1, buildSelectionForField$1, parentRecordId);
}
function select$9(field, variables, fragments) {
    return (source, reader, parentRecordId) => {
        const sink = {};
        return selectCalculateSink(sink, field, buildSelectionForNode$1, source, reader, variables, fragments, parentRecordId);
    };
}
function getMinimumSelections$g() {
    const minimumSelections = [];
    return minimumSelections;
}
function getTypeCacheKeys$9(cacheKeySink, astNode, state) {
    const { path, data, luvio } = state;
    const rootKey = keyBuilder$e(luvio, astNode, state.variables, state.fragments);
    cacheKeySink.set(rootKey, {
        namespace: keyPrefix,
        mergeable: true,
        representationName: name
    });
    // get all cache keys from normalizable fields
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable$1) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // Skip unknown field.
        }
        const fieldType = getFieldType$e(requestedField);
        if (fieldType === undefined) {
            continue;
        }
        const fieldKey = getSerializedKeyForField(requestedField, state.variables, fieldType);
        switch (fieldType.typename) {
            case 'UIAPI': {
                const uiapiState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$v(cacheKeySink, requestedField, uiapiState);
                break;
            }
            case 'Analytics__Analytics': {
                const analyticsState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$n(cacheKeySink, requestedField, analyticsState);
                break;
            }
            case 'Setup__Setup': {
                const setupState = buildFieldState(state, rootKey + '__' + fieldKey, fieldData);
                getTypeCacheKeys$a(cacheKeySink, requestedField, setupState);
                break;
            }
        }
    }
    // Deal with mapped types' cache keys
}
function getFieldType$e(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'uiapi': {
            return {
                isArray: false,
                typename: 'UIAPI'
            };
        }
        case 'analytics': {
            return {
                isArray: false,
                typename: 'Analytics__Analytics'
            };
        }
        case 'setup': {
            return {
                isArray: false,
                typename: 'Setup__Setup'
            };
        }
        default:
            return undefined;
    }
}
function ingestFieldByType(typename, parentKey, requestedField, sink, fieldKey, fieldData, state) {
    // TODO: add validation logic to only allow nullable fields to be null in the future
    if (fieldData === null) {
        sink[fieldKey] = null;
        return;
    }
    switch (typename) {
        case 'String': {
            sink[fieldKey] = fieldData;
            break;
        }
        case 'UIAPI': {
            const UIAPIState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$u(requestedField, UIAPIState);
            break;
        }
        case 'Analytics__Analytics': {
            const Analytics__AnalyticsState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$m(requestedField, Analytics__AnalyticsState);
            break;
        }
        case 'Setup__Setup': {
            const Setup__SetupState = buildFieldState(state, parentKey + '__' + fieldKey, fieldData);
            sink[fieldKey] = ingest$9(requestedField, Setup__SetupState);
            break;
        }
    }
}
function isMappedField(field) {
    return false;
}
function isFragmentApplicable$1(fragmentNode, typename) {
    return isFragmentApplicable(fragmentNode, [], "Query", typename);
}
function getInContextFragmentType$f(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$f() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$d(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'Date'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$e(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$e() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$c(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'Record': {
            return {
                isArray: false,
                typename: 'RecordRepresentation'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$d(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const DEFAULT_FIELD_VALUE_SELECTIONS = [
    {
        kind: 'Field',
        name: {
            kind: 'Name',
            value: '__typename',
        },
    },
    {
        kind: 'Field',
        name: {
            kind: 'Name',
            value: 'value',
        },
    },
    {
        kind: 'Field',
        name: {
            kind: 'Name',
            value: 'displayValue',
        },
    },
];
// All FieldValues should use theseMinimumSelections
function getMinimumSelections$d() {
    return DEFAULT_FIELD_VALUE_SELECTIONS;
}

function getMinimumSelections$c() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$b(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'value': {
            return {
                isArray: false,
                typename: 'DateTime'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'calendarMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'calendarQuarter': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'calendarYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'count': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'countDistinct': {
            return {
                isArray: false,
                typename: 'LongValue'
            };
        }
        case 'dayInMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'dayInWeek': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'dayInYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'dayOnly': {
            return {
                isArray: false,
                typename: 'DateOnlyAggregation'
            };
        }
        case 'fiscalMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'fiscalQuarter': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'fiscalYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'format': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'hourInDay': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'max': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'min': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'weekInMonth': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        case 'weekInYear': {
            return {
                isArray: false,
                typename: 'DateFunctionAggregation'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$c(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$b() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$a(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'displayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$b(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$a() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$9(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'success': {
            return {
                isArray: false,
                typename: 'Boolean'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$a(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$9() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$8(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$9(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

const defaultFieldValueSelection = {
    kind: 'SelectionSet',
    selections: DEFAULT_FIELD_VALUE_SELECTIONS,
};
const recordTypeIdFieldWithFallback = {
    kind: 'Field',
    name: {
        kind: 'Name',
        value: 'RecordTypeId',
    },
    arguments: [
        {
            kind: 'Argument',
            name: {
                kind: 'Name',
                value: 'fallback',
            },
            value: {
                kind: 'BooleanValue',
                value: true,
            },
        },
    ],
    alias: {
        kind: 'Name',
        value: 'ldsRecordTypeId', // Alias fields that have arguments to avoid a conflict.
    },
    selectionSet: defaultFieldValueSelection,
};
/**
 * This is a special inline fragment that can be applied to both PolymorphicParentRelationship fields
 * and parentRelationship Fields. It looks just like the defaultRecord fields fragment from the handrolled adapter.
 *
 *
 *
 */
const minimumRecordFieldsFragment = {
    kind: 'InlineFragment',
    typeCondition: {
        kind: 'NamedType',
        name: {
            kind: 'Name',
            value: 'Record',
        },
    },
    selectionSet: {
        kind: 'SelectionSet',
        selections: [
            getFieldSelection('__typename'),
            getFieldSelection('Id'),
            getFieldSelection('DisplayValue'),
            getFieldSelection('ApiName'),
            getFieldSelection('WeakEtag'),
            getFieldSelection('LastModifiedById', defaultFieldValueSelection),
            getFieldSelection('LastModifiedDate', defaultFieldValueSelection),
            getFieldSelection('SystemModstamp', defaultFieldValueSelection),
            recordTypeIdFieldWithFallback,
        ],
    },
};
function getMinimumSelections$8() {
    return [minimumRecordFieldsFragment];
}
function getFieldSelection(name, selectionSet) {
    return {
        kind: 'Field',
        name: {
            kind: 'Name',
            value: name,
        },
        selectionSet,
    };
}

// RecordRepresentation is a generic type, so all we can do is the default if it's Record, else we assume RecordRepresentation
function getInContextFragmentType$8(fragment, fragmentMap) {
    const fragmentType = sharedGetFragmentType(fragment, fragmentMap);
    return fragmentType === 'Record' ? 'Record' : 'RecordRepresentation';
}

function getMinimumSelections$7() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$7(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'cost': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'limit': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'remaining': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'resetAt': {
            return {
                isArray: false,
                typename: 'DateTime'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$7(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$6() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$6(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'recordCreate': {
            return {
                isArray: false,
                typename: 'RecordCreatePayload'
            };
        }
        case 'recordDelete': {
            return {
                isArray: false,
                typename: 'RecordDeletePayload'
            };
        }
        case 'recordUpdate': {
            return {
                isArray: false,
                typename: 'RecordUpdatePayload'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$6(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$5() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$5(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'WeakEtag': {
            return {
                isArray: false,
                typename: 'Long'
            };
        }
        case 'DisplayValue': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'LastModifiedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'SystemModstamp': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'RecordTypeId': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$5(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$4() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$4(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$4(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$3() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$3(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'node': {
            return {
                isArray: false,
                typename: 'Analytics__SemanticSubMetricDefinitionRepresentation'
            };
        }
        case 'cursor': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$3(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$2() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        ...getCursorConnectionMinimumSelections()
    ];
    return minimumSelections;
}
function getFieldType$2(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'edges': {
            return {
                isArray: true,
                typename: 'Analytics__SemanticSubMetricDefinitionEdge'
            };
        }
        case 'totalCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageResultCount': {
            return {
                isArray: false,
                typename: 'Int'
            };
        }
        case 'pageInfo': {
            return {
                isArray: false,
                typename: 'PageInfo'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$2(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections$1() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType$1(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'MasterLabel': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'DeveloperName': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'CreatedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'CreatedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'NamespacePrefix': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'AnalyticsWorkspaces': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsWorkspaceAssetConnection'
            };
        }
        case 'SubMetrics': {
            return {
                isArray: false,
                typename: 'Analytics__SemanticSubMetricDefinitionConnection'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType$1(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function getMinimumSelections() {
    const minimumSelections = [
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: '__typename',
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'Id',
            },
        },
    ];
    return minimumSelections;
}
function getFieldType(field) {
    switch (field.name.value) {
        case '__typename': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'MasterLabel': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'DeveloperName': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'CreatedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedById': {
            return {
                isArray: false,
                typename: 'IDValue'
            };
        }
        case 'LastModifiedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'CreatedDate': {
            return {
                isArray: false,
                typename: 'DateTimeValue'
            };
        }
        case 'Id': {
            return {
                isArray: false,
                typename: 'ID'
            };
        }
        case 'ApiName': {
            return {
                isArray: false,
                typename: 'String'
            };
        }
        case 'NamespacePrefix': {
            return {
                isArray: false,
                typename: 'StringValue'
            };
        }
        case 'AnalyticsWorkspaces': {
            return {
                isArray: false,
                typename: 'Analytics__AnalyticsWorkspaceAssetConnection'
            };
        }
        default:
            return undefined;
    }
}
function getInContextFragmentType(fragment, fragmentMap) {
    return sharedGetFragmentType(fragment, fragmentMap);
}

function applyToSelection$6(selection, directive, variables) {
    return selection;
}

function applyToSelection$5(selection, directive, variables) {
    return selection;
}

function applyToSelection$4(selection, directive, variables) {
    return selection;
}

function applyToSelection$3(selection, directive, variables) {
    var _a;
    const ifArg = (_a = directive.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => argument.name.value === "if");
    if (ifArg && ifArg.value) {
        if (ifArg.value.kind === "BooleanValue") {
            return ifArg.value.value ? selection : undefined;
        }
        if (ifArg.value.kind === "Variable") {
            return variables[ifArg.value.name.value] ? selection : undefined;
        }
    }
    return selection;
}

function applyToSelection$2(selection, directive, variables) {
    var _a;
    const ifArg = (_a = directive.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => argument.name.value === "if");
    if (ifArg && ifArg.value) {
        if (ifArg.value.kind === "BooleanValue") {
            return ifArg.value.value ? undefined : selection;
        }
        if (ifArg.value.kind === "Variable") {
            return variables[ifArg.value.name.value] ? undefined : selection;
        }
    }
    return selection;
}

function applyToSelection$1(selection, directive, variables) {
    return selection;
}

function applyToSelection(selection, directive, variables) {
    return selection;
}

function applyDirectivesForSelection(selection, variables) {
    let result = selection;
    if (selection && selection.directives) {
        // iterate through directives in order. Directives _can_ be stacked.
        for (var i = 0; i < selection.directives.length; i++) {
            const directive = selection.directives[i];
            switch (directive.name.value) {
                // Generate Case for each directive in the schema that supports SelectionNodes
                case ("generic"):
                    result = applyToSelection$6(selection);
                    break;
                case ("fieldCategory"):
                    result = applyToSelection$5(selection);
                    break;
                case ("category"):
                    result = applyToSelection$4(selection);
                    break;
                case ("include"):
                    result = applyToSelection$3(selection, directive, variables);
                    break;
                case ("skip"):
                    result = applyToSelection$2(selection, directive, variables);
                    break;
                case ("deprecated"):
                    result = applyToSelection$1(selection);
                    break;
                case ("specifiedBy"):
                    result = applyToSelection(selection);
                    break;
            }
            if (result === undefined) {
                // Selection was removed.
                break;
            }
        }
    }
    return result;
}

function createLink(ref) {
    return {
        __ref: serializeStructuredKey(ref),
    };
}
function ingestCursorConnectionType(astNode, state, specifics) {
    const { data } = state;
    const { key, ingestFieldByType: typeIngestFieldByType, hasGraphQLPaginationFields } = specifics;
    return ingestTypeWithStrategy(astNode, state, true, specifics, {
        getPaginationOffset() {
            return hasGraphQLPaginationFields(data)
                ? getArrayOffsetForPage(state, astNode, key)
                : 0;
        },
        ingestFieldByType(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
            typeIngestFieldByType(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state);
        }
    });
}
function ingestNonCursorConnectionType(astNode, state, specifics) {
    const { ingestFieldByType: typeIngestFieldByType } = specifics;
    return ingestTypeWithStrategy(astNode, state, false, specifics, {
        getPaginationOffset() {
            return 0;
        },
        ingestFieldByType(typename, parentKey, fullPathKey, requestedField, sink, fieldKey, fieldData, state) {
            typeIngestFieldByType(typename, parentKey, requestedField, sink, fieldKey, fieldData, state);
        }
    });
}
function ingestTypeWithStrategy(astNode, state, isCursorConnectionType, { key, isFragmentApplicable, isMappedFieldImport, getFieldType, ramlIngestBlock, ingestPaginatedMetadata, mergeData, storeMetadataParams, createLink, }, strategy) {
    const { data, variables, luvio, store } = state;
    const sink = {};
    const requestedFields = astNode.selectionSet ? getRequestedFieldsForType(data.__typename, astNode.selectionSet, state.fragments, isFragmentApplicable) : new Map();
    for (const fieldDataKey of Object.keys(data)) {
        const fieldData = data[fieldDataKey];
        const requestedField = requestedFields.get(fieldDataKey);
        if (requestedField === undefined) {
            continue; // TODO: (W-11132802) We got a field back we didn't ask for. Call handleUnknownField.
        }
        if (isMappedFieldImport(requestedField)) {
            continue; // Skips scalar fields that are mapped to RAML.
        }
        const fieldType = getFieldType(requestedField);
        const fieldKey = getSerializedKeyForField(requestedField, variables, fieldType);
        if (fieldType === undefined) {
            console.warn(`Unknown Field: ${requestedField}. This field or nested fields cannot participate in normalization`);
            deepFreeze(fieldData);
            sink[fieldKey] = fieldData;
            continue;
        }
        if (isCursorConnectionType) {
            if (fieldKey === 'pageInfo')
                continue; // pageInfo is always synthesized from pagination metadata
        }
        if (fieldData === null) {
            sink[fieldKey] = fieldData;
            continue; // Don't delegate null to type ingest.
        }
        if (fieldType.isArray && fieldData.length !== undefined) {
            const arrayLength = fieldData.length;
            const arraySink = new Array(arrayLength);
            let paginationOffset = strategy.getPaginationOffset();
            for (let i = 0, len = arrayLength; i < len; i += 1) {
                const arrayFullPathKey = key + '__' + fieldKey;
                strategy.ingestFieldByType(fieldType.typename, arrayFullPathKey, arrayFullPathKey + '__' + (i + paginationOffset), requestedField, arraySink, i, fieldData[i], state);
            }
            sink[fieldKey] = arraySink;
        }
        else {
            strategy.ingestFieldByType(fieldType.typename, key, key + '__' + fieldKey, requestedField, sink, fieldKey, fieldData, state);
        }
    }
    // merge existing cache data and new request response data
    const newData = { data: sink };
    if (ramlIngestBlock !== undefined) {
        ramlIngestBlock(newData);
    }
    const existingData = store.readEntry(key);
    if (ingestPaginatedMetadata !== undefined) {
        ingestPaginatedMetadata(astNode, state, key, sink, existingData);
    }
    if (existingData !== undefined && existingData !== null && ObjectPrototypeHasOwnProperty.call(existingData, "__type") === false) {
        const mergedData = mergeData(existingData, newData);
        if (!equalsObject(existingData, mergedData, (propA, propB) => stableJSONStringify(propA) === stableJSONStringify(propB))) { // naive impl
            luvio.storePublish(key, mergedData);
        }
    }
    else {
        luvio.storePublish(key, newData);
    }
    luvio.publishStoreMetadata(key, storeMetadataParams);
    return createLink(key);
}
const TTL$6 = 900000;
function applyMinimumFieldsToQuery(query) {
    const fragmentMap = createFragmentMap(query);
    query.definitions.forEach(queryDefinitionNode => {
        if (queryDefinitionNode.kind === 'OperationDefinition' && queryDefinitionNode.operation === 'query') {
            injectSelectionSet(queryDefinitionNode.selectionSet, getQueryTransformerForType("Query"), fragmentMap);
        }
    });
    return query;
}
function injectSelectionSet(selectionSetNode, queryTransformHelper, fragmentMap) {
    if (queryTransformHelper === undefined ||
        queryTransformHelper === null ||
        selectionSetNode === undefined) {
        return;
    }
    const minimumSelectionSet = {
        kind: 'SelectionSet',
        selections: queryTransformHelper.getMinimumSelections()
    };
    const mergedSelections = mergeSelectionSets(selectionSetNode, minimumSelectionSet);
    mergedSelections === null || mergedSelections === void 0 ? void 0 : mergedSelections.selections.forEach(selection => {
        if (selection.kind === 'Field') {
            const fieldType = queryTransformHelper.getFieldType(selection);
            const fieldTransformHelper = fieldType === undefined ? undefined : getQueryTransformerForType(fieldType.typename);
            if (fieldTransformHelper && !selection.selectionSet) {
                Object.assign(selection, {
                    selectionSet: {
                        kind: 'SelectionSet',
                        selections: []
                    }
                });
            }
            injectSelectionSet(selection.selectionSet, fieldTransformHelper, fragmentMap);
        }
        else {
            const fragmentType = queryTransformHelper.getInContextFragmentType(selection, fragmentMap);
            const fragmentSelections = getSelectionSetFromFragment(selection, fragmentMap);
            if (fragmentType === undefined) { // Untyped or unknown fragment
                injectSelectionSet(fragmentSelections, queryTransformHelper, fragmentMap);
            }
            else {
                const fragmentTransformHelper = getQueryTransformerForType(fragmentType);
                if (fragmentTransformHelper === undefined) {
                    return;
                }
                injectSelectionSet(fragmentSelections, fragmentTransformHelper, fragmentMap);
            }
        }
    });
    Object.assign(selectionSetNode, mergedSelections);
}
function getPageMetadata(paginationMetadata, paginationParams) {
    const metadataProperties = {};
    const sink = {
        metadataProperties
    };
    if (paginationMetadata === undefined) {
        return sink;
    }
    const { token, pageSize } = paginationParams;
    const pageTokenAndOffset = getPageTokenAndOffset$1(paginationMetadata, token);
    if (pageTokenAndOffset === undefined) {
        return sink;
    }
    const [pageToken, startOffset] = pageTokenAndOffset;
    const endOffset = pageEndOffset$1(paginationMetadata, startOffset, pageSize);
    sink.startOffset = startOffset;
    sink.endOffset = endOffset;
    metadataProperties.startCursor = pageToken;
    const nextToken = getNextPageToken(paginationMetadata, endOffset);
    if (nextToken !== undefined) {
        metadataProperties.endCursor = nextToken;
    }
    return sink;
}
const CURSOR_CONNECTION_TYPES = [
    'RecordAggregateConnection',
    'Setup__SetupAggregateConnection',
    'Setup__SetupConnection',
    'RecordConnection',
    'Analytics__AnalyticsWorkspaceAssetConnection',
    'Analytics__SemanticSubMetricDefinitionConnection',
];
function isCursorConnectionType(fieldType) {
    return fieldType !== undefined && CURSOR_CONNECTION_TYPES.includes(fieldType.typename);
}
function getSerializedKeyForField(field, variables, fieldType) {
    let mutableArgumentNodes = Object.assign([], field.arguments);
    // Paginated types are stored as a consolidated collection.  Their keys should not include
    // the "first" and "after" pagination parameters.
    if (isCursorConnectionType(fieldType)) {
        mutableArgumentNodes = mutableArgumentNodes.filter(node => {
            return node.name.value !== 'first' && node.name.value !== 'after';
        });
    }
    // Serialize the arguments given to this field. Luvio can't "eval" these arguments atm
    const argumentString = mutableArgumentNodes.length > 0 ? '__' + serializeFieldArguments(mutableArgumentNodes, variables) : '';
    return field.name.value + argumentString; // It should be safe to always use the fieldName - If an alias is meaningful, there will be arguments on the key also.
}
function getQueryTransformerForType(typename, fragmentMap) {
    switch (typename) {
        case "PercentAggregate": return {
            getMinimumSelections: getMinimumSelections$1f,
            getFieldType: getFieldType$1z,
            getInContextFragmentType: getInContextFragmentType$1z
        };
        case "StringAggregate": return {
            getMinimumSelections: getMinimumSelections$1e,
            getFieldType: getFieldType$1w,
            getInContextFragmentType: getInContextFragmentType$1w
        };
        case "Query": return {
            getMinimumSelections: getMinimumSelections$g,
            getFieldType: getFieldType$e,
            getInContextFragmentType: getInContextFragmentType$f
        };
        case "DateOnlyAggregation": return {
            getMinimumSelections: getMinimumSelections$f,
            getFieldType: getFieldType$d,
            getInContextFragmentType: getInContextFragmentType$e
        };
        case "RecordCreatePayload": return {
            getMinimumSelections: getMinimumSelections$e,
            getFieldType: getFieldType$c,
            getInContextFragmentType: getInContextFragmentType$d
        };
        case "DateAggregate": return {
            getMinimumSelections: getMinimumSelections$14,
            getFieldType: getFieldType$12,
            getInContextFragmentType: getInContextFragmentType$13
        };
        case "RecordTypeInfo": return {
            getMinimumSelections: getMinimumSelections$I,
            getFieldType: getFieldType$G,
            getInContextFragmentType: getInContextFragmentType$H
        };
        case "BooleanValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1t,
            getInContextFragmentType: getInContextFragmentType$1t
        };
        case "ReferenceToInfo": return {
            getMinimumSelections: getMinimumSelections$K,
            getFieldType: getFieldType$I,
            getInContextFragmentType: getInContextFragmentType$J
        };
        case "FieldValue": return {
            getMinimumSelections: getMinimumSelections$b,
            getFieldType: getFieldType$a,
            getInContextFragmentType: getInContextFragmentType$b
        };
        case "LongitudeValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1i,
            getInContextFragmentType: getInContextFragmentType$1i
        };
        case "StringValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1x,
            getInContextFragmentType: getInContextFragmentType$1x
        };
        case "IntValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1y,
            getInContextFragmentType: getInContextFragmentType$1y
        };
        case "UrlValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1l,
            getInContextFragmentType: getInContextFragmentType$1l
        };
        case "LongAggregate": return {
            getMinimumSelections: getMinimumSelections$Z,
            getFieldType: getFieldType$X,
            getInContextFragmentType: getInContextFragmentType$Y
        };
        case "PhoneNumberAggregate": return {
            getMinimumSelections: getMinimumSelections$Y,
            getFieldType: getFieldType$W,
            getInContextFragmentType: getInContextFragmentType$X
        };
        case "PicklistValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1g,
            getInContextFragmentType: getInContextFragmentType$1g
        };
        case "CurrencyAggregate": return {
            getMinimumSelections: getMinimumSelections$16,
            getFieldType: getFieldType$14,
            getInContextFragmentType: getInContextFragmentType$15
        };
        case "RelatedListInfo": return {
            getMinimumSelections: getMinimumSelections$D,
            getFieldType: getFieldType$B,
            getInContextFragmentType: getInContextFragmentType$C
        };
        case "UIAPI": return {
            getMinimumSelections: getMinimumSelections$C,
            getFieldType: getFieldType$A,
            getInContextFragmentType: getInContextFragmentType$B
        };
        case "DateTimeAggregate": return {
            getMinimumSelections: getMinimumSelections$c,
            getFieldType: getFieldType$b,
            getInContextFragmentType: getInContextFragmentType$c
        };
        case "EmailAggregate": return {
            getMinimumSelections: getMinimumSelections$12,
            getFieldType: getFieldType$10,
            getInContextFragmentType: getInContextFragmentType$11
        };
        case "RichTextAreaValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1o,
            getInContextFragmentType: getInContextFragmentType$1o
        };
        case "MultiPicklistValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1f,
            getInContextFragmentType: getInContextFragmentType$1f
        };
        case "Setup__SetupEdge": return {
            getMinimumSelections: getMinimumSelections$q,
            getFieldType: getFieldType$o,
            getInContextFragmentType: getInContextFragmentType$p
        };
        case "TimeAggregate": return {
            getMinimumSelections: getMinimumSelections$V,
            getFieldType: getFieldType$T,
            getInContextFragmentType: getInContextFragmentType$U
        };
        case "ListColumn": return {
            getMinimumSelections: getMinimumSelections$F,
            getFieldType: getFieldType$D,
            getInContextFragmentType: getInContextFragmentType$E
        };
        case "Setup__SetupQuery": return {
            getMinimumSelections: getMinimumSelections$o,
            getFieldType: getFieldType$m,
            getInContextFragmentType: getInContextFragmentType$n
        };
        case "Setup__SetupQueryAggregate": return {
            getMinimumSelections: getMinimumSelections$i,
            getFieldType: getFieldType$g,
            getInContextFragmentType: getInContextFragmentType$h
        };
        case "Setup__SetupRecordAggregate": return {
            getMinimumSelections: getMinimumSelections$m,
            getFieldType: getFieldType$k,
            getInContextFragmentType: getInContextFragmentType$l
        };
        case "Setup__SetupRecordResult": return {
            getMinimumSelections: getMinimumSelections$l,
            getFieldType: getFieldType$j,
            getInContextFragmentType: getInContextFragmentType$k
        };
        case "SObject__Field": return {
            getMinimumSelections: getMinimumSelections$t,
            getFieldType: getFieldType$r,
            getInContextFragmentType: getInContextFragmentType$s
        };
        case "Setup__EntityRepresentation": return {
            getMinimumSelections: getMinimumSelections$r,
            getFieldType: getFieldType$p,
            getInContextFragmentType: getInContextFragmentType$q
        };
        case "LatitudeAggregate": return {
            getMinimumSelections: getMinimumSelections$$,
            getFieldType: getFieldType$Z,
            getInContextFragmentType: getInContextFragmentType$_
        };
        case "PolymorphicAggregateParentRelationship": return {
            getMinimumSelections: getMinimumSelections$T,
            getFieldType: getFieldType$R,
            getInContextFragmentType: getInContextFragmentType$S
        };
        case "PolymorphicParentRelationship": return {
            getMinimumSelections: getMinimumSelections$1d,
            getFieldType: getFieldType$1c,
            getInContextFragmentType: getInContextFragmentType$1c
        };
        case "LongTextAreaValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1p,
            getInContextFragmentType: getInContextFragmentType$1p
        };
        case "LatitudeValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1h,
            getInContextFragmentType: getInContextFragmentType$1h
        };
        case "RecordAggregateConnection": return {
            getMinimumSelections: getMinimumSelections$P,
            getFieldType: getFieldType$N,
            getInContextFragmentType: getInContextFragmentType$O
        };
        case "LongitudeAggregate": return {
            getMinimumSelections: getMinimumSelections$_,
            getFieldType: getFieldType$Y,
            getInContextFragmentType: getInContextFragmentType$Z
        };
        case "RecordEdge": return {
            getMinimumSelections: getMinimumSelections$1a,
            getFieldType: getFieldType$18,
            getInContextFragmentType: getInContextFragmentType$19
        };
        case "Setup__SetupPolymorphicParentRelationship": return {
            getMinimumSelections: getMinimumSelections$s,
            getFieldType: getFieldType$q,
            getInContextFragmentType: getInContextFragmentType$r
        };
        case "DateValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1r,
            getInContextFragmentType: getInContextFragmentType$1r
        };
        case "PhoneNumberValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1n,
            getInContextFragmentType: getInContextFragmentType$1n
        };
        case "DoubleAggregate": return {
            getMinimumSelections: getMinimumSelections$13,
            getFieldType: getFieldType$11,
            getInContextFragmentType: getInContextFragmentType$12
        };
        case "TextAreaValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1q,
            getInContextFragmentType: getInContextFragmentType$1q
        };
        case "RecordUpdatePayload": return {
            getMinimumSelections: getMinimumSelections$a,
            getFieldType: getFieldType$9,
            getInContextFragmentType: getInContextFragmentType$a
        };
        case "DoubleValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1C,
            getInContextFragmentType: getInContextFragmentType$1C
        };
        case "IDAggregate": return {
            getMinimumSelections: getMinimumSelections$11,
            getFieldType: getFieldType$$,
            getInContextFragmentType: getInContextFragmentType$10
        };
        case "RecordAggregateEdge": return {
            getMinimumSelections: getMinimumSelections$Q,
            getFieldType: getFieldType$O,
            getInContextFragmentType: getInContextFragmentType$P
        };
        case "ThemeInfo": return {
            getMinimumSelections: getMinimumSelections$H,
            getFieldType: getFieldType$F,
            getInContextFragmentType: getInContextFragmentType$G
        };
        case "RecordDeletePayload": return {
            getMinimumSelections: getMinimumSelections$9,
            getFieldType: getFieldType$8,
            getInContextFragmentType: getInContextFragmentType$9
        };
        case "UrlAggregate": return {
            getMinimumSelections: getMinimumSelections$U,
            getFieldType: getFieldType$S,
            getInContextFragmentType: getInContextFragmentType$T
        };
        case "RecordRepresentation": return {
            getMinimumSelections: getMinimumSelections$8,
            getFieldType: getFieldType$1a,
            getInContextFragmentType: getInContextFragmentType$8
        };
        case "IDValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1v,
            getInContextFragmentType: getInContextFragmentType$1v
        };
        case "TextAreaAggregate": return {
            getMinimumSelections: getMinimumSelections$W,
            getFieldType: getFieldType$U,
            getInContextFragmentType: getInContextFragmentType$V
        };
        case "TimeValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1s,
            getInContextFragmentType: getInContextFragmentType$1s
        };
        case "PicklistAggregate": return {
            getMinimumSelections: getMinimumSelections$X,
            getFieldType: getFieldType$V,
            getInContextFragmentType: getInContextFragmentType$W
        };
        case "DateTimeValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1u,
            getInContextFragmentType: getInContextFragmentType$1u
        };
        case "IntAggregate": return {
            getMinimumSelections: getMinimumSelections$10,
            getFieldType: getFieldType$_,
            getInContextFragmentType: getInContextFragmentType$$
        };
        case "ListOrder": return {
            getMinimumSelections: getMinimumSelections$E,
            getFieldType: getFieldType$C,
            getInContextFragmentType: getInContextFragmentType$D
        };
        case "RecordAggregate": return {
            getMinimumSelections: getMinimumSelections$S,
            getFieldType: getFieldType$Q,
            getInContextFragmentType: getInContextFragmentType$R
        };
        case "JSONValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1d,
            getInContextFragmentType: getInContextFragmentType$1d
        };
        case "EmailValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1m,
            getInContextFragmentType: getInContextFragmentType$1m
        };
        case "Setup__Setup": return {
            getMinimumSelections: getMinimumSelections$h,
            getFieldType: getFieldType$f,
            getInContextFragmentType: getInContextFragmentType$g
        };
        case "Setup__SetupAggregateConnection": return {
            getMinimumSelections: getMinimumSelections$j,
            getFieldType: getFieldType$h,
            getInContextFragmentType: getInContextFragmentType$i
        };
        case "Setup__SetupAggregateEdge": return {
            getMinimumSelections: getMinimumSelections$k,
            getFieldType: getFieldType$i,
            getInContextFragmentType: getInContextFragmentType$j
        };
        case "LongValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1B,
            getInContextFragmentType: getInContextFragmentType$1B
        };
        case "DependentField": return {
            getMinimumSelections: getMinimumSelections$M,
            getFieldType: getFieldType$K,
            getInContextFragmentType: getInContextFragmentType$L
        };
        case "Setup__SetupConnection": return {
            getMinimumSelections: getMinimumSelections$p,
            getFieldType: getFieldType$n,
            getInContextFragmentType: getInContextFragmentType$o
        };
        case "PercentValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1A,
            getInContextFragmentType: getInContextFragmentType$1A
        };
        case "BooleanAggregate": return {
            getMinimumSelections: getMinimumSelections$17,
            getFieldType: getFieldType$15,
            getInContextFragmentType: getInContextFragmentType$16
        };
        case "RecordQueryAggregate": return {
            getMinimumSelections: getMinimumSelections$O,
            getFieldType: getFieldType$M,
            getInContextFragmentType: getInContextFragmentType$N
        };
        case "RecordConnection": return {
            getMinimumSelections: getMinimumSelections$19,
            getFieldType: getFieldType$17,
            getInContextFragmentType: getInContextFragmentType$18
        };
        case "FilteredLookupInfo": return {
            getMinimumSelections: getMinimumSelections$L,
            getFieldType: getFieldType$J,
            getInContextFragmentType: getInContextFragmentType$K
        };
        case "ObjectInfo": return {
            getMinimumSelections: getMinimumSelections$G,
            getFieldType: getFieldType$E,
            getInContextFragmentType: getInContextFragmentType$F
        };
        case "Field": return {
            getMinimumSelections: getMinimumSelections$J,
            getFieldType: getFieldType$H,
            getInContextFragmentType: getInContextFragmentType$I
        };
        case "RateLimit": return {
            getMinimumSelections: getMinimumSelections$7,
            getFieldType: getFieldType$7,
            getInContextFragmentType: getInContextFragmentType$7
        };
        case "UIAPIMutations": return {
            getMinimumSelections: getMinimumSelections$6,
            getFieldType: getFieldType$6,
            getInContextFragmentType: getInContextFragmentType$6
        };
        case "Base64Value": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1e,
            getInContextFragmentType: getInContextFragmentType$1e
        };
        case "EncryptedStringValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1k,
            getInContextFragmentType: getInContextFragmentType$1k
        };
        case "Record": return {
            getMinimumSelections: getMinimumSelections$5,
            getFieldType: getFieldType$5,
            getInContextFragmentType: getInContextFragmentType$5
        };
        case "Setup__SetupPolymorphicAggregateParentRelationship": return {
            getMinimumSelections: getMinimumSelections$n,
            getFieldType: getFieldType$l,
            getInContextFragmentType: getInContextFragmentType$m
        };
        case "CompoundField": return {
            getMinimumSelections: getMinimumSelections$1c,
            getFieldType: getFieldType$1b,
            getInContextFragmentType: getInContextFragmentType$1b
        };
        case "ChildRelationship": return {
            getMinimumSelections: getMinimumSelections$N,
            getFieldType: getFieldType$L,
            getInContextFragmentType: getInContextFragmentType$M
        };
        case "RecordResult": return {
            getMinimumSelections: getMinimumSelections$R,
            getFieldType: getFieldType$P,
            getInContextFragmentType: getInContextFragmentType$Q
        };
        case "PageInfo": return {
            getMinimumSelections: getMinimumSelections$1b,
            getFieldType: getFieldType$19,
            getInContextFragmentType: getInContextFragmentType$1a
        };
        case "CurrencyValue": return {
            getMinimumSelections: getMinimumSelections$d,
            getFieldType: getFieldType$1j,
            getInContextFragmentType: getInContextFragmentType$1j
        };
        case "DateFunctionAggregation": return {
            getMinimumSelections: getMinimumSelections$15,
            getFieldType: getFieldType$13,
            getInContextFragmentType: getInContextFragmentType$14
        };
        case "RecordQuery": return {
            getMinimumSelections: getMinimumSelections$18,
            getFieldType: getFieldType$16,
            getInContextFragmentType: getInContextFragmentType$17
        };
        case "Analytics__Analytics": return {
            getMinimumSelections: getMinimumSelections$u,
            getFieldType: getFieldType$s,
            getInContextFragmentType: getInContextFragmentType$t
        };
        case "Analytics__AnalyticsBrowse": return {
            getMinimumSelections: getMinimumSelections$v,
            getFieldType: getFieldType$t,
            getInContextFragmentType: getInContextFragmentType$u
        };
        case "Analytics__AnalyticsEdge": return {
            getMinimumSelections: getMinimumSelections$w,
            getFieldType: getFieldType$u,
            getInContextFragmentType: getInContextFragmentType$v
        };
        case "Analytics__AnalyticsRepresentationInterface": return {
            getMinimumSelections: getMinimumSelections,
            getFieldType: getFieldType,
            getInContextFragmentType: getInContextFragmentType
        };
        case "Analytics__AnalyticsRepresentation": return {
            getMinimumSelections: getMinimumSelections$x,
            getFieldType: getFieldType$v,
            getInContextFragmentType: getInContextFragmentType$w
        };
        case "Analytics__SemanticDefinition": return {
            getMinimumSelections: getMinimumSelections$1,
            getFieldType: getFieldType$1,
            getInContextFragmentType: getInContextFragmentType$1
        };
        case "Analytics__AnalyticsWorkspaceAssetConnection": return {
            getMinimumSelections: getMinimumSelections$y,
            getFieldType: getFieldType$w,
            getInContextFragmentType: getInContextFragmentType$x
        };
        case "Analytics__AnalyticsWorkspaceAssetEdge": return {
            getMinimumSelections: getMinimumSelections$z,
            getFieldType: getFieldType$x,
            getInContextFragmentType: getInContextFragmentType$y
        };
        case "Analytics__AnalyticsWorkspaceAsset": return {
            getMinimumSelections: getMinimumSelections$A,
            getFieldType: getFieldType$y,
            getInContextFragmentType: getInContextFragmentType$z
        };
        case "Analytics__AnalyticsWorkspace": return {
            getMinimumSelections: getMinimumSelections$B,
            getFieldType: getFieldType$z,
            getInContextFragmentType: getInContextFragmentType$A
        };
        case "Analytics__SemanticSubMetricDefinitionConnection": return {
            getMinimumSelections: getMinimumSelections$2,
            getFieldType: getFieldType$2,
            getInContextFragmentType: getInContextFragmentType$2
        };
        case "Analytics__SemanticSubMetricDefinitionEdge": return {
            getMinimumSelections: getMinimumSelections$3,
            getFieldType: getFieldType$3,
            getInContextFragmentType: getInContextFragmentType$3
        };
        case "Analytics__SemanticSubMetricDefinitionRepresentation": return {
            getMinimumSelections: getMinimumSelections$4,
            getFieldType: getFieldType$4,
            getInContextFragmentType: getInContextFragmentType$4
        };
        default:
            return undefined;
    }
}
function sharedGetFragmentType(fragment, fragmentMap) {
    const typeCondition = fragment.kind === 'FragmentSpread' ? fragmentMap[fragment.name.value].typeCondition : fragment.typeCondition;
    return typeCondition !== undefined ? typeCondition.name.value : undefined;
}
function getSelectionSetFromFragment(fragment, fragmentMap) {
    return fragment.kind === 'FragmentSpread' ? fragmentMap[fragment.name.value].selectionSet : fragment.selectionSet;
}
// Shared export so that we dont repeat this in every connection type. Using fields for now since untyped inline fragment fails on Core
function getCursorConnectionMinimumSelections() {
    return [{
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'edges',
            },
            selectionSet: {
                kind: 'SelectionSet',
                selections: [{
                        kind: 'Field',
                        name: {
                            kind: 'Name',
                            value: 'node',
                        },
                    },
                    {
                        kind: 'Field',
                        name: {
                            kind: 'Name',
                            value: 'cursor',
                        },
                    },
                ]
            },
        },
        {
            kind: 'Field',
            name: {
                kind: 'Name',
                value: 'pageInfo',
            },
            selectionSet: {
                kind: 'SelectionSet',
                selections: [{
                        kind: 'Field',
                        name: {
                            kind: 'Name',
                            value: 'hasNextPage',
                        },
                    },
                    {
                        kind: 'Field',
                        name: {
                            kind: 'Name',
                            value: 'endCursor',
                        },
                    },
                    {
                        kind: 'Field',
                        name: {
                            kind: 'Name',
                            value: 'startCursor',
                        },
                    },
                ]
            },
        },
    ];
}
// Shared export for evaluating the value of an argument
function evaluateArgumentValue(argumentNode, expectedKind, variables) {
    if (argumentNode === undefined) {
        return undefined;
    }
    switch (argumentNode.value.kind) {
        case expectedKind:
            return argumentNode.value.value;
        case 'Variable':
            return variables[argumentNode.value.name.value] !== null ? variables[argumentNode.value.name.value] : undefined;
        default:
            return undefined;
    }
}
// Shared export for getting the offset of the current page in the edges array
function getArrayOffsetForPage(state, node, key) {
    var _a;
    const { variables, data, store } = state;
    // if we have pagination metadata, see if we can locate either the start or
    // after cursor in it to determine the correct index to use for the fullPath
    const paginationKey = key + '__pagination';
    let existingPaginationMetadata = store.readEntry(paginationKey);
    const afterArg = (_a = node.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', variables);
    if (existingPaginationMetadata !== undefined) {
        // if the starting cursor for current page exists in the existing metadata use that as offset
        if (existingPaginationMetadata[data.pageInfo.startCursor] !== undefined) {
            return existingPaginationMetadata[data.pageInfo.startCursor];
        }
        else if (afterArgVal !== undefined && existingPaginationMetadata[afterArgVal] !== undefined) {
            // We need to ingest at the end of the array so use the after value and add 1
            const previousPageOffset = existingPaginationMetadata[afterArgVal];
            return previousPageOffset + 1;
        }
    }
    return 0;
}
function isFragmentApplicable(fragmentNode, implementedInterfaces, graphqlTypeName, typename) {
    const isUntypedInlineFragment = fragmentNode.typeCondition === undefined;
    const matchesTypeCondition = !isUntypedInlineFragment && fragmentNode.typeCondition.name.value === graphqlTypeName;
    const doesImplementInterface = !isUntypedInlineFragment && implementedInterfaces.includes(fragmentNode.typeCondition.name.value);
    const typenameMatchesCondition = typename !== undefined && !isUntypedInlineFragment && typename === fragmentNode.typeCondition.name.value;
    return isUntypedInlineFragment || matchesTypeCondition || doesImplementInterface || typenameMatchesCondition;
}
function buildSelectionForNode(source, reader, parentNode, selectionNode, variables, fragments, isFragmentApplicable, buildSelectionForField, parentRecordId) {
    const sink = {};
    if (selectionNode.directives) {
        const selectionWithDirectives = applyDirectivesForSelection(selectionNode, variables);
        if (selectionWithDirectives === undefined) {
            return;
        }
        selectionNode = selectionWithDirectives;
    }
    if (selectionNode.kind === "InlineFragment") {
        if (isFragmentApplicable(selectionNode, source.data.__typename)) {
            // Get selections out of the inline fragment
            selectionNode.selectionSet.selections.forEach(fragmentSelection => {
                // Call into field handling with selections
                Object.assign(sink, buildSelectionForNode(source, reader, parentNode, fragmentSelection, variables, fragments, isFragmentApplicable, buildSelectionForField, parentRecordId));
            });
        }
    }
    if (selectionNode.kind === "FragmentSpread") {
        const namedFragment = fragments[selectionNode.name.value];
        if (namedFragment && isFragmentApplicable(namedFragment, source.data.__typename)) {
            namedFragment.selectionSet.selections.forEach(fragmentSelection => {
                // Call into field handling with selections
                Object.assign(sink, buildSelectionForNode(source, reader, parentNode, fragmentSelection, variables, fragments, isFragmentApplicable, buildSelectionForField, parentRecordId));
            });
        }
    }
    if (selectionNode.kind === "Field") {
        Object.assign(sink, buildSelectionForField(source, reader, selectionNode, variables, fragments, parentRecordId));
    }
    return sink;
}
function buildSelectionForField(source, reader, sel, variables, fragments, isCursorConnectionType, getFieldData, getFieldType, selectType, parentRecordId) {
    const sink = {};
    const fieldData = getFieldData(source, sel, variables);
    const requestedFieldName = sel.alias ? sel.alias.value : sel.name.value;
    if (isCursorConnectionType && requestedFieldName === 'pageInfo') {
        return;
    }
    reader.enterPath(requestedFieldName);
    if (fieldData === undefined) {
        reader.markMissing();
        reader.exitPath();
        return;
    }
    if (fieldData === null) {
        reader.assignScalar(requestedFieldName, sink, fieldData);
        reader.exitPath();
        return sink;
    }
    const fieldType = getFieldType(sel);
    if (fieldType === undefined) {
        // This field's type is unknown to us. This breaks normalization.
        console.warn(`Unknown Field: ${requestedFieldName}. This field or nested fields cannot participate in normalization`);
        reader.assignNonScalar(sink, requestedFieldName, fieldData);
    }
    else if (fieldType.isArray) {
        const arraySink = [];
        reader.markSeenId(parentRecordId); // Always mark the parent of an array as seen.
        for (let i = 0, len = fieldData.length; i < len; i += 1) {
            reader.enterPath(i);
            selectType(fieldType.typename, sel, fieldData[i], reader, i, arraySink, variables, fragments, parentRecordId);
            reader.exitPath();
        }
        reader.assignNonScalar(sink, requestedFieldName, arraySink);
    }
    else {
        selectType(fieldType.typename, sel, fieldData, reader, requestedFieldName, sink, variables, fragments, parentRecordId);
    }
    reader.exitPath();
    return sink;
}
function selectTypeLink(sel, fieldData, reader, key, sink, variables, fragments, version, selectFn, isCursorConnection) {
    const resolvedLink = resolveLink(reader, fieldData, version);
    if (resolvedLink && resolvedLink.data !== undefined) {
        if (isCursorConnection) {
            selectTypeLinkWithPagination(resolvedLink, sel, fieldData, reader, key, sink, variables, fragments, selectFn);
        }
        else {
            const result = selectFn(sel, variables, fragments)(resolvedLink.data, reader, resolvedLink.recordId);
            reader.assignNonScalar(sink, key, result);
        }
    }
    else {
        reader.markMissingLink(fieldData.__ref);
        reader.markMissing();
    }
}
function augmentDefaultVariableValues(variableDefinitions, variables) {
    variableDefinitions.forEach(({ defaultValue, variable }) => {
        if (defaultValue !== undefined && variables[variable.name.value] === undefined) {
            switch (defaultValue.kind) {
                case ("IntValue"):
                case ("FloatValue"):
                    variables[variable.name.value] = Number(defaultValue.value);
                    break;
                case ("NullValue"):
                    variables[variable.name.value] = null;
                    break;
                case ("BooleanValue"):
                case ("StringValue"):
                case ("EnumValue"):
                    variables[variable.name.value] = defaultValue.value;
            }
        }
    });
}
function selectCalculateSink(sink, field, buildSelectionForNodeFn, source, reader, variables, fragments, parentRecordId) {
    var _a;
    (_a = field.selectionSet) === null || _a === void 0 ? void 0 : _a.selections.forEach((sel) => {
        const builtSelection = buildSelectionForNodeFn(source, reader, field, sel, variables, fragments, parentRecordId);
        if (builtSelection !== undefined) {
            deepMerge(sink, builtSelection);
        }
    });
    return sink;
}
function selectTypeLinkWithPagination(resolvedLink, sel, fieldData, reader, key, sink, variables, fragments, selectFn) {
    var _a, _b, _c, _d, _e;
    const source = resolvedLink.data.data;
    const parentRecordId = resolvedLink.recordId;
    const totalEdges = source.edges !== undefined ? source.edges.length : 0;
    const metadata = reader.resolveMetadata(source, PAGINATION_VERSION$1);
    if (metadata === undefined) {
        reader.markMissingLink(fieldData.__ref);
        reader.markMissing();
        return;
    }
    const firstArg = (_a = sel.arguments) === null || _a === void 0 ? void 0 : _a.find(argument => { return argument.name.value === 'first'; });
    const firstArgVal = evaluateArgumentValue(firstArg, 'IntValue', variables);
    const pageSize = firstArgVal !== undefined ? firstArgVal : totalEdges;
    const afterArg = (_b = sel.arguments) === null || _b === void 0 ? void 0 : _b.find(argument => { return argument.name.value === 'after'; });
    const afterArgVal = evaluateArgumentValue(afterArg, 'StringValue', variables);
    const paginationParams = {
        pageSize: parseInt(pageSize, 10),
        token: afterArgVal
    };
    let { startOffset, endOffset } = getPageMetadata(metadata, paginationParams);
    const listIsComplete = (metadata.__END__ !== undefined);
    let trimmedEdges = [];
    let maxOffset = undefined;
    if (process.env.NODE_ENV !== 'production') {
        const afterArgIsUnknownToLuvio = afterArgVal !== undefined && (startOffset === undefined || endOffset === undefined);
        if (afterArgIsUnknownToLuvio) {
            throw new RangeError("after argument value '" + afterArgVal + "' is unknown in the list. Check that non-paging arguments are constant when paginating.");
        }
    }
    if (totalEdges === 0 && listIsComplete) {
        // We have an empty edge list
        // If after token was specified, it will never be a good cursor, so defer to a cache miss
        if (afterArgVal !== undefined) {
            reader.markMissingLink(fieldData.__ref);
            reader.markMissing();
            return;
        }
    }
    else if (totalEdges === 0 && !listIsComplete) {
        // empty edge list, but the list isn't complete?
        if (process.env.NODE_ENV !== 'production') {
            throw new Error("page size of zero, but pageInfo .hasNextPage is true.");
        }
    }
    else {
        if (startOffset === undefined || endOffset === undefined) {
            reader.markMissingLink(fieldData.__ref);
            reader.markMissing();
            return;
        }
        // gql paging semantics is "after" the requested cursor, so shift the offsets provided a valid cursor is provided
        if (afterArgVal !== undefined) {
            startOffset++;
            // if the "first" argument was not provided then the endOffset
            // is already set to the index of the last edge.
            // if the "first" argument was provided then we need to increment the endOffset
            // however we still need to make sure the endOffset doesn't go past totalEdges
            // if the list is already complete and we can't fetch any more edges
            if (firstArgVal !== undefined && (!listIsComplete || endOffset < totalEdges)) {
                endOffset++;
            }
        }
        if (startOffset >= totalEdges && listIsComplete) {
            // Data after the last page was requested, return empty set
            startOffset = undefined;
            endOffset = undefined;
        }
        else {
            trimmedEdges = trimEdges(source.edges, startOffset, endOffset);
            maxOffset = findMaxOffset(metadata);
            const listHasEnoughItems = (endOffset - 1 <= maxOffset);
            if (trimmedEdges === undefined || !listHasEnoughItems) {
                reader.markMissingLink(fieldData.__ref);
                reader.markMissing();
                // optimize pagination when "first" and "after" args are part of the variable
                if (((_c = firstArg === null || firstArg === void 0 ? void 0 : firstArg.value) === null || _c === void 0 ? void 0 : _c.kind) === 'Variable' && ((_d = afterArg === null || afterArg === void 0 ? void 0 : afterArg.value) === null || _d === void 0 ? void 0 : _d.kind) === 'Variable') {
                    optimizePagination(metadata, firstArg.value.name.value, afterArg.value.name.value, variables);
                }
                return;
            }
        }
    }
    deepFreeze(trimmedEdges);
    const paginatedData = {
        data: {
            ...source,
            edges: trimmedEdges,
        }
    };
    reader.markSeenId(parentRecordId);
    const result = selectFn(sel, variables, fragments)(paginatedData, reader, parentRecordId);
    const pageInfo = createPageInfo(result === null || result === void 0 ? void 0 : result.edges, startOffset, endOffset, maxOffset, metadata);
    if (pageInfo !== undefined) {
        const selectedPageInfo = selectPageInfo((_e = sel.selectionSet) === null || _e === void 0 ? void 0 : _e.selections, fragments, pageInfo);
        if (selectedPageInfo !== undefined) {
            reader.assignNonScalar(result, 'pageInfo', selectedPageInfo);
        }
    }
    reader.assignNonScalar(sink, key, result);
}
function selectPageInfo(selections, fragments, pageInfo) {
    var _a;
    const pageInfoSelection = getSelectionNode(selections, fragments, 'pageInfo');
    if (pageInfo === undefined || pageInfoSelection === undefined || pageInfoSelection.kind !== 'Field')
        return undefined;
    const pageInfoSelections = (_a = pageInfoSelection.selectionSet) === null || _a === void 0 ? void 0 : _a.selections;
    const selectedPageInfo = Object.assign({}, pageInfo);
    Object.keys(pageInfo).forEach((fieldName) => {
        if (getSelectionNode(pageInfoSelections, fragments, fieldName) === undefined) {
            delete selectedPageInfo[fieldName];
        }
    });
    return selectedPageInfo;
}
function getSelectionNode(selections, fragments, fieldName) {
    if (selections === undefined)
        return undefined;
    for (let i = 0; i < selections.length; i++) {
        const selection = selections[i];
        if (selection.kind === 'Field' && selection.name.value === fieldName) {
            return selection;
        }
        else if (selection.kind === 'FragmentSpread') {
            const fragment = fragments[selection.name.value];
            return getSelectionNode(fragment === null || fragment === void 0 ? void 0 : fragment.selectionSet.selections, fragments, fieldName);
        }
        else if (selection.kind === 'InlineFragment') {
            return getSelectionNode(selection.selectionSet.selections, fragments, fieldName);
        }
    }
    return undefined;
}

const TTL$5 = 900000;
const VERSION$b = "b440235e7e724631f92002fe50e3e096";
const RepresentationType$c = 'GraphQLRepresentation';
function select$8(luvio, document, variables, operationName) {
    const operationToExecute = getOperationFromDocument(document, operationName);
    return {
        kind: 'Fragment',
        synthetic: true,
        reader: true,
        read: (builder) => {
            builder.enterPath('data');
            const gqlData = {};
            if (operationToExecute === undefined) {
                builder.markMissing(); // Never give a cache hit for an undefined operation
                return gqlData;
            }
            if (operationToExecute.variableDefinitions !== undefined) {
                augmentDefaultVariableValues(operationToExecute.variableDefinitions, variables);
            }
            const fragments = createFragmentMap(document);
            const queryTypeRecordId = keyBuilder$e(luvio, operationToExecute, variables, fragments);
            const snapshot = builder.read({
                recordId: queryTypeRecordId,
                node: {
                    kind: 'Fragment',
                    private: [],
                    opaque: true,
                    version: VERSION$c
                },
                variables: {}
            });
            if (snapshot.data === undefined) {
                builder.markMissingLink(queryTypeRecordId);
            }
            else {
                const data = select$9(operationToExecute, variables, fragments)(snapshot.data, builder, queryTypeRecordId);
                builder.assignNonScalar(gqlData, 'data', data);
            }
            builder.exitPath();
            return gqlData;
        }
    };
}
function ingestOperationNode(luvio, input, node, state) {
    if (node.operation === 'query') {
        if (node.variableDefinitions !== undefined) {
            augmentDefaultVariableValues(node.variableDefinitions, state.variables);
        }
        ingest$8(node, {
            ...state,
            path: {
                ...state.path,
                fullPath: ''
            }
        });
    }
}
const ingest$7 = function GraphQLRepresentationIngest(document, variables, operationName) {
    return (input, path, luvio, store, timestamp) => {
        if (input.data) {
            const fragments = createFragmentMap(document);
            const state = {
                data: input.data,
                luvio,
                store,
                timestamp,
                path: path,
                variables,
                fragments,
            };
            const operationToExecute = getOperationFromDocument(document, operationName);
            if (operationToExecute !== undefined) {
                ingestOperationNode(luvio, input, operationToExecute, state);
            }
        }
        return {
            __ref: undefined
        };
    };
};
function getTypeCacheKeys$8(sink, luvio, document, variables, data, operationName) {
    if (data.data) {
        const fragments = createFragmentMap(document);
        const operationToExecute = getOperationFromDocument(document, operationName);
        if (operationToExecute !== undefined) {
            const state = {
                luvio,
                variables,
                fragments,
                data: data.data,
                path: {
                    parent: null,
                    propertyName: null,
                    fullPath: '' // TODO: W-13079691 - Need to take another pass at what we want to do for structured keys
                }
            };
            getTypeCacheKeys$9(sink, operationToExecute, state);
        }
    }
    return sink;
}

function select$7(luvio, config) {
    const { query, variables, operationName } = config;
    return select$8(luvio, query, variables, operationName);
}
function keyBuilder$d(luvio, params) {
    const { query, operationName, variables } = params.body;
    const operationToExecute = getOperationFromDocument(query, operationName);
    if (operationToExecute !== undefined) {
        const fragments = createFragmentMap(query);
        return keyBuilder$e(luvio, operationToExecute, variables || {}, fragments);
    }
    return `adapters_adapter$45$utils_keyPrefix::GraphQLRepresentation::InvalidOperation`;
}
function getResponseCacheKeys$c(cacheKeys, luvio, resourceParams, response) {
    const query = resourceParams.body.query;
    const variables = resourceParams.body.variables || {};
    const operationName = resourceParams.body.operationName;
    getTypeCacheKeys$8(cacheKeys, luvio, query, variables, response, operationName);
}
function ingestSuccess$6(luvio, config, resourceParams, response, snapshotRefresh) {
    const { body } = response;
    const key = keyBuilder$d(luvio, resourceParams);
    const { query, variables, operationName } = resourceParams.body;
    luvio.storeIngest(key, ingest$7(query, variables, operationName), body);
    // revert the optimized pagination variables back to its original so that subsequent store lookup will include all records, not just the ones loaded by an optimized query
    // see optimizePagination for initial update to optimize the query
    revertPaginationOptimization(variables);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$7(luvio, config),
        variables: {},
    }, snapshotRefresh);
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    return snapshot;
}
function ingestError$2(luvio, config, params, error, snapshotRefresh) {
    const key = keyBuilder$d(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL$5,
        namespace: keyPrefix,
        representationName: RepresentationType$c,
        version: VERSION$b
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$h(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/graphql',
        method: 'post',
        body: { ...config.body,
            query: print(config.body.query)
        },
        urlParams: {},
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

function stripMetaschemaFromFieldNode(fieldNode) {
    if (fieldNode.directives) {
        const indexOfCategoryDirective = fieldNode.directives.findIndex((directive) => directive.name.value === 'category');
        if (indexOfCategoryDirective !== -1) {
            const newDirectives = fieldNode.directives
                .slice(0, indexOfCategoryDirective)
                .concat(fieldNode.directives.slice(indexOfCategoryDirective + 1));
            return {
                ...fieldNode,
                directives: newDirectives,
            };
        }
    }
    return fieldNode;
}
function stripDocumentOfMetaschema(documentNode) {
    return visit(documentNode, {
        Field(node) {
            return stripMetaschemaFromFieldNode(node);
        },
    });
}
function createResourceRequest$g(config) {
    const queryWithoutMetaschemaDirectives = stripDocumentOfMetaschema(config.body.query);
    const newBody = {
        ...config.body,
        query: queryWithoutMetaschemaDirectives,
    };
    return createResourceRequest$h({
        ...config,
        body: newBody,
    });
}

function createResourceRequest$f(config) {
    const headers = {};
    const transformedConfig = stripDocumentsOfMetaschema(config);
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/graphql/batch',
        method: 'post',
        body: transformedConfig.body,
        urlParams: {},
        queryParams: {},
        headers,
        priority: 'normal',
    };
}
function stripDocumentsOfMetaschema(config) {
    // Map returns a copy of the config before running transform to avoid mutating the original config
    const batchQueryTransformed = config.body.batchQuery.map((singleConfig) => {
        return {
            ...singleConfig,
            query: print(stripDocumentOfMetaschema(singleConfig.query)),
        };
    });
    return {
        ...config,
        body: {
            batchQuery: batchQueryTransformed,
        },
    };
}
// Just iterate over each item
function createChildResourceParams(resourceParams) {
    const childConfigs = [];
    for (let index = 0, len = resourceParams.body.batchQuery.length; index < len; index += 1) {
        const item = resourceParams.body.batchQuery[index];
        childConfigs.push({
            body: {
                ...item,
            },
        });
    }
    return childConfigs;
}
// This function required a few tweaks to make Typescript not angry. And also properly Select based off the original config, not the injected config
function selectChildResourceParams(luvio, childResources, resourceParams) {
    const envelopeBodyPath = 'result';
    const envelopeStatusCodePath = 'statusCode';
    const envelopePath = 'results';
    return {
        kind: 'Fragment',
        reader: true,
        synthetic: true,
        read: (reader) => {
            // Top-level 404 lookup
            const result = reader.storeLookup(keyBuilder$c(luvio, resourceParams));
            if (result.state === reader.StoreResolveResultState.Error &&
                result.value.status === 404) {
                return {
                    state: 'Error',
                    value: result.value,
                };
            }
            const sink = {};
            reader.enterPath(envelopePath);
            const results = [];
            for (let i = 0, len = childResources.length; i < len; i += 1) {
                reader.enterPath(i);
                reader.enterPath(envelopeBodyPath);
                const childResource = childResources[i];
                const childKey = keyBuilder$d(luvio, childResource);
                const childFragment = select$7(luvio, childResource.body); // MUST read from the ORIGINAL config not injected
                const isMissingDataBeforeChildRead = reader.getIsDataMissing();
                const childSnapshot = reader.read({
                    recordId: childKey,
                    node: childFragment,
                    variables: {},
                });
                reader.exitPath();
                const childSink = {};
                reader.markSeenId(childKey);
                switch (childSnapshot.state) {
                    case 'Stale':
                        reader.markStale();
                    // Stale needs envelope bodies filled in so don't break
                    // eslint-disable-next-line no-fallthrough
                    case 'Fulfilled':
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, 200);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshot.data);
                        reader.exitPath();
                        break;
                    case 'Error': {
                        const childSnapshotError = childSnapshot.error;
                        reader.enterPath(envelopeStatusCodePath);
                        reader.assignScalar(envelopeStatusCodePath, childSink, childSnapshotError.status);
                        reader.exitPath();
                        reader.enterPath(envelopeBodyPath);
                        reader.assignNonScalar(childSink, envelopeBodyPath, childSnapshotError.body);
                        reader.exitPath();
                        break;
                    }
                    case 'Unfulfilled':
                        // if child snapshot doesn't have any data then
                        // that means the child record key is missing
                        if (childSnapshot.data === undefined) {
                            if (reader.isRebuilding() === false) {
                                // not a rebuild, mark as missing and move on
                                reader.markMissingLink(childKey);
                                break;
                            }
                            // On rebuilds we have to check if there is a non-cached
                            // error that we know about.  If we don't do this then
                            // rebuilds will go into endless refresh loop if a child
                            // has non-cached errors (since the top-level composite
                            // snapshot will look like an Unfulfilled snapshot
                            // instead of an error snapshot).
                            const nonCachedError = nonCachedErrors[childKey];
                            if (nonCachedError === undefined) {
                                reader.markMissingLink(childKey);
                            }
                            else {
                                // if this child error was the only reason the reader
                                // is marked as missing then we want to undo that
                                if (isMissingDataBeforeChildRead === false) {
                                    reader.unMarkMissing();
                                }
                                // put status code and body into reader path
                                const { response: nonCachedBody, status: nonCachedStatus } = nonCachedError;
                                reader.enterPath(envelopeStatusCodePath);
                                reader.assignScalar(envelopeStatusCodePath, childSink, nonCachedStatus);
                                reader.exitPath();
                                reader.enterPath(envelopeBodyPath);
                                reader.assignNonScalar(childSink, envelopeBodyPath, nonCachedBody);
                                reader.exitPath();
                            }
                        }
                        break;
                    case 'Pending':
                        reader.markPending();
                        break;
                }
                ArrayPrototypePush.call(results, childSink);
                reader.exitPath();
            }
            reader.assignNonScalar(sink, envelopePath, results);
            deepFreeze(sink);
            reader.exitPath();
            return sink;
        },
    };
}
// The rest of this is copy-pasted. Sorry.
const nonCachedErrors = ObjectCreate$1(null);
function select$6(luvio, resourceParams) {
    const childResources = createChildResourceParams(resourceParams);
    return selectChildResourceParams(luvio, childResources, resourceParams);
}
function keyBuilder$c(luvio, params) {
    return (keyPrefix +
        '::GraphQLBatchRepresentation:(' +
        'batchQuery:' +
        params.body.batchQuery +
        ')');
}
function getResponseCacheKeys$b(keys, luvio, resourceParams, response) {
    const childEnvelopes = response.results;
    const childResourceParamsArray = createChildResourceParams(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' +
                childResourceParamsArray.length +
                ' items, received ' +
                childEnvelopes.length);
        }
    }
    // get children keys
    for (let index = 0, len = childResourceParamsArray.length; index < len; index++) {
        const childResourceParams = childResourceParamsArray[index];
        const childResult = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = childResult;
        if (childStatusCode === 200) {
            getResponseCacheKeys$c(keys, luvio, childResourceParams, childBody);
        }
        else if (childStatusCode === 404) {
            const childKey = keyBuilder$d(luvio, childResourceParams);
            keys.set(childKey, {
                namespace: keyPrefix,
                representationName: RepresentationType$c,
                mergeable: false,
            });
        }
    }
}
function ingestSuccessChildResourceParams$1(luvio, childConfigs, childResourceParamsArray, childEnvelopes) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let snapshotState = 'Fulfilled';
    for (let index = 0, len = childResourceParamsArray.length; index < len; index += 1) {
        // There's an assumption here that the config and resource params aren't reordered.
        const childConfig = childConfigs[index];
        const childResourceParams = childResourceParamsArray[index];
        const childKey = keyBuilder$d(luvio, childResourceParams);
        const result = childEnvelopes[index];
        const { statusCode: childStatusCode, result: childBody } = result;
        if (childStatusCode === 200 &&
            childBody.errors !== undefined &&
            childBody.errors.length === 0) {
            const childResponse = {
                status: 200,
                body: childBody,
                ok: true,
                statusText: 'OK',
                headers: {},
            };
            const childSnapshot = ingestSuccess$6(luvio, childConfig, childResourceParams, childResponse);
            if (childSnapshot.state === 'Stale') {
                snapshotState = 'Stale';
            }
            seenRecords.merge(childSnapshot.seenRecords);
            seenRecords.add(childSnapshot.recordId);
            const childValue = {
                statusCode: 200,
                result: childSnapshot.data,
            };
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        else {
            const childStatusCodeText = getFetchResponseStatusText(result.statusCode);
            const childResponse = {
                status: childStatusCode,
                body: childBody,
                ok: false,
                statusText: childStatusCodeText,
                headers: {},
                errorType: 'fetchResponse',
            };
            ingestError$2(luvio, childResourceParams.body, childResourceParams, childResponse);
            seenRecords.add(childKey);
            const childValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            ArrayPrototypePush.call(childSnapshotDataResponses, childValue);
        }
        // track non-cached responses so rebuilds work properly
        if (childStatusCode !== 404 && childStatusCode !== 200) {
            nonCachedErrors[childKey] = { response: childBody, status: childStatusCode };
        }
        else {
            delete nonCachedErrors[childKey];
        }
    }
    const childSnapshotData = {
        results: childSnapshotDataResponses,
    };
    deepFreeze(childSnapshotData);
    return {
        childSnapshotData: childSnapshotData,
        seenRecords,
        snapshotState,
    };
}
function ingestSuccess$5(luvio, resourceParams, response, snapshotRefresh // Handrolled to not be optional.
) {
    const childEnvelopes = response.body.results;
    const childResourceParamsArray = createChildResourceParams(resourceParams);
    if (process.env.NODE_ENV !== 'production') {
        if (childResourceParamsArray.length !== childEnvelopes.length) {
            throw new Error('Invalid composite resource response. Expected ' +
                childResourceParamsArray.length +
                ' items, received ' +
                childEnvelopes.length);
        }
    }
    const key = keyBuilder$c(luvio, resourceParams);
    const originalConfig = snapshotRefresh.config; // Can we.. not use the snapshot refresh?
    const { childSnapshotData, seenRecords, snapshotState } = ingestSuccessChildResourceParams$1(luvio, originalConfig.batchQuery, // can we make this safer?
    childResourceParamsArray, childEnvelopes);
    const syntheticSnapshot = {
        recordId: key,
        data: childSnapshotData,
        state: snapshotState,
        seenRecords: seenRecords,
        select: {
            recordId: key,
            node: select$6(luvio, resourceParams),
            variables: {},
        },
        refresh: snapshotRefresh,
        variables: {},
    };
    // evict top level composite record from the store. This covers the case where a previous resource request returned a 404.
    luvio.storeEvict(key);
    return syntheticSnapshot;
}
function ingestError$1(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$c(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    luvio.storeIngestError(key, errorSnapshot);
    return errorSnapshot;
}

function createResourceParams$c(config) {
    const resourceParams = {
        body: {
            batchQuery: config.batchQuery,
        },
    };
    return resourceParams;
}
function transformConfiguration$1(config) {
    const batchQueryTransformed = config.batchQuery.map((singleConfig) => {
        return {
            ...singleConfig,
            query: applyMinimumFieldsToQuery(parse$1(print(singleConfig.query))), // Stringifies and parses again to avoid mutating original. Should we just JSON.stringify to avoid the dependency on the parser?
        };
    });
    return {
        ...config,
        batchQuery: batchQueryTransformed,
    };
}
function typeCheckConfig$d(untrustedConfig) {
    const config = {};
    const untrustedConfig_batchQuery = untrustedConfig.batchQuery;
    if (untrustedConfig_batchQuery !== undefined && ArrayIsArray$1(untrustedConfig_batchQuery)) {
        const untrustedConfig_batchQuery_array = [];
        for (let i = 0, arrayLength = untrustedConfig_batchQuery.length; i < arrayLength; i++) {
            const untrustedConfig_batchQuery_item = untrustedConfig_batchQuery[i];
            // No real validation here - if it's an object and has a query key, add it.
            if (untrustedIsObject(untrustedConfig_batchQuery_item)) {
                const untrustedConfig_batchQuery_item_object = untrustedConfig_batchQuery_item;
                if (untrustedConfig_batchQuery_item_object !== undefined &&
                    ObjectPrototypeHasOwnProperty.call(untrustedConfig_batchQuery_item_object, 'query') // Handrolled validation
                ) {
                    untrustedConfig_batchQuery_array.push(untrustedConfig_batchQuery_item_object);
                }
            }
        }
        config.batchQuery = untrustedConfig_batchQuery_array;
    }
    return config;
}
//  Handroll offline support here
const factory$g = (luvio) => function UiApi__graphqlBatch(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$d(untrustedConfig, graphqlBatch_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    const { getEnvironmentAwareGraphQLBatchAdapter } = configurationForGraphQLAdapters;
    const adapter = getEnvironmentAwareGraphQLBatchAdapter();
    if (adapter !== undefined) {
        return adapter(config, requestContext);
    }
    return luvio.applyCachePolicy(requestContext || {}, { config, luvio }, // BuildSnapshotContext
    buildCachedSnapshotCachePolicy$5, buildNetworkSnapshotCachePolicy$6);
};
const graphqlBatch_ConfigPropertyNames = {
    displayName: 'graphqlBatch',
    parameters: {
        required: ['batchQuery'],
        optional: [],
    },
};
function keyBuilder$b(luvio, config) {
    const resourceParams = createResourceParams$c(config);
    return keyBuilder$c(luvio, resourceParams);
}
function validateAdapterConfig$d(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$d(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$3(luvio, config) {
    const resourceParams = createResourceParams$c(config);
    return select$6(luvio, resourceParams);
}
function onFetchResponseSuccess$4(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$5(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$e(luvio, config, snapshotRefreshOptions),
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$3(luvio, config, resourceParams, response) {
    const snapshot = ingestError$1(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$e(luvio, config, snapshotRefreshOptions),
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$e(luvio, config, options) {
    const resourceParams = createResourceParams$c(transformConfiguration$1(config));
    const request = createResourceRequest$f(resourceParams);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$4(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$b(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$3(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$6(context, coercedAdapterRequestContext) {
    const { luvio, config } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
            luvioRequestMethod: 'get',
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$e(luvio, config, dispatchOptions);
}
function buildCachedSnapshotCachePolicy$5(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$b(luvio, config),
        node: adapterFragment$3(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$e(luvio, config, snapshotRefreshOptions),
    });
    return cacheSnapshot;
}

function getResponseCacheKeys$a(cacheKeyMap, luvio, _resourceParams, // For consistency of interface with other adapters
response) {
    const results = response.results;
    for (let index = 0, len = results.length; index < len; index += 1) {
        const { statusCode: childStatusCode, result: childBody } = results[index];
        if (childStatusCode === 200 || childStatusCode === 201) {
            if (isDeleteResponse(childBody)) {
                const recordKey = keyBuilder$3E(luvio, { recordId: childBody.id });
                cacheKeyMap.set(recordKey, {
                    namespace: keyPrefix,
                    representationName: RepresentationType$_,
                    mergeable: false,
                });
            }
            else {
                getTypeCacheKeys$2l(cacheKeyMap, luvio, childBody);
            }
        }
    }
}
function ingestSuccessChildResourceParams(luvio, response) {
    const childSnapshotDataResponses = [];
    let seenRecords = new StoreKeySet();
    let batchSnapshotState = 'Fulfilled';
    const results = response.results;
    for (let index = 0, len = results.length; index < len; index += 1) {
        const individualResponse = results[index];
        const { statusCode: childStatusCode, result: childBody } = individualResponse;
        if (childStatusCode === 200 || childStatusCode === 201) {
            if (isDeleteResponse(childBody)) {
                const recordKey = keyBuilder$3E(luvio, { recordId: childBody.id });
                luvio.storeEvict(recordKey);
                ArrayPrototypePush.call(childSnapshotDataResponses, {
                    ...individualResponse,
                });
            }
            else {
                const childSnapshot = ingestRecord(childBody, luvio);
                if (childSnapshot.state !== 'Error') {
                    // Not sure why it's possible to get Error here but it is
                    seenRecords.merge(childSnapshot.seenRecords);
                    seenRecords.add(childSnapshot.recordId);
                    if (childSnapshot.state === 'Stale') {
                        batchSnapshotState = 'Stale';
                    }
                }
                const childDataValue = {
                    ...individualResponse,
                    result: childSnapshot.data, // verify, is this safe w/ pending snapshots and stuff? Do we even need to return the snapshot really?
                };
                ArrayPrototypePush.call(childSnapshotDataResponses, childDataValue);
            }
        }
        else {
            // Got an error back
            const childErrorValue = {
                statusCode: childStatusCode,
                result: childBody,
            };
            ArrayPrototypePush.call(childSnapshotDataResponses, childErrorValue);
        }
    }
    const childSnapshotData = {
        hasErrors: response.hasErrors,
        results: childSnapshotDataResponses,
    };
    deepFreeze(childSnapshotData);
    return {
        result: childSnapshotData,
        seenRecords,
        snapshotState: batchSnapshotState,
    };
}
function ingestRecord(childBody, luvio) {
    const childResponse = {
        status: 200,
        body: childBody,
        ok: true,
        statusText: 'OK',
        headers: {},
    };
    const fieldTrie = BLANK_RECORD_FIELDS_TRIE;
    const optionalFieldTrie = BLANK_RECORD_FIELDS_TRIE;
    const conflictMap = {
        conflicts: {},
        serverRequestCount: 0,
    };
    const recordIngest = createRecordIngest(fieldTrie, optionalFieldTrie, conflictMap);
    const childSnapshot = ingestRecordResponse(luvio, childResponse, childBody.id, recordIngest, conflictMap);
    return childSnapshot;
}
function isDeleteResponse(childBody) {
    return childBody.fields === undefined && childBody.id !== undefined;
}
function ingestSuccess$4(luvio, response) {
    const { result, snapshotState, seenRecords } = ingestSuccessChildResourceParams(luvio, response.body);
    const syntheticSnapshot = {
        state: snapshotState,
        seenRecords,
        data: result,
    };
    return syntheticSnapshot;
}
function createResourceRequest$e(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/batch',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$8 = 'executeBatchRecordOperations';
const executeBatchRecordOperations_ConfigPropertyMetadata = [
    generateParamConfigMetadata('allOrNone', true, 2, 1, false),
    generateParamConfigMetadata('operations', true, 2, 4, true),
    generateParamConfigMetadata('useDefaultRule', false, 1, 1, false), // Not Requred, QueryParam, Boolean, single
];
const executeBatchRecordOperations_ConfigPropertyNames = 
/*#__PURE__*/ buildAdapterValidationConfig(adapterName$8, executeBatchRecordOperations_ConfigPropertyMetadata);
const acceptedOperations = ['CREATE', 'UPDATE', 'DELETE'];
const batchUpdateRecordsCreateResourceParams = /*#__PURE__*/ createResourceParams$k(executeBatchRecordOperations_ConfigPropertyMetadata);
function batchUpdateRecordsTypeCheckConfig(untrustedConfig) {
    const config = {};
    const untrustedConfigOperations = untrustedConfig.operations;
    if (untrustedConfigOperations !== undefined && ArrayIsArray$1(untrustedConfigOperations)) {
        const trustedOperations = [];
        let hasInvalidOperation = false;
        for (let i = 0, arrayLength = untrustedConfigOperations.length; i < arrayLength; i++) {
            const untrustedSingleOperation = untrustedConfigOperations[i];
            // Not much deep validation here - if it's got a CREATE/UPDATE/DELETE type and a 'records' property, we add it
            if (untrustedIsObject(untrustedSingleOperation)) {
                if (untrustedSingleOperation !== undefined &&
                    untrustedSingleOperation.type !== undefined &&
                    typeof untrustedSingleOperation.type === 'string' &&
                    acceptedOperations.includes(untrustedSingleOperation.type.toUpperCase()) &&
                    ObjectPrototypeHasOwnProperty.call(untrustedSingleOperation, 'records')) {
                    trustedOperations.push(untrustedSingleOperation);
                }
                else {
                    hasInvalidOperation = true;
                }
            }
        }
        if (!hasInvalidOperation && trustedOperations.length > 0) {
            config.operations = trustedOperations;
        }
    }
    const untrustedConfigAllOrNone = untrustedConfig.allOrNone;
    if (untrustedConfigAllOrNone !== undefined && typeof untrustedConfigAllOrNone === 'boolean') {
        config.allOrNone = untrustedConfigAllOrNone;
    }
    const untrustedConfiguseDefaultRule = untrustedConfig.useDefaultRule;
    if (untrustedConfiguseDefaultRule !== undefined &&
        typeof untrustedConfiguseDefaultRule === 'boolean') {
        config.useDefaultRule = untrustedConfiguseDefaultRule;
    }
    return config;
}
function validateAdapterConfig$c(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    // Do we need to coerce all the incoming recordIds to 18 character? We don't use the config for any ingestion so might be okay.
    const coercedConfig = coerceConfig$1(untrustedConfig, executeBatchRecordOperations_ConfigPropertyMetadata);
    const config = batchUpdateRecordsTypeCheckConfig(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$d(luvio, config, options) {
    const resourceParams = batchUpdateRecordsCreateResourceParams(config);
    const request = createResourceRequest$e(resourceParams);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$4(luvio, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$a(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const factory$f = (luvio) => {
    return function executeBatchRecordOperations(untrustedConfig) {
        const config = validateAdapterConfig$c(untrustedConfig, executeBatchRecordOperations_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            // eslint-disable-next-line @salesforce/lds/no-error-in-production
            throw new Error('Invalid config for "executeBatchRecordOperations"');
        }
        return buildNetworkSnapshot$d(luvio, config);
    };
};

const adapterName$7 = 'graphql';
const graphql_ConfigPropertyMetadata = [
    generateParamConfigMetadata('operationName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('query', true, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('variables', false, 2 /* Body */, 4 /* Unsupported */),
];
const graphql_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$7, graphql_ConfigPropertyMetadata);
const createResourceParams$b = /*#__PURE__*/ createResourceParams$k(graphql_ConfigPropertyMetadata);
function keyBuilder$a(luvio, config) {
    const resourceParams = createResourceParams$b(config);
    return keyBuilder$d(luvio, resourceParams);
}
function typeCheckConfig$c(untrustedConfig) {
    const config = {};
    const untrustedConfig_operationName = untrustedConfig.operationName;
    if (typeof untrustedConfig_operationName === 'string') {
        config.operationName = untrustedConfig_operationName;
    }
    const untrustedConfig_query = untrustedConfig.query;
    if (untrustedIsObject(untrustedConfig_query) && untrustedConfig_query.kind === 'Document') {
        config.query = untrustedConfig_query;
    }
    const untrustedConfig_variables = untrustedConfig.variables;
    if (untrustedIsObject(untrustedConfig_variables)) {
        const untrustedConfig_variables_object = {};
        const untrustedConfig_variables_keys = Object.keys(untrustedConfig_variables);
        for (let i = 0, arrayLength = untrustedConfig_variables_keys.length; i < arrayLength; i++) {
            const key = untrustedConfig_variables_keys[i];
            const untrustedConfig_variables_prop = untrustedConfig_variables[key];
            if (untrustedConfig_variables_object !== undefined) {
                untrustedConfig_variables_object[key] = untrustedConfig_variables_prop;
            }
        }
        if (untrustedConfig_variables_object !== undefined && Object.keys(untrustedConfig_variables_object).length >= 0) {
            config.variables = untrustedConfig_variables_object;
        }
    }
    return config;
}
function validateAdapterConfig$b(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$c(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function transformConfiguration(config) {
    // Make a copy of the config before running transform to avoid mutating the original config
    const adapterConfigCopy = {
        ...config,
        query: parse$1(print(config.query))
    };
    return {
        ...adapterConfigCopy,
        query: applyMinimumFieldsToQuery(adapterConfigCopy.query)
    };
}
function adapterFragment$2(luvio, config) {
    return select$7(luvio, config);
}
function onFetchResponseSuccess$3(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$6(luvio, config, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$c(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$2(luvio, config, resourceParams, response) {
    const snapshot = ingestError$2(luvio, config, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$c(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$c(luvio, config, options) {
    const resourceParams = createResourceParams$b(transformConfiguration(config));
    const request = createResourceRequest$g(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        var _a, _b;
        const graphqlErrors = (_a = response === null || response === void 0 ? void 0 : response.body) === null || _a === void 0 ? void 0 : _a.errors;
        if ((graphqlErrors || []).length === 0) {
            return response;
        }
        return Promise.reject({ errorType: 'adapterError', error: (_b = response === null || response === void 0 ? void 0 : response.body) === null || _b === void 0 ? void 0 : _b.errors });
    })
        .then((response) => {
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$3(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$c(cache, luvio, resourceParams, response.body);
            return cache;
        });
    })
        .catch((response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$2(luvio, config, resourceParams, response));
    });
}
function buildNetworkSnapshotCachePolicy$5(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$c, undefined, false);
}
function buildCachedSnapshotCachePolicy$4(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$a(luvio, config),
        node: adapterFragment$2(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$c(luvio, config, snapshotRefreshOptions)
    });
    return cacheSnapshot;
}

function validateAdapterConfig$a(untrustedConfig, _configPropertyNames) {
    if (untrustedConfig !== null && typeof untrustedConfig === 'object') {
        const operationNameIsDefinedAsUndefined = ObjectPrototypeHasOwnProperty.call(untrustedConfig, 'operationName') &&
            untrustedConfig.operationName === undefined;
        const variablesIsDefinedAsUndefined = ObjectPrototypeHasOwnProperty.call(untrustedConfig, 'variables') &&
            untrustedConfig.variables === undefined;
        if (operationNameIsDefinedAsUndefined || variablesIsDefinedAsUndefined) {
            return null;
        }
    }
    return validateAdapterConfig$b(untrustedConfig, _configPropertyNames);
}

const factory$e = (luvio) => function graphql(untrustedConfig, requestContext) {
    const config = validateAdapterConfig$a(untrustedConfig, graphql_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    const { getDraftAwareGraphQLAdapter } = configurationForGraphQLAdapters;
    const adapter = getDraftAwareGraphQLAdapter();
    if (adapter !== undefined) {
        return adapter(config, requestContext);
    }
    return luvio.applyCachePolicy(requestContext || {}, { config, luvio }, buildCachedSnapshotCachePolicy$4, buildNetworkSnapshotCachePolicy$5);
};

function select$5(luvio, params) {
    return select$38();
}
function getResponseCacheKeys$9(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$27(storeKeyMap, luvio, response);
}
function ingestSuccess$3(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType$s(luvio, body);
    luvio.storeIngest(key, ingest$23, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$5(),
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function createResourceRequest$d(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/perform-quick-action/' + config.urlParams.actionApiName + '',
        method: 'patch',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const performUpdateRecordQuickAction_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('allowSaveOnDuplicate', false, 2 /* Body */, 1 /* Boolean */),
    generateParamConfigMetadata('apiName', true, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('contextId', false, 2 /* Body */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('fields', true, 2 /* Body */, 4 /* Unsupported */),
];
const createResourceParams$a = /*#__PURE__*/ createResourceParams$k(performUpdateRecordQuickAction_ConfigPropertyMetadata);
function buildNetworkSnapshot$b(luvio, config, options) {
    const resourceParams = createResourceParams$a(config);
    const request = createResourceRequest$d(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$3(luvio, resourceParams, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$9(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}

function select$4(luvio, params) {
    return select$38();
}
function getResponseCacheKeys$8(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$27(storeKeyMap, luvio, response);
}
function ingestSuccess$2(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType$s(luvio, body);
    luvio.storeIngest(key, ingest$23, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: select$4(),
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    deepFreeze(snapshot.data);
    return snapshot;
}
function createResourceRequest$c(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/actions/perform-quick-action/' + config.urlParams.actionApiName + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const performQuickAction_ConfigPropertyMetadata = [
    generateParamConfigMetadata('actionApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('allowSaveOnDuplicate', false, 2 /* Body */, 1 /* Boolean */),
    generateParamConfigMetadata('apiName', true, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('contextId', false, 2 /* Body */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('fields', true, 2 /* Body */, 4 /* Unsupported */),
];
const createResourceParams$9 = /*#__PURE__*/ createResourceParams$k(performQuickAction_ConfigPropertyMetadata);
function buildNetworkSnapshot$a(luvio, config, options) {
    const resourceParams = createResourceParams$9(config);
    const request = createResourceRequest$c(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$2(luvio, resourceParams, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$8(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}

const factory$d = (luvio) => {
    return function (untrustedConfig) {
        const config = untrustedConfig;
        const draftAdapter = configurationForRestAdapters.getDraftAwarePerformUpdateRecordQuickActionAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, (config) => {
                const resourceParams = createResourceParams$9(config);
                return createResourceRequest$d(resourceParams);
            });
        }
        return buildNetworkSnapshot$b(luvio, config);
    };
};

const factory$c = (luvio) => {
    return function (untrustedConfig) {
        const config = untrustedConfig;
        const draftAdapter = configurationForRestAdapters.getDraftAwarePerformQuickActionAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, (config) => {
                const resourceParams = createResourceParams$9(config);
                return createResourceRequest$c(resourceParams);
            });
        }
        return buildNetworkSnapshot$a(luvio, config);
    };
};

function buildNetworkSnapshot$9(luvio, config, options, cacheSnapshot) {
    const resourceParams = createResourceParams$8(config);
    const request = prepareRequest$1(luvio, config, resourceParams, cacheSnapshot);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        const { body } = response;
        // Hand-rolling: The server returns sortBy as a comma-delimited String, but LDS expects an array.
        // getListUi does the same in its index.ts.
        const sortBy = body.sortBy;
        if (sortBy && typeof sortBy === 'string') {
            response.body.sortBy = sortBy.split(',');
        }
        // Hand-rolling: The server returns sortBy as null for MRU when we don't pass it as a parameter.
        // Changing server response to avoid key mismatch
        const listViewApiName = body.listReference && body.listReference.listViewApiName
            ? body.listReference.listViewApiName
            : null;
        if (listViewApiName === '__Recent' && !sortBy) {
            response.body.sortBy = [];
        }
        return luvio.handleSuccessResponse(() => onFetchResponseSuccess$1(luvio, config, resourceParams, response), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$13(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => onFetchResponseError$1(luvio, config, resourceParams, response));
    });
}

const adapterName$6 = 'getListRecordsByName';
const getListRecordsByName_ConfigPropertyMetadata = [
    generateParamConfigMetadata('listViewApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('fields', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('optionalFields', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('pageSize', false, 2 /* Body */, 3 /* Integer */),
    generateParamConfigMetadata('pageToken', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('searchTerm', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('sortBy', false, 2 /* Body */, 0 /* String */, true),
    generateParamConfigMetadata('where', false, 2 /* Body */, 0 /* String */),
];
const getListRecordsByName_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$6, getListRecordsByName_ConfigPropertyMetadata);
const createResourceParams$8 = /*#__PURE__*/ createResourceParams$k(getListRecordsByName_ConfigPropertyMetadata);
function keyBuilder$9(luvio, config) {
    const resourceParams = createResourceParams$8(config);
    return keyBuilder$3t(luvio, resourceParams);
}
function typeCheckConfig$b(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getListRecordsByName_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$9(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$b(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function adapterFragment$1(luvio, config) {
    const resourceParams = createResourceParams$8(config);
    return select$3i(luvio, resourceParams);
}
function onFetchResponseSuccess$2(luvio, config, resourceParams, response) {
    const snapshot = ingestSuccess$U(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$9(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function onFetchResponseError$1(luvio, config, resourceParams, response) {
    const snapshot = ingestError$N(luvio, resourceParams, response, {
        config,
        resolve: () => buildNetworkSnapshot$9(luvio, config, snapshotRefreshOptions)
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function prepareRequest$1(luvio, config, resourceParams, cacheSnapshot) {
    if (cacheSnapshot === undefined) {
        return createResourceRequest$1b(resourceParams);
    }
    const [newToken, newPageSize] = minimizePaginationParams(luvio, resourceParams, cacheSnapshot);
    if (newToken === undefined) {
        return createResourceRequest$1b(resourceParams);
    }
    return createResourceRequest$1b(createResourceParams$8({
        ...config,
        pageSize: newPageSize,
        pageToken: newToken
    }));
}
function minimizePaginationParams(luvio, resourceParams, cacheSnapshot) {
    const num = getCachedItemsNumber(cacheSnapshot);
    if (num > 0) {
        const paginationMetadata = getPaginationMetadata(luvio, resourceParams);
        if (paginationMetadata !== undefined) {
            const { token: configToken, pageSize: configPageSize } = createPaginationParams$4(resourceParams);
            const configTokenOffset = offsetFromToken(paginationMetadata, configToken);
            if (configTokenOffset < 0) {
                return [undefined, -1];
            }
            const maxOffset = configTokenOffset + num;
            const [token, offset] = tokenForAtMost(paginationMetadata, maxOffset);
            if (token !== undefined) {
                return [token, configPageSize - (offset - configTokenOffset)];
            }
        }
    }
    return [undefined, -1];
}
function getCachedItemsNumber(cacheSnapshot) {
    const { data } = cacheSnapshot;
    if (data === undefined || !data.records) {
        return 0;
    }
    const items = data.records;
    for (let i = 0; i < items.length; i++) {
        if (items[i] === undefined) {
            return i;
        }
    }
    return items.length;
}
function getPaginationMetadata(luvio, resourceParams) {
    const key = keyBuilder$3t(luvio, resourceParams);
    const selector = {
        recordId: key,
        node: {
            kind: 'Fragment',
            version: VERSION$2I,
            private: [],
            selections: [
                {
                    kind: 'Link',
                    name: '__metadata',
                    fragment: {
                        kind: 'Fragment',
                        version: PAGINATION_VERSION$1,
                        opaque: true,
                        private: [],
                    },
                },
            ],
        },
        variables: {}
    };
    const node = luvio.storeLookup(selector);
    if (node.state !== 'Fulfilled') {
        return;
    }
    return node.data.__metadata;
}
function buildNetworkSnapshotCachePolicy$4(context, coercedAdapterRequestContext) {
    return buildNetworkSnapshotCachePolicy$s(context, coercedAdapterRequestContext, buildNetworkSnapshot$9, 'get', true);
}
function buildCachedSnapshotCachePolicy$3(context, storeLookup) {
    const { luvio, config } = context;
    const selector = {
        recordId: keyBuilder$9(luvio, config),
        node: adapterFragment$1(luvio, config),
        variables: {},
    };
    const cacheSnapshot = storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$9(luvio, config, snapshotRefreshOptions)
    });
    if (isUnfulfilledSnapshot$1(cacheSnapshot) && cacheSnapshot.data !== undefined) {
        context.cacheSnapshot = cacheSnapshot;
    }
    return cacheSnapshot;
}

let context;
function setContext(adapterContext) {
    context = adapterContext;
}
function onFetchResponseSuccess$1(luvio, config, resourceParams, response) {
    const { body } = response, originalListRef = body.listReference;
    // If the response contains a listReference, then we need to add it/update it in the internal store
    if (originalListRef !== undefined) {
        const mutatedListRef = { ...originalListRef };
        // Setting the id to the eTag in the stored AdapterContext entry only, because
        // keyBuilder needs the eTag to generate the correct key for a cache hit.
        // Ideally, this can be removed once we deprecate getListUi and remove the hardcoded eTag reference in ListRecordCollectionRepresentation
        // Note: The original listReference will remain unchanged in the response.
        mutatedListRef.id = body.listInfoETag;
        addListReferenceWithId(mutatedListRef, context, body.listReference.id);
        // storing sortBy ensuring a cache hit when requesting records with no sortBy provided
        addServerDefaults(config, body, originalListRef, context);
    }
    return onFetchResponseSuccess$2(luvio, config, resourceParams, response);
}

const contextId$3 = `${keyPrefix}__${adapterName$6}`;
const factory$a = (luvio) => {
    return luvio.withContext(function UiApi__getListRecordsByName(untrustedConfig, adapterContext, requestContext) {
        const config = validateAdapterConfig$9(untrustedConfig, getListRecordsByName_ConfigPropertyNames);
        if (config === null) {
            return null;
        }
        setContext$1(adapterContext);
        setContext(adapterContext);
        return luvio.applyCachePolicy(requestContext || {}, { adapterContext, config, luvio }, buildCachedSnapshotCachePolicy$3, buildNetworkSnapshotCachePolicy$4);
    }, { contextId: contextId$3 });
};

function validate$b(obj, path = 'RecordInputRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        if (obj.allowSaveOnDuplicate !== undefined) {
            const obj_allowSaveOnDuplicate = obj.allowSaveOnDuplicate;
            const path_allowSaveOnDuplicate = path + '.allowSaveOnDuplicate';
            if (typeof obj_allowSaveOnDuplicate !== 'boolean') {
                return new TypeError('Expected "boolean" but received "' + typeof obj_allowSaveOnDuplicate + '" (at "' + path_allowSaveOnDuplicate + '")');
            }
        }
        if (obj.apiName !== undefined) {
            const obj_apiName = obj.apiName;
            const path_apiName = path + '.apiName';
            if (typeof obj_apiName !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
            }
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            let obj_fields_prop_union0 = null;
            const obj_fields_prop_union0_error = (() => {
                if (typeof obj_fields_prop !== 'string') {
                    return new TypeError('Expected "string" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
                }
            })();
            if (obj_fields_prop_union0_error != null) {
                obj_fields_prop_union0 = obj_fields_prop_union0_error.message;
            }
            let obj_fields_prop_union1 = null;
            const obj_fields_prop_union1_error = (() => {
                if (typeof obj_fields_prop !== 'number') {
                    return new TypeError('Expected "number" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
                }
            })();
            if (obj_fields_prop_union1_error != null) {
                obj_fields_prop_union1 = obj_fields_prop_union1_error.message;
            }
            let obj_fields_prop_union2 = null;
            const obj_fields_prop_union2_error = (() => {
                if (obj_fields_prop !== null) {
                    return new TypeError('Expected "null" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
                }
            })();
            if (obj_fields_prop_union2_error != null) {
                obj_fields_prop_union2 = obj_fields_prop_union2_error.message;
            }
            let obj_fields_prop_union3 = null;
            const obj_fields_prop_union3_error = (() => {
                if (typeof obj_fields_prop !== 'boolean') {
                    return new TypeError('Expected "boolean" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
                }
            })();
            if (obj_fields_prop_union3_error != null) {
                obj_fields_prop_union3 = obj_fields_prop_union3_error.message;
            }
            if (obj_fields_prop_union0 && obj_fields_prop_union1 && obj_fields_prop_union2 && obj_fields_prop_union3) {
                let message = 'Object doesn\'t match union (at "' + path_fields_prop + '")';
                message += '\n' + obj_fields_prop_union0.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_fields_prop_union1.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_fields_prop_union2.split('\n').map((line) => '\t' + line).join('\n');
                message += '\n' + obj_fields_prop_union3.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        }
    })();
    return v_error === undefined ? null : v_error;
}

const RepresentationType$b = 'LookupValuesRepresentation';
function getTypeCacheKeys$7(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$b,
        mergeable: false
    });
    const input_lookupResults = input.lookupResults;
    const input_lookupResults_keys = ObjectKeys(input_lookupResults);
    const input_lookupResults_length = input_lookupResults_keys.length;
    for (let i = 0; i < input_lookupResults_length; i++) {
        const key = input_lookupResults_keys[i];
        getTypeCacheKeys$2k(rootKeySet, luvio, input_lookupResults[key], () => rootKey + "__lookupResults" + "__" + key);
    }
}

function keyBuilder$8(luvio, params) {
    var _a, _b, _c, _d, _e;
    return keyPrefix + '::LookupValuesRepresentation:(' + 'dependentFieldBindings:' + params.queryParams.dependentFieldBindings + ',' + 'page:' + params.queryParams.page + ',' + 'pageSize:' + params.queryParams.pageSize + ',' + 'q:' + params.queryParams.q + ',' + 'searchType:' + params.queryParams.searchType + ',' + 'sourceRecordId:' + params.queryParams.sourceRecordId + ',' + 'targetApiName:' + params.queryParams.targetApiName + ',' + 'fieldApiName:' + params.urlParams.fieldApiName + ',' + 'objectApiName:' + params.urlParams.objectApiName + ',' + (((_a = params.body.sourceRecord) === null || _a === void 0 ? void 0 : _a.allowSaveOnDuplicate) === undefined ? 'sourceRecord.allowSaveOnDuplicate' : 'sourceRecord.allowSaveOnDuplicate:' + ((_b = params.body.sourceRecord) === null || _b === void 0 ? void 0 : _b.allowSaveOnDuplicate)) + '::' + (((_c = params.body.sourceRecord) === null || _c === void 0 ? void 0 : _c.apiName) === undefined ? 'sourceRecord.apiName' : 'sourceRecord.apiName:' + ((_d = params.body.sourceRecord) === null || _d === void 0 ? void 0 : _d.apiName)) + '::' + stableJSONStringify((_e = params.body.sourceRecord) === null || _e === void 0 ? void 0 : _e.fields) + '::' + (params.body.orderBy === undefined ? undefined : ('[' + params.body.orderBy.map(element => 'orderBy.fieldApiName:' + element.fieldApiName + '::' + 'orderBy.isAscending:' + element.isAscending).join(',') + ']')) + ')';
}
function getResponseCacheKeys$7(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$7(storeKeyMap, luvio, response, () => keyBuilder$8(luvio, resourceParams));
}
function createResourceRequest$a(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/lookups/' + config.urlParams.objectApiName + '/' + config.urlParams.fieldApiName + '',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const getLookupRecords_ConfigPropertyMetadata = [
    generateParamConfigMetadata('fieldApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getFieldApiName),
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('dependentFieldBindings', false, 1 /* QueryParameter */, 0 /* String */, true),
    generateParamConfigMetadata('page', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('pageSize', false, 1 /* QueryParameter */, 3 /* Integer */),
    generateParamConfigMetadata('q', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('searchType', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sourceRecordId', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('targetApiName', false, 1 /* QueryParameter */, 0 /* String */),
    generateParamConfigMetadata('sourceRecord', false, 2 /* Body */, 4 /* Unsupported */),
    generateParamConfigMetadata('orderBy', false, 2 /* Body */, 4 /* Unsupported */, true),
];
function typeCheckConfig$a(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getLookupRecords_ConfigPropertyMetadata);
    const untrustedConfig_sourceRecord = untrustedConfig.sourceRecord;
    const referenceRecordInputRepresentationValidationError = validate$b(untrustedConfig_sourceRecord);
    if (referenceRecordInputRepresentationValidationError === null) {
        config.sourceRecord = untrustedConfig_sourceRecord;
    }
    const untrustedConfig_orderBy = untrustedConfig.orderBy;
    if (ArrayIsArray$1(untrustedConfig_orderBy)) {
        const untrustedConfig_orderBy_array = [];
        for (let i = 0, arrayLength = untrustedConfig_orderBy.length; i < arrayLength; i++) {
            const untrustedConfig_orderBy_item = untrustedConfig_orderBy[i];
            const referenceListOrderedByInfoInputRepresentationValidationError = validate$15(untrustedConfig_orderBy_item);
            if (referenceListOrderedByInfoInputRepresentationValidationError === null) {
                untrustedConfig_orderBy_array.push(untrustedConfig_orderBy_item);
            }
        }
        config.orderBy = untrustedConfig_orderBy_array;
    }
    return config;
}
function validateAdapterConfig$8(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getLookupRecords_ConfigPropertyMetadata);
    const config = typeCheckConfig$a(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const paramNames = {
    displayName: 'getLookupRecords',
    parameters: {
        required: ['fieldApiName'],
        optional: ['requestParams'],
    },
};
function coerceRequestParams(untrusted) {
    if (!untrustedIsObject(untrusted)) {
        return {};
    }
    const coercedConfig = {};
    const requestParams = untrusted.requestParams || {};
    const dependentFieldBindings = requestParams.dependentFieldBindings;
    if (dependentFieldBindings !== undefined) {
        coercedConfig.dependentFieldBindings = dependentFieldBindings;
    }
    const page = requestParams.page;
    if (page !== undefined) {
        coercedConfig.page = page;
    }
    const pageSize = requestParams.pageSize;
    if (pageSize !== undefined) {
        coercedConfig.pageSize = pageSize;
    }
    const q = requestParams.q;
    if (q !== undefined) {
        coercedConfig.q = q;
    }
    const searchType = requestParams.searchType;
    if (searchType !== undefined) {
        coercedConfig.searchType = searchType;
    }
    const sourceRecordId = requestParams.sourceRecordId;
    if (sourceRecordId !== undefined) {
        coercedConfig.sourceRecordId = sourceRecordId;
    }
    const targetApiName = requestParams.targetApiName;
    if (targetApiName !== undefined) {
        coercedConfig.targetApiName = targetApiName;
    }
    const sourceRecord = requestParams.sourceRecord;
    if (sourceRecord !== undefined) {
        coercedConfig.sourceRecord = sourceRecord;
    }
    const orderBy = requestParams.orderBy;
    if (orderBy !== undefined) {
        coercedConfig.orderBy = orderBy;
    }
    return coercedConfig;
}
function coerceConfigWithDefaults$1(untrusted) {
    const config = validateAdapterConfig$8(untrusted, paramNames);
    if (config === null) {
        return config;
    }
    const coercedRequestParams = coerceRequestParams(untrusted);
    const { objectApiName, fieldApiName } = getFieldId(config.fieldApiName);
    return {
        ...config,
        objectApiName,
        fieldApiName,
        ...coercedRequestParams,
    };
}
function removeEtags(recordRep) {
    const { fields } = recordRep;
    delete recordRep.eTag;
    delete recordRep.weakEtag;
    Object.keys(fields).forEach((fieldName) => {
        const { value: nestedValue } = fields[fieldName];
        if (isSpanningRecord(nestedValue)) {
            removeEtags(nestedValue);
        }
    });
}
function buildNetworkSnapshot$8(luvio, config, options) {
    const { objectApiName, fieldApiName } = config;
    const resourceParams = {
        /*
            targetApiName is not set in the config in order to call /lookups/{objectApiName}/{fieldApiName} resource only.
            This is a temporary measure while aligning the representation of lookup resources (W-10320080)
        */
        urlParams: {
            objectApiName,
            fieldApiName,
        },
        queryParams: {
            page: config.page,
            pageSize: config.pageSize,
            q: config.q,
            searchType: config.searchType,
            dependentFieldBindings: config.dependentFieldBindings,
            sourceRecordId: config.sourceRecordId,
            targetApiName: config.targetApiName,
        },
        body: {
            sourceRecord: config.sourceRecord,
            orderBy: config.orderBy,
        },
    };
    const request = createResourceRequest$a(resourceParams);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        return luvio.handleSuccessResponse(() => {
            // TODO [W-7235112]: remove this hack to never ingest lookup responses that
            // avoids issues caused by them not being real RecordRepresentations
            const key = keyBuilder$8(luvio, resourceParams);
            const { body } = response;
            Object.keys(body.lookupResults)
                .map((targetApiName) => body.lookupResults[targetApiName].records)
                .reduce((allRecords, records) => allRecords.concat(records), [])
                .forEach((record) => removeEtags(record));
            deepFreeze(body);
            return Promise.resolve({
                state: 'Fulfilled',
                recordId: key,
                variables: {},
                seenRecords: new StoreKeySet(),
                select: {
                    recordId: key,
                    node: {
                        kind: 'Fragment',
                    },
                    variables: {},
                },
                data: body,
            });
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$7(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            return Promise.resolve(luvio.errorSnapshot(err));
        });
    });
}
function buildCachedSnapshot$2(_context, _storeLookup) {
    return;
}
function buildNetworkSnapshotCachePolicy$3(context, coercedAdapterRequestContext) {
    const { networkPriority, requestCorrelator, eventObservers } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$8(context.luvio, context.config, dispatchOptions);
}
const factory$9 = (luvio) => {
    return (untrustedConfig, requestContext) => {
        const config = coerceConfigWithDefaults$1(untrustedConfig);
        if (config === null) {
            return null;
        }
        const refresh = {
            config,
            resolve: () => buildNetworkSnapshot$8(luvio, config),
        };
        const promiseOrSnapshot = luvio.applyCachePolicy(requestContext || {}, { config, luvio }, buildCachedSnapshot$2, buildNetworkSnapshotCachePolicy$3);
        if (isPromise(promiseOrSnapshot)) {
            return promiseOrSnapshot.then((snapshot) => {
                snapshot.refresh = refresh;
                return snapshot;
            });
        }
        promiseOrSnapshot.refresh = refresh;
        return promiseOrSnapshot;
    };
};

function createResourceRequest$9(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/record-avatars/' + config.urlParams.recordId + '/association',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$5 = 'updateRecordAvatar';
const updateRecordAvatar_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('actionType', true, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('blueMasterId', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('externalId', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('photoUrl', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('profileName', false, 2 /* Body */, 0 /* String */),
];
const updateRecordAvatar_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$5, updateRecordAvatar_ConfigPropertyMetadata);
function typeCheckConfig$9(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, updateRecordAvatar_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$7(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$9(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const factory$8 = (luvio) => {
    return (untrustedConfig) => {
        const config = validateAdapterConfig$7(untrustedConfig, updateRecordAvatar_ConfigPropertyNames);
        if (config === null) {
            // eslint-disable-next-line @salesforce/lds/no-error-in-production
            throw new Error('updateRecordAvatar invalid configuration');
        }
        const resourceParams = {
            urlParams: {
                recordId: config.recordId,
            },
            body: {
                externalId: config.externalId,
                blueMasterId: config.blueMasterId,
                profileName: config.profileName,
                photoUrl: config.photoUrl,
                actionType: config.actionType,
            },
        };
        const request = createResourceRequest$9(resourceParams);
        return luvio.dispatchResourceRequest(request).then((response) => {
            const { body } = response;
            const key = keyBuilderFromType$f(luvio, body);
            return luvio.handleSuccessResponse(() => {
                let selectors;
                if (body.type === 'Theme') {
                    selectors = select$22;
                    luvio.storeIngest(key, ingest$1L, body);
                }
                else if (body.type === 'Photo') {
                    selectors = select$23;
                    luvio.storeIngest(key, ingest$1M, body);
                }
                else {
                    // eslint-disable-next-line @salesforce/lds/no-error-in-production
                    throw new Error('Unsupported avatar type');
                }
                // TODO [W-6804405]: support unions on fragments (only supported on links today)
                const snapshot = luvio.storeLookup({
                    recordId: key,
                    node: selectors(),
                    variables: {},
                });
                return luvio.storeBroadcast().then(() => snapshot);
            }, () => {
                const cache = new StoreKeyMap();
                getTypeCacheKeys$1N(cache, luvio, body);
                return cache;
            });
        }, (err) => {
            deepFreeze(err);
            throw err;
        });
    };
};

const VERSION$a = "809ada3563b00c364360826037620d7e";
function validate$a(obj, path = 'RecordCreateDefaultRecordRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_childRelationships = obj.childRelationships;
        const path_childRelationships = path + '.childRelationships';
        if (typeof obj_childRelationships !== 'object' || ArrayIsArray(obj_childRelationships) || obj_childRelationships === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_childRelationships + '" (at "' + path_childRelationships + '")');
        }
        const obj_childRelationships_keys = ObjectKeys(obj_childRelationships);
        for (let i = 0; i < obj_childRelationships_keys.length; i++) {
            const key = obj_childRelationships_keys[i];
            const obj_childRelationships_prop = obj_childRelationships[key];
            const path_childRelationships_prop = path_childRelationships + '["' + key + '"]';
            if (typeof obj_childRelationships_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_childRelationships_prop + '" (at "' + path_childRelationships_prop + '")');
            }
        }
        const obj_eTag = obj.eTag;
        const path_eTag = path + '.eTag';
        if (typeof obj_eTag !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_eTag + '" (at "' + path_eTag + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            if (typeof obj_fields_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
            }
        }
        const obj_id = obj.id;
        const path_id = path + '.id';
        if (obj_id !== null) {
            return new TypeError('Expected "null" but received "' + typeof obj_id + '" (at "' + path_id + '")');
        }
        const obj_lastModifiedById = obj.lastModifiedById;
        const path_lastModifiedById = path + '.lastModifiedById';
        let obj_lastModifiedById_union0 = null;
        const obj_lastModifiedById_union0_error = (() => {
            if (typeof obj_lastModifiedById !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lastModifiedById + '" (at "' + path_lastModifiedById + '")');
            }
        })();
        if (obj_lastModifiedById_union0_error != null) {
            obj_lastModifiedById_union0 = obj_lastModifiedById_union0_error.message;
        }
        let obj_lastModifiedById_union1 = null;
        const obj_lastModifiedById_union1_error = (() => {
            if (obj_lastModifiedById !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lastModifiedById + '" (at "' + path_lastModifiedById + '")');
            }
        })();
        if (obj_lastModifiedById_union1_error != null) {
            obj_lastModifiedById_union1 = obj_lastModifiedById_union1_error.message;
        }
        if (obj_lastModifiedById_union0 && obj_lastModifiedById_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lastModifiedById + '")';
            message += '\n' + obj_lastModifiedById_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lastModifiedById_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_lastModifiedDate = obj.lastModifiedDate;
        const path_lastModifiedDate = path + '.lastModifiedDate';
        let obj_lastModifiedDate_union0 = null;
        const obj_lastModifiedDate_union0_error = (() => {
            if (typeof obj_lastModifiedDate !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_lastModifiedDate + '" (at "' + path_lastModifiedDate + '")');
            }
        })();
        if (obj_lastModifiedDate_union0_error != null) {
            obj_lastModifiedDate_union0 = obj_lastModifiedDate_union0_error.message;
        }
        let obj_lastModifiedDate_union1 = null;
        const obj_lastModifiedDate_union1_error = (() => {
            if (obj_lastModifiedDate !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_lastModifiedDate + '" (at "' + path_lastModifiedDate + '")');
            }
        })();
        if (obj_lastModifiedDate_union1_error != null) {
            obj_lastModifiedDate_union1 = obj_lastModifiedDate_union1_error.message;
        }
        if (obj_lastModifiedDate_union0 && obj_lastModifiedDate_union1) {
            let message = 'Object doesn\'t match union (at "' + path_lastModifiedDate + '")';
            message += '\n' + obj_lastModifiedDate_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_lastModifiedDate_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        let obj_recordTypeId_union0 = null;
        const obj_recordTypeId_union0_error = (() => {
            if (typeof obj_recordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union0_error != null) {
            obj_recordTypeId_union0 = obj_recordTypeId_union0_error.message;
        }
        let obj_recordTypeId_union1 = null;
        const obj_recordTypeId_union1_error = (() => {
            if (obj_recordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union1_error != null) {
            obj_recordTypeId_union1 = obj_recordTypeId_union1_error.message;
        }
        if (obj_recordTypeId_union0 && obj_recordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeId + '")';
            message += '\n' + obj_recordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_recordTypeInfo = obj.recordTypeInfo;
        const path_recordTypeInfo = path + '.recordTypeInfo';
        let obj_recordTypeInfo_union0 = null;
        const obj_recordTypeInfo_union0_error = (() => {
            const referencepath_recordTypeInfoValidationError = validate$21(obj_recordTypeInfo, path_recordTypeInfo);
            if (referencepath_recordTypeInfoValidationError !== null) {
                let message = 'Object doesn\'t match RecordTypeInfoRepresentation (at "' + path_recordTypeInfo + '")\n';
                message += referencepath_recordTypeInfoValidationError.message.split('\n').map((line) => '\t' + line).join('\n');
                return new TypeError(message);
            }
        })();
        if (obj_recordTypeInfo_union0_error != null) {
            obj_recordTypeInfo_union0 = obj_recordTypeInfo_union0_error.message;
        }
        let obj_recordTypeInfo_union1 = null;
        const obj_recordTypeInfo_union1_error = (() => {
            if (obj_recordTypeInfo !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeInfo + '" (at "' + path_recordTypeInfo + '")');
            }
        })();
        if (obj_recordTypeInfo_union1_error != null) {
            obj_recordTypeInfo_union1 = obj_recordTypeInfo_union1_error.message;
        }
        if (obj_recordTypeInfo_union0 && obj_recordTypeInfo_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeInfo + '")';
            message += '\n' + obj_recordTypeInfo_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeInfo_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_systemModstamp = obj.systemModstamp;
        const path_systemModstamp = path + '.systemModstamp';
        let obj_systemModstamp_union0 = null;
        const obj_systemModstamp_union0_error = (() => {
            if (typeof obj_systemModstamp !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_systemModstamp + '" (at "' + path_systemModstamp + '")');
            }
        })();
        if (obj_systemModstamp_union0_error != null) {
            obj_systemModstamp_union0 = obj_systemModstamp_union0_error.message;
        }
        let obj_systemModstamp_union1 = null;
        const obj_systemModstamp_union1_error = (() => {
            if (obj_systemModstamp !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_systemModstamp + '" (at "' + path_systemModstamp + '")');
            }
        })();
        if (obj_systemModstamp_union1_error != null) {
            obj_systemModstamp_union1 = obj_systemModstamp_union1_error.message;
        }
        if (obj_systemModstamp_union0 && obj_systemModstamp_union1) {
            let message = 'Object doesn\'t match union (at "' + path_systemModstamp + '")';
            message += '\n' + obj_systemModstamp_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_systemModstamp_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_weakEtag = obj.weakEtag;
        const path_weakEtag = path + '.weakEtag';
        if (typeof obj_weakEtag !== 'number' || (typeof obj_weakEtag === 'number' && Math.floor(obj_weakEtag) !== obj_weakEtag)) {
            return new TypeError('Expected "integer" but received "' + typeof obj_weakEtag + '" (at "' + path_weakEtag + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$a = 'RecordCreateDefaultRecordRepresentation';
function normalize$6(input, existing, path, luvio, store, timestamp) {
    const input_childRelationships = input.childRelationships;
    const input_childRelationships_id = path.fullPath + '__childRelationships';
    const input_childRelationships_keys = Object.keys(input_childRelationships);
    const input_childRelationships_length = input_childRelationships_keys.length;
    for (let i = 0; i < input_childRelationships_length; i++) {
        const key = input_childRelationships_keys[i];
        const input_childRelationships_prop = input_childRelationships[key];
        const input_childRelationships_prop_id = input_childRelationships_id + '__' + key;
        input_childRelationships[key] = ingest$2h(input_childRelationships_prop, {
            fullPath: input_childRelationships_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_fields = input.fields;
    const input_fields_id = path.fullPath + '__fields';
    const input_fields_keys = Object.keys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        const input_fields_prop = input_fields[key];
        const input_fields_prop_id = input_fields_id + '__' + key;
        input_fields[key] = ingest$2i(input_fields_prop, {
            fullPath: input_fields_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    return input;
}
function equals$a(existing, incoming) {
    const existing_weakEtag = existing.weakEtag;
    const incoming_weakEtag = incoming.weakEtag;
    if (!(existing_weakEtag === incoming_weakEtag)) {
        return false;
    }
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_eTag = existing.eTag;
    const incoming_eTag = incoming.eTag;
    if (!(existing_eTag === incoming_eTag)) {
        return false;
    }
    const existing_childRelationships = existing.childRelationships;
    const incoming_childRelationships = incoming.childRelationships;
    const equals_childRelationships_props = equalsObject(existing_childRelationships, incoming_childRelationships, (existing_childRelationships_prop, incoming_childRelationships_prop) => {
        if (!(existing_childRelationships_prop.__ref === incoming_childRelationships_prop.__ref)) {
            return false;
        }
    });
    if (equals_childRelationships_props === false) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_props = equalsObject(existing_fields, incoming_fields, (existing_fields_prop, incoming_fields_prop) => {
        if (!(existing_fields_prop.__ref === incoming_fields_prop.__ref)) {
            return false;
        }
    });
    if (equals_fields_props === false) {
        return false;
    }
    const existing_id = existing.id;
    const incoming_id = incoming.id;
    if (!(existing_id === incoming_id)) {
        return false;
    }
    const existing_lastModifiedById = existing.lastModifiedById;
    const incoming_lastModifiedById = incoming.lastModifiedById;
    if (!(existing_lastModifiedById === incoming_lastModifiedById)) {
        return false;
    }
    const existing_lastModifiedDate = existing.lastModifiedDate;
    const incoming_lastModifiedDate = incoming.lastModifiedDate;
    if (!(existing_lastModifiedDate === incoming_lastModifiedDate)) {
        return false;
    }
    const existing_recordTypeId = existing.recordTypeId;
    const incoming_recordTypeId = incoming.recordTypeId;
    if (!(existing_recordTypeId === incoming_recordTypeId)) {
        return false;
    }
    const existing_recordTypeInfo = existing.recordTypeInfo;
    const incoming_recordTypeInfo = incoming.recordTypeInfo;
    if (!(existing_recordTypeInfo === incoming_recordTypeInfo
        || (existing_recordTypeInfo != null &&
            incoming_recordTypeInfo != null &&
            equals$1c(existing_recordTypeInfo, incoming_recordTypeInfo)))) {
        return false;
    }
    const existing_systemModstamp = existing.systemModstamp;
    const incoming_systemModstamp = incoming.systemModstamp;
    if (!(existing_systemModstamp === incoming_systemModstamp)) {
        return false;
    }
    return true;
}
const ingest$6 = function RecordCreateDefaultRecordRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$a(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$6, "UiApi", VERSION$a, RepresentationType$a, equals$a);
    return createLink$1(key);
};
function getTypeCacheKeys$6(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$a,
        mergeable: false
    });
    const input_childRelationships = input.childRelationships;
    const input_childRelationships_keys = ObjectKeys(input_childRelationships);
    const input_childRelationships_length = input_childRelationships_keys.length;
    for (let i = 0; i < input_childRelationships_length; i++) {
        const key = input_childRelationships_keys[i];
        getTypeCacheKeys$2k(rootKeySet, luvio, input_childRelationships[key], () => rootKey + "__childRelationships" + "__" + key);
    }
    const input_fields = input.fields;
    const input_fields_keys = ObjectKeys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        getTypeCacheKeys$2m(rootKeySet, luvio, input_fields[key], () => rootKey + "__fields" + "__" + key);
    }
}

const TTL$4 = 900000;
const VERSION$9 = "894a62b8f5de5535f2483fb2c3abca8e";
function validate$9(obj, path = 'RecordDefaultsRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_layout = obj.layout;
        const path_layout = path + '.layout';
        let obj_layout_union0 = null;
        const obj_layout_union0_error = (() => {
            if (typeof obj_layout !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_layout + '" (at "' + path_layout + '")');
            }
        })();
        if (obj_layout_union0_error != null) {
            obj_layout_union0 = obj_layout_union0_error.message;
        }
        let obj_layout_union1 = null;
        const obj_layout_union1_error = (() => {
            if (obj_layout !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_layout + '" (at "' + path_layout + '")');
            }
        })();
        if (obj_layout_union1_error != null) {
            obj_layout_union1 = obj_layout_union1_error.message;
        }
        if (obj_layout_union0 && obj_layout_union1) {
            let message = 'Object doesn\'t match union (at "' + path_layout + '")';
            message += '\n' + obj_layout_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_layout_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
        const obj_objectInfos = obj.objectInfos;
        const path_objectInfos = path + '.objectInfos';
        if (typeof obj_objectInfos !== 'object' || ArrayIsArray(obj_objectInfos) || obj_objectInfos === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_objectInfos + '" (at "' + path_objectInfos + '")');
        }
        const obj_objectInfos_keys = ObjectKeys(obj_objectInfos);
        for (let i = 0; i < obj_objectInfos_keys.length; i++) {
            const key = obj_objectInfos_keys[i];
            const obj_objectInfos_prop = obj_objectInfos[key];
            const path_objectInfos_prop = path_objectInfos + '["' + key + '"]';
            if (typeof obj_objectInfos_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_objectInfos_prop + '" (at "' + path_objectInfos_prop + '")');
            }
        }
        const obj_record = obj.record;
        const path_record = path + '.record';
        if (typeof obj_record !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_record + '" (at "' + path_record + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$9 = 'RecordDefaultsRepresentation';
function normalize$5(input, existing, path, luvio, store, timestamp) {
    const input_layout = input.layout;
    const input_layout_id = path.fullPath + '__layout';
    if (input_layout !== null && typeof input_layout === 'object') {
        input.layout = ingest$28(input_layout, {
            fullPath: input_layout_id,
            propertyName: 'layout',
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_objectInfos = input.objectInfos;
    const input_objectInfos_id = path.fullPath + '__objectInfos';
    const input_objectInfos_keys = Object.keys(input_objectInfos);
    const input_objectInfos_length = input_objectInfos_keys.length;
    for (let i = 0; i < input_objectInfos_length; i++) {
        const key = input_objectInfos_keys[i];
        const input_objectInfos_prop = input_objectInfos[key];
        const input_objectInfos_prop_id = input_objectInfos_id + '__' + key;
        input_objectInfos[key] = ingest$29(input_objectInfos_prop, {
            fullPath: input_objectInfos_prop_id,
            propertyName: key,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_record = input.record;
    const input_record_id = path.fullPath + '__record';
    input.record = ingest$6(input_record, {
        fullPath: input_record_id,
        propertyName: 'record',
        parent: {
            data: input,
            key: path.fullPath,
            existing: existing,
        },
        ttl: path.ttl
    }, luvio, store, timestamp);
    return input;
}
function equals$9(existing, incoming) {
    const existing_layout = existing.layout;
    const incoming_layout = incoming.layout;
    if (!(existing_layout === incoming_layout
        || (existing_layout != null &&
            incoming_layout != null &&
            existing_layout.__ref != null &&
            incoming_layout.__ref != null &&
            existing_layout.__ref === incoming_layout.__ref))) {
        return false;
    }
    const existing_objectInfos = existing.objectInfos;
    const incoming_objectInfos = incoming.objectInfos;
    const equals_objectInfos_props = equalsObject(existing_objectInfos, incoming_objectInfos, (existing_objectInfos_prop, incoming_objectInfos_prop) => {
        if (!(existing_objectInfos_prop.__ref === incoming_objectInfos_prop.__ref)) {
            return false;
        }
    });
    if (equals_objectInfos_props === false) {
        return false;
    }
    const existing_record = existing.record;
    const incoming_record = incoming.record;
    if (!(existing_record.__ref === incoming_record.__ref)) {
        return false;
    }
    return true;
}
const ingest$5 = function RecordDefaultsRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$9(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = path.fullPath;
    const ttlToUse = TTL$4;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$5, "UiApi", VERSION$9, RepresentationType$9, equals$9);
    return createLink$1(key);
};
function getTypeCacheKeys$5(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = fullPathFactory();
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$9,
        mergeable: false
    });
    if (input.layout !== null && typeof input.layout === 'object') {
        getTypeCacheKeys$2b(rootKeySet, luvio, input.layout);
    }
    const input_objectInfos = input.objectInfos;
    const input_objectInfos_keys = ObjectKeys(input_objectInfos);
    const input_objectInfos_length = input_objectInfos_keys.length;
    for (let i = 0; i < input_objectInfos_length; i++) {
        const key = input_objectInfos_keys[i];
        getTypeCacheKeys$2c(rootKeySet, luvio, input_objectInfos[key]);
    }
    getTypeCacheKeys$6(rootKeySet, luvio, input.record, () => rootKey + "__" + "record");
}

function keyBuilder$7(luvio, params) {
    return keyPrefix + '::RecordDefaultsRepresentation:(' + 'formFactor:' + params.queryParams.formFactor + ',' + 'optionalFields:' + params.queryParams.optionalFields + ',' + 'recordTypeId:' + params.queryParams.recordTypeId + ',' + 'objectApiName:' + params.urlParams.objectApiName + ')';
}
function getResponseCacheKeys$6(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$5(storeKeyMap, luvio, response, () => keyBuilder$7(luvio, resourceParams));
}
function createResourceRequest$8(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/record-defaults/create/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$4 = 'getRecordCreateDefaults';
const getRecordCreateDefaults_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('formFactor', false, 1 /* QueryParameter */, 0 /* String */, false, coerceFormFactor),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */, false, getRecordId18),
];
const getRecordCreateDefaults_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$4, getRecordCreateDefaults_ConfigPropertyMetadata);
const createResourceParams$7 = /*#__PURE__*/ createResourceParams$k(getRecordCreateDefaults_ConfigPropertyMetadata);
function typeCheckConfig$8(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordCreateDefaults_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$6(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordCreateDefaults_ConfigPropertyMetadata);
    const config = typeCheckConfig$8(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const contextId$2 = `${keyPrefix}__${adapterName$4}`;
const layoutSelections = select$3c();
const objectInfoSelections = select$3e();
function buildSelector(resp) {
    const recordSelections = buildSelectionFromRecord(resp.record);
    return [
        {
            kind: 'Link',
            name: 'layout',
            nullable: true,
            fragment: layoutSelections,
        },
        {
            kind: 'Link',
            name: 'objectInfos',
            map: true,
            fragment: objectInfoSelections,
        },
        {
            kind: 'Link',
            name: 'record',
            fragment: {
                kind: 'Fragment',
                private: [],
                selections: recordSelections,
                version: VERSION$a,
            },
        },
    ];
}
function buildSnapshotRefresh$1(luvio, context, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot$7(luvio, context, config),
    };
}
function buildNetworkSnapshot$7(luvio, context, config, options) {
    const params = createResourceParams$7(config);
    const request = createResourceRequest$8(params);
    const key = keyBuilder$7(luvio, params);
    const selectorKey = buildSelectorKey(luvio, config);
    return luvio.dispatchResourceRequest(request, options).then((response) => {
        return luvio.handleSuccessResponse(() => {
            const { body } = response;
            const cacheSelector = {
                recordId: key,
                node: {
                    kind: 'Fragment',
                    private: [],
                    version: VERSION$9,
                    selections: buildSelector(body),
                },
                variables: {},
            };
            context.set(selectorKey, cacheSelector);
            luvio.storeIngest(key, ingest$5, body);
            const snapshot = luvio.storeLookup(cacheSelector, buildSnapshotRefresh$1(luvio, context, config));
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$6(cache, luvio, params, response.body);
            return cache;
        });
    }, (err) => {
        return luvio.handleErrorResponse(() => {
            const errorSnapshot = luvio.errorSnapshot(err, buildSnapshotRefresh$1(luvio, context, config));
            luvio.storeIngestError(key, errorSnapshot);
            return luvio.storeBroadcast().then(() => errorSnapshot);
        });
    });
}
function coerceConfigWithDefaults(untrusted) {
    const config = validateAdapterConfig$6(untrusted, getRecordCreateDefaults_ConfigPropertyNames);
    if (config === null) {
        return null;
    }
    let formFactor = config.formFactor;
    if (formFactor === undefined) {
        if (untrusted.formFactor === undefined) {
            formFactor = FormFactor.Large;
        }
        else {
            return null;
        }
    }
    const recordTypeId = config.recordTypeId === undefined ? MAIN_RECORD_TYPE_ID : config.recordTypeId;
    const optionalFields = config.optionalFields === undefined ? [] : config.optionalFields;
    return {
        ...config,
        formFactor,
        recordTypeId,
        optionalFields,
    };
}
function buildSelectorKey(luvio, config) {
    const params = createResourceParams$7(config);
    const key = keyBuilder$7(luvio, params);
    return `${key}__selector`;
}
function buildCachedSnapshotCachePolicy$2(context, storeLookup) {
    const { luvio, config, cachedSelector, adapterContext } = context;
    return cachedSelector === undefined
        ? undefined
        : storeLookup(cachedSelector, buildSnapshotRefresh$1(luvio, adapterContext, config));
}
function buildNetworkSnapshotCachePolicy$2(context, coercedAdapterRequestContext) {
    const { config, adapterContext, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$7(luvio, adapterContext, config, dispatchOptions);
}
const factory$7 = (luvio) => {
    return luvio.withContext(function UiApi__getRecordCreateDefaults(untrusted, adapterContext, requestContext) {
        const config = coerceConfigWithDefaults(untrusted);
        if (config === null) {
            return null;
        }
        const selectorKey = buildSelectorKey(luvio, config);
        /**
         * getRecordCreateDefaults returns a value that includes a map of ObjectInfos,
         * a layout and a record. The returned record includes fields that are not
         * known to the client. Because we don't know what the return shape will be,
         * we have to store a selector from a previous response and see if we can
         * extract those values back out.
         *
         * store cached selector in the adapter context and if it does not exist
         * we need to fetch it from the network and set it.
         */
        const cachedSelector = adapterContext.get(selectorKey);
        return luvio.applyCachePolicy(requestContext || {}, {
            luvio,
            config,
            adapterContext,
            cachedSelector,
        }, buildCachedSnapshotCachePolicy$2, buildNetworkSnapshotCachePolicy$2);
    }, { contextId: contextId$2 });
};

const TTL$3 = 1000;
const VERSION$8 = "6a63c9ca6dda2e125d94e004f83240d8";
function validate$8(obj, path = 'RecordTemplateCloneRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_cloneSourceId = obj.cloneSourceId;
        const path_cloneSourceId = path + '.cloneSourceId';
        if (typeof obj_cloneSourceId !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_cloneSourceId + '" (at "' + path_cloneSourceId + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            if (typeof obj_fields_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
            }
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        let obj_recordTypeId_union0 = null;
        const obj_recordTypeId_union0_error = (() => {
            if (typeof obj_recordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union0_error != null) {
            obj_recordTypeId_union0 = obj_recordTypeId_union0_error.message;
        }
        let obj_recordTypeId_union1 = null;
        const obj_recordTypeId_union1_error = (() => {
            if (obj_recordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union1_error != null) {
            obj_recordTypeId_union1 = obj_recordTypeId_union1_error.message;
        }
        if (obj_recordTypeId_union0 && obj_recordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeId + '")';
            message += '\n' + obj_recordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$8 = 'RecordTemplateCloneRepresentation';
function keyBuilder$6(luvio, config) {
    return keyPrefix + '::' + RepresentationType$8 + ':' + config.cloneSourceId + ':' + (config.recordTypeId === null ? '' : config.recordTypeId);
}
function keyBuilderFromType$4(luvio, object) {
    const keyParams = {
        cloneSourceId: object.cloneSourceId,
        recordTypeId: object.recordTypeId
    };
    return keyBuilder$6(luvio, keyParams);
}
function dynamicNormalize$3(ingestParams) {
    return function normalize_dynamic(input, existing, path, luvio, store, timestamp) {
        const input_fields = input.fields;
        const input_fields_id = path.fullPath + '__fields';
        const input_fields_keys = Object.keys(input_fields);
        const input_fields_length = input_fields_keys.length;
        for (let i = 0; i < input_fields_length; i++) {
            const key = input_fields_keys[i];
            const input_fields_prop = input_fields[key];
            const input_fields_prop_id = input_fields_id + '__' + key;
            input_fields[key] = ingestParams.fields(input_fields_prop, {
                fullPath: input_fields_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        return input;
    };
}
const select$3 = function RecordTemplateCloneRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$8,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'cloneSourceId',
                kind: 'Scalar'
            },
            {
                name: 'fields',
                kind: 'Link',
                map: true,
                fragment: select$3r()
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            }
        ]
    };
};
const dynamicSelect$3 = function dynamicRecordTemplateCloneRepresentationSelect(params) {
    const fieldsPathSelection = params.fields === undefined ? {
        name: 'fields',
        kind: 'Link',
        map: true,
        fragment: select$3r()
    } : params.fields;
    return {
        kind: 'Fragment',
        version: VERSION$8,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'cloneSourceId',
                kind: 'Scalar'
            },
            fieldsPathSelection,
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$8(existing, incoming) {
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_cloneSourceId = existing.cloneSourceId;
    const incoming_cloneSourceId = incoming.cloneSourceId;
    if (!(existing_cloneSourceId === incoming_cloneSourceId)) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_props = equalsObject(existing_fields, incoming_fields, (existing_fields_prop, incoming_fields_prop) => {
        if (!(existing_fields_prop.__ref === incoming_fields_prop.__ref)) {
            return false;
        }
    });
    if (equals_fields_props === false) {
        return false;
    }
    const existing_recordTypeId = existing.recordTypeId;
    const incoming_recordTypeId = incoming.recordTypeId;
    if (!(existing_recordTypeId === incoming_recordTypeId)) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$4(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$4(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$8,
        mergeable: false
    });
    const input_fields = input.fields;
    const input_fields_keys = ObjectKeys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        getTypeCacheKeys$2m(rootKeySet, luvio, input_fields[key], () => rootKey + "__fields" + "__" + key);
    }
}
function dynamicIngest$3(ingestParams) {
    return function RecordTemplateCloneRepresentationIngest(input, path, luvio, store, timestamp) {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$8(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        const key = keyBuilderFromType$4(luvio, input);
        const ttlToUse = TTL$3;
        ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, dynamicNormalize$3(ingestParams), "UiApi", VERSION$8, RepresentationType$8, equals$8);
        return createLink$1(key);
    };
}

const TTL$2 = 1000;
const VERSION$7 = "b352a734f919895e97682703b486b649";
function validate$7(obj, path = 'RecordDefaultsTemplateCloneRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_objectInfos = obj.objectInfos;
        const path_objectInfos = path + '.objectInfos';
        if (typeof obj_objectInfos !== 'object' || ArrayIsArray(obj_objectInfos) || obj_objectInfos === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_objectInfos + '" (at "' + path_objectInfos + '")');
        }
        const obj_objectInfos_keys = ObjectKeys(obj_objectInfos);
        for (let i = 0; i < obj_objectInfos_keys.length; i++) {
            const key = obj_objectInfos_keys[i];
            const obj_objectInfos_prop = obj_objectInfos[key];
            const path_objectInfos_prop = path_objectInfos + '["' + key + '"]';
            if (typeof obj_objectInfos_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_objectInfos_prop + '" (at "' + path_objectInfos_prop + '")');
            }
        }
        const obj_record = obj.record;
        const path_record = path + '.record';
        if (typeof obj_record !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_record + '" (at "' + path_record + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$7 = 'RecordDefaultsTemplateCloneRepresentation';
function keyBuilder$5(luvio, config) {
    return keyPrefix + '::' + RepresentationType$7 + ':' + config.cloneSourceId + ':' + (config.recordTypeId === null ? '' : config.recordTypeId);
}
function keyBuilderFromType$3(luvio, object) {
    const keyParams = {
        cloneSourceId: object.record.cloneSourceId,
        recordTypeId: object.record.recordTypeId
    };
    return keyBuilder$5(luvio, keyParams);
}
function dynamicNormalize$2(ingestParams) {
    return function normalize_dynamic(input, existing, path, luvio, store, timestamp) {
        const input_objectInfos = input.objectInfos;
        const input_objectInfos_id = path.fullPath + '__objectInfos';
        const input_objectInfos_keys = Object.keys(input_objectInfos);
        const input_objectInfos_length = input_objectInfos_keys.length;
        for (let i = 0; i < input_objectInfos_length; i++) {
            const key = input_objectInfos_keys[i];
            const input_objectInfos_prop = input_objectInfos[key];
            const input_objectInfos_prop_id = input_objectInfos_id + '__' + key;
            input_objectInfos[key] = ingestParams.objectInfos(input_objectInfos_prop, {
                fullPath: input_objectInfos_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        const input_record = input.record;
        const input_record_id = path.fullPath + '__record';
        input.record = ingestParams.record(input_record, {
            fullPath: input_record_id,
            propertyName: 'record',
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
        return input;
    };
}
const dynamicSelect$2 = function dynamicRecordDefaultsTemplateCloneRepresentationSelect(params) {
    const objectInfosPathSelection = params.objectInfos === undefined ? {
        name: 'objectInfos',
        kind: 'Link',
        map: true,
        fragment: select$3e()
    } : params.objectInfos;
    const recordPathSelection = params.record === undefined ? {
        name: 'record',
        kind: 'Link',
        fragment: select$3()
    } : params.record;
    return {
        kind: 'Fragment',
        version: VERSION$7,
        private: [],
        selections: [
            objectInfosPathSelection,
            recordPathSelection
        ]
    };
};
function equals$7(existing, incoming) {
    const existing_objectInfos = existing.objectInfos;
    const incoming_objectInfos = incoming.objectInfos;
    const equals_objectInfos_props = equalsObject(existing_objectInfos, incoming_objectInfos, (existing_objectInfos_prop, incoming_objectInfos_prop) => {
        if (!(existing_objectInfos_prop.__ref === incoming_objectInfos_prop.__ref)) {
            return false;
        }
    });
    if (equals_objectInfos_props === false) {
        return false;
    }
    const existing_record = existing.record;
    const incoming_record = incoming.record;
    if (!(existing_record.__ref === incoming_record.__ref)) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$3(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$3(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$7,
        mergeable: false
    });
    const input_objectInfos = input.objectInfos;
    const input_objectInfos_keys = ObjectKeys(input_objectInfos);
    const input_objectInfos_length = input_objectInfos_keys.length;
    for (let i = 0; i < input_objectInfos_length; i++) {
        const key = input_objectInfos_keys[i];
        getTypeCacheKeys$2c(rootKeySet, luvio, input_objectInfos[key]);
    }
    getTypeCacheKeys$4(rootKeySet, luvio, input.record);
}
function dynamicIngest$2(ingestParams) {
    return function RecordDefaultsTemplateCloneRepresentationIngest(input, path, luvio, store, timestamp) {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$7(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        const key = keyBuilderFromType$3(luvio, input);
        const ttlToUse = TTL$2;
        ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, dynamicNormalize$2(ingestParams), "UiApi", VERSION$7, RepresentationType$7, equals$7);
        return createLink$1(key);
    };
}

function createFieldsIngest$3(params) {
    const { fields, optionalFields, trackedFields, recordConflictMap } = params;
    const ingest = dynamicIngest$3({
        fields: createFieldsIngestion(fields, optionalFields, recordConflictMap)
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$5(fields) {
    return dynamicSelect$3({
        fields: createPathSelection('fields', fields)
    });
}

function createFieldsIngest$2(params) {
    const { fields, optionalFields, trackedFields, recordConflictMap } = params;
    const ingest = dynamicIngest$2({
        objectInfos: ingest$29,
        record: createFieldsIngest$3(params)
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$4(fields) {
    return dynamicSelect$2({
        record: {
            kind: 'Link',
            name: 'record',
            fragment: selectFields$5(fields),
        }
    });
}

function createFieldsIngestSuccess$1(params) {
    const { trackedFields } = params;
    const recordConflictMap = {
        conflicts: {},
        serverRequestCount: 1,
    };
    const ingest = createFieldsIngest$2({
        ...params,
        recordConflictMap,
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        resolveConflict(luvio, recordConflictMap);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$3(luvio, params) {
    const optionalFields = params.queryParams.optionalFields || [];
    const fields = [];
    const trie = convertRecordFieldsArrayToTrie(fields, optionalFields);
    return selectFields$4(trie);
}

const select$2 = (luvio, params) => {
    let { optionalFields } = params.queryParams;
    const entry = luvio.getNode(keyBuilder$6(luvio, {
        cloneSourceId: params.urlParams.recordId,
        recordTypeId: params.queryParams.recordTypeId || null,
    }));
    // If we have data in the cache, we know the only fields that are relevant start with that record's objectApiName
    if (entry === null || entry === void 0 ? void 0 : entry.data) {
        const apiName = entry.data.apiName;
        if (optionalFields !== undefined) {
            optionalFields = optionalFields.filter((field) => field.startsWith(`${apiName}.`));
        }
    }
    optionalFields =
        optionalFields === undefined ? ['.CloneSourceId'] : [...optionalFields, '.CloneSourceId'];
    return selectFields$3(luvio, {
        ...params,
        queryParams: {
            ...params.queryParams,
            optionalFields,
        },
    });
};

function getResponseCacheKeys$5(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$3(storeKeyMap, luvio, response);
}
function createResourceRequest$7(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/record-defaults/template/clone/' + config.urlParams.recordId + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName$3 = 'getRecordTemplateClone';
const getRecordTemplateClone_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getRecordTemplateClone_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$3, getRecordTemplateClone_ConfigPropertyMetadata);
const createResourceParams$6 = /*#__PURE__*/ createResourceParams$k(getRecordTemplateClone_ConfigPropertyMetadata);
function typeCheckConfig$7(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordTemplateClone_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$5(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordTemplateClone_ConfigPropertyMetadata);
    const config = typeCheckConfig$7(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const contextId$1 = `${keyPrefix}__${adapterName$3}`;
const DEFAULT_RECORD_TYPE_ID_KEY = 'defaultRecordTypeId';
const RECORD_TEMPLATE_CLONE_ERROR_VERSION = 'RECORD_TEMPLATE_CLONE_ERROR_VERSION_1';
const RECORD_TEMPLATE_CLONE_ERROR_STORE_METADATA_PARAMS = {
    ttl: TTL$2,
    representationName: '',
    namespace: keyPrefix,
    version: RECORD_TEMPLATE_CLONE_ERROR_VERSION,
};
function saveDefaultRecordTypeId(context, objectInfo) {
    context.set(DEFAULT_RECORD_TYPE_ID_KEY, objectInfo.defaultRecordTypeId);
}
function getRecordTypeId$1(adapterConfig, context) {
    const { recordTypeId } = adapterConfig;
    if (recordTypeId !== undefined && recordTypeId !== null) {
        return recordTypeId;
    }
    const contextValue = context.get(DEFAULT_RECORD_TYPE_ID_KEY);
    if (contextValue === null || contextValue === undefined) {
        return undefined;
    }
    return contextValue;
}
const buildNetworkSnapshot$6 = (luvio, context, config, options) => {
    const resourceParams = createResourceParams$6(config);
    const recordTypeId = getRecordTypeId$1(config, context);
    const { recordId } = config;
    const resourceRequest = createResourceRequest$7(resourceParams);
    const coercedRecordTypeId = recordTypeId === undefined ? null : recordTypeId;
    const templateRecordKey = keyBuilder$6(luvio, {
        cloneSourceId: recordId,
        recordTypeId: coercedRecordTypeId,
    });
    const request = recordTypeId === undefined
        ? resourceRequest
        : createResourceRequest$7({
            ...resourceParams,
            queryParams: {
                ...resourceRequest.queryParams,
                optionalFields: getTrackedFields(templateRecordKey, luvio.getNode(templateRecordKey), {
                    maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMiss(),
                    onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
                }, config.optionalFields),
            },
        });
    return luvio
        .dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const { body } = response;
            const key = keyBuilderFromType$3(luvio, body);
            const responseRecordTypeId = body.record.recordTypeId;
            const objectApiName = body.record.apiName;
            // publish metadata for recordTypeId
            saveDefaultRecordTypeId(context, body.objectInfos[objectApiName]);
            const optionalFieldsTrie = convertFieldsToTrie(resourceParams.queryParams.optionalFields, true, objectApiName);
            luvio.storeIngest(key, createFieldsIngestSuccess$1({
                fields: BLANK_RECORD_FIELDS_TRIE,
                optionalFields: optionalFieldsTrie,
                trackedFields: optionalFieldsTrie,
                serverRequestCount: 1,
            }), body);
            const snapshot = buildCachedSnapshot$1(luvio, context, {
                ...config,
                recordTypeId: responseRecordTypeId,
            });
            if (process.env.NODE_ENV !== 'production') {
                if (snapshot.state !== 'Fulfilled') {
                    throw new Error('Invalid network response. Expected network response to result in Fulfilled snapshot');
                }
            }
            return luvio.storeBroadcast().then(() => {
                return snapshot;
            });
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$5(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => {
            const key = keyBuilder$5(luvio, {
                cloneSourceId: config.recordId,
                recordTypeId: config.recordTypeId || null,
            });
            const errorSnapshot = luvio.errorSnapshot(response, {
                config,
                resolve: () => buildNetworkSnapshot$6(luvio, context, config, snapshotRefreshOptions),
            });
            luvio.storeIngestError(key, errorSnapshot, RECORD_TEMPLATE_CLONE_ERROR_STORE_METADATA_PARAMS);
            return Promise.resolve(errorSnapshot);
        });
    });
};
const buildCachedSnapshot$1 = (luvio, context, config) => {
    const resourceParams = createResourceParams$6(config);
    const key = keyBuilder$5(luvio, {
        cloneSourceId: config.recordId,
        recordTypeId: config.recordTypeId || null,
    });
    const selector = {
        recordId: key,
        node: select$2(luvio, resourceParams),
        variables: {},
    };
    return luvio.storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$6(luvio, context, config, snapshotRefreshOptions),
    });
};
function buildNetworkSnapshotCachePolicy$1(context, coercedAdapterRequestContext) {
    const { config, adapterContext, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$6(luvio, adapterContext, config, dispatchOptions);
}
function buildCachedSnapshotCachePolicy$1(context, storeLookup) {
    const { adapterContext, config, luvio, recordTypeId } = context;
    const updatedConfig = {
        ...config,
        recordTypeId,
    };
    const resourceParams = createResourceParams$6(updatedConfig);
    const key = keyBuilder$5(luvio, {
        cloneSourceId: updatedConfig.recordId,
        recordTypeId: updatedConfig.recordTypeId || null,
    });
    const selector = {
        recordId: key,
        node: select$2(luvio, resourceParams),
        variables: {},
    };
    return storeLookup(selector, {
        config,
        resolve: () => buildNetworkSnapshot$6(luvio, adapterContext, updatedConfig, snapshotRefreshOptions),
    });
}
const factory$6 = (luvio) => luvio.withContext(function getRecordTemplateClone_ContextWrapper(untrustedConfig, adapterContext, requestContext) {
    const config = validateAdapterConfig$5(untrustedConfig, getRecordTemplateClone_ConfigPropertyNames);
    // Invalid or incomplete config
    if (config === null) {
        return null;
    }
    const recordTypeId = getRecordTypeId$1(config, adapterContext);
    return luvio.applyCachePolicy(requestContext || {}, {
        luvio,
        config,
        recordTypeId,
        adapterContext,
    }, buildCachedSnapshotCachePolicy$1, buildNetworkSnapshotCachePolicy$1);
}, { contextId: contextId$1 });

const TTL$1 = 30000;
const VERSION$6 = "665de0fdde39efd6f1098e374117ef58";
function validate$6(obj, path = 'RecordTemplateCreateRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_apiName = obj.apiName;
        const path_apiName = path + '.apiName';
        if (typeof obj_apiName !== 'string') {
            return new TypeError('Expected "string" but received "' + typeof obj_apiName + '" (at "' + path_apiName + '")');
        }
        const obj_fields = obj.fields;
        const path_fields = path + '.fields';
        if (typeof obj_fields !== 'object' || ArrayIsArray(obj_fields) || obj_fields === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_fields + '" (at "' + path_fields + '")');
        }
        const obj_fields_keys = ObjectKeys(obj_fields);
        for (let i = 0; i < obj_fields_keys.length; i++) {
            const key = obj_fields_keys[i];
            const obj_fields_prop = obj_fields[key];
            const path_fields_prop = path_fields + '["' + key + '"]';
            if (typeof obj_fields_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_fields_prop + '" (at "' + path_fields_prop + '")');
            }
        }
        const obj_recordTypeId = obj.recordTypeId;
        const path_recordTypeId = path + '.recordTypeId';
        let obj_recordTypeId_union0 = null;
        const obj_recordTypeId_union0_error = (() => {
            if (typeof obj_recordTypeId !== 'string') {
                return new TypeError('Expected "string" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union0_error != null) {
            obj_recordTypeId_union0 = obj_recordTypeId_union0_error.message;
        }
        let obj_recordTypeId_union1 = null;
        const obj_recordTypeId_union1_error = (() => {
            if (obj_recordTypeId !== null) {
                return new TypeError('Expected "null" but received "' + typeof obj_recordTypeId + '" (at "' + path_recordTypeId + '")');
            }
        })();
        if (obj_recordTypeId_union1_error != null) {
            obj_recordTypeId_union1 = obj_recordTypeId_union1_error.message;
        }
        if (obj_recordTypeId_union0 && obj_recordTypeId_union1) {
            let message = 'Object doesn\'t match union (at "' + path_recordTypeId + '")';
            message += '\n' + obj_recordTypeId_union0.split('\n').map((line) => '\t' + line).join('\n');
            message += '\n' + obj_recordTypeId_union1.split('\n').map((line) => '\t' + line).join('\n');
            return new TypeError(message);
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$6 = 'RecordTemplateCreateRepresentation';
function keyBuilder$4(luvio, config) {
    return keyPrefix + '::' + RepresentationType$6 + ':' + config.apiName + ':' + (config.recordTypeId === null ? '' : config.recordTypeId);
}
function keyBuilderFromType$2(luvio, object) {
    const keyParams = {
        apiName: object.apiName,
        recordTypeId: object.recordTypeId
    };
    return keyBuilder$4(luvio, keyParams);
}
function dynamicNormalize$1(ingestParams) {
    return function normalize_dynamic(input, existing, path, luvio, store, timestamp) {
        const input_fields = input.fields;
        const input_fields_id = path.fullPath + '__fields';
        const input_fields_keys = Object.keys(input_fields);
        const input_fields_length = input_fields_keys.length;
        for (let i = 0; i < input_fields_length; i++) {
            const key = input_fields_keys[i];
            const input_fields_prop = input_fields[key];
            const input_fields_prop_id = input_fields_id + '__' + key;
            input_fields[key] = ingestParams.fields(input_fields_prop, {
                fullPath: input_fields_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        return input;
    };
}
const select$1 = function RecordTemplateCreateRepresentationSelect() {
    return {
        kind: 'Fragment',
        version: VERSION$6,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            {
                name: 'fields',
                kind: 'Link',
                map: true,
                fragment: select$3r()
            },
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            }
        ]
    };
};
const dynamicSelect$1 = function dynamicRecordTemplateCreateRepresentationSelect(params) {
    const fieldsPathSelection = params.fields === undefined ? {
        name: 'fields',
        kind: 'Link',
        map: true,
        fragment: select$3r()
    } : params.fields;
    return {
        kind: 'Fragment',
        version: VERSION$6,
        private: [],
        selections: [
            {
                name: 'apiName',
                kind: 'Scalar'
            },
            fieldsPathSelection,
            {
                name: 'recordTypeId',
                kind: 'Scalar'
            }
        ]
    };
};
function equals$6(existing, incoming) {
    const existing_apiName = existing.apiName;
    const incoming_apiName = incoming.apiName;
    if (!(existing_apiName === incoming_apiName)) {
        return false;
    }
    const existing_fields = existing.fields;
    const incoming_fields = incoming.fields;
    const equals_fields_props = equalsObject(existing_fields, incoming_fields, (existing_fields_prop, incoming_fields_prop) => {
        if (!(existing_fields_prop.__ref === incoming_fields_prop.__ref)) {
            return false;
        }
    });
    if (equals_fields_props === false) {
        return false;
    }
    const existing_recordTypeId = existing.recordTypeId;
    const incoming_recordTypeId = incoming.recordTypeId;
    if (!(existing_recordTypeId === incoming_recordTypeId)) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$2(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$2(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$6,
        mergeable: false
    });
    const input_fields = input.fields;
    const input_fields_keys = ObjectKeys(input_fields);
    const input_fields_length = input_fields_keys.length;
    for (let i = 0; i < input_fields_length; i++) {
        const key = input_fields_keys[i];
        getTypeCacheKeys$2m(rootKeySet, luvio, input_fields[key], () => rootKey + "__fields" + "__" + key);
    }
}
function dynamicIngest$1(ingestParams) {
    return function RecordTemplateCreateRepresentationIngest(input, path, luvio, store, timestamp) {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$6(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        const key = keyBuilderFromType$2(luvio, input);
        const ttlToUse = TTL$1;
        ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, dynamicNormalize$1(ingestParams), "UiApi", VERSION$6, RepresentationType$6, equals$6);
        return createLink$1(key);
    };
}

const TTL = 900000;
const VERSION$5 = "30493e7ac0677e35242026bb959c079e";
function validate$5(obj, path = 'RecordDefaultsTemplateCreateRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_objectInfos = obj.objectInfos;
        const path_objectInfos = path + '.objectInfos';
        if (typeof obj_objectInfos !== 'object' || ArrayIsArray(obj_objectInfos) || obj_objectInfos === null) {
            return new TypeError('Expected "object" but received "' + typeof obj_objectInfos + '" (at "' + path_objectInfos + '")');
        }
        const obj_objectInfos_keys = ObjectKeys(obj_objectInfos);
        for (let i = 0; i < obj_objectInfos_keys.length; i++) {
            const key = obj_objectInfos_keys[i];
            const obj_objectInfos_prop = obj_objectInfos[key];
            const path_objectInfos_prop = path_objectInfos + '["' + key + '"]';
            if (typeof obj_objectInfos_prop !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_objectInfos_prop + '" (at "' + path_objectInfos_prop + '")');
            }
        }
        const obj_record = obj.record;
        const path_record = path + '.record';
        if (typeof obj_record !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_record + '" (at "' + path_record + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$5 = 'RecordDefaultsTemplateCreateRepresentation';
function keyBuilder$3(luvio, config) {
    return keyPrefix + '::' + RepresentationType$5 + ':' + config.objectApiName + ':' + (config.recordTypeId === null ? '' : config.recordTypeId);
}
function keyBuilderFromType$1(luvio, object) {
    const keyParams = {
        objectApiName: object.record.apiName,
        recordTypeId: object.record.recordTypeId
    };
    return keyBuilder$3(luvio, keyParams);
}
function dynamicNormalize(ingestParams) {
    return function normalize_dynamic(input, existing, path, luvio, store, timestamp) {
        const input_objectInfos = input.objectInfos;
        const input_objectInfos_id = path.fullPath + '__objectInfos';
        const input_objectInfos_keys = Object.keys(input_objectInfos);
        const input_objectInfos_length = input_objectInfos_keys.length;
        for (let i = 0; i < input_objectInfos_length; i++) {
            const key = input_objectInfos_keys[i];
            const input_objectInfos_prop = input_objectInfos[key];
            const input_objectInfos_prop_id = input_objectInfos_id + '__' + key;
            input_objectInfos[key] = ingestParams.objectInfos(input_objectInfos_prop, {
                fullPath: input_objectInfos_prop_id,
                propertyName: key,
                parent: {
                    data: input,
                    key: path.fullPath,
                    existing: existing,
                },
                ttl: path.ttl
            }, luvio, store, timestamp);
        }
        const input_record = input.record;
        const input_record_id = path.fullPath + '__record';
        input.record = ingestParams.record(input_record, {
            fullPath: input_record_id,
            propertyName: 'record',
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
        return input;
    };
}
const dynamicSelect = function dynamicRecordDefaultsTemplateCreateRepresentationSelect(params) {
    const objectInfosPathSelection = params.objectInfos === undefined ? {
        name: 'objectInfos',
        kind: 'Link',
        map: true,
        fragment: select$3e()
    } : params.objectInfos;
    const recordPathSelection = params.record === undefined ? {
        name: 'record',
        kind: 'Link',
        fragment: select$1()
    } : params.record;
    return {
        kind: 'Fragment',
        version: VERSION$5,
        private: [],
        selections: [
            objectInfosPathSelection,
            recordPathSelection
        ]
    };
};
function equals$5(existing, incoming) {
    const existing_objectInfos = existing.objectInfos;
    const incoming_objectInfos = incoming.objectInfos;
    const equals_objectInfos_props = equalsObject(existing_objectInfos, incoming_objectInfos, (existing_objectInfos_prop, incoming_objectInfos_prop) => {
        if (!(existing_objectInfos_prop.__ref === incoming_objectInfos_prop.__ref)) {
            return false;
        }
    });
    if (equals_objectInfos_props === false) {
        return false;
    }
    const existing_record = existing.record;
    const incoming_record = incoming.record;
    if (!(existing_record.__ref === incoming_record.__ref)) {
        return false;
    }
    return true;
}
function getTypeCacheKeys$1(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType$1(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$5,
        mergeable: false
    });
    const input_objectInfos = input.objectInfos;
    const input_objectInfos_keys = ObjectKeys(input_objectInfos);
    const input_objectInfos_length = input_objectInfos_keys.length;
    for (let i = 0; i < input_objectInfos_length; i++) {
        const key = input_objectInfos_keys[i];
        getTypeCacheKeys$2c(rootKeySet, luvio, input_objectInfos[key]);
    }
    getTypeCacheKeys$2(rootKeySet, luvio, input.record);
}
function dynamicIngest(ingestParams) {
    return function RecordDefaultsTemplateCreateRepresentationIngest(input, path, luvio, store, timestamp) {
        if (process.env.NODE_ENV !== 'production') {
            const validateError = validate$5(input);
            if (validateError !== null) {
                throw validateError;
            }
        }
        const key = keyBuilderFromType$1(luvio, input);
        const ttlToUse = TTL;
        ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, dynamicNormalize(ingestParams), "UiApi", VERSION$5, RepresentationType$5, equals$5);
        return createLink$1(key);
    };
}

function createFieldsIngest$1(params) {
    const { fields, optionalFields, trackedFields, recordConflictMap } = params;
    const ingest = dynamicIngest$1({
        fields: createFieldsIngestion(fields, optionalFields, recordConflictMap)
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$2(fields) {
    return dynamicSelect$1({
        fields: createPathSelection('fields', fields)
    });
}

function createFieldsIngest(params) {
    const { fields, optionalFields, trackedFields, recordConflictMap } = params;
    const ingest = dynamicIngest({
        objectInfos: ingest$29,
        record: createFieldsIngest$1(params)
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields$1(fields) {
    return dynamicSelect({
        record: {
            kind: 'Link',
            name: 'record',
            fragment: selectFields$2(fields),
        }
    });
}

function createFieldsIngestSuccess(params) {
    const { trackedFields } = params;
    const recordConflictMap = {
        conflicts: {},
        serverRequestCount: 1,
    };
    const ingest = createFieldsIngest({
        ...params,
        recordConflictMap,
    });
    return (data, path, luvio, store, timestamp) => {
        const link = ingest(data, path, luvio, store, timestamp);
        resolveConflict(luvio, recordConflictMap);
        const recordNode = luvio.getNode(link.__ref);
        markMissingOptionalFields(recordNode, convertTrieToFields(trackedFields));
        return link;
    };
}
function selectFields(luvio, params) {
    const optionalFields = params.queryParams.optionalFields || [];
    const fields = [];
    const trie = convertRecordFieldsArrayToTrie(fields, optionalFields);
    return selectFields$1(trie);
}

function keyBuilder$2(luvio, params) {
    return keyBuilder$3(luvio, {
        objectApiName: params.urlParams.objectApiName,
        recordTypeId: params.queryParams.recordTypeId || null
    });
}
function getResponseCacheKeys$4(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$1(storeKeyMap, luvio, response);
}
function ingestError(luvio, params, error, snapshotRefresh) {
    const key = keyBuilder$2(luvio, params);
    const errorSnapshot = luvio.errorSnapshot(error, snapshotRefresh);
    const storeMetadataParams = {
        ttl: TTL,
        namespace: keyPrefix,
        version: VERSION$5,
        representationName: RepresentationType$5
    };
    luvio.storeIngestError(key, errorSnapshot, storeMetadataParams);
    return errorSnapshot;
}
function createResourceRequest$6(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/record-defaults/template/create/' + config.urlParams.objectApiName + '',
        method: 'get',
        body: null,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

function adapterFragment(luvio, config) {
    const resourceParams = createResourceParams$5(config);
    return selectFields(luvio, resourceParams);
}

const adapterName$2 = 'getRecordTemplateCreate';
const getRecordTemplateCreate_ConfigPropertyMetadata = [
    generateParamConfigMetadata('objectApiName', true, 0 /* UrlParameter */, 0 /* String */, false, getObjectApiName$1),
    generateParamConfigMetadata('optionalFields', false, 1 /* QueryParameter */, 0 /* String */, true, getFieldApiNamesArray),
    generateParamConfigMetadata('recordTypeId', false, 1 /* QueryParameter */, 0 /* String */),
];
const getRecordTemplateCreate_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$2, getRecordTemplateCreate_ConfigPropertyMetadata);
const createResourceParams$5 = /*#__PURE__*/ createResourceParams$k(getRecordTemplateCreate_ConfigPropertyMetadata);
function typeCheckConfig$6(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, getRecordTemplateCreate_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$4(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, getRecordTemplateCreate_ConfigPropertyMetadata);
    const config = typeCheckConfig$6(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const contextId = `${keyPrefix}__${adapterName$2}`;
function buildSnapshotRefresh(luvio, context, config) {
    return {
        config,
        resolve: () => buildNetworkSnapshot$5(luvio, context, config, snapshotRefreshOptions),
    };
}
function buildRecordTypeIdContextKey(objectApiName) {
    return `DEFAULTS::recordTypeId:${objectApiName}`;
}
function getRecordTypeId(context, adapterConfig) {
    const config = createResourceParams$5(adapterConfig);
    const { recordTypeId } = config.queryParams;
    if (recordTypeId !== undefined && recordTypeId !== null) {
        return recordTypeId;
    }
    const saved = context.get(buildRecordTypeIdContextKey(config.urlParams.objectApiName));
    if (saved === null || saved === undefined) {
        return undefined;
    }
    return saved;
}
function prepareRequest(luvio, context, config) {
    const resourceParams = createResourceParams$5(config);
    const recordTypeId = getRecordTypeId(context, config);
    const { objectApiName } = config;
    const resourceRequest = createResourceRequest$6(resourceParams);
    if (recordTypeId === undefined) {
        return resourceRequest;
    }
    const recordTemplateKey = keyBuilder$4(luvio, {
        apiName: objectApiName,
        recordTypeId: recordTypeId,
    });
    return createResourceRequest$6({
        ...resourceParams,
        queryParams: {
            ...resourceRequest.queryParams,
            optionalFields: getTrackedFields(recordTemplateKey, luvio.getNode(recordTemplateKey), {
                maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMiss(),
                onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
            }, config.optionalFields),
        },
    });
}
function onFetchResponseSuccess(luvio, context, config, request, response, resourceParams) {
    const { urlParams: { objectApiName }, queryParams: { optionalFields }, } = resourceParams;
    const { body } = response;
    const key = keyBuilderFromType$1(luvio, body);
    const responseRecordTypeId = body.record.recordTypeId;
    // publish metadata for recordTypeId
    const recordTypeId = body.objectInfos[objectApiName].defaultRecordTypeId;
    context.set(buildRecordTypeIdContextKey(objectApiName), recordTypeId);
    // mark missing optionalFields
    const templateRecordKey = keyBuilder$4(luvio, {
        apiName: objectApiName,
        recordTypeId: responseRecordTypeId,
    });
    const allTrackedFields = getTrackedFields(templateRecordKey, luvio.getNode(templateRecordKey), {
        maxDepth: configurationForRestAdapters.getTrackedFieldDepthOnCacheMiss(),
        onlyFetchLeafNodeIdAndName: configurationForRestAdapters.getTrackedFieldLeafNodeIdAndNameOnly(),
    }, optionalFields);
    const allTrackedFieldsTrie = convertFieldsToTrie(allTrackedFields, true);
    const ingest = createFieldsIngestSuccess({
        fields: BLANK_RECORD_FIELDS_TRIE,
        optionalFields: allTrackedFieldsTrie,
        trackedFields: allTrackedFieldsTrie,
        serverRequestCount: 1,
    });
    luvio.storeIngest(key, ingest, body);
    const snapshot = buildCachedSnapshot(luvio, context, {
        ...config,
        recordTypeId: responseRecordTypeId,
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected network response to result in Fulfilled snapshot');
        }
    }
    return luvio
        .storeBroadcast()
        .then(() => snapshot);
}
function onFetchResponseError(luvio, context, config, resourceParams, error) {
    const snapshot = ingestError(luvio, resourceParams, error, buildSnapshotRefresh(luvio, context, config));
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$5(luvio, context, config, options) {
    const resourceParams = createResourceParams$5(config);
    const request = prepareRequest(luvio, context, config);
    return luvio
        .dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            return onFetchResponseSuccess(luvio, context, config, request, response, resourceParams);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$4(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        return luvio.handleErrorResponse(() => {
            return onFetchResponseError(luvio, context, config, resourceParams, response);
        });
    });
}
function buildCachedSnapshot(luvio, context, config) {
    const resourceParams = createResourceParams$5(config);
    const selector = {
        recordId: keyBuilder$2(luvio, resourceParams),
        node: adapterFragment(luvio, config),
        variables: {},
    };
    return luvio.storeLookup(selector, buildSnapshotRefresh(luvio, context, config));
}
const buildNetworkSnapshotCachePolicy = (context, coercedAdapterRequestContext) => {
    const { config, adapterContext, luvio } = context;
    const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
    const dispatchOptions = {
        resourceRequestContext: {
            requestCorrelator,
            sourceContext,
        },
        eventObservers,
    };
    if (networkPriority !== 'normal') {
        dispatchOptions.overrides = {
            priority: networkPriority,
        };
    }
    return buildNetworkSnapshot$5(luvio, adapterContext, config, dispatchOptions);
};
const buildCachedSnapshotCachePolicy = (context, storeLookup) => {
    const { adapterContext, config, luvio, recordTypeId } = context;
    const updatedConfig = {
        ...config,
        recordTypeId,
    };
    const resourceParams = createResourceParams$5(updatedConfig);
    const selector = {
        recordId: keyBuilder$2(luvio, resourceParams),
        node: adapterFragment(luvio, config),
        variables: {},
    };
    return storeLookup(selector, buildSnapshotRefresh(luvio, adapterContext, config));
};
const factory$5 = (luvio) => {
    return luvio.withContext(function UiApi__getRecordDefaultsTemplateForCreate(untrustedConfig, adapterContext, requestContext) {
        const config = validateAdapterConfig$4(untrustedConfig, getRecordTemplateCreate_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            return null;
        }
        const recordTypeId = getRecordTypeId(adapterContext, config);
        return luvio.applyCachePolicy(requestContext || {}, {
            luvio,
            config,
            recordTypeId,
            adapterContext,
        }, buildCachedSnapshotCachePolicy, buildNetworkSnapshotCachePolicy);
    }, { contextId });
};

function getResponseCacheKeys$3(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2l(storeKeyMap, luvio, response);
}
function createResourceRequest$5(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const createRecord_ConfigPropertyMetadata = [
    generateParamConfigMetadata('handleOwnerChange', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('includeFieldsInBody', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('triggerOtherEmail', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('triggerUserEmail', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('useDefaultRule', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('allowSaveOnDuplicate', false, 2 /* Body */, 1 /* Boolean */),
    generateParamConfigMetadata('apiName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('fields', true, 2 /* Body */, 4 /* Unsupported */),
];
const createResourceParams$4 = /*#__PURE__*/ createResourceParams$k(createRecord_ConfigPropertyMetadata);

function onResponseSuccess(luvio, response, recordIngest, conflictMap) {
    const { body, status } = response;
    if (status === HttpStatusCode.NoContent) {
        const syntheticSnapshot = {
            state: 'Fulfilled',
            data: {},
        };
        return luvio.storeBroadcast().then(() => syntheticSnapshot);
    }
    const selections = buildSelectionFromRecord(body);
    const key = keyBuilder$3E(luvio, {
        recordId: body.id,
    });
    luvio.storeIngest(key, recordIngest, body);
    resolveConflict(luvio, conflictMap);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node: {
            kind: 'Fragment',
            private: [],
            selections,
            version: VERSION$2K,
        },
        variables: {},
    });
    return luvio.storeBroadcast().then(() => snapshot);
}
function buildNetworkSnapshot$4(luvio, config) {
    const resourceParams = createResourceParams$4(config);
    const request = createResourceRequest$5(resourceParams);
    const fieldTrie = BLANK_RECORD_FIELDS_TRIE;
    const optionalFieldTrie = BLANK_RECORD_FIELDS_TRIE;
    const conflictMap = {
        conflicts: {},
        serverRequestCount: 1,
    };
    const recordIngest = createRecordIngest(fieldTrie, optionalFieldTrie, conflictMap);
    return luvio.dispatchResourceRequest(request).then((response) => {
        return luvio.handleSuccessResponse(() => onResponseSuccess(luvio, response, recordIngest, conflictMap), () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$3(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (err) => {
        deepFreeze(err);
        throw err;
    });
}
function createResourceRequest$4(config) {
    const resourceParams = createResourceParams$4(config);
    return createResourceRequest$5(resourceParams);
}
const factory$4 = (luvio) => {
    return function (untrustedConfig) {
        const config = untrustedConfig;
        const draftAdapter = configurationForRestAdapters.getDraftAwareCreateRecordAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, createResourceRequest$4);
        }
        return buildNetworkSnapshot$4(luvio, config);
    };
};

function keyBuilder$1(luvio, params) {
    return keyBuilder$3E(luvio, {
        recordId: params.urlParams.recordId
    });
}
function getResponseCacheKeys$2(cacheKeyMap, luvio, resourceParams) {
    const key = keyBuilder$1(luvio, resourceParams);
    cacheKeyMap.set(key, {
        namespace: keyPrefix,
        representationName: RepresentationType$_,
        mergeable: false
    });
}
function evictSuccess(luvio, resourceParams) {
    const key = keyBuilder$1(luvio, resourceParams);
    luvio.storeEvict(key);
}
function createResourceRequest$3(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/' + config.urlParams.recordId + '',
        method: 'delete',
        body: null,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}

const adapterName$1 = 'deleteRecord';
const deleteRecord_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
];
const deleteRecord_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName$1, deleteRecord_ConfigPropertyMetadata);
const createResourceParams$3 = /*#__PURE__*/ createResourceParams$k(deleteRecord_ConfigPropertyMetadata);
function typeCheckConfig$5(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, deleteRecord_ConfigPropertyMetadata);
    return config;
}
function validateAdapterConfig$3(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, deleteRecord_ConfigPropertyMetadata);
    const config = typeCheckConfig$5(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}
function buildNetworkSnapshot$3(luvio, config, options) {
    const resourceParams = createResourceParams$3(config);
    const request = createResourceRequest$3(resourceParams);
    return luvio.dispatchResourceRequest(request, options)
        .then(() => {
        return luvio.handleSuccessResponse(() => {
            evictSuccess(luvio, resourceParams);
            return luvio.storeBroadcast();
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$2(cache, luvio, resourceParams);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}
const deleteRecordAdapterFactory = (luvio) => {
    return function UiApideleteRecord(untrustedConfig) {
        const config = validateAdapterConfig$3(untrustedConfig, deleteRecord_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            throw new Error(`Invalid config for "${adapterName$1}"`);
        }
        return buildNetworkSnapshot$3(luvio, config);
    };
};

const factory$3 = (luvio) => {
    const deleteRecordAdapterInstance = deleteRecordAdapterFactory(luvio);
    return (recordId) => {
        const config = { recordId };
        const draftAdapter = configurationForRestAdapters.getDraftAwareDeleteRecordAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, (config) => {
                const params = createResourceParams$3(config);
                return createResourceRequest$3(params);
            });
        }
        return deleteRecordAdapterInstance(config);
    };
};

function getResponseCacheKeys$1(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys$2l(storeKeyMap, luvio, response);
}
function createResourceRequest$2(config) {
    const headers = {};
    const header_ifUnmodifiedSince = config.headers.ifUnmodifiedSince;
    if (header_ifUnmodifiedSince !== undefined) {
        headers['If-Unmodified-Since'] = header_ifUnmodifiedSince;
    }
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/' + config.urlParams.recordId + '',
        method: 'patch',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: config.queryParams,
        headers,
        priority: 'normal',
    };
}

const adapterName = 'updateRecord';
const updateRecord_ConfigPropertyMetadata = [
    generateParamConfigMetadata('recordId', true, 0 /* UrlParameter */, 0 /* String */, false, getRecordId18),
    generateParamConfigMetadata('handleOwnerChange', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('triggerOtherEmail', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('triggerUserEmail', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('useDefaultRule', false, 1 /* QueryParameter */, 1 /* Boolean */),
    generateParamConfigMetadata('allowSaveOnDuplicate', false, 2 /* Body */, 1 /* Boolean */),
    generateParamConfigMetadata('apiName', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('fields', true, 2 /* Body */, 4 /* Unsupported */),
    generateParamConfigMetadata('ifUnmodifiedSince', false, 3 /* Header */, 0 /* String */),
];
const updateRecord_ConfigPropertyNames = /*#__PURE__*/ buildAdapterValidationConfig(adapterName, updateRecord_ConfigPropertyMetadata);
const createResourceParams$2 = /*#__PURE__*/ createResourceParams$k(updateRecord_ConfigPropertyMetadata);
function typeCheckConfig$4(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, updateRecord_ConfigPropertyMetadata);
    const untrustedConfig_fields = untrustedConfig.fields;
    if (untrustedIsObject(untrustedConfig_fields)) {
        const untrustedConfig_fields_object = {};
        const untrustedConfig_fields_keys = Object.keys(untrustedConfig_fields);
        for (let i = 0, arrayLength = untrustedConfig_fields_keys.length; i < arrayLength; i++) {
            const key = untrustedConfig_fields_keys[i];
            const untrustedConfig_fields_prop = untrustedConfig_fields[key];
            if (typeof untrustedConfig_fields_prop === 'string') {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
            if (typeof untrustedConfig_fields_prop === 'number') {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
            if (untrustedConfig_fields_prop === null) {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
            if (typeof untrustedConfig_fields_prop === 'boolean') {
                if (untrustedConfig_fields_object !== undefined) {
                    untrustedConfig_fields_object[key] = untrustedConfig_fields_prop;
                }
            }
        }
        if (untrustedConfig_fields_object !== undefined && Object.keys(untrustedConfig_fields_object).length >= 0) {
            config.fields = untrustedConfig_fields_object;
        }
    }
    return config;
}
function validateAdapterConfig$2(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const coercedConfig = coerceConfig$1(untrustedConfig, updateRecord_ConfigPropertyMetadata);
    const config = typeCheckConfig$4(coercedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const ISO8601_DATE_REGEX = /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d+Z$/;
function getHeaders(clientOptions) {
    const headers = {};
    if (untrustedIsObject(clientOptions)) {
        if (typeof clientOptions.ifUnmodifiedSince === 'string') {
            headers.ifUnmodifiedSince = clientOptions.ifUnmodifiedSince;
            // HTTP standard format date is expected by UI-API
            // https://developer.mozilla.org/en-US/docs/Web/HTTP/Headers/If-Unmodified-Since
            // A component built for Connect API must use If-Unmodified-Since
            // dates formatted as ISO 8601, which does not match the HTTP spec.
            // For compatibility, convert the date to match the standard.
            if (headers.ifUnmodifiedSince.match(ISO8601_DATE_REGEX)) {
                const utcString = new Date(headers.ifUnmodifiedSince).toUTCString();
                if (utcString !== 'Invalid Date') {
                    headers.ifUnmodifiedSince = utcString;
                }
            }
        }
    }
    return headers;
}
function buildNetworkSnapshot$2(luvio, config, clientOptions) {
    const { recordId } = config;
    const headers = getHeaders(clientOptions);
    const resourceParams = createResourceParams$2({ ...config, ...headers });
    const request = createResourceRequest$2(resourceParams);
    const fieldTrie = BLANK_RECORD_FIELDS_TRIE;
    const optionalFieldTrie = BLANK_RECORD_FIELDS_TRIE;
    const conflictMap = {
        conflicts: {},
        serverRequestCount: 0, // do not count the update request we're about to make
    };
    const recordIngest = createRecordIngest(fieldTrie, optionalFieldTrie, conflictMap);
    return luvio.dispatchResourceRequest(request).then((response) => {
        return luvio.handleSuccessResponse(() => {
            if (response.status === HttpStatusCode.NoContent) {
                const syntheticSnapshot = {
                    state: 'Fulfilled',
                    data: {},
                };
                luvio.storeEvict(keyBuilder$3E(luvio, { recordId }));
                return luvio.storeBroadcast().then(() => syntheticSnapshot);
            }
            const snapshot = ingestRecordResponse(luvio, response, recordId, recordIngest, conflictMap);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys$1(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (err) => {
        deepFreeze(err);
        throw err;
    });
}
const factory$2 = (luvio) => {
    return (untrusted, clientOptions) => {
        let config = null;
        if (untrustedIsObject(untrusted) &&
            untrustedIsObject(untrusted.fields)) {
            config = validateAdapterConfig$2({ recordId: untrusted.fields.Id, ...untrusted }, updateRecord_ConfigPropertyNames);
        }
        // Invalid or incomplete config
        if (config === null) {
            // eslint-disable-next-line @salesforce/lds/no-error-in-production
            throw new Error('Invalid recordInput');
        }
        const draftAdapter = configurationForRestAdapters.getDraftAwareUpdateRecordAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, (config) => {
                const headers = getHeaders(clientOptions);
                const resourceParams = createResourceParams$2({ ...config, ...headers });
                return createResourceRequest$2(resourceParams);
            });
        }
        return buildNetworkSnapshot$2(luvio, config, clientOptions);
    };
};

const VERSION$4 = "5f97eb4f2c3f805ef9d98ba7f0530b3e";
function validate$4(obj, path = 'ContentDocumentCompositeRepresentation') {
    const v_error = (() => {
        if (typeof obj !== 'object' || ArrayIsArray(obj) || obj === null) {
            return new TypeError('Expected "object" but received "' + typeof obj + '" (at "' + path + '")');
        }
        const obj_contentDocument = obj.contentDocument;
        const path_contentDocument = path + '.contentDocument';
        if (typeof obj_contentDocument !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_contentDocument + '" (at "' + path_contentDocument + '")');
        }
        const obj_contentDocumentLinks = obj.contentDocumentLinks;
        const path_contentDocumentLinks = path + '.contentDocumentLinks';
        if (!ArrayIsArray(obj_contentDocumentLinks)) {
            return new TypeError('Expected "array" but received "' + typeof obj_contentDocumentLinks + '" (at "' + path_contentDocumentLinks + '")');
        }
        for (let i = 0; i < obj_contentDocumentLinks.length; i++) {
            const obj_contentDocumentLinks_item = obj_contentDocumentLinks[i];
            const path_contentDocumentLinks_item = path_contentDocumentLinks + '[' + i + ']';
            if (typeof obj_contentDocumentLinks_item !== 'object') {
                return new TypeError('Expected "object" but received "' + typeof obj_contentDocumentLinks_item + '" (at "' + path_contentDocumentLinks_item + '")');
            }
        }
        const obj_contentVersion = obj.contentVersion;
        const path_contentVersion = path + '.contentVersion';
        if (typeof obj_contentVersion !== 'object') {
            return new TypeError('Expected "object" but received "' + typeof obj_contentVersion + '" (at "' + path_contentVersion + '")');
        }
    })();
    return v_error === undefined ? null : v_error;
}
const RepresentationType$4 = 'ContentDocumentCompositeRepresentation';
function keyBuilder(luvio, config) {
    return keyPrefix + '::' + RepresentationType$4 + ':' + config.contentDocumentId;
}
function keyBuilderFromType(luvio, object) {
    const keyParams = {
        contentDocumentId: object.contentDocument.id
    };
    return keyBuilder(luvio, keyParams);
}
function normalize$4(input, existing, path, luvio, store, timestamp) {
    const input_contentDocument = input.contentDocument;
    const input_contentDocument_id = path.fullPath + '__contentDocument';
    input.contentDocument = ingest$2g(input_contentDocument, {
        fullPath: input_contentDocument_id,
        propertyName: 'contentDocument',
        parent: {
            data: input,
            key: path.fullPath,
            existing: existing,
        },
        ttl: path.ttl
    }, luvio, store, timestamp);
    const input_contentDocumentLinks = input.contentDocumentLinks;
    const input_contentDocumentLinks_id = path.fullPath + '__contentDocumentLinks';
    for (let i = 0; i < input_contentDocumentLinks.length; i++) {
        const input_contentDocumentLinks_item = input_contentDocumentLinks[i];
        let input_contentDocumentLinks_item_id = input_contentDocumentLinks_id + '__' + i;
        input_contentDocumentLinks[i] = ingest$2g(input_contentDocumentLinks_item, {
            fullPath: input_contentDocumentLinks_item_id,
            propertyName: i,
            parent: {
                data: input,
                key: path.fullPath,
                existing: existing,
            },
            ttl: path.ttl
        }, luvio, store, timestamp);
    }
    const input_contentVersion = input.contentVersion;
    const input_contentVersion_id = path.fullPath + '__contentVersion';
    input.contentVersion = ingest$2g(input_contentVersion, {
        fullPath: input_contentVersion_id,
        propertyName: 'contentVersion',
        parent: {
            data: input,
            key: path.fullPath,
            existing: existing,
        },
        ttl: path.ttl
    }, luvio, store, timestamp);
    return input;
}
function equals$4(existing, incoming) {
    const existing_contentDocument = existing.contentDocument;
    const incoming_contentDocument = incoming.contentDocument;
    if (!(existing_contentDocument.__ref === incoming_contentDocument.__ref)) {
        return false;
    }
    const existing_contentDocumentLinks = existing.contentDocumentLinks;
    const incoming_contentDocumentLinks = incoming.contentDocumentLinks;
    const equals_contentDocumentLinks_items = equalsArray(existing_contentDocumentLinks, incoming_contentDocumentLinks, (existing_contentDocumentLinks_item, incoming_contentDocumentLinks_item) => {
        if (!(existing_contentDocumentLinks_item.__ref === incoming_contentDocumentLinks_item.__ref)) {
            return false;
        }
    });
    if (equals_contentDocumentLinks_items === false) {
        return false;
    }
    const existing_contentVersion = existing.contentVersion;
    const incoming_contentVersion = incoming.contentVersion;
    if (!(existing_contentVersion.__ref === incoming_contentVersion.__ref)) {
        return false;
    }
    return true;
}
const ingest$4 = function ContentDocumentCompositeRepresentationIngest(input, path, luvio, store, timestamp) {
    if (process.env.NODE_ENV !== 'production') {
        const validateError = validate$4(input);
        if (validateError !== null) {
            throw validateError;
        }
    }
    const key = keyBuilderFromType(luvio, input);
    const ttlToUse = path.ttl !== undefined ? path.ttl : 2592000000;
    ingestShape(input, path, luvio, store, timestamp, ttlToUse, key, normalize$4, "UiApi", VERSION$4, RepresentationType$4, equals$4);
    return createLink$1(key);
};
function getTypeCacheKeys(rootKeySet, luvio, input, fullPathFactory) {
    // root cache key (uses fullPathFactory if keyBuilderFromType isn't defined)
    const rootKey = keyBuilderFromType(luvio, input);
    rootKeySet.set(rootKey, {
        namespace: keyPrefix,
        representationName: RepresentationType$4,
        mergeable: false
    });
    getTypeCacheKeys$2l(rootKeySet, luvio, input.contentDocument);
    const input_contentDocumentLinks_length = input.contentDocumentLinks.length;
    for (let i = 0; i < input_contentDocumentLinks_length; i++) {
        getTypeCacheKeys$2l(rootKeySet, luvio, input.contentDocumentLinks[i]);
    }
    getTypeCacheKeys$2l(rootKeySet, luvio, input.contentVersion);
}

function getResponseCacheKeys(storeKeyMap, luvio, resourceParams, response) {
    getTypeCacheKeys(storeKeyMap, luvio, response);
}

function createResourceParams$1(config) {
    const { fileData, title, description, contentDocumentId } = config;
    const body = {
        namedEntries: [
            {
                name: 'fileData',
                value: fileData,
            },
        ],
    };
    if (title !== undefined) {
        body.namedEntries.push({
            name: 'title',
            value: title,
        });
    }
    if (description !== undefined) {
        body.namedEntries.push({
            name: 'description',
            value: description,
        });
    }
    return {
        urlParams: {
            contentDocumentId,
        },
        body,
    };
}
function createResourceRequest$1(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/content-documents/' +
            config.urlParams.contentDocumentId +
            '/content-versions',
        method: 'post',
        body: config.body,
        urlParams: config.urlParams,
        queryParams: {},
        headers,
        priority: 'normal',
    };
}
const select = function ContentDocumentCompositeRepresentationSelect(body) {
    return {
        kind: 'Fragment',
        private: [],
        version: VERSION$4,
        selections: [
            {
                name: 'contentDocument',
                kind: 'Link',
                fragment: {
                    kind: 'Fragment',
                    private: [],
                    version: VERSION$2K,
                    selections: buildSelectionFromRecord(body.contentDocument),
                },
            },
            {
                name: 'contentDocumentLinks',
                kind: 'Link',
                plural: true,
                fragment: {
                    kind: 'Fragment',
                    private: [],
                    version: VERSION$2K,
                    selections: buildSelectionFromRecord(body.contentDocumentLinks[0] || { fields: {} }),
                },
            },
            {
                name: 'contentVersion',
                kind: 'Link',
                fragment: {
                    kind: 'Fragment',
                    private: [],
                    version: VERSION$2K,
                    selections: buildSelectionFromRecord(body.contentVersion),
                },
            },
        ],
    };
};
function ingestSuccess$1(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType(luvio, body);
    const node = select(body);
    luvio.storeIngest(key, ingest$4, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node,
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    return snapshot;
}
function buildNetworkSnapshot$1(luvio, config, options) {
    const resourceParams = createResourceParams$1(config);
    const request = createResourceRequest$1(resourceParams);
    return luvio
        .dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess$1(luvio, {}, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys(cache, luvio, {}, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}

function createResourceParams(config) {
    const { fileData, title, description } = config;
    const body = {
        namedEntries: [
            {
                name: 'fileData',
                value: fileData,
            },
        ],
    };
    if (title !== undefined) {
        body.namedEntries.push({
            name: 'title',
            value: title,
        });
    }
    if (description !== undefined) {
        body.namedEntries.push({
            name: 'description',
            value: description,
        });
    }
    return {
        body,
    };
}
function createResourceRequest(config) {
    const headers = {};
    return {
        baseUri: '/services/data/v65.0',
        basePath: '/ui-api/records/content-documents/content-versions',
        method: 'post',
        body: config.body,
        urlParams: {},
        queryParams: {},
        headers,
        priority: 'normal',
    };
}
function ingestSuccess(luvio, resourceParams, response) {
    const { body } = response;
    const key = keyBuilderFromType(luvio, body);
    const node = select(body);
    luvio.storeIngest(key, ingest$4, body);
    const snapshot = luvio.storeLookup({
        recordId: key,
        node,
        variables: {},
    });
    if (process.env.NODE_ENV !== 'production') {
        if (snapshot.state !== 'Fulfilled') {
            throw new Error('Invalid network response. Expected resource response to result in Fulfilled snapshot');
        }
    }
    return snapshot;
}
function buildNetworkSnapshot(luvio, config, options) {
    const resourceParams = createResourceParams(config);
    const request = createResourceRequest(resourceParams);
    return luvio
        .dispatchResourceRequest(request, options)
        .then((response) => {
        return luvio.handleSuccessResponse(() => {
            const snapshot = ingestSuccess(luvio, {}, response);
            return luvio.storeBroadcast().then(() => snapshot);
        }, () => {
            const cache = new StoreKeyMap();
            getResponseCacheKeys(cache, luvio, resourceParams, response.body);
            return cache;
        });
    }, (response) => {
        deepFreeze(response);
        throw response;
    });
}

const createContentDocumentAndVersion_ConfigPropertyMetadata = [
    generateParamConfigMetadata('description', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('title', false, 2 /* Body */, 0 /* String */),
];
function typeCheckConfig$3(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, createContentDocumentAndVersion_ConfigPropertyMetadata);
    return config;
}

// This override is due to the addition of Blob formatted file data. There is not a convenient way to express this in RAML, so this validation
// must be handrolled
const createContentDocumentAndVersion_ConfigPropertyNames = {
    displayName: 'createContentDocumentAndVersion',
    parameters: {
        required: ['fileData'],
        optional: ['description', 'title'],
    },
};
function typeCheckConfig$2(untrustedConfig) {
    const config = typeCheckConfig$3(untrustedConfig);
    const untrustedConfig_fileData = untrustedConfig.fileData;
    if (untrustedConfig_fileData !== undefined && untrustedConfig_fileData instanceof Object) {
        config.fileData = untrustedConfig_fileData;
    }
    return config;
}
function validateAdapterConfig$1(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig$2(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const factory$1 = (luvio) => {
    return function createContentDocumentAndVersion(untrustedConfig, requestContext) {
        let config = validateAdapterConfig$1(untrustedConfig, createContentDocumentAndVersion_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            // eslint-disable-next-line @salesforce/lds/no-error-in-production
            throw new Error('Invalid config for "createContentDocumentAndVersion"');
        }
        const draftAdapter = configurationForRestAdapters.getDraftAwareCreateContentDocumentAndVersionAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, (config) => {
                const resourceParams = createResourceParams(config);
                return createResourceRequest(resourceParams);
            }, requestContext);
        }
        return buildNetworkSnapshot(luvio, config, createDispatchResourceRequestContext$1(requestContext));
    };
};
function createDispatchResourceRequestContext$1(requestContext) {
    let dispatchOptions = undefined;
    if (requestContext !== undefined) {
        const coercedAdapterRequestContext = coerceAdapterRequestContext(requestContext);
        const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
        dispatchOptions = {
            resourceRequestContext: {
                requestCorrelator,
                sourceContext,
                luvioRequestMethod: undefined,
            },
            eventObservers,
        };
        if (networkPriority !== 'normal') {
            dispatchOptions.overrides = {
                priority: networkPriority,
            };
        }
    }
    return dispatchOptions;
}

const createContentVersion_ConfigPropertyMetadata = [
    generateParamConfigMetadata('contentDocumentId', true, 0 /* UrlParameter */, 0 /* String */),
    generateParamConfigMetadata('description', false, 2 /* Body */, 0 /* String */),
    generateParamConfigMetadata('title', false, 2 /* Body */, 0 /* String */),
];
function typeCheckConfig$1(untrustedConfig) {
    const config = {};
    typeCheckConfig$l(untrustedConfig, config, createContentVersion_ConfigPropertyMetadata);
    return config;
}

// This override is due to the addition of Blob formatted file data. There is not a convenient way to express this in RAML, so this validation
// must be handrolled
const createContentVersion_ConfigPropertyNames = {
    displayName: 'createContentVersion',
    parameters: {
        required: ['contentDocumentId', 'fileData'],
        optional: ['description', 'title'],
    },
};
function typeCheckConfig(untrustedConfig) {
    const config = typeCheckConfig$1(untrustedConfig);
    const untrustedConfig_fileData = untrustedConfig.fileData;
    if (untrustedConfig_fileData !== undefined && untrustedConfig_fileData instanceof Object) {
        config.fileData = untrustedConfig_fileData;
    }
    return config;
}
function validateAdapterConfig(untrustedConfig, configPropertyNames) {
    if (!untrustedIsObject(untrustedConfig)) {
        return null;
    }
    if (process.env.NODE_ENV !== 'production') {
        validateConfig(untrustedConfig, configPropertyNames);
    }
    const config = typeCheckConfig(untrustedConfig);
    if (!areRequiredParametersPresent(config, configPropertyNames)) {
        return null;
    }
    return config;
}

const factory = (luvio) => {
    return function createContentVersion(untrustedConfig, requestContext) {
        const config = validateAdapterConfig(untrustedConfig, createContentVersion_ConfigPropertyNames);
        // Invalid or incomplete config
        if (config === null) {
            // eslint-disable-next-line @salesforce/lds/no-error-in-production
            throw new Error('Invalid config for "createContentVersion"');
        }
        const draftAdapter = configurationForRestAdapters.getDraftAwareCreateContentVersionAdapter();
        if (draftAdapter !== undefined) {
            return draftAdapter(config, buildNetworkSnapshot$1, requestContext);
        }
        return buildNetworkSnapshot$1(luvio, config, createDispatchResourceRequestContext(requestContext));
    };
};
function createDispatchResourceRequestContext(requestContext) {
    let dispatchOptions = undefined;
    if (requestContext !== undefined) {
        const coercedAdapterRequestContext = coerceAdapterRequestContext(requestContext);
        const { networkPriority, requestCorrelator, eventObservers, sourceContext } = coercedAdapterRequestContext;
        dispatchOptions = {
            resourceRequestContext: {
                requestCorrelator,
                sourceContext,
                luvioRequestMethod: undefined,
            },
            eventObservers,
        };
        if (networkPriority !== 'normal') {
            dispatchOptions.overrides = {
                priority: networkPriority,
            };
        }
    }
    return dispatchOptions;
}

({
    ...configurationForRestAdapters,
    ...configurationForGraphQLAdapters,
    ...configurationForOneStoreEnabledAdapters,
});
ensureRegisteredOnce({
    id: '@salesforce/lds-adapters-uiapi',
    configuration: {
        ...configurationForRestAdapters,
        ...configurationForGraphQLAdapters,
        ...configurationForOneStoreEnabledAdapters,
    },
    instrument,
});

let createContentDocumentAndVersion;
let createContentVersion;
let createListInfo;
let createRecord;
let deleteListInfo;
let deleteRecord;
let executeBatchRecordOperations;
let getActionOverrides;
let getAllApps;
let getAppDetails;
let getDuplicateConfiguration;
let getDuplicates;
let getFlexipageFormulaOverrides;
let getGlobalActions;
let getKeywordSearchResults;
let getLayout;
let getLayoutUserState;
let getListInfoByName;
let getListInfosByName;
let getListInfosByObjectName;
let getListObjectInfo;
let getListPreferences;
let getListRecordsByName;
let getListUi;
let getLookupActions;
let getLookupMetadata;
let getLookupRecords;
let getNavItems;
let getObjectCreateActions;
let getObjectInfo;
let getObjectInfos;
let getPathLayout;
let getPicklistValues;
let getPicklistValuesByRecordType;
let getQuickActionDefaults;
let getQuickActionInfo;
let getQuickActionLayout;
let getRecord;
let getRecordActions;
let getRecordAvatars;
let getRecordCreateDefaults;
let getRecordEditActions;
let getRecordTemplateClone;
let getRecordTemplateCreate;
let getRecordUi;
let getRecords;
let getRelatedListActions;
let getRelatedListCount;
let getRelatedListInfo;
let getRelatedListInfoBatch;
let getRelatedListPreferences;
let getRelatedListPreferencesBatch;
let getRelatedListRecordActions;
let getRelatedListRecords;
let getRelatedListRecordsBatch;
let getRelatedListsActions;
let getRelatedListsCount;
let getRelatedListsInfo;
let getSearchFilterMetadata;
let getSearchFilterOptions;
let getSearchResults;
let graphql;
let graphqlBatch;
let performQuickAction;
let performUpdateRecordQuickAction;
let updateListInfoByName;
let updateListPreferences;
let updateRecord;
let updateRecordAvatar;
function bindExportsTo(luvio) {
    function unwrapSnapshotData(factory) {
        const adapter = factory(luvio);
        return (...config) => adapter(...config).then(snapshot => snapshot.data);
    }
    return {
        createContentDocumentAndVersion: unwrapSnapshotData(factory$1),
        createContentVersion: unwrapSnapshotData(factory),
        createListInfo: unwrapSnapshotData(createListInfoAdapterFactory),
        createRecord: unwrapSnapshotData(factory$4),
        deleteListInfo: deleteListInfoAdapterFactory(luvio),
        deleteRecord: factory$3(luvio),
        executeBatchRecordOperations: unwrapSnapshotData(factory$f),
        getActionOverrides: createWireAdapterConstructor(getActionOverridesAdapterFactory(luvio), "getActionOverrides", luvio),
        getAllApps: createWireAdapterConstructor(getAllAppsAdapterFactory(luvio), "getAllApps", luvio),
        getAppDetails: createWireAdapterConstructor(getAppDetailsAdapterFactory(luvio), "getAppDetails", luvio),
        getDuplicateConfiguration: createWireAdapterConstructor(getDuplicateConfigurationAdapterFactory(luvio), "getDuplicateConfiguration", luvio),
        getDuplicates: createWireAdapterConstructor(getDuplicatesAdapterFactory(luvio), "getDuplicates", luvio),
        getFlexipageFormulaOverrides: createWireAdapterConstructor(getFlexipageFormulaOverridesAdapterFactory(luvio), "getFlexipageFormulaOverrides", luvio),
        getGlobalActions: createWireAdapterConstructor(getGlobalActionsAdapterFactory(luvio), "getGlobalActions", luvio),
        getKeywordSearchResults: createWireAdapterConstructor(getKeywordSearchResultsAdapterFactory(luvio), "getKeywordSearchResults", luvio),
        getLayout: createWireAdapterConstructor(getLayoutAdapterFactory(luvio), "getLayout", luvio),
        getLayoutUserState: createWireAdapterConstructor(getLayoutUserStateAdapterFactory(luvio), "getLayoutUserState", luvio),
        getListInfoByName: createWireAdapterConstructor(getListInfoByNameAdapterFactory(luvio), "getListInfoByName", luvio),
        getListInfosByName: createWireAdapterConstructor(getListInfosByNameAdapterFactory(luvio), "getListInfosByName", luvio),
        getListInfosByObjectName: createWireAdapterConstructor(getListInfosByObjectNameAdapterFactory(luvio), "getListInfosByObjectName", luvio),
        getListObjectInfo: createWireAdapterConstructor(getListObjectInfoAdapterFactory(luvio), "getListObjectInfo", luvio),
        getListPreferences: createWireAdapterConstructor(getListPreferencesAdapterFactory(luvio), "getListPreferences", luvio),
        getListRecordsByName: createWireAdapterConstructor(factory$a(luvio), "getListRecordsByName", luvio),
        getListUi: createWireAdapterConstructor(factory$j(luvio), "getListUi", luvio),
        getLookupActions: createWireAdapterConstructor(getLookupActionsAdapterFactory(luvio), "getLookupActions", luvio),
        getLookupMetadata: createWireAdapterConstructor(getLookupMetadataAdapterFactory(luvio), "getLookupMetadata", luvio),
        getLookupRecords: createWireAdapterConstructor(factory$9(luvio), "getLookupRecords", luvio),
        getNavItems: createWireAdapterConstructor(getNavItemsAdapterFactory(luvio), "getNavItems", luvio),
        getObjectCreateActions: createWireAdapterConstructor(getObjectCreateActionsAdapterFactory(luvio), "getObjectCreateActions", luvio),
        getObjectInfo: createWireAdapterConstructor(getObjectInfoAdapterFactory(luvio), "getObjectInfo", luvio),
        getObjectInfos: createWireAdapterConstructor(getObjectInfosAdapterFactory(luvio), "getObjectInfos", luvio),
        getPathLayout: createWireAdapterConstructor(getPathLayoutAdapterFactory(luvio), "getPathLayout", luvio),
        getPicklistValues: createWireAdapterConstructor(getPicklistValuesAdapterFactory(luvio), "getPicklistValues", luvio),
        getPicklistValuesByRecordType: createWireAdapterConstructor(getPicklistValuesByRecordTypeAdapterFactory(luvio), "getPicklistValuesByRecordType", luvio),
        getQuickActionDefaults: createWireAdapterConstructor(getQuickActionDefaultsAdapterFactory(luvio), "getQuickActionDefaults", luvio),
        getQuickActionInfo: createWireAdapterConstructor(getQuickActionInfoAdapterFactory(luvio), "getQuickActionInfo", luvio),
        getQuickActionLayout: createWireAdapterConstructor(getQuickActionLayoutAdapterFactory(luvio), "getQuickActionLayout", luvio),
        getRecord: createWireAdapterConstructor(factory$h(luvio), "getRecord", luvio),
        getRecordActions: createWireAdapterConstructor(getRecordActionsAdapterFactory(luvio), "getRecordActions", luvio),
        getRecordAvatars: createWireAdapterConstructor(getRecordAvatarsAdapterFactory(luvio), "getRecordAvatars", luvio),
        getRecordCreateDefaults: createWireAdapterConstructor(factory$7(luvio), "getRecordCreateDefaults", luvio),
        getRecordEditActions: createWireAdapterConstructor(getRecordEditActionsAdapterFactory(luvio), "getRecordEditActions", luvio),
        getRecordTemplateClone: createWireAdapterConstructor(factory$6(luvio), "getRecordTemplateClone", luvio),
        getRecordTemplateCreate: createWireAdapterConstructor(factory$5(luvio), "getRecordTemplateCreate", luvio),
        getRecordUi: createWireAdapterConstructor(factory$i(luvio), "getRecordUi", luvio),
        getRecords: createWireAdapterConstructor(getRecordsAdapterFactory(luvio), "getRecords", luvio),
        getRelatedListActions: createWireAdapterConstructor(getRelatedListActionsAdapterFactory(luvio), "getRelatedListActions", luvio),
        getRelatedListCount: createWireAdapterConstructor(getRelatedListCountAdapterFactory(luvio), "getRelatedListCount", luvio),
        getRelatedListInfo: createWireAdapterConstructor(getRelatedListInfoAdapterFactory(luvio), "getRelatedListInfo", luvio),
        getRelatedListInfoBatch: createWireAdapterConstructor(getRelatedListInfoBatchAdapterFactory(luvio), "getRelatedListInfoBatch", luvio),
        getRelatedListPreferences: createWireAdapterConstructor(getRelatedListPreferencesAdapterFactory(luvio), "getRelatedListPreferences", luvio),
        getRelatedListPreferencesBatch: createWireAdapterConstructor(getRelatedListPreferencesBatchAdapterFactory(luvio), "getRelatedListPreferencesBatch", luvio),
        getRelatedListRecordActions: createWireAdapterConstructor(getRelatedListRecordActionsAdapterFactory(luvio), "getRelatedListRecordActions", luvio),
        getRelatedListRecords: createWireAdapterConstructor(getRelatedListRecordsAdapterFactory(luvio), "getRelatedListRecords", luvio),
        getRelatedListRecordsBatch: createWireAdapterConstructor(getRelatedListRecordsBatchAdapterFactory(luvio), "getRelatedListRecordsBatch", luvio),
        getRelatedListsActions: createWireAdapterConstructor(getRelatedListsActionsAdapterFactory(luvio), "getRelatedListsActions", luvio),
        getRelatedListsCount: createWireAdapterConstructor(getRelatedListsCountAdapterFactory(luvio), "getRelatedListsCount", luvio),
        getRelatedListsInfo: createWireAdapterConstructor(getRelatedListsInfoAdapterFactory(luvio), "getRelatedListsInfo", luvio),
        getSearchFilterMetadata: createWireAdapterConstructor(getSearchFilterMetadataAdapterFactory(luvio), "getSearchFilterMetadata", luvio),
        getSearchFilterOptions: createWireAdapterConstructor(getSearchFilterOptionsAdapterFactory(luvio), "getSearchFilterOptions", luvio),
        getSearchResults: createWireAdapterConstructor(getSearchResultsAdapterFactory(luvio), "getSearchResults", luvio),
        graphql: createWireAdapterConstructor(factory$e(luvio), "graphql", luvio),
        graphqlBatch: createWireAdapterConstructor(factory$g(luvio), "graphqlBatch", luvio),
        performQuickAction: unwrapSnapshotData(factory$c),
        performUpdateRecordQuickAction: unwrapSnapshotData(factory$d),
        updateListInfoByName: unwrapSnapshotData(updateListInfoByNameAdapterFactory),
        updateListPreferences: unwrapSnapshotData(updateListPreferencesAdapterFactory),
        updateRecord: unwrapSnapshotData(factory$2),
        updateRecordAvatar: unwrapSnapshotData(factory$8)
    };
}
withDefaultLuvio((luvio) => {
    ({
        createContentDocumentAndVersion,
        createContentVersion,
        createListInfo,
        createRecord,
        deleteListInfo,
        deleteRecord,
        executeBatchRecordOperations,
        getActionOverrides,
        getAllApps,
        getAppDetails,
        getDuplicateConfiguration,
        getDuplicates,
        getFlexipageFormulaOverrides,
        getGlobalActions,
        getKeywordSearchResults,
        getLayout,
        getLayoutUserState,
        getListInfoByName,
        getListInfosByName,
        getListInfosByObjectName,
        getListObjectInfo,
        getListPreferences,
        getListRecordsByName,
        getListUi,
        getLookupActions,
        getLookupMetadata,
        getLookupRecords,
        getNavItems,
        getObjectCreateActions,
        getObjectInfo,
        getObjectInfos,
        getPathLayout,
        getPicklistValues,
        getPicklistValuesByRecordType,
        getQuickActionDefaults,
        getQuickActionInfo,
        getQuickActionLayout,
        getRecord,
        getRecordActions,
        getRecordAvatars,
        getRecordCreateDefaults,
        getRecordEditActions,
        getRecordTemplateClone,
        getRecordTemplateCreate,
        getRecordUi,
        getRecords,
        getRelatedListActions,
        getRelatedListCount,
        getRelatedListInfo,
        getRelatedListInfoBatch,
        getRelatedListPreferences,
        getRelatedListPreferencesBatch,
        getRelatedListRecordActions,
        getRelatedListRecords,
        getRelatedListRecordsBatch,
        getRelatedListsActions,
        getRelatedListsCount,
        getRelatedListsInfo,
        getSearchFilterMetadata,
        getSearchFilterOptions,
        getSearchResults,
        graphql,
        graphqlBatch,
        performQuickAction,
        performUpdateRecordQuickAction,
        updateListInfoByName,
        updateListPreferences,
        updateRecord,
        updateRecordAvatar
    } = bindExportsTo(luvio));
});

let getRecordNotifyChange, refresh, notifyRecordUpdateAvailable;
withDefaultLuvio((luvio) => {
    getRecordNotifyChange = notifyChangeFactory(luvio);
    notifyRecordUpdateAvailable = notifyUpdateAvailableFactory$5(luvio);
    refresh = bindWireRefresh(luvio);
});

export { MRU, createContentDocumentAndVersion, createContentVersion, createListInfo, createRecord, createRecordInputFilteredByEditedFields, deleteListInfo, deleteRecord, executeBatchRecordOperations, generateRecordInputForCreate, generateRecordInputForUpdate, getActionOverrides, getAllApps, getAppDetails, getDuplicateConfiguration, getDuplicates, getFieldDisplayValue$1 as getFieldDisplayValue, getFieldValue, getFlexipageFormulaOverrides, getGlobalActions, getKeywordSearchResults, getLayout, getLayoutUserState, getListInfoByName, getListInfosByName, getListInfosByObjectName, getListObjectInfo, getListPreferences, getListRecordsByName, getListUi, getLookupActions, getLookupMetadata, getLookupRecords, getNavItems, getObjectCreateActions, getObjectInfo, getObjectInfos, getPathLayout, getPicklistValues, getPicklistValuesByRecordType, getQuickActionDefaults, getQuickActionInfo, getQuickActionLayout, getRecord, getRecordActions, getRecordAvatars, getRecordCreateDefaults, getRecordEditActions, getRecordInput, getRecordNotifyChange, getRecordTemplateClone, getRecordTemplateCreate, getRecordUi, getRecords, getRelatedListActions, getRelatedListCount, getRelatedListInfo, getRelatedListInfoBatch, getRelatedListPreferences, getRelatedListPreferencesBatch, getRelatedListRecordActions, getRelatedListRecords, getRelatedListRecordsBatch, getRelatedListsActions, getRelatedListsCount, getRelatedListsInfo, getSearchFilterMetadata, getSearchFilterOptions, getSearchResults, graphql, graphqlBatch, notifyRecordUpdateAvailable, performQuickAction, performUpdateRecordQuickAction, refresh, updateListInfoByName, updateListPreferences, updateRecord, updateRecordAvatar };
